@@ -19,7 +19,7 @@
 script_directory="$(cd "$(dirname "$0")" && pwd)"
 
 CMAKE_GLOBAL_MIN_VERSION_MAJOR=3
-CMAKE_GLOBAL_MIN_VERSION_MINOR=7
+CMAKE_GLOBAL_MIN_VERSION_MINOR=17
 CMAKE_GLOBAL_MIN_VERSION_REVISION=0
 
 export RED='\033[0;101m'
@@ -39,6 +39,7 @@ export CMAKE_MIN_VERSION=()
 export INCOMPATIBLE_WITH=()
 export DEPLOY_LIMITS=()
 export DEPENDENCIES=()
+export DEPENDS_ON=()
 
 CORES=1
 BUILD="false"
@@ -207,6 +208,8 @@ else
     . "${script_directory}/rheldistro.sh"
   elif [[ "$OS" = Amazon* ]]; then
     . "${script_directory}/centos.sh"
+  elif [[ "$OS" = Rocky* ]]; then
+    . "${script_directory}/centos.sh"
   elif [[ "$OS" = CentOS* ]]; then
     . "${script_directory}/centos.sh"
   elif [[ "$OS" = Fedora* ]]; then
@@ -265,13 +268,13 @@ add_enabled_option HTTP_CURL_ENABLED ${TRUE} "DISABLE_CURL"
 add_enabled_option LIBARCHIVE_ENABLED ${TRUE} "DISABLE_LIBARCHIVE"
 add_dependency LIBARCHIVE_ENABLED "libarchive"
 
-add_enabled_option EXECUTE_SCRIPT_ENABLED ${TRUE} "DISABLE_SCRIPTING"
+add_disabled_option EXECUTE_SCRIPT_ENABLED ${FALSE} "ENABLE_SCRIPTING"
 add_dependency EXECUTE_SCRIPT_ENABLED "python"
 add_dependency EXECUTE_SCRIPT_ENABLED "lua"
 
-add_enabled_option EXPRESSION_LANGAUGE_ENABLED ${TRUE} "DISABLE_EXPRESSION_LANGUAGE"
-add_dependency EXPRESSION_LANGAUGE_ENABLED "bison"
-add_dependency EXPRESSION_LANGAUGE_ENABLED "flex"
+add_enabled_option EXPRESSION_LANGUAGE_ENABLED ${TRUE} "DISABLE_EXPRESSION_LANGUAGE"
+add_dependency EXPRESSION_LANGUAGE_ENABLED "bison"
+add_dependency EXPRESSION_LANGUAGE_ENABLED "flex"
 
 add_disabled_option PCAP_ENABLED ${FALSE} "ENABLE_PCAP"
 add_dependency PCAP_ENABLED "libpcap"
@@ -287,6 +290,8 @@ add_disabled_option AWS_ENABLED ${FALSE} "ENABLE_AWS"
 
 add_disabled_option KAFKA_ENABLED ${FALSE} "ENABLE_LIBRDKAFKA"
 
+add_disabled_option KUBERNETES_ENABLED ${FALSE} "ENABLE_KUBERNETES"
+
 add_disabled_option MQTT_ENABLED ${FALSE} "ENABLE_MQTT"
 
 add_disabled_option PYTHON_ENABLED ${FALSE} "ENABLE_PYTHON"
@@ -326,6 +331,15 @@ add_disabled_option AZURE_ENABLED ${FALSE} "ENABLE_AZURE"
 
 add_disabled_option SYSTEMD_ENABLED ${TRUE} "ENABLE_SYSTEMD"
 
+add_disabled_option NANOFI_ENABLED ${FALSE} "ENABLE_NANOFI"
+set_dependency PYTHON_ENABLED NANOFI_ENABLED
+
+add_disabled_option SPLUNK_ENABLED ${FALSE} "ENABLE_SPLUNK"
+
+add_disabled_option GCP_ENABLED ${FALSE} "ENABLE_GCP"
+
+add_disabled_option PROCFS_ENABLED ${FALSE} "ENABLE_PROCFS"
+
 USE_SHARED_LIBS=${TRUE}
 ASAN_ENABLED=${FALSE}
 FAIL_ON_WARNINGS=${FALSE}
@@ -498,7 +512,7 @@ build_cmake_command(){
 
   CMAKE_BUILD_COMMAND="${CMAKE_BUILD_COMMAND} -DBUILD_IDENTIFIER=${BUILD_IDENTIFIER}"
 
-    CMAKE_BUILD_COMMAND="${CMAKE_BUILD_COMMAND} -DCMAKE_BUILD_TYPE=${BUILD_PROFILE}"
+  CMAKE_BUILD_COMMAND="${CMAKE_BUILD_COMMAND} -DCMAKE_BUILD_TYPE=${BUILD_PROFILE}"
 
   add_os_flags
 
