@@ -69,6 +69,10 @@ add_multi_option(){
 	done
 }
 
+set_dependency(){
+  DEPENDS_ON+=("$1:$2")
+}
+
 set_incompatible_with(){
   INCOMPATIBLE_WITH+=("$1:$2")
   INCOMPATIBLE_WITH+=("$2:$1")
@@ -187,6 +191,17 @@ check_compatibility(){
       fi
     fi
   done
+  for option in "${DEPENDS_ON[@]}" ; do
+    OPT=${option%%:*}
+    if [ "$OPT" = "$1" ]; then
+      OTHER_FEATURE=${option#*:}
+      OTHER_FEATURE_VALUE=${!OTHER_FEATURE}
+      if [ "${OTHER_FEATURE_VALUE}" != "Enabled" ]; then
+        echo "false"
+        return
+      fi
+    fi
+  done
   echo "true"
 }
 
@@ -214,6 +229,13 @@ ToggleFeature(){
   ALL_FEATURES_ENABLED="Disabled"
   if [ "${VARIABLE_VALUE}" = "Enabled" ]; then
     eval "$1=${FALSE}"
+    for option in "${DEPENDS_ON[@]}" ; do
+      DEPENDENT_FEATURE=${option%%:*}
+      FEATURE=${option#*:}
+      if [ "$FEATURE" = "$1" ]; then
+        eval "$DEPENDENT_FEATURE=${FALSE}"
+      fi
+    done
   else
     for option in "${CMAKE_MIN_VERSION[@]}" ; do
       OPT=${option%%:*}
@@ -350,7 +372,7 @@ show_supported_features() {
   echo "B. libcurl features ............$(print_feature_status HTTP_CURL_ENABLED)"
   echo "C. libarchive features .........$(print_feature_status LIBARCHIVE_ENABLED)"
   echo "D. Execute Script support ......$(print_feature_status EXECUTE_SCRIPT_ENABLED)"
-  echo "E. Expression Language support .$(print_feature_status EXPRESSION_LANGAUGE_ENABLED)"
+  echo "E. Expression Language support .$(print_feature_status EXPRESSION_LANGUAGE_ENABLED)"
   echo "F. Kafka support ...............$(print_feature_status KAFKA_ENABLED)"
   echo "G. PCAP support ................$(print_feature_status PCAP_ENABLED)"
   echo "H. USB Camera support ..........$(print_feature_status USB_ENABLED)"
@@ -368,6 +390,11 @@ show_supported_features() {
   echo "W. Openwsman Support ...........$(print_feature_status OPENWSMAN_ENABLED)"
   echo "X. Azure Support ...............$(print_feature_status AZURE_ENABLED)"
   echo "Y. Systemd Support .............$(print_feature_status SYSTEMD_ENABLED)"
+  echo "Z. NanoFi Support ..............$(print_feature_status NANOFI_ENABLED)"
+  echo "AA. Splunk Support .............$(print_feature_status SPLUNK_ENABLED)"
+  echo "AB. Kubernetes Support .........$(print_feature_status KUBERNETES_ENABLED)"
+  echo "AC. Google Cloud Support .......$(print_feature_status GCP_ENABLED)"
+  echo "AD. ProcFs Support .............$(print_feature_status PROCFS_ENABLED)"
   echo "****************************************"
   echo "            Build Options."
   echo "****************************************"
@@ -390,7 +417,7 @@ show_supported_features() {
 
 read_feature_options(){
   local choice
-  echo -n "Enter choice [ A - Y or 1-7 ] "
+  echo -n "Enter choice [A-Z or AA-AD or 1-7] "
   read -r choice
   choice=$(echo "${choice}" | tr '[:upper:]' '[:lower:]')
   case $choice in
@@ -398,7 +425,7 @@ read_feature_options(){
     b) ToggleFeature HTTP_CURL_ENABLED ;;
     c) ToggleFeature LIBARCHIVE_ENABLED ;;
     d) ToggleFeature EXECUTE_SCRIPT_ENABLED ;;
-    e) ToggleFeature EXPRESSION_LANGAUGE_ENABLED ;;
+    e) ToggleFeature EXPRESSION_LANGUAGE_ENABLED ;;
     f) ToggleFeature KAFKA_ENABLED ;;
     g) ToggleFeature PCAP_ENABLED ;;
     h) ToggleFeature USB_ENABLED ;;
@@ -421,6 +448,11 @@ read_feature_options(){
     w) ToggleFeature OPENWSMAN_ENABLED ;;
     x) ToggleFeature AZURE_ENABLED ;;
     y) ToggleFeature SYSTEMD_ENABLED ;;
+    z) ToggleFeature NANOFI_ENABLED ;;
+    aa) ToggleFeature SPLUNK_ENABLED ;;
+    ab) ToggleFeature KUBERNETES_ENABLED ;;
+    ac) ToggleFeature GCP_ENABLED ;;
+    ad) ToggleFeature PROCFS_ENABLED ;;
     1) ToggleFeature TESTS_ENABLED ;;
     2) EnableAllFeatures ;;
     3) ToggleFeature JNI_ENABLED;;
@@ -439,7 +471,7 @@ read_feature_options(){
       fi
       ;;
     q) exit 0;;
-    *) echo -e "${RED}Please enter an option A-X or 1-7...${NO_COLOR}" && sleep 2
+    *) echo -e "${RED}Please enter an option A-Z or AA-AD or 1-7...${NO_COLOR}" && sleep 2
   esac
 }
 
