@@ -1,27 +1,59 @@
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+
 import logging
 import os
 import shutil
 
 
 class DockerTestDirectoryBindings:
-    def __init__(self):
+    def __init__(self, test_id):
         self.data_directories = {}
+        self.test_id = test_id
 
     def __del__(self):
         self.delete_data_directories()
 
-    def create_new_data_directories(self, test_id):
-        self.data_directories[test_id] = {
-            "input_dir": "/tmp/.nifi-test-input." + test_id,
-            "output_dir": "/tmp/.nifi-test-output." + test_id,
-            "resources_dir": "/tmp/.nifi-test-resources." + test_id
+    def cleanup_io(self):
+        for folder in [self.data_directories[self.test_id]["input_dir"], self.data_directories[self.test_id]["output_dir"]]:
+            for filename in os.listdir(folder):
+                file_path = os.path.join(folder, filename)
+                if os.path.isfile(file_path) or os.path.islink(file_path):
+                    os.unlink(file_path)
+                elif os.path.isdir(file_path):
+                    shutil.rmtree(file_path)
+
+    def create_new_data_directories(self):
+        self.data_directories[self.test_id] = {
+            "input_dir": "/tmp/.nifi-test-input." + self.test_id,
+            "output_dir": "/tmp/.nifi-test-output." + self.test_id,
+            "resources_dir": "/tmp/.nifi-test-resources." + self.test_id,
+            "minifi_config_dir": "/tmp/.nifi-test-minifi-config-dir." + self.test_id,
+            "nifi_config_dir": "/tmp/.nifi-test-nifi-config-dir." + self.test_id,
+            "kubernetes_temp_dir": "/tmp/.nifi-test-kubernetes-temp-dir." + self.test_id
         }
 
-        [self.create_directory(directory) for directory in self.data_directories[test_id].values()]
+        [self.create_directory(directory) for directory in self.data_directories[self.test_id].values()]
 
         # Add resources
-        test_dir = os.environ['PYTHONPATH'].split(':')[-1]  # Based on DockerVerify.sh
-        shutil.copytree(test_dir + "/resources/kafka_broker/conf/certs", self.data_directories[test_id]["resources_dir"] + "/certs")
+        test_dir = os.environ['TEST_DIRECTORY']  # Based on DockerVerify.sh
+        shutil.copytree(test_dir + "/resources/kafka_broker/conf/certs", self.data_directories[self.test_id]["resources_dir"] + "/certs")
+        shutil.copytree(test_dir + "/resources/python", self.data_directories[self.test_id]["resources_dir"] + "/python")
+        shutil.copytree(test_dir + "/resources/opcua", self.data_directories[self.test_id]["resources_dir"] + "/opcua")
+        shutil.copytree(test_dir + "/resources/lua", self.data_directories[self.test_id]["resources_dir"] + "/lua")
 
     def get_data_directories(self, test_id):
         return self.data_directories[test_id]
@@ -52,6 +84,8 @@ class DockerTestDirectoryBindings:
         vols[self.data_directories[test_id]["input_dir"]] = {"bind": "/tmp/input", "mode": "rw"}
         vols[self.data_directories[test_id]["output_dir"]] = {"bind": "/tmp/output", "mode": "rw"}
         vols[self.data_directories[test_id]["resources_dir"]] = {"bind": "/tmp/resources", "mode": "rw"}
+        vols[self.data_directories[test_id]["minifi_config_dir"]] = {"bind": "/tmp/minifi_config", "mode": "rw"}
+        vols[self.data_directories[test_id]["nifi_config_dir"]] = {"bind": "/tmp/nifi_config", "mode": "rw"}
         return vols
 
     @staticmethod
@@ -77,6 +111,7 @@ class DockerTestDirectoryBindings:
         logging.info('Writing %d bytes of content to file: %s', len(contents), file_abs_path)
         with open(file_abs_path, 'ab') as test_input_file:
             test_input_file.write(contents)
+        os.chmod(file_abs_path, 0o0777)
 
     def put_test_resource(self, test_id, file_name, contents):
         """
@@ -95,9 +130,6 @@ class DockerTestDirectoryBindings:
         file_abs_path = os.path.join(self.docker_path_to_local_path(test_id, path), file_name)
         self.put_file_contents(file_abs_path, contents)
 
-    def get_out_subdir(self, test_id, dir):
-        return os.path.join(self.data_directories[test_id]["output_dir"], dir)
-
     def rm_out_child(self, test_id, dir):
         child = os.path.join(self.data_directories[test_id]["output_dir"], dir)
         logging.info('Removing %s from output folder', child)
