@@ -1,12 +1,38 @@
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+
 import uuid
 import yaml
 
 from ..core.Processor import Processor
 from ..core.InputPort import InputPort
+from ..core.Funnel import Funnel
 
 
 class Minifi_flow_yaml_serializer:
-    def serialize(self, connectable, root=None, visited=None):
+    def serialize(self, start_nodes):
+        res = None
+        visited = None
+
+        for node in start_nodes:
+            res, visited = self.serialize_node(node, res, visited)
+
+        return yaml.dump(res, default_flow_style=False)
+
+    def serialize_node(self, connectable, root=None, visited=None):
         if visited is None:
             visited = []
 
@@ -16,6 +42,7 @@ class Minifi_flow_yaml_serializer:
                     'name': 'MiNiFi Flow'
                 },
                 'Processors': [],
+                'Funnels': [],
                 'Connections': [],
                 'Remote Processing Groups': [],
                 'Controller Services': []
@@ -62,14 +89,15 @@ class Minifi_flow_yaml_serializer:
             res['Processors'].append({
                 'name': connectable_name,
                 'id': str(connectable.uuid),
-                'class': 'org.apache.nifi.processors.standard.' + connectable.clazz,
+                'class': connectable.class_prefix + connectable.clazz,
                 'scheduling strategy': connectable.schedule['scheduling strategy'],
                 'scheduling period': connectable.schedule['scheduling period'],
                 'penalization period': connectable.schedule['penalization period'],
                 'yield period': connectable.schedule['yield period'],
                 'run duration nanos': connectable.schedule['run duration nanos'],
                 'Properties': connectable.properties,
-                'auto-terminated relationships list': connectable.auto_terminate
+                'auto-terminated relationships list': connectable.auto_terminate,
+                'max concurrent tasks': connectable.max_concurrent_tasks
             })
 
             for svc in connectable.controller_services:
@@ -84,6 +112,11 @@ class Minifi_flow_yaml_serializer:
                     'Properties': svc.properties
                 })
 
+        if isinstance(connectable, Funnel):
+            res['Funnels'].append({
+                'id': str(connectable.uuid)
+            })
+
         for conn_name in connectable.connections:
             conn_procs = connectable.connections[conn_name]
 
@@ -92,21 +125,23 @@ class Minifi_flow_yaml_serializer:
                     res['Connections'].append({
                         'name': str(uuid.uuid4()),
                         'source id': str(connectable.uuid),
-                        'source relationship name': conn_name,
                         'destination id': str(proc.uuid),
                         'drop empty': ("true" if proc.drop_empty_flowfiles else "false")
                     })
+                    if (all(str(connectable.uuid) != x['id'] for x in res['Funnels'])):
+                        res['Connections'][-1]['source relationship name'] = conn_name
                     if proc not in visited:
-                        self.serialize(proc, res, visited)
+                        self.serialize_node(proc, res, visited)
             else:
                 res['Connections'].append({
                     'name': str(uuid.uuid4()),
                     'source id': str(connectable.uuid),
-                    'source relationship name': conn_name,
-                    'destination id': str(conn_procs.uuid)
+                    'destination id': str(conn_procs.uuid),
+                    'drop empty': ("true" if proc.drop_empty_flowfiles else "false")
                 })
+                if (all(str(connectable.uuid) != x['id'] for x in res['Funnels'])):
+                    res['Connections'][-1]['source relationship name'] = conn_name
                 if conn_procs not in visited:
-                    self.serialize(conn_procs, res, visited)
+                    self.serialize_node(conn_procs, res, visited)
 
-        if root is None:
-            return yaml.dump(res, default_flow_style=False)
+        return (res, visited)
