@@ -15,34 +15,22 @@
  * limitations under the License.
  */
 
-#include "ConfigFile.h"
+#include <fstream>
+#include <optional>
+#include <string>
+#include <vector>
 
-#include "gsl/gsl-lite.hpp"
+#include "ConfigFile.h"
+#include "properties/Configuration.h"
+
+#include "utils/gsl.h"
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/file/FileUtils.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace encrypt_config {
-
-class ConfigFileTestAccessor {
- public:
-  static std::vector<std::string> mergeProperties(std::vector<std::string> left, const std::vector<std::string>& right) {
-    return ConfigFile::mergeProperties(left, right);
-  }
-};
-
-}  // namespace encrypt_config
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
-
 using org::apache::nifi::minifi::encrypt_config::ConfigFile;
-using org::apache::nifi::minifi::encrypt_config::ConfigFileTestAccessor;
+using org::apache::nifi::minifi::Configuration;
 
 TEST_CASE("ConfigLine can be constructed from a line", "[encrypt-config][constructor]") {
   auto line_is_parsed_correctly = [](const std::string& line, const std::string& expected_key, const std::string& expected_value) {
@@ -102,26 +90,25 @@ TEST_CASE("ConfigFile creates an empty object from a nonexistent file", "[encryp
 
 TEST_CASE("ConfigFile can parse a simple config file", "[encrypt-config][constructor]") {
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
-  REQUIRE(test_file.size() == 101);
+  REQUIRE(test_file.size() == 100);
 }
 
 TEST_CASE("ConfigFile can test whether a key is present", "[encrypt-config][hasValue]") {
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
-  REQUIRE(test_file.hasValue("nifi.version"));
-  REQUIRE(test_file.hasValue("nifi.c2.flow.id"));  // present but blank
-  REQUIRE(!test_file.hasValue("nifi.remote.input.secure"));  // commented out
+  REQUIRE(test_file.hasValue(Configuration::nifi_c2_flow_id));  // present but blank
+  REQUIRE(!test_file.hasValue(Configuration::nifi_remote_input_secure));  // commented out
   REQUIRE(!test_file.hasValue("nifi.this.property.does.not.exist"));
 }
 
 TEST_CASE("ConfigFile can read empty properties correctly", "[encrypt-config][constructor]") {
   ConfigFile test_file{std::ifstream{"resources/with-additional-sensitive-props.minifi.properties"}};
-  REQUIRE(test_file.size() == 103);
+  REQUIRE(test_file.size() == 102);
 
-  auto empty_property = test_file.getValue("nifi.security.need.ClientAuth");
+  auto empty_property = test_file.getValue(Configuration::nifi_security_need_ClientAuth);
   REQUIRE(empty_property);
   REQUIRE(empty_property->empty());
 
-  auto whitespace_property = test_file.getValue("nifi.security.client.certificate");  // value = " \t\r"
+  auto whitespace_property = test_file.getValue(Configuration::nifi_security_client_certificate);  // value = " \t\r"
   REQUIRE(whitespace_property);
   REQUIRE(whitespace_property->empty());
 }
@@ -130,11 +117,11 @@ TEST_CASE("ConfigFile can find the value for a key", "[encrypt-config][getValue]
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
 
   SECTION("valid key") {
-    REQUIRE(test_file.getValue("nifi.bored.yield.duration") == utils::optional<std::string>{"10 millis"});
+    REQUIRE(test_file.getValue(Configuration::nifi_bored_yield_duration) == "10 millis");
   }
 
   SECTION("nonexistent key") {
-    REQUIRE(test_file.getValue("nifi.bored.panda") == utils::nullopt);
+    REQUIRE(test_file.getValue("nifi.bored.panda") == std::nullopt);
   }
 }
 
@@ -142,8 +129,8 @@ TEST_CASE("ConfigFile can update the value for a key", "[encrypt-config][update]
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
 
   SECTION("valid key") {
-    test_file.update("nifi.bored.yield.duration", "20 millis");
-    REQUIRE(test_file.getValue("nifi.bored.yield.duration") == utils::optional<std::string>{"20 millis"});
+    test_file.update(Configuration::nifi_bored_yield_duration, "20 millis");
+    REQUIRE(test_file.getValue(Configuration::nifi_bored_yield_duration) == "20 millis");
   }
 
   SECTION("nonexistent key") {
@@ -155,9 +142,9 @@ TEST_CASE("ConfigFile can add a new setting after an existing setting", "[encryp
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
 
   SECTION("valid key") {
-    test_file.insertAfter("nifi.rest.api.password", "nifi.rest.api.password.protected", "my-cipher-name");
-    REQUIRE(test_file.size() == 102);
-    REQUIRE(test_file.getValue("nifi.rest.api.password.protected") == utils::optional<std::string>{"my-cipher-name"});
+    test_file.insertAfter(Configuration::nifi_rest_api_password, "nifi.rest.api.password.protected", "my-cipher-name");
+    REQUIRE(test_file.size() == 101);
+    REQUIRE(test_file.getValue("nifi.rest.api.password.protected") == "my-cipher-name");
   }
 
   SECTION("nonexistent key") {
@@ -171,24 +158,24 @@ TEST_CASE("ConfigFile can add a new setting at the end", "[encrypt-config][appen
   const std::string KEY = "nifi.bootstrap.sensitive.key";
   const std::string VALUE = "aa411f289c91685ef9d5a9e5a4fad9393ff4c7a78ab978484323488caed7a9ab";
   test_file.append(KEY, VALUE);
-  REQUIRE(test_file.size() == 102);
-  REQUIRE(test_file.getValue(KEY) == utils::make_optional(VALUE));
+  REQUIRE(test_file.size() == 101);
+  REQUIRE(test_file.getValue(KEY) == std::make_optional(VALUE));
 }
 
 TEST_CASE("ConfigFile can write to a new file", "[encrypt-config][writeTo]") {
   ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
-  test_file.update("nifi.bored.yield.duration", "20 millis");
+  test_file.update(Configuration::nifi_bored_yield_duration, "20 millis");
 
-  char format[] = "/tmp/ConfigFileTests.tmp.XXXXXX";
-  std::string temp_dir = utils::file::create_temp_directory(format);
-  auto remove_directory = gsl::finally([&temp_dir]() { utils::file::delete_dir(temp_dir); });
+  TestController test_controller;
+  std::string temp_dir = test_controller.createTempDirectory();
+  auto remove_directory = minifi::gsl::finally([&temp_dir]() { utils::file::delete_dir(temp_dir); });
   std::string file_path = utils::file::concat_path(temp_dir, "minifi.properties");
 
   test_file.writeTo(file_path);
 
   ConfigFile test_file_copy{std::ifstream{file_path}};
   REQUIRE(test_file.size() == test_file_copy.size());
-  REQUIRE(test_file_copy.getValue("nifi.bored.yield.duration") == utils::optional<std::string>{"20 millis"});
+  REQUIRE(test_file_copy.getValue(Configuration::nifi_bored_yield_duration) == "20 millis");
 }
 
 TEST_CASE("ConfigFile will throw if we try to write to an invalid file name", "[encrypt-config][writeTo]") {
@@ -197,41 +184,18 @@ TEST_CASE("ConfigFile will throw if we try to write to an invalid file name", "[
   REQUIRE_THROWS(test_file.writeTo(file_path));
 }
 
-TEST_CASE("ConfigFile can merge lists of property names", "[encrypt-config][mergeProperties]") {
-  using vector = std::vector<std::string>;
-
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{}, vector{}) == vector{});
-
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{}) == vector{"a"});
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{"a"}) == vector{"a"});
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{"b"}) == (vector{"a", "b"}));
-
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"c"}) == (vector{"a", "b", "c"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"a", "b"}) == (vector{"a", "b"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"a", "c"}) == (vector{"a", "b", "c"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"b", "c"}) == (vector{"a", "b", "c"}));
-
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{" a"}) == vector{"a"});
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{"a "}) == vector{"a"});
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a"}, vector{" a "}) == vector{"a"});
-
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"\tc"}) == (vector{"a", "b", "c"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"a\n", "b"}) == (vector{"a", "b"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"a", "c\r\n"}) == (vector{"a", "b", "c"}));
-  REQUIRE(ConfigFileTestAccessor::mergeProperties(vector{"a", "b"}, vector{"b\n", "\t c"}) == (vector{"a", "b", "c"}));
-}
-
 TEST_CASE("ConfigFile can find the list of sensitive properties", "[encrypt-config][getSensitiveProperties]") {
   SECTION("default properties") {
     ConfigFile test_file{std::ifstream{"resources/minifi.properties"}};
-    std::vector<std::string> expected_properties{"nifi.rest.api.password"};
+    std::vector<std::string> expected_properties{Configuration::nifi_rest_api_password};
     REQUIRE(test_file.getSensitiveProperties() == expected_properties);
   }
 
   SECTION("with additional properties") {
     ConfigFile test_file{std::ifstream{"resources/with-additional-sensitive-props.minifi.properties"}};
     std::vector<std::string> expected_properties{
-        "nifi.c2.enable", "nifi.flow.configuration.file", "nifi.rest.api.password", "nifi.security.client.pass.phrase"};
+        Configuration::nifi_c2_enable, Configuration::nifi_flow_configuration_file,
+        Configuration::nifi_rest_api_password, Configuration::nifi_security_client_pass_phrase};
     REQUIRE(test_file.getSensitiveProperties() == expected_properties);
   }
 }
