@@ -20,6 +20,8 @@
 
 #include <set>
 
+#include "core/Resource.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -36,13 +38,11 @@ const core::Property AWSCredentialsService::UseDefaultCredentials(
 
 const core::Property AWSCredentialsService::AccessKey(
     core::PropertyBuilder::createProperty("Access Key")->withDescription("Specifies the AWS Access Key.")
-    ->supportsExpressionLanguage(true)
     ->build());
 
 const core::Property AWSCredentialsService::SecretKey(
     core::PropertyBuilder::createProperty("Secret Key")
     ->withDescription("Specifies the AWS Secret Key.")
-    ->supportsExpressionLanguage(true)
     ->build());
 
 const core::Property AWSCredentialsService::CredentialsFile(
@@ -76,13 +76,15 @@ void AWSCredentialsService::onEnable() {
   }
 }
 
-minifi::utils::optional<Aws::Auth::AWSCredentials> AWSCredentialsService::getAWSCredentials() {
-  if (!aws_credentials_ || aws_credentials_->IsExpiredOrEmpty()) {
+std::optional<Aws::Auth::AWSCredentials> AWSCredentialsService::getAWSCredentials() {
+  if (aws_credentials_provider_.getUseDefaultCredentials() || !aws_credentials_ || aws_credentials_->IsExpiredOrEmpty()) {
     aws_credentials_ = aws_credentials_provider_.getAWSCredentials();
   }
   return aws_credentials_;
 }
 
+REGISTER_RESOURCE(AWSCredentialsService, "AWS Credentials Management Service");
+
 }  // namespace controllers
 }  // namespace aws
 }  // namespace minifi
