@@ -23,6 +23,10 @@
 #include <set>
 #include <memory>
 
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -46,20 +50,22 @@ const core::Relationship DeleteS3Object::Failure("failure", "FlowFiles are route
 
 void DeleteS3Object::initialize() {
   // Add new supported properties
-  updateSupportedProperties({ObjectKey, Version});
+  setSupportedProperties({Bucket, AccessKey, SecretKey, CredentialsFile, CredentialsFile, AWSCredentialsProviderService, Region, CommunicationsTimeout,
+                          EndpointOverrideURL, ProxyHost, ProxyPort, ProxyUsername, ProxyPassword, UseDefaultCredentials, ObjectKey, Version});
   // Set the supported relationships
   setSupportedRelationships({Failure, Success});
 }
 
-minifi::utils::optional<aws::s3::DeleteObjectRequestParameters> DeleteS3Object::buildDeleteS3RequestParams(
+std::optional<aws::s3::DeleteObjectRequestParameters> DeleteS3Object::buildDeleteS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const {
-  aws::s3::DeleteObjectRequestParameters params(common_properties.credentials, client_config_);
+  gsl_Expects(client_config_);
+  aws::s3::DeleteObjectRequestParameters params(common_properties.credentials, *client_config_);
   context->getProperty(ObjectKey, params.object_key, flow_file);
   if (params.object_key.empty() && (!flow_file->getAttribute("filename", params.object_key) || params.object_key.empty())) {
     logger_->log_error("No Object Key is set and default object key 'filename' attribute could not be found!");
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   logger_->log_debug("DeleteS3Object: Object Key [%s]", params.object_key);
 
@@ -72,7 +78,7 @@ minifi::utils::optional<aws::s3::DeleteObjectRequestParameters> DeleteS3Object::
 }
 
 void DeleteS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
-  logger_->log_debug("DeleteS3Object onTrigger");
+  logger_->log_trace("DeleteS3Object onTrigger");
   std::shared_ptr<core::FlowFile> flow_file = session->get();
   if (!flow_file) {
     context->yield();
@@ -100,6 +106,8 @@ void DeleteS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &cont
   }
 }
 
+REGISTER_RESOURCE(DeleteS3Object, "This Processor deletes FlowFiles on an Amazon S3 Bucket.");
+
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
