@@ -20,10 +20,11 @@
 
 #pragma once
 
-#include <sstream>
-#include <utility>
 #include <memory>
+#include <optional>
+#include <sstream>
 #include <string>
+#include <utility>
 
 #include "S3Processor.h"
 #include "utils/GeneralUtils.h"
@@ -51,7 +52,7 @@ class DeleteS3Object : public S3Processor {
   static const core::Relationship Success;
 
   explicit DeleteS3Object(const std::string& name, const minifi::utils::Identifier& uuid = minifi::utils::Identifier())
-    : S3Processor(name, uuid, logging::LoggerFactory<DeleteS3Object>::getLogger()) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<DeleteS3Object>::getLogger()) {
   }
 
   ~DeleteS3Object() override = default;
@@ -64,20 +65,22 @@ class DeleteS3Object : public S3Processor {
     return core::annotation::Input::INPUT_REQUIRED;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
   friend class ::S3TestsFixture<DeleteS3Object>;
 
   explicit DeleteS3Object(const std::string& name, const minifi::utils::Identifier& uuid, std::unique_ptr<aws::s3::S3RequestSender> s3_request_sender)
-    : S3Processor(name, uuid, logging::LoggerFactory<DeleteS3Object>::getLogger(), std::move(s3_request_sender)) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<DeleteS3Object>::getLogger(), std::move(s3_request_sender)) {
   }
 
-  minifi::utils::optional<aws::s3::DeleteObjectRequestParameters> buildDeleteS3RequestParams(
+  std::optional<aws::s3::DeleteObjectRequestParameters> buildDeleteS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const;
 };
 
-REGISTER_RESOURCE(DeleteS3Object, "This Processor deletes FlowFiles on an Amazon S3 Bucket.");
-
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
