@@ -23,6 +23,11 @@
 #include <set>
 #include <memory>
 
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
+#include "utils/OptionalUtils.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -53,7 +58,8 @@ const core::Relationship FetchS3Object::Failure("failure", "FlowFiles are routed
 
 void FetchS3Object::initialize() {
   // Add new supported properties
-  updateSupportedProperties({ObjectKey, Version, RequesterPays});
+  setSupportedProperties({Bucket, AccessKey, SecretKey, CredentialsFile, CredentialsFile, AWSCredentialsProviderService, Region, CommunicationsTimeout,
+                          EndpointOverrideURL, ProxyHost, ProxyPort, ProxyUsername, ProxyPassword, UseDefaultCredentials, ObjectKey, Version, RequesterPays});
   // Set the supported relationships
   setSupportedRelationships({Failure, Success});
 }
@@ -65,18 +71,19 @@ void FetchS3Object::onSchedule(const std::shared_ptr<core::ProcessContext> &cont
   logger_->log_debug("FetchS3Object: RequesterPays [%s]", requester_pays_ ? "true" : "false");
 }
 
-minifi::utils::optional<aws::s3::GetObjectRequestParameters> FetchS3Object::buildFetchS3RequestParams(
+std::optional<aws::s3::GetObjectRequestParameters> FetchS3Object::buildFetchS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const {
-  minifi::aws::s3::GetObjectRequestParameters get_object_params(common_properties.credentials, client_config_);
+  gsl_Expects(client_config_);
+  minifi::aws::s3::GetObjectRequestParameters get_object_params(common_properties.credentials, *client_config_);
   get_object_params.bucket = common_properties.bucket;
   get_object_params.requester_pays = requester_pays_;
 
   context->getProperty(ObjectKey, get_object_params.object_key, flow_file);
   if (get_object_params.object_key.empty() && (!flow_file->getAttribute("filename", get_object_params.object_key) || get_object_params.object_key.empty())) {
     logger_->log_error("No Object Key is set and default object key 'filename' attribute could not be found!");
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   logger_->log_debug("FetchS3Object: Object Key [%s]", get_object_params.object_key);
 
@@ -87,7 +94,7 @@ minifi::utils::optional<aws::s3::GetObjectRequestParameters> FetchS3Object::buil
 }
 
 void FetchS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
-  logger_->log_debug("FetchS3Object onTrigger");
+  logger_->log_trace("FetchS3Object onTrigger");
   std::shared_ptr<core::FlowFile> flow_file = session->get();
   if (!flow_file) {
     context->yield();
@@ -106,10 +113,13 @@ void FetchS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &conte
     return;
   }
 
-  WriteCallback callback(flow_file->getSize(), *get_object_params, s3_wrapper_);
-  session->write(flow_file, &callback);
+  std::optional<minifi::aws::s3::GetObjectResult> result;
+  session->write(flow_file, [&get_object_params, &result, this](const std::shared_ptr<io::BaseStream>& stream) -> int64_t {
+    result = s3_wrapper_.getObject(*get_object_params, *stream);
+    return (result | minifi::utils::map(&s3::GetObjectResult::write_size)).value_or(0);
+  });
 
-  if (callback.result_) {
+  if (result) {
     auto putAttributeIfNotEmpty = [&](const std::string& attribute, const std::string& value) {
       if (!value.empty()) {
         session->putAttribute(flow_file, attribute, value);
@@ -118,15 +128,15 @@ void FetchS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &conte
 
     logger_->log_debug("Successfully fetched S3 object %s from bucket %s", get_object_params->object_key, get_object_params->bucket);
     session->putAttribute(flow_file, "s3.bucket", get_object_params->bucket);
-    session->putAttribute(flow_file, core::SpecialFlowAttribute::PATH, callback.result_->path);
-    session->putAttribute(flow_file, core::SpecialFlowAttribute::ABSOLUTE_PATH, callback.result_->absolute_path);
-    session->putAttribute(flow_file, core::SpecialFlowAttribute::FILENAME, callback.result_->filename);
-    putAttributeIfNotEmpty(core::SpecialFlowAttribute::MIME_TYPE, callback.result_->mime_type);
-    putAttributeIfNotEmpty("s3.etag", callback.result_->etag);
-    putAttributeIfNotEmpty("s3.expirationTime", callback.result_->expiration.expiration_time);
-    putAttributeIfNotEmpty("s3.expirationTimeRuleId", callback.result_->expiration.expiration_time_rule_id);
-    putAttributeIfNotEmpty("s3.sseAlgorithm", callback.result_->ssealgorithm);
-    putAttributeIfNotEmpty("s3.version", callback.result_->version);
+    session->putAttribute(flow_file, core::SpecialFlowAttribute::PATH, result->path);
+    session->putAttribute(flow_file, core::SpecialFlowAttribute::ABSOLUTE_PATH, result->absolute_path);
+    session->putAttribute(flow_file, core::SpecialFlowAttribute::FILENAME, result->filename);
+    putAttributeIfNotEmpty(core::SpecialFlowAttribute::MIME_TYPE, result->mime_type);
+    putAttributeIfNotEmpty("s3.etag", result->etag);
+    putAttributeIfNotEmpty("s3.expirationTime", result->expiration.expiration_time);
+    putAttributeIfNotEmpty("s3.expirationTimeRuleId", result->expiration.expiration_time_rule_id);
+    putAttributeIfNotEmpty("s3.sseAlgorithm", result->ssealgorithm);
+    putAttributeIfNotEmpty("s3.version", result->version);
     session->transfer(flow_file, Success);
   } else {
     logger_->log_error("Failed to fetch S3 object %s from bucket %s", get_object_params->object_key, get_object_params->bucket);
@@ -134,6 +144,8 @@ void FetchS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &conte
   }
 }
 
+REGISTER_RESOURCE(FetchS3Object, "This Processor retrieves the contents of an S3 Object and writes it to the content of a FlowFile.");
+
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
