@@ -20,12 +20,14 @@
 
 #pragma once
 
+#include <memory>
+#include <optional>
+#include <string>
 #include <sstream>
 #include <utility>
-#include <memory>
-#include <string>
 #include <vector>
 
+#include "io/StreamPipe.h"
 #include "S3Processor.h"
 #include "utils/GeneralUtils.h"
 
@@ -53,7 +55,7 @@ class FetchS3Object : public S3Processor {
   static const core::Relationship Success;
 
   explicit FetchS3Object(const std::string& name, const minifi::utils::Identifier& uuid = minifi::utils::Identifier())
-    : S3Processor(name, uuid, logging::LoggerFactory<FetchS3Object>::getLogger()) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<FetchS3Object>::getLogger()) {
   }
 
   ~FetchS3Object() override = default;
@@ -62,44 +64,22 @@ class FetchS3Object : public S3Processor {
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(uint64_t flow_size, const minifi::aws::s3::GetObjectRequestParameters& get_object_params, aws::s3::S3Wrapper& s3_wrapper)
-      : flow_size_(flow_size)
-      , get_object_params_(get_object_params)
-      , s3_wrapper_(s3_wrapper) {
-    }
-
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
-      std::vector<uint8_t> buffer;
-      result_ = s3_wrapper_.getObject(get_object_params_, *stream);
-      if (!result_) {
-        return 0;
-      }
-
-      return result_->write_size;
-    }
-
-    uint64_t flow_size_;
-
-    const minifi::aws::s3::GetObjectRequestParameters& get_object_params_;
-    aws::s3::S3Wrapper& s3_wrapper_;
-    uint64_t write_size_ = 0;
-    minifi::utils::optional<minifi::aws::s3::GetObjectResult> result_ = minifi::utils::nullopt;
-  };
-
  private:
   core::annotation::Input getInputRequirement() const override {
     return core::annotation::Input::INPUT_REQUIRED;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
   friend class ::S3TestsFixture<FetchS3Object>;
 
   explicit FetchS3Object(const std::string& name, const minifi::utils::Identifier& uuid, std::unique_ptr<aws::s3::S3RequestSender> s3_request_sender)
-    : S3Processor(name, uuid, logging::LoggerFactory<FetchS3Object>::getLogger(), std::move(s3_request_sender)) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<FetchS3Object>::getLogger(), std::move(s3_request_sender)) {
   }
 
-  minifi::utils::optional<aws::s3::GetObjectRequestParameters> buildFetchS3RequestParams(
+  std::optional<aws::s3::GetObjectRequestParameters> buildFetchS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const;
@@ -107,8 +87,6 @@ class FetchS3Object : public S3Processor {
   bool requester_pays_ = false;
 };
 
-REGISTER_RESOURCE(FetchS3Object, "This Processor retrieves the contents of an S3 Object and writes it to the content of a FlowFile.");
-
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
