@@ -29,6 +29,9 @@
 #include "properties/Properties.h"
 #include "utils/StringUtils.h"
 #include "utils/MapUtils.h"
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -106,8 +109,9 @@ const core::Relationship PutS3Object::Failure("failure", "FlowFiles are routed t
 
 void PutS3Object::initialize() {
   // Add new supported properties
-  updateSupportedProperties({ObjectKey, ContentType, StorageClass, FullControlUserList, ReadPermissionUserList,
-    ReadACLUserList, WriteACLUserList, CannedACL, ServerSideEncryption});
+  setSupportedProperties({Bucket, AccessKey, SecretKey, CredentialsFile, CredentialsFile, AWSCredentialsProviderService, Region, CommunicationsTimeout,
+                          EndpointOverrideURL, ProxyHost, ProxyPort, ProxyUsername, ProxyPassword, UseDefaultCredentials, ObjectKey, ContentType, StorageClass,
+                          FullControlUserList, ReadPermissionUserList, ReadACLUserList, WriteACLUserList, CannedACL, ServerSideEncryption});
   // Set the supported relationships
   setSupportedRelationships({Failure, Success});
 }
@@ -202,11 +206,12 @@ bool PutS3Object::setAccessControl(
   return setCannedAcl(context, flow_file, put_s3_request_params);
 }
 
-minifi::utils::optional<aws::s3::PutObjectRequestParameters> PutS3Object::buildPutS3RequestParams(
+std::optional<aws::s3::PutObjectRequestParameters> PutS3Object::buildPutS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const {
-  aws::s3::PutObjectRequestParameters params(common_properties.credentials, client_config_);
+  gsl_Expects(client_config_);
+  aws::s3::PutObjectRequestParameters params(common_properties.credentials, *client_config_);
   params.setClientConfig(common_properties.proxy, common_properties.endpoint_override_url);
   params.bucket = common_properties.bucket;
   params.user_metadata_map = user_metadata_map_;
@@ -216,7 +221,7 @@ minifi::utils::optional<aws::s3::PutObjectRequestParameters> PutS3Object::buildP
   context->getProperty(ObjectKey, params.object_key, flow_file);
   if (params.object_key.empty() && (!flow_file->getAttribute("filename", params.object_key) || params.object_key.empty())) {
     logger_->log_error("No Object Key is set and default object key 'filename' attribute could not be found!");
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   logger_->log_debug("PutS3Object: Object Key [%s]", params.object_key);
 
@@ -224,7 +229,7 @@ minifi::utils::optional<aws::s3::PutObjectRequestParameters> PutS3Object::buildP
   logger_->log_debug("PutS3Object: Content Type [%s]", params.content_type);
 
   if (!setAccessControl(context, flow_file, params)) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   return params;
 }
@@ -256,7 +261,7 @@ void PutS3Object::setAttributes(
 }
 
 void PutS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
-  logger_->log_debug("PutS3Object onTrigger");
+  logger_->log_trace("PutS3Object onTrigger");
   std::shared_ptr<core::FlowFile> flow_file = session->get();
   if (!flow_file) {
     context->yield();
@@ -276,8 +281,8 @@ void PutS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &context
   }
 
   PutS3Object::ReadCallback callback(flow_file->getSize(), *put_s3_request_params, s3_wrapper_);
-  session->read(flow_file, &callback);
-  if (callback.result_ == minifi::utils::nullopt) {
+  session->read(flow_file, std::ref(callback));
+  if (!callback.result_.has_value()) {
     logger_->log_error("Failed to upload S3 object to bucket '%s'", put_s3_request_params->bucket);
     session->transfer(flow_file, Failure);
   } else {
@@ -287,6 +292,8 @@ void PutS3Object::onTrigger(const std::shared_ptr<core::ProcessContext> &context
   }
 }
 
+REGISTER_RESOURCE(PutS3Object, "This Processor puts FlowFiles to an Amazon S3 Bucket.");
+
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
