@@ -20,17 +20,20 @@
 
 #pragma once
 
+#include <algorithm>
+#include <map>
+#include <memory>
+#include <optional>
+#include <set>
 #include <sstream>
+#include <string>
 #include <utility>
 #include <vector>
-#include <memory>
-#include <string>
-#include <set>
-#include <map>
-#include <algorithm>
 
+#include "io/StreamPipe.h"
 #include "S3Processor.h"
-#include "utils/GeneralUtils.h"
+#include "utils/gsl.h"
+#include "utils/Id.h"
 
 template<typename T>
 class S3TestsFixture;
@@ -66,7 +69,7 @@ class PutS3Object : public S3Processor {
   static const core::Relationship Success;
 
   explicit PutS3Object(const std::string& name, const minifi::utils::Identifier& uuid = minifi::utils::Identifier())
-    : S3Processor(name, uuid, logging::LoggerFactory<PutS3Object>::getLogger()) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<PutS3Object>::getLogger()) {
   }
 
   ~PutS3Object() override = default;
@@ -75,7 +78,7 @@ class PutS3Object : public S3Processor {
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
-  class ReadCallback : public InputStreamCallback {
+  class ReadCallback {
    public:
     static const uint64_t MAX_SIZE;
     static const uint64_t BUFFER_SIZE;
@@ -86,35 +89,36 @@ class PutS3Object : public S3Processor {
       , s3_wrapper_(s3_wrapper) {
     }
 
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) {
       if (flow_size_ > MAX_SIZE) {
         return -1;
       }
-      std::vector<uint8_t> buffer;
+      std::vector<std::byte> buffer;
+      buffer.resize(BUFFER_SIZE);
       auto data_stream = std::make_shared<std::stringstream>();
       read_size_ = 0;
       while (read_size_ < flow_size_) {
-        auto next_read_size = (std::min)(flow_size_ - read_size_, BUFFER_SIZE);
-        int read_ret = stream->read(buffer, next_read_size);
-        if (read_ret < 0) {
+        const auto next_read_size = (std::min)(flow_size_ - read_size_, BUFFER_SIZE);
+        const auto read_ret = stream->read(gsl::make_span(buffer).subspan(0, next_read_size));
+        if (io::isError(read_ret)) {
           return -1;
         }
         if (read_ret > 0) {
-          data_stream->write(reinterpret_cast<char*>(buffer.data()), next_read_size);
+          data_stream->write(reinterpret_cast<char*>(buffer.data()), gsl::narrow<std::streamsize>(next_read_size));
           read_size_ += read_ret;
         } else {
           break;
         }
       }
       result_ = s3_wrapper_.putObject(options_, data_stream);
-      return read_size_;
+      return gsl::narrow<int64_t>(read_size_);
     }
 
     uint64_t flow_size_;
     const minifi::aws::s3::PutObjectRequestParameters& options_;
     aws::s3::S3Wrapper& s3_wrapper_;
     uint64_t read_size_ = 0;
-    minifi::utils::optional<minifi::aws::s3::PutObjectResult> result_ = minifi::utils::nullopt;
+    std::optional<minifi::aws::s3::PutObjectResult> result_;
   };
 
  private:
@@ -122,10 +126,14 @@ class PutS3Object : public S3Processor {
     return core::annotation::Input::INPUT_REQUIRED;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
   friend class ::S3TestsFixture<PutS3Object>;
 
   explicit PutS3Object(const std::string& name, const minifi::utils::Identifier& uuid, std::unique_ptr<aws::s3::S3RequestSender> s3_request_sender)
-    : S3Processor(name, uuid, logging::LoggerFactory<PutS3Object>::getLogger(), std::move(s3_request_sender)) {
+    : S3Processor(name, uuid, core::logging::LoggerFactory<PutS3Object>::getLogger(), std::move(s3_request_sender)) {
   }
 
   void fillUserMetadata(const std::shared_ptr<core::ProcessContext> &context);
@@ -137,7 +145,7 @@ class PutS3Object : public S3Processor {
     const std::shared_ptr<core::FlowFile> &flow_file,
     const aws::s3::PutObjectRequestParameters &put_s3_request_params,
     const minifi::aws::s3::PutObjectResult &put_object_result) const;
-  minifi::utils::optional<aws::s3::PutObjectRequestParameters> buildPutS3RequestParams(
+  std::optional<aws::s3::PutObjectRequestParameters> buildPutS3RequestParams(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file,
     const CommonProperties &common_properties) const;
@@ -148,8 +156,6 @@ class PutS3Object : public S3Processor {
   std::string server_side_encryption_;
 };
 
-REGISTER_RESOURCE(PutS3Object, "This Processor puts FlowFiles to an Amazon S3 Bucket.");
-
 }  // namespace processors
 }  // namespace aws
 }  // namespace minifi
