@@ -23,7 +23,9 @@
 #include <string>
 #include <set>
 #include <memory>
+#include <utility>
 
+#include "core/ProcessContext.h"
 #include "S3Wrapper.h"
 #include "AWSCredentialsService.h"
 #include "properties/Properties.h"
@@ -114,43 +116,39 @@ const core::Property S3Processor::UseDefaultCredentials(
     ->isRequired(true)
     ->build());
 
-S3Processor::S3Processor(const std::string& name, const minifi::utils::Identifier& uuid, const std::shared_ptr<logging::Logger> &logger)
-  : core::Processor(name, uuid)
-  , logger_(logger) {
-  setSupportedProperties({Bucket, AccessKey, SecretKey, CredentialsFile, CredentialsFile, AWSCredentialsProviderService, Region, CommunicationsTimeout,
-                          EndpointOverrideURL, ProxyHost, ProxyPort, ProxyUsername, ProxyPassword, UseDefaultCredentials});
+S3Processor::S3Processor(const std::string& name, const minifi::utils::Identifier& uuid, std::shared_ptr<core::logging::Logger> logger)
+  : core::Processor(name, uuid),
+    logger_(std::move(logger)) {
 }
 
-S3Processor::S3Processor(const std::string& name, const minifi::utils::Identifier& uuid, const std::shared_ptr<logging::Logger> &logger, std::unique_ptr<aws::s3::S3RequestSender> s3_request_sender)
-  : core::Processor(name, uuid)
-  , logger_(logger)
-  , s3_wrapper_(std::move(s3_request_sender)) {
-  setSupportedProperties({Bucket, AccessKey, SecretKey, CredentialsFile, CredentialsFile, AWSCredentialsProviderService, Region, CommunicationsTimeout,
-                          EndpointOverrideURL, ProxyHost, ProxyPort, ProxyUsername, ProxyPassword, UseDefaultCredentials});
+S3Processor::S3Processor(const std::string& name, const minifi::utils::Identifier& uuid, std::shared_ptr<core::logging::Logger> logger, std::unique_ptr<aws::s3::S3RequestSender> s3_request_sender)
+  : core::Processor(name, uuid),
+    logger_(std::move(logger)),
+    s3_wrapper_(std::move(s3_request_sender)) {
 }
 
-minifi::utils::optional<Aws::Auth::AWSCredentials> S3Processor::getAWSCredentialsFromControllerService(const std::shared_ptr<core::ProcessContext> &context) const {
+std::optional<Aws::Auth::AWSCredentials> S3Processor::getAWSCredentialsFromControllerService(const std::shared_ptr<core::ProcessContext> &context) const {
   std::string service_name;
   if (!context->getProperty(AWSCredentialsProviderService.getName(), service_name) || service_name.empty()) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
 
   std::shared_ptr<core::controller::ControllerService> service = context->getControllerService(service_name);
   if (!service) {
     logger_->log_error("AWS credentials service with name: '%s' could not be found", service_name);
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
 
   auto aws_credentials_service = std::dynamic_pointer_cast<minifi::aws::controllers::AWSCredentialsService>(service);
   if (!aws_credentials_service) {
     logger_->log_error("Controller service with name: '%s' is not an AWS credentials service", service_name);
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
 
   return aws_credentials_service->getAWSCredentials();
 }
 
-minifi::utils::optional<Aws::Auth::AWSCredentials> S3Processor::getAWSCredentials(
+std::optional<Aws::Auth::AWSCredentials> S3Processor::getAWSCredentials(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file) {
   auto service_cred = getAWSCredentialsFromControllerService(context);
@@ -178,13 +176,13 @@ minifi::utils::optional<Aws::Auth::AWSCredentials> S3Processor::getAWSCredential
   return aws_credentials_provider.getAWSCredentials();
 }
 
-minifi::utils::optional<aws::s3::ProxyOptions> S3Processor::getProxy(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::FlowFile> &flow_file) {
+std::optional<aws::s3::ProxyOptions> S3Processor::getProxy(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::FlowFile> &flow_file) {
   aws::s3::ProxyOptions proxy;
   context->getProperty(ProxyHost, proxy.host, flow_file);
   std::string port_str;
   if (context->getProperty(ProxyPort, port_str, flow_file) && !port_str.empty() && !core::Property::StringToInt(port_str, proxy.port)) {
     logger_->log_error("Proxy port invalid");
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   context->getProperty(ProxyUsername, proxy.username, flow_file);
   context->getProperty(ProxyPassword, proxy.password, flow_file);
@@ -195,45 +193,45 @@ minifi::utils::optional<aws::s3::ProxyOptions> S3Processor::getProxy(const std::
 }
 
 void S3Processor::onSchedule(const std::shared_ptr<core::ProcessContext>& context, const std::shared_ptr<core::ProcessSessionFactory>& /*sessionFactory*/) {
+  client_config_ = Aws::Client::ClientConfiguration();
   std::string value;
   if (!context->getProperty(Bucket.getName(), value) || value.empty()) {
     throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Bucket property missing or invalid");
   }
 
-  if (!context->getProperty(Region.getName(), client_config_.region) || client_config_.region.empty() || REGIONS.count(client_config_.region) == 0) {
+  if (!context->getProperty(Region.getName(), client_config_->region) || client_config_->region.empty() || REGIONS.count(client_config_->region) == 0) {
     throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Region property missing or invalid");
   }
-  logger_->log_debug("S3Processor: Region [%s]", client_config_.region);
+  logger_->log_debug("S3Processor: Region [%s]", client_config_->region);
 
-  uint64_t timeout_val;
-  if (context->getProperty(CommunicationsTimeout.getName(), value) && !value.empty() && core::Property::getTimeMSFromString(value, timeout_val)) {
-    logger_->log_debug("S3Processor: Communications Timeout [%llu]", timeout_val);
-    client_config_.connectTimeoutMs = gsl::narrow<int64_t>(timeout_val);
+  if (auto communications_timeout = context->getProperty<core::TimePeriodValue>(CommunicationsTimeout)) {
+    logger_->log_debug("S3Processor: Communications Timeout %" PRId64 " ms", communications_timeout->getMilliseconds().count());
+    client_config_->connectTimeoutMs = gsl::narrow<int64_t>(communications_timeout->getMilliseconds().count());
   } else {
     throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Communications Timeout missing or invalid");
   }
 }
 
-minifi::utils::optional<CommonProperties> S3Processor::getCommonELSupportedProperties(
+std::optional<CommonProperties> S3Processor::getCommonELSupportedProperties(
     const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::FlowFile> &flow_file) {
   CommonProperties properties;
   if (!context->getProperty(Bucket, properties.bucket, flow_file) || properties.bucket.empty()) {
     logger_->log_error("Bucket '%s' is invalid or empty!", properties.bucket);
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   logger_->log_debug("S3Processor: Bucket [%s]", properties.bucket);
 
   auto credentials = getAWSCredentials(context, flow_file);
   if (!credentials) {
     logger_->log_error("AWS Credentials have not been set!");
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   properties.credentials = credentials.value();
 
   auto proxy = getProxy(context, flow_file);
   if (!proxy) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   properties.proxy = proxy.value();
 
