@@ -20,15 +20,14 @@
 #include "S3Wrapper.h"
 
 #include <memory>
-#include <regex>
 #include <utility>
 #include <vector>
 
-#include "utils/GeneralUtils.h"
+#include "S3ClientRequestSender.h"
 #include "utils/StringUtils.h"
 #include "utils/file/FileUtils.h"
+#include "utils/gsl.h"
 #include "utils/RegexUtils.h"
-#include "S3ClientRequestSender.h"
 
 namespace org {
 namespace apache {
@@ -39,10 +38,10 @@ namespace s3 {
 
 void HeadObjectResult::setFilePaths(const std::string& key) {
   absolute_path = key;
-  std::tie(path, filename) = minifi::utils::file::FileUtils::split_path(key, true /*force_posix*/);
+  std::tie(path, filename) = minifi::utils::file::split_path(key, true /*force_posix*/);
 }
 
-S3Wrapper::S3Wrapper() : request_sender_(minifi::utils::make_unique<S3ClientRequestSender>()) {
+S3Wrapper::S3Wrapper() : request_sender_(std::make_unique<S3ClientRequestSender>()) {
 }
 
 S3Wrapper::S3Wrapper(std::unique_ptr<S3RequestSender>&& request_sender) : request_sender_(std::move(request_sender)) {
@@ -58,11 +57,11 @@ void S3Wrapper::setCannedAcl(Aws::S3::Model::PutObjectRequest& request, const st
 
 Expiration S3Wrapper::getExpiration(const std::string& expiration) {
   minifi::utils::Regex expr("expiry-date=\"(.*)\", rule-id=\"(.*)\"");
-  const auto match = expr.match(expiration);
-  const auto& results = expr.getResult();
-  if (!match || results.size() < 3)
+  minifi::utils::SMatch matches;
+  const bool matched = minifi::utils::regexSearch(expiration, matches, expr);
+  if (!matched || matches.size() < 3)
     return Expiration{};
-  return Expiration{results[1], results[2]};
+  return Expiration{matches[1], matches[2]};
 }
 
 std::string S3Wrapper::getEncryptionString(Aws::S3::Model::ServerSideEncryption encryption) {
@@ -80,7 +79,7 @@ std::string S3Wrapper::getEncryptionString(Aws::S3::Model::ServerSideEncryption
   return "";
 }
 
-minifi::utils::optional<PutObjectResult> S3Wrapper::putObject(const PutObjectRequestParameters& put_object_params, std::shared_ptr<Aws::IOStream> data_stream) {
+std::optional<PutObjectResult> S3Wrapper::putObject(const PutObjectRequestParameters& put_object_params, std::shared_ptr<Aws::IOStream> data_stream) {
   Aws::S3::Model::PutObjectRequest request;
   request.SetBucket(put_object_params.bucket);
   request.SetKey(put_object_params.object_key);
@@ -97,7 +96,7 @@ minifi::utils::optional<PutObjectResult> S3Wrapper::putObject(const PutObjectReq
 
   auto aws_result = request_sender_->sendPutObjectRequest(request, put_object_params.credentials, put_object_params.client_config);
   if (!aws_result) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
 
   PutObjectResult result;
@@ -124,26 +123,27 @@ bool S3Wrapper::deleteObject(const DeleteObjectRequestParameters& params) {
 
 int64_t S3Wrapper::writeFetchedBody(Aws::IOStream& source, const int64_t data_size, io::BaseStream& output) {
   std::vector<uint8_t> buffer(4096);
-  int64_t write_size = 0;
-  while (write_size < data_size) {
-    auto next_write_size = (std::min)(data_size - write_size, static_cast<int64_t>(4096));
-    if (!source.read(reinterpret_cast<char*>(buffer.data()), next_write_size)) {
+  size_t write_size = 0;
+  if (data_size < 0) return 0;
+  while (write_size < gsl::narrow<uint64_t>(data_size)) {
+    const auto next_write_size = (std::min)(gsl::narrow<size_t>(data_size) - write_size, size_t{4096});
+    if (!source.read(reinterpret_cast<char*>(buffer.data()), gsl::narrow<std::streamsize>(next_write_size))) {
       return -1;
     }
-    auto ret = output.write(buffer.data(), next_write_size);
-    if (ret < 0) {
-      return ret;
+    const auto ret = output.write(buffer.data(), next_write_size);
+    if (io::isError(ret)) {
+      return -1;
     }
     write_size += next_write_size;
   }
-  return write_size;
+  return gsl::narrow<int64_t>(write_size);
 }
 
-minifi::utils::optional<GetObjectResult> S3Wrapper::getObject(const GetObjectRequestParameters& get_object_params, io::BaseStream& out_body) {
+std::optional<GetObjectResult> S3Wrapper::getObject(const GetObjectRequestParameters& get_object_params, io::BaseStream& out_body) {
   auto request = createFetchObjectRequest<Aws::S3::Model::GetObjectRequest>(get_object_params);
   auto aws_result = request_sender_->sendGetObjectRequest(request, get_object_params.credentials, get_object_params.client_config);
   if (!aws_result) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   auto result = fillFetchObjectResult<Aws::S3::Model::GetObjectResult, GetObjectResult>(get_object_params, *aws_result);
   result.write_size = writeFetchedBody(aws_result->GetBody(), aws_result->GetContentLength(), out_body);
@@ -161,7 +161,7 @@ void S3Wrapper::addListResults(const Aws::Vector<Aws::S3::Model::ObjectVersion>&
     attributes.etag = minifi::utils::StringUtils::removeFramingCharacters(version.GetETag(), '"');
     attributes.filename = version.GetKey();
     attributes.is_latest = version.GetIsLatest();
-    attributes.last_modified = version.GetLastModified().Millis();
+    attributes.last_modified = version.GetLastModified().UnderlyingTimestamp();
     attributes.length = version.GetSize();
     attributes.store_class = VERSION_STORAGE_CLASS_MAP.at(version.GetStorageClass());
     attributes.version = version.GetVersionId();
@@ -180,21 +180,21 @@ void S3Wrapper::addListResults(const Aws::Vector<Aws::S3::Model::Object>& conten
     attributes.etag = minifi::utils::StringUtils::removeFramingCharacters(object.GetETag(), '"');
     attributes.filename = object.GetKey();
     attributes.is_latest = true;
-    attributes.last_modified = object.GetLastModified().Millis();
+    attributes.last_modified = object.GetLastModified().UnderlyingTimestamp();
     attributes.length = object.GetSize();
     attributes.store_class = OBJECT_STORAGE_CLASS_MAP.at(object.GetStorageClass());
     listed_objects.push_back(attributes);
   }
 }
 
-minifi::utils::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listVersions(const ListRequestParameters& params) {
+std::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listVersions(const ListRequestParameters& params) {
   auto request = createListRequest<Aws::S3::Model::ListObjectVersionsRequest>(params);
   std::vector<ListedObjectAttributes> attribute_list;
-  nonstd::optional_lite::optional<Aws::S3::Model::ListObjectVersionsResult> aws_result;
+  std::optional<Aws::S3::Model::ListObjectVersionsResult> aws_result;
   do {
     aws_result = request_sender_->sendListVersionsRequest(request, params.credentials, params.client_config);
     if (!aws_result) {
-      return minifi::utils::nullopt;
+      return std::nullopt;
     }
     const auto& versions = aws_result->GetVersions();
     logger_->log_debug("AWS S3 List operation returned %zu versions. This result is%s truncated.", versions.size(), aws_result->GetIsTruncated() ? "" : " not");
@@ -208,14 +208,14 @@ minifi::utils::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listVers
   return attribute_list;
 }
 
-minifi::utils::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listObjects(const ListRequestParameters& params) {
+std::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listObjects(const ListRequestParameters& params) {
   auto request = createListRequest<Aws::S3::Model::ListObjectsV2Request>(params);
   std::vector<ListedObjectAttributes> attribute_list;
-  nonstd::optional_lite::optional<Aws::S3::Model::ListObjectsV2Result> aws_result;
+  std::optional<Aws::S3::Model::ListObjectsV2Result> aws_result;
   do {
     aws_result = request_sender_->sendListObjectsRequest(request, params.credentials, params.client_config);
     if (!aws_result) {
-      return minifi::utils::nullopt;
+      return std::nullopt;
     }
     const auto& objects = aws_result->GetContents();
     logger_->log_debug("AWS S3 List operation returned %zu objects. This result is%s truncated.", objects.size(), aws_result->GetIsTruncated() ? "" : "not");
@@ -228,15 +228,15 @@ minifi::utils::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listObje
   return attribute_list;
 }
 
-minifi::utils::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listBucket(const ListRequestParameters& params) {
-  last_bucket_list_timestamp_ = Aws::Utils::DateTime::CurrentTimeMillis();
+std::optional<std::vector<ListedObjectAttributes>> S3Wrapper::listBucket(const ListRequestParameters& params) {
+  last_bucket_list_timestamp_ = gsl::narrow<uint64_t>(Aws::Utils::DateTime::CurrentTimeMillis());
   if (params.use_versions) {
     return listVersions(params);
   }
   return listObjects(params);
 }
 
-minifi::utils::optional<std::map<std::string, std::string>> S3Wrapper::getObjectTags(const GetObjectTagsParameters& params) {
+std::optional<std::map<std::string, std::string>> S3Wrapper::getObjectTags(const GetObjectTagsParameters& params) {
   Aws::S3::Model::GetObjectTaggingRequest request;
   request.SetBucket(params.bucket);
   request.SetKey(params.object_key);
@@ -245,7 +245,7 @@ minifi::utils::optional<std::map<std::string, std::string>> S3Wrapper::getObject
   }
   auto aws_result = request_sender_->sendGetObjectTaggingRequest(request, params.credentials, params.client_config);
   if (!aws_result) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   std::map<std::string, std::string> tags;
   for (const auto& tag : aws_result->GetTagSet()) {
@@ -254,11 +254,11 @@ minifi::utils::optional<std::map<std::string, std::string>> S3Wrapper::getObject
   return tags;
 }
 
-minifi::utils::optional<HeadObjectResult> S3Wrapper::headObject(const HeadObjectRequestParameters& head_object_params) {
+std::optional<HeadObjectResult> S3Wrapper::headObject(const HeadObjectRequestParameters& head_object_params) {
   auto request = createFetchObjectRequest<Aws::S3::Model::HeadObjectRequest>(head_object_params);
   auto aws_result = request_sender_->sendHeadObjectRequest(request, head_object_params.credentials, head_object_params.client_config);
   if (!aws_result) {
-    return minifi::utils::nullopt;
+    return std::nullopt;
   }
   return fillFetchObjectResult<Aws::S3::Model::HeadObjectResult, HeadObjectResult>(head_object_params, aws_result.value());
 }
