@@ -21,34 +21,28 @@
 #include <string>
 #include <memory>
 
-#include "core/Resource.h"
 #include "core/controller/ControllerService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "storage/AzureStorageCredentials.h"
+#include "utils/Export.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace azure {
-namespace controllers {
+namespace org::apache::nifi::minifi::azure::controllers {
 
 class AzureStorageCredentialsService : public core::controller::ControllerService {
  public:
-  static const core::Property StorageAccountName;
-  static const core::Property StorageAccountKey;
-  static const core::Property SASToken;
-  static const core::Property CommonStorageAccountEndpointSuffix;
-  static const core::Property ConnectionString;
+  EXTENSIONAPI static const core::Property StorageAccountName;
+  EXTENSIONAPI static const core::Property StorageAccountKey;
+  EXTENSIONAPI static const core::Property SASToken;
+  EXTENSIONAPI static const core::Property CommonStorageAccountEndpointSuffix;
+  EXTENSIONAPI static const core::Property ConnectionString;
+  EXTENSIONAPI static const core::Property UseManagedIdentityCredentials;
 
   explicit AzureStorageCredentialsService(const std::string& name, const minifi::utils::Identifier& uuid = {})
-      : ControllerService(name, uuid),
-        logger_(logging::LoggerFactory<AzureStorageCredentialsService>::getLogger()) {
+      : ControllerService(name, uuid) {
   }
 
   explicit AzureStorageCredentialsService(const std::string& name, const std::shared_ptr<Configure>& /*configuration*/)
-      : ControllerService(name),
-        logger_(logging::LoggerFactory<AzureStorageCredentialsService>::getLogger()) {
+      : ControllerService(name) {
   }
 
   void initialize() override;
@@ -66,20 +60,13 @@ class AzureStorageCredentialsService : public core::controller::ControllerServic
 
   void onEnable() override;
 
-  std::string getConnectionString() const {
-    return credentials_.getConnectionString();
+  storage::AzureStorageCredentials getCredentials() const {
+    return credentials_;
   }
 
  private:
   storage::AzureStorageCredentials credentials_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AzureStorageCredentialsService>::getLogger();
 };
 
-REGISTER_RESOURCE(AzureStorageCredentialsService, "Azure Storage Credentials Management Service");
-
-}  // namespace controllers
-}  // namespace azure
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+}  // namespace org::apache::nifi::minifi::azure::controllers
