@@ -20,6 +20,13 @@
  */
 #include "ListenHTTP.h"
 
+#include <memory>
+#include <set>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "core/Resource.h"
 #include "utils/gsl.h"
 
 namespace org {
@@ -249,14 +256,13 @@ void ListenHTTP::processIncomingFlowFile(core::ProcessSession *session) {
 
   if (type == "response_body" && handler_) {
     ResponseBody response;
-    ResponseBodyReadCallback cb(&response.body);
     flow_file->getAttribute("filename", response.uri);
     flow_file->getAttribute("mime.type", response.mime_type);
     if (response.mime_type.empty()) {
       logger_->log_warn("Using default mime type of application/octet-stream for response body file: %s", response.uri);
       response.mime_type = "application/octet-stream";
     }
-    session->read(flow_file, &cb);
+    response.body = to_string(session->readBuffer(flow_file));
     handler_->setResponseBody(std::move(response));
   }
 
@@ -275,8 +281,10 @@ void ListenHTTP::processRequestBuffer(core::ProcessSession *session) {
     session->add(flow_file);
 
     if (flow_file_buffer_pair.second) {
-      WriteCallback callback(std::move(flow_file_buffer_pair.second));
-      session->write(flow_file, &callback);
+      session->write(flow_file, [request_content = flow_file_buffer_pair.second.get()](const std::shared_ptr<io::BaseStream>& stream) -> int64_t {
+        const auto write_ret = stream->write(request_content->getBuffer());
+        return io::isError(write_ret) ? -1 : gsl::narrow<int64_t>(write_ret);
+      });
     }
 
     session->transfer(flow_file, Success);
@@ -289,8 +297,7 @@ ListenHTTP::Handler::Handler(std::string base_uri, core::ProcessContext *context
     : base_uri_(std::move(base_uri)),
       auth_dn_regex_(std::move(auth_dn_regex)),
       headers_as_attrs_regex_(std::move(header_as_attrs_regex)),
-      process_context_(context),
-      logger_(logging::LoggerFactory<ListenHTTP::Handler>::getLogger()) {
+      process_context_(context) {
   context->getProperty(BufferSize.getName(), buffer_size_);
   logger_->log_debug("ListenHTTP using %s: %zu", BufferSize.getName(), buffer_size_);
 }
@@ -314,7 +321,7 @@ void ListenHTTP::Handler::setHeaderAttributes(const mg_request_info *req_info, c
 
     if (strcmp("filename", header->name) == 0) {
       flow_file->setAttribute("filename", header->value);
-    } else if (std::regex_match(header->name, headers_as_attrs_regex_)) {
+    } else if (utils::regexMatch(header->name, headers_as_attrs_regex_)) {
       flow_file->setAttribute(header->name, header->value);
     }
   }
@@ -368,7 +375,7 @@ bool ListenHTTP::Handler::authRequest(mg_connection *conn, const mg_request_info
   // If this is a two-way TLS connection, authorize the peer against the configured pattern
   bool authorized = true;
   if (req_info->is_ssl && req_info->client_cert != nullptr) {
-    if (!std::regex_match(req_info->client_cert->subject, auth_dn_regex_)) {
+    if (!utils::regexMatch(req_info->client_cert->subject, auth_dn_regex_)) {
       mg_printf(conn, "HTTP/1.1 403 Forbidden\r\n"
                 "Content-Type: text/html\r\n"
                 "Content-Length: 0\r\n\r\n");
@@ -470,30 +477,28 @@ void ListenHTTP::Handler::writeBody(mg_connection *conn, const mg_request_info *
 }
 
 std::unique_ptr<io::BufferStream> ListenHTTP::Handler::createContentBuffer(struct mg_connection *conn, const struct mg_request_info *req_info) {
-  auto content_buffer = utils::make_unique<io::BufferStream>();
-  int64_t rlen;
-  int64_t nlen = 0;
+  auto content_buffer = std::make_unique<io::BufferStream>();
+  size_t nlen = 0;
   int64_t tlen = req_info->content_length;
   uint8_t buf[16384];
 
   // if we have no content length we should call mg_read until
   // there is no data left from the stream to be HTTP/1.1 compliant
-  while (tlen == -1 || nlen < tlen) {
-    rlen = tlen == -1 ? sizeof(buf) : tlen - nlen;
-
-    if (rlen > (int64_t) sizeof(buf)) {
-      rlen = (int64_t) sizeof(buf);
+  while (tlen == -1 || (tlen > 0 && nlen < gsl::narrow<size_t>(tlen))) {
+    auto rlen = tlen == -1 ? sizeof(buf) : gsl::narrow<size_t>(tlen) - nlen;
+    if (rlen > sizeof(buf)) {
+      rlen = sizeof(buf);
     }
 
     // Read a buffer of data from client
-    rlen = mg_read(conn, &buf[0], (size_t) rlen);
-
-    if (rlen <= 0) {
+    const auto mg_read_return = mg_read(conn, &buf[0], rlen);
+    if (mg_read_return <= 0) {
       break;
     }
+    rlen = gsl::narrow<size_t>(mg_read_return);
 
     // Transfer buffer data to the output stream
-    content_buffer->write(&buf[0], gsl::narrow<int>(rlen));
+    content_buffer->write(&buf[0], rlen);
 
     nlen += rlen;
   }
@@ -501,14 +506,6 @@ std::unique_ptr<io::BufferStream> ListenHTTP::Handler::createContentBuffer(struc
   return content_buffer;
 }
 
-ListenHTTP::WriteCallback::WriteCallback(std::unique_ptr<io::BufferStream> request_content)
-    : request_content_(std::move(request_content)) {
-}
-
-int64_t ListenHTTP::WriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-  return stream->write(const_cast<uint8_t*>(request_content_->getBuffer()), gsl::narrow<int>(request_content_->size()));
-}
-
 bool ListenHTTP::isSecure() const {
   return (listeningPort.length() > 0) && *listeningPort.rbegin() == 's';
 }
@@ -525,6 +522,13 @@ void ListenHTTP::notifyStop() {
   handler_.reset();
 }
 
+REGISTER_RESOURCE(ListenHTTP, "Starts an HTTP Server and listens on a given base path to transform incoming requests into FlowFiles. The default URI of the Service will be "
+    "http://{hostname}:{port}/contentListener. Only HEAD, POST, and GET requests are supported. PUT, and DELETE will result in an error and the HTTP response status code 405."
+    " The response body text for all requests, by default, is empty (length of 0). A static response body can be set for a given URI by sending input files to ListenHTTP with "
+    "the http.type attribute set to response_body. The response body FlowFile filename attribute is appended to the Base Path property (separated by a /) when mapped to incoming requests. "
+    "The mime.type attribute of the response body FlowFile is used for the Content-type header in responses. Response body content can be cleared by sending an empty (size 0) "
+    "FlowFile for a given URI mapping.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
