@@ -17,11 +17,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __LISTEN_HTTP_H__
-#define __LISTEN_HTTP_H__
+#pragma once
 
+#include <map>
 #include <memory>
-#include <regex>
+#include <string>
+#include <utility>
 
 #include <CivetServer.h>
 
@@ -29,9 +30,11 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/MinifiConcurrentQueue.h"
+#include "utils/gsl.h"
+#include "utils/Export.h"
+#include "utils/RegexUtils.h"
 
 namespace org {
 namespace apache {
@@ -48,30 +51,29 @@ class ListenHTTP : public core::Processor {
   /*!
    * Create a new processor
    */
-  ListenHTTP(const std::string& name, const utils::Identifier& uuid = {})
+  explicit ListenHTTP(const std::string& name, const utils::Identifier& uuid = {})
       : Processor(name, uuid),
-        logger_(logging::LoggerFactory<ListenHTTP>::getLogger()),
         batch_size_(0) {
     callbacks_.log_message = &logMessage;
     callbacks_.log_access = &logAccess;
   }
   // Destructor
-  virtual ~ListenHTTP();
+  ~ListenHTTP() override;
   // Processor Name
-  static constexpr char const *ProcessorName = "ListenHTTP";
+  EXTENSIONAPI static constexpr char const *ProcessorName = "ListenHTTP";
   // Supported Properties
-  static core::Property BasePath;
-  static core::Property Port;
-  static core::Property AuthorizedDNPattern;
-  static core::Property SSLCertificate;
-  static core::Property SSLCertificateAuthority;
-  static core::Property SSLVerifyPeer;
-  static core::Property SSLMinimumVersion;
-  static core::Property HeadersAsAttributesRegex;
-  static core::Property BatchSize;
-  static core::Property BufferSize;
+  EXTENSIONAPI static core::Property BasePath;
+  EXTENSIONAPI static core::Property Port;
+  EXTENSIONAPI static core::Property AuthorizedDNPattern;
+  EXTENSIONAPI static core::Property SSLCertificate;
+  EXTENSIONAPI static core::Property SSLCertificateAuthority;
+  EXTENSIONAPI static core::Property SSLVerifyPeer;
+  EXTENSIONAPI static core::Property SSLMinimumVersion;
+  EXTENSIONAPI static core::Property HeadersAsAttributesRegex;
+  EXTENSIONAPI static core::Property BatchSize;
+  EXTENSIONAPI static core::Property BufferSize;
   // Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
 
   void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
   void initialize() override;
@@ -114,60 +116,29 @@ class ListenHTTP : public core::Processor {
     void enqueueRequest(mg_connection *conn, const mg_request_info *req_info, std::unique_ptr<io::BufferStream>);
 
     std::string base_uri_;
-    std::regex auth_dn_regex_;
-    std::regex headers_as_attrs_regex_;
+    utils::Regex auth_dn_regex_;
+    utils::Regex headers_as_attrs_regex_;
     core::ProcessContext *process_context_;
-    std::shared_ptr<logging::Logger> logger_;
+    std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ListenHTTP>::getLogger();
     std::map<std::string, ResponseBody> response_uri_map_;
     std::mutex uri_map_mutex_;
     uint64_t buffer_size_;
     utils::ConcurrentQueue<FlowFileBufferPair> request_buffer_;
   };
 
-  class ResponseBodyReadCallback : public InputStreamCallback {
-   public:
-    explicit ResponseBodyReadCallback(std::string *out_str)
-        : out_str_(out_str) {
-    }
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
-      out_str_->resize(stream->size());
-      uint64_t num_read = stream->read(reinterpret_cast<uint8_t *>(&(*out_str_)[0]),
-                                           gsl::narrow<int>(stream->size()));
-
-      if (num_read != stream->size()) {
-        throw std::runtime_error("GraphReadCallback failed to fully read flow file input stream");
-      }
-
-      return num_read;
-    }
-
-   private:
-    std::string *out_str_;
-  };
-
-  // Write callback for transferring data from HTTP request to content repo
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(std::unique_ptr<io::BufferStream>);
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
-
-   private:
-    std::unique_ptr<io::BufferStream> request_content_;
-  };
-
   static int logMessage(const struct mg_connection *conn, const char *message) {
     try {
       struct mg_context* ctx = mg_get_context(conn);
       /* CivetServer stores 'this' as the userdata when calling mg_start */
-      CivetServer* server = static_cast<CivetServer*>(mg_get_user_data(ctx));
+      auto* const server = static_cast<CivetServer*>(mg_get_user_data(ctx));
       if (server == nullptr) {
         return 0;
       }
-      std::shared_ptr<logging::Logger>* logger = static_cast<std::shared_ptr<logging::Logger>*>(const_cast<void*>(server->getUserContext()));
+      auto* const logger = static_cast<std::shared_ptr<core::logging::Logger>*>(const_cast<void*>(server->getUserContext()));
       if (logger == nullptr) {
         return 0;
       }
-      logging::LOG_ERROR((*logger)) << "CivetWeb error: " << message;
+      core::logging::LOG_ERROR((*logger)) << "CivetWeb error: " << message;
     } catch (...) {
     }
     return 0;
@@ -177,19 +148,20 @@ class ListenHTTP : public core::Processor {
     try {
       struct mg_context* ctx = mg_get_context(conn);
       /* CivetServer stores 'this' as the userdata when calling mg_start */
-      CivetServer* server = static_cast<CivetServer*>(mg_get_user_data(ctx));
+      auto* server = static_cast<CivetServer*>(mg_get_user_data(ctx));
       if (server == nullptr) {
         return 0;
       }
-      std::shared_ptr<logging::Logger>* logger = static_cast<std::shared_ptr<logging::Logger>*>(const_cast<void*>(server->getUserContext()));
+      auto* logger = static_cast<std::shared_ptr<core::logging::Logger>*>(const_cast<void*>(server->getUserContext()));
       if (logger == nullptr) {
         return 0;
       }
-      logging::LOG_DEBUG((*logger)) << "CivetWeb access: " << message;
+      core::logging::LOG_DEBUG((*logger)) << "CivetWeb access: " << message;
     } catch (...) {
     }
     return 0;
   }
+
  protected:
   void notifyStop() override;
 
@@ -203,7 +175,7 @@ class ListenHTTP : public core::Processor {
   void processIncomingFlowFile(core::ProcessSession *session);
   void processRequestBuffer(core::ProcessSession *session);
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ListenHTTP>::getLogger();
   CivetCallbacks callbacks_;
   std::unique_ptr<CivetServer> server_;
   std::unique_ptr<Handler> handler_;
@@ -211,17 +183,8 @@ class ListenHTTP : public core::Processor {
   uint64_t batch_size_;
 };
 
-REGISTER_RESOURCE(ListenHTTP, "Starts an HTTP Server and listens on a given base path to transform incoming requests into FlowFiles. The default URI of the Service will be "
-    "http://{hostname}:{port}/contentListener. Only HEAD, POST, and GET requests are supported. PUT, and DELETE will result in an error and the HTTP response status code 405."
-    " The response body text for all requests, by default, is empty (length of 0). A static response body can be set for a given URI by sending input files to ListenHTTP with "
-    "the http.type attribute set to response_body. The response body FlowFile filename attribute is appended to the Base Path property (separated by a /) when mapped to incoming requests. "
-    "The mime.type attribute of the response body FlowFile is used for the Content-type header in responses. Response body content can be cleared by sending an empty (size 0) "
-    "FlowFile for a given URI mapping.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
