@@ -23,6 +23,7 @@
 #include <iostream>
 
 #include "TestBase.h"
+#include "Catch.h"
 
 #include "utils/file/FileUtils.h"
 #include "processors/GetFile.h"
@@ -32,6 +33,9 @@
 #include "client/HTTPClient.h"
 #include "controllers/SSLContextService.h"
 #include "properties/Configure.h"
+#include "FlowController.h"
+#include "SchedulingAgent.h"
+#include "core/ProcessGroup.h"
 
 class ListenHTTPTestsFixture {
  public:
@@ -50,15 +54,14 @@ class ListenHTTPTestsFixture {
     LogTestController::getInstance().setDebug<minifi::core::ProcessGroup>();
     LogTestController::getInstance().setDebug<minifi::core::Processor>();
     LogTestController::getInstance().setTrace<minifi::core::ProcessSession>();
-    LogTestController::getInstance().setTrace<processors::ListenHTTP>();
-    LogTestController::getInstance().setTrace<processors::ListenHTTP::Handler>();
-    LogTestController::getInstance().setDebug<processors::LogAttribute>();
+    LogTestController::getInstance().setTrace<minifi::processors::ListenHTTP>();
+    LogTestController::getInstance().setTrace<minifi::processors::ListenHTTP::Handler>();
+    LogTestController::getInstance().setDebug<minifi::processors::LogAttribute>();
     LogTestController::getInstance().setDebug<utils::HTTPClient>();
     LogTestController::getInstance().setDebug<minifi::controllers::SSLContextService>();
 
     // Create temporary directories
-    char tmp_dir_format[] = "/tmp/gt.XXXXXX";
-    tmp_dir = testController.createTempDirectory(tmp_dir_format);
+    tmp_dir = testController.createTempDirectory();
     REQUIRE(!tmp_dir.empty());
 
     // Define test input file
