@@ -18,13 +18,14 @@
 
 #include "CoapConnector.h"
 
-#include "core/logging/LoggerConfiguration.h"
-#include "core/controller/ControllerService.h"
 #include <string>
 #include <memory>
 #include <set>
+
+#include "core/logging/LoggerConfiguration.h"
+#include "core/controller/ControllerService.h"
 #include "core/Property.h"
-#include "CoapConnector.h"
+#include "core/Resource.h"
 #include "io/validation.h"
 #include "properties/Configure.h"
 
@@ -46,13 +47,13 @@ core::Property CoapConnectorService::MaxQueueSize(
     core::PropertyBuilder::createProperty("Max Queue Size")->withDescription("Max queue size for received data ")->withDefaultValue<uint64_t>(1000)->isRequired(false)->build());
 
 void CoapConnectorService::initialize() {
-  if (initialized_)
+  std::lock_guard<std::mutex> lock(initialization_mutex_);
+  if (initialized_) {
     return;
+  }
 
   CoapMessaging::getInstance();
 
-  std::lock_guard<std::mutex> lock(initialization_mutex_);
-
   ControllerService::initialize();
 
   initializeProperties();
@@ -66,7 +67,7 @@ void CoapConnectorService::onEnable() {
     core::Property::StringToInt(port_str, port_);
   } else {
     // this is the case where we aren't being used in the context of a single controller service.
-    if (configuration_->get("nifi.c2.agent.coap.host", host_) && configuration_->get("nifi.c2.agent.coap.port", port_str)) {
+    if (configuration_->get(Configuration::nifi_c2_agent_coap_host, host_) && configuration_->get(Configuration::nifi_c2_agent_coap_port, port_str)) {
       core::Property::StringToInt(port_str, port_);
     }
   }
@@ -90,6 +91,8 @@ void CoapConnectorService::initializeProperties() {
   setSupportedProperties(supportedProperties);
 }
 
+REGISTER_INTERNAL_RESOURCE(CoapConnectorService);
+
 } /* namespace controllers */
 } /* namespace coap */
 } /* namespace minifi */
