@@ -15,8 +15,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_CONTROLLERS_COAPCONNECTOR_H_
-#define LIBMINIFI_INCLUDE_CONTROLLERS_COAPCONNECTOR_H_
+#pragma once
+
+#include <memory>
+#include <string>
+#include <mutex>
+#include <atomic>
+#include <utility>
 
 #include "core/logging/LoggerConfiguration.h"
 #include "core/controller/ControllerService.h"
@@ -26,9 +31,6 @@
 #include "coap_functions.h"
 #include "coap_connection.h"
 #include "coap_message.h"
-#include <memory>
-#include <unordered_map>
-#include <utility>
 
 namespace org {
 namespace apache {
@@ -87,7 +89,6 @@ class CoapConnectorService : public core::controller::ControllerService {
   CoapResponse sendPayload(uint8_t type, const std::string &endpoint, const CoapMessage * message);
 
  protected:
-
   void initializeProperties();
 
   // connector mutex to controll access to the mapping, above.
@@ -96,16 +97,15 @@ class CoapConnectorService : public core::controller::ControllerService {
   // initialization mutex.
   std::mutex initialization_mutex_;
 
-  std::atomic<bool> initialized_{ false };
+  bool initialized_ = false;
 
  private:
-
   // host connecting to.
   std::string host_;
   // port connecting to
   unsigned int port_{ 0 };
 
-  std::shared_ptr<logging::Logger> logger_{ logging::LoggerFactory<CoapConnectorService>::getLogger() };
+  std::shared_ptr<core::logging::Logger> logger_{ core::logging::LoggerFactory<CoapConnectorService>::getLogger() };
 };
 
 } /* namespace controllers */
@@ -114,5 +114,3 @@ class CoapConnectorService : public core::controller::ControllerService {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* LIBMINIFI_INCLUDE_CONTROLLERS_COAPCONNECTOR_H_ */
