@@ -21,6 +21,7 @@
 #include "coap_functions.h"
 #include "coap_message.h"
 #include "io/BaseStream.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -33,15 +34,14 @@ uint8_t CoapProtocol::REGISTRATION_MSG[8] = { 0x72, 0x65, 0x67, 0x69, 0x73, 0x74
 
 CoapProtocol::CoapProtocol(const std::string &name, const utils::Identifier &uuid)
     : RESTSender(name, uuid),
-      require_registration_(false),
-      logger_(logging::LoggerFactory<CoapProtocol>::getLogger()) {
+      require_registration_(false) {
 }
 
 CoapProtocol::~CoapProtocol() = default;
 
 void CoapProtocol::initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<Configure> &configure) {
   RESTSender::initialize(controller, configure);
-  if (configure->get("nifi.c2.coap.connector.service", controller_service_name_)) {
+  if (configure->get(minifi::Configuration::nifi_c2_coap_connector_service, controller_service_name_)) {
     auto service = controller->getControllerService(controller_service_name_);
     coap_service_ = std::static_pointer_cast<coap::controllers::CoapConnectorService>(service);
   } else {
@@ -65,6 +65,7 @@ int CoapProtocol::writeAcknowledgement(io::BaseStream *stream, const minifi::c2:
     case state::UpdateState::INITIATE:
     case state::UpdateState::FULLY_APPLIED:
     case state::UpdateState::READ_COMPLETE:
+    case state::UpdateState::NO_OPERATION:
       payloadState = 0;
       break;
     case state::UpdateState::NOT_APPLIED:
@@ -237,20 +238,21 @@ minifi::c2::C2Payload CoapProtocol::serialize(const minifi::c2::C2Payload &paylo
     default:
       logger_->log_error("Could not identify operation");
       return minifi::c2::C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR);
-  };
+  }
 
   size_t bsize = stream.size();
 
   CoapMessage msg;
-  msg.data_ = const_cast<uint8_t *>(stream.getBuffer());
+  msg.data_ = const_cast<uint8_t *>(stream.getBuffer().as_span<const uint8_t>().data());
   msg.size_ = bsize;
 
   coap::controllers::CoapResponse message = coap_service_->sendPayload(COAP_REQUEST_POST, endpoint, &msg);
+  const auto message_data = message.getData();
 
   if (isRegistrationMessage(message)) {
     require_registration_ = true;
-  } else if (message.getSize() > 0) {
-    io::BufferStream responseStream(message.getData(), message.getSize());
+  } else if (!message_data.empty()) {
+    io::BufferStream responseStream(message_data);
     responseStream.read(version);
     responseStream.read(size);
     logger_->log_trace("Received ack. version %d. number of operations %d", version, size);
@@ -290,6 +292,8 @@ minifi::c2::C2Payload CoapProtocol::serialize(const minifi::c2::C2Payload &paylo
   return minifi::c2::C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR);
 }
 
+REGISTER_INTERNAL_RESOURCE(CoapProtocol);
+
 } /* namespace c2 */
 } /* namespace coap */
 } /* namespace minifi */
