@@ -15,9 +15,9 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_COAPPROTOCOL_H_
-#define EXTENSIONS_COAPPROTOCOL_H_
+#pragma once
 
+#include <cstring>
 #include <algorithm>
 #include <iostream>
 #include <memory>
@@ -25,6 +25,8 @@
 #include <map>
 #include <string>
 #include <vector>
+#include <stdexcept>
+#include <mutex>
 
 #include "c2/C2Protocol.h"
 #include "io/BaseStream.h"
@@ -34,13 +36,8 @@
 #include "coap2/coap.h"
 #include "coap2/uri.h"
 #include "coap2/address.h"
-#include <stdio.h>
-#include <string.h>
 #include "protocols/RESTSender.h"
 
-#undef RAPIDJSON_ASSERT
-#define RAPIDJSON_ASSERT(x) if(!(x)) throw std::logic_error("rapidjson exception"); //NOLINT
-
 namespace org {
 namespace apache {
 namespace nifi {
@@ -49,7 +46,7 @@ namespace coap {
 namespace c2 {
 
 #define REQUIRE_VALID(x) \
-  if (-1 == x) { \
+  if (io::isError(x)) { \
     return minifi::c2::C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR); \
   }
 
@@ -89,12 +86,12 @@ class CoapProtocol : public minifi::c2::RESTSender {
   // Supported Properties
 
  protected:
-
-  bool isRegistrationMessage(controllers::CoapResponse &response) {
-    if (LIKELY(response.getSize() != 8)) {
+  static bool isRegistrationMessage(controllers::CoapResponse &response) {
+    const auto response_data = response.getData();
+    if (LIKELY(response_data.size() != 8)) {
       return false;
     }
-    return response.getCode() == COAP_RESPONSE_400 && !memcmp(response.getData(), REGISTRATION_MSG, response.getSize());
+    return response.getCode() == COAP_RESPONSE_400 && !memcmp(response_data.data(), REGISTRATION_MSG, response_data.size());
   }
 
   /**
@@ -131,15 +128,14 @@ class CoapProtocol : public minifi::c2::RESTSender {
   std::string controller_service_name_;
 
  private:
-
   static uint8_t REGISTRATION_MSG[8];
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<CoapProtocol>::getLogger();
 };
+
 } /* namespace c2 */
 } /* namespace coap */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* EXTENSIONS_COAPPROTOCOL_H_ */
