@@ -32,6 +32,7 @@
 #include "HTTPClient.h"
 #include "processors/InvokeHTTP.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/StringUtils.h"
 #include "core/Core.h"
 #include "core/logging/Logger.h"
@@ -59,17 +60,16 @@ class VerifyCoAPServer : public CoapIntegrationBase {
  public:
   explicit VerifyCoAPServer(bool isSecure)
       : isSecure(isSecure) {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
     LogTestController::getInstance().setDebug<utils::HTTPClient>();
-    LogTestController::getInstance().setOff<processors::InvokeHTTP>();
+    LogTestController::getInstance().setOff<minifi::processors::InvokeHTTP>();
     LogTestController::getInstance().setDebug<minifi::c2::RESTReceiver>();
     LogTestController::getInstance().setDebug<minifi::c2::C2Agent>();
     LogTestController::getInstance().setTrace<minifi::coap::c2::CoapProtocol>();
-    LogTestController::getInstance().setOff<processors::LogAttribute>();
+    LogTestController::getInstance().setOff<minifi::processors::LogAttribute>();
     LogTestController::getInstance().setOff<minifi::core::ProcessSession>();
     std::fstream file;
     ss << dir << "/" << "tstFile.ext";
@@ -93,10 +93,10 @@ class VerifyCoAPServer : public CoapIntegrationBase {
   }
 
   void queryRootProcessGroup(std::shared_ptr<core::ProcessGroup> pg) override {
-    std::shared_ptr<core::Processor> proc = pg->findProcessorByName("invoke");
+    auto proc = pg->findProcessorByName("invoke");
     assert(proc != nullptr);
 
-    std::shared_ptr<minifi::processors::InvokeHTTP> inv = std::dynamic_pointer_cast<minifi::processors::InvokeHTTP>(proc);
+    const auto* const inv = dynamic_cast<minifi::processors::InvokeHTTP*>(proc);
 
     assert(inv != nullptr);
     std::string url = "";
@@ -141,8 +141,8 @@ class VerifyCoAPServer : public CoapIntegrationBase {
       stream.write("id");
       stream.write("operand");
 
-      auto data = std::unique_ptr<uint8_t[]>(new uint8_t[stream.size()]);
-      memcpy(data.get(), stream.getBuffer(), stream.size());
+      auto data = std::make_unique<uint8_t[]>(stream.size());
+      memcpy(data.get(), stream.getBuffer().data(), stream.getBuffer().size());
       minifi::coap::CoapResponse response(205, std::move(data), stream.size());
       responses.enqueue(std::move(response));
     }
@@ -153,20 +153,17 @@ class VerifyCoAPServer : public CoapIntegrationBase {
         responses.try_dequeue(resp);
         return resp;
       }
-      else {
-        minifi::coap::CoapResponse response(500, 0, 0);
-        return response;
-      }
+      minifi::coap::CoapResponse response(500, 0, 0);
+      return response;
     });
     server->start();
-    configuration->set("c2.enable", "true");
-    configuration->set("c2.agent.class", "test");
-    configuration->set("nifi.c2.root.classes", "DeviceInfoNode,AgentInformation,FlowInformation,RepositoryMetrics");
-    configuration->set("nifi.c2.agent.protocol.class", "CoapProtocol");
-    configuration->set("nifi.c2.agent.coap.host", "127.0.0.1");
-    configuration->set("nifi.c2.agent.coap.port", new_port_str);
-    configuration->set("c2.agent.heartbeat.period", "10");
-    configuration->set("c2.rest.listener.heartbeat.rooturi", path);
+    configuration->set(minifi::Configuration::nifi_c2_enable, "true");
+    configuration->set(minifi::Configuration::nifi_c2_agent_class, "test");
+    configuration->set(minifi::Configuration::nifi_c2_root_classes, "DeviceInfoNode,AgentInformation,FlowInformation,RepositoryMetrics");
+    configuration->set(minifi::Configuration::nifi_c2_agent_protocol_class, "CoapProtocol");
+    configuration->set(minifi::Configuration::nifi_c2_agent_coap_host, "127.0.0.1");
+    configuration->set(minifi::Configuration::nifi_c2_agent_coap_port, new_port_str);
+    configuration->set(minifi::Configuration::nifi_c2_agent_heartbeat_period, "10");
   }
 
  protected:
