@@ -15,13 +15,19 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_COAP_TESTS_COAPINTEGRATIONBASE_H
-#define EXTENSIONS_COAP_TESTS_COAPINTEGRATIONBASE_H
+#pragma once
+
+#include <memory>
+#include <optional>
+#include <string>
+#include <utility>
 
 #include "../tests/TestServer.h"
 #include "CivetServer.h"
 #include "integration/IntegrationBase.h"
 
+using std::literals::chrono_literals::operator""s;
+
 int log_message(const struct mg_connection* /*conn*/, const char *message) {
   puts(message);
   return 1;
@@ -33,7 +39,7 @@ int ssl_enable(void *, void *) {
 
 class CoapIntegrationBase : public IntegrationBase {
  public:
-  CoapIntegrationBase(uint64_t waitTime = 5000)
+  explicit CoapIntegrationBase(std::chrono::seconds waitTime = 5s)
       : IntegrationBase(waitTime),
         server(nullptr) {
   }
@@ -44,7 +50,7 @@ class CoapIntegrationBase : public IntegrationBase {
     server.reset();
   }
 
-  void run(const utils::optional<std::string>& test_file_location = {}, const utils::optional<std::string>& = {}) override {
+  void run(const std::optional<std::string>& test_file_location = {}, const std::optional<std::string>& = {}) override {
     testSetup();
 
     std::shared_ptr<core::Repository> test_repo = std::make_shared<TestRepository>();
@@ -53,24 +59,23 @@ class CoapIntegrationBase : public IntegrationBase {
     if (test_file_location) {
       configuration->set(minifi::Configure::nifi_flow_configuration_file, *test_file_location);
     }
-    configuration->set("c2.agent.heartbeat.period", "200");
+    configuration->set(minifi::Configure::nifi_c2_agent_heartbeat_period, "200");
 
     std::shared_ptr<core::ContentRepository> content_repo = std::make_shared<core::repository::VolatileContentRepository>();
     content_repo->initialize(configuration);
     std::shared_ptr<minifi::io::StreamFactory> stream_factory = minifi::io::StreamFactory::getInstance(configuration);
-    std::unique_ptr<core::FlowConfiguration> yaml_ptr = std::unique_ptr<core::YamlConfiguration>(
-        new core::YamlConfiguration(test_repo, test_repo, content_repo, stream_factory, configuration, test_file_location));
+    auto yaml_ptr = std::make_unique<core::YamlConfiguration>(test_repo, test_repo, content_repo, stream_factory, configuration, test_file_location);
 
     core::YamlConfiguration yaml_config(test_repo, test_repo, content_repo, stream_factory, configuration, test_file_location);
 
-    std::shared_ptr<core::ProcessGroup> pg{ yaml_config.getRoot().release() };
+    std::shared_ptr<core::ProcessGroup> pg{ yaml_config.getRoot() };
 
     queryRootProcessGroup(pg);
 
     std::shared_ptr<TestRepository> repo = std::static_pointer_cast<TestRepository>(test_repo);
 
     std::shared_ptr<minifi::FlowController> controller = std::make_shared<minifi::FlowController>(test_repo, test_flow_repo, configuration, std::move(yaml_ptr), content_repo, DEFAULT_ROOT_GROUP_NAME,
-                                                                                                  true);
+      std::make_shared<utils::file::FileSystem>(), []{});
 
     controller->load();
     controller->start();
@@ -78,7 +83,7 @@ class CoapIntegrationBase : public IntegrationBase {
     runAssertions();
 
     shutdownBeforeFlowController();
-    controller->waitUnload(wait_time_);
+    controller->waitUnload(wait_time_.count());
     controller->stopC2();
 
     cleanup();
@@ -98,16 +103,14 @@ void CoapIntegrationBase::setUrl(std::string url, CivetHandler *handler) {
       return;
     }
     if (scheme == "https" && !key_dir.empty()) {
-      std::string cert = "";
+      std::string cert;
       cert = key_dir + "nifi-cert.pem";
       callback.init_ssl = ssl_enable;
       port += "s";
       callback.log_message = log_message;
-      server = utils::make_unique<TestServer>(port, path, handler, &callback, cert, cert);
+      server = std::make_unique<TestServer>(port, path, handler, &callback, cert, cert);
     } else {
-      server = utils::make_unique<TestServer>(port, path, handler);
+      server = std::make_unique<TestServer>(port, path, handler);
     }
   }
 }
-
-#endif  // EXTENSIONS_COAP_TESTS_COAPINTEGRATIONBASE_H
