@@ -21,15 +21,18 @@
 #include <iomanip>
 #include <random>
 #include <algorithm>
+#include <regex>
+#include <functional>
+#include <string>
 
 #include "rapidjson/reader.h"
 #include "rapidjson/writer.h"
 #include "rapidjson/document.h"
 
-#include <utils/StringUtils.h>
-#include <utils/OsUtils.h>
-#include <expression/Expression.h>
-#include <regex>
+#include "utils/StringUtils.h"
+#include "utils/OsUtils.h"
+#include "expression/Expression.h"
+#include "utils/RegexUtils.h"
 
 #ifndef DISABLE_CURL
 #ifdef WIN32
@@ -58,7 +61,6 @@
 #include <unistd.h>
 #endif
 
-#include "utils/StringUtils.h"
 #include "Driver.h"
 
 #ifdef EXPRESSION_LANGUAGE_USE_DATE
@@ -612,7 +614,7 @@ Value expr_escapeCsv(const std::vector<Value> &args) {
 
 Value expr_format(const std::vector<Value> &args) {
   std::chrono::milliseconds dur(args[0].asUnsignedLong());
-  std::chrono::time_point<std::chrono::system_clock> dt(dur);
+  std::chrono::system_clock::time_point dt(dur);
   auto zone = date::current_zone();
   if (args.size() > 2) {
     zone = date::locate_zone(args[2].asString());
@@ -627,7 +629,7 @@ Value expr_toDate(const std::vector<Value> &args) {
   auto arg_0 = args[0].asString();
   std::istringstream arg_s { arg_0 };
   date::sys_time<std::chrono::milliseconds> t;
-  arg_s >> date::parse(args[1].asString(), t);
+  date::from_stream(arg_s, args[1].asString().c_str(), t);
   auto zone = date::current_zone();
   if (args.size() > 2) {
     zone = date::locate_zone(args[2].asString());
@@ -635,15 +637,14 @@ Value expr_toDate(const std::vector<Value> &args) {
   auto utc = date::locate_zone("UTC");
   auto utct = date::make_zoned(utc, t);
   auto zt = date::make_zoned(zone, utct.get_local_time());
-  return Value(std::chrono::duration_cast<std::chrono::milliseconds>(zt.get_sys_time().time_since_epoch()).count());
+  return Value(int64_t{std::chrono::duration_cast<std::chrono::milliseconds>(zt.get_sys_time().time_since_epoch()).count()});
 }
 
 #else
 
-Value expr_format(const std::vector<Value>& args)
-{
+Value expr_format(const std::vector<Value>& args) {
   const std::chrono::milliseconds dur(args.at(0).asUnsignedLong());
-  const std::chrono::time_point<std::chrono::system_clock> dt(dur);
+  const std::chrono::system_clock::time_point dt(dur);
   const auto unix_time = std::chrono::system_clock::to_time_t(dt);
   const auto zoned_time = [&args, unix_time] {
     std::tm buf{};
@@ -686,9 +687,7 @@ Value expr_toDate(const std::vector<Value>&) {
 #endif  // EXPRESSION_LANGUAGE_USE_DATE
 
 Value expr_now(const std::vector<Value>& /*args*/) {
-  using namespace std::chrono;
-  int64_t unix_time_ms{duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count()};
-  return Value(unix_time_ms);
+  return Value(int64_t{std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch()).count()});
 }
 
 Value expr_unescapeCsv(const std::vector<Value> &args) {
@@ -751,7 +750,7 @@ Value expr_urlDecode(const std::vector<Value> &args) {
     int out_len;
     char *output = curl_easy_unescape(curl, arg_0.c_str(), static_cast<int>(arg_0.length()), &out_len);
     if (output != nullptr) {
-      auto result = std::string(output, static_cast<unsigned long>(out_len));
+      auto result = std::string(output, static_cast<size_t>(out_len));
       curl_free(output);
       curl_easy_cleanup(curl);
       return Value(result);
@@ -772,7 +771,7 @@ Value expr_base64Encode(const std::vector<Value> &args) {
 }
 
 Value expr_base64Decode(const std::vector<Value> &args) {
-  return Value(utils::StringUtils::from_base64(args[0].asString()));
+  return Value(utils::StringUtils::from_base64(args[0].asString(), utils::as_string));
 }
 
 #ifdef EXPRESSION_LANGUAGE_USE_REGEX
@@ -824,16 +823,16 @@ Value expr_replaceEmpty(const std::vector<Value> &args) {
 
 Value expr_matches(const std::vector<Value> &args) {
   const auto &subject = args[0].asString();
-  const std::regex expr = std::regex(args[1].asString());
+  const auto expr = utils::Regex(args[1].asString());
 
-  return Value(std::regex_match(subject.begin(), subject.end(), expr));
+  return Value(utils::regexMatch(subject, expr));
 }
 
 Value expr_find(const std::vector<Value> &args) {
   const auto &subject = args[0].asString();
-  const std::regex expr = std::regex(args[1].asString());
+  const auto expr = utils::Regex(args[1].asString());
 
-  return Value(std::regex_search(subject.begin(), subject.end(), expr));
+  return Value(utils::regexSearch(subject, expr));
 }
 
 #endif  // EXPRESSION_LANGUAGE_USE_REGEX
@@ -1186,7 +1185,7 @@ Expression make_allMatchingAttributes(const std::string &function_name, const st
     std::vector<Expression> out_exprs;
 
     for (const auto &arg : args) {
-      const std::regex attr_regex = std::regex(arg(params).asString());
+      const auto attr_regex = utils::Regex(arg(params).asString());
       const auto cur_flow_file = params.flow_file.lock();
       std::map<std::string, std::string> attrs;
 
@@ -1195,7 +1194,7 @@ Expression make_allMatchingAttributes(const std::string &function_name, const st
       }
 
       for (const auto &attr : attrs) {
-        if (std::regex_match(attr.first.begin(), attr.first.end(), attr_regex)) {
+        if (utils::regexMatch(attr.first, attr_regex)) {
           out_exprs.emplace_back(make_dynamic([=](const Parameters& /*params*/,
                       const std::vector<Expression>& /*sub_exprs*/) -> Value {
                     std::string attr_val;
@@ -1239,7 +1238,7 @@ Expression make_anyMatchingAttribute(const std::string &function_name, const std
     std::vector<Expression> out_exprs;
 
     for (const auto &arg : args) {
-      const std::regex attr_regex = std::regex(arg(params).asString());
+      const auto attr_regex = utils::Regex(arg(params).asString());
       const auto cur_flow_file = params.flow_file.lock();
       std::map<std::string, std::string> attrs;
 
@@ -1248,7 +1247,7 @@ Expression make_anyMatchingAttribute(const std::string &function_name, const std
       }
 
       for (const auto &attr : attrs) {
-        if (std::regex_match(attr.first.begin(), attr.first.end(), attr_regex)) {
+        if (utils::regexMatch(attr.first, attr_regex)) {
           out_exprs.emplace_back(make_dynamic([=](const Parameters& /*params*/,
                       const std::vector<Expression>& /*sub_exprs*/) -> Value {
                     std::string attr_val;
