@@ -17,34 +17,25 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-#include <vector>
-#include <queue>
-#include <map>
-#include <set>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <time.h>
-#include <sstream>
-#include <stdio.h>
-#include <string>
-#include <iostream>
-#include <dirent.h>
-#include <limits.h>
-#include <unistd.h>
-#include <regex>
-
 #include "GetGPS.h"
-#include "core/ProcessContext.h"
-#include "core/ProcessSession.h"
-#include "core/PropertyValidation.h"
+#include <sys/types.h>
+#include <dirent.h>
 
 #define policy_t gps_policy_t
 #include <libgpsmm.h>
 #undef  policy_t
 #define policy_t ambiguous use gps_policy_t
 
+#include <vector>
+#include <set>
+#include <string>
+#include <memory>
+
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/PropertyValidation.h"
+#include "core/Resource.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -159,6 +150,8 @@ void GetGPS::onTrigger(const std::shared_ptr<core::ProcessContext>& /*context*/,
   }
 }
 
+REGISTER_RESOURCE(GetGPS, "Obtains GPS coordinates from the GPSDHost and port.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
