@@ -17,24 +17,20 @@
  */
 #include "HTTPClient.h"
 
-#include <memory>
-#include <climits>
+#include <algorithm>
 #include <cinttypes>
 #include <map>
-#include <vector>
+#include <memory>
 #include <string>
-#include <algorithm>
+#include <vector>
 
 #include "Exception.h"
 #include "utils/gsl.h"
 #include "utils/StringUtils.h"
+#include "core/Resource.h"
 #include "utils/RegexUtils.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace utils {
+namespace org::apache::nifi::minifi::utils {
 
 HTTPClient::HTTPClient(const std::string &url, const std::shared_ptr<minifi::controllers::SSLContextService> ssl_context_service)
     : core::Connectable("HTTPClient"),
@@ -53,6 +49,20 @@ HTTPClient::HTTPClient()
   http_session_ = curl_easy_init();
 }
 
+void HTTPClient::addFormPart(const std::string& content_type, const std::string& name, HTTPUploadCallback* read_callback, const std::optional<std::string>& filename) {
+  if (!form_) {
+    form_ = curl_mime_init(http_session_);
+  }
+  curl_mimepart* part = curl_mime_addpart(form_);
+  curl_mime_type(part, content_type.c_str());
+  if (filename) {
+    curl_mime_filename(part, filename->c_str());
+  }
+  curl_mime_name(part, name.c_str());
+  curl_mime_data_cb(part, read_callback->ptr->getBufferSize(),
+      &utils::HTTPRequestResponse::send_write, nullptr, nullptr, static_cast<void*>(read_callback));
+}
+
 HTTPClient::~HTTPClient() {
   if (nullptr != headers_) {
     curl_slist_free_all(headers_);
@@ -62,6 +72,10 @@ HTTPClient::~HTTPClient() {
     curl_easy_cleanup(http_session_);
     http_session_ = nullptr;
   }
+  if (form_ != nullptr) {
+    curl_mime_free(form_);
+    form_ = nullptr;
+  }
   // forceClose ended up not being the issue in MINIFICPP-667, but leaving here
   // out of good hygiene.
   forceClose();
@@ -80,12 +94,12 @@ void HTTPClient::forceClose() {
 }
 
 int HTTPClient::debug_callback(CURL *handle, curl_infotype type, char *data, size_t size, void *userptr) {
-  std::shared_ptr<logging::Logger>* logger = static_cast<std::shared_ptr<logging::Logger>*>(userptr);
+  auto* const logger = static_cast<std::shared_ptr<core::logging::Logger>*>(userptr);
   if (logger == nullptr) {
     return 0;
   }
   if (type == CURLINFO_TEXT) {
-    logging::LOG_DEBUG((*logger)) << "CURL(" << reinterpret_cast<void*>(handle) << "): " << std::string(data, size);
+    core::logging::LOG_DEBUG((*logger)) << "CURL(" << reinterpret_cast<void*>(handle) << "): " << std::string(data, size);
   }
   return 0;
 }
@@ -124,20 +138,16 @@ void HTTPClient::setDisableHostVerification() {
 
 bool HTTPClient::setSpecificSSLVersion(SSLVersion specific_version) {
 #if CURL_AT_LEAST_VERSION(7, 54, 0)
-  CURLcode ret = CURLE_UNKNOWN_OPTION;
+  // bitwise or of different enum types is deprecated in C++20, but the curl api explicitly supports ORing one of CURL_SSLVERSION and one of CURL_SSLVERSION_MAX
   switch (specific_version) {
     case SSLVersion::TLSv1_0:
-      ret = curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_0 | CURL_SSLVERSION_MAX_TLSv1_0);
-      break;
+      return CURLE_OK == curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, static_cast<int>(CURL_SSLVERSION_TLSv1_0) | static_cast<int>(CURL_SSLVERSION_MAX_TLSv1_0));
     case SSLVersion::TLSv1_1:
-      ret = curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_1 | CURL_SSLVERSION_MAX_TLSv1_1);
-      break;
+      return CURLE_OK == curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, static_cast<int>(CURL_SSLVERSION_TLSv1_1) | static_cast<int>(CURL_SSLVERSION_MAX_TLSv1_1));
     case SSLVersion::TLSv1_2:
-      ret = curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2 | CURL_SSLVERSION_MAX_TLSv1_2);
-      break;
+      return CURLE_OK == curl_easy_setopt(http_session_, CURLOPT_SSLVERSION, static_cast<int>(CURL_SSLVERSION_TLSv1_2) | static_cast<int>(CURL_SSLVERSION_MAX_TLSv1_2));
+    default: return false;
   }
-
-  return ret == CURLE_OK;
 #else
   return false;
 #endif
@@ -198,25 +208,16 @@ void HTTPClient::setSeekFunction(HTTPUploadCallback *callbackObj) {
   curl_easy_setopt(http_session_, CURLOPT_SEEKFUNCTION, &utils::HTTPRequestResponse::seek_callback);
 }
 
-struct curl_slist *HTTPClient::build_header_list(std::string regex, const std::map<std::string, std::string> &attributes) {
-  if (http_session_) {
-    for (auto attribute : attributes) {
-      if (matches(attribute.first, regex)) {
-        std::string attr = attribute.first + ":" + attribute.second;
-        headers_ = curl_slist_append(headers_, attr.c_str());
-      }
-    }
-  }
-  return headers_;
-}
-
 void HTTPClient::setContentType(std::string content_type) {
   content_type_ = "Content-Type: " + content_type;
   headers_ = curl_slist_append(headers_, content_type_.c_str());
 }
 
 std::string HTTPClient::escape(std::string string_to_escape) {
-  return curl_easy_escape(http_session_, string_to_escape.c_str(), gsl::narrow<int>(string_to_escape.length()));
+  struct curl_deleter { void operator()(void* p) noexcept { curl_free(p); } };
+  std::unique_ptr<char, curl_deleter> escaped_chars{curl_easy_escape(http_session_, string_to_escape.c_str(), gsl::narrow<int>(string_to_escape.length()))};
+  std::string escaped_string(escaped_chars.get());
+  return escaped_string;
 }
 
 void HTTPClient::setPostFields(const std::string& input) {
@@ -237,9 +238,7 @@ void HTTPClient::appendHeader(const std::string &new_header) {
 }
 
 void HTTPClient::appendHeader(const std::string &key, const std::string &value) {
-  std::stringstream new_header;
-  new_header << key << ": " << value;
-  headers_ = curl_slist_append(headers_, new_header.str().c_str());
+  headers_ = curl_slist_append(headers_, utils::StringUtils::join_pack(key, ": ", value).c_str());
 }
 
 void HTTPClient::setUseChunkedEncoding() {
@@ -261,7 +260,7 @@ bool HTTPClient::submit() {
     progress_.reset();
     curl_easy_setopt(http_session_, CURLOPT_NOPROGRESS, 0);
     curl_easy_setopt(http_session_, CURLOPT_XFERINFOFUNCTION, onProgress);
-    curl_easy_setopt(http_session_, CURLOPT_XFERINFODATA, (void*)this);
+    curl_easy_setopt(http_session_, CURLOPT_XFERINFODATA, this);
   } else {
     // the user explicitly set it to 0
     curl_easy_setopt(http_session_, CURLOPT_NOPROGRESS, 1);
@@ -287,16 +286,18 @@ bool HTTPClient::submit() {
     curl_easy_setopt(http_session_, CURLOPT_TCP_KEEPALIVE, 1L);
     curl_easy_setopt(http_session_, CURLOPT_TCP_KEEPINTVL, keepAlive.count());
     curl_easy_setopt(http_session_, CURLOPT_TCP_KEEPIDLE, keepIdle.count());
-  }
-  else {
+  } else {
     logger_->log_debug("Not using keep alive");
     curl_easy_setopt(http_session_, CURLOPT_TCP_KEEPALIVE, 0L);
   }
+  if (form_ != nullptr) {
+    curl_easy_setopt(http_session_, CURLOPT_MIMEPOST, form_);
+  }
   res = curl_easy_perform(http_session_);
   if (callback == nullptr) {
     read_callback_.close();
   }
-  long http_code;
+  long http_code;  // NOLINT long due to libcurl API
   curl_easy_getinfo(http_session_, CURLINFO_RESPONSE_CODE, &http_code);
   http_code_ = http_code;
   curl_easy_getinfo(http_session_, CURLINFO_CONTENT_TYPE, &content_type_str_);
@@ -351,7 +352,7 @@ void HTTPClient::set_request_method(const std::string method) {
 }
 
 int HTTPClient::onProgress(void *clientp, curl_off_t /*dltotal*/, curl_off_t dlnow, curl_off_t /*ultotal*/, curl_off_t ulnow) {
-  HTTPClient& client = *(HTTPClient*)(clientp);
+  HTTPClient& client = *reinterpret_cast<HTTPClient*>(clientp);
   auto now = std::chrono::steady_clock::now();
   auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(now - client.progress_.last_transferred_);
   if (dlnow != client.progress_.downloaded_data_ || ulnow != client.progress_.uploaded_data_) {
@@ -365,7 +366,7 @@ int HTTPClient::onProgress(void *clientp, curl_off_t /*dltotal*/, curl_off_t dln
   if (elapsed.count() > client.read_timeout_ms_.count()) {
     // timeout
     client.logger_->log_error("HTTP operation has been idle for %dms, limit (%dms) reached, terminating connection\n",
-      (int)elapsed.count(), (int)client.read_timeout_ms_.count());
+      static_cast<int>(elapsed.count()), static_cast<int>(client.read_timeout_ms_.count()));
     return 1;
   }
   return 0;
@@ -376,7 +377,7 @@ bool HTTPClient::matches(const std::string &value, const std::string &sregex) {
     return true;
   try {
     utils::Regex rgx(sregex);
-    return rgx.match(value);
+    return utils::regexSearch(value, rgx);
   } catch (const Exception &) {
     return false;
   }
@@ -388,19 +389,17 @@ void HTTPClient::configure_secure_connection(CURL *http_session) {
   logger_->log_debug("Using CA certificate file \"%s\"", ssl_context_service_->getCACertificate());
 #if 0  // Reenable this path once we change from the direct manipulation of the SSL context to using the cURL API
   if (!ssl_context_service_->getCertificateFile().empty()) {
-    if (utils::StringUtils::endsWithIgnoreCase(ssl_context_service_->getCertificateFile(), "p12")) {
+    if (utils::StringUtils::endsWith(ssl_context_service_->getCertificateFile(), "p12", false)) {
       curl_easy_setopt(http_session, CURLOPT_SSLCERTTYPE, "P12");
-    }
-    else {
+    } else {
       curl_easy_setopt(http_session, CURLOPT_SSLCERTTYPE, "PEM");
     }
     curl_easy_setopt(http_session, CURLOPT_SSLCERT, ssl_context_service_->getCertificateFile().c_str());
   }
   if (!ssl_context_service_->getPrivateKeyFile().empty()) {
-    if (utils::StringUtils::endsWithIgnoreCase(ssl_context_service_->getPrivateKeyFile(), "p12")) {
+    if (utils::StringUtils::endsWith(ssl_context_service_->getPrivateKeyFile(), "p12", false)) {
       curl_easy_setopt(http_session, CURLOPT_SSLKEYTYPE, "P12");
-    }
-    else {
+    } else {
       curl_easy_setopt(http_session, CURLOPT_SSLKEYTYPE, "PEM");
     }
     curl_easy_setopt(http_session, CURLOPT_SSLKEY, ssl_context_service_->getPrivateKeyFile().c_str());
@@ -438,8 +437,39 @@ void HTTPClient::setFollowRedirects(bool follow) {
   curl_easy_setopt(http_session_, CURLOPT_FOLLOWLOCATION, follow);
 }
 
-}  // namespace utils
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+bool HTTPClient::isValidHttpHeaderField(std::string_view field_name) {
+  if (field_name.size() == 0) {
+    return false;
+  }
+
+  // RFC822 3.1.2: The  field-name must be composed of printable ASCII characters
+  // (i.e., characters that  have  values  between  33.  and  126., decimal, except colon).
+  for (auto ch : field_name) {
+    if (ch < 33 || ch > 126 || ch == ':') {
+      return false;
+    }
+  }
+  return true;
+}
+
+std::string HTTPClient::replaceInvalidCharactersInHttpHeaderFieldName(std::string_view field_name) {
+  if (field_name.size() == 0) {
+    return "X-MiNiFi-Empty-Attribute-Name";
+  }
+
+  std::string result;
+  // RFC822 3.1.2: The  field-name must be composed of printable ASCII characters
+  // (i.e., characters that  have  values  between  33.  and  126., decimal, except colon).
+  for (auto ch : field_name) {
+    if (ch < 33 || ch > 126 || ch == ':') {
+      result += '-';
+    } else {
+      result += ch;
+    }
+  }
+  return result;
+}
+
+REGISTER_INTERNAL_RESOURCE(HTTPClient);
+
+}  // namespace org::apache::nifi::minifi::utils
