@@ -24,8 +24,7 @@
 #pragma warning(disable: 4996)
 #endif
 
-#ifndef EXTENSIONS_HTTP_CURL_CLIENT_HTTPCLIENT_H_
-#define EXTENSIONS_HTTP_CURL_CLIENT_HTTPCLIENT_H_
+#pragma once
 
 #include "utils/HTTPClient.h"
 #ifdef WIN32
@@ -41,7 +40,6 @@
 #include <curl/easy.h>
 #include <vector>
 #include <memory>
-#include <iostream>
 #include <map>
 #include <chrono>
 #include <string>
@@ -50,19 +48,14 @@
 #else
 #include <regex.h>
 #endif
+#include <string_view>
 
 #include "utils/ByteArrayCallback.h"
 #include "controllers/SSLContextService.h"
 #include "core/logging/Logger.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "properties/Configure.h"
-#include "io/validation.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace utils {
+namespace org::apache::nifi::minifi::utils {
 
 /**
  * Purpose and Justification: Pull the basics for an HTTPClient into a self contained class. Simply provide
@@ -78,6 +71,10 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   HTTPClient(const std::string& name, const utils::Identifier& uuid);
 
+  // class uses raw pointers
+  HTTPClient(const HTTPClient&) = delete;
+  HTTPClient& operator=(const HTTPClient&) = delete;
+
   explicit HTTPClient(const std::string &url, const std::shared_ptr<minifi::controllers::SSLContextService> ssl_context_service = nullptr);
 
   ~HTTPClient();
@@ -86,6 +83,8 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   void setVerbose(bool use_stderr = false) override;
 
+  void addFormPart(const std::string& content_type, const std::string& name, HTTPUploadCallback* read_callback, const std::optional<std::string>& filename = std::nullopt);
+
   void forceClose();
 
   void initialize(const std::string &method, const std::string url = "", const std::shared_ptr<minifi::controllers::SSLContextService> ssl_context_service = nullptr) override;
@@ -110,8 +109,6 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   virtual void setReadCallback(HTTPReadCallback *callbackObj);
 
-  struct curl_slist *build_header_list(std::string regex, const std::map<std::string, std::string> &attributes);
-
   void setContentType(std::string content_type) override;
 
   std::string escape(std::string string_to_escape) override;
@@ -146,11 +143,11 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   bool setMinimumSSLVersion(SSLVersion minimum_version) override;
 
-  DEPRECATED(/*deprecated in*/ 0.8.0, /*will remove in */ 2.0) void setKeepAliveProbe(long probe) {
+  DEPRECATED(/*deprecated in*/ 0.8.0, /*will remove in */ 2.0) void setKeepAliveProbe(long probe) {  // NOLINT deprecated
     keep_alive_probe_ = std::chrono::milliseconds(probe * 1000);
   }
 
-  DEPRECATED(/*deprecated in*/ 0.8.0, /*will remove in */ 2.0) void setKeepAliveIdle(long idle) {
+  DEPRECATED(/*deprecated in*/ 0.8.0, /*will remove in */ 2.0) void setKeepAliveIdle(long idle) {  // NOLINT deprecated
     keep_alive_idle_ = std::chrono::milliseconds(idle * 1000);
   }
 
@@ -235,6 +232,10 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
       }
     }
   }
+
+  static bool isValidHttpHeaderField(std::string_view field_name);
+  static std::string replaceInvalidCharactersInHttpHeaderFieldName(std::string_view field_name);
+
  private:
   static int onProgress(void *client, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow);
 
@@ -288,7 +289,8 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   CURLcode res{CURLE_OK};
 
-  CURL *http_session_;
+  CURL* http_session_{nullptr};
+  curl_mime* form_{nullptr};
 
   std::string method_;
 
@@ -296,16 +298,10 @@ class HTTPClient : public BaseHTTPClient, public core::Connectable {
 
   std::chrono::milliseconds keep_alive_idle_{-1};
 
-  std::shared_ptr<logging::Logger> logger_{logging::LoggerFactory<HTTPClient>::getLogger()};
+  std::shared_ptr<core::logging::Logger> logger_{core::logging::LoggerFactory<HTTPClient>::getLogger()};
 };
 
-}  // namespace utils
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
-
-#endif  // EXTENSIONS_HTTP_CURL_CLIENT_HTTPCLIENT_H_
+}  // namespace org::apache::nifi::minifi::utils
 
 #ifdef WIN32
 #pragma warning(pop)
