@@ -17,21 +17,10 @@
  */
 
 #include "InvokeHTTP.h"
-#ifdef WIN32
-#include <regex>
-#else
-#include <regex.h>
-#endif
-#include <curl/easy.h>
-#include <memory>
-#include <algorithm>
-#include <cctype>
+
 #include <cinttypes>
 #include <cstdint>
-#include <cstring>
-#include <iostream>
-#include <iterator>
-#include <map>
+#include <memory>
 #include <set>
 #include <string>
 #include <utility>
@@ -39,22 +28,21 @@
 
 #include "utils/ByteArrayCallback.h"
 #include "core/FlowFile.h"
-#include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
 #include "ResourceClaim.h"
 #include "utils/gsl.h"
 #include "utils/StringUtils.h"
+#include "utils/ProcessorConfigUtils.h"
+#include "utils/OptionalUtils.h"
+#include "range/v3/view/filter.hpp"
+#include "range/v3/algorithm/any_of.hpp"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+namespace org::apache::nifi::minifi::processors {
 
-const char *InvokeHTTP::ProcessorName = "InvokeHTTP";
 std::string InvokeHTTP::DefaultContentType = "application/octet-stream";
 
 core::Property InvokeHTTP::Method("HTTP Method", "HTTP request method (GET, POST, PUT, PATCH, DELETE, HEAD, OPTIONS). "
@@ -123,6 +111,16 @@ core::Property InvokeHTTP::AlwaysOutputResponse("Always Output Response", "Will
 core::Property InvokeHTTP::PenalizeOnNoRetry("Penalize on \"No Retry\"", "Enabling this property will penalize FlowFiles that are routed to the \"No Retry\" relationship.", "false");
 
 core::Property InvokeHTTP::DisablePeerVerification("Disable Peer Verification", "Disables peer verification for the SSL session", "false");
+
+core::Property InvokeHTTP::InvalidHTTPHeaderFieldHandlingStrategy(
+    core::PropertyBuilder::createProperty("Invalid HTTP Header Field Handling Strategy")
+      ->withDescription("Indicates what should happen when an attribute's name is not a valid HTTP header field name. "
+        "Options: transform - invalid characters are replaced, fail - flow file is transferred to failure, drop - drops invalid attributes from HTTP message")
+      ->isRequired(true)
+      ->withDefaultValue<std::string>(toString(InvalidHTTPHeaderFieldHandlingOption::TRANSFORM))
+      ->withAllowableValues<std::string>(InvalidHTTPHeaderFieldHandlingOption::values())
+      ->build());
+
 const char* InvokeHTTP::STATUS_CODE = "invokehttp.status.code";
 const char* InvokeHTTP::STATUS_MESSAGE = "invokehttp.status.message";
 const char* InvokeHTTP::RESPONSE_BODY = "invokehttp.response.body";
@@ -132,9 +130,12 @@ const char* InvokeHTTP::REMOTE_DN = "invokehttp.remote.dn";
 const char* InvokeHTTP::EXCEPTION_CLASS = "invokehttp.java.exception.class";
 const char* InvokeHTTP::EXCEPTION_MESSAGE = "invokehttp.java.exception.message";
 
-core::Relationship InvokeHTTP::Success("success", "All files are routed to success");
+core::Relationship InvokeHTTP::Success("success", "The original FlowFile will be routed upon success (2xx status codes). "
+                                       "It will have new attributes detailing the success of the request.");
 
-core::Relationship InvokeHTTP::RelResponse("response", "Represents a response flowfile");
+core::Relationship InvokeHTTP::RelResponse("response", "A Response FlowFile will be routed upon success (2xx status codes). "
+                                           "If the 'Always Output Response' property is true then the response will be sent "
+                                           "to this relationship regardless of the status code received.");
 
 core::Relationship InvokeHTTP::RelRetry("retry", "The original FlowFile will be routed on any status code that can be retried "
                                         "(5xx status codes). It will have new attributes detailing the request.");
@@ -147,40 +148,32 @@ core::Relationship InvokeHTTP::RelFailure("failure", "The original FlowFile will
 
 void InvokeHTTP::initialize() {
   logger_->log_trace("Initializing InvokeHTTP");
-
-  // Set the supported properties
-  std::set<core::Property> properties;
-  properties.insert(Method);
-  properties.insert(URL);
-  properties.insert(ConnectTimeout);
-  properties.insert(ReadTimeout);
-  properties.insert(DateHeader);
-  properties.insert(AttributesToSend);
-  properties.insert(SSLContext);
-  properties.insert(ProxyHost);
-  properties.insert(ProxyPort);
-  properties.insert(ProxyUsername);
-  properties.insert(ProxyPassword);
-  properties.insert(UseChunkedEncoding);
-  properties.insert(ContentType);
-  properties.insert(SendBody);
-  properties.insert(SendMessageBody);
-  properties.insert(DisablePeerVerification);
-  properties.insert(AlwaysOutputResponse);
-  properties.insert(FollowRedirects);
-
-  setSupportedProperties(properties);
-  // Set the supported relationships
+  setSupportedProperties({
+    Method,
+    URL,
+    ConnectTimeout,
+    ReadTimeout,
+    DateHeader,
+    AttributesToSend,
+    SSLContext,
+    ProxyHost,
+    ProxyPort,
+    ProxyUsername,
+    ProxyPassword,
+    UseChunkedEncoding,
+    ContentType,
+    SendBody,
+    SendMessageBody,
+    DisablePeerVerification,
+    AlwaysOutputResponse,
+    FollowRedirects,
+    PropPutOutputAttributes,
+    PenalizeOnNoRetry,
+    InvalidHTTPHeaderFieldHandlingStrategy
+  });
   setSupportedRelationships({Success, RelResponse, RelFailure, RelRetry, RelNoRetry});
 }
 
-bool getTimeMSFromString(const std::string& propertyName, uint64_t& valInt) {
-  core::TimeUnit unit;
-  return !propertyName.empty()
-      && core::Property::StringToTime(propertyName, valInt, unit)
-      && core::Property::ConvertTimeUnitToMS(valInt, unit, valInt);
-}
-
 void InvokeHTTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory>& /*sessionFactory*/) {
   if (!context->getProperty(Method.getName(), method_)) {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", Method.getName(), Method.getValue());
@@ -192,43 +185,39 @@ void InvokeHTTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context
     return;
   }
 
-  uint64_t valInt;
-  std::string timeoutStr;
-  if (context->getProperty(ConnectTimeout.getName(), timeoutStr)
-      && core::Property::getTimeMSFromString(timeoutStr, valInt)) {
-    connect_timeout_ms_ =  std::chrono::milliseconds(valInt);
+  if (auto connect_timeout = context->getProperty<core::TimePeriodValue>(ConnectTimeout)) {
+    connect_timeout_ms_ =  connect_timeout->getMilliseconds();
   } else {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", ConnectTimeout.getName(), ConnectTimeout.getValue());
     return;
   }
 
-  std::string contentTypeStr;
-  if (context->getProperty(ContentType.getName(), contentTypeStr)) {
-    content_type_ = contentTypeStr;
+  std::string content_type_str;
+  if (context->getProperty(ContentType.getName(), content_type_str)) {
+    content_type_ = content_type_str;
   }
 
-  timeoutStr.clear();
-  if (context->getProperty(ReadTimeout.getName(), timeoutStr)
-      && core::Property::getTimeMSFromString(timeoutStr, valInt)) {
-    read_timeout_ms_ =  std::chrono::milliseconds(valInt);
+  if (auto read_timeout = context->getProperty<core::TimePeriodValue>(ReadTimeout)) {
+    read_timeout_ms_ =  read_timeout->getMilliseconds();
   } else {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", ReadTimeout.getName(), ReadTimeout.getValue());
   }
 
-  std::string dateHeaderStr;
-  if (!context->getProperty(DateHeader.getName(), dateHeaderStr)) {
+  std::string date_header_str;
+  if (!context->getProperty(DateHeader.getName(), date_header_str)) {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", DateHeader.getName(), DateHeader.getValue());
   }
 
-  date_header_include_ = utils::StringUtils::toBool(dateHeaderStr).value_or(DateHeader.getValue());
+  date_header_include_ = utils::StringUtils::toBool(date_header_str).value_or(DateHeader.getValue());
 
   if (!context->getProperty(PropPutOutputAttributes.getName(), put_attribute_name_)) {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", PropPutOutputAttributes.getName(), PropPutOutputAttributes.getValue());
   }
 
-  if (!context->getProperty(AttributesToSend.getName(), attribute_to_send_regex_)) {
-    logger_->log_debug("%s attribute is missing, so default value of %s will be used", AttributesToSend.getName(), AttributesToSend.getValue());
-  }
+  attributes_to_send_ = context->getProperty(AttributesToSend)
+      | utils::filter([](const std::string& s) { return !s.empty(); })  // avoid compiling an empty string to regex
+      | utils::map([](const std::string& regex_str) { return utils::Regex{regex_str}; })
+      | utils::orElse([this] { logger_->log_debug("%s is missing, so the default value will be used", AttributesToSend.getName()); });
 
   std::string always_output_response;
   if (!context->getProperty(AlwaysOutputResponse.getName(), always_output_response)) {
@@ -247,20 +236,25 @@ void InvokeHTTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context
   std::string context_name;
   if (context->getProperty(SSLContext.getName(), context_name) && !IsNullOrEmpty(context_name)) {
     std::shared_ptr<core::controller::ControllerService> service = context->getControllerService(context_name);
-    if (nullptr != service) {
-      ssl_context_service_ = std::static_pointer_cast<minifi::controllers::SSLContextService>(service);
+    if (!service) {
+      logger_->log_error("Couldn't find controller service with name '%s'", context_name);
+    } else {
+      ssl_context_service_ = std::dynamic_pointer_cast<minifi::controllers::SSLContextService>(service);
+      if (!ssl_context_service_) {
+        logger_->log_error("Controller service '%s' is not an SSLContextService", context_name);
+      }
     }
   }
 
-  std::string useChunkedEncoding = "false";
-  if (!context->getProperty(UseChunkedEncoding.getName(), useChunkedEncoding)) {
+  std::string use_chunked_encoding = "false";
+  if (!context->getProperty(UseChunkedEncoding.getName(), use_chunked_encoding)) {
     logger_->log_debug("%s attribute is missing, so default value of %s will be used", UseChunkedEncoding.getName(), UseChunkedEncoding.getValue());
   }
 
-  use_chunked_encoding_ = utils::StringUtils::toBool(useChunkedEncoding).value_or(false);
+  use_chunked_encoding_ = utils::StringUtils::toBool(use_chunked_encoding).value_or(false);
 
-  std::string disablePeerVerification;
-  disable_peer_verification_ = (context->getProperty(DisablePeerVerification.getName(), disablePeerVerification) && utils::StringUtils::toBool(disablePeerVerification).value_or(false));
+  std::string disable_peer_verification;
+  disable_peer_verification_ = (context->getProperty(DisablePeerVerification.getName(), disable_peer_verification) && utils::StringUtils::toBool(disable_peer_verification).value_or(false));
 
   proxy_ = {};
   context->getProperty(ProxyHost.getName(), proxy_.host);
@@ -272,27 +266,54 @@ void InvokeHTTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context
   context->getProperty(ProxyPassword.getName(), proxy_.password);
   context->getProperty(FollowRedirects.getName(), follow_redirects_);
   context->getProperty(SendMessageBody.getName(), send_body_);
+
+  invalid_http_header_field_handling_strategy_ = utils::parseEnumProperty<InvalidHTTPHeaderFieldHandlingOption>(*context, InvalidHTTPHeaderFieldHandlingStrategy);
 }
 
-InvokeHTTP::~InvokeHTTP() = default;
-
-std::string InvokeHTTP::generateId() {
-  return utils::IdGenerator::getIdGenerator()->generate().to_string();
+bool InvokeHTTP::shouldEmitFlowFile() const {
+  return ("POST" == method_ || "PUT" == method_ || "PATCH" == method_);
 }
 
-bool InvokeHTTP::emitFlowFile(const std::string &method) {
-  return ("POST" == method || "PUT" == method || "PATCH" == method);
+/**
+ * Calls append_header with valid HTTP header keys, based on attributes_to_send_
+ * @param flow_file
+ * @param append_header Callback to append HTTP header to the request
+ * @return false when the flow file should be routed to failure, true otherwise
+ */
+bool InvokeHTTP::appendHeaders(const core::FlowFile& flow_file, /*std::invocable<std::string, std::string>*/ auto append_header) {
+  static_assert(std::is_invocable_v<decltype(append_header), std::string, std::string>);
+  if (!attributes_to_send_) return true;
+  const auto key_fn = [](const std::pair<std::string, std::string>& pair) { return pair.first; };
+  const auto original_attributes = flow_file.getAttributes();
+  // non-const views, because otherwise it doesn't satisfy viewable_range, and transform would fail
+  ranges::viewable_range auto matching_attributes = original_attributes
+      | ranges::views::filter([this](const auto& key) { return utils::regexMatch(key, *attributes_to_send_); }, key_fn);
+  switch (invalid_http_header_field_handling_strategy_.value()) {
+    case InvalidHTTPHeaderFieldHandlingOption::FAIL:
+      if (ranges::any_of(matching_attributes, std::not_fn(&utils::HTTPClient::isValidHttpHeaderField), key_fn)) return false;
+      for (const auto& header: matching_attributes) append_header(header.first, header.second);
+      return true;
+    case InvalidHTTPHeaderFieldHandlingOption::DROP:
+      for (const auto& header: matching_attributes | ranges::views::filter(&utils::HTTPClient::isValidHttpHeaderField, key_fn)) {
+        append_header(header.first, header.second);
+      }
+      return true;
+    case InvalidHTTPHeaderFieldHandlingOption::TRANSFORM:
+      for (const auto& header: matching_attributes) {
+        append_header(utils::HTTPClient::replaceInvalidCharactersInHttpHeaderFieldName(header.first), header.second);
+      }
+      return true;
+  }
+  return true;
 }
 
 void InvokeHTTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
-  auto flowFile = session->get();
+  auto flow_file = session->get();
 
-  std::string url = url_;
-
-  if (flowFile == nullptr) {
-    if (!emitFlowFile(method_)) {
+  if (flow_file == nullptr) {
+    if (!shouldEmitFlowFile()) {
       logger_->log_debug("InvokeHTTP -- create flow file with  %s", method_);
-      flowFile = session->create();
+      flow_file = session->create();
     } else {
       logger_->log_debug("Exiting because method is %s and there is no flowfile available to execute it, yielding", method_);
       yield();
@@ -305,11 +326,11 @@ void InvokeHTTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
   logger_->log_debug("onTrigger InvokeHTTP with %s to %s", method_, url_);
 
   // create a transaction id
-  std::string tx_id = generateId();
+  std::string tx_id = utils::IdGenerator::getIdGenerator()->generate().to_string();
 
-  // Note: callback must be declared before callbackObj so that they are destructed in the correct order
-  std::unique_ptr<utils::ByteInputCallBack> callback = nullptr;
-  std::unique_ptr<utils::HTTPUploadCallback> callbackObj = nullptr;
+  // Note: callback must be declared before callback_obj so that they are destructed in the correct order
+  std::unique_ptr<utils::ByteInputCallback> callback;
+  std::unique_ptr<utils::HTTPUploadCallback> callback_obj;
 
   // Client declared after the callbacks to make sure the callbacks are still available when the client is destructed
   utils::HTTPClient client(url_, ssl_context_service_);
@@ -334,25 +355,25 @@ void InvokeHTTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
 
   client.setHTTPProxy(proxy_);
 
-  if (emitFlowFile(method_)) {
+  if (shouldEmitFlowFile()) {
     logger_->log_trace("InvokeHTTP -- reading flowfile");
-    std::shared_ptr<ResourceClaim> claim = flowFile->getResourceClaim();
+    std::shared_ptr<ResourceClaim> claim = flow_file->getResourceClaim();
     if (claim) {
-      callback = std::unique_ptr<utils::ByteInputCallBack>(new utils::ByteInputCallBack());
+      callback = std::make_unique<utils::ByteInputCallback>();
       if (send_body_) {
-        session->read(flowFile, callback.get());
+        session->read(flow_file, std::ref(*callback));
       }
-      callbackObj = std::unique_ptr<utils::HTTPUploadCallback>(new utils::HTTPUploadCallback);
-      callbackObj->ptr = callback.get();
-      callbackObj->pos = 0;
+      callback_obj = std::make_unique<utils::HTTPUploadCallback>();
+      callback_obj->ptr = callback.get();
+      callback_obj->pos = 0;
       logger_->log_trace("InvokeHTTP -- Setting callback, size is %d", callback->getBufferSize());
       if (!send_body_) {
         client.appendHeader("Content-Length", "0");
       } else if (!use_chunked_encoding_) {
-        client.appendHeader("Content-Length", std::to_string(flowFile->getSize()));
+        client.appendHeader("Content-Length", std::to_string(flow_file->getSize()));
       }
-      client.setUploadCallback(callbackObj.get());
-      client.setSeekFunction(callbackObj.get());
+      client.setUploadCallback(callback_obj.get());
+      client.setSeekFunction(callback_obj.get());
     } else {
       logger_->log_error("InvokeHTTP -- no resource claim");
     }
@@ -361,35 +382,41 @@ void InvokeHTTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
     logger_->log_trace("InvokeHTTP -- Not emitting flowfile to HTTP Server");
   }
 
-  // append all headers
-  client.build_header_list(attribute_to_send_regex_, flowFile->getAttributes());
+  const auto append_header = [&client](const std::string& key, const std::string& value) { client.appendHeader(key, value); };
+  if (!appendHeaders(*flow_file, append_header)) {
+    session->transfer(flow_file, RelFailure);
+    return;
+  }
 
   logger_->log_trace("InvokeHTTP -- curl performed");
   if (client.submit()) {
     logger_->log_trace("InvokeHTTP -- curl successful");
 
-    bool putToAttribute = !IsNullOrEmpty(put_attribute_name_);
+    bool put_to_attribute = !IsNullOrEmpty(put_attribute_name_);
+    if (put_to_attribute) {
+      logger_->log_debug("Adding http response body to flow file attribute %s", put_attribute_name_);
+    }
 
     const std::vector<char> &response_body = client.getResponseBody();
     const std::vector<std::string> &response_headers = client.getHeaders();
 
     int64_t http_code = client.getResponseCode();
     const char *content_type = client.getContentType();
-    flowFile->addAttribute(STATUS_CODE, std::to_string(http_code));
+    flow_file->addAttribute(STATUS_CODE, std::to_string(http_code));
     if (!response_headers.empty())
-      flowFile->addAttribute(STATUS_MESSAGE, response_headers.at(0));
-    flowFile->addAttribute(REQUEST_URL, url_);
-    flowFile->addAttribute(TRANSACTION_ID, tx_id);
+      flow_file->addAttribute(STATUS_MESSAGE, response_headers.at(0));
+    flow_file->addAttribute(REQUEST_URL, url_);
+    flow_file->addAttribute(TRANSACTION_ID, tx_id);
 
-    bool isSuccess = (static_cast<int32_t>(http_code / 100) == 2);
-    bool output_body_to_content = isSuccess && !putToAttribute;
+    bool is_success = (static_cast<int32_t>(http_code / 100) == 2);
+    bool output_body_to_content = is_success && !put_to_attribute;
 
-    logger_->log_debug("isSuccess: %d, response code %" PRId64, isSuccess, http_code);
+    logger_->log_debug("isSuccess: %d, response code %" PRId64, is_success, http_code);
     std::shared_ptr<core::FlowFile> response_flow = nullptr;
 
     if (output_body_to_content) {
-      if (flowFile != nullptr) {
-        response_flow = session->create(flowFile);
+      if (flow_file != nullptr) {
+        response_flow = session->create(flow_file);
       } else {
         response_flow = session->create();
       }
@@ -400,47 +427,47 @@ void InvokeHTTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
       response_flow->addAttribute(STATUS_CODE, std::to_string(http_code));
       if (!response_headers.empty())
         response_flow->addAttribute(STATUS_MESSAGE, response_headers.at(0));
-      response_flow->addAttribute(REQUEST_URL, url);
+      response_flow->addAttribute(REQUEST_URL, url_);
       response_flow->addAttribute(TRANSACTION_ID, tx_id);
-      io::BufferStream stream((const uint8_t*) response_body.data(), gsl::narrow<unsigned int>(response_body.size()));
+      io::BufferStream stream(gsl::make_span(response_body).as_span<const std::byte>());
       // need an import from the data stream.
       session->importFrom(stream, response_flow);
     }
-    route(flowFile, response_flow, session, context, isSuccess, http_code);
+    route(flow_file, response_flow, session, context, is_success, http_code);
   } else {
-    session->penalize(flowFile);
-    session->transfer(flowFile, RelFailure);
+    session->penalize(flow_file);
+    session->transfer(flow_file, RelFailure);
   }
 }
 
 void InvokeHTTP::route(const std::shared_ptr<core::FlowFile> &request, const std::shared_ptr<core::FlowFile> &response, const std::shared_ptr<core::ProcessSession> &session,
-                       const std::shared_ptr<core::ProcessContext> &context, bool isSuccess, int64_t statusCode) {
+                       const std::shared_ptr<core::ProcessContext> &context, bool is_success, int64_t status_code) {
   // check if we should yield the processor
-  if (!isSuccess && request == nullptr) {
+  if (!is_success && request == nullptr) {
     context->yield();
   }
 
   // If the property to output the response flowfile regardless of status code is set then transfer it
-  bool responseSent = false;
+  bool response_sent = false;
   if (always_output_response_ && response != nullptr) {
     logger_->log_debug("Outputting success and response");
     session->transfer(response, RelResponse);
-    responseSent = true;
+    response_sent = true;
   }
 
   // transfer to the correct relationship
   // 2xx -> SUCCESS
-  if (isSuccess) {
+  if (is_success) {
     // we have two flowfiles to transfer
     if (request != nullptr) {
       session->transfer(request, Success);
     }
-    if (response != nullptr && !responseSent) {
+    if (response != nullptr && !response_sent) {
       logger_->log_debug("Outputting success and response");
       session->transfer(response, RelResponse);
     }
     // 5xx -> RETRY
-  } else if (statusCode / 100 == 5) {
+  } else if (status_code / 100 == 5) {
     if (request != nullptr) {
       session->penalize(request);
       session->transfer(request, RelRetry);
@@ -456,8 +483,8 @@ void InvokeHTTP::route(const std::shared_ptr<core::FlowFile> &request, const std
   }
 }
 
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
+REGISTER_RESOURCE(InvokeHTTP, "An HTTP client processor which can interact with a configurable HTTP Endpoint. "
+    "The destination URL and HTTP Method are configurable. FlowFile attributes are converted to HTTP headers and the "
+    "FlowFile contents are included as the body of the request (if the HTTP Method is PUT, POST or PATCH).");
+
+}  // namespace org::apache::nifi::minifi::processors
