@@ -20,15 +20,10 @@
 
 #include <algorithm>
 #include <memory>
-#include <utility>
-#include <map>
-#include <list>
 #include <string>
-#include <vector>
-
-#include "utils/gsl.h"
 
 #include "rapidjson/prettywriter.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -37,11 +32,10 @@ namespace minifi {
 namespace c2 {
 
 AgentPrinter::AgentPrinter(const std::string& name, const utils::Identifier& uuid)
-    : HeartbeatReporter(name, uuid),
-      logger_(logging::LoggerFactory<AgentPrinter>::getLogger()) {
+    : HeartbeatReporter(name, uuid) {
 }
 
-void AgentPrinter::initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<state::StateMonitor> &updateSink,
+void AgentPrinter::initialize(core::controller::ControllerServiceProvider* controller, state::StateMonitor* updateSink,
                               const std::shared_ptr<Configure> &configure) {
   HeartbeatReporter::initialize(controller, updateSink, configure);
 }
@@ -66,6 +60,8 @@ rapidjson::Value AgentPrinter::serializeJsonPayload(const C2Payload &payload, ra
   return result;
 }
 
+REGISTER_RESOURCE(AgentPrinter, "Encapsulates printing agent information.");
+
 } /* namespace c2 */
 } /* namespace minifi */
 } /* namespace nifi */
