@@ -19,16 +19,14 @@
 #include "RESTSender.h"
 
 #include <algorithm>
-#include <iostream>
 #include <memory>
-#include <utility>
-#include <map>
 #include <string>
 #include <vector>
+#include <utility>
 #include <limits>
 #include "utils/file/FileUtils.h"
-#include "utils/StringUtils.h"
-#include "utils/file/FileManager.h"
+#include "core/Resource.h"
+#include "properties/Configuration.h"
 
 namespace org {
 namespace apache {
@@ -37,8 +35,7 @@ namespace minifi {
 namespace c2 {
 
 RESTSender::RESTSender(const std::string &name, const utils::Identifier &uuid)
-    : C2Protocol(name, uuid),
-      logger_(logging::LoggerFactory<RESTSender>::getLogger()) {
+    : C2Protocol(name, uuid) {
 }
 
 void RESTSender::initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<Configure> &configure) {
@@ -47,9 +44,9 @@ void RESTSender::initialize(core::controller::ControllerServiceProvider* control
   // base URL when one is not specified.
   if (nullptr != configure) {
     std::string update_str, ssl_context_service_str;
-    configure->get("nifi.c2.rest.url", "c2.rest.url", rest_uri_);
-    configure->get("nifi.c2.rest.url.ack", "c2.rest.url.ack", ack_uri_);
-    if (configure->get("nifi.c2.rest.ssl.context.service", "c2.rest.ssl.context.service", ssl_context_service_str)) {
+    configure->get(Configuration::nifi_c2_rest_url, "c2.rest.url", rest_uri_);
+    configure->get(Configuration::nifi_c2_rest_url_ack, "c2.rest.url.ack", ack_uri_);
+    if (configure->get(Configuration::nifi_c2_rest_ssl_context_service, "c2.rest.ssl.context.service", ssl_context_service_str)) {
       auto service = controller->getControllerService(ssl_context_service_str);
       if (nullptr != service) {
         ssl_context_service_ = std::static_pointer_cast<minifi::controllers::SSLContextService>(service);
@@ -60,12 +57,13 @@ void RESTSender::initialize(core::controller::ControllerServiceProvider* control
 }
 
 C2Payload RESTSender::consumePayload(const std::string &url, const C2Payload &payload, Direction direction, bool /*async*/) {
-  std::string outputConfig;
+  std::optional<std::string> data;
 
-  if (direction == Direction::TRANSMIT) {
-    outputConfig = serializeJsonRootPayload(payload);
+  if (direction == Direction::TRANSMIT && payload.getOperation() != Operation::TRANSFER) {
+    // treat payload as json
+    data = serializeJsonRootPayload(payload);
   }
-  return sendPayload(url, direction, payload, outputConfig);
+  return sendPayload(url, direction, payload, std::move(data));
 }
 
 C2Payload RESTSender::consumePayload(const C2Payload &payload, Direction direction, bool async) {
@@ -75,10 +73,7 @@ C2Payload RESTSender::consumePayload(const C2Payload &payload, Direction directi
   return consumePayload(rest_uri_, payload, direction, async);
 }
 
-void RESTSender::update(const std::shared_ptr<Configure> &configure) {
-  std::string url;
-  configure->get("nifi.c2.rest.url", "c2.rest.url", url);
-  configure->get("nifi.c2.rest.url.ack", "c2.rest.url.ack", url);
+void RESTSender::update(const std::shared_ptr<Configure> &) {
 }
 
 void RESTSender::setSecurityContext(utils::HTTPClient &client, const std::string &type, const std::string &url) {
@@ -88,14 +83,14 @@ void RESTSender::setSecurityContext(utils::HTTPClient &client, const std::string
   client.initialize(type, url, generatedService);
 }
 
-const C2Payload RESTSender::sendPayload(const std::string url, const Direction direction, const C2Payload &payload, const std::string outputConfig) {
+C2Payload RESTSender::sendPayload(const std::string url, const Direction direction, const C2Payload &payload, std::optional<std::string> data) {
   if (url.empty()) {
     return C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR);
   }
 
   // Callback for transmit. Declared in order to destruct in proper order - take care!
-  std::unique_ptr<utils::ByteInputCallBack> input = nullptr;
-  std::unique_ptr<utils::HTTPUploadCallback> callback = nullptr;
+  std::vector<std::unique_ptr<utils::ByteInputCallback>> inputs;
+  std::vector<std::unique_ptr<utils::HTTPUploadCallback>> callbacks;
 
   // Callback for transfer. Declared in order to destruct in proper order - take care!
   std::unique_ptr<utils::ByteOutputCallback> file_callback = nullptr;
@@ -107,17 +102,35 @@ const C2Payload RESTSender::sendPayload(const std::string url, const Direction d
   client.setKeepAliveIdle(std::chrono::milliseconds(2000));
   client.setConnectionTimeout(std::chrono::milliseconds(2000));
   if (direction == Direction::TRANSMIT) {
-    input = std::unique_ptr<utils::ByteInputCallBack>(new utils::ByteInputCallBack());
-    callback = std::unique_ptr<utils::HTTPUploadCallback>(new utils::HTTPUploadCallback());
-    input->write(outputConfig);
-    callback->ptr = input.get();
-    callback->pos = 0;
     client.set_request_method("POST");
     if (!ssl_context_service_ && url.find("https://") == 0) {
       setSecurityContext(client, "POST", url);
     }
-    client.setUploadCallback(callback.get());
-    client.setPostSize(outputConfig.size());
+    if (payload.getOperation() == Operation::TRANSFER) {
+      // treat nested payloads as files
+      for (const auto& file : payload.getNestedPayloads()) {
+        std::string filename = file.getLabel();
+        if (filename.empty()) {
+          throw std::logic_error("Missing filename");
+        }
+        auto file_input = std::make_unique<utils::ByteInputCallback>();
+        auto file_cb = std::make_unique<utils::HTTPUploadCallback>();
+        file_input->write(file.getRawDataAsString());
+        file_cb->ptr = file_input.get();
+        client.addFormPart("application/octet-stream", "file", file_cb.get(), filename);
+        inputs.push_back(std::move(file_input));
+        callbacks.push_back(std::move(file_cb));
+      }
+    } else {
+      auto data_input = std::make_unique<utils::ByteInputCallback>();
+      auto data_cb = std::make_unique<utils::HTTPUploadCallback>();
+      data_input->write(data.value_or(""));
+      data_cb->ptr = data_input.get();
+      client.setUploadCallback(data_cb.get());
+      client.setPostSize(data ? data->size() : 0);
+      inputs.push_back(std::move(data_input));
+      callbacks.push_back(std::move(data_cb));
+    }
   } else {
     // we do not need to set the upload callback
     // since we are not uploading anything on a get
@@ -128,7 +141,7 @@ const C2Payload RESTSender::sendPayload(const std::string url, const Direction d
   }
 
   if (payload.getOperation() == Operation::TRANSFER) {
-    file_callback = std::unique_ptr<utils::ByteOutputCallback>(new utils::ByteOutputCallback(std::numeric_limits<size_t>::max()));
+    file_callback = std::make_unique<utils::ByteOutputCallback>(std::numeric_limits<size_t>::max());
     read.pos = 0;
     read.ptr = file_callback.get();
     client.setReadCallback(&read);
@@ -145,19 +158,22 @@ const C2Payload RESTSender::sendPayload(const std::string url, const Direction d
   } else {
     logger_->log_debug("Response code '" "%" PRId64 "' from '%s'", respCode, url);
   }
-  auto rs = client.getResponseBody();
+  const auto response_body_bytes = gsl::make_span(client.getResponseBody()).as_span<const std::byte>();
+  logger_->log_trace("Received response: \"%s\"", [&] {return utils::StringUtils::escapeUnprintableBytes(response_body_bytes);});
   if (isOkay && respCode) {
     if (payload.isRaw()) {
       C2Payload response_payload(payload.getOperation(), state::UpdateState::READ_COMPLETE, true);
-      response_payload.setRawData(client.getResponseBody());
+      response_payload.setRawData(response_body_bytes);
       return response_payload;
     }
-    return parseJsonResponse(payload, client.getResponseBody());
+    return parseJsonResponse(payload, response_body_bytes);
   } else {
     return C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR);
   }
 }
 
+REGISTER_RESOURCE(RESTSender, "Encapsulates the restful protocol that is built upon C2Protocol.");
+
 } /* namespace c2 */
 } /* namespace minifi */
 } /* namespace nifi */
