@@ -15,16 +15,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_C2_RESTSENDER_H_
-#define LIBMINIFI_INCLUDE_C2_RESTSENDER_H_
+#pragma once
 
 #include <string>
-#include <mutex>
+#include <memory>
 
-#include "utils/ByteArrayCallback.h"
 #include "c2/C2Protocol.h"
 #include "c2/protocols/RESTProtocol.h"
-#include "c2/HeartbeatReporter.h"
 #include "controllers/SSLContextService.h"
 #include "../client/HTTPClient.h"
 
@@ -34,9 +31,6 @@ namespace nifi {
 namespace minifi {
 namespace c2 {
 
-#undef RAPIDJSON_ASSERT
-#define RAPIDJSON_ASSERT(x) if(!(x)) throw std::logic_error("rapidjson exception"); //NOLINT
-
 /**
  * Purpose and Justification: Encapsulates the restful protocol that is built upon C2Protocol.
  *
@@ -58,7 +52,7 @@ class RESTSender : public RESTProtocol, public C2Protocol {
   void initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<Configure> &configure) override;
 
  protected:
-  virtual const C2Payload sendPayload(const std::string url, const Direction direction, const C2Payload &payload, const std::string outputConfig);
+  C2Payload sendPayload(const std::string url, const Direction direction, const C2Payload &payload, std::optional<std::string> data);
 
   /**
    * Initializes the SSLContextService onto the HTTP client if one is needed
@@ -74,15 +68,11 @@ class RESTSender : public RESTProtocol, public C2Protocol {
   std::string ack_uri_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<RESTSender>::getLogger();
 };
 
-REGISTER_RESOURCE(RESTSender, "Encapsulates the restful protocol that is built upon C2Protocol.");
-
-}  // namesapce c2
+}  // namespace c2
 }  // namespace minifi
 }  // namespace nifi
 }  // namespace apache
 }  // namespace org
-
-#endif /* LIBMINIFI_INCLUDE_C2_RESTPROTOCOL_H_ */
