@@ -31,6 +31,7 @@
 #include "io/CRCStream.h"
 #include "sitetosite/Peer.h"
 #include "io/validation.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -40,7 +41,7 @@ namespace sitetosite {
 
 std::shared_ptr<utils::IdGenerator> HttpSiteToSiteClient::id_generator_ = utils::IdGenerator::getIdGenerator();
 
-utils::optional<utils::Identifier> HttpSiteToSiteClient::parseTransactionId(const std::string &uri) {
+std::optional<utils::Identifier> HttpSiteToSiteClient::parseTransactionId(const std::string &uri) {
   const size_t last_slash_pos = uri.find_last_of('/');
   size_t id_start_pos = 0;
   if (last_slash_pos != std::string::npos) {
@@ -287,7 +288,7 @@ void HttpSiteToSiteClient::closeTransaction(const utils::Identifier &transaction
   if (client->getResponseCode() == 400) {
     std::string error(client->getResponseBody().data(), client->getResponseBody().size());
 
-    logging::LOG_WARN(logger_) << "400 received: " << error;
+    core::logging::LOG_WARN(logger_) << "400 received: " << error;
     std::stringstream message;
     message << "Received " << client->getResponseCode() << " from " << uri.str();
     throw Exception(SITE2SITE_EXCEPTION, message.str());
@@ -304,6 +305,8 @@ void HttpSiteToSiteClient::deleteTransaction(const utils::Identifier& transactio
   SiteToSiteClient::deleteTransaction(transactionID);
 }
 
+REGISTER_INTERNAL_RESOURCE_AS(HttpSiteToSiteClient, ("HttpSiteToSiteClient", "HttpProtocol"));
+
 } /* namespace sitetosite */
 } /* namespace minifi */
 } /* namespace nifi */
