@@ -15,30 +15,22 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __SITE2SITE_CLIENT_PROTOCOL_H__
-#define __SITE2SITE_CLIENT_PROTOCOL_H__
+#pragma once
 
-#include <stdio.h>
-#include <sys/types.h>
+#include <map>
+#include <memory>
+#include <optional>
 #include <string>
-#include <errno.h>
-#include <chrono>
-#include <set>
-#include <thread>
-#include <algorithm>
+#include <utility>
+#include <vector>
 #include "HTTPTransaction.h"
 #include "sitetosite/SiteToSite.h"
 #include "sitetosite/SiteToSiteClient.h"
-#include "core/Property.h"
-#include "properties/Configure.h"
-#include "FlowFileRecord.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
-#include "io/CRCStream.h"
 #include "sitetosite/Peer.h"
 #include "utils/Id.h"
-#include "../client/HTTPClient.h"
 
 namespace org {
 namespace apache {
@@ -58,30 +50,28 @@ typedef struct Site2SitePeerStatus {
 // HttpSiteToSiteClient Class
 class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
   static constexpr char const* PROTOCOL_VERSION_HEADER = "x-nifi-site-to-site-protocol-version";
- public:
 
+ public:
   /*!
    * Create a new http protocol
    */
-  HttpSiteToSiteClient(const std::string& /*name*/, const utils::Identifier& /*uuid*/ = {})
+  explicit HttpSiteToSiteClient(const std::string& /*name*/, const utils::Identifier& /*uuid*/ = {})
       : SiteToSiteClient(),
-        current_code(UNRECOGNIZED_RESPONSE_CODE),
-        logger_(logging::LoggerFactory<HttpSiteToSiteClient>::getLogger()) {
+        current_code(UNRECOGNIZED_RESPONSE_CODE) {
     peer_state_ = READY;
   }
 
   /*!
    * Create a new http protocol
    */
-  HttpSiteToSiteClient(std::unique_ptr<SiteToSitePeer> peer)
+  explicit HttpSiteToSiteClient(std::unique_ptr<SiteToSitePeer> peer)
       : SiteToSiteClient(),
-        current_code(UNRECOGNIZED_RESPONSE_CODE),
-        logger_(logging::LoggerFactory<HttpSiteToSiteClient>::getLogger()) {
+        current_code(UNRECOGNIZED_RESPONSE_CODE) {
     peer_ = std::move(peer);
     peer_state_ = READY;
   }
   // Destructor
-  virtual ~HttpSiteToSiteClient() = default;
+  ~HttpSiteToSiteClient() override = default;
 
   void setPeer(std::unique_ptr<SiteToSitePeer> peer) override {
     peer_ = std::move(peer);
@@ -108,7 +98,6 @@ class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
   void deleteTransaction(const utils::Identifier& transactionID) override;
 
  protected:
-
   /**
    * Closes the transaction
    * @param transactionID transaction id reference.
@@ -143,7 +132,7 @@ class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
    */
   std::shared_ptr<minifi::utils::HTTPClient> openConnectionForReceive(const std::shared_ptr<HttpTransaction> &transaction);
 
-  const std::string getBaseURI() {
+  std::string getBaseURI() {
     std::string uri = ssl_context_service_ != nullptr ? "https://" : "http://";
     uri.append(peer_->getHostName());
     uri.append(":");
@@ -154,10 +143,10 @@ class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
 
   void tearDown() override;
 
-  utils::optional<utils::Identifier> parseTransactionId(const std::string &uri);
+  std::optional<utils::Identifier> parseTransactionId(const std::string &uri);
 
   std::unique_ptr<utils::HTTPClient> create_http_client(const std::string &uri, const std::string &method = "POST", bool setPropertyHeaders = false) {
-    std::unique_ptr<utils::HTTPClient> http_client_ = std::unique_ptr<utils::HTTPClient>(new minifi::utils::HTTPClient(uri, ssl_context_service_));
+    std::unique_ptr<utils::HTTPClient> http_client_ = std::make_unique<utils::HTTPClient>(uri, ssl_context_service_);
     http_client_->initialize(method, uri, ssl_context_service_);
     if (setPropertyHeaders) {
       if (_currentVersion >= 5) {
@@ -177,9 +166,8 @@ class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
   }
 
  private:
-
   RespondCode current_code;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<HttpSiteToSiteClient>::getLogger();
   // Prevent default copy constructor and assignment operation
   // Only support pass by reference or pointer
   HttpSiteToSiteClient(const HttpSiteToSiteClient &parent);
@@ -192,4 +180,3 @@ class HttpSiteToSiteClient : public sitetosite::SiteToSiteClient {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif
