@@ -18,6 +18,9 @@
 #ifndef EXTENSIONS_HTTP_CURL_SITETOSITE_HTTPTRANSACTION_H_
 #define EXTENSIONS_HTTP_CURL_SITETOSITE_HTTPTRANSACTION_H_
 
+#include <utility>
+#include <string>
+
 #include "sitetosite/SiteToSite.h"
 #include "io/CRCStream.h"
 #include "sitetosite/SiteToSiteClient.h"
@@ -56,6 +59,7 @@ class HttpTransaction : public sitetosite::Transaction {
   const std::string &getTransactionUrl() {
     return transaction_url_;
   }
+
  protected:
   sitetosite::SiteToSiteClient *client_ref_;
   std::string transaction_url_;
