@@ -21,10 +21,12 @@
 #include "HTTPHandlers.h"
 #include "utils/IntegrationTestUtils.h"
 
+using std::literals::chrono_literals::operator""s;
+
 int main(int argc, char **argv) {
   const cmd_args args = parse_cmdline_args(argc, argv, "update");
   C2FailedUpdateHandler handler(args.bad_test_file);
-  VerifyC2FailedUpdate harness(10000);
+  VerifyC2FailedUpdate harness(10s);
   harness.setKeyDir(args.key_dir);
   harness.setUrl(args.url, &handler);
   handler.setC2RestResponse(harness.getC2RestUrl(), "configuration");
