@@ -23,6 +23,7 @@
 #include <iostream>
 #include "InvokeHTTP.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "core/logging/Logger.h"
 #include "core/ProcessGroup.h"
 #include "TestServer.h"
@@ -32,21 +33,22 @@
 #include "HTTPIntegrationBase.h"
 #include "utils/IntegrationTestUtils.h"
 
+namespace org::apache::nifi::minifi {
+
 class VerifyC2Server : public HTTPIntegrationBase {
-public:
+ public:
   explicit VerifyC2Server(bool isSecure)
       : isSecure(isSecure) {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
     LogTestController::getInstance().setDebug<utils::HTTPClient>();
     LogTestController::getInstance().setDebug<processors::InvokeHTTP>();
-    LogTestController::getInstance().setDebug<minifi::c2::RESTReceiver>();
-    LogTestController::getInstance().setDebug<minifi::c2::C2Agent>();
+    LogTestController::getInstance().setDebug<c2::RESTReceiver>();
+    LogTestController::getInstance().setDebug<c2::C2Agent>();
     LogTestController::getInstance().setDebug<processors::LogAttribute>();
-    LogTestController::getInstance().setDebug<minifi::core::ProcessSession>();
+    LogTestController::getInstance().setDebug<core::ProcessSession>();
     std::fstream file;
     ss << dir << "/" << "tstFile.ext";
     file.open(ss.str(), std::ios::out);
@@ -55,33 +57,31 @@ public:
   }
 
   void runAssertions() override {
-    using org::apache::nifi::minifi::utils::verifyLogLinePresenceInPollTime;
-    assert(verifyLogLinePresenceInPollTime(std::chrono::milliseconds(wait_time_),
+    assert(utils::verifyLogLinePresenceInPollTime(std::chrono::milliseconds(wait_time_),
         "C2Agent] [error] Could not instantiate null",
         "Class is RESTSender"));
   }
 
   void queryRootProcessGroup(std::shared_ptr<core::ProcessGroup> pg) override {
-    std::shared_ptr<core::Processor> proc = pg->findProcessorByName("invoke");
+    auto* const proc = pg->findProcessorByName("invoke");
     assert(proc != nullptr);
 
-    std::shared_ptr<minifi::processors::InvokeHTTP> inv = std::dynamic_pointer_cast<minifi::processors::InvokeHTTP>(proc);
+    const auto* const inv = dynamic_cast<minifi::processors::InvokeHTTP*>(proc);
 
     assert(inv != nullptr);
     std::string url;
-    inv->getProperty(minifi::processors::InvokeHTTP::URL.getName(), url);
+    inv->getProperty(processors::InvokeHTTP::URL.getName(), url);
 
     std::string port, scheme, path;
     parse_http_components(url, port, scheme, path);
-    configuration->set("c2.enable", "true");
-    configuration->set("c2.agent.class", "test");
-    configuration->set("c2.agent.protocol.class", "RESTSender");
-    configuration->set("c2.rest.url", "");
-    configuration->set("c2.rest.url.ack", "");
-    configuration->set("c2.agent.heartbeat.reporter.classes", "null");
-    configuration->set("c2.rest.listener.port", "null");
-    configuration->set("c2.agent.heartbeat.period", "null");
-    configuration->set("c2.rest.listener.heartbeat.rooturi", "null");
+    configuration->set(Configuration::nifi_c2_enable, "true");
+    configuration->set(Configuration::nifi_c2_agent_class, "test");
+    configuration->set(Configuration::nifi_c2_agent_protocol_class, "RESTSender");
+    configuration->set(Configuration::nifi_c2_rest_url, "");
+    configuration->set(Configuration::nifi_c2_rest_url_ack, "");
+    configuration->set(Configuration::nifi_c2_agent_heartbeat_reporter_classes, "null");
+    configuration->set(Configuration::nifi_c2_rest_listener_port, "null");
+    configuration->set(Configuration::nifi_c2_agent_heartbeat_period, "null");
   }
 
  protected:
@@ -91,11 +91,13 @@ public:
   TestController testController;
 };
 
+}  // namespace org::apache::nifi::minifi
+
 int main(int argc, char **argv) {
   const cmd_args args = parse_cmdline_args(argc, argv);
   const bool isSecure = args.isUrlSecure();
 
-  VerifyC2Server harness(isSecure);
+  org::apache::nifi::minifi::VerifyC2Server harness(isSecure);
   harness.setKeyDir(args.key_dir);
   harness.run(args.test_file);
   return 0;
