@@ -17,7 +17,11 @@
  */
 
 #undef NDEBUG
+
+#include <memory>
+
 #include "TestBase.h"
+#include "Catch.h"
 #include "HTTPIntegrationBase.h"
 #include "HTTPHandlers.h"
 #include "InvokeHTTP.h"
@@ -27,15 +31,20 @@
 #include "properties/Configure.h"
 #include "io/StreamFactory.h"
 #include "integration/IntegrationBase.h"
-#include "utils/GeneralUtils.h"
 
 class VerifyC2PauseResume : public VerifyC2Base {
  public:
-  explicit VerifyC2PauseResume(const std::atomic_bool& flow_resumed_successfully) : VerifyC2Base(), flow_resumed_successfully_(flow_resumed_successfully) {}
+  explicit VerifyC2PauseResume(const std::atomic_bool& flow_resumed_successfully) : VerifyC2Base(), flow_resumed_successfully_(flow_resumed_successfully) {
+    LogTestController::getInstance().setTrace<minifi::c2::C2Agent>();
+    LogTestController::getInstance().setDebug<minifi::c2::RESTSender>();
+    LogTestController::getInstance().setDebug<minifi::FlowController>();
+    LogTestController::getInstance().setDebug<minifi::core::ProcessContext>();
+    LogTestController::getInstance().setTrace<minifi::core::ProcessSession>();
+  }
 
   void configureC2() override {
     VerifyC2Base::configureC2();
-    configuration->set("nifi.c2.agent.heartbeat.period", "500");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_heartbeat_period, "500");
   }
 
   void runAssertions() override {
@@ -52,7 +61,9 @@ class PauseResumeHandler: public HeartbeatHandler {
   static const uint32_t PAUSE_SECONDS = 3;
   static const uint32_t INITIAL_GET_INVOKE_COUNT = 2;
 
-  explicit PauseResumeHandler(std::atomic_bool& flow_resumed_successfully) : HeartbeatHandler(), flow_resumed_successfully_(flow_resumed_successfully) {}
+  explicit PauseResumeHandler(std::atomic_bool& flow_resumed_successfully, std::shared_ptr<minifi::Configure> configuration)
+    : HeartbeatHandler(std::move(configuration)), flow_resumed_successfully_(flow_resumed_successfully) {
+  }
   bool handleGet(CivetServer* /*server*/, struct mg_connection* conn) override {
     assert(flow_state_ != FlowState::PAUSED);
     ++get_invoke_count_;
@@ -70,7 +81,7 @@ class PauseResumeHandler: public HeartbeatHandler {
       pause_start_time_ = std::chrono::system_clock::now();
       flow_state_ = FlowState::PAUSED;
       operation = "pause";
-    } else if (get_invoke_count_ == INITIAL_GET_INVOKE_COUNT && flow_state_ == FlowState::STARTED) {
+    } else if (get_invoke_count_ >= INITIAL_GET_INVOKE_COUNT && flow_state_ == FlowState::STARTED) {
       flow_state_ = FlowState::PAUSE_INITIATED;
       operation = "pause";
     } else if (flow_state_ == FlowState::PAUSED) {
@@ -100,7 +111,7 @@ class PauseResumeHandler: public HeartbeatHandler {
   };
 
   std::atomic<uint32_t> get_invoke_count_{0};
-  std::chrono::time_point<std::chrono::system_clock> pause_start_time_;
+  std::chrono::system_clock::time_point pause_start_time_;
   std::atomic<FlowState> flow_state_{FlowState::STARTED};
   std::atomic_bool& flow_resumed_successfully_;
 };
@@ -110,7 +121,7 @@ int main(int argc, char **argv) {
   std::atomic_bool flow_resumed_successfully{false};
   VerifyC2PauseResume harness{flow_resumed_successfully};
   harness.setKeyDir(args.key_dir);
-  PauseResumeHandler responder{flow_resumed_successfully};
+  PauseResumeHandler responder{flow_resumed_successfully, harness.getConfiguration()};
 
   std::shared_ptr<core::Repository> test_repo = std::make_shared<TestRepository>();
   std::shared_ptr<core::Repository> test_flow_repo = std::make_shared<TestFlowRepository>();
@@ -122,25 +133,27 @@ int main(int argc, char **argv) {
   std::shared_ptr<core::ContentRepository> content_repo = std::make_shared<core::repository::VolatileContentRepository>();
   content_repo->initialize(configuration);
 
-  std::unique_ptr<core::FlowConfiguration> yaml_ptr = utils::make_unique<core::YamlConfiguration>(
+  std::unique_ptr<core::FlowConfiguration> yaml_ptr = std::make_unique<core::YamlConfiguration>(
     test_repo, test_repo, content_repo, stream_factory, configuration, args.test_file);
 
   std::shared_ptr<minifi::FlowController> controller = std::make_shared<minifi::FlowController>(
-      test_repo, test_flow_repo, configuration, std::move(yaml_ptr), content_repo, DEFAULT_ROOT_GROUP_NAME, true);
+      test_repo, test_flow_repo, configuration, std::move(yaml_ptr), content_repo, DEFAULT_ROOT_GROUP_NAME,
+      std::make_shared<utils::file::FileSystem>(), []{});
 
   core::YamlConfiguration yaml_config(test_repo, test_repo, content_repo, stream_factory, configuration, args.test_file);
 
-  std::shared_ptr<core::Processor> proc = yaml_config.getRoot()->findProcessorByName("invoke");
+  auto root = yaml_config.getRoot();
+  const auto proc = root->findProcessorByName("invoke");
   assert(proc != nullptr);
 
-  const auto inv = std::dynamic_pointer_cast<minifi::processors::InvokeHTTP>(proc);
+  const auto inv = dynamic_cast<minifi::processors::InvokeHTTP*>(proc);
   assert(inv != nullptr);
   std::string url;
   inv->getProperty(minifi::processors::InvokeHTTP::URL.getName(), url);
   std::string port, scheme, path;
   std::unique_ptr<TestServer> server;
   parse_http_components(url, port, scheme, path);
-  server = utils::make_unique<TestServer>(port, path, &responder);
+  server = std::make_unique<TestServer>(port, path, &responder);
 
   harness.setUrl(args.url, &responder);
   harness.run(args.test_file);
