@@ -15,8 +15,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_HTTPCURL_TESTS_HTTPINTEGRATIONBASE_H
-#define EXTENSIONS_HTTPCURL_TESTS_HTTPINTEGRATIONBASE_H
+#pragma once
+
+#include <memory>
+#include <string>
+#include <atomic>
 
 #include "CivetServer.h"
 #include "integration/IntegrationBase.h"
@@ -24,8 +27,10 @@
 #include "protocols/RESTSender.h"
 #include "ServerAwareHandler.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/IntegrationTestUtils.h"
 #include "TestServer.h"
+#include "properties/Configuration.h"
 
 int log_message(const struct mg_connection* /*conn*/, const char *message) {
   puts(message);
@@ -37,11 +42,15 @@ int ssl_enable(void* /*ssl_context*/, void* /*user_data*/) {
 }
 
 class HTTPIntegrationBase : public IntegrationBase {
-public:
-  explicit HTTPIntegrationBase(uint64_t waitTime = DEFAULT_WAITTIME_MSECS)
+ public:
+  explicit HTTPIntegrationBase(std::chrono::milliseconds waitTime = std::chrono::milliseconds(DEFAULT_WAITTIME_MSECS))
       : IntegrationBase(waitTime),
         server(nullptr) {
   }
+  HTTPIntegrationBase(const HTTPIntegrationBase&) = delete;
+  HTTPIntegrationBase(HTTPIntegrationBase&&) = default;
+  HTTPIntegrationBase& operator=(const HTTPIntegrationBase&) = delete;
+  HTTPIntegrationBase& operator=(HTTPIntegrationBase&&) = default;
 
   virtual void setUrl(const std::string &url, ServerAwareHandler *handler);
 
@@ -61,7 +70,7 @@ public:
 
   std::string getC2RestUrl() const {
     std::string c2_rest_url;
-    configuration->get("nifi.c2.rest.url", c2_rest_url);
+    configuration->get(org::apache::nifi::minifi::Configuration::nifi_c2_rest_url, c2_rest_url);
     return c2_rest_url;
   }
 
@@ -91,9 +100,9 @@ void HTTPIntegrationBase::setUrl(const std::string &url, ServerAwareHandler *han
     callback.init_ssl = ssl_enable;
     port += "s";
     callback.log_message = log_message;
-    server = utils::make_unique<TestServer>(port, url_path, handler, &callback, cert, cert);
+    server = std::make_unique<TestServer>(port, url_path, handler, &callback, cert, cert);
   } else {
-    server = utils::make_unique<TestServer>(port, url_path, handler);
+    server = std::make_unique<TestServer>(port, url_path, handler);
   }
   bool secure{false};
   if (port == "0" || port == "0s") {
@@ -104,8 +113,8 @@ void HTTPIntegrationBase::setUrl(const std::string &url, ServerAwareHandler *han
     }
   }
   std::string c2_url = std::string("http") + (secure ? "s" : "") + "://localhost:" + getWebPort() + url_path;
-  configuration->set("nifi.c2.rest.url", c2_url);
-  configuration->set("nifi.c2.rest.url.ack", c2_url);
+  configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_rest_url, c2_url);
+  configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_rest_url_ack, c2_url);
 }
 
 void HTTPIntegrationBase::setC2Url(const std::string &heartbeat_path, const std::string &acknowledge_path) {
@@ -114,23 +123,24 @@ void HTTPIntegrationBase::setC2Url(const std::string &heartbeat_path, const std:
   }
   bool secure = port.back() == 's';
   std::string base = std::string("http") + (secure ? "s" : "") + "://localhost:" + getWebPort();
-  configuration->set("nifi.c2.rest.url", base + heartbeat_path);
-  configuration->set("nifi.c2.rest.url.ack", base + acknowledge_path);
+  configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_rest_url, base + heartbeat_path);
+  configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_rest_url_ack, base + acknowledge_path);
 }
 
 class VerifyC2Base : public HTTPIntegrationBase {
  public:
+  using HTTPIntegrationBase::HTTPIntegrationBase;
   void testSetup() override {
     LogTestController::getInstance().setDebug<utils::HTTPClient>();
     LogTestController::getInstance().setDebug<LogTestController>();
   }
 
   void configureC2() override {
-    configuration->set("nifi.c2.agent.protocol.class", "RESTSender");
-    configuration->set("nifi.c2.enable", "true");
-    configuration->set("nifi.c2.agent.class", "test");
-    configuration->set("nifi.c2.agent.heartbeat.period", "1000");
-    configuration->set("nifi.c2.root.classes", "DeviceInfoNode,AgentInformation,FlowInformation");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_protocol_class, "RESTSender");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "true");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_class, "test");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_heartbeat_period, "1000");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_root_classes, "DeviceInfoNode,AgentInformation,FlowInformation");
   }
 
   void cleanup() override {
@@ -141,6 +151,10 @@ class VerifyC2Base : public HTTPIntegrationBase {
 
 class VerifyC2Describe : public VerifyC2Base {
  public:
+  explicit VerifyC2Describe(std::atomic<bool>& verified)
+    : verified_(verified) {
+  }
+
   void testSetup() override {
     LogTestController::getInstance().setTrace<minifi::c2::C2Agent>();
     LogTestController::getInstance().setDebug<minifi::c2::RESTSender>();
@@ -149,19 +163,21 @@ class VerifyC2Describe : public VerifyC2Base {
   }
 
   void configureFullHeartbeat() override {
-    configuration->set("nifi.c2.full.heartbeat", "false");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_full_heartbeat, "false");
   }
 
   void runAssertions() override {
-    // This class is never used for running assertions, but we are forced to wait for DescribeManifestHandler to verifyJsonHasAgentManifest
-    // if we were to log something on finished verification, we could poll on finding it
-    std::this_thread::sleep_for(std::chrono::milliseconds(wait_time_));
+    using org::apache::nifi::minifi::utils::verifyEventHappenedInPollTime;
+    assert(verifyEventHappenedInPollTime(std::chrono::milliseconds(wait_time_), [&] { return verified_.load(); }));
   }
+
+ protected:
+  std::atomic<bool>& verified_;
 };
 
 class VerifyC2Update : public HTTPIntegrationBase {
-public:
-  explicit VerifyC2Update(uint64_t waitTime)
+ public:
+  explicit VerifyC2Update(std::chrono::milliseconds waitTime)
       : HTTPIntegrationBase(waitTime) {
   }
 
@@ -173,10 +189,10 @@ public:
   }
 
   void configureC2() override {
-    configuration->set("nifi.c2.agent.protocol.class", "RESTSender");
-    configuration->set("nifi.c2.enable", "true");
-    configuration->set("nifi.c2.agent.class", "test");
-    configuration->set("nifi.c2.agent.heartbeat.period", "1000");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_protocol_class, "RESTSender");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "true");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_class, "test");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_heartbeat_period, "1000");
   }
 
   void cleanup() override {
@@ -202,14 +218,14 @@ class VerifyFlowFetched : public HTTPIntegrationBase {
   }
 
   void configureC2() override {
-    configuration->set("nifi.c2.agent.protocol.class", "RESTSender");
-    configuration->set("nifi.c2.enable", "true");
-    configuration->set("nifi.c2.agent.class", "test");
-    configuration->set("nifi.c2.agent.heartbeat.period", "1000");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_protocol_class, "RESTSender");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "true");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_class, "test");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_agent_heartbeat_period, "1000");
   }
 
   void setFlowUrl(const std::string& url) {
-    configuration->set(minifi::Configure::nifi_c2_flow_url, url);
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_flow_url, url);
   }
 
   void cleanup() override {
@@ -224,15 +240,15 @@ class VerifyFlowFetched : public HTTPIntegrationBase {
 };
 
 class VerifyC2FailedUpdate : public VerifyC2Update {
-public:
-  explicit VerifyC2FailedUpdate(uint64_t waitTime)
+ public:
+  explicit VerifyC2FailedUpdate(std::chrono::milliseconds waitTime)
       : VerifyC2Update(waitTime) {
   }
 
   void testSetup() override {
     LogTestController::getInstance().setInfo<minifi::FlowController>();
     LogTestController::getInstance().setDebug<minifi::c2::C2Agent>();
-    utils::file::FileUtils::create_dir("content_repository");
+    utils::file::create_dir("content_repository");
   }
 
   void runAssertions() override {
@@ -241,8 +257,7 @@ public:
   }
 
   void cleanup() override {
-    utils::file::FileUtils::delete_dir("content_repository", true);
+    utils::file::delete_dir("content_repository", true);
     VerifyC2Update::cleanup();
   }
 };
-#endif  // EXTENSIONS_HTTPCURL_TESTS_HTTPINTEGRATIONBASE_H
