@@ -26,6 +26,7 @@
 #include "sitetosite/HTTPProtocol.h"
 #include "InvokeHTTP.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "FlowController.h"
 #include "io/StreamFactory.h"
 #include "RemoteProcessorGroupPort.h"
@@ -33,13 +34,15 @@
 #include "HTTPIntegrationBase.h"
 #include "HTTPHandlers.h"
 #include "client/HTTPStream.h"
+#include "properties/Configuration.h"
+
+using std::literals::chrono_literals::operator""s;
 
 class SiteToSiteTestHarness : public HTTPIntegrationBase {
-public:
-  explicit SiteToSiteTestHarness(bool isSecure, std::chrono::milliseconds waitTime = std::chrono::milliseconds{2000})
-      : HTTPIntegrationBase(waitTime.count()), isSecure(isSecure) {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+ public:
+  explicit SiteToSiteTestHarness(bool isSecure, std::chrono::seconds waitTime = 2s)
+      : HTTPIntegrationBase(waitTime), isSecure(isSecure) {
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
@@ -53,14 +56,12 @@ public:
     LogTestController::getInstance().setTrace<utils::HttpStreamingCallback>();
 
     std::fstream file;
-    ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+    ss << dir << utils::file::get_separator() << "tstFile.ext";
     file.open(ss.str(), std::ios::out);
     file << "tempFile";
     file.close();
 
-    configuration->set("nifi.c2.enable", "false");
-    configuration->set("nifi.remote.input.http.enabled", "true");
-    configuration->set("nifi.remote.input.socket.port", "8099");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "false");
   }
 
   void runAssertions() override {
