@@ -15,14 +15,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_TEST_TESTSERVER_H_
-#define LIBMINIFI_TEST_TESTSERVER_H_
+#pragma once
 
 #include <string>
-#include <iostream>
+#include <memory>
+#include <vector>
 #include "civetweb.h"
 #include "CivetServer.h"
-#include "civetweb.h"
 #include "HTTPUtils.h"
 #include "ServerAwareHandler.h"
 
@@ -49,8 +48,9 @@ class TestServer{
       return counter;
     }
   };
+
  public:
-  TestServer(std::string &port, std::string &rooturi, CivetHandler *handler, CivetCallbacks *callbacks, std::string& /*cert*/, std::string &ca_cert) {
+  TestServer(std::string &port, std::string &rooturi, CivetHandler *handler, CivetCallbacks *callbacks, std::string& cert, std::string &ca_cert) {
     if (!mg_check_feature(2)) {
       throw std::runtime_error("Error: Embedded example built with SSL support, "
                                "but civetweb library build without.\n");
@@ -59,15 +59,15 @@ class TestServer{
 
     // ECDH+AESGCM+AES256:!aNULL:!MD5:!DSS
     const std::vector<std::string> cpp_options{ "document_root", ".", "listening_ports", port, "error_log_file",
-                              "error.log", "ssl_certificate", ca_cert, "ssl_protocol_version", "4", "ssl_cipher_list",
-                              "ALL", "request_timeout_ms", "10000", "enable_auth_domain_check", "no", "ssl_verify_peer", "no"};
-    server_ = utils::make_unique<CivetServer>(cpp_options, callbacks);
+                              "error.log", "ssl_certificate", cert, "ssl_ca_file", ca_cert, "ssl_protocol_version", "4", "ssl_cipher_list",
+                              "ALL", "request_timeout_ms", "10000", "enable_auth_domain_check", "no", "ssl_verify_peer", "yes"};
+    server_ = std::make_unique<CivetServer>(cpp_options, callbacks);
     addHandler(rooturi, handler);
   }
 
   TestServer(const std::string& port, const std::string& rooturi, CivetHandler* handler) {
     const std::vector<std::string> cpp_options{"document_root", ".", "listening_ports", port};
-    server_ = utils::make_unique<CivetServer>(cpp_options);
+    server_ = std::make_unique<CivetServer>(cpp_options);
     addHandler(rooturi, handler);
   }
 
@@ -86,6 +86,7 @@ class TestServer{
       if (serverAwareHandler) serverAwareHandler->stop();
     }
   }
+
  private:
   // server_ depends on lib_ (the library initializer)
   // so their order matters
@@ -94,5 +95,3 @@ class TestServer{
 
   std::vector<CivetHandler*> handlers_;
 };
-
-#endif
