@@ -27,6 +27,7 @@
 #include "sitetosite/HTTPProtocol.h"
 #include "InvokeHTTP.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "FlowController.h"
 #include "io/StreamFactory.h"
 #include "RemoteProcessorGroupPort.h"
@@ -34,13 +35,15 @@
 #include "HTTPIntegrationBase.h"
 #include "HTTPHandlers.h"
 #include "client/HTTPStream.h"
+#include "properties/Configuration.h"
+
+using std::literals::chrono_literals::operator""s;
 
 class SiteToSiteTestHarness : public HTTPIntegrationBase {
-public:
-  explicit SiteToSiteTestHarness(bool isSecure, std::chrono::milliseconds waitTime = std::chrono::milliseconds{1000})
-      : HTTPIntegrationBase(waitTime.count()), isSecure(isSecure) {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+ public:
+  explicit SiteToSiteTestHarness(bool isSecure, std::chrono::seconds waitTime = 1s)
+      : HTTPIntegrationBase(waitTime), isSecure(isSecure) {
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
@@ -59,9 +62,7 @@ public:
     file << "tempFile";
     file.close();
 
-    configuration->set("nifi.c2.enable", "false");
-    configuration->set("nifi.remote.input.http.enabled", "true");
-    configuration->set("nifi.remote.input.socket.port", "8099");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "false");
   }
 
   void runAssertions() override {
@@ -69,7 +70,7 @@ public:
     std::this_thread::sleep_for(std::chrono::milliseconds(wait_time_));
   }
 
-protected:
+ protected:
   bool isSecure;
   std::string dir;
   std::stringstream ss;
