@@ -36,6 +36,7 @@
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "ResourceClaim.h"
 #include "utils/StringUtils.h"
 #include "utils/ByteArrayCallback.h"
@@ -105,6 +106,8 @@ void ExecuteJavaControllerService::onEnable() {
   }
 }
 
+REGISTER_RESOURCE(ExecuteJavaControllerService, "ExecuteJavaClass runs NiFi Controller services given a provided system path ");
+
 } /* namespace controllers */
 } /* namespace jni */
 } /* namespace minifi */
