@@ -16,18 +16,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __EXECUTE_JAVA_CS_
-#define __EXECUTE_JAVA_CS_
+#pragma once
 
 #include <memory>
 #include <regex>
+#include <string>
 
 #include "FlowFileRecord.h"
 #include "core/controller/ControllerService.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "concurrentqueue.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "jvm/JavaControllerService.h"
@@ -55,16 +54,12 @@ namespace controllers {
  */
 class ExecuteJavaControllerService : public ConfigurationContext, public std::enable_shared_from_this<ConfigurationContext> {
  public:
-
   // Constructor
   /*!
    * Create a new processor
    */
   explicit ExecuteJavaControllerService(const std::string& name, const utils::Identifier& uuid = {})
-      : ConfigurationContext(name, uuid),
-        contextInstance(nullptr),
-        clazzInstance(nullptr),
-        logger_(logging::LoggerFactory<ExecuteJavaControllerService>::getLogger()) {
+      : ConfigurationContext(name, uuid) {
   }
   // Destructor
   virtual ~ExecuteJavaControllerService();
@@ -136,32 +131,25 @@ class ExecuteJavaControllerService : public ConfigurationContext, public std::en
     return methodSignatures;
   }
 
- protected:
-
  private:
-
   minifi::jni::JniConfigurationContext config_context_;
 
-  jobject contextInstance;
+  jobject contextInstance = nullptr;
 
   JavaClass current_cs_class;
 
-  jobject clazzInstance;
+  jobject clazzInstance = nullptr;
 
   std::shared_ptr<controllers::JavaControllerService> java_servicer_;
 
   std::string class_name_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ExecuteJavaControllerService>::getLogger();
 };
 
-REGISTER_RESOURCE(ExecuteJavaControllerService, "ExecuteJavaClass runs NiFi Controller services given a provided system path ");
-
 } /* namespace controllers */
 } /* namespace jni */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
