@@ -36,6 +36,7 @@
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "ResourceClaim.h"
 #include "utils/StringUtils.h"
 #include "utils/ByteArrayCallback.h"
@@ -49,10 +50,13 @@ namespace minifi {
 namespace jni {
 namespace processors {
 
-core::Property ExecuteJavaProcessor::JVMControllerService(
-    core::PropertyBuilder::createProperty("JVM Controller Service")->withDescription("Name of controller service defined within this flow")->isRequired(false)->withDefaultValue<std::string>("")->build());
-core::Property ExecuteJavaProcessor::NiFiProcessor(
-    core::PropertyBuilder::createProperty("NiFi Processor")->withDescription("Name of NiFi processor to load and run")->isRequired(true)->withDefaultValue<std::string>("")->build());
+core::Property ExecuteJavaProcessor::JVMControllerService(core::PropertyBuilder::createProperty("JVM Controller Service")
+    ->withDescription("Name of controller service defined within this flow")
+    ->isRequired(false)->withDefaultValue<std::string>("")->build());
+
+core::Property ExecuteJavaProcessor::NiFiProcessor(core::PropertyBuilder::createProperty("NiFi Processor")
+    ->withDescription("Name of NiFi processor to load and run")
+    ->isRequired(true)->withDefaultValue<std::string>("")->build());
 
 const char *ExecuteJavaProcessor::ProcessorName = "ExecuteJavaClass";
 
@@ -95,7 +99,7 @@ void ExecuteJavaProcessor::onSchedule(const std::shared_ptr<core::ProcessContext
     throw std::runtime_error("NiFi Processor must be defined");
   }
 
-  nifi_logger_ = logging::LoggerFactory<ExecuteJavaProcessor>::getAliasedLogger(class_name_);
+  nifi_logger_ = core::logging::LoggerFactory<ExecuteJavaProcessor>::getAliasedLogger(class_name_);
 
   jni_logger_ref_.logger_reference_ = nifi_logger_;
 
@@ -151,7 +155,7 @@ void ExecuteJavaProcessor::onSchedule(const std::shared_ptr<core::ProcessContext
   jpc.nifi_processor_ = clazzInstance;
   jpc.context_ = context;
   jpc.clazz_ = spn.getReference();
-  jpc.processor_ = shared_from_this();
+  jpc.processor_ = this;
   jpc.cslookup_ = init_context_.lookup_ref_;
 
   java_servicer_->setReference<minifi::jni::JniProcessContext>(env, context_instance_, &jpc);
@@ -229,6 +233,8 @@ void ExecuteJavaProcessor::onTrigger(const std::shared_ptr<core::ProcessContext>
   // do nothing.
 }
 
+REGISTER_RESOURCE_AS(ExecuteJavaProcessor, "ExecuteJavaClass runs NiFi processors given a provided system path ", ("ExecuteJavaClass"));
+
 } /* namespace processors */
 } /* namespace jni */
 } /* namespace minifi */
