@@ -16,17 +16,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __EXECUTE_JAVA_CLASS__
-#define __EXECUTE_JAVA_CLASS__
+#pragma once
 
 #include <memory>
+#include <vector>
+#include <string>
 
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "concurrentqueue.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "jvm/JavaControllerService.h"
@@ -52,17 +52,12 @@ namespace processors {
  */
 class ExecuteJavaProcessor : public core::Processor {
  public:
-
   // Constructor
   /*!
    * Create a new processor
    */
   explicit ExecuteJavaProcessor(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(name, uuid),
-        logger_instance_(nullptr),
-        context_instance_(nullptr),
-        logger_(logging::LoggerFactory<ExecuteJavaProcessor>::getLogger()),
-        nifi_logger_(nullptr) {
+      : Processor(name, uuid) {
   }
   // Destructor
   virtual ~ExecuteJavaProcessor();
@@ -82,7 +77,6 @@ class ExecuteJavaProcessor : public core::Processor {
   }
 
  protected:
-
   static JavaSignatures &getLoggerSignatures() {
     static JavaSignatures loggersignatures;
     if (loggersignatures.empty()) {
@@ -104,12 +98,21 @@ class ExecuteJavaProcessor : public core::Processor {
   static JavaSignatures &getProcessContextSignatures() {
     static JavaSignatures methodSignatures;
     if (methodSignatures.empty()) {
-      methodSignatures.addSignature({ "getComponent", "()Lorg/apache/nifi/components/AbstractConfigurableComponent;", reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getComponent) });
-      methodSignatures.addSignature({ "getPropertyNames", "()Ljava/util/List;", reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getPropertyNames) });
-      methodSignatures.addSignature({ "getPropertyValue", "(Ljava/lang/String;)Ljava/lang/String;", reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getPropertyValue) });
-      methodSignatures.addSignature({ "getName", "()Ljava/lang/String;", reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getName) });
-      methodSignatures.addSignature({ "getControllerServiceLookup", "()Lorg/apache/nifi/controller/ControllerServiceLookup;",
-          reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getControllerServiceLookup) });
+      methodSignatures.addSignature({ "getComponent",
+                                      "()Lorg/apache/nifi/components/AbstractConfigurableComponent;",
+                                      reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getComponent) });
+      methodSignatures.addSignature({ "getPropertyNames",
+                                      "()Ljava/util/List;",
+                                      reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getPropertyNames) });
+      methodSignatures.addSignature({ "getPropertyValue",
+                                      "(Ljava/lang/String;)Ljava/lang/String;",
+                                      reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getPropertyValue) });
+      methodSignatures.addSignature({ "getName",
+                                      "()Ljava/lang/String;",
+                                      reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getName) });
+      methodSignatures.addSignature({ "getControllerServiceLookup",
+                                      "()Lorg/apache/nifi/controller/ControllerServiceLookup;",
+                                      reinterpret_cast<void*>(&Java_org_apache_nifi_processor_JniProcessContext_getControllerServiceLookup) });
     }
     return methodSignatures;
   }
@@ -234,7 +237,6 @@ class ExecuteJavaProcessor : public core::Processor {
   }
 
  private:
-
   JniSessionFactory *getFactory(const std::shared_ptr<core::ProcessSessionFactory> &ptr) {
     std::lock_guard<std::mutex> lock(local_mutex_);
     for (const auto &factory : session_factories_) {
@@ -256,7 +258,7 @@ class ExecuteJavaProcessor : public core::Processor {
 
   JavaClass jni_logger_class_;
 
-  jobject logger_instance_;
+  jobject logger_instance_ = nullptr;
 
   std::mutex local_mutex_;
 
@@ -272,30 +274,26 @@ class ExecuteJavaProcessor : public core::Processor {
 
   JavaClass current_processor_class;
 
-  jobject context_instance_;
+  jobject context_instance_ = nullptr;
 
-  jobject clazzInstance;
+  jobject clazzInstance = nullptr;
 
   std::shared_ptr<controllers::JavaControllerService> java_servicer_;
 
   std::string class_name_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ExecuteJavaProcessor>::getLogger();
 
-  std::shared_ptr<logging::Logger> nifi_logger_;
+  std::shared_ptr<core::logging::Logger> nifi_logger_;
 
   JniControllerServiceLookup csl_;
 
   JniInitializationContext init_context_;
 };
 
-REGISTER_RESOURCE(ExecuteJavaProcessor, "ExecuteJavaClass runs NiFi processors given a provided system path ");
-
 } /* namespace processors */
 } /* namespace jni */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
