@@ -17,6 +17,7 @@
  */
 
 #include "JVMCreator.h"
+#include "core/Resource.h"
 
 #ifndef WIN32
 
@@ -40,6 +41,8 @@ namespace jni {
 #endif
 JVMCreator::~JVMCreator() = default;
 
+REGISTER_INTERNAL_RESOURCE(JVMCreator);
+
 } /* namespace jni */
 } /* namespace minifi */
 } /* namespace nifi */
