@@ -16,8 +16,7 @@
  * limitations under the License.
  */
 
-#ifndef EXTENSIONS_JNI_JVMCREATOR_H_
-#define EXTENSIONS_JNI_JVMCREATOR_H_
+#pragma once
 
 #include <vector>
 #include <string>
@@ -27,6 +26,7 @@
 #include "utils/file/FileUtils.h"
 #include "core/Core.h"
 #include "core/logging/LoggerConfiguration.h"
+#include "properties/Configuration.h"
 
 namespace org {
 namespace apache {
@@ -39,11 +39,8 @@ namespace jni {
  */
 class JVMCreator : public minifi::core::CoreComponent {
  public:
-
   explicit JVMCreator(const std::string &name, const utils::Identifier &uuid = {})
-      : minifi::core::CoreComponent(name, uuid),
-        loader_(nullptr),
-        logger_(logging::LoggerFactory<JVMCreator>::getLogger()) {
+      : minifi::core::CoreComponent(name, uuid) {
   }
 
   virtual ~JVMCreator();
@@ -66,19 +63,19 @@ class JVMCreator : public minifi::core::CoreComponent {
 
     // assuming we have the options set and can access the JVMCreator
 
-    if (configuration->get("nifi.framework.dir", pathListings)) {
+    if (configuration->get(minifi::Configuration::nifi_framework_dir, pathListings)) {
       std::vector<std::string> paths;
       paths.emplace_back(pathListings);
       configure(paths);
 
-      if (configuration->get("nifi.jvm.options", jvmOptionsStr)) {
+      if (configuration->get(minifi::Configuration::nifi_jvm_options, jvmOptionsStr)) {
         jvm_options_ = utils::StringUtils::split(jvmOptionsStr, ",");
       }
 
       initializeJVM();
     }
     std::string nar_dir, nar_dep, nar_docs;
-    if (loader_ && configuration->get("nifi.nar.directory", nar_dir) && configuration->get("nifi.nar.deploy.directory", nar_dep)) {
+    if (loader_ && configuration->get(minifi::Configuration::nifi_nar_directory, nar_dir) && configuration->get(minifi::Configuration::nifi_nar_deploy_directory, nar_dep)) {
       std::shared_ptr<jni::controllers::JavaControllerService> servicer = std::make_shared<jni::controllers::JavaControllerService>("BaseService");
       servicer->initialize();
       servicer->setProperty(jni::controllers::JavaControllerService::NarDirectory, nar_dir);
@@ -94,14 +91,13 @@ class JVMCreator : public minifi::core::CoreComponent {
   }
 
  private:
-
-  minifi::jni::JVMLoader *loader_;
+  minifi::jni::JVMLoader *loader_ = nullptr;
 
   std::vector<std::string> jvm_options_;
 
   std::vector<std::string> classpaths_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<JVMCreator>::getLogger();
 };
 
 } /* namespace jni */
@@ -109,5 +105,3 @@ class JVMCreator : public minifi::core::CoreComponent {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* EXTENSIONS_JNI_JVMCREATOR_H_ */
