@@ -16,7 +16,7 @@
  * limitations under the License.
  */
 
-#include "../jvm/JavaControllerService.h"
+#include "JavaControllerService.h"
 
 #include <string>
 #include <memory>
@@ -24,6 +24,7 @@
 #include <iterator>
 #include <set>
 #include "core/Property.h"
+#include "core/Resource.h"
 #include "io/validation.h"
 #include "utils/StringUtils.h"
 #include "utils/file/FileUtils.h"
@@ -58,10 +59,10 @@ core::Property JavaControllerService::NarDocumentDirectory(
     core::PropertyBuilder::createProperty("Nar Document Directory")->withDescription("Directory in which documents will be deployed")->isRequired(true)->supportsExpressionLanguage(false)->build());
 
 void JavaControllerService::initialize() {
-  if (initialized_)
-    return;
-
   std::lock_guard<std::mutex> lock(initialization_mutex_);
+  if (initialized_) {
+    return;
+  }
 
   ControllerService::initialize();
 
@@ -108,6 +109,8 @@ void JavaControllerService::onEnable() {
   nar_loader_ = std::unique_ptr<NarClassLoader>(new NarClassLoader(shared_from_this(), narClassLoaderClazz, nardir, narscratch, nardocs));
 }
 
+REGISTER_RESOURCE(JavaControllerService, "Allows specification of nars to be used within referenced processors. ");
+
 } /* namespace controllers */
 } /* namespace jni */
 } /* namespace minifi */
