@@ -15,17 +15,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_CONTROLLERS_JAVACONTROLLERSERVICE_H_
-#define LIBMINIFI_INCLUDE_CONTROLLERS_JAVACONTROLLERSERVICE_H_
+#pragma once
 
 #include <iostream>
 #include <memory>
 #include <vector>
 #include <string>
+#include <utility>
+#include <map>
 
 #include "../jvm/JVMLoader.h"
 #include "NarClassLoader.h"
-#include "core/Resource.h"
 #include "utils/StringUtils.h"
 #include "JavaServicer.h"
 #include "io/validation.h"
@@ -47,17 +47,11 @@ namespace controllers {
 class JavaControllerService : public core::controller::ControllerService, public std::enable_shared_from_this<JavaControllerService>, public JavaServicer {
  public:
   explicit JavaControllerService(const std::string &name, const utils::Identifier &uuid = {})
-      : ControllerService(name, uuid),
-        loader(nullptr),
-        logger_(logging::LoggerFactory<JavaControllerService>::getLogger()) {
-    initialized_ = false;
+      : ControllerService(name, uuid) {
   }
 
   explicit JavaControllerService(const std::string &name, const std::shared_ptr<Configure> &configuration)
-      : ControllerService(name),
-        loader(nullptr),
-        logger_(logging::LoggerFactory<JavaControllerService>::getLogger()) {
-    initialized_ = false;
+      : ControllerService(name) {
     setConfiguration(configuration);
     initialize();
   }
@@ -143,34 +137,25 @@ class JavaControllerService : public core::controller::ControllerService, public
     return nar_loader_->newInstance(requested_name);
   }
 
- protected:
-
- // void addPath(std::vector<std::string> &jarFiles, const std::string &dir);
-
  private:
-
   JavaClass narClassLoaderClazz;
 
   std::mutex initialization_mutex_;
 
-  std::atomic<bool> initialized_;
+  bool initialized_ = false;
 
   std::vector<std::string> classpaths_;
 
   std::unique_ptr<NarClassLoader> nar_loader_;
 
-  JVMLoader *loader;
+  JVMLoader *loader = nullptr;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<JavaControllerService>::getLogger();
 };
 
-REGISTER_RESOURCE(JavaControllerService, "Allows specification of nars to be used within referenced processors. ");
-
 } /* namespace controllers */
 } /* namespace jni */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* LIBMINIFI_INCLUDE_CONTROLLERS_JAVACONTROLLERSERVICE_H_ */
