@@ -16,19 +16,24 @@
  * limitations under the License.
  */
 
-#ifndef EXTENSIONS_JNI_JVM_REFERNCEOBJECTS_H_
-#define EXTENSIONS_JNI_JVM_REFERNCEOBJECTS_H_
+#pragma once
+
+#include <jni.h>
 
 #include <string>
 #include <vector>
 #include <sstream>
 #include <iterator>
 #include <algorithm>
-#include <jni.h>
+#include <functional>
+#include <memory>
+#include <utility>
+
 #include "JavaServicer.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/WeakReference.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -71,7 +76,6 @@ class JniFlowFile : public core::WeakReference {
   }
 
  protected:
-
   bool removed;
 
   jobject ff_object;
@@ -92,38 +96,16 @@ struct check_empty_ff : public std::unary_function<std::shared_ptr<JniFlowFile>,
   }
 };
 
-class JniByteOutStream : public minifi::OutputStreamCallback {
- public:
-  JniByteOutStream(jbyte *bytes, size_t length)
-      : bytes_(bytes),
-        length_(length) {
-  }
-
-  virtual ~JniByteOutStream() = default;
-  virtual int64_t process(const std::shared_ptr<minifi::io::BaseStream>& stream) {
-    return stream->write((uint8_t*) bytes_, length_);
-  }
- private:
-  jbyte *bytes_;
-  size_t length_;
-};
-
 /**
  * Jni byte input stream
  */
-class JniByteInputStream : public minifi::InputStreamCallback {
+class JniByteInputStream {
  public:
-  JniByteInputStream(uint64_t size)
-      : stream_(nullptr),
+  explicit JniByteInputStream(uint64_t size)
+      : buffer_(size),
         read_size_(0) {
-    buffer_size_ = size;
-    buffer_ = new uint8_t[buffer_size_];
   }
-  ~JniByteInputStream() {
-    if (buffer_)
-      delete[] buffer_;
-  }
-  int64_t process(const std::shared_ptr<minifi::io::BaseStream>& stream) {
+  int64_t operator()(const std::shared_ptr<minifi::io::BaseStream>& stream) {
     stream_ = stream;
     return 0;
   }
@@ -135,11 +117,12 @@ class JniByteInputStream : public minifi::InputStreamCallback {
     }
 
     // seek to offset
-    uint64_t remaining = gsl::narrow<uint64_t>(size);
+    auto remaining = gsl::narrow<uint64_t>(size);
     int writtenOffset = 0;
     int read = 0;
     do {
-      int actual = stream_->read(buffer_, std::min(remaining, buffer_size_));
+      // JNI takes size as int, there's not much we can do here to support 2GB+ sizes
+      int actual = static_cast<int>(stream_->read(gsl::make_span(buffer_).subspan(0, std::min(remaining, buffer_.size()))));
       if (actual <= 0) {
         if (read == 0) {
           stream_ = nullptr;
@@ -149,7 +132,7 @@ class JniByteInputStream : public minifi::InputStreamCallback {
       }
 
       read += actual;
-      env->SetByteArrayRegion(arr, offset + writtenOffset, actual, (jbyte*) buffer_);
+      env->SetByteArrayRegion(arr, offset + writtenOffset, actual, reinterpret_cast<jbyte*>(buffer_.data()));
       writtenOffset += actual;
 
       remaining -= actual;
@@ -158,13 +141,12 @@ class JniByteInputStream : public minifi::InputStreamCallback {
     return read;
   }
 
-  int64_t read(char &arr) {
+  int64_t read(uint8_t &arr) {
     return stream_->read(arr);
   }
 
   std::shared_ptr<minifi::io::BaseStream> stream_;
-  uint8_t *buffer_;
-  uint64_t buffer_size_;
+  std::vector<std::byte> buffer_;
   uint64_t read_size_;
 };
 
@@ -186,7 +168,7 @@ class JniInputStream : public core::WeakReference {
     }
   }
 
-  int64_t read(char &arr) {
+  int64_t read(uint8_t &arr) {
     if (!removed_) {
       return jbi_->read(arr);
     }
@@ -296,7 +278,6 @@ struct check_empty : public std::unary_function<std::shared_ptr<JniSession>, boo
 
 class JniSessionFactory : public core::WeakReference {
  public:
-
   JniSessionFactory(const std::shared_ptr<core::ProcessSessionFactory> &factory, const std::shared_ptr<JavaServicer> &servicer, jobject java_object)
       : servicer_(servicer),
         factory_(factory),
@@ -360,5 +341,3 @@ class JniSessionFactory : public core::WeakReference {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* EXTENSIONS_JNI_JVM_REFERNCEOBJECTS_H_ */
