@@ -15,18 +15,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_NARCLASSLOADER_H
-#define EXTENSIONS_NARCLASSLOADER_H
+#pragma once
+
+#include <jni.h>
 
-#include "JavaServicer.h"
-#include "JniBundle.h"
-#include "../JavaException.h"
 #include <string>
 #include <vector>
 #include <sstream>
 #include <iterator>
+#include <memory>
+#include <utility>
+#include <map>
 #include <algorithm>
-#include <jni.h>
+
+#include "JavaServicer.h"
+#include "JniBundle.h"
+#include "../JavaException.h"
+#include "agent/agent_docs.h"
 
 namespace org {
 namespace apache {
@@ -36,7 +41,6 @@ namespace jni {
 
 class NarClassLoader {
  public:
-
   NarClassLoader(std::shared_ptr<minifi::jni::JavaServicer> servicer, JavaClass &clazz, const std::string &dir_name, const std::string &scratch_nar_dir, const std::string &docs_dir)
       : java_servicer_(servicer) {
     class_ref_ = clazz;
@@ -196,7 +200,6 @@ class NarClassLoader {
   }
 
  private:
-
   /**
    * Call empty constructor
    */
@@ -226,7 +229,7 @@ class NarClassLoader {
           lastOfIdx++;  // if a value is found, increment to move beyond the .
           int nameLength = cd.class_name_.length() - lastOfIdx;
           const auto processorName = cd.class_name_.substr(lastOfIdx, nameLength);
-          if (core::ClassLoader::getDefaultClassLoader().getGroupForClass(processorName).empty()) {
+          if (!core::ClassLoader::getDefaultClassLoader().getGroupForClass(processorName)) {
             minifi::ExternalBuildDescription::addExternalComponent(bundle.getDetails(), cd);
           }
         }
@@ -458,7 +461,6 @@ class NarClassLoader {
     return details;
   }
 
-  std::shared_ptr<logging::Logger> logger_;
   std::shared_ptr<minifi::jni::JavaServicer> java_servicer_;
   JavaClass class_ref_;
   jobject class_loader_;
@@ -469,5 +471,3 @@ class NarClassLoader {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* EXTENSIONS_NARCLASSLOADER_H */
