@@ -23,14 +23,14 @@
 #include <string>
 #include <vector>
 #include <set>
-#include <queue>
+#include <unordered_set>
 #include <map>
 #include <deque>
 #include <utility>
-#include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -118,13 +118,11 @@ void BinFiles::onSchedule(core::ProcessContext *context, core::ProcessSessionFac
   if (context->getProperty(MaxBinCount.getName(), maxBinCount_)) {
     logger_->log_debug("BinFiles: MaxBinCount [%" PRIu32 "]", maxBinCount_);
   }
-  std::string maxBinAgeStr;
-  if (context->getProperty(MaxBinAge.getName(), maxBinAgeStr)) {
-    core::TimeUnit unit;
-    if (core::Property::StringToTime(maxBinAgeStr, val64, unit) && core::Property::ConvertTimeUnitToMS(val64, unit, val64)) {
-      this->binManager_.setBinAge(val64);
-      logger_->log_debug("BinFiles: MaxBinAge [%" PRIu64 "]", val64);
-    }
+  if (auto max_bin_age = context->getProperty<core::TimePeriodValue>(MaxBinAge)) {
+    // We need to trigger the processor even when there are no incoming flow files so that it can flush the bins.
+    setTriggerWhenEmpty(true);
+    this->binManager_.setBinAge(max_bin_age->getMilliseconds());
+    logger_->log_debug("BinFiles: MaxBinAge [%" PRId64 "] ms", int64_t{max_bin_age->getMilliseconds().count()});
   }
   if (context->getProperty(BatchSize.getName(), batchSize_)) {
     logger_->log_debug("BinFiles: BatchSize [%" PRIu32 "]", batchSize_);
@@ -152,7 +150,7 @@ void BinManager::gatherReadyBins() {
     std::unique_ptr < std::deque<std::unique_ptr<Bin>>>&queue = it->second;
     while (!queue->empty()) {
       std::unique_ptr<Bin> &bin = queue->front();
-      if (bin->isReadyForMerge() || (binAge_ != ULLONG_MAX && bin->isOlderThan(binAge_))) {
+      if (bin->isReadyForMerge() || (binAge_ != std::chrono::milliseconds::max() && bin->isOlderThan(binAge_))) {
         readyBin_.push_back(std::move(bin));
         queue->pop_front();
         binCount_--;
@@ -174,19 +172,19 @@ void BinManager::gatherReadyBins() {
 
 void BinManager::removeOldestBin() {
   std::lock_guard < std::mutex > lock(mutex_);
-  uint64_t olddate = ULLONG_MAX;
+  std::chrono::system_clock::time_point olddate = std::chrono::system_clock::time_point::max();
   std::unique_ptr < std::deque<std::unique_ptr<Bin>>>* oldqueue;
   for (std::map<std::string, std::unique_ptr<std::deque<std::unique_ptr<Bin>>>>::iterator it=groupBinMap_.begin(); it !=groupBinMap_.end(); ++it) {
     std::unique_ptr < std::deque<std::unique_ptr<Bin>>>&queue = it->second;
     if (!queue->empty()) {
       std::unique_ptr<Bin> &bin = queue->front();
-      if (bin->getBinAge() < olddate) {
-        olddate = bin->getBinAge();
+      if (bin->getCreationDate() < olddate) {
+        olddate = bin->getCreationDate();
         oldqueue = &queue;
       }
     }
   }
-  if (olddate != ULLONG_MAX) {
+  if (olddate != std::chrono::system_clock::time_point::max()) {
     std::unique_ptr<Bin> &remove = (*oldqueue)->front();
     std::string group = remove->getGroupId();
     readyBin_.push_back(std::move(remove));
@@ -350,31 +348,16 @@ void BinFiles::restore(const std::shared_ptr<core::FlowFile>& flowFile) {
   file_store_.put(flowFile);
 }
 
-void BinFiles::FlowFileStore::put(const std::shared_ptr<core::FlowFile>& flowFile) {
-  {
-    std::lock_guard<std::mutex> guard(flow_file_mutex_);
-    incoming_files_.emplace(std::move(flowFile));
-  }
-  has_new_flow_file_.store(true, std::memory_order_release);
-}
-
-std::unordered_set<std::shared_ptr<core::FlowFile>> BinFiles::FlowFileStore::getNewFlowFiles() {
-  bool hasNewFlowFiles = true;
-  if (!has_new_flow_file_.compare_exchange_strong(hasNewFlowFiles, false, std::memory_order_acquire, std::memory_order_relaxed)) {
-    return {};
-  }
-  std::lock_guard<std::mutex> guard(flow_file_mutex_);
-  return std::move(incoming_files_);
-}
-
-std::set<std::shared_ptr<core::Connectable>> BinFiles::getOutGoingConnections(const std::string &relationship) const {
+std::set<core::Connectable*> BinFiles::getOutGoingConnections(const std::string &relationship) {
   auto result = core::Connectable::getOutGoingConnections(relationship);
   if (relationship == Self.getName()) {
-    result.insert(std::static_pointer_cast<core::Connectable>(std::const_pointer_cast<core::Processor>(shared_from_this())));
+    result.insert(this);
   }
   return result;
 }
 
+REGISTER_RESOURCE(BinFiles, "Bins flow files into buckets based on the number of entries or size of entries");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
