@@ -17,21 +17,24 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __BIN_FILES_H__
-#define __BIN_FILES_H__
+#pragma once
 
 #include <cinttypes>
 #include <limits>
 #include <deque>
+#include <memory>
+#include <unordered_set>
+#include <string>
+#include <set>
 #include <map>
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/gsl.h"
 #include "utils/Id.h"
+#include "utils/Export.h"
+#include "core/FlowFileStore.h"
 
 namespace org {
 namespace apache {
@@ -52,10 +55,9 @@ class Bin {
         maxEntries_(maxEntries),
         minEntries_(minEntries),
         fileCount_(fileCount),
-        groupId_(groupId),
-        logger_(logging::LoggerFactory<Bin>::getLogger()) {
+        groupId_(groupId) {
     queued_data_size_ = 0;
-    creation_dated_ = utils::timeutils::getTimeMillis();
+    creation_dated_ = std::chrono::system_clock::now();
     uuid_ = utils::IdGenerator::getIdGenerator()->generate();
     logger_->log_debug("Bin %s for group %s created", getUUIDStr(), groupId_);
   }
@@ -63,25 +65,18 @@ class Bin {
     logger_->log_debug("Bin %s for group %s destroyed", getUUIDStr(), groupId_);
   }
   // check whether the bin is full
-  bool isFull() {
-    if (queued_data_size_ >= maxSize_ || queue_.size() >= maxEntries_)
-      return true;
-    else
-      return false;
+  [[nodiscard]] bool isFull() const {
+    return queued_data_size_ >= maxSize_ || queue_.size() >= maxEntries_;
   }
   // check whether the bin meet the min required size and entries so that it can be processed for merge
-  bool isReadyForMerge() {
+  [[nodiscard]] bool isReadyForMerge() const {
     return isFull() || (queued_data_size_ >= minSize_ && queue_.size() >= minEntries_);
   }
   // check whether the bin is older than the time specified in msec
-  bool isOlderThan(const uint64_t &duration) {
-    uint64_t currentTime = utils::timeutils::getTimeMillis();
-    if (currentTime > (creation_dated_ + duration))
-      return true;
-    else
-      return false;
+  [[nodiscard]] bool isOlderThan(const std::chrono::milliseconds duration) const {
+    return std::chrono::system_clock::now() > (creation_dated_ + duration);
   }
-  std::deque<std::shared_ptr<core::FlowFile>> & getFlowFile() {
+  std::deque<std::shared_ptr<core::FlowFile>>& getFlowFile() {
     return queue_;
   }
   // offer the flowfile to the bin
@@ -109,23 +104,21 @@ class Bin {
     return true;
   }
   // getBinAge
-  uint64_t getBinAge() {
+  [[nodiscard]] std::chrono::system_clock::time_point getCreationDate() const {
     return creation_dated_;
   }
-  int getSize() {
+  [[nodiscard]] int getSize() const {
     return gsl::narrow<int>(queue_.size());
   }
 
-  utils::SmallString<36> getUUIDStr() {
+  [[nodiscard]] utils::SmallString<36> getUUIDStr() const {
     return uuid_.to_string();
   }
 
-  std::string getGroupId() {
+  [[nodiscard]] std::string getGroupId() const {
     return groupId_;
   }
 
- protected:
-
  private:
   uint64_t minSize_;
   uint64_t maxSize_;
@@ -135,10 +128,10 @@ class Bin {
   uint64_t queued_data_size_;
   // Queue for the Flow File
   std::deque<std::shared_ptr<core::FlowFile>> queue_;
-  uint64_t creation_dated_;
+  std::chrono::system_clock::time_point creation_dated_;
   std::string fileCount_;
   std::string groupId_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<Bin>::getLogger();
   // A global unique identifier
   utils::Identifier uuid_;
 };
@@ -161,10 +154,10 @@ class BinManager {
   void setMinEntries(uint32_t entries) {
     minEntries_ = {entries};
   }
-  void setBinAge(uint64_t age) {
-    binAge_ = {age};
+  void setBinAge(std::chrono::milliseconds age) {
+    binAge_ = age;
   }
-  int getBinCount() {
+  [[nodiscard]] int getBinCount() const {
     return binCount_;
   }
   void setFileCount(const std::string &value) {
@@ -184,8 +177,6 @@ class BinManager {
   // get ready bin from binManager
   void getReadyBin(std::deque<std::unique_ptr<Bin>> &retBins);
 
- protected:
-
  private:
   std::mutex mutex_;
   uint64_t minSize_{0};
@@ -193,47 +184,47 @@ class BinManager {
   uint32_t maxEntries_{std::numeric_limits<decltype(maxEntries_)>::max()};
   uint32_t minEntries_{1};
   std::string fileCount_;
-  // Bin Age in msec
-  uint64_t binAge_{std::numeric_limits<decltype(binAge_)>::max()};
+  std::chrono::milliseconds binAge_{std::chrono::milliseconds::max()};
   std::map<std::string, std::unique_ptr<std::deque<std::unique_ptr<Bin>>> >groupBinMap_;
   std::deque<std::unique_ptr<Bin>> readyBin_;
   int binCount_{0};
-  std::shared_ptr<logging::Logger> logger_{logging::LoggerFactory<BinManager>::getLogger()};
+  std::shared_ptr<core::logging::Logger> logger_{core::logging::LoggerFactory<BinManager>::getLogger()};
 };
 
 // BinFiles Class
 class BinFiles : public core::Processor {
  protected:
   static core::Relationship Self;
+
  public:
   using core::Processor::Processor;
   // Destructor
-  virtual ~BinFiles() = default;
+  ~BinFiles() override = default;
   // Processor Name
   static constexpr char const* ProcessorName = "BinFiles";
   // Supported Properties
-  static core::Property MinSize;
-  static core::Property MaxSize;
-  static core::Property MinEntries;
-  static core::Property MaxEntries;
-  static core::Property MaxBinCount;
-  static core::Property MaxBinAge;
-  static core::Property BatchSize;
+  EXTENSIONAPI static core::Property MinSize;
+  EXTENSIONAPI static core::Property MaxSize;
+  EXTENSIONAPI static core::Property MinEntries;
+  EXTENSIONAPI static core::Property MaxEntries;
+  EXTENSIONAPI static core::Property MaxBinCount;
+  EXTENSIONAPI static core::Property MaxBinAge;
+  EXTENSIONAPI static core::Property BatchSize;
 
   // Supported Relationships
-  static core::Relationship Failure;
-  static core::Relationship Original;
+  EXTENSIONAPI static core::Relationship Failure;
+  EXTENSIONAPI static core::Relationship Original;
 
   // attributes
-  static const char *FRAGMENT_ID_ATTRIBUTE;
-  static const char *FRAGMENT_INDEX_ATTRIBUTE;
-  static const char *FRAGMENT_COUNT_ATTRIBUTE;
+  EXTENSIONAPI static const char *FRAGMENT_ID_ATTRIBUTE;
+  EXTENSIONAPI static const char *FRAGMENT_INDEX_ATTRIBUTE;
+  EXTENSIONAPI static const char *FRAGMENT_COUNT_ATTRIBUTE;
 
-  static const char *SEGMENT_ID_ATTRIBUTE;
-  static const char *SEGMENT_INDEX_ATTRIBUTE;
-  static const char *SEGMENT_COUNT_ATTRIBUTE;
-  static const char *SEGMENT_ORIGINAL_FILENAME;
-  static const char *TAR_PERMISSIONS_ATTRIBUTE;
+  EXTENSIONAPI static const char *SEGMENT_ID_ATTRIBUTE;
+  EXTENSIONAPI static const char *SEGMENT_INDEX_ATTRIBUTE;
+  EXTENSIONAPI static const char *SEGMENT_COUNT_ATTRIBUTE;
+  EXTENSIONAPI static const char *SEGMENT_ORIGINAL_FILENAME;
+  EXTENSIONAPI static const char *TAR_PERMISSIONS_ATTRIBUTE;
 
  public:
   /**
@@ -249,11 +240,11 @@ class BinFiles : public core::Processor {
   // OnTrigger method, implemented by NiFi BinFiles
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
   // Initialize, over write by NiFi BinFiles
-  void initialize(void) override;
+  void initialize() override;
 
   void restore(const std::shared_ptr<core::FlowFile>& flowFile) override;
 
-  std::set<std::shared_ptr<core::Connectable>> getOutGoingConnections(const std::string &relationship) const override;
+  std::set<core::Connectable*> getOutGoingConnections(const std::string &relationship) override;
 
  protected:
   // Allows general pre-processing of a flow file before it is offered to a bin. This is called before getGroupId().
@@ -274,32 +265,15 @@ class BinFiles : public core::Processor {
   BinManager binManager_;
 
  private:
-  class FlowFileStore{
-   public:
-    /**
-     * Returns the already-preprocessed FlowFiles that got restored on restart from the FlowFileRepository
-     * @return the resurrected persisted FlowFiles
-     */
-    std::unordered_set<std::shared_ptr<core::FlowFile>> getNewFlowFiles();
-    void put(const std::shared_ptr<core::FlowFile>& flowFile);
-   private:
-    std::atomic_bool has_new_flow_file_{false};
-    std::mutex flow_file_mutex_;
-    std::unordered_set<std::shared_ptr<core::FlowFile>> incoming_files_;
-  };
-
-  std::shared_ptr<logging::Logger> logger_{logging::LoggerFactory<BinFiles>::getLogger()};
+  std::shared_ptr<core::logging::Logger> logger_{core::logging::LoggerFactory<BinFiles>::getLogger()};
   uint32_t batchSize_{1};
   uint32_t maxBinCount_{100};
-  FlowFileStore file_store_;
+  core::FlowFileStore file_store_;
 };
 
-REGISTER_RESOURCE(BinFiles, "Bins flow files into buckets based on the number of entries or size of entries");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
 
-#endif
