@@ -19,15 +19,15 @@
  */
 #include "CompressContent.h"
 #include <stdio.h>
-#include <algorithm>
 #include <memory>
 #include <string>
 #include <map>
 #include <set>
-#include "utils/TimeUtil.h"
-#include "utils/StringUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "utils/StringUtils.h"
+#include "core/Resource.h"
+#include "io/StreamPipe.h"
 
 namespace org {
 namespace apache {
@@ -65,19 +65,21 @@ core::Property CompressContent::BatchSize(
 core::Relationship CompressContent::Success("success", "FlowFiles will be transferred to the success relationship after successfully being compressed or decompressed");
 core::Relationship CompressContent::Failure("failure", "FlowFiles will be transferred to the failure relationship if they fail to compress/decompress");
 
-const std::map<std::string, CompressContent::CompressionFormat> CompressContent::compressionFormatMimeTypeMap_{
-  {"application/gzip", CompressionFormat::GZIP},
-  {"application/bzip2", CompressionFormat::BZIP2},
-  {"application/x-bzip2", CompressionFormat::BZIP2},
-  {"application/x-lzma", CompressionFormat::LZMA},
-  {"application/x-xz", CompressionFormat::XZ_LZMA2}
+const std::string CompressContent::TAR_EXT = ".tar";
+
+const std::map<std::string, io::CompressionFormat> CompressContent::compressionFormatMimeTypeMap_{
+  {"application/gzip", io::CompressionFormat::GZIP},
+  {"application/bzip2", io::CompressionFormat::BZIP2},
+  {"application/x-bzip2", io::CompressionFormat::BZIP2},
+  {"application/x-lzma", io::CompressionFormat::LZMA},
+  {"application/x-xz", io::CompressionFormat::XZ_LZMA2}
 };
 
-const std::map<CompressContent::CompressionFormat, std::string> CompressContent::fileExtension_{
-  {CompressionFormat::GZIP, ".gz"},
-  {CompressionFormat::LZMA, ".lzma"},
-  {CompressionFormat::BZIP2, ".bz2"},
-  {CompressionFormat::XZ_LZMA2, ".xz"}
+const std::map<io::CompressionFormat, std::string> CompressContent::fileExtension_{
+  {io::CompressionFormat::GZIP, ".gz"},
+  {io::CompressionFormat::LZMA, ".lzma"},
+  {io::CompressionFormat::BZIP2, ".bz2"},
+  {io::CompressionFormat::XZ_LZMA2, ".xz"}
 };
 
 void CompressContent::initialize() {
@@ -128,7 +130,7 @@ void CompressContent::onTrigger(const std::shared_ptr<core::ProcessContext> &con
 void CompressContent::processFlowFile(const std::shared_ptr<core::FlowFile>& flowFile, const std::shared_ptr<core::ProcessSession>& session) {
   session->remove(flowFile);
 
-  CompressionFormat compressFormat;
+  io::CompressionFormat compressFormat;
   if (compressFormat_ == ExtendedCompressionFormat::USE_MIME_TYPE) {
     std::string attr;
     flowFile->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, attr);
@@ -146,22 +148,22 @@ void CompressContent::processFlowFile(const std::shared_ptr<core::FlowFile>& flo
       return;
     }
   } else {
-    compressFormat = compressFormat_.cast<CompressionFormat>();
+    compressFormat = compressFormat_.cast<io::CompressionFormat>();
   }
   std::string mimeType = toMimeType(compressFormat);
 
   // Validate
-  if (!encapsulateInTar_ && compressFormat != CompressionFormat::GZIP) {
+  if (!encapsulateInTar_ && compressFormat != io::CompressionFormat::GZIP) {
     logger_->log_error("non-TAR encapsulated format only supports GZIP compression");
     session->transfer(flowFile, Failure);
     return;
   }
-  if (compressFormat == CompressionFormat::BZIP2 && archive_bzlib_version() == nullptr) {
+  if (compressFormat == io::CompressionFormat::BZIP2 && archive_bzlib_version() == nullptr) {
     logger_->log_error("%s compression format is requested, but the agent was compiled without BZip2 support", compressFormat.toString());
     session->transfer(flowFile, Failure);
     return;
   }
-  if ((compressFormat == CompressionFormat::LZMA || compressFormat == CompressionFormat::XZ_LZMA2) && archive_liblzma_version() == nullptr) {
+  if ((compressFormat == io::CompressionFormat::LZMA || compressFormat == io::CompressionFormat::XZ_LZMA2) && archive_liblzma_version() == nullptr) {
     logger_->log_error("%s compression format is requested, but the agent was compiled without LZMA support ", compressFormat.toString());
     session->transfer(flowFile, Failure);
     return;
@@ -173,14 +175,43 @@ void CompressContent::processFlowFile(const std::shared_ptr<core::FlowFile>& flo
     fileExtension = search->second;
   }
   std::shared_ptr<core::FlowFile> result = session->create(flowFile);
-  bool success = false;
+  bool success = true;
   if (encapsulateInTar_) {
-    CompressContent::WriteCallback callback(compressMode_, compressLevel_, compressFormat, flowFile, session);
-    session->write(result, &callback);
-    success = callback.status_ >= 0;
+    std::function<int64_t(const std::shared_ptr<io::InputStream>&, const std::shared_ptr<io::OutputStream>&)> transformer;
+
+    if (compressMode_ == CompressionMode::Compress) {
+      std::string filename;
+      flowFile->getAttribute(core::SpecialFlowAttribute::FILENAME, filename);
+      transformer = [&, filename] (const std::shared_ptr<io::InputStream>& in, const std::shared_ptr<io::OutputStream>& out) -> int64_t {
+        io::WriteArchiveStreamImpl compressor(compressLevel_, compressFormat, out);
+        if (!compressor.newEntry({filename, in->size()})) {
+          return -1;
+        }
+        return internal::pipe(*in, compressor);
+      };
+    } else {
+      transformer = [&] (const std::shared_ptr<io::InputStream>& in, const std::shared_ptr<io::OutputStream>& out) -> int64_t {
+        io::ReadArchiveStreamImpl decompressor(in);
+        if (!decompressor.nextEntry()) {
+          success = false;
+          return 0;  // prevents a session rollback
+        }
+        auto ret = internal::pipe(decompressor, *out);
+        if (ret < 0) {
+          success = false;
+          return 0;  // prevents a session rollback
+        }
+        return ret;
+      };
+    }
+    session->write(result, [&] (const auto& out) {
+      return session->read(flowFile, [&] (const auto& in) {
+        return transformer(in, out);
+      });
+    });
   } else {
     CompressContent::GzipWriteCallback callback(compressMode_, compressLevel_, flowFile, session);
-    session->write(result, &callback);
+    session->write(result, std::ref(callback));
     success = callback.success_;
   }
 
@@ -194,14 +225,20 @@ void CompressContent::processFlowFile(const std::shared_ptr<core::FlowFile>& flo
     if (compressMode_ == CompressionMode::Compress) {
       session->putAttribute(result, core::SpecialFlowAttribute::MIME_TYPE, mimeType);
       if (updateFileName_) {
+        if (encapsulateInTar_) {
+          fileName = fileName + TAR_EXT;
+        }
         fileName = fileName + fileExtension;
         session->putAttribute(result, core::SpecialFlowAttribute::FILENAME, fileName);
       }
     } else {
       session->removeAttribute(result, core::SpecialFlowAttribute::MIME_TYPE);
       if (updateFileName_) {
-        if (fileName.size() >= fileExtension.size() && fileName.compare(fileName.size() - fileExtension.size(), fileExtension.size(), fileExtension) == 0) {
+        if (utils::StringUtils::endsWith(fileName, fileExtension)) {
           fileName = fileName.substr(0, fileName.size() - fileExtension.size());
+          if (encapsulateInTar_ && utils::StringUtils::endsWith(fileName, TAR_EXT)) {
+            fileName = fileName.substr(0, fileName.size() - TAR_EXT.size());
+          }
           session->putAttribute(result, core::SpecialFlowAttribute::FILENAME, fileName);
         }
       }
@@ -211,16 +248,18 @@ void CompressContent::processFlowFile(const std::shared_ptr<core::FlowFile>& flo
   }
 }
 
-std::string CompressContent::toMimeType(CompressionFormat format) {
+std::string CompressContent::toMimeType(io::CompressionFormat format) {
   switch (format.value()) {
-    case CompressionFormat::GZIP: return "application/gzip";
-    case CompressionFormat::BZIP2: return "application/bzip2";
-    case CompressionFormat::LZMA: return "application/x-lzma";
-    case CompressionFormat::XZ_LZMA2: return "application/x-xz";
+    case io::CompressionFormat::GZIP: return "application/gzip";
+    case io::CompressionFormat::BZIP2: return "application/bzip2";
+    case io::CompressionFormat::LZMA: return "application/x-lzma";
+    case io::CompressionFormat::XZ_LZMA2: return "application/x-xz";
   }
   throw Exception(GENERAL_EXCEPTION, "Invalid compression format");
 }
 
+REGISTER_RESOURCE(CompressContent, "Compresses or decompresses the contents of FlowFiles using a user-specified compression algorithm and updates the mime.type attribute as appropriate");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
