@@ -24,16 +24,17 @@
 
 #include <string.h>
 
+#include <array>
 #include <string>
 #include <set>
 
-#include <iostream>
-#include <fstream>
 #include <memory>
 
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "Exception.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -64,15 +65,14 @@ void FocusArchiveEntry::onTrigger(core::ProcessContext *context, core::ProcessSe
     return;
   }
 
-  fileutils::FileManager file_man;
+  utils::file::FileManager file_man;
 
   // Extract archive contents
   ArchiveMetadata archiveMetadata;
   context->getProperty(Path.getName(), archiveMetadata.focusedEntry);
   flowFile->getAttribute("filename", archiveMetadata.archiveName);
 
-  ReadCallback cb(this, &file_man, &archiveMetadata);
-  session->read(flowFile, &cb);
+  session->read(flowFile, ReadCallback{this, &file_man, &archiveMetadata});
 
   // For each extracted entry, import & stash to key
   std::string targetEntryStashKey;
@@ -122,9 +122,7 @@ void FocusArchiveEntry::onTrigger(core::ProcessContext *context, core::ProcessSe
 
     archiveStack.push(archiveMetadata);
 
-    std::string stackStr = archiveStack.toJsonString();
-  
-    flowFile->setAttribute("lens.archive.stack", stackStr);
+    flowFile->setAttribute("lens.archive.stack", archiveStack.toJsonString());
   }
 
   // Update filename attribute to that of focused entry
@@ -142,30 +140,30 @@ void FocusArchiveEntry::onTrigger(core::ProcessContext *context, core::ProcessSe
 typedef struct {
   std::shared_ptr<io::BaseStream> stream;
   core::Processor *processor;
-  char buf[8196];
+  std::array<std::byte, 8196> buf;
 } FocusArchiveEntryReadData;
 
 // Read callback which reads from the flowfile stream
 la_ssize_t FocusArchiveEntry::ReadCallback::read_cb(struct archive * a, void *d, const void **buf) {
   auto data = static_cast<FocusArchiveEntryReadData *>(d);
-  *buf = data->buf;
-  int read = 0;
-  int last_read = 0;
+  *buf = data->buf.data();
+  size_t read = 0;
+  size_t last_read = 0;
 
   do {
-    last_read = data->stream->read(reinterpret_cast<uint8_t *>(data->buf), 8196 - read);
+    last_read = data->stream->read(data->buf);
     read += last_read;
-  } while (data->processor->isRunning() && last_read > 0 && read < 8196);
+  } while (data->processor->isRunning() && last_read > 0 && !io::isError(last_read) && read < 8196);
 
   if (!data->processor->isRunning()) {
     archive_set_error(a, EINTR, "Processor shut down during read");
     return -1;
   }
 
-  return read;
+  return gsl::narrow<la_ssize_t>(read);
 }
 
-int64_t FocusArchiveEntry::ReadCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
+int64_t FocusArchiveEntry::ReadCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) const {
   auto inputArchive = archive_read_new();
   struct archive_entry *entry;
   int64_t nlen = 0;
@@ -185,7 +183,7 @@ int64_t FocusArchiveEntry::ReadCallback::process(const std::shared_ptr<io::BaseS
     return nlen;
   }
 
-  while (isRunning()) {
+  while (proc_->isRunning()) {
     res = archive_read_next_header(inputArchive, &entry);
 
     if (res == ARCHIVE_EOF) {
@@ -250,14 +248,15 @@ int64_t FocusArchiveEntry::ReadCallback::process(const std::shared_ptr<io::BaseS
   return nlen;
 }
 
-FocusArchiveEntry::ReadCallback::ReadCallback(core::Processor *processor, fileutils::FileManager *file_man, ArchiveMetadata *archiveMetadata)
+FocusArchiveEntry::ReadCallback::ReadCallback(core::Processor *processor, utils::file::FileManager *file_man, ArchiveMetadata *archiveMetadata)
     : file_man_(file_man),
       proc_(processor) {
-  logger_ = logging::LoggerFactory<FocusArchiveEntry>::getLogger();
   _archiveMetadata = archiveMetadata;
 }
 
-FocusArchiveEntry::ReadCallback::~ReadCallback() = default;
+REGISTER_RESOURCE(FocusArchiveEntry, "Allows manipulation of entries within an archive (e.g. TAR) by focusing on one entry within the archive at a time. "
+    "When an archive entry is focused, that entry is treated as the content of the FlowFile and may be manipulated independently of the rest of the archive."
+    " To restore the FlowFile to its original state, use UnfocusArchiveEntry.");
 
 } /* namespace processors */
 } /* namespace minifi */
