@@ -17,14 +17,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_PROCESSORS_FOCUSARCHIVEENTRY_H_
-#define LIBMINIFI_INCLUDE_PROCESSORS_FOCUSARCHIVEENTRY_H_
+#pragma once
 
-#include <list>
 #include <memory>
 #include <string>
 
-#include <archive.h>
+#include "archive.h"
 
 #include "ArchiveMetadata.h"
 #include "FlowFileRecord.h"
@@ -32,8 +30,8 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "core/Resource.h"
 #include "utils/file/FileManager.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -49,35 +47,33 @@ class FocusArchiveEntry : public core::Processor {
    * Create a new processor
    */
   explicit FocusArchiveEntry(const std::string& name, const utils::Identifier& uuid = {})
-  : core::Processor(name, uuid),
-    logger_(logging::LoggerFactory<FocusArchiveEntry>::getLogger()) {
+  : core::Processor(name, uuid) {
   }
   //! Destructor
-  virtual ~FocusArchiveEntry()   = default;
+  ~FocusArchiveEntry()   override = default;
   //! Processor Name
-  static constexpr char const* ProcessorName = "FocusArchiveEntry";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "FocusArchiveEntry";
   //! Supported Properties
-  static core::Property Path;
+  EXTENSIONAPI static core::Property Path;
   //! Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
 
   //! OnTrigger method, implemented by NiFi FocusArchiveEntry
-  virtual void onTrigger(core::ProcessContext *context,
-      core::ProcessSession *session);
+  void onTrigger(core::ProcessContext *context,
+      core::ProcessSession *session) override;
   //! Initialize, over write by NiFi FocusArchiveEntry
-  virtual void initialize(void);
+  void initialize() override;
 
-  class ReadCallback : public InputStreamCallback {
+  class ReadCallback {
    public:
-    explicit ReadCallback(core::Processor*, fileutils::FileManager *file_man, ArchiveMetadata *archiveMetadata);
-    ~ReadCallback();
-    virtual int64_t process(const std::shared_ptr<io::BaseStream>& stream);
+    explicit ReadCallback(core::Processor*, utils::file::FileManager *file_man, ArchiveMetadata *archiveMetadata);
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) const;
     bool isRunning() {return proc_->isRunning();}
 
    private:
-    fileutils::FileManager *file_man_;
+    utils::file::FileManager *file_man_;
     core::Processor * const proc_;
-    std::shared_ptr<logging::Logger> logger_;
+    std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<FocusArchiveEntry>::getLogger();
     ArchiveMetadata *_archiveMetadata;
     static int ok_cb(struct archive *, void* /*d*/) { return ARCHIVE_OK; }
     static la_ssize_t read_cb(struct archive * a, void *d, const void **buf);
@@ -85,18 +81,12 @@ class FocusArchiveEntry : public core::Processor {
 
  private:
   //! Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<FocusArchiveEntry>::getLogger();
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 };
 
-REGISTER_RESOURCE(FocusArchiveEntry, "Allows manipulation of entries within an archive (e.g. TAR) by focusing on one entry within the archive at a time. "
-    "When an archive entry is focused, that entry is treated as the content of the FlowFile and may be manipulated independently of the rest of the archive."
-    " To restore the FlowFile to its original state, use UnfocusArchiveEntry.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // LIBMINIFI_INCLUDE_PROCESSORS_FOCUSARCHIVEENTRY_H_
