@@ -17,23 +17,25 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include <string.h>
 #include <iostream>
-#include <fstream>
 #include <memory>
 #include <string>
 #include <set>
 #include <list>
+#include <algorithm>
 
-#include <archive.h>
-#include <archive_entry.h>
+#include "archive.h"
+#include "archive_entry.h"
 
 #include "ManipulateArchive.h"
 #include "Exception.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
 #include "core/FlowFile.h"
+#include "core/Resource.h"
 #include "utils/file/FileManager.h"
+#include "FocusArchiveEntry.h"
+#include "UnfocusArchiveEntry.h"
 
 namespace org {
 namespace apache {
@@ -107,7 +109,7 @@ void ManipulateArchive::onSchedule(core::ProcessContext *context, core::ProcessS
     if (before_.size() && after_.size()) {
         logger_->log_error("ManipulateArchive: cannot specify both before and after.");
         invalid = true;
-    }   
+    }
 
     if (invalid) {
         throw Exception(GENERAL_EXCEPTION, "Invalid ManipulateArchive configuration");
@@ -122,10 +124,9 @@ void ManipulateArchive::onTrigger(core::ProcessContext* /*context*/, core::Proce
     }
 
     ArchiveMetadata archiveMetadata;
-    fileutils::FileManager file_man;
+    utils::file::FileManager file_man;
 
-    FocusArchiveEntry::ReadCallback readCallback(this, &file_man, &archiveMetadata);
-    session->read(flowFile, &readCallback);
+    session->read(flowFile, FocusArchiveEntry::ReadCallback{this, &file_man, &archiveMetadata});
 
     auto entries_end = archiveMetadata.entryMetadata.end();
 
@@ -208,12 +209,12 @@ void ManipulateArchive::onTrigger(core::ProcessContext* /*context*/, core::Proce
         archiveMetadata.entryMetadata.insert(position, touchEntry);
     }
 
-    UnfocusArchiveEntry::WriteCallback writeCallback(&archiveMetadata);
-    session->write(flowFile, &writeCallback);
-
+    session->write(flowFile, UnfocusArchiveEntry::WriteCallback{&archiveMetadata});
     session->transfer(flowFile, Success);
 }
 
+REGISTER_RESOURCE(ManipulateArchive, "Performs an operation which manipulates an archive without needing to split the archive into multiple FlowFiles.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
