@@ -17,19 +17,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_PROCESSORS_MANIPULATEARCHIVE_H_
-#define LIBMINIFI_INCLUDE_PROCESSORS_MANIPULATEARCHIVE_H_
+#pragma once
 
-#include <list>
 #include <string>
+#include <memory>
 
 #include "FlowFileRecord.h"
 #include "ArchiveMetadata.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 
-#include "FocusArchiveEntry.h"
-#include "UnfocusArchiveEntry.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -37,53 +35,46 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-using logging::Logger;
+using core::logging::Logger;
 
 class ManipulateArchive : public core::Processor {
-public:
-  ManipulateArchive(const std::string& name, const utils::Identifier& uuid = {})
-  : core::Processor(name, uuid),
-    logger_(logging::LoggerFactory<ManipulateArchive>::getLogger()) {
+ public:
+  explicit ManipulateArchive(const std::string& name, const utils::Identifier& uuid = {})
+      : core::Processor(name, uuid) {
   }
-  virtual ~ManipulateArchive() = default;
-  static constexpr char const* ProcessorName = "ManipulateArchive";
+  ~ManipulateArchive() override = default;
+  EXTENSIONAPI static constexpr char const* ProcessorName = "ManipulateArchive";
 
   // Supported operations
-  static char const* OPERATION_REMOVE;
-  static char const* OPERATION_COPY;
-  static char const* OPERATION_MOVE;
-  static char const* OPERATION_TOUCH;
+  EXTENSIONAPI static char const* OPERATION_REMOVE;
+  EXTENSIONAPI static char const* OPERATION_COPY;
+  EXTENSIONAPI static char const* OPERATION_MOVE;
+  EXTENSIONAPI static char const* OPERATION_TOUCH;
 
   // Supported Properties
-  static core::Property Operation;
-  static core::Property Target;
-  static core::Property Destination;
-  static core::Property Before;
-  static core::Property After;
+  EXTENSIONAPI static core::Property Operation;
+  EXTENSIONAPI static core::Property Target;
+  EXTENSIONAPI static core::Property Destination;
+  EXTENSIONAPI static core::Property Before;
+  EXTENSIONAPI static core::Property After;
   // Supported Relationships
-  static core::Relationship Success;
-  static core::Relationship Failure;
+  EXTENSIONAPI static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Failure;
 
   // OnTrigger method, implemented by NiFi ManipulateArchive
-  void onTrigger(core::ProcessContext *context, core::ProcessSession *session);
-  void onSchedule(core::ProcessContext *context, core::ProcessSessionFactory *sessionFactory);
+  void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
+  void onSchedule(core::ProcessContext *context, core::ProcessSessionFactory *sessionFactory) override;
   // Initialize, over write by NiFi ManipulateArchive
-  void initialize(void);
+  void initialize() override;
 
-protected:
-
-private:
+ private:
   // Logger
-  std::shared_ptr<Logger> logger_;
+  std::shared_ptr<Logger> logger_ = core::logging::LoggerFactory<ManipulateArchive>::getLogger();
   std::string before_, after_, operation_, destination_, targetEntry_;
 };
 
-REGISTER_RESOURCE(ManipulateArchive, "Performs an operation which manipulates an archive without needing to split the archive into multiple FlowFiles.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // LIBMINIFI_INCLUDE_PROCESSORS_MANIPULATEARCHIVE_H_
