@@ -21,19 +21,16 @@
 #include <stdio.h>
 #include <memory>
 #include <string>
-#include <vector>
 #include <set>
-#include <queue>
 #include <map>
 #include <deque>
 #include <utility>
 #include <algorithm>
 #include <numeric>
 #include "utils/TimeUtil.h"
-#include "utils/StringUtils.h"
-#include "utils/GeneralUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "serialization/PayloadSerializer.h"
 #include "serialization/FlowFileV3Serializer.h"
 
@@ -289,35 +286,35 @@ bool MergeContent::processBin(core::ProcessContext *context, core::ProcessSessio
   }
 
   std::shared_ptr<core::FlowFile> merge_flow = std::static_pointer_cast<FlowFileRecord>(session->create());
-  if (attributeStrategy_ == merge_content_options::ATTRIBUTE_STRATEGY_KEEP_COMMON)
+  if (attributeStrategy_ == merge_content_options::ATTRIBUTE_STRATEGY_KEEP_COMMON) {
     KeepOnlyCommonAttributesMerger(bin->getFlowFile()).mergeAttributes(session, merge_flow);
-  else if (attributeStrategy_ == merge_content_options::ATTRIBUTE_STRATEGY_KEEP_ALL_UNIQUE)
+  } else if (attributeStrategy_ == merge_content_options::ATTRIBUTE_STRATEGY_KEEP_ALL_UNIQUE) {
     KeepAllUniqueAttributesMerger(bin->getFlowFile()).mergeAttributes(session, merge_flow);
-  else {
+  } else {
     logger_->log_error("Attribute strategy not supported %s", attributeStrategy_);
     return false;
   }
 
-  auto flowFileReader = [&] (const std::shared_ptr<core::FlowFile>& ff, InputStreamCallback* cb) {
+  auto flowFileReader = [&] (const std::shared_ptr<core::FlowFile>& ff, const io::InputStreamCallback& cb) {
     return session->read(ff, cb);
   };
 
   const char* mimeType;
   std::unique_ptr<MergeBin> mergeBin;
-  std::unique_ptr<minifi::FlowFileSerializer> serializer = utils::make_unique<PayloadSerializer>(flowFileReader);
+  std::unique_ptr<minifi::FlowFileSerializer> serializer = std::make_unique<PayloadSerializer>(flowFileReader);
   if (mergeFormat_ == merge_content_options::MERGE_FORMAT_CONCAT_VALUE) {
-    mergeBin = utils::make_unique<BinaryConcatenationMerge>(headerContent_, footerContent_, demarcatorContent_);
+    mergeBin = std::make_unique<BinaryConcatenationMerge>(headerContent_, footerContent_, demarcatorContent_);
     mimeType = "application/octet-stream";
   } else if (mergeFormat_ == merge_content_options::MERGE_FORMAT_FLOWFILE_STREAM_V3_VALUE) {
     // disregard header, demarcator, footer
-    mergeBin = utils::make_unique<BinaryConcatenationMerge>("", "", "");
-    serializer = utils::make_unique<FlowFileV3Serializer>(flowFileReader);
+    mergeBin = std::make_unique<BinaryConcatenationMerge>("", "", "");
+    serializer = std::make_unique<FlowFileV3Serializer>(flowFileReader);
     mimeType = "application/flowfile-v3";
   } else if (mergeFormat_ == merge_content_options::MERGE_FORMAT_TAR_VALUE) {
-    mergeBin = utils::make_unique<TarMerge>();
+    mergeBin = std::make_unique<TarMerge>();
     mimeType = "application/tar";
   } else if (mergeFormat_ == merge_content_options::MERGE_FORMAT_ZIP_VALUE) {
-    mergeBin = utils::make_unique<ZipMerge>();
+    mergeBin = std::make_unique<ZipMerge>();
     mimeType = "application/zip";
   } else {
     logger_->log_error("Merge format not supported %s", mergeFormat_);
@@ -352,8 +349,7 @@ BinaryConcatenationMerge::BinaryConcatenationMerge(const std::string &header, co
 
 void BinaryConcatenationMerge::merge(core::ProcessContext* /*context*/, core::ProcessSession *session,
     std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer, const std::shared_ptr<core::FlowFile>& merge_flow) {
-  BinaryConcatenationMerge::WriteCallback callback(header_, footer_, demarcator_, flows, serializer);
-  session->write(merge_flow, &callback);
+  session->write(merge_flow, BinaryConcatenationMerge::WriteCallback{header_, footer_, demarcator_, flows, serializer});
   std::string fileName;
   if (flows.size() == 1) {
     flows.front()->getAttribute(core::SpecialFlowAttribute::FILENAME, fileName);
@@ -366,8 +362,7 @@ void BinaryConcatenationMerge::merge(core::ProcessContext* /*context*/, core::Pr
 
 void TarMerge::merge(core::ProcessContext* /*context*/, core::ProcessSession *session,
     std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer, const std::shared_ptr<core::FlowFile>& merge_flow) {
-  ArchiveMerge::WriteCallback callback(std::string(merge_content_options::MERGE_FORMAT_TAR_VALUE), flows, serializer);
-  session->write(merge_flow, &callback);
+  session->write(merge_flow, ArchiveMerge::WriteCallback{merge_content_options::MERGE_FORMAT_TAR_VALUE, flows, serializer});
   std::string fileName;
   merge_flow->getAttribute(core::SpecialFlowAttribute::FILENAME, fileName);
   if (flows.size() == 1) {
@@ -383,8 +378,7 @@ void TarMerge::merge(core::ProcessContext* /*context*/, core::ProcessSession *se
 
 void ZipMerge::merge(core::ProcessContext* /*context*/, core::ProcessSession *session,
     std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer, const std::shared_ptr<core::FlowFile>& merge_flow) {
-  ArchiveMerge::WriteCallback callback(std::string(merge_content_options::MERGE_FORMAT_ZIP_VALUE), flows, serializer);
-  session->write(merge_flow, &callback);
+  session->write(merge_flow, ArchiveMerge::WriteCallback{merge_content_options::MERGE_FORMAT_ZIP_VALUE, flows, serializer});
   std::string fileName;
   merge_flow->getAttribute(core::SpecialFlowAttribute::FILENAME, fileName);
   if (flows.size() == 1) {
@@ -438,6 +432,10 @@ void KeepAllUniqueAttributesMerger::processFlowFile(const std::shared_ptr<core::
   }
 }
 
+REGISTER_RESOURCE(MergeContent, "Merges a Group of FlowFiles together based on a user-defined strategy and packages them into a single FlowFile. "
+    "MergeContent should be configured with only one incoming connection as it won't create grouped Flow Files."
+    "This processor updates the mime.type attribute as appropriate.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
