@@ -17,8 +17,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __MERGE_CONTENT_H__
-#define __MERGE_CONTENT_H__
+#pragma once
+
+#include <deque>
+#include <map>
+#include <utility>
+#include <vector>
+#include <memory>
+#include <string>
 
 #include "ArchiveCommon.h"
 #include "BinFiles.h"
@@ -27,6 +33,7 @@
 #include "core/logging/LoggerConfiguration.h"
 #include "serialization/FlowFileSerializer.h"
 #include "utils/gsl.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -64,48 +71,50 @@ class BinaryConcatenationMerge : public MergeBin {
   BinaryConcatenationMerge(const std::string& header, const std::string& footer, const std::string& demarcator);
 
   void merge(core::ProcessContext *context, core::ProcessSession *session,
-      std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer, const std::shared_ptr<core::FlowFile> &flowFile);
+      std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer, const std::shared_ptr<core::FlowFile> &flowFile) override;
   // Nest Callback Class for write stream
-  class WriteCallback: public OutputStreamCallback {
+  class WriteCallback {
    public:
     WriteCallback(std::string &header, std::string &footer, std::string &demarcator,
         std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer) :
       header_(header), footer_(footer), demarcator_(demarcator), flows_(flows), serializer_(serializer) {
     }
+
     std::string &header_;
     std::string &footer_;
     std::string &demarcator_;
     std::deque<std::shared_ptr<core::FlowFile>> &flows_;
     FlowFileSerializer& serializer_;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      int64_t ret = 0;
+
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) const {
+      size_t write_size_sum = 0;
       if (!header_.empty()) {
-        int64_t len = stream->write(reinterpret_cast<uint8_t*>(const_cast<char*>(header_.data())), gsl::narrow<int>(header_.size()));
-        if (len < 0)
-          return len;
-        ret += len;
+        const auto write_ret = stream->write(reinterpret_cast<const uint8_t*>(header_.data()), header_.size());
+        if (io::isError(write_ret))
+          return -1;
+        write_size_sum += write_ret;
       }
       bool isFirst = true;
-      for (auto flow : flows_) {
+      for (const auto& flow : flows_) {
         if (!isFirst && !demarcator_.empty()) {
-          int64_t len = stream->write(reinterpret_cast<uint8_t*>(const_cast<char*>(demarcator_.data())), gsl::narrow<int>(demarcator_.size()));
-          if (len < 0)
-            return len;
-          ret += len;
+          const auto write_ret = stream->write(reinterpret_cast<const uint8_t*>(demarcator_.data()), demarcator_.size());
+          if (io::isError(write_ret))
+            return -1;
+          write_size_sum += write_ret;
         }
-        int len = serializer_.serialize(flow, stream);
+        const auto len = serializer_.serialize(flow, stream);
         if (len < 0)
           return len;
-        ret += len;
+        write_size_sum += gsl::narrow<size_t>(len);
         isFirst = false;
       }
       if (!footer_.empty()) {
-        int64_t len = stream->write(reinterpret_cast<uint8_t*>(const_cast<char*>(footer_.data())), gsl::narrow<int>(footer_.size()));
-        if (len < 0)
-          return len;
-        ret += len;
+        const auto write_ret = stream->write(reinterpret_cast<const uint8_t*>(footer_.data()), footer_.size());
+        if (io::isError(write_ret))
+          return -1;
+        write_size_sum += write_ret;
       }
-      return ret;
+      return gsl::narrow<int64_t>(write_size_sum);
     }
   };
 
@@ -122,25 +131,26 @@ class ArchiveMerge {
   class ArchiveWriter : public io::OutputStream {
    public:
     ArchiveWriter(struct archive *arch, struct archive_entry *entry) : arch_(arch), entry_(entry) {}
-    int write(const uint8_t* data, int size) override {
+    size_t write(const uint8_t* data, size_t size) override {
       if (!header_emitted_) {
         if (archive_write_header(arch_, entry_) != ARCHIVE_OK) {
-          return -1;
+          return io::STREAM_ERROR;
         }
         header_emitted_ = true;
       }
-      int totalWrote = 0;
-      int remaining = size;
+      size_t totalWrote = 0;
+      size_t remaining = size;
       while (remaining > 0) {
         const auto ret = archive_write_data(arch_, data + totalWrote, remaining);
         if (ret < 0) {
-          return ret;
+          return io::STREAM_ERROR;
         }
-        if (ret == 0) {
+        const auto zret = gsl::narrow<size_t>(ret);
+        if (zret == 0) {
           break;
         }
-        totalWrote += ret;
-        remaining -= ret;
+        totalWrote += zret;
+        remaining -= zret;
       }
       return totalWrote;
     }
@@ -151,33 +161,31 @@ class ArchiveMerge {
     bool header_emitted_{false};
   };
   // Nest Callback Class for write stream
-  class WriteCallback: public OutputStreamCallback {
+  class WriteCallback {
    public:
     WriteCallback(std::string merge_type, std::deque<std::shared_ptr<core::FlowFile>> &flows, FlowFileSerializer& serializer)
-        : merge_type_(merge_type),
+        : merge_type_(std::move(merge_type)),
           flows_(flows),
-          logger_(logging::LoggerFactory<ArchiveMerge>::getLogger()),
           serializer_(serializer) {
       size_ = 0;
       stream_ = nullptr;
     }
-    ~WriteCallback() = default;
 
     std::string merge_type_;
     std::deque<std::shared_ptr<core::FlowFile>> &flows_;
     std::shared_ptr<io::BaseStream> stream_;
     size_t size_;
-    std::shared_ptr<logging::Logger> logger_;
+    std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ArchiveMerge>::getLogger();
     FlowFileSerializer& serializer_;
 
     static la_ssize_t archive_write(struct archive* /*arch*/, void *context, const void *buff, size_t size) {
-      WriteCallback *callback = (WriteCallback *) context;
+      WriteCallback *callback = reinterpret_cast<WriteCallback *>(context);
       uint8_t* data = reinterpret_cast<uint8_t*>(const_cast<void*>(buff));
       la_ssize_t totalWrote = 0;
-      int remaining = gsl::narrow<int>(size);
+      size_t remaining = size;
       while (remaining > 0) {
-        la_ssize_t ret = callback->stream_->write(data + totalWrote, remaining);
-        if (ret < 0) {
+        const auto ret = callback->stream_->write(data + totalWrote, remaining);
+        if (io::isError(ret)) {
           // libarchive expects us to return -1 on error
           return -1;
         }
@@ -185,13 +193,13 @@ class ArchiveMerge {
           break;
         }
         callback->size_ += ret;
-        totalWrote += ret;
+        totalWrote += static_cast<la_ssize_t>(ret);
         remaining -= ret;
       }
       return totalWrote;
     }
 
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) {
       struct archive *arch;
 
       arch = archive_write_new();
@@ -225,7 +233,7 @@ class ArchiveMerge {
             }
           }
         }
-        int ret = serializer_.serialize(flow, std::make_shared<ArchiveWriter>(arch, entry));
+        const auto ret = serializer_.serialize(flow, std::make_shared<ArchiveWriter>(arch, entry));
         if (ret < 0) {
           return ret;
         }
@@ -296,8 +304,7 @@ class MergeContent : public processors::BinFiles {
    * Create a new processor
    */
   explicit MergeContent(const std::string& name, const utils::Identifier& uuid = {})
-      : processors::BinFiles(name, uuid),
-        logger_(logging::LoggerFactory<MergeContent>::getLogger()) {
+      : processors::BinFiles(name, uuid) {
     mergeStrategy_ = merge_content_options::MERGE_STRATEGY_DEFRAGMENT;
     mergeFormat_ = merge_content_options::MERGE_FORMAT_CONCAT_VALUE;
     delimiterStrategy_ = merge_content_options::DELIMITER_STRATEGY_FILENAME;
@@ -305,22 +312,22 @@ class MergeContent : public processors::BinFiles {
     attributeStrategy_ = merge_content_options::ATTRIBUTE_STRATEGY_KEEP_COMMON;
   }
   // Destructor
-  virtual ~MergeContent() = default;
+  ~MergeContent() override = default;
   // Processor Name
-  static constexpr char const* ProcessorName = "MergeContent";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "MergeContent";
   // Supported Properties
-  static core::Property MergeStrategy;
-  static core::Property MergeFormat;
-  static core::Property CorrelationAttributeName;
-  static core::Property DelimiterStrategy;
-  static core::Property KeepPath;
-  static core::Property Header;
-  static core::Property Footer;
-  static core::Property Demarcator;
-  static core::Property AttributeStrategy;
+  EXTENSIONAPI static core::Property MergeStrategy;
+  EXTENSIONAPI static core::Property MergeFormat;
+  EXTENSIONAPI static core::Property CorrelationAttributeName;
+  EXTENSIONAPI static core::Property DelimiterStrategy;
+  EXTENSIONAPI static core::Property KeepPath;
+  EXTENSIONAPI static core::Property Header;
+  EXTENSIONAPI static core::Property Footer;
+  EXTENSIONAPI static core::Property Demarcator;
+  EXTENSIONAPI static core::Property AttributeStrategy;
 
   // Supported Relationships
-  static core::Relationship Merge;
+  EXTENSIONAPI static core::Relationship Merge;
 
  public:
   /**
@@ -349,7 +356,7 @@ class MergeContent : public processors::BinFiles {
     return core::annotation::Input::INPUT_REQUIRED;
   }
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<MergeContent>::getLogger();
   std::string mergeStrategy_;
   std::string mergeFormat_;
   std::string correlationAttributeName_;
@@ -366,14 +373,8 @@ class MergeContent : public processors::BinFiles {
   std::string readContent(std::string path);
 };
 
-REGISTER_RESOURCE(MergeContent, "Merges a Group of FlowFiles together based on a user-defined strategy and packages them into a single FlowFile. "
-    "MergeContent should be configured with only one incoming connection as it won't create grouped Flow Files."
-    "This processor updates the mime.type attribute as appropriate.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
