@@ -25,12 +25,14 @@
 #include <memory>
 #include <string>
 #include <set>
+#include <system_error>
 
-#include <archive.h>
-#include <archive_entry.h>
+#include "archive.h"
+#include "archive_entry.h"
 
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "utils/gsl.h"
 
 namespace org {
@@ -58,7 +60,7 @@ void UnfocusArchiveEntry::onTrigger(core::ProcessContext *context, core::Process
     return;
   }
 
-  fileutils::FileManager file_man;
+  utils::file::FileManager file_man;
   ArchiveMetadata lensArchiveMetadata;
 
   // Get lens stack from attribute
@@ -137,14 +139,13 @@ void UnfocusArchiveEntry::onTrigger(core::ProcessContext *context, core::Process
 
   // Create archive by restoring each entry in the archive from tmp files
   WriteCallback cb(&lensArchiveMetadata);
-  session->write(flowFile, &cb);
+  session->write(flowFile, std::cref(cb));
 
   // Transfer to the relationship
   session->transfer(flowFile, Success);
 }
 
 UnfocusArchiveEntry::WriteCallback::WriteCallback(ArchiveMetadata *archiveMetadata) {
-  logger_ = logging::LoggerFactory<UnfocusArchiveEntry>::getLogger();
   _archiveMetadata = archiveMetadata;
 }
 
@@ -153,12 +154,12 @@ typedef struct {
 } UnfocusArchiveEntryWriteData;
 
 la_ssize_t UnfocusArchiveEntry::WriteCallback::write_cb(struct archive *, void *d, const void *buffer, size_t length) {
-  auto data = static_cast<UnfocusArchiveEntryWriteData *>(d);
-  const uint8_t *ui_buffer = static_cast<const uint8_t*>(buffer);
-  return data->stream->write(const_cast<uint8_t*>(ui_buffer), gsl::narrow<int>(length));
+  auto* const data = static_cast<UnfocusArchiveEntryWriteData *>(d);
+  const auto write_ret = data->stream->write(static_cast<const uint8_t*>(buffer), length);
+  return io::isError(write_ret) ? -1 : gsl::narrow<la_ssize_t>(write_ret);
 }
 
-int64_t UnfocusArchiveEntry::WriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
+int64_t UnfocusArchiveEntry::WriteCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) const {
   auto outputArchive = archive_write_new();
   int64_t nlen = 0;
 
@@ -181,7 +182,7 @@ int64_t UnfocusArchiveEntry::WriteCallback::process(const std::shared_ptr<io::Ba
     if (entryMetadata.entryType == AE_IFREG && entryMetadata.entrySize > 0) {
       size_t stat_ok = stat(entryMetadata.tmpFileName.c_str(), &st);
       if (stat_ok != 0) {
-        logger_->log_error("Error statting %s: %d", entryMetadata.tmpFileName, stat_ok);
+        logger_->log_error("Error statting %s: %s", entryMetadata.tmpFileName, std::system_category().default_error_condition(errno).message());
       }
       archive_entry_copy_stat(entry, &st);
     }
@@ -192,7 +193,7 @@ int64_t UnfocusArchiveEntry::WriteCallback::process(const std::shared_ptr<io::Ba
     archive_entry_set_size(entry, entryMetadata.entrySize);
     archive_entry_set_uid(entry, entryMetadata.entryUID);
     archive_entry_set_gid(entry, entryMetadata.entryGID);
-    archive_entry_set_mtime(entry, entryMetadata.entryMTime, gsl::narrow<long>(entryMetadata.entryMTimeNsec));
+    archive_entry_set_mtime(entry, entryMetadata.entryMTime, gsl::narrow<long>(entryMetadata.entryMTimeNsec));  // NOLINT long comes from libarchive API
 
     logger_->log_info("Writing %s with type %d, perms %d, size %d, uid %d, gid %d, mtime %d,%d", entryMetadata.entryName, entryMetadata.entryType, entryMetadata.entryPerm,
                       entryMetadata.entrySize, entryMetadata.entryUID, entryMetadata.entryGID, entryMetadata.entryMTime, entryMetadata.entryMTimeNsec);
@@ -234,6 +235,8 @@ int64_t UnfocusArchiveEntry::WriteCallback::process(const std::shared_ptr<io::Ba
   return nlen;
 }
 
+REGISTER_RESOURCE(UnfocusArchiveEntry, "Restores a FlowFile which has had an archive entry focused via FocusArchiveEntry to its original state.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
