@@ -25,14 +25,15 @@
 #include <string>
 #include <map>
 #include <set>
+#include <type_traits>
 #include <vector>
 
 #include "utils/gsl.h"
 #include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
-#include "utils/GeneralUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -40,23 +41,9 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-#define COMPRESSION_CODEC_NONE "none"
-#define COMPRESSION_CODEC_GZIP "gzip"
-#define COMPRESSION_CODEC_SNAPPY "snappy"
-#define ROUND_ROBIN_PARTITIONING "Round Robin"
-#define RANDOM_PARTITIONING "Random Robin"
-#define USER_DEFINED_PARTITIONING "User-Defined"
-#define DELIVERY_REPLICATED "all"
-#define DELIVERY_ONE_NODE "1"
-#define DELIVERY_BEST_EFFORT "0"
-#define SECURITY_PROTOCOL_PLAINTEXT "plaintext"
-#define SECURITY_PROTOCOL_SSL "ssl"
-#define KAFKA_KEY_ATTRIBUTE "kafka.key"
-
 const core::Property PublishKafka::SeedBrokers(
     core::PropertyBuilder::createProperty("Known Brokers")->withDescription("A comma-separated list of known Kafka Brokers in the format <host>:<port>")
         ->isRequired(true)->supportsExpressionLanguage(true)->build());
-
 const core::Property PublishKafka::Topic(
     core::PropertyBuilder::createProperty("Topic Name")->withDescription("The Kafka Topic of interest")
         ->isRequired(true)->supportsExpressionLanguage(true)->build());
@@ -67,7 +54,6 @@ const core::Property PublishKafka::DeliveryGuarantee(
                                                                                  "-1 or all (block until message is committed by all in sync replicas) "
                                                                                  "or any concrete number of nodes.")
         ->isRequired(false)->supportsExpressionLanguage(true)->withDefaultValue(DELIVERY_ONE_NODE)->build());
-
 const core::Property PublishKafka::MaxMessageSize(
     core::PropertyBuilder::createProperty("Max Request Size")->withDescription("Maximum Kafka protocol request message size")
         ->isRequired(false)->build());
@@ -95,7 +81,7 @@ const core::Property PublishKafka::AttributeNameRegex("Attributes to Send as Hea
 const core::Property PublishKafka::QueueBufferMaxTime(
         core::PropertyBuilder::createProperty("Queue Buffering Max Time")
         ->isRequired(false)
-        ->withDefaultValue<core::TimePeriodValue>("10 sec")
+        ->withDefaultValue<core::TimePeriodValue>("5 millis")
         ->withDescription("Delay to wait for messages in the producer queue to accumulate before constructing message batches")
         ->build());
 const core::Property PublishKafka::QueueBufferMaxSize(
@@ -117,26 +103,14 @@ const core::Property PublishKafka::CompressCodec(
         ->withAllowableValues<std::string>({COMPRESSION_CODEC_NONE, COMPRESSION_CODEC_GZIP, COMPRESSION_CODEC_SNAPPY})
         ->withDescription("compression codec to use for compressing message sets")
         ->build());
-
 const core::Property PublishKafka::MaxFlowSegSize(
     core::PropertyBuilder::createProperty("Max Flow Segment Size")->withDescription("Maximum flow content payload segment size for the kafka record. 0 B means unlimited.")
         ->isRequired(false)->withDefaultValue<core::DataSizeValue>("0 B")->build());
-const core::Property PublishKafka::SecurityProtocol(
-        core::PropertyBuilder::createProperty("Security Protocol")
-        ->withDescription("Protocol used to communicate with brokers")
-        ->withDefaultValue<std::string>(SECURITY_PROTOCOL_PLAINTEXT)
-        ->withAllowableValues<std::string>({SECURITY_PROTOCOL_PLAINTEXT, SECURITY_PROTOCOL_SSL})
-        ->isRequired(true)
-        ->build());
-const core::Property PublishKafka::SecurityCA("Security CA", "File or directory path to CA certificate(s) for verifying the broker's key", "");
-const core::Property PublishKafka::SecurityCert("Security Cert", "Path to client's public key (PEM) used for authentication", "");
-const core::Property PublishKafka::SecurityPrivateKey("Security Private Key", "Path to client's private key (PEM) used for authentication", "");
-const core::Property PublishKafka::SecurityPrivateKeyPassWord("Security Pass Phrase", "Private key passphrase", "");
-const core::Property PublishKafka::KerberosServiceName("Kerberos Service Name", "Kerberos Service Name", "");
-const core::Property PublishKafka::KerberosPrincipal("Kerberos Principal", "Keberos Principal", "");
-const core::Property PublishKafka::KerberosKeytabPath("Kerberos Keytab Path",
-                                                "The path to the location on the local filesystem where the kerberos keytab is located. Read permission on the file is required.", "");
 
+const core::Property PublishKafka::SecurityCA("Security CA", "DEPRECATED in favor of SSL Context Service. File or directory path to CA certificate(s) for verifying the broker's key", "");
+const core::Property PublishKafka::SecurityCert("Security Cert", "DEPRECATED in favor of SSL Context Service.Path to client's public key (PEM) used for authentication", "");
+const core::Property PublishKafka::SecurityPrivateKey("Security Private Key", "DEPRECATED in favor of SSL Context Service.Path to client's private key (PEM) used for authentication", "");
+const core::Property PublishKafka::SecurityPrivateKeyPassWord("Security Pass Phrase", "DEPRECATED in favor of SSL Context Service.Private key passphrase", "");
 const core::Property PublishKafka::KafkaKey(
     core::PropertyBuilder::createProperty("Kafka Key")
         ->withDescription("The key to use for the message. If not specified, the UUID of the flow file is used as the message key.")
@@ -198,7 +172,7 @@ class PublishKafka::Messages {
   std::condition_variable cv_;
   std::vector<FlowFileResult> flow_files_;
   bool interrupted_ = false;
-  const std::shared_ptr<logging::Logger> logger_;
+  const std::shared_ptr<core::logging::Logger> logger_;
 
   std::string logStatus(const std::unique_lock<std::mutex>& lock) const {
     gsl_Expects(lock.owns_lock());
@@ -232,12 +206,12 @@ class PublishKafka::Messages {
   }
 
  public:
-  explicit Messages(std::shared_ptr<logging::Logger> logger) :logger_{std::move(logger)} {}
+  explicit Messages(std::shared_ptr<core::logging::Logger> logger) :logger_{std::move(logger)} {}
 
   void waitForCompletion() {
     std::unique_lock<std::mutex> lock(mutex_);
     cv_.wait(lock, [this, &lock] {
-      if (logger_->should_log(logging::LOG_LEVEL::trace)) {
+      if (logger_->should_log(core::logging::LOG_LEVEL::trace)) {
         logger_->log_trace("%s", logStatus(lock));
       }
       return interrupted_ || std::all_of(std::begin(this->flow_files_), std::end(this->flow_files_), [](const FlowFileResult& flow_file) {
@@ -267,7 +241,7 @@ class PublishKafka::Messages {
   }
 
   template<typename Func>
-  auto iterateFlowFiles(Func fun) -> utils::void_t<decltype(fun(size_t{0}, flow_files_.front()))> {
+  auto iterateFlowFiles(Func fun) -> std::void_t<decltype(fun(size_t{0}, flow_files_.front()))> {
     std::lock_guard<std::mutex> lock(mutex_);
     for (size_t index = 0U; index < flow_files_.size(); index++) {
       fun(index, flow_files_[index]);
@@ -288,7 +262,7 @@ class PublishKafka::Messages {
 };
 
 namespace {
-class ReadCallback : public InputStreamCallback {
+class ReadCallback {
  public:
   struct rd_kafka_headers_deleter {
     void operator()(rd_kafka_headers_t* ptr) const noexcept {
@@ -314,14 +288,14 @@ class ReadCallback : public InputStreamCallback {
     if (!result) { throw std::bad_alloc{}; }
 
     for (const auto& kv : flow_file.getAttributes()) {
-      if (attribute_name_regex.match(kv.first)) {
+      if (utils::regexSearch(kv.first, attribute_name_regex)) {
         rd_kafka_header_add(result, kv.first.c_str(), kv.first.size(), kv.second.c_str(), kv.second.size());
       }
     }
     return rd_kafka_headers_unique_ptr{ result };
   }
 
-  rd_kafka_resp_err_t produce(const size_t segment_num, std::vector<unsigned char>& buffer, const size_t buflen) const {
+  rd_kafka_resp_err_t produce(const size_t segment_num, std::vector<std::byte>& buffer, const size_t buflen) const {
     const std::shared_ptr<PublishKafka::Messages> messages_ptr_copy = this->messages_;
     const auto flow_file_index_copy = this->flow_file_index_;
     const auto logger = logger_;
@@ -338,7 +312,7 @@ class ReadCallback : public InputStreamCallback {
       });
     };
     // release()d below, deallocated in PublishKafka::messageDeliveryCallback
-    auto callback_ptr = utils::make_unique<std::function<void(rd_kafka_t*, const rd_kafka_message_t*)>>(std::move(produce_callback));
+    auto callback_ptr = std::make_unique<std::function<void(rd_kafka_t*, const rd_kafka_message_t*)>>(std::move(produce_callback));
 
     allocate_message_object(segment_num);
 
@@ -367,7 +341,7 @@ class ReadCallback : public InputStreamCallback {
       std::shared_ptr<PublishKafka::Messages> messages,
       const size_t flow_file_index,
       const bool fail_empty_flow_files,
-      std::shared_ptr<logging::Logger> logger)
+      std::shared_ptr<core::logging::Logger> logger)
       : flow_size_(flowFile.getSize()),
       max_seg_size_(max_seg_size == 0 || flow_size_ < max_seg_size ? flow_size_ : max_seg_size),
       key_(std::move(key)),
@@ -383,8 +357,8 @@ class ReadCallback : public InputStreamCallback {
   ReadCallback(const ReadCallback&) = delete;
   ReadCallback& operator=(ReadCallback) = delete;
 
-  int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
-    std::vector<unsigned char> buffer;
+  int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) {
+    std::vector<std::byte> buffer;
 
     buffer.resize(max_seg_size_);
     read_size_ = 0;
@@ -409,14 +383,13 @@ class ReadCallback : public InputStreamCallback {
     }
 
     for (size_t segment_num = 0; read_size_ < flow_size_; ++segment_num) {
-      const int readRet = stream->read(buffer.data(), gsl::narrow<int>(buffer.size()));
-      if (readRet < 0) {
+      const auto readRet = stream->read(buffer);
+      if (io::isError(readRet)) {
         status_ = -1;
         error_ = "Failed to read from stream";
         return read_size_;
       }
-
-      if (readRet <= 0) { break; }
+      if (readRet == 0) { break; }
 
       const auto err = produce(segment_num, buffer, readRet);
       if (err) {
@@ -447,7 +420,7 @@ class ReadCallback : public InputStreamCallback {
   uint32_t read_size_ = 0;
   bool called_ = false;
   const bool fail_empty_flow_files_ = true;
-  const std::shared_ptr<logging::Logger> logger_;
+  const std::shared_ptr<core::logging::Logger> logger_;
 };
 
 /**
@@ -468,40 +441,44 @@ void messageDeliveryCallback(rd_kafka_t* rk, const rd_kafka_message_t* rkmessage
 
 void PublishKafka::initialize() {
   // Set the supported properties
-  std::set<core::Property> properties;
-  properties.insert(SeedBrokers);
-  properties.insert(Topic);
-  properties.insert(DeliveryGuarantee);
-  properties.insert(MaxMessageSize);
-  properties.insert(RequestTimeOut);
-  properties.insert(MessageTimeOut);
-  properties.insert(ClientName);
-  properties.insert(AttributeNameRegex);
-  properties.insert(BatchSize);
-  properties.insert(TargetBatchPayloadSize);
-  properties.insert(QueueBufferMaxTime);
-  properties.insert(QueueBufferMaxSize);
-  properties.insert(QueueBufferMaxMessage);
-  properties.insert(CompressCodec);
-  properties.insert(MaxFlowSegSize);
-  properties.insert(SecurityProtocol);
-  properties.insert(SecurityCA);
-  properties.insert(SecurityCert);
-  properties.insert(SecurityPrivateKey);
-  properties.insert(SecurityPrivateKeyPassWord);
-  properties.insert(KerberosServiceName);
-  properties.insert(KerberosPrincipal);
-  properties.insert(KerberosKeytabPath);
-  properties.insert(KafkaKey);
-  properties.insert(MessageKeyField);
-  properties.insert(DebugContexts);
-  properties.insert(FailEmptyFlowFiles);
-  setSupportedProperties(properties);
+  setSupportedProperties({
+    SeedBrokers,
+    Topic,
+    DeliveryGuarantee,
+    MaxMessageSize,
+    RequestTimeOut,
+    MessageTimeOut,
+    ClientName,
+    AttributeNameRegex,
+    BatchSize,
+    TargetBatchPayloadSize,
+    QueueBufferMaxTime,
+    QueueBufferMaxSize,
+    QueueBufferMaxMessage,
+    CompressCodec,
+    MaxFlowSegSize,
+    SecurityProtocol,
+    SSLContextService,
+    SecurityCA,
+    SecurityCert,
+    SecurityPrivateKey,
+    SecurityPrivateKeyPassWord,
+    KerberosServiceName,
+    KerberosPrincipal,
+    KerberosKeytabPath,
+    KafkaKey,
+    MessageKeyField,
+    DebugContexts,
+    FailEmptyFlowFiles,
+    SASLMechanism,
+    Username,
+    Password
+  });
   // Set the supported relationships
-  std::set<core::Relationship> relationships;
-  relationships.insert(Failure);
-  relationships.insert(Success);
-  setSupportedRelationships(relationships);
+  setSupportedRelationships({
+    Success,
+    Failure
+  });
 }
 
 void PublishKafka::onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory>& /*sessionFactory*/) {
@@ -540,7 +517,7 @@ void PublishKafka::onSchedule(const std::shared_ptr<core::ProcessContext> &conte
   key_.brokers_ = brokers;
   key_.client_id_ = client_id;
 
-  conn_ = utils::make_unique<KafkaConnection>(key_);
+  conn_ = std::make_unique<KafkaConnection>(key_);
   configureNewConnection(context);
 
   std::string message_key_field;
@@ -554,11 +531,18 @@ void PublishKafka::onSchedule(const std::shared_ptr<core::ProcessContext> &conte
 void PublishKafka::notifyStop() {
   logger_->log_debug("notifyStop called");
   interrupted_ = true;
-  std::lock_guard<std::mutex> conn_lock(connection_mutex_);
-  std::lock_guard<std::mutex> lock(messages_mutex_);
-  for (auto& messages : messages_set_) {
-    messages->interrupt();
+  {
+    // Normally when we need both connection_mutex_ and messages_mutex_, we need to take connection_mutex_ first to avoid a deadlock.
+    // It's not possible to do that here, because we need to interrupt the messages while onTrigger is running and holding connection_mutex_.
+    // For this reason, we take messages_mutex_ only, interrupt the messages, then release the lock to let a possibly running onTrigger take it and finish.
+    // After onTrigger finishes, we can take connection_mutex_ and close the connection without needing to wait for message finishes/timeouts in onTrigger.
+    // A possible new onTrigger between our critical sections won't produce more messages because we set interrupted_ = true above.
+    std::lock_guard<std::mutex> lock(messages_mutex_);
+    for (auto& messages : messages_set_) {
+      messages->interrupt();
+    }
   }
+  std::lock_guard<std::mutex> conn_lock(connection_mutex_);
   conn_.reset();
 }
 
@@ -607,33 +591,7 @@ bool PublishKafka::configureNewConnection(const std::shared_ptr<core::ProcessCon
       throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
     }
   }
-  value = "";
-  if (context->getProperty(KerberosServiceName.getName(), value) && !value.empty()) {
-    result = rd_kafka_conf_set(conf_.get(), "sasl.kerberos.service.name", value.c_str(), errstr.data(), errstr.size());
-    logger_->log_debug("PublishKafka: sasl.kerberos.service.name [%s]", value);
-    if (result != RD_KAFKA_CONF_OK) {
-      auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-      throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-    }
-  }
-  value = "";
-  if (context->getProperty(KerberosPrincipal.getName(), value) && !value.empty()) {
-    result = rd_kafka_conf_set(conf_.get(), "sasl.kerberos.principal", value.c_str(), errstr.data(), errstr.size());
-    logger_->log_debug("PublishKafka: sasl.kerberos.principal [%s]", value);
-    if (result != RD_KAFKA_CONF_OK) {
-      auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-      throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-    }
-  }
-  value = "";
-  if (context->getProperty(KerberosKeytabPath.getName(), value) && !value.empty()) {
-    result = rd_kafka_conf_set(conf_.get(), "sasl.kerberos.keytab", value.c_str(), errstr.data(), errstr.size());
-    logger_->log_debug("PublishKafka: sasl.kerberos.keytab [%s]", value);
-    if (result != RD_KAFKA_CONF_OK) {
-      auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-      throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-    }
-  }
+
   value = "";
   if (context->getProperty(MaxMessageSize.getName(), value) && !value.empty()) {
     result = rd_kafka_conf_set(conf_.get(), "message.max.bytes", value.c_str(), errstr.data(), errstr.size());
@@ -644,7 +602,13 @@ bool PublishKafka::configureNewConnection(const std::shared_ptr<core::ProcessCon
     }
   }
   value = "";
-  if (context->getProperty(QueueBufferMaxMessage.getName(), value) && !value.empty()) {
+  uint32_t int_val;
+  if (context->getProperty(QueueBufferMaxMessage.getName(), int_val)) {
+    if (int_val < batch_size_) {
+      throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Invalid configuration: Batch Size cannot be larger than Queue Max Message");
+    }
+
+    value = std::to_string(int_val);
     result = rd_kafka_conf_set(conf_.get(), "queue.buffering.max.messages", value.c_str(), errstr.data(), errstr.size());
     logger_->log_debug("PublishKafka: queue.buffering.max.messages [%s]", value);
     if (result != RD_KAFKA_CONF_OK) {
@@ -663,17 +627,14 @@ bool PublishKafka::configureNewConnection(const std::shared_ptr<core::ProcessCon
       throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
     }
   }
-  value = "";
-  if (context->getProperty(QueueBufferMaxTime.getName(), value) && !value.empty()) {
-    core::TimeUnit unit;
-    if (core::Property::StringToTime(value, valInt, unit) && core::Property::ConvertTimeUnitToMS(valInt, unit, valInt)) {
-      valueConf = std::to_string(valInt);
-      result = rd_kafka_conf_set(conf_.get(), "queue.buffering.max.ms", valueConf.c_str(), errstr.data(), errstr.size());
-      logger_->log_debug("PublishKafka: queue.buffering.max.ms [%s]", valueConf);
-      if (result != RD_KAFKA_CONF_OK) {
-        auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-        throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-      }
+
+  if (auto queue_buffer_max_time = context->getProperty<core::TimePeriodValue>(QueueBufferMaxTime)) {
+    valueConf = std::to_string(queue_buffer_max_time->getMilliseconds().count());
+    result = rd_kafka_conf_set(conf_.get(), "queue.buffering.max.ms", valueConf.c_str(), errstr.data(), errstr.size());
+    logger_->log_debug("PublishKafka: queue.buffering.max.ms [%s]", valueConf);
+    if (result != RD_KAFKA_CONF_OK) {
+      auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
+      throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
     }
   }
   value = "";
@@ -694,58 +655,8 @@ bool PublishKafka::configureNewConnection(const std::shared_ptr<core::ProcessCon
       throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
     }
   }
-  value = "";
-  if (context->getProperty(SecurityProtocol.getName(), value) && !value.empty()) {
-    if (value == SECURITY_PROTOCOL_SSL) {
-      result = rd_kafka_conf_set(conf_.get(), "security.protocol", value.c_str(), errstr.data(), errstr.size());
-      logger_->log_debug("PublishKafka: security.protocol [%s]", value);
-      if (result != RD_KAFKA_CONF_OK) {
-        auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-        throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-      }
-      value = "";
-      if (context->getProperty(SecurityCA.getName(), value) && !value.empty()) {
-        result = rd_kafka_conf_set(conf_.get(), "ssl.ca.location", value.c_str(), errstr.data(), errstr.size());
-        logger_->log_debug("PublishKafka: ssl.ca.location [%s]", value);
-        if (result != RD_KAFKA_CONF_OK) {
-          auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-          throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-        }
-      }
-      value = "";
-      if (context->getProperty(SecurityCert.getName(), value) && !value.empty()) {
-        result = rd_kafka_conf_set(conf_.get(), "ssl.certificate.location", value.c_str(), errstr.data(), errstr.size());
-        logger_->log_debug("PublishKafka: ssl.certificate.location [%s]", value);
-        if (result != RD_KAFKA_CONF_OK) {
-          auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-          throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-        }
-      }
-      value = "";
-      if (context->getProperty(SecurityPrivateKey.getName(), value) && !value.empty()) {
-        result = rd_kafka_conf_set(conf_.get(), "ssl.key.location", value.c_str(), errstr.data(), errstr.size());
-        logger_->log_debug("PublishKafka: ssl.key.location [%s]", value);
-        if (result != RD_KAFKA_CONF_OK) {
-          auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-          throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-        }
-      }
-      value = "";
-      if (context->getProperty(SecurityPrivateKeyPassWord.getName(), value) && !value.empty()) {
-        result = rd_kafka_conf_set(conf_.get(), "ssl.key.password", value.c_str(), errstr.data(), errstr.size());
-        logger_->log_debug("PublishKafka: ssl.key.password [%s]", value);
-        if (result != RD_KAFKA_CONF_OK) {
-          auto error_msg = utils::StringUtils::join_pack(PREFIX_ERROR_MSG, errstr.data());
-          throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-        }
-      }
-    } else if (value == SECURITY_PROTOCOL_PLAINTEXT) {
-      // Do nothing
-    } else {
-      auto error_msg = utils::StringUtils::join_pack("PublishKafka: unknown Security Protocol: ", value);
-      throw Exception(PROCESS_SCHEDULE_EXCEPTION, error_msg);
-    }
-  }
+
+  setKafkaAuthenticationParameters(*context, gsl::make_not_null(conf_.get()));
 
   // Add all of the dynamic properties as librdkafka configurations
   const auto &dynamic_prop_keys = context->getDynamicPropertyKeys();
@@ -795,7 +706,6 @@ bool PublishKafka::createNewTopic(const std::shared_ptr<core::ProcessContext> &c
   rd_kafka_conf_res_t result;
   std::string value;
   std::array<char, 512U> errstr{};
-  int64_t valInt;
   std::string valueConf;
 
   value = "";
@@ -822,32 +732,24 @@ bool PublishKafka::createNewTopic(const std::shared_ptr<core::ProcessContext> &c
       return false;
     }
   }
-  value = "";
-  if (context->getProperty(RequestTimeOut.getName(), value) && !value.empty()) {
-    core::TimeUnit unit;
-    if (core::Property::StringToTime(value, valInt, unit) &&
-        core::Property::ConvertTimeUnitToMS(valInt, unit, valInt)) {
-      valueConf = std::to_string(valInt);
-      result = rd_kafka_topic_conf_set(topic_conf_.get(), "request.timeout.ms", valueConf.c_str(), errstr.data(), errstr.size());
-      logger_->log_debug("PublishKafka: request.timeout.ms [%s]", valueConf);
-      if (result != RD_KAFKA_CONF_OK) {
-        logger_->log_error("PublishKafka: configure request.timeout.ms error result [%s]", errstr.data());
-        return false;
-      }
+
+  if (auto request_timeout = context->getProperty<core::TimePeriodValue>(RequestTimeOut)) {
+    valueConf = std::to_string(request_timeout->getMilliseconds().count());
+    result = rd_kafka_topic_conf_set(topic_conf_.get(), "request.timeout.ms", valueConf.c_str(), errstr.data(), errstr.size());
+    logger_->log_debug("PublishKafka: request.timeout.ms [%s]", valueConf);
+    if (result != RD_KAFKA_CONF_OK) {
+      logger_->log_error("PublishKafka: configure request.timeout.ms error result [%s]", errstr.data());
+      return false;
     }
   }
-  value = "";
-  if (context->getProperty(MessageTimeOut.getName(), value) && !value.empty()) {
-    core::TimeUnit unit;
-    if (core::Property::StringToTime(value, valInt, unit) &&
-        core::Property::ConvertTimeUnitToMS(valInt, unit, valInt)) {
-      valueConf = std::to_string(valInt);
-      result = rd_kafka_topic_conf_set(topic_conf_.get(), "message.timeout.ms", valueConf.c_str(), errstr.data(), errstr.size());
-      logger_->log_debug("PublishKafka: message.timeout.ms [%s]", valueConf);
-      if (result != RD_KAFKA_CONF_OK) {
-        logger_->log_error("PublishKafka: configure message.timeout.ms error result [%s]", errstr.data());
-        return false;
-      }
+
+  if (auto message_timeout = context->getProperty<core::TimePeriodValue>(MessageTimeOut)) {
+    valueConf = std::to_string(message_timeout->getMilliseconds().count());
+    result = rd_kafka_topic_conf_set(topic_conf_.get(), "message.timeout.ms", valueConf.c_str(), errstr.data(), errstr.size());
+    logger_->log_debug("PublishKafka: message.timeout.ms [%s]", valueConf);
+    if (result != RD_KAFKA_CONF_OK) {
+      logger_->log_error("PublishKafka: configure message.timeout.ms error result [%s]", errstr.data());
+      return false;
     }
   }
 
@@ -865,6 +767,19 @@ bool PublishKafka::createNewTopic(const std::shared_ptr<core::ProcessContext> &c
   return true;
 }
 
+std::optional<utils::SSL_data> PublishKafka::getSslData(core::ProcessContext& context) const {
+  if (auto result = KafkaProcessorBase::getSslData(context); result) {
+    return result;
+  }
+
+  utils::SSL_data ssl_data;
+  context.getProperty(SecurityCA.getName(), ssl_data.ca_loc);
+  context.getProperty(SecurityCert.getName(), ssl_data.cert_loc);
+  context.getProperty(SecurityPrivateKey.getName(), ssl_data.key_loc);
+  context.getProperty(SecurityPrivateKeyPassWord.getName(), ssl_data.key_pw);
+  return ssl_data;
+}
+
 void PublishKafka::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
   // Check whether we have been interrupted
   if (interrupted_) {
@@ -955,11 +870,11 @@ void PublishKafka::onTrigger(const std::shared_ptr<core::ProcessContext> &contex
 
     ReadCallback callback(max_flow_seg_size_, kafkaKey, thisTopic->getTopic(), conn_->getConnection(), *flowFile,
                                         attributeNameRegex_, messages, flow_file_index, failEmptyFlowFiles, logger_);
-    session->read(flowFile, &callback);
+    session->read(flowFile, std::ref(callback));
 
     if (!callback.called_) {
       // workaround: call callback since ProcessSession doesn't do so for empty flow files without resource claims
-      callback.process(nullptr);
+      callback(nullptr);
     }
 
     if (flowFile->getSize() == 0 && failEmptyFlowFiles) {
@@ -1024,6 +939,9 @@ void PublishKafka::onTrigger(const std::shared_ptr<core::ProcessContext> &contex
   });
 }
 
+REGISTER_RESOURCE(PublishKafka, "This Processor puts the contents of a FlowFile to a Topic in Apache Kafka. The content of a FlowFile becomes the contents of a Kafka message. "
+                  "This message is optionally assigned a key by using the <Kafka Key> Property.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
