@@ -31,18 +31,18 @@
 #include <utility>
 #include <vector>
 
+#include "KafkaProcessorBase.h"
 #include "utils/GeneralUtils.h"
 #include "FlowFileRecord.h"
-#include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/Property.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "core/logging/Logger.h"
-#include "utils/RegexUtils.h"
+#include "controllers/SSLContextService.h"
 #include "rdkafka.h"
 #include "KafkaConnection.h"
+#include "utils/RegexUtils.h"
 
 namespace org {
 namespace apache {
@@ -51,45 +51,52 @@ namespace minifi {
 namespace processors {
 
 // PublishKafka Class
-class PublishKafka : public core::Processor {
+class PublishKafka : public KafkaProcessorBase {
  public:
   static constexpr char const* ProcessorName = "PublishKafka";
 
   // Supported Properties
-  static const core::Property SeedBrokers;
-  static const core::Property Topic;
-  static const core::Property DeliveryGuarantee;
-  static const core::Property MaxMessageSize;
-  static const core::Property RequestTimeOut;
-  static const core::Property MessageTimeOut;
-  static const core::Property ClientName;
-  static const core::Property BatchSize;
-  static const core::Property TargetBatchPayloadSize;
-  static const core::Property AttributeNameRegex;
-  static const core::Property QueueBufferMaxTime;
-  static const core::Property QueueBufferMaxSize;
-  static const core::Property QueueBufferMaxMessage;
-  static const core::Property CompressCodec;
-  static const core::Property MaxFlowSegSize;
-  static const core::Property SecurityProtocol;
-  static const core::Property SecurityCA;
-  static const core::Property SecurityCert;
-  static const core::Property SecurityPrivateKey;
-  static const core::Property SecurityPrivateKeyPassWord;
-  static const core::Property KerberosServiceName;
-  static const core::Property KerberosPrincipal;
-  static const core::Property KerberosKeytabPath;
-  static const core::Property KafkaKey;
-  static const core::Property MessageKeyField;
-  static const core::Property DebugContexts;
-  static const core::Property FailEmptyFlowFiles;
+  EXTENSIONAPI static const core::Property SeedBrokers;
+  EXTENSIONAPI static const core::Property Topic;
+  EXTENSIONAPI static const core::Property DeliveryGuarantee;
+  EXTENSIONAPI static const core::Property MaxMessageSize;
+  EXTENSIONAPI static const core::Property RequestTimeOut;
+  EXTENSIONAPI static const core::Property MessageTimeOut;
+  EXTENSIONAPI static const core::Property ClientName;
+  EXTENSIONAPI static const core::Property BatchSize;
+  EXTENSIONAPI static const core::Property TargetBatchPayloadSize;
+  EXTENSIONAPI static const core::Property AttributeNameRegex;
+  EXTENSIONAPI static const core::Property QueueBufferMaxTime;
+  EXTENSIONAPI static const core::Property QueueBufferMaxSize;
+  EXTENSIONAPI static const core::Property QueueBufferMaxMessage;
+  EXTENSIONAPI static const core::Property CompressCodec;
+  EXTENSIONAPI static const core::Property MaxFlowSegSize;
+  EXTENSIONAPI static const core::Property SecurityCA;
+  EXTENSIONAPI static const core::Property SecurityCert;
+  EXTENSIONAPI static const core::Property SecurityPrivateKey;
+  EXTENSIONAPI static const core::Property SecurityPrivateKeyPassWord;
+  EXTENSIONAPI static const core::Property KafkaKey;
+  EXTENSIONAPI static const core::Property MessageKeyField;
+  EXTENSIONAPI static const core::Property DebugContexts;
+  EXTENSIONAPI static const core::Property FailEmptyFlowFiles;
 
   // Supported Relationships
-  static const core::Relationship Failure;
-  static const core::Relationship Success;
+  EXTENSIONAPI static const core::Relationship Failure;
+  EXTENSIONAPI static const core::Relationship Success;
+
+  static constexpr const char* COMPRESSION_CODEC_NONE = "none";
+  static constexpr const char* COMPRESSION_CODEC_GZIP = "gzip";
+  static constexpr const char* COMPRESSION_CODEC_SNAPPY = "snappy";
+  static constexpr const char* ROUND_ROBIN_PARTITIONING = "Round Robin";
+  static constexpr const char* RANDOM_PARTITIONING = "Random Robin";
+  static constexpr const char* USER_DEFINED_PARTITIONING = "User-Defined";
+  static constexpr const char* DELIVERY_REPLICATED = "all";
+  static constexpr const char* DELIVERY_ONE_NODE = "1";
+  static constexpr const char* DELIVERY_BEST_EFFORT = "0";
+  static constexpr const char* KAFKA_KEY_ATTRIBUTE = "kafka.key";
 
   explicit PublishKafka(const std::string& name, const utils::Identifier& uuid = {})
-      : core::Processor(std::move(name), uuid) {
+      : KafkaProcessorBase(name, uuid, core::logging::LoggerFactory<PublishKafka>::getLogger()) {
   }
 
   ~PublishKafka() override = default;
@@ -112,14 +119,13 @@ class PublishKafka : public core::Processor {
  protected:
   bool configureNewConnection(const std::shared_ptr<core::ProcessContext> &context);
   bool createNewTopic(const std::shared_ptr<core::ProcessContext> &context, const std::string& topic_name, const std::shared_ptr<core::FlowFile>& flow_file);
+  std::optional<utils::SSL_data> getSslData(core::ProcessContext& context) const override;
 
  private:
   core::annotation::Input getInputRequirement() const override {
     return core::annotation::Input::INPUT_REQUIRED;
   }
 
-  std::shared_ptr<logging::Logger> logger_{logging::LoggerFactory<PublishKafka>::getLogger()};
-
   KafkaConnectionKey key_;
   std::unique_ptr<KafkaConnection> conn_;
   std::mutex connection_mutex_;
@@ -134,9 +140,6 @@ class PublishKafka : public core::Processor {
   std::set<std::shared_ptr<Messages>> messages_set_;
 };
 
-REGISTER_RESOURCE(PublishKafka, "This Processor puts the contents of a FlowFile to a Topic in Apache Kafka. The content of a FlowFile becomes the contents of a Kafka message. "
-                  "This message is optionally assigned a key by using the <Kafka Key> Property.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
