@@ -24,6 +24,7 @@
 #include <memory>
 #include <set>
 #include "core/Property.h"
+#include "core/Resource.h"
 #include "io/validation.h"
 #include "properties/Configure.h"
 
@@ -38,16 +39,16 @@ core::Property MQTTControllerService::ClientID("Client ID", "MQTT client ID to u
 core::Property MQTTControllerService::UserName("Username", "Username to use when connecting to the broker", "");
 core::Property MQTTControllerService::Password("Password", "Password to use when connecting to the broker", "");
 core::Property MQTTControllerService::KeepLiveInterval("Keep Alive Interval", "Defines the maximum time interval between messages sent or received", "60 sec");
-core::Property MQTTControllerService::ConnectionTimeOut("Connection Timeout", "Maximum time interval the client will wait for the network connection to the MQTT server", "30 sec");
+core::Property MQTTControllerService::ConnectionTimeout("Connection Timeout", "Maximum time interval the client will wait for the network connection to the MQTT server", "30 sec");
 core::Property MQTTControllerService::QOS("Quality of Service", "The Quality of Service(QoS) to send the message with. Accepts three values '0', '1' and '2'", "MQTT_QOS_0");
 core::Property MQTTControllerService::Topic("Topic", "The topic to publish the message to", "");
 core::Property MQTTControllerService::SecurityProtocol("Security Protocol", "Protocol used to communicate with brokers", "");
 
 void MQTTControllerService::initialize() {
-  if (initialized_)
-    return;
-
   std::lock_guard<std::mutex> lock(initialization_mutex_);
+  if (initialized_) {
+    return;
+  }
 
   ControllerService::initialize();
 
@@ -70,7 +71,7 @@ void MQTTControllerService::onEnable() {
     }
 
     if (client_) {
-      MQTTClient_setCallbacks(client_, (void *) this, reconnectCallback, receiveCallback, deliveryCallback);
+      MQTTClient_setCallbacks(client_, this, reconnectCallback, receiveCallback, deliveryCallback);
       // call reconnect to bootstrap
       this->reconnect();
     }
@@ -85,13 +86,15 @@ void MQTTControllerService::initializeProperties() {
   supportedProperties.insert(Password);
 
   supportedProperties.insert(KeepLiveInterval);
-  supportedProperties.insert(ConnectionTimeOut);
+  supportedProperties.insert(ConnectionTimeout);
   supportedProperties.insert(Topic);
   supportedProperties.insert(QOS);
   supportedProperties.insert(SecurityProtocol);
   setSupportedProperties(supportedProperties);
 }
 
+REGISTER_INTERNAL_RESOURCE_AS(MQTTControllerService, ("MQTTContextService"));
+
 } /* namespace controllers */
 } /* namespace minifi */
 } /* namespace nifi */
