@@ -15,16 +15,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_CONTROLLERS_MQTTCONTEXTSERVICE_H_
-#define LIBMINIFI_INCLUDE_CONTROLLERS_MQTTCONTEXTSERVICE_H_
+#pragma once
 
 #include <openssl/err.h>
 #include <openssl/ssl.h>
+
 #include <iostream>
+#include <limits>
+#include <map>
 #include <memory>
-#include "core/Resource.h"
+#include <string>
+#include <utility>
+#include <vector>
+
 #include "utils/StringUtils.h"
-#include "io/validation.h"
 #include "core/controller/ControllerService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "controllers/SSLContextService.h"
@@ -45,9 +49,9 @@ class Message {
  public:
   // empty constructor facilitates moves
   Message() = default;
-  explicit Message(const std::string &topic, void *data, size_t dataLen)
-      : topic_(topic),
-        data_((uint8_t*) data, ((uint8_t*)data + dataLen)) {
+  explicit Message(std::string topic, void *data, size_t dataLen)
+      : topic_(std::move(topic)),
+        data_(reinterpret_cast<std::byte*>(data), (reinterpret_cast<std::byte*>(data) + dataLen)) {
   }
 
   Message(const Message &other) = default;
@@ -59,7 +63,7 @@ class Message {
   Message &operator=(Message &&other) = default;
 
   std::string topic_;
-  std::vector<uint8_t> data_;
+  std::vector<std::byte> data_;
 };
 
 /**
@@ -72,22 +76,16 @@ class MQTTControllerService : public core::controller::ControllerService {
       : ControllerService(name, uuid),
         initialized_(false),
         client_(nullptr),
-        keepAliveInterval_(0),
-        connectionTimeOut_(0),
         qos_(2),
-        ssl_context_service_(nullptr),
-        logger_(logging::LoggerFactory<MQTTControllerService>::getLogger()) {
+        ssl_context_service_(nullptr) {
   }
 
   explicit MQTTControllerService(const std::string &name, const std::shared_ptr<Configure> &configuration)
       : ControllerService(name),
         initialized_(false),
         client_(nullptr),
-        keepAliveInterval_(0),
-        connectionTimeOut_(0),
         qos_(2),
-        ssl_context_service_(nullptr),
-        logger_(logging::LoggerFactory<MQTTControllerService>::getLogger()) {
+        ssl_context_service_(nullptr) {
     setConfiguration(configuration);
     initialize();
   }
@@ -98,7 +96,7 @@ class MQTTControllerService : public core::controller::ControllerService {
   static core::Property Password;
   static core::Property CleanSession;
   static core::Property KeepLiveInterval;
-  static core::Property ConnectionTimeOut;
+  static core::Property ConnectionTimeout;
   static core::Property Topic;
   static core::Property QOS;
   static core::Property SecurityProtocol;
@@ -128,12 +126,14 @@ class MQTTControllerService : public core::controller::ControllerService {
     return token;
   }
 
-  int send(const std::string &topic, const uint8_t *data, size_t dataSize) {
+  int send(const std::string &topic, gsl::span<const std::byte> data) {
+    gsl_Expects(data.size() < std::numeric_limits<int>::max());
     int token;
 
+    std::vector<std::byte> copy_because_const_correctness{std::begin(data), std::end(data)};
     MQTTClient_message pubmsg = MQTTClient_message_initializer;
-    pubmsg.payload = const_cast<uint8_t*>(data);
-    pubmsg.payloadlen = dataSize;
+    pubmsg.payload = copy_because_const_correctness.data();
+    pubmsg.payloadlen = gsl::narrow<int>(copy_because_const_correctness.size());
     pubmsg.qos = qos_;
     pubmsg.retained = 0;
 
@@ -179,7 +179,7 @@ class MQTTControllerService : public core::controller::ControllerService {
     }
   }
 
-  bool get(const uint64_t millisToWait, const std::string &topic, std::vector<uint8_t> &data) {
+  bool get(const uint64_t millisToWait, const std::string &topic, std::vector<std::byte> &data) {
     std::unique_lock<std::mutex> lock(delivery_mutex_);
     if (delivery_notification_.wait_for(lock, std::chrono::milliseconds(millisToWait), [&] {return topics_[topic].size_approx() > 0;})) {
       Message resp;
@@ -194,7 +194,7 @@ class MQTTControllerService : public core::controller::ControllerService {
     }
   }
 
-  bool awaitResponse(const uint64_t millisToWait, int token, const std::string &topic, std::vector<uint8_t> &data) {
+  bool awaitResponse(const uint64_t millisToWait, int token, const std::string &topic, std::vector<std::byte> &data) {
     std::unique_lock<std::mutex> lock(delivery_mutex_);
     if (delivery_notification_.wait_for(lock, std::chrono::milliseconds(millisToWait), [&] {
       return
@@ -223,7 +223,6 @@ class MQTTControllerService : public core::controller::ControllerService {
   }
 
  protected:
-
   void acknowledgeDelivery(MQTTClient_deliveryToken token) {
     std::lock_guard<std::mutex> lock(delivery_mutex_);
     // locked the mutex
@@ -242,12 +241,12 @@ class MQTTControllerService : public core::controller::ControllerService {
   }
 
   static void deliveryCallback(void *context, MQTTClient_deliveryToken dt) {
-    MQTTControllerService *service = (MQTTControllerService *) context;
+    MQTTControllerService *service = reinterpret_cast<MQTTControllerService *>(context);
     service->acknowledgeDelivery(dt);
   }
 
   static int receiveCallback(void *context, char *topicName, int topicLen, MQTTClient_message *message) {
-    MQTTControllerService *service = (MQTTControllerService *) context;
+    MQTTControllerService *service = reinterpret_cast<MQTTControllerService *>(context);
     std::string topic(topicName, topicLen == 0 ? strlen(topicName) : topicLen);
     Message queueMessage(topic, message->payload, message->payloadlen);
     service->enqueue(topic, std::move(queueMessage));
@@ -256,7 +255,7 @@ class MQTTControllerService : public core::controller::ControllerService {
     return 1;
   }
   static void reconnectCallback(void *context, char* /*cause*/) {
-    MQTTControllerService *service = (MQTTControllerService *) context;
+    MQTTControllerService *service = reinterpret_cast<MQTTControllerService *>(context);
     service->reconnect();
   }
 
@@ -266,7 +265,7 @@ class MQTTControllerService : public core::controller::ControllerService {
     if (MQTTClient_isConnected(client_))
       return true;
     MQTTClient_connectOptions conn_opts = MQTTClient_connectOptions_initializer;
-    conn_opts.keepAliveInterval = keepAliveInterval_;
+    conn_opts.keepAliveInterval = std::chrono::duration_cast<std::chrono::seconds>(keepAliveInterval_).count();
     conn_opts.cleansession = 1;
     if (!userName_.empty()) {
       conn_opts.username = userName_.c_str();
@@ -289,20 +288,19 @@ class MQTTControllerService : public core::controller::ControllerService {
   virtual void initializeProperties();
 
   std::mutex initialization_mutex_;
-  std::atomic<bool> initialized_;
+  bool initialized_;
 
   MQTTClient client_;
   std::string uri_;
   std::string topic_;
-  int64_t keepAliveInterval_;
-  int64_t connectionTimeOut_;
+  std::chrono::milliseconds keepAliveInterval_{0};
+  std::chrono::milliseconds connectionTimeout_{0};
   int64_t qos_;
   std::string clientID_;
   std::string userName_;
   std::string passWord_;
 
  private:
-
   std::map<int, bool> delivered_;
   std::map<std::string, moodycamel::ConcurrentQueue<Message> > topics_;
 
@@ -313,7 +311,7 @@ class MQTTControllerService : public core::controller::ControllerService {
 
   std::shared_ptr<controllers::SSLContextService> ssl_context_service_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<MQTTControllerService>::getLogger();
 };
 
 } /* namespace controllers */
@@ -321,5 +319,3 @@ class MQTTControllerService : public core::controller::ControllerService {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* LIBMINIFI_INCLUDE_CONTROLLERS_MQTTCONTEXTSERVICE_H_ */
