@@ -17,14 +17,16 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __ABSTRACTMQTT_H__
-#define __ABSTRACTMQTT_H__
+#pragma once
+
+#include <set>
+#include <string>
+#include <memory>
 
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "MQTTClient.h"
 
@@ -49,22 +51,19 @@ class AbstractMQTTProcessor : public core::Processor {
    * Create a new processor
    */
   explicit AbstractMQTTProcessor(const std::string& name, const utils::Identifier& uuid = {})
-      : core::Processor(name, uuid),
-        logger_(logging::LoggerFactory<AbstractMQTTProcessor>::getLogger()) {
+      : core::Processor(name, uuid) {
     client_ = nullptr;
     cleanSession_ = false;
-    keepAliveInterval_ = 60;
-    connectionTimeOut_ = 30;
     qos_ = 0;
     isSubscriber_ = false;
   }
   // Destructor
-  virtual ~AbstractMQTTProcessor() {
+  ~AbstractMQTTProcessor() override {
     if (isSubscriber_) {
       MQTTClient_unsubscribe(client_, topic_.c_str());
     }
     if (client_ && MQTTClient_isConnected(client_)) {
-      MQTTClient_disconnect(client_, connectionTimeOut_);
+      MQTTClient_disconnect(client_, std::chrono::milliseconds{connectionTimeout_}.count());
     }
     if (client_)
       MQTTClient_destroy(&client_);
@@ -78,7 +77,7 @@ class AbstractMQTTProcessor : public core::Processor {
   static core::Property PassWord;
   static core::Property CleanSession;
   static core::Property KeepLiveInterval;
-  static core::Property ConnectionTimeOut;
+  static core::Property ConnectionTimeout;
   static core::Property Topic;
   static core::Property QOS;
   static core::Property SecurityProtocol;
@@ -98,11 +97,11 @@ class AbstractMQTTProcessor : public core::Processor {
 
   // MQTT async callbacks
   static void msgDelivered(void *context, MQTTClient_deliveryToken dt) {
-    AbstractMQTTProcessor *processor = (AbstractMQTTProcessor *) context;
+    AbstractMQTTProcessor *processor = reinterpret_cast<AbstractMQTTProcessor *>(context);
     processor->delivered_token_ = dt;
   }
   static int msgReceived(void *context, char *topicName, int /*topicLen*/, MQTTClient_message *message) {
-    AbstractMQTTProcessor *processor = (AbstractMQTTProcessor *) context;
+    AbstractMQTTProcessor *processor = reinterpret_cast<AbstractMQTTProcessor *>(context);
     if (processor->isSubscriber_) {
       if (!processor->enqueueReceiveMQTTMsg(message))
         MQTTClient_freeMessage(&message);
@@ -113,7 +112,7 @@ class AbstractMQTTProcessor : public core::Processor {
     return 1;
   }
   static void connectionLost(void *context, char* /*cause*/) {
-    AbstractMQTTProcessor *processor = (AbstractMQTTProcessor *) context;
+    AbstractMQTTProcessor *processor = reinterpret_cast<AbstractMQTTProcessor *>(context);
     processor->reconnect();
   }
   bool reconnect();
@@ -129,8 +128,8 @@ class AbstractMQTTProcessor : public core::Processor {
   MQTTClient_deliveryToken delivered_token_;
   std::string uri_;
   std::string topic_;
-  int64_t keepAliveInterval_;
-  int64_t connectionTimeOut_;
+  std::chrono::milliseconds keepAliveInterval_ = std::chrono::seconds(60);
+  std::chrono::milliseconds connectionTimeout_ = std::chrono::seconds(30);
   int64_t qos_;
   bool cleanSession_;
   std::string clientID_;
@@ -139,7 +138,7 @@ class AbstractMQTTProcessor : public core::Processor {
   bool isSubscriber_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AbstractMQTTProcessor>::getLogger();
   MQTTClient_SSLOptions sslopts_;
   bool sslEnabled_;
   std::string securityCA_;
@@ -154,5 +153,3 @@ class AbstractMQTTProcessor : public core::Processor {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
