@@ -17,21 +17,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __CONSUME_MQTT_H__
-#define __CONSUME_MQTT_H__
+#pragma once
 
 #include <limits>
+#include <string>
+#include <memory>
+
 #include <deque>
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/Property.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "concurrentqueue.h"
 #include "MQTTClient.h"
 #include "AbstractMQTTProcessor.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -50,14 +52,13 @@ class ConsumeMQTT : public processors::AbstractMQTTProcessor {
    * Create a new processor
    */
   explicit ConsumeMQTT(const std::string& name, const utils::Identifier& uuid = {})
-      : processors::AbstractMQTTProcessor(name, uuid),
-        logger_(logging::LoggerFactory<ConsumeMQTT>::getLogger()) {
+      : processors::AbstractMQTTProcessor(name, uuid) {
     isSubscriber_ = true;
     maxQueueSize_ = 100;
     maxSegSize_ = ULLONG_MAX;
   }
   // Destructor
-  virtual ~ConsumeMQTT() {
+  ~ConsumeMQTT() override {
     MQTTClient_message *message;
     while (queue_.try_dequeue(message)) {
       MQTTClient_freeMessage(&message);
@@ -71,23 +72,6 @@ class ConsumeMQTT : public processors::AbstractMQTTProcessor {
 
   static core::Relationship Success;
 
-  // Nest Callback Class for write stream
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(MQTTClient_message *message)
-        : message_(message) {
-      status_ = 0;
-    }
-    MQTTClient_message *message_;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      int64_t len = stream->write(reinterpret_cast<uint8_t*>(message_->payload), message_->payloadlen);
-      if (len < 0)
-        status_ = -1;
-      return len;
-    }
-    int status_;
-  };
-
  public:
   /**
    * Function that's executed when the processor is scheduled.
@@ -99,7 +83,7 @@ class ConsumeMQTT : public processors::AbstractMQTTProcessor {
   // OnTrigger method, implemented by NiFi ConsumeMQTT
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
   // Initialize, over write by NiFi ConsumeMQTT
-  void initialize(void) override;
+  void initialize() override;
   bool enqueueReceiveMQTTMsg(MQTTClient_message *message) override;
 
  protected:
@@ -115,19 +99,15 @@ class ConsumeMQTT : public processors::AbstractMQTTProcessor {
     return core::annotation::Input::INPUT_FORBIDDEN;
   }
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ConsumeMQTT>::getLogger();
   std::mutex mutex_;
   uint64_t maxQueueSize_;
   uint64_t maxSegSize_;
   moodycamel::ConcurrentQueue<MQTTClient_message *> queue_;
 };
 
-REGISTER_RESOURCE(ConsumeMQTT, "This Processor gets the contents of a FlowFile from a MQTT broker for a specified topic. The the payload of the MQTT message becomes content of a FlowFile");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
