@@ -20,11 +20,14 @@
 #include <memory>
 #include <string>
 #include <map>
+#include <vector>
 #include <set>
+
 #include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "ConvertHeartBeat.h"
 #include "c2/PayloadSerializer.h"
 #include "utils/ByteArrayCallback.h"
@@ -44,7 +47,7 @@ void ConvertHeartBeat::onTrigger(const std::shared_ptr<core::ProcessContext> &co
     context->yield();
     return;
   }
-  std::vector<uint8_t> heartbeat;
+  std::vector<std::byte> heartbeat;
   bool received_heartbeat = false;
   // while we have heartbeats we can continue to loop.
   while (mqtt_service_->get(100, listening_topic, heartbeat)) {
@@ -55,7 +58,7 @@ void ConvertHeartBeat::onTrigger(const std::shared_ptr<core::ProcessContext> &co
       minifi::utils::StreamOutputCallback byteCallback(serialized.size() + 1);
       byteCallback.write(const_cast<char*>(serialized.c_str()), serialized.size());
       auto newff = session->create();
-      session->write(newff, &byteCallback);
+      session->write(newff, std::ref(byteCallback));
       session->transfer(newff, Success);
       received_heartbeat = true;
     } else {
@@ -67,6 +70,8 @@ void ConvertHeartBeat::onTrigger(const std::shared_ptr<core::ProcessContext> &co
   }
 }
 
+REGISTER_INTERNAL_RESOURCE(ConvertHeartBeat);
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
