@@ -15,19 +15,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __CONVERT_HEARTBEAT_H__
-#define __CONVERT_HEARTBEAT_H__
+#pragma once
+
+#include <string>
+#include <memory>
 
-#include "MQTTControllerService.h"
-#include "FlowFileRecord.h"
-#include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
-#include "core/Property.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "MQTTClient.h"
-#include "c2/protocols/RESTProtocol.h"
 #include "ConvertBase.h"
 
 namespace org {
@@ -40,20 +35,20 @@ namespace processors {
  * Purpose: ConvertHeartBeat converts heatbeats into MQTT messages.
  */
 class ConvertHeartBeat: public ConvertBase{
-public:
+ public:
   // Constructor
   /*!
    * Create a new processor
    */
   explicit ConvertHeartBeat(const std::string& name, const utils::Identifier& uuid = {})
-    : ConvertBase(name, uuid), logger_(logging::LoggerFactory<ConvertHeartBeat>::getLogger()) {
+    : ConvertBase(name, uuid) {
   }
   // Destructor
-  virtual ~ConvertHeartBeat() = default;
+  ~ConvertHeartBeat() override = default;
   // Processor Name
   static constexpr char const* ProcessorName = "ConvertHeartBeat";
 
-public:
+ public:
   /**
    * Function that's executed when the processor is triggered.
    * @param context process context.
@@ -63,15 +58,12 @@ public:
 
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
-private:
-  std::shared_ptr<logging::Logger> logger_;
+ private:
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ConvertHeartBeat>::getLogger();
 };
 
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
