@@ -17,25 +17,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __CONVERT_ACKNOWLEDGEMENT_H__
-#define __CONVERT_ACKNOWLEDGEMENT_H__
+#pragma once
+
+#include <vector>
+#include <string>
+#include <memory>
 
-#include "MQTTControllerService.h"
 #include "FlowFileRecord.h"
-#include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
-#include "core/Property.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "MQTTClient.h"
-#include "c2/protocols/RESTProtocol.h"
 #include "ConvertBase.h"
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+
+
+namespace org::apache::nifi::minifi::processors {
 
 /**
  * Purpose: Converts JSON acks into an MQTT consumable by
@@ -48,11 +43,10 @@ class ConvertJSONAck : public ConvertBase {
    * Create a new processor
    */
   explicit ConvertJSONAck(const std::string& name, const utils::Identifier& uuid = {})
-      : ConvertBase(name, uuid),
-        logger_(logging::LoggerFactory<ConvertJSONAck>::getLogger()) {
+      : ConvertBase(name, uuid) {
   }
   // Destructor
-  virtual ~ConvertJSONAck() = default;
+  ~ConvertJSONAck() override = default;
   // Processor Name
   static constexpr char const* ProcessorName = "ConvertJSONAck";
 
@@ -64,39 +58,17 @@ class ConvertJSONAck : public ConvertBase {
    * @param sessionFactory process session factory that is used when creating
    * ProcessSession objects.
    */
-
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
  protected:
-
-  class ReadCallback : public InputStreamCallback {
-   public:
-    ReadCallback() = default;
-    ~ReadCallback() = default;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      int64_t ret = 0;
-      if (nullptr == stream)
-        return 0;
-      buffer_.resize(stream->size());
-      ret = stream->read(reinterpret_cast<uint8_t*>(buffer_.data()), stream->size());
-      return ret;
-    }
-    std::vector<char> buffer_;
-  };
-
   /**
    * Parse Topic name from the json -- given a known structure that we expect.
    * @param json json representation defined by the restful protocol
    */
-  std::string parseTopicName(const std::string &json);
+  static std::string parseTopicName(const std::string &json);
+
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ConvertJSONAck>::getLogger();
 };
 
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
-
-#endif
+}  // namespace org::apache::nifi::minifi::processors
