@@ -15,10 +15,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include <string>
+#include <memory>
+#include <vector>
+#include <algorithm>
+#include <set>
+
 #include "ConvertUpdate.h"
 #include "utils/HTTPClient.h"
 #include "io/BaseStream.h"
 #include "io/BufferStream.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -33,12 +40,12 @@ void ConvertUpdate::onTrigger(const std::shared_ptr<core::ProcessContext> &conte
     context->yield();
     return;
   }
-  std::vector<uint8_t> update;
+  std::vector<std::byte> update;
   bool received_update = false;
   while (mqtt_service_->get(100, listening_topic, update)) {
     // first we have the input topic string followed by the update URI
-    if (update.size() > 0) {
-      io::BufferStream stream(update.data(), update.size());
+    if (!update.empty()) {
+      io::BufferStream stream(update);
 
       std::string returnTopic, url;
 
@@ -97,6 +104,8 @@ void ConvertUpdate::initialize() {
   setSupportedRelationships(relationships);
 }
 
+REGISTER_INTERNAL_RESOURCE(ConvertUpdate);
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
