@@ -19,18 +19,20 @@
  */
 #include "PublishMQTT.h"
 
-#include <stdio.h>
 #include <algorithm>
-#include <memory>
-#include <string>
-#include <map>
-#include <set>
 #include <cinttypes>
+#include <cstdio>
+#include <map>
+#include <memory>
+#include <optional>
+#include <set>
+#include <string>
 
 #include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -64,9 +66,9 @@ void PublishMQTT::onSchedule(const std::shared_ptr<core::ProcessContext> &contex
     logger_->log_debug("PublishMQTT: max flow segment size [%" PRIu64 "]", max_seg_size_);
   }
 
-  const auto retain_parsed = [&] () -> utils::optional<bool> {
+  const auto retain_parsed = [&] () -> std::optional<bool> {
     std::string property_value;
-    if (!context->getProperty(CleanSession.getName(), property_value)) return utils::nullopt;
+    if (!context->getProperty(CleanSession.getName(), property_value)) return std::nullopt;
     return utils::StringUtils::toBool(property_value);
   }();
   if ( retain_parsed ) {
@@ -88,7 +90,7 @@ void PublishMQTT::onTrigger(const std::shared_ptr<core::ProcessContext>& /*conte
   }
 
   PublishMQTT::ReadCallback callback(flowFile->getSize(), max_seg_size_, topic_, client_, qos_, retain_, delivered_token_);
-  session->read(flowFile, &callback);
+  session->read(flowFile, std::ref(callback));
   if (callback.status_ < 0) {
     logger_->log_error("Failed to send flow to MQTT topic %s", topic_);
     session->transfer(flowFile, Failure);
@@ -98,6 +100,8 @@ void PublishMQTT::onTrigger(const std::shared_ptr<core::ProcessContext>& /*conte
   }
 }
 
+REGISTER_RESOURCE(PublishMQTT, "PublishMQTT serializes FlowFile content as an MQTT payload, sending the message to the configured topic and broker.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
