@@ -17,18 +17,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __PUBLISH_MQTT_H__
-#define __PUBLISH_MQTT_H__
+#pragma once
+
+#include <vector>
+#include <string>
+#include <memory>
+
+#include <limits>
 
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/Property.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "MQTTClient.h"
 #include "AbstractMQTTProcessor.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -44,13 +49,12 @@ class PublishMQTT : public processors::AbstractMQTTProcessor {
    * Create a new processor
    */
   explicit PublishMQTT(const std::string& name, const utils::Identifier& uuid = {})
-      : processors::AbstractMQTTProcessor(name, uuid),
-        logger_(logging::LoggerFactory<PublishMQTT>::getLogger()) {
+      : processors::AbstractMQTTProcessor(name, uuid) {
     retain_ = false;
     max_seg_size_ = ULLONG_MAX;
   }
   // Destructor
-  virtual ~PublishMQTT() = default;
+  ~PublishMQTT() override = default;
   // Processor Name
   static constexpr char const* ProcessorName = "PublishMQTT";
   // Supported Properties
@@ -61,7 +65,7 @@ class PublishMQTT : public processors::AbstractMQTTProcessor {
   static core::Relationship Success;
 
   // Nest Callback Class for read stream
-  class ReadCallback : public InputStreamCallback {
+  class ReadCallback {
    public:
     ReadCallback(uint64_t flow_size, uint64_t max_seg_size, const std::string &key, MQTTClient client, int qos, bool retain, MQTTClient_deliveryToken &token)
         : flow_size_(flow_size),
@@ -74,35 +78,36 @@ class PublishMQTT : public processors::AbstractMQTTProcessor {
       status_ = 0;
       read_size_ = 0;
     }
-    ~ReadCallback() = default;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream) {
       if (flow_size_ < max_seg_size_)
         max_seg_size_ = flow_size_;
-      std::vector<unsigned char> buffer(max_seg_size_);
+      gsl_Expects(max_seg_size_ < gsl::narrow<uint64_t>(std::numeric_limits<int>::max()));
+      std::vector<std::byte> buffer(max_seg_size_);
       read_size_ = 0;
       status_ = 0;
       while (read_size_ < flow_size_) {
-        int readRet = stream->read(&buffer[0], max_seg_size_);
-        if (readRet < 0) {
+        // MQTTClient_message::payloadlen is int, so we can't handle 2GB+
+        const auto readRet = stream->read(buffer);
+        if (io::isError(readRet)) {
           status_ = -1;
-          return read_size_;
+          return gsl::narrow<int64_t>(read_size_);
         }
         if (readRet > 0) {
           MQTTClient_message pubmsg = MQTTClient_message_initializer;
-          pubmsg.payload = &buffer[0];
-          pubmsg.payloadlen = readRet;
+          pubmsg.payload = buffer.data();
+          pubmsg.payloadlen = gsl::narrow<int>(readRet);
           pubmsg.qos = qos_;
           pubmsg.retained = retain_;
           if (MQTTClient_publishMessage(client_, key_.c_str(), &pubmsg, &token_) != MQTTCLIENT_SUCCESS) {
             status_ = -1;
             return -1;
           }
-          read_size_ += readRet;
+          read_size_ += gsl::narrow<size_t>(readRet);
         } else {
           break;
         }
       }
-      return read_size_;
+      return gsl::narrow<int64_t>(read_size_);
     }
     uint64_t flow_size_;
     uint64_t max_seg_size_;
@@ -127,7 +132,7 @@ class PublishMQTT : public processors::AbstractMQTTProcessor {
   // OnTrigger method, implemented by NiFi PublishMQTT
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
   // Initialize, over write by NiFi PublishMQTT
-  void initialize(void) override;
+  void initialize() override;
 
  private:
   core::annotation::Input getInputRequirement() const override {
@@ -136,15 +141,11 @@ class PublishMQTT : public processors::AbstractMQTTProcessor {
 
   uint64_t max_seg_size_;
   bool retain_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<PublishMQTT>::getLogger();
 };
 
-REGISTER_RESOURCE(PublishMQTT, "PublishMQTT serializes FlowFile content as an MQTT payload, sending the message to the configured topic and broker.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
