@@ -16,6 +16,8 @@
  * limitations under the License.
  */
 #include "MQTTC2Protocol.h"
+#include "core/Resource.h"
+#include "properties/Configuration.h"
 
 namespace org {
 namespace apache {
@@ -24,29 +26,29 @@ namespace minifi {
 namespace c2 {
 
 MQTTC2Protocol::MQTTC2Protocol(const std::string& name, const utils::Identifier& uuid)
-    : C2Protocol(name, uuid),
-      logger_(logging::LoggerFactory<MQTTC2Protocol>::getLogger()) {
+    : C2Protocol(name, uuid) {
 }
 
 MQTTC2Protocol::~MQTTC2Protocol() = default;
 
 void MQTTC2Protocol::initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<Configure> &configure) {
-  if (configure->get("nifi.c2.mqtt.connector.service", controller_service_name_)) {
+  if (configure->get(minifi::Configuration::nifi_c2_mqtt_connector_service, controller_service_name_)) {
     auto service = controller->getControllerService(controller_service_name_);
     mqtt_service_ = std::static_pointer_cast<controllers::MQTTControllerService>(service);
-  } else
+  } else {
     mqtt_service_ = nullptr;
+  }
 
   agent_identifier_ = configure->getAgentIdentifier();
 
   std::stringstream outputStream;
   std::string updateTopicOpt, heartbeatTopicOpt;
-  if (configure->get("nifi.c2.mqtt.heartbeat.topic", heartbeatTopicOpt)) {
+  if (configure->get(minifi::Configuration::nifi_c2_mqtt_heartbeat_topic, heartbeatTopicOpt)) {
     heartbeat_topic_ = heartbeatTopicOpt;
   } else {
     heartbeat_topic_ = "heartbeats";  // outputStream.str();
   }
-  if (configure->get("nifi.c2.mqtt.update.topic", updateTopicOpt)) {
+  if (configure->get(minifi::Configuration::nifi_c2_mqtt_update_topic, updateTopicOpt)) {
     update_topic_ = updateTopicOpt;
   } else {
     update_topic_ = "updates";
@@ -67,8 +69,8 @@ C2Payload MQTTC2Protocol::consumePayload(const std::string &url, const C2Payload
   io::BufferStream stream;
   stream.write(in_topic_);
   stream.write(url);
-  std::vector<uint8_t> response;
-  auto transmit_id = mqtt_service_->send(update_topic_, stream.getBuffer(), stream.size());
+  std::vector<std::byte> response;
+  auto transmit_id = mqtt_service_->send(update_topic_, stream.getBuffer());
   if (transmit_id > 0 && mqtt_service_->awaitResponse(5000, transmit_id, in_topic_, response)) {
     C2Payload response_payload(payload.getOperation(), state::UpdateState::READ_COMPLETE, true);
     response_payload.setRawData(response);
@@ -87,14 +89,16 @@ C2Payload MQTTC2Protocol::serialize(const C2Payload &payload) {
 
   auto stream = c2::PayloadSerializer::serialize(0x00, payload);
 
-  auto transmit_id = mqtt_service_->send(heartbeat_topic_, stream->getBuffer(), stream->size());
-  std::vector<uint8_t> response;
+  auto transmit_id = mqtt_service_->send(heartbeat_topic_, stream->getBuffer());
+  std::vector<std::byte> response;
   if (transmit_id > 0 && mqtt_service_->awaitResponse(5000, transmit_id, in_topic_, response)) {
     return c2::PayloadSerializer::deserialize(response);
   }
   return C2Payload(payload.getOperation(), state::UpdateState::READ_ERROR);
 }
 
+REGISTER_INTERNAL_RESOURCE(MQTTC2Protocol);
+
 } /* namespace c2 */
 } /* namespace minifi */
 } /* namespace nifi */
