@@ -15,8 +15,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_MQTT_PROTOCOL_MQTTC2PROTOCOL_H_
-#define EXTENSIONS_MQTT_PROTOCOL_MQTTC2PROTOCOL_H_
+#pragma once
 
 #include <algorithm>
 #include <iostream>
@@ -67,7 +66,6 @@ class MQTTC2Protocol : public C2Protocol {
   void initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<Configure> &configure) override;
 
  protected:
-
   C2Payload serialize(const C2Payload &payload);
 
   std::mutex input_mutex_;
@@ -82,14 +80,13 @@ class MQTTC2Protocol : public C2Protocol {
 
   // mqtt controller service reference.
   std::shared_ptr<controllers::MQTTControllerService> mqtt_service_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<MQTTC2Protocol>::getLogger();
   // mqtt controller serviec name.
   std::string controller_service_name_;
-
 };
+
 } /* namespace c2 */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* EXTENSIONS_MQTT_PROTOCOL_MQTTC2PROTOCOL_H_ */
