@@ -16,28 +16,23 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef NIFI_MINIFI_CPP_FetchOPCProcessor_H
-#define NIFI_MINIFI_CPP_FetchOPCProcessor_H
+#pragma once
 
 #include <memory>
 #include <string>
-#include <list>
 #include <unordered_map>
-#include <mutex>
-#include <thread>
+#include <vector>
 
 #include "opc.h"
 #include "opcbase.h"
-#include "utils/ByteArrayCallback.h"
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "controllers/SSLContextService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -46,7 +41,7 @@ namespace minifi {
 namespace processors {
 
 class FetchOPCProcessor : public BaseOPCProcessor {
-public:
+ public:
   static constexpr char const* ProcessorName = "FetchOPC";
   // Supported Properties
   static core::Property NodeIDType;
@@ -59,33 +54,23 @@ public:
   static core::Relationship Success;
   static core::Relationship Failure;
 
-  FetchOPCProcessor(const std::string& name, const utils::Identifier& uuid = {})
-  : BaseOPCProcessor(name, uuid), nameSpaceIdx_(0), nodesFound_(0), variablesFound_(0), maxDepth_(0) {
-    logger_ = logging::LoggerFactory<FetchOPCProcessor>::getLogger();
+  explicit FetchOPCProcessor(const std::string& name, const utils::Identifier& uuid = {})
+      : BaseOPCProcessor(name, uuid), nameSpaceIdx_(0), nodesFound_(0), variablesFound_(0), maxDepth_(0) {
+    logger_ = core::logging::LoggerFactory<FetchOPCProcessor>::getLogger();
   }
 
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &factory) override;
 
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
-  void initialize(void) override;
+  void initialize() override;
 
-protected:
+ protected:
   bool nodeFoundCallBack(opc::Client& client, const UA_ReferenceDescription *ref, const std::string& path,
                          const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session);
 
   void OPCData2FlowFile(const opc::NodeData& opcnode, const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session);
 
-  class WriteCallback : public OutputStreamCallback {
-    std::string data_;
-   public:
-    WriteCallback(std::string&& data)
-      : data_(data) {
-    }
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      return stream->write(reinterpret_cast<uint8_t*>(const_cast<char*>(data_.c_str())), data_.size());
-    }
-  };
   std::string nodeID_;
   int32_t nameSpaceIdx_;
   opc::OPCNodeIDType idType_;
@@ -94,18 +79,17 @@ protected:
   uint64_t maxDepth_;
   bool lazy_mode_;
 
-private:
-  std::mutex onTriggerMutex_;
+ private:
+  core::annotation::Input getInputRequirement() const override {
+    return core::annotation::Input::INPUT_FORBIDDEN;
+  }
+
   std::vector<UA_NodeId> translatedNodeIDs_;  // Only used when user provides path, path->nodeid translation is only done once
   std::unordered_map<std::string, std::string> node_timestamp_;  // Key = Full path, Value = Timestamp
 };
 
-REGISTER_RESOURCE(FetchOPCProcessor, "Fetches OPC-UA node");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // NIFI_MINIFI_CPP_FetchOPCProcessor_H
