@@ -16,18 +16,17 @@
  */
 
 
-#ifndef NIFI_MINIFI_CPP_OPC_H
-#define NIFI_MINIFI_CPP_OPC_H
-
-#include "open62541/client.h"
-#include "open62541/client_highlevel.h"
-#include "open62541/client_config_default.h"
-#include "logging/Logger.h"
-#include "Exception.h"
+#pragma once
 
 #include <string>
 #include <functional>
 #include <map>
+#include <vector>
+#include <memory>
+
+#include "open62541/client.h"
+#include "logging/Logger.h"
+#include "Exception.h"
 
 namespace org {
 namespace apache {
@@ -67,7 +66,7 @@ class Client {
   UA_StatusCode update_node(const UA_NodeId nodeId, T value);
 
   template<typename T>
-  UA_StatusCode add_node(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, T value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
+  UA_StatusCode add_node(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, T value, UA_NodeId *receivedNodeId);
 
   static std::unique_ptr<Client> createClient(std::shared_ptr<core::logging::Logger> logger, const std::string& applicationURI,
                                               const std::vector<char>& certBuffer, const std::vector<char>& keyBuffer,
@@ -108,7 +107,7 @@ struct NodeData {
  private:
   UA_Variant* var_;
 
-  NodeData(UA_Variant * var = nullptr) {
+  explicit NodeData(UA_Variant * var = nullptr) {
     var_ = var;
   }
   void addVariant(UA_Variant * var) {
@@ -126,8 +125,6 @@ static std::map<std::string, OPCNodeDataType>  StringToOPCDataTypeMap = {{"Int64
                                                                          {"UInt32", OPCNodeDataType::UInt32}, {"Boolean", OPCNodeDataType::Boolean}, {"Float", OPCNodeDataType::Float},
                                                                          {"Double", OPCNodeDataType::Double}, {"String", OPCNodeDataType::String}};
 
-int32_t OPCNodeDataTypeToTypeID(OPCNodeDataType dt);
-
 std::string nodeValue2String(const NodeData& nd);
 
 std::string OPCDateTime2String(UA_DateTime raw_date);
@@ -140,5 +137,3 @@ void logFunc(void *context, UA_LogLevel level, UA_LogCategory category, const ch
 } /* namespace apache */
 } /* namespace org */
 
-
-#endif  // NIFI_MINIFI_CPP_OPC_H
