@@ -17,10 +17,12 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_OPCBASE_H
-#define NIFI_MINIFI_CPP_OPCBASE_H
+#pragma once
 
 #include <string>
+#include <memory>
+#include <vector>
+#include <set>
 
 #include "opc.h"
 #include "core/Processor.h"
@@ -45,16 +47,20 @@ class BaseOPCProcessor : public core::Processor {
   static core::Property KeyPath;
   static core::Property TrustedPath;
 
-  BaseOPCProcessor(const std::string& name, const utils::Identifier& uuid = {})
+  explicit BaseOPCProcessor(const std::string& name, const utils::Identifier& uuid = {})
   : Processor(name, uuid) {
   }
 
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &factory) override;
 
  protected:
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
   virtual bool reconnect();
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_;
 
   opc::ClientPtr connection_;
 
@@ -79,5 +85,3 @@ class BaseOPCProcessor : public core::Processor {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // NIFI_MINIFI_CPP_OPCBASE_H
