@@ -21,7 +21,6 @@
 #include <string>
 #include <list>
 #include <map>
-#include <mutex>
 #include <thread>
 
 #include "opc.h"
@@ -136,12 +135,6 @@ namespace processors {
   void FetchOPCProcessor::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
     logger_->log_trace("FetchOPCProcessor::onTrigger");
 
-    std::unique_lock<std::mutex> lock(onTriggerMutex_, std::try_to_lock);
-    if (!lock.owns_lock()) {
-      logger_->log_warn("processor was triggered before previous listing finished, configuration should be revised!");
-      return;
-    }
-
     if (!reconnect()) {
       yield();
       return;
@@ -207,8 +200,8 @@ namespace processors {
           variablesFound_++;
         }
       } catch (const std::exception& exception) {
-        std::string browsename((char*)ref->browseName.name.data, ref->browseName.name.length);
-        logger_->log_warn("Caught Exception while trying to get data from node %s: %s", path + "/" + browsename,  exception.what());
+        std::string browse_name(reinterpret_cast<char*>(ref->browseName.name.data), ref->browseName.name.length);
+        logger_->log_warn("Caught Exception while trying to get data from node %s: %s", path + "/" + browse_name, exception.what());
       }
     }
     return true;
@@ -223,10 +216,9 @@ namespace processors {
     for (const auto& attr : opcnode.attributes) {
       flowFile->setAttribute(attr.first, attr.second);
     }
-    if (opcnode.data.size() > 0) {
+    if (!opcnode.data.empty()) {
       try {
-        FetchOPCProcessor::WriteCallback callback(opc::nodeValue2String(opcnode));
-        session->write(flowFile, &callback);
+        session->writeBuffer(flowFile, opc::nodeValue2String(opcnode));
       } catch (const std::exception& e) {
         std::string browsename;
         flowFile->getAttribute("Browsename", browsename);
@@ -238,6 +230,8 @@ namespace processors {
     session->transfer(flowFile, Success);
   }
 
+REGISTER_RESOURCE(FetchOPCProcessor, "Fetches OPC-UA node");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
