@@ -15,23 +15,22 @@
  * limitations under the License.
  */
 
-// OPC includes
 #include "opc.h"
 
-// MiNiFi includes
+#include <stdlib.h>
+#include <memory>
+#include <vector>
+#include <string>
+#include <functional>
+
 #include "utils/StringUtils.h"
 #include "logging/Logger.h"
 #include "Exception.h"
 
 #include "utils/gsl.h"
 
-// Standard includes
-#include <stdlib.h>
-#include <iostream>
-#include <memory>
-#include <vector>
-#include <string>
-#include <functional>
+#include "open62541/client_highlevel.h"
+#include "open62541/client_config_default.h"
 
 namespace org {
 namespace apache {
@@ -45,70 +44,70 @@ namespace opc {
 
 namespace {
 
-  void add_value_to_variant(UA_Variant *variant, std::string &value) {
-    UA_String ua_value = UA_STRING(&value[0]);
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_STRING]);
-  }
+void add_value_to_variant(UA_Variant *variant, std::string &value) {
+  UA_String ua_value = UA_STRING(&value[0]);
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_STRING]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, const char *value) {
-    std::string strvalue(value);
-    add_value_to_variant(variant, strvalue);
-  }
+void add_value_to_variant(UA_Variant *variant, const char *value) {
+  std::string strvalue(value);
+  add_value_to_variant(variant, strvalue);
+}
 
-  void add_value_to_variant(UA_Variant *variant, int64_t value) {
-    UA_Int64 ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_INT64]);
-  }
+void add_value_to_variant(UA_Variant *variant, int64_t value) {
+  UA_Int64 ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_INT64]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, uint64_t value) {
-    UA_UInt64 ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_UINT64]);
-  }
+void add_value_to_variant(UA_Variant *variant, uint64_t value) {
+  UA_UInt64 ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_UINT64]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, int32_t value) {
-    UA_Int32 ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_INT32]);
-  }
+void add_value_to_variant(UA_Variant *variant, int32_t value) {
+  UA_Int32 ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_INT32]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, uint32_t value) {
-    UA_UInt32 ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_UINT32]);
-  }
+void add_value_to_variant(UA_Variant *variant, uint32_t value) {
+  UA_UInt32 ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_UINT32]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, bool value) {
-    UA_Boolean ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_BOOLEAN]);
-  }
+void add_value_to_variant(UA_Variant *variant, bool value) {
+  UA_Boolean ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_BOOLEAN]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, float value) {
-    UA_Float ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_FLOAT]);
-  }
+void add_value_to_variant(UA_Variant *variant, float value) {
+  UA_Float ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_FLOAT]);
+}
 
-  void add_value_to_variant(UA_Variant *variant, double value) {
-    UA_Double ua_value = value;
-    UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_DOUBLE]);
-  }
+void add_value_to_variant(UA_Variant *variant, double value) {
+  UA_Double ua_value = value;
+  UA_Variant_setScalarCopy(variant, &ua_value, &UA_TYPES[UA_TYPES_DOUBLE]);
+}
 
-  core::logging::LOG_LEVEL MapOPCLogLevel(UA_LogLevel ualvl) {
-    switch (ualvl) {
-      case UA_LOGLEVEL_TRACE:
-        return core::logging::trace;
-      case UA_LOGLEVEL_DEBUG:
-        return core::logging::debug;
-      case UA_LOGLEVEL_INFO:
-        return core::logging::info;
-      case UA_LOGLEVEL_WARNING:
-        return core::logging::warn;
-      case UA_LOGLEVEL_ERROR:
-        return core::logging::err;
-      case UA_LOGLEVEL_FATAL:
-        return core::logging::critical;
-      default:
-        return core::logging::critical;
-    }
+core::logging::LOG_LEVEL MapOPCLogLevel(UA_LogLevel ualvl) {
+  switch (ualvl) {
+    case UA_LOGLEVEL_TRACE:
+      return core::logging::trace;
+    case UA_LOGLEVEL_DEBUG:
+      return core::logging::debug;
+    case UA_LOGLEVEL_INFO:
+      return core::logging::info;
+    case UA_LOGLEVEL_WARNING:
+      return core::logging::warn;
+    case UA_LOGLEVEL_ERROR:
+      return core::logging::err;
+    case UA_LOGLEVEL_FATAL:
+      return core::logging::critical;
+    default:
+      return core::logging::critical;
   }
 }
+}  // namespace
 
 /*
  * End of internal functions
@@ -127,13 +126,13 @@ Client::Client(std::shared_ptr<core::logging::Logger> logger, const std::string&
     // Certificate
     UA_ByteString certByteString = UA_STRING_NULL;
     certByteString.length = certBuffer.size();
-    certByteString.data = (UA_Byte*)UA_malloc(certByteString.length * sizeof(UA_Byte));
+    certByteString.data = reinterpret_cast<UA_Byte*>(UA_malloc(certByteString.length * sizeof(UA_Byte)));
     memcpy(certByteString.data, certBuffer.data(), certByteString.length);
 
     // Key
     UA_ByteString keyByteString = UA_STRING_NULL;
     keyByteString.length = keyBuffer.size();
-    keyByteString.data = (UA_Byte*)UA_malloc(keyByteString.length * sizeof(UA_Byte));
+    keyByteString.data = reinterpret_cast<UA_Byte*>(UA_malloc(keyByteString.length * sizeof(UA_Byte)));
     memcpy(keyByteString.data, keyBuffer.data(), keyByteString.length);
 
     // Trusted certificates
@@ -142,7 +141,7 @@ Client::Client(std::shared_ptr<core::logging::Logger> logger, const std::string&
     for (size_t i = 0; i < trustBuffers.size(); i++) {
       trustList[i] = UA_STRING_NULL;
       trustList[i].length = trustBuffers[i].size();
-      trustList[i].data = (UA_Byte*)UA_malloc(trustList[i].length * sizeof(UA_Byte));
+      trustList[i].data = reinterpret_cast<UA_Byte*>(UA_malloc(trustList[i].length * sizeof(UA_Byte)));
       memcpy(trustList[i].data, trustBuffers[i].data(), trustList[i].length);
     }
     UA_StatusCode sc = UA_ClientConfig_setDefaultEncryption(cc, certByteString, keyByteString,
@@ -177,7 +176,10 @@ Client::~Client() {
   if (client_ == nullptr) {
     return;
   }
-  if (UA_Client_getState(client_) != UA_CLIENTSTATE_DISCONNECTED) {
+
+  UA_SecureChannelState channel_state;
+  UA_Client_getState(client_, &channel_state, nullptr, nullptr);
+  if (channel_state != UA_SECURECHANNELSTATE_CLOSED) {
     auto sc = UA_Client_disconnect(client_);
     if (sc != UA_STATUSCODE_GOOD) {
       logger_->log_warn("Failed to disconnect OPC client: %s", UA_StatusCode_name(sc));
@@ -190,14 +192,17 @@ bool Client::isConnected() {
   if (!client_) {
     return false;
   }
-  return UA_Client_getState(client_) != UA_CLIENTSTATE_DISCONNECTED;
+
+  UA_SessionState session_state;
+  UA_Client_getState(client_, nullptr, &session_state, nullptr);
+  return session_state == UA_SESSIONSTATE_ACTIVATED;
 }
 
 UA_StatusCode Client::connect(const std::string& url, const std::string& username, const std::string& password) {
   if (username.empty()) {
     return UA_Client_connect(client_, url.c_str());
   } else {
-    return UA_Client_connect_username(client_, url.c_str(), username.c_str(), password.c_str());
+    return UA_Client_connectUsername(client_, url.c_str(), username.c_str(), password.c_str());
   }
 }
 
@@ -241,7 +246,7 @@ NodeData Client::getNodeData(const UA_ReferenceDescription *ref, const std::stri
       auto server_timestamp = OPCDateTime2String(dv->serverTimestamp);
       auto source_timestamp = OPCDateTime2String(dv->sourceTimestamp);
       nodedata.attributes["Sourcetimestamp"] = source_timestamp;
-      UA_ReadResponse_deleteMembers(&response);
+      UA_ReadResponse_clear(&response);
 
       nodedata.dataTypeID = var->type->typeIndex;
       nodedata.addVariant(var);
@@ -303,18 +308,18 @@ void Client::traverse(UA_NodeId nodeId, std::function<nodeFoundCallBackFunc> cb,
   UA_BrowseResponse bResp = UA_Client_Service_browse(client_, bReq);
 
   const auto guard = gsl::finally([&bResp]() {
-    UA_BrowseResponse_deleteMembers(&bResp);
+    UA_BrowseResponse_clear(&bResp);
   });
 
-  UA_BrowseRequest_deleteMembers(&bReq);
+  UA_BrowseRequest_clear(&bReq);
 
   for (size_t i = 0; i < bResp.resultsSize; ++i) {
     for (size_t j = 0; j < bResp.results[i].referencesSize; ++j) {
       UA_ReferenceDescription *ref = &(bResp.results[i].references[j]);
       if (cb(*this, ref, basePath)) {
         if (ref->nodeClass == UA_NODECLASS_VARIABLE || ref->nodeClass == UA_NODECLASS_OBJECT) {
-          std::string browsename((char *) ref->browseName.name.data, ref->browseName.name.length);
-          traverse(ref->nodeId.nodeId, cb, basePath + browsename, maxDepth, false);
+          std::string browse_name(reinterpret_cast<char *>(ref->browseName.name.data), ref->browseName.name.length);
+          traverse(ref->nodeId.nodeId, cb, basePath + browse_name, maxDepth, false);
         }
       } else {
         return;
@@ -347,7 +352,7 @@ UA_StatusCode Client::translateBrowsePathsToNodeIdsRequest(const std::string& pa
   UA_BrowsePath_init(&browsePath);
   browsePath.startingNode = UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER);
 
-  browsePath.relativePath.elements = (UA_RelativePathElement*)UA_Array_new(tokens.size(), &UA_TYPES[UA_TYPES_RELATIVEPATHELEMENT]);
+  browsePath.relativePath.elements = reinterpret_cast<UA_RelativePathElement*>(UA_Array_new(tokens.size(), &UA_TYPES[UA_TYPES_RELATIVEPATHELEMENT]));
   browsePath.relativePath.elementsSize = tokens.size();
 
   for (size_t i = 0; i < tokens.size(); ++i) {
@@ -364,7 +369,7 @@ UA_StatusCode Client::translateBrowsePathsToNodeIdsRequest(const std::string& pa
   UA_TranslateBrowsePathsToNodeIdsResponse response = UA_Client_Service_translateBrowsePathsToNodeIds(client_, request);
 
   const auto guard = gsl::finally([&browsePath]() {
-    UA_BrowsePath_deleteMembers(&browsePath);
+    UA_BrowsePath_clear(&browsePath);
   });
 
   if (response.resultsSize < 1) {
@@ -381,11 +386,11 @@ UA_StatusCode Client::translateBrowsePathsToNodeIdsRequest(const std::string& pa
       UA_NodeId resultId;
       UA_NodeId_copy(&res.targets[j].targetId.nodeId, &resultId);
       foundNodeIDs.push_back(resultId);
-      std::string namespaceUri((char*)res.targets[j].targetId.namespaceUri.data, res.targets[j].targetId.namespaceUri.length);
+      std::string namespaceUri(reinterpret_cast<char*>(res.targets[j].targetId.namespaceUri.data), res.targets[j].targetId.namespaceUri.length);
     }
   }
 
-  UA_TranslateBrowsePathsToNodeIdsResponse_deleteMembers(&response);
+  UA_TranslateBrowsePathsToNodeIdsResponse_clear(&response);
 
   if (foundData) {
     logger->log_debug("Found %lu nodes for path %s", foundNodeIDs.size(), path.c_str());
@@ -397,7 +402,7 @@ UA_StatusCode Client::translateBrowsePathsToNodeIdsRequest(const std::string& pa
 }
 
 template<typename T>
-UA_StatusCode Client::add_node(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, T value, OPCNodeDataType dt, UA_NodeId *receivedNodeId) {
+UA_StatusCode Client::add_node(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, T value, UA_NodeId *receivedNodeId) {
   UA_VariableAttributes attr = UA_VariableAttributes_default;
   add_value_to_variant(&attr.value, value);
   char local[6] = "en-US";
@@ -405,7 +410,7 @@ UA_StatusCode Client::add_node(const UA_NodeId parentNodeId, const UA_NodeId tar
   UA_StatusCode sc = UA_Client_addVariableNode(client_,
                                                targetNodeId,
                                                parentNodeId,
-                                               UA_NODEID_NUMERIC(0, OPCNodeDataTypeToTypeID(dt)),
+                                               UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES),
                                                UA_QUALIFIEDNAME(1, const_cast<char*>(browseName.c_str())),
                                                UA_NODEID_NULL,
                                                attr, receivedNodeId);
@@ -443,38 +448,17 @@ template UA_StatusCode Client::update_node<bool>(const UA_NodeId nodeId, bool va
 template UA_StatusCode Client::update_node<const char *>(const UA_NodeId nodeId, const char * value);
 template UA_StatusCode Client::update_node<std::string>(const UA_NodeId nodeId, std::string value);
 
-template UA_StatusCode Client::add_node<int64_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, int64_t value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<uint64_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, uint64_t value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<int32_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, int32_t value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<uint32_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, uint32_t value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<float>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, float value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<double>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, double value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<bool>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, bool value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<const char *>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, const char * value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-template UA_StatusCode Client::add_node<std::string>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, std::string value, OPCNodeDataType dt, UA_NodeId *receivedNodeId);
-
-int32_t OPCNodeDataTypeToTypeID(OPCNodeDataType dt) {
-  switch (dt) {
-    case OPCNodeDataType::Boolean:
-      return UA_NS0ID_BOOLEAN;
-    case OPCNodeDataType::Int32:
-      return UA_NS0ID_INT32;
-    case OPCNodeDataType::UInt32:
-      return UA_NS0ID_UINT32;
-    case OPCNodeDataType::Int64:
-      return UA_NS0ID_INT64;
-    case OPCNodeDataType::UInt64:
-      return UA_NS0ID_UINT64;
-    case OPCNodeDataType::Float:
-      return UA_NS0ID_FLOAT;
-    case OPCNodeDataType::Double:
-      return UA_NS0ID_DOUBLE;
-    case OPCNodeDataType::String:
-      return UA_NS0ID_STRING;
-    default:
-      throw OPCException(GENERAL_EXCEPTION, "Data type is not supported");
-  }
-}
+template UA_StatusCode Client::add_node<int64_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, int64_t value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<uint64_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, uint64_t value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<int32_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, int32_t value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<uint32_t>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, uint32_t value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<float>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, float value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<double>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, double value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<bool>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName, bool value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<const char *>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName,
+    const char * value, UA_NodeId *receivedNodeId);
+template UA_StatusCode Client::add_node<std::string>(const UA_NodeId parentNodeId, const UA_NodeId targetNodeId, std::string browseName,
+    std::string value, UA_NodeId *receivedNodeId);
 
 std::string nodeValue2String(const NodeData& nd) {
   std::string ret_val;
@@ -482,7 +466,7 @@ std::string nodeValue2String(const NodeData& nd) {
     case UA_TYPES_STRING:
     case UA_TYPES_LOCALIZEDTEXT:
     case UA_TYPES_BYTESTRING: {
-      UA_String value = *(UA_String *)(nd.var_->data);
+      UA_String value = *reinterpret_cast<UA_String *>(nd.var_->data);
       ret_val = std::string(reinterpret_cast<const char *>(value.data), value.length);
       break;
     }
@@ -572,7 +556,7 @@ std::string OPCDateTime2String(UA_DateTime raw_date) {
 
 void logFunc(void *context, UA_LogLevel level, UA_LogCategory /*category*/, const char *msg, va_list args) {
   char buffer[1024];
-  vsnprintf(buffer, 1024, msg, args);
+  vsnprintf(buffer, sizeof buffer, msg, args);
   auto loggerPtr = reinterpret_cast<core::logging::BaseLogger*>(context);
   loggerPtr->log_string(MapOPCLogLevel(level), buffer);
 }
