@@ -17,32 +17,21 @@
  * limitations under the License.
  */
 
-#include <memory>
-#include <string>
 #include <list>
-#include <map>
-#include <mutex>
-#include <thread>
+#include <memory>
+#include <set>
+#include <string>
 
 #include "opc.h"
 #include "putopc.h"
-#include "utils/ByteArrayCallback.h"
-#include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
 #include "core/Property.h"
 #include "core/Resource.h"
 #include "controllers/SSLContextService.h"
-#include "core/logging/LoggerConfiguration.h"
-#include "utils/Id.h"
 #include "utils/StringUtils.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+namespace org::apache::nifi::minifi::processors {
 
   core::Property PutOPCProcessor::ParentNodeID(
       core::PropertyBuilder::createProperty("Parent node ID")
@@ -155,12 +144,6 @@ namespace processors {
   void PutOPCProcessor::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
     logger_->log_trace("PutOPCProcessor::onTrigger");
 
-    std::unique_lock<std::mutex> lock(onTriggerMutex_, std::try_to_lock);
-    if (!lock.owns_lock()) {
-      logger_->log_warn("processor was triggered before previous listing finished, configuration should be revised!");
-      return;
-    }
-
     if (!reconnect()) {
       yield();
       return;
@@ -267,13 +250,7 @@ namespace processors {
       targetNodeExists = connection_->exists(targetnode);
     }
 
-    ReadCallback cb(logger_);
-    session->read(flowFile, &cb);
-
-    const auto &vec = cb.getContent();
-
-    std::string contentstr(reinterpret_cast<const char *>(vec.data()), vec.size());
-
+    const auto contentstr = to_string(session->readBuffer(flowFile));
     if (targetNodeExists) {
       logger_->log_trace("Node exists, trying to update it");
       try {
@@ -300,7 +277,7 @@ namespace processors {
             break;
           }
           case opc::OPCNodeDataType::Boolean: {
-            utils::optional<bool> contentstr_parsed = utils::StringUtils::toBool(contentstr);
+            const auto contentstr_parsed = utils::StringUtils::toBool(contentstr);
             if (contentstr_parsed) {
               sc = connection_->update_node(targetnode, contentstr_parsed.value());
             } else {
@@ -358,28 +335,28 @@ namespace processors {
         switch (nodeDataType_) {
           case opc::OPCNodeDataType::Int64: {
             int64_t value = std::stoll(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::UInt64: {
             uint64_t value = std::stoull(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::Int32: {
             int32_t value = std::stoi(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::UInt32: {
             uint32_t value = std::stoul(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::Boolean: {
-            utils::optional<bool> contentstr_parsed = utils::StringUtils::toBool(contentstr);
+            const auto contentstr_parsed = utils::StringUtils::toBool(contentstr);
             if (contentstr_parsed) {
-              sc = connection_->add_node(parentNodeID_, targetnode, browsename, contentstr_parsed.value(), nodeDataType_, &resultnode);
+              sc = connection_->add_node(parentNodeID_, targetnode, browsename, contentstr_parsed.value(), &resultnode);
             } else {
               throw opc::OPCException(GENERAL_EXCEPTION, "Content cannot be converted to bool");
             }
@@ -387,16 +364,16 @@ namespace processors {
           }
           case opc::OPCNodeDataType::Float: {
             float value = std::stof(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::Double: {
             double value = std::stod(contentstr);
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, value, &resultnode);
             break;
           }
           case opc::OPCNodeDataType::String: {
-            sc = connection_->add_node(parentNodeID_, targetnode, browsename, contentstr, nodeDataType_, &resultnode);
+            sc = connection_->add_node(parentNodeID_, targetnode, browsename, contentstr, &resultnode);
             break;
           }
           default:
@@ -420,32 +397,6 @@ namespace processors {
     }
   }
 
-  int64_t PutOPCProcessor::ReadCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-    buf_.clear();
-    buf_.resize(stream->size());
+REGISTER_RESOURCE(PutOPCProcessor, "Creates/updates  OPC nodes");
 
-    uint64_t size = 0;
-
-    do {
-      int read = stream->read(buf_.data() + size, 1024);
-
-      if (read < 0) {
-        return -1;
-      }
-
-      if (read == 0) {
-        break;
-      }
-      size += read;
-    } while (size < stream->size());
-
-    logger_->log_trace("Read %llu bytes from flowfile content to buffer", stream->size());
-
-    return size;
-  }
-
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
+}  // namespace org::apache::nifi::minifi::processors
