@@ -14,8 +14,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include <set>
 
 #include "CaptureRTSPFrame.h"
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -30,32 +34,31 @@ static core::Property rtspURI;
 static core::Property captureFrameRate;
 static core::Property imageEncoding;
 
-core::Property CaptureRTSPFrame::RTSPUsername(  // NOLINT
+core::Property CaptureRTSPFrame::RTSPUsername(
     "RTSP Username",
     "The username for connecting to the RTSP stream", "");
-core::Property CaptureRTSPFrame::RTSPPassword(  // NOLINT
+core::Property CaptureRTSPFrame::RTSPPassword(
     "RTSP Password",
     "Password used to connect to the RTSP stream", "");
-core::Property CaptureRTSPFrame::RTSPHostname(  // NOLINT
+core::Property CaptureRTSPFrame::RTSPHostname(
     "RTSP Hostname",
     "Hostname of the RTSP stream we are trying to connect to", "");
-core::Property CaptureRTSPFrame::RTSPURI(  // NOLINT
+core::Property CaptureRTSPFrame::RTSPURI(
     "RTSP URI",
     "URI that should be appended to the RTSP stream hostname", "");
-core::Property CaptureRTSPFrame::RTSPPort(  // NOLINT
+core::Property CaptureRTSPFrame::RTSPPort(
     "RTSP Port",
     "Port that should be connected to to receive RTSP Frames",
     "");
-core::Property CaptureRTSPFrame::ImageEncoding( // NOLINT
+core::Property CaptureRTSPFrame::ImageEncoding(
     "Image Encoding",
     "The encoding that should be applied the the frame images captured from the RTSP stream",
-    ".jpg"
-    );
+    ".jpg");
 
-core::Relationship CaptureRTSPFrame::Success(  // NOLINT
+core::Relationship CaptureRTSPFrame::Success(
     "success",
     "Successful capture of RTSP frame");
-core::Relationship CaptureRTSPFrame::Failure(  // NOLINT
+core::Relationship CaptureRTSPFrame::Failure(
     "failure",
     "Failures to capture RTSP frame");
 
@@ -145,8 +148,6 @@ void CaptureRTSPFrame::onTrigger(const std::shared_ptr<core::ProcessContext> &co
   // retrieve a frame of your source
   if (video_capture_.read(frame)) {
     if (!frame.empty()) {
-      CaptureRTSPFrameWriteCallback write_cb(frame, image_encoding_);
-
       auto t = std::time(nullptr);
       auto tm = *std::localtime(&t);
 
@@ -158,7 +159,12 @@ void CaptureRTSPFrame::onTrigger(const std::shared_ptr<core::ProcessContext> &co
       session->putAttribute(flow_file, "filename", filename);
       session->putAttribute(flow_file, "video.backend.driver", video_backend_driver_);
 
-      session->write(flow_file, &write_cb);
+      session->write(flow_file, [&frame, this](const std::shared_ptr<io::BaseStream>& output_stream) -> int64_t {
+        std::vector<uchar> image_buf;
+        imencode(image_encoding_, frame, image_buf);
+        const auto ret = output_stream->write(image_buf.data(), image_buf.size());
+        return io::isError(ret) ? -1 : gsl::narrow<int64_t>(ret);
+      });
       session->transfer(flow_file, Success);
       logger_->log_info("A frame is captured");
     } else {
@@ -174,6 +180,8 @@ void CaptureRTSPFrame::onTrigger(const std::shared_ptr<core::ProcessContext> &co
 void CaptureRTSPFrame::notifyStop() {
 }
 
+REGISTER_RESOURCE(CaptureRTSPFrame, "Captures a frame from the RTSP stream at specified intervals."); // NOLINT
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
