@@ -15,41 +15,40 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_CAPTURERTSPFRAME_H
-#define NIFI_MINIFI_CPP_CAPTURERTSPFRAME_H
+#pragma once
 
 #include <atomic>
-
-#include <core/Resource.h>
-#include <core/Processor.h>
-#include <opencv2/opencv.hpp>
-
 #include <iomanip>
 #include <ctime>
+#include <utility>
+#include <vector>
+#include <memory>
+#include <string>
+#include <opencv2/opencv.hpp>
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+#include "core/logging/LoggerConfiguration.h"
+#include "core/Processor.h"
+#include "io/StreamPipe.h"
+#include "utils/gsl.h"
+#include "utils/Export.h"
+
+namespace org::apache::nifi::minifi::processors {
 
 class CaptureRTSPFrame : public core::Processor {
  public:
-
   explicit CaptureRTSPFrame(const std::string &name, const utils::Identifier &uuid = {})
-      : Processor(name, uuid),
-        logger_(logging::LoggerFactory<CaptureRTSPFrame>::getLogger()) {
+      : Processor(name, uuid) {
   }
 
-  static core::Property RTSPUsername;
-  static core::Property RTSPPassword;
-  static core::Property RTSPHostname;
-  static core::Property RTSPURI;
-  static core::Property RTSPPort;
-  static core::Property ImageEncoding;
+  EXTENSIONAPI static core::Property RTSPUsername;
+  EXTENSIONAPI static core::Property RTSPPassword;
+  EXTENSIONAPI static core::Property RTSPHostname;
+  EXTENSIONAPI static core::Property RTSPURI;
+  EXTENSIONAPI static core::Property RTSPPort;
+  EXTENSIONAPI static core::Property ImageEncoding;
 
-  static core::Relationship Success;
-  static core::Relationship Failure;
+  EXTENSIONAPI static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Failure;
 
   void initialize() override;
   void onSchedule(core::ProcessContext *context, core::ProcessSessionFactory *sessionFactory) override;
@@ -61,28 +60,8 @@ class CaptureRTSPFrame : public core::Processor {
 
   void notifyStop() override;
 
-  class CaptureRTSPFrameWriteCallback : public OutputStreamCallback {
-   public:
-    explicit CaptureRTSPFrameWriteCallback(cv::Mat image_mat, std::string image_encoding_)
-        : image_mat_(std::move(image_mat)), image_encoding_(image_encoding_) {
-    }
-    ~CaptureRTSPFrameWriteCallback() override = default;
-
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
-      int64_t ret = 0;
-      imencode(image_encoding_, image_mat_, image_buf_);
-      ret = stream->write(image_buf_.data(), image_buf_.size());
-      return ret;
-    }
-
-   private:
-    std::vector<uchar> image_buf_;
-    cv::Mat image_mat_;
-    std::string image_encoding_;
-  };
-
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<CaptureRTSPFrame>::getLogger();
   std::mutex mutex_;
   std::string rtsp_username_;
   std::string rtsp_password_;
@@ -129,13 +108,4 @@ class CaptureRTSPFrame : public core::Processor {
 //  std::shared_ptr<minifi::controllers::SSLContextService> ssl_service_;
 };
 
-REGISTER_RESOURCE(CaptureRTSPFrame, "Captures a frame from the RTSP stream at specified intervals."); // NOLINT
-
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
-
-
-#endif  // NIFI_MINIFI_CPP_CAPTURERTSPFRAME_H
+}   // namespace org::apache::nifi::minifi::processors
