@@ -15,7 +15,14 @@
  * limitations under the License.
  */
 
+#include <set>
+#include <utility>
+#include <vector>
+
 #include "MotionDetector.h"
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -155,8 +162,16 @@ void MotionDetector::onTrigger(const std::shared_ptr<core::ProcessContext> &cont
   }
   cv::Mat frame;
 
-  opencv::FrameReadCallback cb(frame);
-  session->read(flow_file, &cb);
+  session->read(flow_file, [&frame](const std::shared_ptr<io::BaseStream>& input_stream) -> int64_t {
+    std::vector<uchar> image_buf;
+    image_buf.resize(input_stream->size());
+    const auto ret = input_stream->read(gsl::make_span(image_buf).as_span<std::byte>());
+    if (io::isError(ret) || ret != input_stream->size()) {
+      throw std::runtime_error("ImageReadCallback failed to fully read flow file input stream");
+    }
+    frame = cv::imdecode(image_buf, -1);
+    return gsl::narrow<int64_t>(ret);
+  });
 
   if (frame.empty()) {
     logger_->log_error("Empty frame.");
@@ -184,11 +199,14 @@ void MotionDetector::onTrigger(const std::shared_ptr<core::ProcessContext> &cont
 
   detectAndDraw(frame);
 
-  opencv::FrameWriteCallback write_cb(frame, image_encoding_);
-
   session->putAttribute(flow_file, "filename", filename);
 
-  session->write(flow_file, &write_cb);
+  session->write(flow_file, [&frame, this](const auto& output_stream) -> int64_t {
+    std::vector<uchar> image_buf;
+    imencode(image_encoding_, frame, image_buf);
+    const auto ret = output_stream->write(image_buf.data(), image_buf.size());
+    return io::isError(ret) ? -1 : gsl::narrow<int64_t>(ret);
+  });
   session->transfer(flow_file, Success);
   logger_->log_trace("Finish motion detecting");
 }
@@ -196,6 +214,8 @@ void MotionDetector::onTrigger(const std::shared_ptr<core::ProcessContext> &cont
 void MotionDetector::notifyStop() {
 }
 
+REGISTER_RESOURCE(MotionDetector, "Detect motion from captured images.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
