@@ -15,20 +15,16 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_MOTIONDETECTOR_H
-#define NIFI_MINIFI_CPP_MOTIONDETECTOR_H
-
-#include <atomic>
-
-#include <core/Resource.h>
-#include <core/Processor.h>
-#include <opencv2/opencv.hpp>
-#include <opencv2/objdetect.hpp>
-#include <opencv2/imgproc.hpp>
-#include "FrameIO.h"
+#pragma once
 
 #include <iomanip>
-#include <ctime>
+#include <memory>
+#include <string>
+
+#include "core/logging/LoggerConfiguration.h"
+#include "core/Processor.h"
+#include "opencv2/opencv.hpp"
+#include "opencv2/imgproc.hpp"
 
 namespace org {
 namespace apache {
@@ -38,10 +34,8 @@ namespace processors {
 
 class MotionDetector : public core::Processor {
  public:
-
   explicit MotionDetector(const std::string &name, const utils::Identifier &uuid = {})
-      : Processor(name, uuid),
-        logger_(logging::LoggerFactory<MotionDetector>::getLogger()) {
+      : Processor(name, uuid) {
   }
 
   static core::Property ImageEncoding;
@@ -53,35 +47,30 @@ class MotionDetector : public core::Processor {
   static core::Relationship Success;
   static core::Relationship Failure;
 
-  void initialize(void) override;
+  void initialize() override;
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
 
   void notifyStop() override;
 
  private:
-
   bool detectAndDraw(cv::Mat &frame);
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<MotionDetector>::getLogger();
   std::mutex mutex_;
   cv::Mat background_;
   cv::Mat bg_img_;
   std::string image_encoding_;
-  int min_area_;
-  int threshold_;
-  int dil_iter_;
+  int min_area_{};
+  int threshold_{};
+  int dil_iter_{};
 
   // hardcoded width to 500
   const double IMG_WIDTH = 500.0;
 };
 
-REGISTER_RESOURCE(MotionDetector, "Detect motion from captured images."); // NOLINT
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // NIFI_MINIFI_CPP_MOTIONDETECTOR_H
