@@ -47,7 +47,9 @@ extern "C" {
 #include "core/FlowFile.h"
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
+#include "core/ProcessSessionFactory.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
 #include "ResourceClaim.h"
@@ -64,11 +66,8 @@ extern "C" {
 #define WSMAN_CUSTOM_ACTION_HEARTBEAT "http://schemas.dmtf.org/wbem/wsman/1/wsman/Heartbeat"
 #define WSMAN_CUSTOM_ACTION_EVENTS "http://schemas.dmtf.org/wbem/wsman/1/wsman/Events"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+namespace org::apache::nifi::minifi::processors {
+
 core::Property SourceInitiatedSubscriptionListener::ListenHostname(
     core::PropertyBuilder::createProperty("Listen Hostname")->withDescription("The hostname or IP of this machine that will be advertised to event sources to connect to. "
                                                                               "It must be contained as a Subject Alternative Name in the server certificate, "
@@ -144,7 +143,6 @@ constexpr char const* SourceInitiatedSubscriptionListener::ProcessorName;
 
 SourceInitiatedSubscriptionListener::SourceInitiatedSubscriptionListener(const std::string& name, const utils::Identifier& uuid)
     : Processor(name, uuid)
-    , logger_(logging::LoggerFactory<SourceInitiatedSubscriptionListener>::getLogger())
     , session_factory_(nullptr)
     , listen_port_(0U)
     , subscription_expiration_interval_(0)
@@ -248,9 +246,9 @@ bool SourceInitiatedSubscriptionListener::loadState() {
   return true;
 }
 
-std::string SourceInitiatedSubscriptionListener::Handler::millisecondsToXsdDuration(int64_t milliseconds) {
+std::string SourceInitiatedSubscriptionListener::Handler::millisecondsToXsdDuration(std::chrono::milliseconds milliseconds) {
   char buf[1024];
-  snprintf(buf, sizeof(buf), "PT%" PRId64 ".%03" PRId64 "S", milliseconds / 1000, milliseconds % 1000);
+  snprintf(buf, sizeof(buf), "PT%" PRId64 ".%03" PRId64 "S", int64_t{milliseconds.count() / 1000}, int64_t{milliseconds.count() % 1000});
   return buf;
 }
 
@@ -323,7 +321,7 @@ bool SourceInitiatedSubscriptionListener::Handler::handlePost(CivetServer* /*ser
     int xml_buf_size = 0;
     ws_xml_dump_memory_node_tree_enc(node, &xml_buf, &xml_buf_size, "UTF-8");
     if (xml_buf != nullptr) {
-        logging::LOG_TRACE(processor_.logger_) << "Received request: \"" << std::string(xml_buf, xml_buf_size) << "\"";
+        core::logging::LOG_TRACE(processor_.logger_) << "Received request: \"" << std::string(xml_buf, xml_buf_size) << "\"";
         ws_xml_free_memory(xml_buf);
     }
   }
@@ -382,7 +380,7 @@ bool SourceInitiatedSubscriptionListener::Handler::isAckRequested(WsXmlDocH doc)
 }
 
 void SourceInitiatedSubscriptionListener::Handler::sendResponse(struct mg_connection* conn, const std::string& machineId, const std::string& remoteIp, char* xml_buf, size_t xml_buf_size) {
-  logging::LOG_TRACE(processor_.logger_) << "Sending response to " << machineId << " (" << remoteIp << "): \"" << std::string(xml_buf, xml_buf_size) << "\"";
+  core::logging::LOG_TRACE(processor_.logger_) << "Sending response to " << machineId << " (" << remoteIp << "): \"" << std::string(xml_buf, xml_buf_size) << "\"";
 
   mg_printf(conn, "HTTP/1.1 200 OK\r\n");
   mg_printf(conn, "Content-Type: application/soap+xml;charset=UTF-8\r\n");
@@ -601,24 +599,16 @@ bool SourceInitiatedSubscriptionListener::Handler::handleSubscriptionManager(str
   return true;
 }
 
-SourceInitiatedSubscriptionListener::Handler::WriteCallback::WriteCallback(char* text)
-    : text_(text) {
-}
-
-int64_t SourceInitiatedSubscriptionListener::Handler::WriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-  return stream->write(reinterpret_cast<uint8_t*>(text_), strlen(text_));
-}
-
 int SourceInitiatedSubscriptionListener::Handler::enumerateEventCallback(WsXmlNodeH node, void* data) {
   if (data == nullptr) {
     return 1;
   }
 
   std::shared_ptr<core::ProcessSession> session;
-  std::shared_ptr<logging::Logger> logger;
+  std::shared_ptr<core::logging::Logger> logger;
   std::string machine_id;
   std::string remote_ip;
-  std::tie(session, logger, machine_id, remote_ip) = *static_cast<std::tuple<std::shared_ptr<core::ProcessSession>, std::shared_ptr<logging::Logger>, std::string, std::string>*>(data);
+  std::tie(session, logger, machine_id, remote_ip) = *static_cast<std::tuple<std::shared_ptr<core::ProcessSession>, std::shared_ptr<core::logging::Logger>, std::string, std::string>*>(data);
 
   char* text = ws_xml_get_node_text(node);
   if (text == nullptr) {
@@ -634,15 +624,13 @@ int SourceInitiatedSubscriptionListener::Handler::enumerateEventCallback(WsXmlNo
       return 1;
     }
 
-    WriteCallback callback(text);
-    session->write(flow_file, &callback);
+    session->writeBuffer(flow_file, std::string_view{text});
 
     session->putAttribute(flow_file, core::SpecialFlowAttribute::MIME_TYPE, "application/xml");
     flow_file->addAttribute(ATTRIBUTE_WEF_REMOTE_MACHINEID, machine_id);
     flow_file->addAttribute(ATTRIBUTE_WEF_REMOTE_IP, remote_ip);
 
     session->transfer(flow_file, SourceInitiatedSubscriptionListener::Success);
-    session->commit();
   } catch (const std::exception& e) {
     logger->log_error("Caught exception while processing Events: %s", e.what());
     return 1;
@@ -687,7 +675,7 @@ bool SourceInitiatedSubscriptionListener::Handler::handleSubscriptions(struct mg
     }
     // Enumare Body/Events/Event nodes
     auto session = processor_.session_factory_->createSession();
-    std::tuple<std::shared_ptr<core::ProcessSession>, std::shared_ptr<logging::Logger>, std::string, std::string> callback_args =
+    std::tuple<std::shared_ptr<core::ProcessSession>, std::shared_ptr<core::logging::Logger>, std::string, std::string> callback_args =
         std::forward_as_tuple(session, processor_.logger_, machine_id, remote_ip);
     int ret = ws_xml_enum_children(events_node, &SourceInitiatedSubscriptionListener::Handler::enumerateEventCallback, &callback_args, 0 /*bRecursive*/);
     if (ret != 0) {
@@ -814,43 +802,30 @@ void SourceInitiatedSubscriptionListener::onSchedule(const std::shared_ptr<core:
   if (!context->getProperty(InitialExistingEventsStrategy.getName(), initial_existing_events_strategy_)) {
     throw Exception(PROCESSOR_EXCEPTION, "Initial Existing Events Strategy attribute is missing or invalid");
   }
-  if (!context->getProperty(SubscriptionExpirationInterval.getName(), value)) {
+  if (auto subscription_expiration_interval = context->getProperty<core::TimePeriodValue>(SubscriptionExpirationInterval)) {
+    subscription_expiration_interval_ = subscription_expiration_interval->getMilliseconds();
+  } else {
     throw Exception(PROCESSOR_EXCEPTION, "Subscription Expiration Interval attribute is missing or invalid");
-  } else {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, subscription_expiration_interval_, unit) ||
-        !core::Property::ConvertTimeUnitToMS(subscription_expiration_interval_, unit, subscription_expiration_interval_)) {
-      throw Exception(PROCESSOR_EXCEPTION, "Subscription Expiration Interval attribute is invalid");
-    }
   }
-  if (!context->getProperty(HeartbeatInterval.getName(), value)) {
-    throw Exception(PROCESSOR_EXCEPTION, "Heartbeat Interval attribute is missing or invalid");
+  if (auto heartbeat_interval = context->getProperty<core::TimePeriodValue>(HeartbeatInterval)) {
+    heartbeat_interval_ = heartbeat_interval->getMilliseconds();
   } else {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, heartbeat_interval_, unit) || !core::Property::ConvertTimeUnitToMS(heartbeat_interval_, unit, heartbeat_interval_)) {
-      throw Exception(PROCESSOR_EXCEPTION, "Heartbeat Interval attribute is invalid");
-    }
+    throw Exception(PROCESSOR_EXCEPTION, "Heartbeat Interval attribute is missing or invalid");
   }
   if (!context->getProperty(MaxElements.getName(), value)) {
     throw Exception(PROCESSOR_EXCEPTION, "Max Elements attribute is missing or invalid");
   } else if (!core::Property::StringToInt(value, max_elements_)) {
     throw Exception(PROCESSOR_EXCEPTION, "Max Elements attribute is invalid");
   }
-  if (!context->getProperty(MaxLatency.getName(), value)) {
+  if (auto max_latency = context->getProperty<core::TimePeriodValue>(MaxLatency)) {
+    max_latency_ = max_latency->getMilliseconds();
+  } else {
     throw Exception(PROCESSOR_EXCEPTION, "Max Latency attribute is missing or invalid");
-  } else {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, max_latency_, unit) || !core::Property::ConvertTimeUnitToMS(max_latency_, unit, max_latency_)) {
-      throw Exception(PROCESSOR_EXCEPTION, "Max Latency attribute is invalid");
-    }
   }
-  if (!context->getProperty(ConnectionRetryInterval.getName(), value)) {
-    throw Exception(PROCESSOR_EXCEPTION, "Connection Retry Interval attribute is missing or invalid");
+  if (auto connection_retry_interval = context->getProperty<core::TimePeriodValue>(ConnectionRetryInterval)) {
+    connection_retry_interval_ = connection_retry_interval->getMilliseconds();
   } else {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, connection_retry_interval_, unit) || !core::Property::ConvertTimeUnitToMS(connection_retry_interval_, unit, connection_retry_interval_)) {
-      throw Exception(PROCESSOR_EXCEPTION, "Connection Retry Interval attribute is invalid");
-    }
+    throw Exception(PROCESSOR_EXCEPTION, "Connection Retry Interval attribute is missing or invalid");
   }
   if (!context->getProperty(ConnectionRetryCount.getName(), value)) {
     throw Exception(PROCESSOR_EXCEPTION, "Connection Retry Count attribute is missing or invalid");
@@ -870,12 +845,12 @@ void SourceInitiatedSubscriptionListener::onSchedule(const std::shared_ptr<core:
   }
   utils::tls::X509_unique_ptr ca_ptr{ca};
 
-  std::array<uint8_t, 20U> hash_buf;
-  int ret = X509_digest(ca, EVP_sha1(), hash_buf.data(), nullptr);
+  std::array<std::byte, 20U> hash_buf{};
+  int ret = X509_digest(ca, EVP_sha1(), gsl::make_span(hash_buf).as_span<unsigned char>().data(), nullptr);
   if (ret != 1) {
     throw Exception(PROCESSOR_EXCEPTION, "Failed to get fingerprint for CA specified by SSL Certificate Authority attribute");
   }
-  ssl_ca_cert_thumbprint_ = utils::StringUtils::to_hex(hash_buf.data(), hash_buf.size(), true /*uppercase*/);
+  ssl_ca_cert_thumbprint_ = utils::StringUtils::to_hex(hash_buf, true /*uppercase*/);
   logger_->log_debug("%s SHA-1 thumbprint is %s", ssl_ca_file.c_str(), ssl_ca_cert_thumbprint_.c_str());
 
   session_factory_ = sessionFactory;
@@ -915,8 +890,7 @@ void SourceInitiatedSubscriptionListener::notifyStop() {
   server_.release();
 }
 
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
+REGISTER_RESOURCE(SourceInitiatedSubscriptionListener, "This processor implements a Windows Event Forwarding Source Initiated Subscription server with the help of OpenWSMAN. "
+                                                       "Windows hosts can be set up to connect and forward Event Logs to this processor.");
+
+}  // namespace org::apache::nifi::minifi::processors
