@@ -15,8 +15,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __SOURCE_INITIATED_SUBSCRIPTION_PROCESSOR_H__
-#define __SOURCE_INITIATED_SUBSCRIPTION_PROCESSOR_H__
+#pragma once
 
 #include <memory>
 #include <string>
@@ -36,16 +35,11 @@ extern "C" {
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "controllers/SSLContextService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+namespace org::apache::nifi::minifi::processors {
 
 class SourceInitiatedSubscriptionListener : public core::Processor {
  public:
@@ -88,16 +82,7 @@ class SourceInitiatedSubscriptionListener : public core::Processor {
   class Handler: public CivetHandler {
    public:
     explicit Handler(SourceInitiatedSubscriptionListener& processor);
-    bool handlePost(CivetServer* server, struct mg_connection* conn);
-
-    class WriteCallback : public OutputStreamCallback {
-     public:
-      explicit WriteCallback(char* text);
-      int64_t process(const std::shared_ptr<io::BaseStream>& stream);
-
-     private:
-      char* text_;
-    };
+    bool handlePost(CivetServer* server, struct mg_connection* conn) override;
 
    private:
     SourceInitiatedSubscriptionListener& processor_;
@@ -111,13 +96,17 @@ class SourceInitiatedSubscriptionListener : public core::Processor {
     bool isAckRequested(WsXmlDocH doc);
     void sendResponse(struct mg_connection* conn, const std::string& machineId, const std::string& remoteIp, char* xml_buf, size_t xml_buf_size);
 
-    static std::string millisecondsToXsdDuration(int64_t milliseconds);
+    static std::string millisecondsToXsdDuration(std::chrono::milliseconds milliseconds);
   };
 
  protected:
-  std::shared_ptr<logging::Logger> logger_;
+  bool isSingleThreaded() const override {
+    return true;
+  }
 
-  std::shared_ptr<core::CoreComponentStateManager> state_manager_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<SourceInitiatedSubscriptionListener>::getLogger();
+
+  core::CoreComponentStateManager* state_manager_;
 
   std::shared_ptr<core::ProcessSessionFactory> session_factory_;
 
@@ -128,11 +117,11 @@ class SourceInitiatedSubscriptionListener : public core::Processor {
   std::string ssl_ca_cert_thumbprint_;
   std::string xpath_xml_query_;
   std::string initial_existing_events_strategy_;
-  int64_t subscription_expiration_interval_;
-  int64_t heartbeat_interval_;
+  std::chrono::milliseconds subscription_expiration_interval_;
+  std::chrono::milliseconds heartbeat_interval_;
   uint32_t max_elements_;
-  int64_t max_latency_;
-  int64_t connection_retry_interval_;
+  std::chrono::milliseconds max_latency_;
+  std::chrono::milliseconds connection_retry_interval_;
   uint32_t connection_retry_count_;
 
   std::unique_ptr<CivetServer> server_;
@@ -161,13 +150,4 @@ class SourceInitiatedSubscriptionListener : public core::Processor {
   bool loadState();
 };
 
-REGISTER_RESOURCE(SourceInitiatedSubscriptionListener, "This processor implements a Windows Event Forwarding Source Initiated Subscription server with the help of OpenWSMAN. "
-                                                       "Windows hosts can be set up to connect and forward Event Logs to this processor.");
-
-} /* namespace processors */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
-
-#endif  // __SOURCE_INITIATED_SUBSCRIPTION_PROCESSOR_H__
+}  // namespace org::apache::nifi::minifi::processors
