@@ -16,7 +16,6 @@
  * limitations under the License.
  */
 
-#include <regex>
 #include <memory>
 #include <algorithm>
 #include <cctype>
@@ -34,15 +33,16 @@
 #include "PcapFilter.h"
 #include "PcapPlusPlusVersion.h"
 #include "PcapFileDevice.h"
-#include "PlatformSpecificUtils.h"
 #include "core/FlowFile.h"
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "CapturePacket.h"
 #include "ResourceClaim.h"
 #include "utils/StringUtils.h"
 #include "utils/ByteArrayCallback.h"
+#include "utils/RegexUtils.h"
 
 namespace org {
 namespace apache {
@@ -51,11 +51,18 @@ namespace minifi {
 namespace processors {
 
 std::shared_ptr<utils::IdGenerator> CapturePacket::id_generator_ = utils::IdGenerator::getIdGenerator();
-core::Property CapturePacket::BaseDir(core::PropertyBuilder::createProperty("Base Directory")->withDescription("Scratch directory for PCAP files")->withDefaultValue<std::string>("/tmp/")->build());
+core::Property CapturePacket::BaseDir(core::PropertyBuilder::createProperty("Base Directory")
+    ->withDescription("Scratch directory for PCAP files")
+    ->withDefaultValue<std::string>("/tmp/")->build());
+
+core::Property CapturePacket::BatchSize(core::PropertyBuilder::createProperty("Batch Size")
+    ->withDescription("The number of packets to combine within a given PCAP")
+    ->withDefaultValue<uint64_t>(50)->build());
 
-core::Property CapturePacket::BatchSize(core::PropertyBuilder::createProperty("Batch Size")->withDescription("The number of packets to combine within a given PCAP")->withDefaultValue<uint64_t>(50)->build());
 core::Property CapturePacket::NetworkControllers("Network Controllers", "Regular expression of the network controller(s) to which we will attach", ".*");
-core::Property CapturePacket::CaptureBluetooth(core::PropertyBuilder::createProperty("Capture Bluetooth")->withDescription("True indicates that we support bluetooth interfaces")->withDefaultValue<bool>(false)->build());
+core::Property CapturePacket::CaptureBluetooth(core::PropertyBuilder::createProperty("Capture Bluetooth")
+    ->withDescription("True indicates that we support bluetooth interfaces")
+    ->withDefaultValue<bool>(false)->build());
 
 const char *CapturePacket::ProcessorName = "CapturePacket";
 
@@ -160,9 +167,9 @@ void CapturePacket::onSchedule(const std::shared_ptr<core::ProcessContext> &cont
       bool found_match = false;
       std::string matching_regex = "";
       for (const auto &filter : allowed_interfaces) {
-        std::regex r(filter);
-        std::smatch m;
-        if (std::regex_match(name, m, r)) {
+        utils::Regex r(filter);
+        utils::SMatch m;
+        if (utils::regexMatch(name, m, r)) {
           matching_regex = filter;
           found_match = true;
           break;
@@ -223,6 +230,10 @@ void CapturePacket::onTrigger(const std::shared_ptr<core::ProcessContext> &conte
   }
 }
 
+REGISTER_RESOURCE(CapturePacket, "CapturePacket captures and writes one or more packets into a PCAP file that will be used as the content of a flow file."
+    " Configuration options exist to adjust the batching of PCAP files. PCAP batching will place a single PCAP into a flow file. "
+    "A regular expression selects network interfaces. Bluetooth network interfaces can be selected through a separate option.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
