@@ -16,11 +16,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_CAPTUREPACKET_H
-#define EXTENSIONS_CAPTUREPACKET_H
+#pragma once
 
 #include <memory>
 #include <regex>
+#include <vector>
+#include <string>
 
 #include "PcapLiveDeviceList.h"
 #include "PcapFilter.h"
@@ -30,7 +31,6 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "concurrentqueue.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
@@ -73,15 +73,16 @@ class CapturePacketMechanism {
     return file_;
   }
 
-  long getSize() const {
+  int64_t getSize() const {
     return atomic_count_;
   }
+
  protected:
   CapturePacketMechanism &operator=(const CapturePacketMechanism &other) = delete;
   std::string path_;
   std::string file_;
   int64_t *max_size_;
-  std::atomic<long> atomic_count_;
+  std::atomic<int64_t> atomic_count_;
 };
 
 struct PacketMovers {
@@ -92,16 +93,12 @@ struct PacketMovers {
 // CapturePacket Class
 class CapturePacket : public core::Processor {
  public:
-
   // Constructor
   /*!
    * Create a new processor
    */
   explicit CapturePacket(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(name, uuid),
-        capture_bluetooth_(false),
-        pcap_batch_size_(50),
-        logger_(logging::LoggerFactory<CapturePacket>::getLogger()) {
+      : Processor(name, uuid) {
     mover = std::unique_ptr<PacketMovers>(new PacketMovers());
   }
   // Destructor
@@ -122,7 +119,6 @@ class CapturePacket : public core::Processor {
   static void packet_callback(pcpp::RawPacket* packet, pcpp::PcapLiveDevice* dev, void* data);
 
  protected:
-
   void notifyStop() override {
     logger_->log_debug("Stopping capture");
     for (auto dev : device_list_) {
@@ -149,30 +145,23 @@ class CapturePacket : public core::Processor {
   static CapturePacketMechanism *create_new_capture(const std::string &base_path, int64_t *max_size);
 
  private:
-
   inline std::string getPath() {
     return base_dir_ + "/" + base_path_;
   }
-  bool capture_bluetooth_;
+  bool capture_bluetooth_ = false;
   std::string base_dir_;
   std::vector<std::string> attached_controllers_;
   std::string base_path_;
-  int64_t pcap_batch_size_;
+  int64_t pcap_batch_size_ = 50;
   std::unique_ptr<PacketMovers> mover;
   static std::atomic<int> num_;
   std::vector<pcpp::PcapLiveDevice*> device_list_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<CapturePacket>::getLogger();
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 };
 
-REGISTER_RESOURCE(CapturePacket, "CapturePacket captures and writes one or more packets into a PCAP file that will be used as the content of a flow file."
-    " Configuration options exist to adjust the batching of PCAP files. PCAP batching will place a single PCAP into a flow file. "
-    "A regular expression selects network interfaces. Bluetooth network interfaces can be selected through a separate option.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* EXTENSIONS_CAPTUREPACKET_H */
