@@ -25,6 +25,10 @@
 #include "MemoryConsumptionCounter.h"
 #include "utils/StringUtils.h"
 #include "utils/JsonCallback.h"
+#include "utils/OpenTelemetryLogDataModelUtils.h"
+#include "core/ProcessContext.h"
+#include "core/ProcessSession.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -119,11 +123,11 @@ void PerformanceDataMonitor::onTrigger(core::ProcessContext* context, core::Proc
   }
   if (pretty_output_) {
     utils::PrettyJsonOutputCallback callback(std::move(root), decimal_places_);
-    session->write(flowFile, &callback);
+    session->write(flowFile, std::ref(callback));
     session->transfer(flowFile, Success);
   } else {
     utils::JsonOutputCallback callback(std::move(root), decimal_places_);
-    session->write(flowFile, &callback);
+    session->write(flowFile, std::ref(callback));
     session->transfer(flowFile, Success);
   }
 }
@@ -136,13 +140,9 @@ void PerformanceDataMonitor::initialize() {
 rapidjson::Value& PerformanceDataMonitor::prepareJSONBody(rapidjson::Document& root) {
   switch (output_format_) {
     case OutputFormat::OPENTELEMETRY: {
-      root.AddMember("Name", "PerformanceData", root.GetAllocator());
-      root.AddMember("Timestamp", std::time(0), root.GetAllocator());
-      std::string hostname = io::Socket::getMyHostName();
-      rapidjson::Value hostname_value;
-      hostname_value.SetString(hostname.c_str(), hostname.length(), root.GetAllocator());
-      root.AddMember("Hostname", hostname_value, root.GetAllocator());
-      root.AddMember("Body", rapidjson::Value{ rapidjson::kObjectType }, root.GetAllocator());
+      utils::OpenTelemetryLogDataModel::appendEventInformation(root, "PerformanceData");
+      utils::OpenTelemetryLogDataModel::appendHostInformation(root);
+      utils::OpenTelemetryLogDataModel::appendBody(root);
       return root["Body"];
     }
     case OutputFormat::JSON:
@@ -318,7 +318,7 @@ void PerformanceDataMonitor::setupOutputFormatFromProperties(const std::shared_p
 void PerformanceDataMonitor::setupDecimalPlacesFromProperties(const std::shared_ptr<core::ProcessContext>& context) {
   std::string decimal_places_str;
   if (!context->getProperty(DecimalPlaces.getName(), decimal_places_str) || decimal_places_str == "") {
-    decimal_places_ = utils::nullopt;
+    decimal_places_ = std::nullopt;
     return;
   }
 
@@ -339,6 +339,8 @@ void PerformanceDataMonitor::setupMembersFromProperties(const std::shared_ptr<co
   setupDecimalPlacesFromProperties(context);
 }
 
+REGISTER_RESOURCE(PerformanceDataMonitor, "This processor can create FlowFiles with various performance data through Performance Data Helper. (Windows only)");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
