@@ -18,11 +18,13 @@
 #pragma once
 
 #include <pdh.h>
-#include <string>
-#include <vector>
 #include <memory>
+#include <optional>
+#include <string>
 #include <utility>
+#include <vector>
 
+#include "core/logging/LoggerConfiguration.h"
 #include "core/Processor.h"
 
 #include "rapidjson/stream.h"
@@ -47,19 +49,19 @@ class PerformanceDataMonitor : public core::Processor {
 
   explicit PerformanceDataMonitor(const std::string& name, utils::Identifier uuid = utils::Identifier())
       : Processor(name, uuid), output_format_(OutputFormat::JSON), pretty_output_(false),
-        decimal_places_(utils::nullopt), logger_(logging::LoggerFactory<PerformanceDataMonitor>::getLogger()),
+        decimal_places_(std::nullopt), logger_(core::logging::LoggerFactory<PerformanceDataMonitor>::getLogger()),
         pdh_query_(nullptr), resource_consumption_counters_() {}
 
   ~PerformanceDataMonitor() override;
   static constexpr char const* ProcessorName = "PerformanceDataMonitor";
   // Supported Properties
-  static core::Property PredefinedGroups;
-  static core::Property CustomPDHCounters;
-  static core::Property OutputFormatProperty;
-  static core::Property OutputCompactness;
-  static core::Property DecimalPlaces;
+  EXTENSIONAPI static core::Property PredefinedGroups;
+  EXTENSIONAPI static core::Property CustomPDHCounters;
+  EXTENSIONAPI static core::Property OutputFormatProperty;
+  EXTENSIONAPI static core::Property OutputCompactness;
+  EXTENSIONAPI static core::Property DecimalPlaces;
   // Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
 
 
  public:
@@ -72,6 +74,14 @@ class PerformanceDataMonitor : public core::Processor {
     JSON, OPENTELEMETRY
   };
 
+  core::annotation::Input getInputRequirement() const override {
+    return core::annotation::Input::INPUT_FORBIDDEN;
+  }
+
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
   rapidjson::Value& prepareJSONBody(rapidjson::Document& root);
 
   void setupMembersFromProperties(const std::shared_ptr<core::ProcessContext>& context);
@@ -85,14 +95,12 @@ class PerformanceDataMonitor : public core::Processor {
   OutputFormat output_format_;
   bool pretty_output_;
 
-  utils::optional<uint8_t> decimal_places_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::optional<uint8_t> decimal_places_;
+  std::shared_ptr<core::logging::Logger> logger_;
   PDH_HQUERY pdh_query_;
   std::vector<std::unique_ptr<PerformanceDataCounter>> resource_consumption_counters_;
 };
 
-REGISTER_RESOURCE(PerformanceDataMonitor, "This processor can create FlowFiles with various performance data through Performance Data Helper. (Windows only)");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
