@@ -15,17 +15,17 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_CORE_REPOSITORY_DatabaseContentRepository_H_
-#define LIBMINIFI_INCLUDE_CORE_REPOSITORY_DatabaseContentRepository_H_
+#pragma once
+
+#include <string>
+#include <memory>
 
-#include "rocksdb/db.h"
-#include "rocksdb/merge_operator.h"
 #include "core/Core.h"
 #include "core/Connectable.h"
 #include "core/ContentRepository.h"
 #include "properties/Configure.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "RocksDatabase.h"
+#include "database/RocksDatabase.h"
 #include "core/ContentSession.h"
 
 namespace org {
@@ -35,36 +35,6 @@ namespace minifi {
 namespace core {
 namespace repository {
 
-class StringAppender : public rocksdb::AssociativeMergeOperator {
- public:
-  // Constructor: specify delimiter
-  explicit StringAppender() = default;
-
-  virtual bool Merge(const rocksdb::Slice& /*key*/, const rocksdb::Slice* existing_value, const rocksdb::Slice& value, std::string* new_value, rocksdb::Logger* /*logger*/) const {
-    // Clear the *new_value for writing.
-    if (nullptr == new_value) {
-      return false;
-    }
-    new_value->clear();
-
-    if (!existing_value) {
-      // No existing_value. Set *new_value = value
-      new_value->assign(value.data(), value.size());
-    } else {
-      new_value->reserve(existing_value->size() + value.size());
-      new_value->assign(existing_value->data(), existing_value->size());
-      new_value->append(value.data(), value.size());
-    }
-
-    return true;
-  }
-
-  virtual const char* Name() const {
-    return "StringAppender";
-  }
-
- private:
-};
 
 /**
  * DatabaseContentRepository is a content repository that stores data onto the local file system.
@@ -76,9 +46,11 @@ class DatabaseContentRepository : public core::ContentRepository, public core::C
 
     void commit() override;
   };
- public:
 
-  DatabaseContentRepository(const std::string& name = getClassName<DatabaseContentRepository>(), const utils::Identifier& uuid = {})
+ public:
+  static constexpr const char* ENCRYPTION_KEY_NAME = "nifi.database.content.repository.encryption.key";
+
+  explicit DatabaseContentRepository(const std::string& name = getClassName<DatabaseContentRepository>(), const utils::Identifier& uuid = {})
       : core::Connectable(name, uuid),
         is_valid_(false),
         db_(nullptr),
@@ -125,7 +97,7 @@ class DatabaseContentRepository : public core::ContentRepository, public core::C
   }
 
  private:
-  std::shared_ptr<io::BaseStream> write(const minifi::ResourceClaim &claim, bool append, rocksdb::WriteBatch* batch);
+  std::shared_ptr<io::BaseStream> write(const minifi::ResourceClaim &claim, bool append, minifi::internal::WriteBatch* batch);
 
   bool is_valid_;
   std::unique_ptr<minifi::internal::RocksDatabase> db_;
@@ -138,5 +110,3 @@ class DatabaseContentRepository : public core::ContentRepository, public core::C
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* LIBMINIFI_INCLUDE_CORE_REPOSITORY_DatabaseContentRepository_H_ */
