@@ -18,17 +18,21 @@
 #include "FlowFileRepository.h"
 
 #include <chrono>
+#include <list>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
-#include <list>
 
 #include "rocksdb/options.h"
 #include "rocksdb/write_batch.h"
 #include "rocksdb/slice.h"
 #include "FlowFileRecord.h"
 #include "utils/gsl.h"
+#include "core/Resource.h"
+
+using namespace std::literals::chrono_literals;
 
 namespace org {
 namespace apache {
@@ -42,7 +46,7 @@ void FlowFileRepository::flush() {
   if (!opendb) {
     return;
   }
-  rocksdb::WriteBatch batch;
+  auto batch = opendb->createWriteBatch();
   rocksdb::ReadOptions options;
 
   std::vector<std::shared_ptr<FlowFile>> purgeList;
@@ -58,7 +62,6 @@ void FlowFileRepository::flush() {
       keys.push_back(keystrings.back());
     }
   }
-
   auto multistatus = opendb->MultiGet(options, keys, &values);
 
   for (size_t i = 0; i < keys.size() && i < values.size() && i < multistatus.size(); ++i) {
@@ -69,7 +72,7 @@ void FlowFileRepository::flush() {
     }
 
     utils::Identifier containerId;
-    auto eventRead = FlowFileRecord::DeSerialize(reinterpret_cast<const uint8_t *>(values[i].data()), gsl::narrow<int>(values[i].size()), content_repo_, containerId);
+    auto eventRead = FlowFileRecord::DeSerialize(gsl::make_span(values[i]).as_span<const std::byte>(), content_repo_, containerId);
     if (eventRead) {
       purgeList.push_back(eventRead);
     }
@@ -118,7 +121,7 @@ void FlowFileRepository::run() {
     prune_stored_flowfiles();
   }
   while (running_) {
-    std::this_thread::sleep_for(std::chrono::milliseconds(purge_period_));
+    std::this_thread::sleep_for(purge_period_);
     flush();
     auto now = std::chrono::steady_clock::now();
     if ((now-last) > std::chrono::seconds(30)) {
@@ -130,15 +133,27 @@ void FlowFileRepository::run() {
 }
 
 void FlowFileRepository::prune_stored_flowfiles() {
-  rocksdb::Options options;
-  options.create_if_missing = true;
-  options.use_direct_io_for_flush_and_compaction = true;
-  options.use_direct_reads = true;
-  minifi::internal::RocksDatabase checkpointDB(options, checkpoint_dir_, minifi::internal::RocksDatabase::Mode::ReadOnly);
-  utils::optional<minifi::internal::OpenRocksDB> opendb;
+  const auto encrypted_env = createEncryptingEnv(utils::crypto::EncryptionManager{config_->getHome()}, DbEncryptionOptions{checkpoint_dir_, ENCRYPTION_KEY_NAME});
+  logger_->log_info("Using %s FlowFileRepository checkpoint", encrypted_env ? "encrypted" : "plaintext");
+
+  auto set_db_opts = [encrypted_env] (minifi::internal::Writable<rocksdb::DBOptions>& db_opts) {
+    db_opts.set(&rocksdb::DBOptions::create_if_missing, true);
+    db_opts.set(&rocksdb::DBOptions::use_direct_io_for_flush_and_compaction, true);
+    db_opts.set(&rocksdb::DBOptions::use_direct_reads, true);
+    if (encrypted_env) {
+      db_opts.set(&rocksdb::DBOptions::env, encrypted_env.get(), EncryptionEq{});
+    } else {
+      db_opts.set(&rocksdb::DBOptions::env, rocksdb::Env::Default());
+    }
+  };
+  auto checkpointDB = minifi::internal::RocksDatabase::create(set_db_opts, {}, checkpoint_dir_, minifi::internal::RocksDbMode::ReadOnly);
+  std::optional<minifi::internal::OpenRocksDb> opendb;
   if (nullptr != checkpoint_) {
-    opendb = checkpointDB.open();
-    if (!opendb) {
+    opendb = checkpointDB->open();
+    if (opendb) {
+      logger_->log_trace("Successfully opened checkpoint database at '%s'", checkpoint_dir_);
+    } else {
+      logger_->log_error("Couldn't open checkpoint database at '%s' using live database", checkpoint_dir_);
       opendb = db_->open();
     }
     if (!opendb) {
@@ -153,19 +168,19 @@ void FlowFileRepository::prune_stored_flowfiles() {
   auto it = opendb->NewIterator(rocksdb::ReadOptions());
   for (it->SeekToFirst(); it->Valid(); it->Next()) {
     utils::Identifier containerId;
-    auto eventRead = FlowFileRecord::DeSerialize(reinterpret_cast<const uint8_t *>(it->value().data()), gsl::narrow<int>(it->value().size()), content_repo_, containerId);
+    auto eventRead = FlowFileRecord::DeSerialize(gsl::make_span(it->value()).as_span<const std::byte>(), content_repo_, containerId);
     std::string key = it->key().ToString();
     if (eventRead) {
       // on behalf of the just resurrected persisted instance
       auto claim = eventRead->getResourceClaim();
       if (claim) claim->increaseFlowFileRecordOwnedCount();
       bool found = false;
-      auto search = containers.find(containerId.to_string());
-      found = (search != containers.end());
+      auto search = containers_.find(containerId.to_string());
+      found = (search != containers_.end());
       if (!found) {
         // for backward compatibility
-        search = connectionMap.find(containerId.to_string());
-        found = (search != connectionMap.end());
+        search = connection_map_.find(containerId.to_string());
+        found = (search != connection_map_.end());
       }
       if (found) {
         logger_->log_debug("Found connection for %s, path %s ", containerId.to_string(), eventRead->getContentFullPath());
@@ -185,7 +200,7 @@ void FlowFileRepository::prune_stored_flowfiles() {
 }
 
 bool FlowFileRepository::ExecuteWithRetry(std::function<rocksdb::Status()> operation) {
-  int waitTime = 0;
+  std::chrono::milliseconds waitTime = 0ms;
   for (int i=0; i < 3; ++i) {
     auto status = operation();
     if (status.ok()) {
@@ -194,7 +209,7 @@ bool FlowFileRepository::ExecuteWithRetry(std::function<rocksdb::Status()> opera
     }
     logger_->log_error("Rocksdb operation failed: %s", status.ToString());
     waitTime += FLOWFILE_REPOSITORY_RETRY_INTERVAL_INCREMENTS;
-    std::this_thread::sleep_for(std::chrono::milliseconds(waitTime));
+    std::this_thread::sleep_for(waitTime);
   }
   return false;
 }
@@ -203,7 +218,7 @@ bool FlowFileRepository::ExecuteWithRetry(std::function<rocksdb::Status()> opera
  * Returns True if there is data to interrogate.
  * @return true if our db has data stored.
  */
-bool FlowFileRepository::need_checkpoint(minifi::internal::OpenRocksDB& opendb) {
+bool FlowFileRepository::need_checkpoint(minifi::internal::OpenRocksDb& opendb) {
   auto it = opendb.NewIterator(rocksdb::ReadOptions());
   it->SeekToFirst();
   return it->Valid();
@@ -219,17 +234,24 @@ void FlowFileRepository::initialize_repository() {
     logger_->log_trace("Do not need checkpoint");
     return;
   }
-  rocksdb::Checkpoint *checkpoint;
   // delete any previous copy
-  if (utils::file::FileUtils::delete_dir(checkpoint_dir_) >= 0 && opendb->NewCheckpoint(&checkpoint).ok()) {
-    if (checkpoint->CreateCheckpoint(checkpoint_dir_).ok()) {
-      checkpoint_ = std::unique_ptr<rocksdb::Checkpoint>(checkpoint);
-      logger_->log_trace("Created checkpoint directory");
-    } else {
-      logger_->log_trace("Could not create checkpoint. Corrupt?");
-    }
-  } else
-    logger_->log_trace("Could not create checkpoint directory. Not properly deleted?");
+  if (utils::file::delete_dir(checkpoint_dir_) < 0) {
+    logger_->log_error("Could not delete existing checkpoint directory '%s'", checkpoint_dir_);
+    return;
+  }
+  std::unique_ptr<rocksdb::Checkpoint> checkpoint;
+  rocksdb::Status checkpoint_status = opendb->NewCheckpoint(checkpoint);
+  if (!checkpoint_status.ok()) {
+    logger_->log_error("Could not create checkpoint object: %s", checkpoint_status.ToString());
+    return;
+  }
+  checkpoint_status = checkpoint->CreateCheckpoint(checkpoint_dir_);
+  if (!checkpoint_status.ok()) {
+    logger_->log_error("Could not initialize checkpoint: %s", checkpoint_status.ToString());
+    return;
+  }
+  checkpoint_ = std::move(checkpoint);
+  logger_->log_trace("Created checkpoint in directory '%s'", checkpoint_dir_);
 }
 
 void FlowFileRepository::loadComponent(const std::shared_ptr<core::ContentRepository> &content_repo) {
@@ -239,6 +261,8 @@ void FlowFileRepository::loadComponent(const std::shared_ptr<core::ContentReposi
   initialize_repository();
 }
 
+REGISTER_INTERNAL_RESOURCE_AS(FlowFileRepository, ("FlowFileRepository", "flowfilerepository"));
+
 } /* namespace repository */
 } /* namespace core */
 } /* namespace minifi */
