@@ -14,8 +14,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_PROVENANCE_PROVENANCEREPOSITORY_H_
-#define LIBMINIFI_INCLUDE_PROVENANCE_PROVENANCEREPOSITORY_H_
+#pragma once
+
+#include <cinttypes>
+#include <vector>
+#include <string>
+#include <memory>
+#include <algorithm>
+#include <utility>
 
 #include "rocksdb/db.h"
 #include "rocksdb/options.h"
@@ -24,6 +30,7 @@
 #include "core/Core.h"
 #include "provenance/Provenance.h"
 #include "core/logging/LoggerConfiguration.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -32,10 +39,10 @@ namespace provenance {
 
 #define PROVENANCE_DIRECTORY "./provenance_repository"
 #define MAX_PROVENANCE_STORAGE_SIZE (10*1024*1024)  // 10M
-#define MAX_PROVENANCE_ENTRY_LIFE_TIME (60000)  // 1 minute
-#define PROVENANCE_PURGE_PERIOD (2500)  // 2500 msec
+constexpr auto MAX_PROVENANCE_ENTRY_LIFE_TIME = std::chrono::minutes(1);
+constexpr auto PROVENANCE_PURGE_PERIOD = std::chrono::milliseconds(2500);
 
-class ProvenanceRepository : public core::Repository, public std::enable_shared_from_this<ProvenanceRepository> {
+class ProvenanceRepository : public core::Repository {
  public:
   ProvenanceRepository(const std::string& name, const utils::Identifier& /*uuid*/)
       : ProvenanceRepository(name) {
@@ -44,31 +51,33 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
   /*!
    * Create a new provenance repository
    */
-  ProvenanceRepository(const std::string repo_name = "", std::string directory = PROVENANCE_DIRECTORY, int64_t maxPartitionMillis = MAX_PROVENANCE_ENTRY_LIFE_TIME, int64_t maxPartitionBytes =
-  MAX_PROVENANCE_STORAGE_SIZE,
-                       uint64_t purgePeriod = PROVENANCE_PURGE_PERIOD)
+  explicit ProvenanceRepository(const std::string& repo_name = "", std::string directory = PROVENANCE_DIRECTORY, std::chrono::milliseconds maxPartitionMillis = MAX_PROVENANCE_ENTRY_LIFE_TIME,
+      int64_t maxPartitionBytes = MAX_PROVENANCE_STORAGE_SIZE, std::chrono::milliseconds purgePeriod = PROVENANCE_PURGE_PERIOD)
       : core::SerializableComponent(repo_name),
-        Repository(repo_name.length() > 0 ? repo_name : core::getClassName<ProvenanceRepository>(), directory, maxPartitionMillis, maxPartitionBytes, purgePeriod),
-        logger_(logging::LoggerFactory<ProvenanceRepository>::getLogger()) {
-    db_ = NULL;
+        Repository(repo_name.length() > 0 ? repo_name : core::getClassName<ProvenanceRepository>(), directory, maxPartitionMillis, maxPartitionBytes, purgePeriod) {
+    db_ = nullptr;
+  }
+
+  ~ProvenanceRepository() override {
+    stop();
   }
 
   void printStats();
 
-  virtual bool isNoop() {
+  bool isNoop() override {
     return false;
   }
 
-  void start() {
+  void start() override {
     if (running_)
       return;
     running_ = true;
-    thread_ = std::thread(&ProvenanceRepository::run, shared_from_this());
+    thread_ = std::thread(&ProvenanceRepository::run, this);
     logger_->log_debug("%s Repository Monitor Thread Start", name_);
   }
 
   // initialize
-  virtual bool initialize(const std::shared_ptr<org::apache::nifi::minifi::Configure> &config) {
+  bool initialize(const std::shared_ptr<org::apache::nifi::minifi::Configure> &config) override {
     std::string value;
     if (config->get(Configure::nifi_provenance_repository_directory_default, value)) {
       directory_ = value;
@@ -79,11 +88,10 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
     }
     logger_->log_debug("MiNiFi Provenance Max Partition Bytes %d", max_partition_bytes_);
     if (config->get(Configure::nifi_provenance_repository_max_storage_time, value)) {
-      core::TimeUnit unit;
-      if (core::Property::StringToTime(value, max_partition_millis_, unit) && core::Property::ConvertTimeUnitToMS(max_partition_millis_, unit, max_partition_millis_)) {
-      }
+      if (auto max_partition = utils::timeutils::StringToDuration<std::chrono::milliseconds>(value))
+          max_partition_millis_ = *max_partition;
     }
-    logger_->log_debug("MiNiFi Provenance Max Storage Time: [%d] ms", max_partition_millis_);
+    logger_->log_debug("MiNiFi Provenance Max Storage Time: [%" PRId64 "] ms", int64_t{max_partition_millis_.count()});
     rocksdb::Options options;
     options.create_if_missing = true;
     options.use_direct_io_for_flush_and_compaction = true;
@@ -97,8 +105,8 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
 
     options.compaction_style = rocksdb::CompactionStyle::kCompactionStyleFIFO;
     options.compaction_options_fifo = rocksdb::CompactionOptionsFIFO(max_partition_bytes_, false);
-    if (max_partition_millis_ > 0) {
-      options.ttl = max_partition_millis_ / 1000;
+    if (max_partition_millis_ > std::chrono::milliseconds(0)) {
+      options.ttl = std::chrono::duration_cast<std::chrono::seconds>(max_partition_millis_).count();
     }
 
     logger_->log_info("Write buffer: %llu", options.write_buffer_size);
@@ -118,16 +126,17 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
     return true;
   }
   // Put
-  virtual bool Put(std::string key, const uint8_t *buf, size_t bufLen) {
+  bool Put(std::string key, const uint8_t *buf, size_t bufLen) override {
     // persist to the DB
     rocksdb::Slice value((const char *) buf, bufLen);
     return db_->Put(rocksdb::WriteOptions(), key, value).ok();
   }
 
-  virtual bool MultiPut(const std::vector<std::pair<std::string, std::unique_ptr<minifi::io::BufferStream>>>& data) {
+  bool MultiPut(const std::vector<std::pair<std::string, std::unique_ptr<minifi::io::BufferStream>>>& data) override {
     rocksdb::WriteBatch batch;
     for (const auto &item : data) {
-      rocksdb::Slice value((const char *) item.second->getBuffer(), item.second->size());
+      const auto buf = item.second->getBuffer().as_span<const char>();
+      rocksdb::Slice value(buf.data(), buf.size());
       if (!batch.Put(item.first, value).ok()) {
         return false;
       }
@@ -136,16 +145,16 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
   }
 
   // Delete
-  virtual bool Delete(std::string /*key*/) {
+  bool Delete(std::string /*key*/) override {
     // The repo is cleaned up by itself, there is no need to delete items.
     return true;
   }
   // Get
-  virtual bool Get(const std::string &key, std::string &value) {
+  bool Get(const std::string &key, std::string &value) override {
     return db_->Get(rocksdb::ReadOptions(), key, &value).ok();
   }
 
-  virtual bool Serialize(const std::string &key, const uint8_t *buffer, const size_t bufferSize) {
+  bool Serialize(const std::string &key, const uint8_t *buffer, const size_t bufferSize) override {
     return Put(key, buffer, bufferSize);
   }
 
@@ -156,14 +165,14 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
       std::string key = it->key().ToString();
       if (store.size() >= max_size)
         break;
-      if (eventRead->DeSerialize((uint8_t *) it->value().data(), (int) it->value().size())) {
+      if (eventRead->DeSerialize(gsl::make_span(it->value()).as_span<const std::byte>())) {
         store.push_back(std::dynamic_pointer_cast<core::CoreComponent>(eventRead));
       }
     }
     return true;
   }
 
-  virtual bool DeSerialize(std::vector<std::shared_ptr<core::SerializableComponent>> &records, size_t &max_size, std::function<std::shared_ptr<core::SerializableComponent>()> lambda) {
+  bool DeSerialize(std::vector<std::shared_ptr<core::SerializableComponent>> &records, size_t &max_size, std::function<std::shared_ptr<core::SerializableComponent>()> lambda) override {
     std::unique_ptr<rocksdb::Iterator> it(db_->NewIterator(rocksdb::ReadOptions()));
     size_t requested_batch = max_size;
     max_size = 0;
@@ -172,7 +181,7 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
         break;
       std::shared_ptr<core::SerializableComponent> eventRead = lambda();
       std::string key = it->key().ToString();
-      if (eventRead->DeSerialize((uint8_t *) it->value().data(), (int) it->value().size())) {
+      if (eventRead->DeSerialize(gsl::make_span(it->value()).as_span<const std::byte>())) {
         max_size++;
         records.push_back(eventRead);
       }
@@ -188,20 +197,20 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
       std::string key = it->key().ToString();
       if (records.size() >= (uint64_t)maxSize)
         break;
-      if (eventRead->DeSerialize((uint8_t *) it->value().data(), (int) it->value().size())) {
+      if (eventRead->DeSerialize(gsl::make_span(it->value()).as_span<const std::byte>())) {
         records.push_back(eventRead);
       }
     }
   }
 
-  virtual bool DeSerialize(std::vector<std::shared_ptr<core::SerializableComponent>> &store, size_t &max_size) {
+  bool DeSerialize(std::vector<std::shared_ptr<core::SerializableComponent>> &store, size_t &max_size) override {
     std::unique_ptr<rocksdb::Iterator> it(db_->NewIterator(rocksdb::ReadOptions()));
     max_size = 0;
     for (it->SeekToFirst(); it->Valid(); it->Next()) {
       std::shared_ptr<ProvenanceEventRecord> eventRead = std::make_shared<ProvenanceEventRecord>();
       std::string key = it->key().ToString();
 
-      if (store.at(max_size)->DeSerialize((uint8_t *) it->value().data(), (int) it->value().size())) {
+      if (store.at(max_size)->DeSerialize(gsl::make_span(it->value()).as_span<const std::byte>())) {
         max_size++;
       }
       if (store.size() >= max_size)
@@ -215,7 +224,7 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
     db_.reset();
   }
   // Run function for the thread
-  void run();
+  void run() override;
 
   uint64_t getKeyCount() const {
     std::string key_count;
@@ -231,7 +240,7 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
 
  private:
   std::unique_ptr<rocksdb::DB> db_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ProvenanceRepository>::getLogger();
 };
 
 } /* namespace provenance */
@@ -239,5 +248,3 @@ class ProvenanceRepository : public core::Repository, public std::enable_shared_
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* LIBMINIFI_INCLUDE_PROVENANCE_PROVENANCEREPOSITORY_H_ */
-
