@@ -15,11 +15,14 @@
  * limitations under the License.
  */
 
-#include "RocksDbPersistableKeyValueStoreService.h"
-
-#include "utils/StringUtils.h"
-
 #include <fstream>
+#include <set>
+
+#include "RocksDbPersistableKeyValueStoreService.h"
+#include "../encryption/RocksDbEncryptionProvider.h"
+#include "utils/StringUtils.h"
+#include "core/Resource.h"
+
 
 namespace org {
 namespace apache {
@@ -33,15 +36,12 @@ core::Property RocksDbPersistableKeyValueStoreService::Directory(
 
 RocksDbPersistableKeyValueStoreService::RocksDbPersistableKeyValueStoreService(const std::string& name, const utils::Identifier& uuid /*= utils::Identifier()*/)
     : PersistableKeyValueStoreService(name, uuid)
-    , AbstractAutoPersistingKeyValueStoreService(name, uuid)
-    , logger_(logging::LoggerFactory<RocksDbPersistableKeyValueStoreService>::getLogger()) {
+    , AbstractAutoPersistingKeyValueStoreService(name, uuid) {
 }
 
 void RocksDbPersistableKeyValueStoreService::initialize() {
   AbstractAutoPersistingKeyValueStoreService::initialize();
-  std::set<core::Property> supportedProperties;
-  supportedProperties.insert(Directory);
-  updateSupportedProperties(supportedProperties);
+  updateSupportedProperties({Directory});
 }
 
 void RocksDbPersistableKeyValueStoreService::onEnable() {
@@ -58,18 +58,26 @@ void RocksDbPersistableKeyValueStoreService::onEnable() {
   }
 
   db_.reset();
-  rocksdb::Options options;
-  options.create_if_missing = true;
-  options.use_direct_io_for_flush_and_compaction = true;
-  options.use_direct_reads = true;
+
+  const auto encrypted_env = createEncryptingEnv(utils::crypto::EncryptionManager{configuration_->getHome()}, core::repository::DbEncryptionOptions{directory_, ENCRYPTION_KEY_NAME});
+  logger_->log_info("Using %s RocksDbPersistableKeyValueStoreService", encrypted_env ? "encrypted" : "plaintext");
+
+  auto set_db_opts = [encrypted_env] (internal::Writable<rocksdb::DBOptions>& db_opts) {
+    db_opts.set(&rocksdb::DBOptions::create_if_missing, true);
+    db_opts.set(&rocksdb::DBOptions::use_direct_io_for_flush_and_compaction, true);
+    db_opts.set(&rocksdb::DBOptions::use_direct_reads, true);
+    if (encrypted_env) {
+      db_opts.set(&rocksdb::DBOptions::env, encrypted_env.get(), core::repository::EncryptionEq{});
+    } else {
+      db_opts.set(&rocksdb::DBOptions::env, rocksdb::Env::Default());
+    }
+  };
   // Use the same buffer settings as the FlowFileRepository
-  options.write_buffer_size = 8 << 20;
-  options.max_write_buffer_number = 20;
-  options.min_write_buffer_number_to_merge = 1;
-  if (!always_persist_) {
-    options.manual_wal_flush = true;
-  }
-  db_ = utils::make_unique<minifi::internal::RocksDatabase>(options, directory_);
+  auto set_cf_opts = [] (minifi::internal::Writable<rocksdb::ColumnFamilyOptions>& cf_opts) {
+    cf_opts.set(&rocksdb::ColumnFamilyOptions::write_buffer_size, 8ULL << 20U);
+    cf_opts.set<int>(&rocksdb::ColumnFamilyOptions::min_write_buffer_number_to_merge, 1);
+  };
+  db_ = minifi::internal::RocksDatabase::create(set_db_opts, set_cf_opts, directory_);
   if (db_->open()) {
     logger_->log_trace("Successfully opened RocksDB database at %s", directory_.c_str());
   } else {
@@ -117,7 +125,11 @@ bool RocksDbPersistableKeyValueStoreService::get(const std::string& key, std::st
   }
   rocksdb::Status status = opendb->Get(rocksdb::ReadOptions(), key, &value);
   if (!status.ok()) {
-    logger_->log_error("Failed to Get key %s from RocksDB database at %s, error: %s", key.c_str(), directory_.c_str(), status.getState());
+    if (status.getState() != nullptr) {
+      logger_->log_error("Failed to Get key %s from RocksDB database at %s, error: %s", key.c_str(), directory_.c_str(), status.getState());
+    } else {
+      logger_->log_warn("Failed to Get key %s from RocksDB database at %s (it may not have been initialized yet)", key.c_str(), directory_.c_str());
+    }
     return false;
   }
   return true;
@@ -207,6 +219,9 @@ bool RocksDbPersistableKeyValueStoreService::persist() {
   return opendb->FlushWAL(true /*sync*/).ok();
 }
 
+REGISTER_RESOURCE_AS(RocksDbPersistableKeyValueStoreService, "A key-value service implemented by RocksDB",
+                     ("RocksDbPersistableKeyValueStoreService", "rocksdbpersistablekeyvaluestoreservice"));
+
 } /* namespace controllers */
 } /* namespace minifi */
 } /* namespace nifi */
