@@ -15,10 +15,11 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_SCRIPTENGINE_H
-#define NIFI_MINIFI_CPP_SCRIPTENGINE_H
+#pragma once
 
 #include <string>
+#include <vector>
+#include <utility>
 
 namespace org {
 namespace apache {
@@ -42,7 +43,14 @@ class ScriptEngine {
    */
   virtual void evalFile(const std::string &file_name) = 0;
 
+  void setModulePaths(std::vector<std::string>&& module_paths) {
+    module_paths_ = std::move(module_paths);
+  }
+
   virtual ~ScriptEngine() = default;
+
+ protected:
+  std::vector<std::string> module_paths_;
 };
 
 } /* namespace script */
@@ -50,5 +58,3 @@ class ScriptEngine {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // NIFI_MINIFI_CPP_SCRIPTENGINE_H
