@@ -1,6 +1,6 @@
 /**
- * @file ExecuteScript.h
- * ExecuteScript class declaration
+ * @file ExecutePythonProcessor.h
+ * ExecutePythonProcessor class declaration
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -18,15 +18,13 @@
  * limitations under the License.
  */
 
-#ifndef EXTENSIONS_SCRIPT_PYTHON_EXECUTEPYTHONPROCESSOR_H_
-#define EXTENSIONS_SCRIPT_PYTHON_EXECUTEPYTHONPROCESSOR_H_
+#pragma once
 
 #include <memory>
 #include <string>
 #include <vector>
 
 #include "concurrentqueue.h"
-#include "core/Resource.h"
 #include "core/Processor.h"
 
 #include "../ScriptEngine.h"
@@ -47,18 +45,18 @@ class ExecutePythonProcessor : public core::Processor {
  public:
   explicit ExecutePythonProcessor(const std::string &name, const utils::Identifier &uuid = {})
       : Processor(name, uuid),
+        processor_initialized_(false),
         python_dynamic_(false),
-        valid_init_(false),
-        logger_(logging::LoggerFactory<ExecutePythonProcessor>::getLogger()),
-        script_engine_q_() {
+        reload_on_script_change_(true) {
   }
 
-  static core::Property ScriptFile;
-  static core::Property ScriptBody;
-  static core::Property ModuleDirectory;
+  EXTENSIONAPI static const core::Property ScriptFile;
+  EXTENSIONAPI static const core::Property ScriptBody;
+  EXTENSIONAPI static const core::Property ModuleDirectory;
+  EXTENSIONAPI static const core::Property ReloadOnScriptChange;
 
-  static core::Relationship Success;
-  static core::Relationship Failure;
+  EXTENSIONAPI static const core::Relationship Success;
+  EXTENSIONAPI static const core::Relationship Failure;
 
   void initialize() override;
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
@@ -93,50 +91,36 @@ class ExecutePythonProcessor : public core::Processor {
     return false;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
  private:
   std::vector<core::Property> python_properties_;
 
   std::string description_;
 
+  bool processor_initialized_;
   bool python_dynamic_;
 
-  bool valid_init_;
-
-  std::shared_ptr<logging::Logger> logger_;
-  std::shared_ptr<logging::Logger> python_logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ExecutePythonProcessor>::getLogger();
 
   std::string script_to_exec_;
-  std::string module_directory_;
+  bool reload_on_script_change_;
+  std::optional<std::filesystem::file_time_type> last_script_write_time_;
+  std::string script_file_path_;
+  std::shared_ptr<core::logging::Logger> python_logger_;
+  std::unique_ptr<PythonScriptEngine> python_script_engine_;
 
-  moodycamel::ConcurrentQueue<std::shared_ptr<python::PythonScriptEngine>> script_engine_q_;
-
-  std::shared_ptr<python::PythonScriptEngine> getScriptEngine();
-  void handleEngineNoLongerInUse(std::shared_ptr<python::PythonScriptEngine>&& engine);
   void appendPathForImportModules();
-  void loadScriptFromFile(const std::string& file_path);
+  void loadScriptFromFile();
   void loadScript();
   void reloadScriptIfUsingScriptFileProperty();
+  void initalizeThroughScriptEngine();
 
-
-  template<typename T>
-  std::shared_ptr<T> createEngine() const {
-    auto engine = std::make_shared<T>();
-
-    engine->bind("log", python_logger_);
-    engine->bind("REL_SUCCESS", Success);
-    engine->bind("REL_FAILURE", Failure);
-
-    return engine;
-  }
+  std::unique_ptr<PythonScriptEngine> createScriptEngine();
 };
 
-REGISTER_RESOURCE(
-    ExecutePythonProcessor, "Executes a script given the flow file and a process session. The script is responsible for handling the incoming flow file (transfer to SUCCESS or remove, e.g.) "
-    "as well as any flow files created by the script. If the handling is incomplete or incorrect, the session will be rolled back.Scripts must define an onTrigger function which accepts NiFi Context"
-    " and Property objects. For efficiency, scripts are executed once when the processor is run, then the onTrigger method is called for each incoming flowfile. This enables scripts to keep state "
-    "if they wish, although there will be a script context per concurrent task of the processor. In order to, e.g., compute an arithmetic sum based on incoming flow file information, set the "
-    "concurrent tasks to 1.");  // NOLINT
-
 } /* namespace processors */
 } /* namespace python */
 } /* namespace minifi */
@@ -145,5 +129,3 @@ REGISTER_RESOURCE(
 } /* namespace org */
 
 #pragma GCC visibility pop
-
-#endif  // EXTENSIONS_SCRIPT_PYTHON_EXECUTEPYTHONPROCESSOR_H_
