@@ -16,12 +16,12 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_PYBYTESTREAM_H
-#define NIFI_MINIFI_CPP_PYBYTESTREAM_H
+#pragma once
 
-#include <pybind11/embed.h>
+#include <memory>
 
-#include <io/BaseStream.h>
+#include "pybind11/embed.h"
+#include "io/BaseStream.h"
 
 namespace org {
 namespace apache {
@@ -37,7 +37,7 @@ class PyBaseStream {
 
   py::bytes read();
   py::bytes read(size_t len = 0);
-  size_t write(py::bytes buf);
+  size_t write(const py::bytes& buf);
 
  private:
   std::shared_ptr<io::BaseStream> stream_;
@@ -48,5 +48,3 @@ class PyBaseStream {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // NIFI_MINIFI_CPP_PYBYTESTREAM_H
