@@ -15,13 +15,16 @@
  * limitations under the License.
  */
 
-#ifndef NIFI_MINIFI_CPP_PYTHONSCRIPTENGINE_H
-#define NIFI_MINIFI_CPP_PYTHONSCRIPTENGINE_H
+#pragma once
 
 #include <mutex>
-#include <pybind11/embed.h>
-#include <core/ProcessSession.h>
-#include <core/Processor.h>
+#include <memory>
+#include <string>
+#include <utility>
+
+#include "pybind11/embed.h"
+#include "core/ProcessSession.h"
+#include "core/Processor.h"
 
 #include "../ScriptEngine.h"
 #include "../ScriptProcessContext.h"
@@ -54,7 +57,7 @@ struct Interpreter {
 
 Interpreter *getInterpreter();
 
-class PythonScriptEngine : public script::ScriptEngine {
+class __attribute__((visibility("default"))) PythonScriptEngine : public script::ScriptEngine {
  public:
   PythonScriptEngine();
   virtual ~PythonScriptEngine() {
@@ -147,7 +150,7 @@ class PythonScriptEngine : public script::ScriptEngine {
 
   class TriggerSchedule {
    public:
-    TriggerSchedule(std::shared_ptr<script::ScriptProcessContext> script_context)
+    explicit TriggerSchedule(std::shared_ptr<script::ScriptProcessContext> script_context)
         : script_context_(script_context) {
     }
 
@@ -169,13 +172,13 @@ class PythonScriptEngine : public script::ScriptEngine {
     std::shared_ptr<script::ScriptProcessContext> script_context_;
   };
 
-  void onInitialize(const std::shared_ptr<core::Processor> &proc) {
+  void onInitialize(core::Processor* proc) {
     TriggerInit trigger_session;
     auto newproc = convertProcessor(proc);
     call("onInitialize", newproc);
   }
 
-  void describe(const std::shared_ptr<core::Processor> &proc) {
+  void describe(core::Processor* proc) {
     TriggerInit trigger_session;
     auto newproc = convertProcessor(proc);
     callRequiredFunction("describe", newproc);
@@ -220,11 +223,13 @@ class PythonScriptEngine : public script::ScriptEngine {
     return std::make_shared<script::ScriptProcessContext>(context);
   }
 
-  std::shared_ptr<python::PythonProcessor> convertProcessor(const std::shared_ptr<core::Processor> &proc) {
+  std::shared_ptr<python::PythonProcessor> convertProcessor(core::Processor* proc) {
     return std::make_shared<python::PythonProcessor>(proc);
   }
 
  private:
+  void evaluateModuleImports();
+
   std::unique_ptr<py::dict> bindings_;
 };
 
@@ -235,5 +240,3 @@ class PythonScriptEngine : public script::ScriptEngine {
 } /* namespace org */
 
 #pragma GCC visibility pop
-
-#endif  // NIFI_MINIFI_CPP_PYTHONSCRIPTENGINE_H
