@@ -15,15 +15,25 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include "core/FlowConfiguration.h"
-#include "PyProcLoader.h"
+#include "core/extension/Extension.h"
+#include "PythonCreator.h"
+#include "PythonBindings.h"
 
-bool PyProcFactory::added = core::FlowConfiguration::add_static_func("createPyProcFactory");
+namespace minifi = org::apache::nifi::minifi;
 
-extern "C" {
-
-void *createPyProcFactory(void) {
-  return new PyProcFactory();
+static minifi::python::PythonCreator& getPythonCreator() {
+  static minifi::python::PythonCreator instance("PythonCreator");
+  return instance;
 }
 
+static bool init(const std::shared_ptr<minifi::Configure>& config) {
+  getPythonCreator().configure(config);
+  return true;
 }
+
+static void deinit() {
+  // TODO(adebreceni): deinitialization is not implemented
+}
+
+REGISTER_EXTENSION("PythonExtension", init, deinit);
+
