@@ -34,6 +34,7 @@
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "GetEnvironmentalSensors.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
@@ -67,14 +68,14 @@ void GetEnvironmentalSensors::initialize() {
 void GetEnvironmentalSensors::onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) {
   SensorBase::onSchedule(context, sessionFactory);
 
-  humidity_sensor_ = RTHumidity::createHumidity(&settings);
+  humidity_sensor_ = RTHumidity::createHumidity(&settings_.value());
   if (humidity_sensor_) {
     humidity_sensor_->humidityInit();
   } else {
     throw std::runtime_error("RTHumidity could not be initialized");
   }
 
-  pressure_sensor_ = RTPressure::createPressure(&settings);
+  pressure_sensor_ = RTPressure::createPressure(&settings_.value());
   if (pressure_sensor_) {
     pressure_sensor_->pressureInit();
   } else {
@@ -92,8 +93,8 @@ GetEnvironmentalSensors::~GetEnvironmentalSensors() = default;
 void GetEnvironmentalSensors::onTrigger(const std::shared_ptr<core::ProcessContext>& /*context*/, const std::shared_ptr<core::ProcessSession>& session) {
   auto flow_file_ = session->create();
 
-  if (imu->IMURead()) {
-    RTIMU_DATA imuData = imu->getIMUData();
+  if (imu_->IMURead()) {
+    RTIMU_DATA imuData = imu_->getIMUData();
     auto vector = imuData.accel;
     std::string degrees = RTMath::displayDegrees("acceleration", vector);
     flow_file_->addAttribute("ACCELERATION", degrees);
@@ -136,12 +137,13 @@ void GetEnvironmentalSensors::onTrigger(const std::shared_ptr<core::ProcessConte
   }
 
   if (have_sensor) {
-    WriteCallback callback("GetEnvironmentalSensors");
-    session->write(flow_file_, &callback);
+    session->writeBuffer(flow_file_, "GetEnvironmentalSensors");
     session->transfer(flow_file_, Success);
   }
 }
 
+REGISTER_RESOURCE(GetEnvironmentalSensors, "Provides sensor information from known sensors to include humidity and pressure data");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
