@@ -15,13 +15,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_SENSORS_GETENVIRONMENTALSENSORS_H_
-#define EXTENSIONS_SENSORS_GETENVIRONMENTALSENSORS_H_
-
-
+#pragma once
 
 #include <memory>
 #include <regex>
+#include <string>
 
 #include "utils/ByteArrayCallback.h"
 #include "FlowFileRecord.h"
@@ -29,7 +27,6 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "utils/Id.h"
 #include "RTIMULib.h"
 #include "SensorBase.h"
@@ -44,16 +41,12 @@ namespace processors {
 // EnvironmentalSensors Class
 class GetEnvironmentalSensors : public SensorBase {
  public:
-
   // Constructor
   /*!
    * Create a new processor
    */
-  GetEnvironmentalSensors(const std::string& name, const utils::Identifier& uuid = {})
-      : SensorBase(name, uuid),
-        humidity_sensor_(nullptr),
-        pressure_sensor_(nullptr),
-        logger_(logging::LoggerFactory<GetEnvironmentalSensors>::getLogger()) {
+  explicit GetEnvironmentalSensors(const std::string& name, const utils::Identifier& uuid = {})
+      : SensorBase(name, uuid) {
   }
   // Destructor
   ~GetEnvironmentalSensors() override;
@@ -68,18 +61,16 @@ class GetEnvironmentalSensors : public SensorBase {
 
  protected:
   void notifyStop() override;
+
  private:
-  RTHumidity *humidity_sensor_;
-  RTPressure *pressure_sensor_;
-  std::shared_ptr<logging::Logger> logger_;
+  RTHumidity *humidity_sensor_ = nullptr;
+  RTPressure *pressure_sensor_ = nullptr;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GetEnvironmentalSensors>::getLogger();
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 };
 
-REGISTER_RESOURCE(GetEnvironmentalSensors, "Provides sensor information from known sensors to include humidity and pressure data");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* EXTENSIONS_SENSORS_GETENVIRONMENTALSENSORS_H_ */
