@@ -33,6 +33,7 @@
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "GetMovementSensors.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
@@ -66,8 +67,8 @@ GetMovementSensors::~GetMovementSensors() = default;
 void GetMovementSensors::onTrigger(const std::shared_ptr<core::ProcessContext>& /*context*/, const std::shared_ptr<core::ProcessSession>& session) {
   auto flow_file_ = session->create();
 
-  if (imu->IMURead()) {
-    RTIMU_DATA imuData = imu->getIMUData();
+  if (imu_->IMURead()) {
+    RTIMU_DATA imuData = imu_->getIMUData();
 
     if (imuData.accelValid) {
       auto vector = imuData.accel;
@@ -84,11 +85,13 @@ void GetMovementSensors::onTrigger(const std::shared_ptr<core::ProcessContext>&
       logger_->log_trace("Could not read gyroscope");
     }
 
-    WriteCallback callback("GetMovementSensors");
-    session->write(flow_file_, &callback);
+    session->writeBuffer(flow_file_, "GetMovementSensors");
     session->transfer(flow_file_, Success);
   }
 }
+
+REGISTER_RESOURCE(GetMovementSensors, "Defines a processor that is able to retrieve sensor information from a class of known servo sensors");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
