@@ -15,13 +15,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_SENSORS_GETMOVEMENT_H_
-#define EXTENSIONS_SENSORS_GETMOVEMENT_H_
-
-
+#pragma once
 
 #include <memory>
 #include <regex>
+#include <string>
 
 #include "utils/ByteArrayCallback.h"
 #include "FlowFileRecord.h"
@@ -29,7 +27,6 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "utils/Id.h"
 #include "SensorBase.h"
 #include "RTIMULib.h"
@@ -44,14 +41,12 @@ namespace processors {
 // GetMovementSensors Class
 class GetMovementSensors : public SensorBase {
  public:
-
   // Constructor
   /*!
    * Create a new processor
    */
-  GetMovementSensors(const std::string& name, const utils::Identifier& uuid = {})
-      : SensorBase(name, uuid),
-        logger_(logging::LoggerFactory<GetMovementSensors>::getLogger()) {
+  explicit GetMovementSensors(const std::string& name, const utils::Identifier& uuid = {})
+      : SensorBase(name, uuid) {
   }
   // Destructor
   virtual ~GetMovementSensors();
@@ -64,14 +59,11 @@ class GetMovementSensors : public SensorBase {
   void initialize() override;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GetMovementSensors>::getLogger();
 };
 
-REGISTER_RESOURCE(GetMovementSensors, "Defines a processor that is able to retrieve sensor information from a class of known servo sensors");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* EXTENSIONS_SENSORS_GETMOVEMENT_H_ */
