@@ -16,28 +16,11 @@
  * limitations under the License.
  */
 #include <memory>
-#include <algorithm>
-#include <cctype>
-#include <cstdint>
-#include <cstring>
-#include <iostream>
-#include <iterator>
-#include <map>
 #include <set>
-#include <string>
-#include <utility>
-#include <vector>
 
-#include "utils/ByteArrayCallback.h"
-#include "core/FlowFile.h"
-#include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
 #include "SensorBase.h"
-#include "io/BufferStream.h"
-#include "io/StreamFactory.h"
-#include "ResourceClaim.h"
-#include "utils/StringUtils.h"
 
 namespace org {
 namespace apache {
@@ -51,11 +34,15 @@ void SensorBase::initialize() {
 }
 
 void SensorBase::onSchedule(const std::shared_ptr<core::ProcessContext>& /*context*/, const std::shared_ptr<core::ProcessSessionFactory>& /*sessionFactory*/) {
-  imu = std::unique_ptr<RTIMU>(RTIMU::createIMU(&settings));
-  if (imu) {
-    imu->IMUInit();
-    imu->setGyroEnable(true);
-    imu->setAccelEnable(true);
+  // Deferred instantiation of RTIMUSettings, because it can create a file "RTIMULib.ini" in the working directory.
+  // SensorBase is instantiated when creating the manifest.
+  settings_ = std::make_optional<RTIMUSettings>();
+
+  imu_ = std::unique_ptr<RTIMU>(RTIMU::createIMU(&settings_.value()));
+  if (imu_) {
+    imu_->IMUInit();
+    imu_->setGyroEnable(true);
+    imu_->setAccelEnable(true);
   } else {
     throw std::runtime_error("RTIMU could not be initialized");
   }
