@@ -15,22 +15,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef EXTENSIONS_SENSORS_SENSORBASE_H_
-#define EXTENSIONS_SENSORS_SENSORBASE_H_
-
-
+#pragma once
 
 #include <memory>
-#include <regex>
+#include <string>
+#include <utility>
 
-#include "utils/ByteArrayCallback.h"
-#include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
-#include "core/Property.h"
-#include "core/Resource.h"
-#include "utils/Id.h"
+#include "io/StreamPipe.h"
 #include "RTIMULib.h"
 #include "RTMath.h"
 
@@ -40,48 +33,22 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-// SensorBase Class
 class SensorBase : public core::Processor {
  public:
-
-  // Constructor
-  /*!
-   * Create a new processor
-   */
-  SensorBase(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(name, uuid),
-        imu(nullptr),
-        logger_(logging::LoggerFactory<SensorBase>::getLogger()) {
+  explicit SensorBase(const std::string& name, const utils::Identifier& uuid = {})
+    : Processor(name, uuid) {
   }
-  // Destructor
-  virtual ~SensorBase();
-  // Processor Name
+  ~SensorBase() override;
   static core::Relationship Success;
-  // Supported Properties
 
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
   void initialize() override;
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
 
-  class WriteCallback : public OutputStreamCallback {
-     public:
-      WriteCallback(std::string data)
-          : _data(const_cast<char*>(data.data())),
-            _dataSize(data.size()) {
-      }
-      char *_data;
-      uint64_t _dataSize;
-      int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-        int64_t ret = 0;
-        if (_data && _dataSize > 0)
-          ret = stream->write(reinterpret_cast<uint8_t*>(_data), _dataSize);
-        return ret;
-      }
-    };
  protected:
-  RTIMUSettings settings;
-  std::unique_ptr<RTIMU> imu;
-  std::shared_ptr<logging::Logger> logger_;
+  std::optional<RTIMUSettings> settings_;
+  std::unique_ptr<RTIMU> imu_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<SensorBase>::getLogger();
 };
 
 } /* namespace processors */
@@ -89,4 +56,3 @@ class SensorBase : public core::Processor {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-#endif /* EXTENSIONS_SENSORS_SENSORBASE_H_ */
