@@ -16,15 +16,18 @@
  * limitations under the License.
  */
 #include "SFTPClient.h"
-#include <memory>
-#include <set>
-#include <vector>
-#include <string>
+#include <algorithm>
 #include <exception>
+#include <memory>
+#include <optional>
+#include <set>
 #include <sstream>
-#include <iomanip>
-#include "utils/StringUtils.h"
+#include <string>
+#include <tuple>
+#include <utility>
+#include <vector>
 
+#include "utils/StringUtils.h"
 #include "utils/gsl.h"
 
 namespace org {
@@ -35,7 +38,7 @@ namespace utils {
 
 #define SFTP_ERROR(CODE) case CODE: \
                           return #CODE
-static const char* sftp_strerror(unsigned long err) {
+static const char* sftp_strerror(unsigned long err) {  // NOLINT(runtime/int) unsigned long comes from libssh2 API
   switch (err) {
     SFTP_ERROR(LIBSSH2_FX_OK);
     SFTP_ERROR(LIBSSH2_FX_EOF);
@@ -64,7 +67,7 @@ static const char* sftp_strerror(unsigned long err) {
   }
 }
 
-static SFTPError libssh2_sftp_error_to_sftp_error(unsigned long libssh2_sftp_error) {
+static SFTPError libssh2_sftp_error_to_sftp_error(unsigned long libssh2_sftp_error) {  // NOLINT(runtime/int) unsigned long comes from libssh2 API
   switch (libssh2_sftp_error) {
     case LIBSSH2_FX_OK:
       return SFTPError::Ok;
@@ -106,7 +109,7 @@ LastSFTPError::LastSFTPError()
     , sftp_error_(SFTPError::Ok) {
 }
 
-LastSFTPError& LastSFTPError::setLibssh2Error(unsigned long libssh2_sftp_error) {
+LastSFTPError& LastSFTPError::setLibssh2Error(unsigned long libssh2_sftp_error) {  // NOLINT(runtime/int) unsigned long comes from libssh2 API
   sftp_error_set_ = false;
   libssh2_sftp_error_ = libssh2_sftp_error;
   return *this;
@@ -118,7 +121,7 @@ LastSFTPError& LastSFTPError::setSftpError(const SFTPError& sftp_error) {
   return *this;
 }
 
-LastSFTPError::operator unsigned long() const {
+LastSFTPError::operator unsigned long() const {  // NOLINT(runtime/int) unsigned long comes from libssh2 API
   if (sftp_error_set_) {
     return LIBSSH2_FX_OK;
   } else {
@@ -136,7 +139,7 @@ LastSFTPError::operator SFTPError() const {
 
 
 SFTPClient::SFTPClient(const std::string &hostname, uint16_t port, const std::string& username)
-    : logger_(logging::LoggerFactory<SFTPClient>::getLogger()),
+    : logger_(core::logging::LoggerFactory<SFTPClient>::getLogger()),
       hostname_(hostname),
       port_(port),
       username_(username),
@@ -229,13 +232,12 @@ bool SFTPClient::setProxy(ProxyType type, const utils::HTTPProxy& proxy) {
   return true;
 }
 
-bool SFTPClient::setConnectionTimeout(int64_t timeout) {
-  return curl_easy_setopt(easy_, CURLOPT_CONNECTTIMEOUT_MS, timeout) == CURLE_OK;
+bool SFTPClient::setConnectionTimeout(std::chrono::milliseconds timeout) {
+  return curl_easy_setopt(easy_, CURLOPT_CONNECTTIMEOUT_MS, timeout.count()) == CURLE_OK;
 }
 
-void SFTPClient::setDataTimeout(int64_t timeout) {
-  data_timeout_ = timeout;
-  libssh2_session_set_timeout(ssh_session_, timeout);
+void SFTPClient::setDataTimeout(std::chrono::milliseconds timeout) {
+  libssh2_session_set_timeout(ssh_session_, timeout.count());
 }
 
 void SFTPClient::setSendKeepAlive(bool send_keepalive) {
@@ -289,7 +291,7 @@ bool SFTPClient::connect() {
   /* Only CURLINFO_ACTIVESOCKET works on Win64 */
   curl_res = curl_easy_getinfo(easy_, CURLINFO_ACTIVESOCKET, &sockfd);
 #else
-  long sockfd;
+  long sockfd;  // NOLINT(runtime/int) long due to libcurl API
   /* Some older cURL versions only support CURLINFO_LASTSOCKET */
   curl_res = curl_easy_getinfo(easy_, CURLINFO_LASTSOCKET, &sockfd);
 #endif
@@ -307,17 +309,23 @@ bool SFTPClient::connect() {
 
   /* Checking remote host */
   if (ssh_known_hosts_ != nullptr) {
-    size_t hostkey_len = 0U;
-    int type = LIBSSH2_HOSTKEY_TYPE_UNKNOWN;
-    const char *hostkey = libssh2_session_hostkey(ssh_session_, &hostkey_len, &type);
-    if (hostkey == nullptr) {
-      char *err_msg = nullptr;
-      libssh2_session_last_error(ssh_session_, &err_msg, nullptr, 0);
-      logger_->log_info("Failed to get session hostkey, error: %s", err_msg);
-      return false;
-    }
+    auto hostkey_opt = [this]() -> std::optional<std::tuple<std::string_view, int>> {
+      size_t hostkey_len = 0U;
+      int type = LIBSSH2_HOSTKEY_TYPE_UNKNOWN;
+      const char *hostkey_ptr = libssh2_session_hostkey(ssh_session_, &hostkey_len, &type);
+      if (hostkey_ptr == nullptr) {
+        char *err_msg = nullptr;
+        libssh2_session_last_error(ssh_session_, &err_msg, nullptr, 0);
+        logger_->log_info("Failed to get session hostkey, error: %s", err_msg);
+        return std::nullopt;
+      }
+      const auto hostkey = std::string_view{hostkey_ptr, hostkey_len};
+      return std::make_optional(std::make_tuple(hostkey, type));
+    }();
+    if (!hostkey_opt) return false;
+    const auto [hostkey, hostkey_type] = *std::move(hostkey_opt);
     int keybit = 0;
-    switch (type) {
+    switch (hostkey_type) {
       case LIBSSH2_HOSTKEY_TYPE_RSA:
         keybit = LIBSSH2_KNOWNHOST_KEY_SSHRSA;
         break;
@@ -325,14 +333,14 @@ bool SFTPClient::connect() {
         keybit = LIBSSH2_KNOWNHOST_KEY_SSHDSS;
         break;
       default:
-        logger_->log_error("Unknown host key type: %d", type);
+        logger_->log_error("Unknown host key type: %d", hostkey_type);
         return false;
     }
     struct libssh2_knownhost* known_host = nullptr;
     int keycheck_result = libssh2_knownhost_checkp(ssh_known_hosts_,
                             hostname_.c_str(),
                             -1 /*port*/,
-                            hostkey, hostkey_len,
+                            hostkey.data(), hostkey.size(),
                             LIBSSH2_KNOWNHOST_TYPE_PLAIN |
                             LIBSSH2_KNOWNHOST_KEYENC_RAW |
                             keybit,
@@ -346,7 +354,7 @@ bool SFTPClient::connect() {
         logger_->log_warn("Host %s not found in the host key file", hostname_.c_str());
         break;
       case LIBSSH2_KNOWNHOST_CHECK_MISMATCH: {
-        auto hostkey_b64 = utils::StringUtils::to_base64(reinterpret_cast<const uint8_t*>(hostkey), hostkey_len);
+        auto hostkey_b64 = utils::StringUtils::to_base64(hostkey);
         logger_->log_warn("Host key mismatch for %s, expected: %s, actual: %s", hostname_.c_str(),
                           known_host == nullptr ? "" : known_host->key, hostkey_b64.c_str());
         break;
@@ -364,10 +372,11 @@ bool SFTPClient::connect() {
     }
   } else {
     const char* fingerprint = libssh2_hostkey_hash(ssh_session_, LIBSSH2_HOSTKEY_HASH_SHA1);
+    const auto fingerprint_span = gsl::make_span(fingerprint, 20);
     if (fingerprint == nullptr) {
       logger_->log_warn("Cannot get remote server fingerprint");
     } else {
-      auto fingerprint_hex = utils::StringUtils::to_hex(reinterpret_cast<const uint8_t*>(fingerprint), 20);
+      auto fingerprint_hex = utils::StringUtils::to_hex(fingerprint_span.as_span<const std::byte>());
       std::stringstream fingerprint_hex_colon;
       for (size_t i = 0; i < 20; i++) {
         fingerprint_hex_colon << fingerprint_hex.substr(i * 2, 2);
@@ -513,16 +522,16 @@ bool SFTPClient::getFile(const std::string& path, io::BaseStream& output, int64_
       break;
     }
     logger_->log_trace("Read %d bytes from remote file \"%s\"", read_ret, path.c_str());
-    total_read += read_ret;
-    int remaining = read_ret;
+    total_read += gsl::narrow<uint64_t>(read_ret);
+    auto remaining = read_ret;
     while (remaining > 0) {
-      int write_ret = output.write(buf.data() + (read_ret - remaining), remaining);
-      if (write_ret < 0) {
+      const auto write_ret = output.write(buf.data() + (read_ret - remaining), gsl::narrow<size_t>(remaining));
+      if (io::isError(write_ret)) {
         last_error_.setLibssh2Error(LIBSSH2_FX_OK);
         logger_->log_error("Failed to write output");
         return false;
       }
-      remaining -= write_ret;
+      remaining -= gsl::narrow<decltype(remaining)>(write_ret);
     }
   } while (true);
 
@@ -562,12 +571,12 @@ bool SFTPClient::putFile(const std::string& path, io::BaseStream& input, bool ov
     return true;
   }
 
-  const size_t buf_size = expected_size < 0 ? MAX_BUFFER_SIZE : std::min<size_t>(expected_size, MAX_BUFFER_SIZE);
-  std::vector<uint8_t> buf(buf_size);
+  const size_t buf_size = expected_size < 0 ? MAX_BUFFER_SIZE : std::min(gsl::narrow<size_t>(expected_size), MAX_BUFFER_SIZE);
+  std::vector<std::byte> buf(buf_size);
   uint64_t total_read = 0U;
   do {
-    int read_ret = input.read(buf.data(), buf.size());
-    if (read_ret < 0) {
+    const auto read_ret = input.read(buf);
+    if (io::isError(read_ret)) {
       last_error_.setLibssh2Error(LIBSSH2_FX_OK);
       logger_->log_error("Error while reading input");
       return false;
@@ -577,20 +586,20 @@ bool SFTPClient::putFile(const std::string& path, io::BaseStream& input, bool ov
     }
     logger_->log_trace("Read %d bytes", read_ret);
     total_read += read_ret;
-    ssize_t remaining = read_ret;
+    auto remaining = read_ret;
     while (remaining > 0) {
-      int write_ret = libssh2_sftp_write(file_handle, reinterpret_cast<char*>(buf.data() + (read_ret - remaining)), remaining);
+      const auto write_ret = libssh2_sftp_write(file_handle, reinterpret_cast<char*>(buf.data() + (read_ret - remaining)), remaining);
       if (write_ret < 0) {
         last_error_.setSftpError(SFTPError::IoError);
         logger_->log_error("Failed to write remote file \"%s\"", path.c_str());
         return false;
       }
       logger_->log_trace("Wrote %d bytes to remote file \"%s\"", write_ret, path.c_str());
-      remaining -= write_ret;
+      remaining -= gsl::narrow<size_t>(write_ret);
     }
   } while (true);
 
-  if (expected_size >= 0 && total_read != gsl::narrow<uint64_t>(expected_size)) {
+  if (expected_size >= 0 && total_read != gsl::narrow<size_t>(expected_size)) {
     last_error_.setLibssh2Error(LIBSSH2_FX_OK);
     logger_->log_error("Input has unexpected size, expected: %ld, actual: %lu", path.c_str(), expected_size, total_read);
     return false;
