@@ -34,6 +34,7 @@
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
 #include "ResourceClaim.h"
@@ -46,41 +47,50 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-core::Property FetchSFTP::RemoteFile(
-    core::PropertyBuilder::createProperty("Remote File")->withDescription("The fully qualified filename on the remote system")
-        ->isRequired(true)->supportsExpressionLanguage(true)->build());
-core::Property FetchSFTP::CompletionStrategy(
-    core::PropertyBuilder::createProperty("Completion Strategy")->withDescription("Specifies what to do with the original file on the server once it has been pulled into NiFi. If the Completion Strategy fails, a warning will be logged but the data will still be transferred.")
-        ->isRequired(true)
-        ->withAllowableValues<std::string>({COMPLETION_STRATEGY_NONE,
-                                            COMPLETION_STRATEGY_MOVE_FILE,
-                                            COMPLETION_STRATEGY_DELETE_FILE})
-        ->withDefaultValue(COMPLETION_STRATEGY_NONE)->build());
-core::Property FetchSFTP::MoveDestinationDirectory(
-    core::PropertyBuilder::createProperty("Move Destination Directory")->withDescription("The directory on the remote server to move the original file to once it has been ingested into NiFi. "
-                                                                                         "This property is ignored unless the Completion Strategy is set to 'Move File'. "
-                                                                                         "The specified directory must already exist on the remote system if 'Create Directory' is disabled, or the rename will fail.")
-        ->isRequired(false)->supportsExpressionLanguage(true)->build());
-core::Property FetchSFTP::CreateDirectory(
-    core::PropertyBuilder::createProperty("Create Directory")->withDescription("Specifies whether or not the remote directory should be created if it does not exist.")
-        ->isRequired(true)->withDefaultValue<bool>(false)->build());
-core::Property FetchSFTP::DisableDirectoryListing(
-    core::PropertyBuilder::createProperty("Disable Directory Listing")->withDescription("Control how 'Move Destination Directory' is created when 'Completion Strategy' is 'Move File' and 'Create Directory' is enabled. "
-                                                                                        "If set to 'true', directory listing is not performed prior to create missing directories. "
-                                                                                        "By default, this processor executes a directory listing command to see target directory existence before creating missing directories. "
-                                                                                        "However, there are situations that you might need to disable the directory listing such as the following. "
-                                                                                        "Directory listing might fail with some permission setups (e.g. chmod 100) on a directory. "
-                                                                                        "Also, if any other SFTP client created the directory after this processor performed a listing and before a directory creation request by this processor is finished, "
-                                                                                        "then an error is returned because the directory already exists.")
-        ->isRequired(false)->withDefaultValue<bool>(false)->build());
-core::Property FetchSFTP::UseCompression(
-    core::PropertyBuilder::createProperty("Use Compression")->withDescription("Indicates whether or not ZLIB compression should be used when transferring files")
-        ->isRequired(true)->withDefaultValue<bool>(false)->build());
+core::Property FetchSFTP::RemoteFile(core::PropertyBuilder::createProperty("Remote File")
+    ->withDescription("The fully qualified filename on the remote system")
+    ->isRequired(true)->supportsExpressionLanguage(true)->build());
 
-core::Relationship FetchSFTP::Success("success", "All FlowFiles that are received are routed to success");
-core::Relationship FetchSFTP::CommsFailure("comms.failure", "Any FlowFile that could not be fetched from the remote server due to a communications failure will be transferred to this Relationship.");
-core::Relationship FetchSFTP::NotFound("not.found", "Any FlowFile for which we receive a 'Not Found' message from the remote server will be transferred to this Relationship.");
-core::Relationship FetchSFTP::PermissionDenied("permission.denied", "Any FlowFile that could not be fetched from the remote server due to insufficient permissions will be transferred to this Relationship.");
+core::Property FetchSFTP::CompletionStrategy(
+    core::PropertyBuilder::createProperty("Completion Strategy")
+    ->withDescription("Specifies what to do with the original file on the server once it has been pulled into NiFi. "
+                      "If the Completion Strategy fails, a warning will be logged but the data will still be transferred.")
+    ->isRequired(true)
+    ->withAllowableValues<std::string>({COMPLETION_STRATEGY_NONE, COMPLETION_STRATEGY_MOVE_FILE, COMPLETION_STRATEGY_DELETE_FILE})
+    ->withDefaultValue(COMPLETION_STRATEGY_NONE)->build());
+
+core::Property FetchSFTP::MoveDestinationDirectory(core::PropertyBuilder::createProperty("Move Destination Directory")
+    ->withDescription("The directory on the remote server to move the original file to once it has been ingested into NiFi. "
+                      "This property is ignored unless the Completion Strategy is set to 'Move File'. "
+                      "The specified directory must already exist on the remote system if 'Create Directory' is disabled, or the rename will fail.")
+    ->isRequired(false)->supportsExpressionLanguage(true)->build());
+
+core::Property FetchSFTP::CreateDirectory(core::PropertyBuilder::createProperty("Create Directory")
+    ->withDescription("Specifies whether or not the remote directory should be created if it does not exist.")
+    ->isRequired(true)->withDefaultValue<bool>(false)->build());
+
+core::Property FetchSFTP::DisableDirectoryListing(core::PropertyBuilder::createProperty("Disable Directory Listing")
+    ->withDescription("Control how 'Move Destination Directory' is created when 'Completion Strategy' is 'Move File' and 'Create Directory' is enabled. "
+                      "If set to 'true', directory listing is not performed prior to create missing directories. "
+                      "By default, this processor executes a directory listing command to see target directory existence before creating missing directories. "
+                      "However, there are situations that you might need to disable the directory listing such as the following. "
+                      "Directory listing might fail with some permission setups (e.g. chmod 100) on a directory. "
+                      "Also, if any other SFTP client created the directory after this processor performed a listing and before a directory creation request by this processor is finished, "
+                      "then an error is returned because the directory already exists.")
+    ->isRequired(false)->withDefaultValue<bool>(false)->build());
+
+core::Property FetchSFTP::UseCompression(core::PropertyBuilder::createProperty("Use Compression")
+    ->withDescription("Indicates whether or not ZLIB compression should be used when transferring files")
+    ->isRequired(true)->withDefaultValue<bool>(false)->build());
+
+core::Relationship FetchSFTP::Success("success",
+                                      "All FlowFiles that are received are routed to success");
+core::Relationship FetchSFTP::CommsFailure("comms.failure",
+                                           "Any FlowFile that could not be fetched from the remote server due to a communications failure will be transferred to this Relationship.");
+core::Relationship FetchSFTP::NotFound("not.found",
+                                       "Any FlowFile for which we receive a 'Not Found' message from the remote server will be transferred to this Relationship.");
+core::Relationship FetchSFTP::PermissionDenied("permission.denied",
+                                               "Any FlowFile that could not be fetched from the remote server due to insufficient permissions will be transferred to this Relationship.");
 
 constexpr char const* FetchSFTP::COMPLETION_STRATEGY_NONE;
 constexpr char const* FetchSFTP::COMPLETION_STRATEGY_MOVE_FILE;
@@ -119,7 +129,7 @@ FetchSFTP::FetchSFTP(const std::string& name, const utils::Identifier& uuid /*=
     : SFTPProcessorBase(name, uuid),
       create_directory_(false),
       disable_directory_listing_(false) {
-  logger_ = logging::LoggerFactory<FetchSFTP>::getLogger();
+  logger_ = core::logging::LoggerFactory<FetchSFTP>::getLogger();
 }
 
 FetchSFTP::~FetchSFTP() = default;
@@ -148,22 +158,6 @@ void FetchSFTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context,
   startKeepaliveThreadIfNeeded();
 }
 
-FetchSFTP::WriteCallback::WriteCallback(const std::string& remote_file,
-                                    utils::SFTPClient& client)
-    : logger_(logging::LoggerFactory<FetchSFTP::WriteCallback>::getLogger())
-    , remote_file_(remote_file)
-    , client_(client) {
-}
-
-FetchSFTP::WriteCallback::~WriteCallback() = default;
-
-int64_t FetchSFTP::WriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-  if (!client_.getFile(remote_file_, *stream)) {
-    throw utils::SFTPException{client_.getLastError()};
-  }
-  return stream->size();
-}
-
 void FetchSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
   auto flow_file = session->get();
   if (flow_file == nullptr) {
@@ -211,9 +205,13 @@ void FetchSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
   };
 
   /* Download file */
-  WriteCallback write_callback(remote_file, *client);
   try {
-    session->write(flow_file, &write_callback);
+    session->write(flow_file, [&remote_file, &client](const std::shared_ptr<io::BaseStream>& stream) -> int64_t {
+      if (!client->getFile(remote_file, *stream)) {
+        throw utils::SFTPException{client->getLastError()};
+      }
+      return gsl::narrow<int64_t>(stream->size());
+    });
   } catch (const utils::SFTPException& ex) {
     logger_->log_debug(ex.what());
     switch (ex.error().value()) {
@@ -238,7 +236,7 @@ void FetchSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
   /* Set attributes */
   std::string parent_path;
   std::string child_path;
-  std::tie(parent_path, child_path) = utils::file::FileUtils::split_path(remote_file, true /*force_posix*/);
+  std::tie(parent_path, child_path) = utils::file::split_path(remote_file, true /*force_posix*/);
 
   session->putAttribute(flow_file, ATTRIBUTE_SFTP_REMOTE_HOST, common_properties.hostname);
   session->putAttribute(flow_file, ATTRIBUTE_SFTP_REMOTE_PORT, std::to_string(common_properties.port));
@@ -264,7 +262,7 @@ void FetchSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
     if (!should_move) {
       logger_->log_warn("Completion Strategy is Move File, but failed to create Move Destination Directory \"%s\"", move_destination_directory);
     } else {
-      auto target_path = utils::file::FileUtils::concat_path(move_destination_directory, child_path);
+      auto target_path = utils::file::concat_path(move_destination_directory, child_path);
       if (!client->rename(remote_file, target_path, false /*overwrite*/)) {
         logger_->log_warn("Completion Strategy is Move File, but failed to move file \"%s\" to \"%s\"", remote_file, target_path);
       }
@@ -275,6 +273,8 @@ void FetchSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context,
   put_connection_back_to_cache();
 }
 
+REGISTER_RESOURCE(FetchSFTP, "Fetches the content of a file from a remote SFTP server and overwrites the contents of an incoming FlowFile with the content of the remote file.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
