@@ -14,8 +14,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __FETCH_SFTP_H__
-#define __FETCH_SFTP_H__
+#pragma once
 
 #include <memory>
 #include <string>
@@ -27,7 +26,6 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
 #include "../client/SFTPClient.h"
@@ -40,7 +38,6 @@ namespace processors {
 
 class FetchSFTP : public SFTPProcessorBase {
  public:
-
   static constexpr char const *COMPLETION_STRATEGY_NONE = "None";
   static constexpr char const *COMPLETION_STRATEGY_MOVE_FILE = "Move File";
   static constexpr char const *COMPLETION_STRATEGY_DELETE_FILE = "Delete File";
@@ -51,8 +48,8 @@ class FetchSFTP : public SFTPProcessorBase {
   /*!
    * Create a new processor
    */
-  FetchSFTP(const std::string& name, const utils::Identifier& uuid = {});
-  virtual ~FetchSFTP();
+  explicit FetchSFTP(const std::string& name, const utils::Identifier& uuid = {});
+  ~FetchSFTP() override;
 
   // Supported Properties
   static core::Property RemoteFile;
@@ -77,19 +74,6 @@ class FetchSFTP : public SFTPProcessorBase {
   void initialize() override;
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
 
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(const std::string& remote_file,
-                 utils::SFTPClient& client);
-    ~WriteCallback();
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
-
-   private:
-    std::shared_ptr<logging::Logger> logger_;
-    const std::string remote_file_;
-    utils::SFTPClient& client_;
-  };
-
  private:
   core::annotation::Input getInputRequirement() const override {
     return core::annotation::Input::INPUT_REQUIRED;
@@ -100,12 +84,8 @@ class FetchSFTP : public SFTPProcessorBase {
   bool disable_directory_listing_;
 };
 
-REGISTER_RESOURCE(FetchSFTP, "Fetches the content of a file from a remote SFTP server and overwrites the contents of an incoming FlowFile with the content of the remote file.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
