@@ -38,12 +38,12 @@
 #include "utils/ByteArrayCallback.h"
 #include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
-#include "utils/RegexUtils.h"
 #include "utils/file/FileUtils.h"
 #include "core/FlowFile.h"
 #include "core/logging/Logger.h"
 #include "core/ProcessContext.h"
 #include "core/Relationship.h"
+#include "core/Resource.h"
 #include "io/BufferStream.h"
 #include "io/StreamFactory.h"
 #include "ResourceClaim.h"
@@ -53,79 +53,93 @@
 #include "rapidjson/istreamwrapper.h"
 #include "rapidjson/writer.h"
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
 namespace minifi {
 namespace processors {
 
-core::Property ListSFTP::ListingStrategy(
-    core::PropertyBuilder::createProperty("Listing Strategy")->withDescription("Specify how to determine new/updated entities. See each strategy descriptions for detail.")
-        ->isRequired(true)
-        ->withAllowableValues<std::string>({LISTING_STRATEGY_TRACKING_TIMESTAMPS,
-                                            LISTING_STRATEGY_TRACKING_ENTITIES})
-        ->withDefaultValue(LISTING_STRATEGY_TRACKING_TIMESTAMPS)->build());
-core::Property ListSFTP::RemotePath(
-    core::PropertyBuilder::createProperty("Remote Path")->withDescription("The fully qualified filename on the remote system")
-        ->isRequired(false)->supportsExpressionLanguage(true)->build());
-core::Property ListSFTP::SearchRecursively(
-    core::PropertyBuilder::createProperty("Search Recursively")->withDescription("If true, will pull files from arbitrarily nested subdirectories; "
-                                                                                 "otherwise, will not traverse subdirectories")
-        ->isRequired(true)->withDefaultValue<bool>(false)->build());
-core::Property ListSFTP::FollowSymlink(
-    core::PropertyBuilder::createProperty("Follow symlink")->withDescription("If true, will pull even symbolic files and also nested symbolic subdirectories; "
-                                                                             "otherwise, will not read symbolic files and will not traverse symbolic link subdirectories")
-        ->isRequired(true)->withDefaultValue<bool>(false)->build());
-core::Property ListSFTP::FileFilterRegex(
-    core::PropertyBuilder::createProperty("File Filter Regex")->withDescription("Provides a Java Regular Expression for filtering Filenames; "
-                                                                                "if a filter is supplied, only files whose names match that Regular Expression will be fetched")
-        ->isRequired(false)->build());
-core::Property ListSFTP::PathFilterRegex(
-    core::PropertyBuilder::createProperty("Path Filter Regex")->withDescription("When Search Recursively is true, then only subdirectories whose path matches the given Regular Expression will be scanned")
-        ->isRequired(false)->build());
-core::Property ListSFTP::IgnoreDottedFiles(
-    core::PropertyBuilder::createProperty("Ignore Dotted Files")->withDescription("If true, files whose names begin with a dot (\".\") will be ignored")
-        ->isRequired(true)->withDefaultValue<bool>(true)->build());
-core::Property ListSFTP::TargetSystemTimestampPrecision(
-    core::PropertyBuilder::createProperty("Target System Timestamp Precision")->withDescription("Specify timestamp precision at the target system. "
-                                                                                                "Since this processor uses timestamp of entities to decide which should be listed, "
-                                                                                                "it is crucial to use the right timestamp precision.")
-        ->isRequired(true)
-        ->withAllowableValues<std::string>({TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT,
+core::Property ListSFTP::ListingStrategy(core::PropertyBuilder::createProperty("Listing Strategy")
+    ->withDescription("Specify how to determine new/updated entities. See each strategy descriptions for detail.")
+    ->isRequired(true)
+    ->withAllowableValues<std::string>({LISTING_STRATEGY_TRACKING_TIMESTAMPS, LISTING_STRATEGY_TRACKING_ENTITIES})
+    ->withDefaultValue(LISTING_STRATEGY_TRACKING_TIMESTAMPS)->build());
+
+core::Property ListSFTP::RemotePath(core::PropertyBuilder::createProperty("Remote Path")
+    ->withDescription("The fully qualified filename on the remote system")
+    ->isRequired(false)->supportsExpressionLanguage(true)->build());
+
+core::Property ListSFTP::SearchRecursively(core::PropertyBuilder::createProperty("Search Recursively")
+    ->withDescription("If true, will pull files from arbitrarily nested subdirectories; "
+                      "otherwise, will not traverse subdirectories")
+    ->isRequired(true)->withDefaultValue<bool>(false)->build());
+
+core::Property ListSFTP::FollowSymlink(core::PropertyBuilder::createProperty("Follow symlink")
+    ->withDescription("If true, will pull even symbolic files and also nested symbolic subdirectories; "
+                      "otherwise, will not read symbolic files and will not traverse symbolic link subdirectories")
+    ->isRequired(true)->withDefaultValue<bool>(false)->build());
+
+core::Property ListSFTP::FileFilterRegex(core::PropertyBuilder::createProperty("File Filter Regex")
+    ->withDescription("Provides a Java Regular Expression for filtering Filenames; "
+                      "if a filter is supplied, only files whose names match that Regular Expression will be fetched")
+    ->isRequired(false)->build());
+
+core::Property ListSFTP::PathFilterRegex(core::PropertyBuilder::createProperty("Path Filter Regex")
+    ->withDescription("When Search Recursively is true, then only subdirectories whose path matches the given Regular Expression will be scanned")
+    ->isRequired(false)->build());
+
+core::Property ListSFTP::IgnoreDottedFiles(core::PropertyBuilder::createProperty("Ignore Dotted Files")
+    ->withDescription("If true, files whose names begin with a dot (\".\") will be ignored")
+    ->isRequired(true)->withDefaultValue<bool>(true)->build());
+
+core::Property ListSFTP::TargetSystemTimestampPrecision(core::PropertyBuilder::createProperty("Target System Timestamp Precision")
+    ->withDescription("Specify timestamp precision at the target system. "
+                      "Since this processor uses timestamp of entities to decide which should be listed, "
+                      "it is crucial to use the right timestamp precision.")
+    ->isRequired(true)
+    ->withAllowableValues<std::string>({TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT,
                                             TARGET_SYSTEM_TIMESTAMP_PRECISION_MILLISECONDS,
                                             TARGET_SYSTEM_TIMESTAMP_PRECISION_SECONDS,
                                             TARGET_SYSTEM_TIMESTAMP_PRECISION_MINUTES})
-        ->withDefaultValue(TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT)->build());
-core::Property ListSFTP::EntityTrackingTimeWindow(
-    core::PropertyBuilder::createProperty("Entity Tracking Time Window")->withDescription("Specify how long this processor should track already-listed entities. "
-                                                                                          "'Tracking Entities' strategy can pick any entity whose timestamp is inside the specified time window. "
-                                                                                          "For example, if set to '30 minutes', any entity having timestamp in recent 30 minutes will be the listing target when this processor runs. "
-                                                                                          "A listed entity is considered 'new/updated' and a FlowFile is emitted if one of following condition meets: "
-                                                                                          "1. does not exist in the already-listed entities, "
-                                                                                          "2. has newer timestamp than the cached entity, "
-                                                                                          "3. has different size than the cached entity. "
-                                                                                          "If a cached entity's timestamp becomes older than specified time window, that entity will be removed from the cached already-listed entities. "
-                                                                                          "Used by 'Tracking Entities' strategy.")
-        ->isRequired(false)->build());
-core::Property ListSFTP::EntityTrackingInitialListingTarget(
-    core::PropertyBuilder::createProperty("Entity Tracking Initial Listing Target")->withDescription("Specify how initial listing should be handled. Used by 'Tracking Entities' strategy.")
-        ->withAllowableValues<std::string>({ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW,
+    ->withDefaultValue(TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT)->build());
+
+core::Property ListSFTP::EntityTrackingTimeWindow(core::PropertyBuilder::createProperty("Entity Tracking Time Window")
+    ->withDescription("Specify how long this processor should track already-listed entities. "
+                      "'Tracking Entities' strategy can pick any entity whose timestamp is inside the specified time window. "
+                      "For example, if set to '30 minutes', any entity having timestamp in recent 30 minutes will be the listing target when this processor runs. "
+                      "A listed entity is considered 'new/updated' and a FlowFile is emitted if one of following condition meets: "
+                      "1. does not exist in the already-listed entities, "
+                      "2. has newer timestamp than the cached entity, "
+                      "3. has different size than the cached entity. "
+                      "If a cached entity's timestamp becomes older than specified time window, that entity will be removed from the cached already-listed entities. "
+                      "Used by 'Tracking Entities' strategy.")
+    ->isRequired(false)->build());
+
+core::Property ListSFTP::EntityTrackingInitialListingTarget(core::PropertyBuilder::createProperty("Entity Tracking Initial Listing Target")
+    ->withDescription("Specify how initial listing should be handled. Used by 'Tracking Entities' strategy.")
+    ->withAllowableValues<std::string>({ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW,
                                             ENTITY_TRACKING_INITIAL_LISTING_TARGET_ALL_AVAILABLE})
-        ->isRequired(false)->withDefaultValue(ENTITY_TRACKING_INITIAL_LISTING_TARGET_ALL_AVAILABLE)->build());
-core::Property ListSFTP::MinimumFileAge(
-    core::PropertyBuilder::createProperty("Minimum File Age")->withDescription("The minimum age that a file must be in order to be pulled; "
-                                                                               "any file younger than this amount of time (according to last modification date) will be ignored")
-        ->isRequired(true)->withDefaultValue<core::TimePeriodValue>("0 sec")->build());
-core::Property ListSFTP::MaximumFileAge(
-    core::PropertyBuilder::createProperty("Maximum File Age")->withDescription("The maximum age that a file must be in order to be pulled; "
-                                                                               "any file older than this amount of time (according to last modification date) will be ignored")
-        ->isRequired(false)->build());
-core::Property ListSFTP::MinimumFileSize(
-    core::PropertyBuilder::createProperty("Minimum File Size")->withDescription("The minimum size that a file must be in order to be pulled")
-        ->isRequired(true)->withDefaultValue<core::DataSizeValue>("0 B")->build());
-core::Property ListSFTP::MaximumFileSize(
-    core::PropertyBuilder::createProperty("Maximum File Size")->withDescription("The maximum size that a file must be in order to be pulled")
-        ->isRequired(false)->build());
+    ->isRequired(false)->withDefaultValue(ENTITY_TRACKING_INITIAL_LISTING_TARGET_ALL_AVAILABLE)->build());
+
+core::Property ListSFTP::MinimumFileAge(core::PropertyBuilder::createProperty("Minimum File Age")
+    ->withDescription("The minimum age that a file must be in order to be pulled; "
+                      "any file younger than this amount of time (according to last modification date) will be ignored")
+    ->isRequired(true)->withDefaultValue<core::TimePeriodValue>("0 sec")->build());
+
+core::Property ListSFTP::MaximumFileAge(core::PropertyBuilder::createProperty("Maximum File Age")
+    ->withDescription("The maximum age that a file must be in order to be pulled; "
+                      "any file older than this amount of time (according to last modification date) will be ignored")
+    ->isRequired(false)->build());
+
+core::Property ListSFTP::MinimumFileSize(core::PropertyBuilder::createProperty("Minimum File Size")
+    ->withDescription("The minimum size that a file must be in order to be pulled")
+    ->isRequired(true)->withDefaultValue<core::DataSizeValue>("0 B")->build());
+
+core::Property ListSFTP::MaximumFileSize(core::PropertyBuilder::createProperty("Maximum File Size")
+    ->withDescription("The maximum size that a file must be in order to be pulled")
+    ->isRequired(false)->build());
 
 core::Relationship ListSFTP::Success("success", "All FlowFiles that are received are routed to success");
 
@@ -190,7 +204,7 @@ ListSFTP::ListSFTP(const std::string& name, const utils::Identifier& uuid /*= ut
     , last_listed_latest_entry_timestamp_(0U)
     , last_processed_latest_entry_timestamp_(0U)
     , initial_listing_complete_(false) {
-  logger_ = logging::LoggerFactory<ListSFTP>::getLogger();
+  logger_ = core::logging::LoggerFactory<ListSFTP>::getLogger();
 }
 
 ListSFTP::~ListSFTP() = default;
@@ -248,20 +262,19 @@ void ListSFTP::onSchedule(const std::shared_ptr<core::ProcessContext> &context,
   }
   context->getProperty(TargetSystemTimestampPrecision.getName(), target_system_timestamp_precision_);
   context->getProperty(EntityTrackingInitialListingTarget.getName(), entity_tracking_initial_listing_target_);
-  if (!context->getProperty(MinimumFileAge.getName(), value)) {
-    logger_->log_error("Minimum File Age attribute is missing or invalid");
+
+  if (auto minimum_file_age = context->getProperty<core::TimePeriodValue>(MinimumFileAge)) {
+    minimum_file_age_ = minimum_file_age->getMilliseconds();
   } else {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, minimum_file_age_, unit) || !core::Property::ConvertTimeUnitToMS(minimum_file_age_, unit, minimum_file_age_)) {
-      logger_->log_error("Minimum File Age attribute is invalid");
-    }
+    logger_->log_error("Minimum File Age attribute is missing or invalid");
   }
-  if (context->getProperty(MaximumFileAge.getName(), value)) {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, maximum_file_age_, unit) || !core::Property::ConvertTimeUnitToMS(maximum_file_age_, unit, maximum_file_age_)) {
-      logger_->log_error("Maximum File Age attribute is invalid");
-    }
+
+  if (auto maximum_file_age = context->getProperty<core::TimePeriodValue>(MaximumFileAge)) {
+    maximum_file_age_ = maximum_file_age->getMilliseconds();
+  } else {
+    logger_->log_error("Maximum File Age attribute is missing or invalid");
   }
+
   if (!context->getProperty(MinimumFileSize.getName(), minimum_file_size_)) {
     logger_->log_error("Minimum File Size attribute is invalid");
   }
@@ -279,7 +292,7 @@ void ListSFTP::invalidateCache() {
 
   already_loaded_from_cache_ = false;
 
-  last_run_time_ = std::chrono::time_point<std::chrono::steady_clock>();
+  last_run_time_ = std::chrono::steady_clock::time_point();
   last_listed_latest_entry_timestamp_ = 0U;
   last_processed_latest_entry_timestamp_ = 0U;
   latest_identifiers_processed_.clear();
@@ -345,22 +358,21 @@ bool ListSFTP::filterFile(const std::string& parent_path, const std::string& fil
   }
 
   /* Age */
-  time_t now = time(nullptr);
-  uint64_t file_age = (now - attrs.mtime) * 1000;
+  auto file_age = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now() - std::chrono::system_clock::from_time_t(attrs.mtime));
   if (file_age < minimum_file_age_) {
     logger_->log_debug("Ignoring \"%s/%s\" because it is younger than the Minimum File Age: %ld ms < %lu ms",
         parent_path.c_str(),
         filename.c_str(),
-        file_age,
-        minimum_file_age_);
+        file_age.count(),
+        minimum_file_age_.count());
     return false;
   }
-  if (maximum_file_age_ != 0U && file_age > maximum_file_age_) {
-    logger_->log_debug("Ignoring \"%s/%s\" because it is older than the Maximum File Age: %ld ms > %lu ms",
+  if (maximum_file_age_ != 0ms && file_age > maximum_file_age_) {
+    logger_->log_debug("Ignoring \"%s/%s\" because it is older than the Maximum File Age: %" PRId64 " ms > %" PRId64 " ms",
                        parent_path.c_str(),
                        filename.c_str(),
-                       file_age,
-                       maximum_file_age_);
+                       int64_t{file_age.count()},
+                       int64_t{maximum_file_age_.count()});
     return false;
   }
 
@@ -385,7 +397,7 @@ bool ListSFTP::filterFile(const std::string& parent_path, const std::string& fil
   /* File Filter Regex */
   if (file_filter_regex_set_) {
     bool match = false;
-    match = compiled_file_filter_regex_.match(filename);
+    match = utils::regexSearch(filename, compiled_file_filter_regex_);
     if (!match) {
       logger_->log_debug("Ignoring \"%s/%s\" because it did not match the File Filter Regex \"%s\"",
                          parent_path.c_str(),
@@ -407,7 +419,7 @@ bool ListSFTP::filterDirectory(const std::string& parent_path, const std::string
   if (path_filter_regex_set_) {
     std::string dir_path = utils::file::FileUtils::concat_path(parent_path, filename, true /*force_posix*/);
     bool match = false;
-    match = compiled_path_filter_regex_.match(dir_path);
+    match = utils::regexSearch(dir_path, compiled_path_filter_regex_);
     if (!match) {
       logger_->log_debug("Not recursing into \"%s\" because it did not match the Path Filter Regex \"%s\"",
                          dir_path.c_str(),
@@ -598,7 +610,7 @@ void ListSFTP::listByTrackingTimestamps(
     already_loaded_from_cache_ = true;
   }
 
-  std::chrono::time_point<std::chrono::steady_clock> current_run_time = std::chrono::steady_clock::now();
+  std::chrono::steady_clock::time_point current_run_time = std::chrono::steady_clock::now();
   time_t now = time(nullptr);
 
   /* Order children by timestamp and try to detect timestamp precision if needed  */
@@ -842,7 +854,7 @@ void ListSFTP::listByTrackingEntities(
     uint16_t port,
     const std::string& username,
     const std::string& remote_path,
-    uint64_t entity_tracking_time_window,
+    std::chrono::milliseconds entity_tracking_time_window,
     std::vector<Child>&& files) {
   /* Load state from cache file if needed */
   if (!already_loaded_from_cache_) {
@@ -857,7 +869,7 @@ void ListSFTP::listByTrackingEntities(
 
   time_t now = time(nullptr);
   uint64_t min_timestamp_to_list = (!initial_listing_complete_ && entity_tracking_initial_listing_target_ == ENTITY_TRACKING_INITIAL_LISTING_TARGET_ALL_AVAILABLE)
-      ? 0U : (now * 1000 - entity_tracking_time_window);
+      ? 0U : (now * 1000 - entity_tracking_time_window.count());
 
   /* Skip files not in the tracking window */
   for (auto it = files.begin(); it != files.end(); ) {
@@ -952,7 +964,7 @@ void ListSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, c
 
   /* Parse processor-specific properties */
   std::string remote_path;
-  uint64_t entity_tracking_time_window = 0U;
+  std::chrono::milliseconds entity_tracking_time_window = 3h;  /* The default is 3 hours */
 
   std::string value;
   context->getProperty(RemotePath.getName(), remote_path);
@@ -961,16 +973,11 @@ void ListSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, c
     remote_path.resize(remote_path.size() - 1);
   }
   if (context->getProperty(EntityTrackingTimeWindow.getName(), value)) {
-    core::TimeUnit unit;
-    if (!core::Property::StringToTime(value, entity_tracking_time_window, unit) ||
-        !core::Property::ConvertTimeUnitToMS(entity_tracking_time_window, unit, entity_tracking_time_window)) {
-      /* The default is 3 hours */
-      entity_tracking_time_window = 3 * 3600 * 1000;
+    if (auto parsed_entity_time_window = utils::timeutils::StringToDuration<std::chrono::milliseconds>(value)) {
+      entity_tracking_time_window = parsed_entity_time_window.value();
+    } else {
       logger_->log_error("Entity Tracking Time Window attribute is invalid");
     }
-  } else {
-    /* The default is 3 hours */
-    entity_tracking_time_window = 3 * 3600 * 1000;
   }
 
   /* Check whether we need to invalidate the cache based on the new properties */
@@ -1059,6 +1066,11 @@ void ListSFTP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, c
   put_connection_back_to_cache();
 }
 
+REGISTER_RESOURCE(ListSFTP, "Performs a listing of the files residing on an SFTP server. "
+                            "For each file that is found on the remote server, a new FlowFile will be created with "
+                            "the filename attribute set to the name of the file on the remote server. "
+                            "This can then be used in conjunction with FetchSFTP in order to fetch those files.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
