@@ -14,8 +14,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __LIST_SFTP_H__
-#define __LIST_SFTP_H__
+#pragma once
 
 #include <memory>
 #include <string>
@@ -23,20 +22,17 @@
 #include <chrono>
 #include <cstdint>
 #include <unordered_map>
+#include <set>
+#include <tuple>
+#include <vector>
 
 #include "SFTPProcessorBase.h"
-#include "utils/ByteArrayCallback.h"
-#include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
-#include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
-#include "utils/RegexUtils.h"
 #include "controllers/keyvalue/PersistableKeyValueStoreService.h"
-#include "../client/SFTPClient.h"
+#include "utils/RegexUtils.h"
 
 namespace org {
 namespace apache {
@@ -46,7 +42,6 @@ namespace processors {
 
 class ListSFTP : public SFTPProcessorBase {
  public:
-
   static constexpr char const *LISTING_STRATEGY_TRACKING_TIMESTAMPS = "Tracking Timestamps";
   static constexpr char const *LISTING_STRATEGY_TRACKING_ENTITIES = "Tracking Entities";
 
@@ -64,8 +59,8 @@ class ListSFTP : public SFTPProcessorBase {
   /*!
    * Create a new processor
    */
-  ListSFTP(const std::string& name, const utils::Identifier& uuid = {});
-  virtual ~ListSFTP();
+  explicit ListSFTP(const std::string& name, const utils::Identifier& uuid = {});
+  ~ListSFTP() override;
 
   // Supported Properties
   static core::Property ListingStrategy;
@@ -107,8 +102,11 @@ class ListSFTP : public SFTPProcessorBase {
     return core::annotation::Input::INPUT_FORBIDDEN;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
 
-  std::shared_ptr<core::CoreComponentStateManager> state_manager_;
+  core::CoreComponentStateManager* state_manager_;
   std::string listing_strategy_;
   bool search_recursively_;
   bool follow_symlink_;
@@ -121,8 +119,8 @@ class ListSFTP : public SFTPProcessorBase {
   bool ignore_dotted_files_;
   std::string target_system_timestamp_precision_;
   std::string entity_tracking_initial_listing_target_;
-  uint64_t minimum_file_age_;
-  uint64_t maximum_file_age_;
+  std::chrono::milliseconds minimum_file_age_;
+  std::chrono::milliseconds maximum_file_age_;
   uint64_t minimum_file_size_;
   uint64_t maximum_file_size_;
 
@@ -144,7 +142,7 @@ class ListSFTP : public SFTPProcessorBase {
 
   bool already_loaded_from_cache_;
 
-  std::chrono::time_point<std::chrono::steady_clock> last_run_time_;
+  std::chrono::steady_clock::time_point last_run_time_;
   uint64_t last_listed_latest_entry_timestamp_;
   uint64_t last_processed_latest_entry_timestamp_;
   std::set<std::string> latest_identifiers_processed_;
@@ -194,19 +192,12 @@ class ListSFTP : public SFTPProcessorBase {
       uint16_t port,
       const std::string& username,
       const std::string& remote_path,
-      uint64_t entity_tracking_time_window,
+      std::chrono::milliseconds entity_tracking_time_window,
       std::vector<Child>&& files);
 };
 
-REGISTER_RESOURCE(ListSFTP, "Performs a listing of the files residing on an SFTP server. "
-                            "For each file that is found on the remote server, a new FlowFile will be created with "
-                            "the filename attribute set to the name of the file on the remote server. "
-                            "This can then be used in conjunction with FetchSFTP in order to fetch those files.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
