@@ -16,8 +16,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __PUT_SFTP_H__
-#define __PUT_SFTP_H__
+#pragma once
 
 #include <memory>
 #include <string>
@@ -33,7 +32,6 @@
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "controllers/SSLContextService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
@@ -45,9 +43,8 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
- class PutSFTP : public SFTPProcessorBase {
+class PutSFTP : public SFTPProcessorBase {
  public:
-
   static constexpr char const *CONFLICT_RESOLUTION_REPLACE = "REPLACE";
   static constexpr char const *CONFLICT_RESOLUTION_IGNORE = "IGNORE";
   static constexpr char const *CONFLICT_RESOLUTION_RENAME = "RENAME";
@@ -61,8 +58,8 @@ namespace processors {
   /*!
    * Create a new processor
    */
-  PutSFTP(const std::string& name, const utils::Identifier& uuid = {});
-  virtual ~PutSFTP();
+  explicit PutSFTP(const std::string& name, const utils::Identifier& uuid = {});
+  ~PutSFTP() override;
 
   // Supported Properties
   static core::Property RemotePath;
@@ -92,25 +89,10 @@ namespace processors {
   void initialize() override;
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
 
-  class ReadCallback : public InputStreamCallback {
-   public:
-    ReadCallback(const std::string& target_path,
-        utils::SFTPClient& client,
-        const std::string& conflict_resolution);
-    ~ReadCallback();
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
-
-   private:
-    std::shared_ptr<logging::Logger> logger_;
-    const std::string target_path_;
-    utils::SFTPClient& client_;
-    const std::string conflict_resolution_;
-  };
-
  private:
-   core::annotation::Input getInputRequirement() const override {
-     return core::annotation::Input::INPUT_REQUIRED;
-   }
+  core::annotation::Input getInputRequirement() const override {
+    return core::annotation::Input::INPUT_REQUIRED;
+  }
 
   bool create_directory_;
   uint64_t batch_size_;
@@ -121,12 +103,8 @@ namespace processors {
   bool processOne(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session);
 };
 
-REGISTER_RESOURCE(PutSFTP, "Sends FlowFiles to an SFTP Server");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
