@@ -16,8 +16,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef __SFTP_PROCESSOR_BASE_H__
-#define __SFTP_PROCESSOR_BASE_H__
+#pragma once
 
 #include <memory>
 #include <string>
@@ -25,16 +24,13 @@
 #include <map>
 #include <mutex>
 #include <thread>
+#include <set>
 
-#include "utils/ByteArrayCallback.h"
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Core.h"
 #include "core/Property.h"
-#include "core/Resource.h"
 #include "controllers/SSLContextService.h"
-#include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
 #include "../client/SFTPClient.h"
 
@@ -75,10 +71,10 @@ class SFTPProcessorBase : public core::Processor {
   void notifyStop() override;
 
  protected:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_;
 
-  int64_t connection_timeout_;
-  int64_t data_timeout_;
+  std::chrono::milliseconds connection_timeout_;
+  std::chrono::milliseconds data_timeout_;
   std::string host_key_file_;
   bool strict_host_checking_;
   bool use_keepalive_on_timeout_;
@@ -152,5 +148,3 @@ class SFTPProcessorBase : public core::Processor {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
