@@ -39,6 +39,7 @@
 #endif
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/StringUtils.h"
 #include "utils/file/FileUtils.h"
 #include "core/Core.h"
@@ -55,26 +56,24 @@
 #include "processors/UpdateAttribute.h"
 #include "tools/SFTPTestServer.h"
 #include "utils/TestUtils.h"
+using namespace std::literals::chrono_literals;
 
 class ListSFTPTestsFixture {
  public:
-  ListSFTPTestsFixture(const std::shared_ptr<minifi::Configure>& configuration = nullptr)
-  : src_dir(strdup("/var/tmp/sftps.XXXXXX")) {
+  explicit ListSFTPTestsFixture(const std::shared_ptr<minifi::Configure>& configuration = nullptr) {
     LogTestController::getInstance().setTrace<TestPlan>();
     LogTestController::getInstance().setDebug<minifi::FlowController>();
     LogTestController::getInstance().setDebug<minifi::SchedulingAgent>();
     LogTestController::getInstance().setDebug<minifi::core::ProcessGroup>();
     LogTestController::getInstance().setDebug<minifi::core::Processor>();
     LogTestController::getInstance().setTrace<minifi::core::ProcessSession>();
-    LogTestController::getInstance().setDebug<processors::GenerateFlowFile>();
+    LogTestController::getInstance().setDebug<minifi::processors::GenerateFlowFile>();
     LogTestController::getInstance().setTrace<minifi::utils::SFTPClient>();
-    LogTestController::getInstance().setTrace<processors::ListSFTP>();
-    LogTestController::getInstance().setDebug<processors::LogAttribute>();
+    LogTestController::getInstance().setTrace<minifi::processors::ListSFTP>();
+    LogTestController::getInstance().setDebug<minifi::processors::LogAttribute>();
     LogTestController::getInstance().setDebug<SFTPTestServer>();
 
-    // Create temporary directories
-    testController.createTempDirectory(src_dir);
-    REQUIRE(src_dir != nullptr);
+    REQUIRE_FALSE(src_dir.empty());
 
     // Start SFTP server
     sftp_server = std::unique_ptr<SFTPTestServer>(new SFTPTestServer(src_dir));
@@ -85,13 +84,11 @@ class ListSFTPTestsFixture {
   }
 
   virtual ~ListSFTPTestsFixture() {
-    free(src_dir);
     LogTestController::getInstance().reset();
   }
 
   void createPlan(utils::Identifier* list_sftp_uuid = nullptr, const std::shared_ptr<minifi::Configure>& configuration = nullptr) {
-    char format[] = "/var/tmp/gt.XXXXXX";
-    const std::string state_dir = plan == nullptr ? minifi::utils::createTempDir(&testController, format) : plan->getStateDir();
+    const std::string state_dir = plan == nullptr ? testController.createTempDirectory() : plan->getStateDir();
 
     log_attribute.reset();
     list_sftp.reset();
@@ -115,7 +112,7 @@ class ListSFTPTestsFixture {
                                        true);
 
     // Configure ListSFTP processor
-    plan->setProperty(list_sftp, "Listing Strategy", processors::ListSFTP::LISTING_STRATEGY_TRACKING_TIMESTAMPS);
+    plan->setProperty(list_sftp, "Listing Strategy", minifi::processors::ListSFTP::LISTING_STRATEGY_TRACKING_TIMESTAMPS);
     plan->setProperty(list_sftp, "Hostname", "localhost");
     plan->setProperty(list_sftp, "Port", std::to_string(sftp_server->getPort()));
     plan->setProperty(list_sftp, "Username", "nifiuser");
@@ -127,7 +124,7 @@ class ListSFTPTestsFixture {
     plan->setProperty(list_sftp, "Connection Timeout", "30 sec");
     plan->setProperty(list_sftp, "Data Timeout", "30 sec");
     plan->setProperty(list_sftp, "Send Keep Alive On Timeout", "true");
-    plan->setProperty(list_sftp, "Target System Timestamp Precision", processors::ListSFTP::TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT);
+    plan->setProperty(list_sftp, "Target System Timestamp Precision", minifi::processors::ListSFTP::TARGET_SYSTEM_TIMESTAMP_PRECISION_AUTO_DETECT);
     plan->setProperty(list_sftp, "Minimum File Age", "0 sec");
     plan->setProperty(list_sftp, "Minimum File Size", "0 B");
     plan->setProperty(list_sftp, "Target System Timestamp Precision", "Seconds");
@@ -138,7 +135,7 @@ class ListSFTPTestsFixture {
   }
 
   // Create source file
-  void createFile(const std::string& relative_path, const std::string& content, uint64_t modification_timestamp = 0U) {
+  void createFile(const std::string& relative_path, const std::string& content, std::optional<std::filesystem::file_time_type> modification_time) {
     std::fstream file;
     std::stringstream ss;
     ss << src_dir << "/vfs/" << relative_path;
@@ -154,27 +151,26 @@ class ListSFTPTestsFixture {
     file.open(ss.str(), std::ios::out);
     file << content;
     file.close();
-    if (modification_timestamp != 0U) {
-      REQUIRE(true == utils::file::FileUtils::set_last_write_time(full_path, modification_timestamp));
+    if (modification_time.has_value()) {
+      REQUIRE(utils::file::set_last_write_time(full_path, modification_time.value()));
     }
   }
 
-  void createFileWithModificationTimeDiff(const std::string& relative_path, const std::string& content, int64_t modification_timediff = -300 /*5 minutes ago*/) {
-    time_t now = time(nullptr);
-    return createFile(relative_path, content, now + modification_timediff);
+  void createFileWithModificationTimeDiff(const std::string& relative_path, const std::string& content, std::chrono::seconds modification_timediff = -5min) {
+    return createFile(relative_path, content, std::chrono::file_clock::now() + modification_timediff);
   }
 
  protected:
-  char *src_dir;
-  std::unique_ptr<SFTPTestServer> sftp_server;
   TestController testController;
+  std::string src_dir = testController.createTempDirectory();
   std::shared_ptr<TestPlan> plan;
+  std::unique_ptr<SFTPTestServer> sftp_server;
   std::shared_ptr<core::Processor> list_sftp;
   std::shared_ptr<core::Processor> log_attribute;
 };
 
 class PersistentListSFTPTestsFixture : public ListSFTPTestsFixture {
-public:
+ public:
   PersistentListSFTPTestsFixture() :
     ListSFTPTestsFixture(std::make_shared<minifi::Configure>()) {
   }
@@ -191,7 +187,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP list one file", "[ListSFTP][bas
 
 TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP public key authentication", "[ListSFTP][basic]") {
   plan->setProperty(list_sftp, "Remote File", "nifi_test/tstFile.ext");
-  plan->setProperty(list_sftp, "Private Key Path", utils::file::FileUtils::concat_path(utils::file::FileUtils::get_executable_dir(), "resources/id_rsa"));
+  plan->setProperty(list_sftp, "Private Key Path", utils::file::FileUtils::concat_path(get_sftp_test_dir(), "resources/id_rsa"));
   plan->setProperty(list_sftp, "Private Key Passphrase", "privatekeypassword");
 
   createFileWithModificationTimeDiff("nifi_test/tstFile.ext", "Test content 1");
@@ -221,13 +217,14 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP list non-readable dir", "[ListS
     return;
   }
   createFileWithModificationTimeDiff("nifi_test/tstFile.ext", "Test content 1");
-  REQUIRE(0 == chmod((std::string(src_dir) + "/vfs/nifi_test").c_str(), 0000));
+  REQUIRE(0 == chmod((src_dir + "/vfs/nifi_test").c_str(), 0000));
 
   testController.runSession(plan, true);
 
   REQUIRE(false == LogTestController::getInstance().contains("from ListSFTP to relationship success"));
   REQUIRE(LogTestController::getInstance().contains("Failed to open remote directory \"nifi_test\", error: LIBSSH2_FX_PERMISSION_DENIED"));
   REQUIRE(LogTestController::getInstance().contains("There are no files to list. Yielding."));
+  REQUIRE(0 == chmod((src_dir + "/vfs/nifi_test").c_str(), 0755));
 }
 #endif
 
@@ -236,10 +233,9 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP list one file writes attributes
 
   testController.runSession(plan, true);
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/tstFile.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
+  auto file = src_dir + "/vfs/nifi_test/tstFile.ext";
   std::string mtime_str;
-  REQUIRE(true == utils::timeutils::getDateTimeStr(mtime, mtime_str));
+  REQUIRE(true == utils::timeutils::getDateTimeStr(utils::file::to_time_t(utils::file::last_write_time(file).value()), mtime_str));
   uint64_t uid, gid;
   REQUIRE(true == utils::file::FileUtils::get_uid_gid(file, uid, gid));
   uint32_t permissions;
@@ -367,8 +363,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Path Filter Regex", "[ListSFTP]
 #ifndef WIN32
 TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Follow symlink false file symlink", "[ListSFTP][follow-symlink]") {
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
-  auto file1 = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto file2 = std::string(src_dir) + "/vfs/nifi_test/file2.ext";
+  auto file1 = src_dir + "/vfs/nifi_test/file1.ext";
+  auto file2 = src_dir + "/vfs/nifi_test/file2.ext";
   REQUIRE(0 == symlink(file1.c_str(), file2.c_str()));
 
   testController.runSession(plan, true);
@@ -383,8 +379,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Follow symlink true file symlin
   plan->setProperty(list_sftp, "Follow symlink", "true");
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
-  auto file1 = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto file2 = std::string(src_dir) + "/vfs/nifi_test/file2.ext";
+  auto file1 = src_dir + "/vfs/nifi_test/file1.ext";
+  auto file2 = src_dir + "/vfs/nifi_test/file2.ext";
   REQUIRE(0 == symlink(file1.c_str(), file2.c_str()));
 
   testController.runSession(plan, true);
@@ -399,8 +395,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Follow symlink false directory
   plan->setProperty(list_sftp, "Search Recursively", "true");
 
   createFileWithModificationTimeDiff("nifi_test/dir1/file1.ext", "Test content 1");
-  auto dir1 = std::string(src_dir) + "/vfs/nifi_test/dir1";
-  auto dir2 = std::string(src_dir) + "/vfs/nifi_test/dir2";
+  auto dir1 = src_dir + "/vfs/nifi_test/dir1";
+  auto dir2 = src_dir + "/vfs/nifi_test/dir2";
   REQUIRE(0 == symlink(dir1.c_str(), dir2.c_str()));
 
   testController.runSession(plan, true);
@@ -416,8 +412,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Follow symlink true directory s
   plan->setProperty(list_sftp, "Follow symlink", "true");
 
   createFileWithModificationTimeDiff("nifi_test/dir1/file1.ext", "Test content 1");
-  auto dir1 = std::string(src_dir) + "/vfs/nifi_test/dir1";
-  auto dir2 = std::string(src_dir) + "/vfs/nifi_test/dir2";
+  auto dir1 = src_dir + "/vfs/nifi_test/dir1";
+  auto dir2 = src_dir + "/vfs/nifi_test/dir2";
   REQUIRE(0 == symlink(dir1.c_str(), dir2.c_str()));
 
   testController.runSession(plan, true);
@@ -460,7 +456,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps one file on
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
@@ -481,7 +477,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps one file on
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -360 /* 6 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -6min);
 
   testController.runSession(plan, true);
 
@@ -495,9 +491,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps one file an
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
-
+  auto file = src_dir + "/vfs/nifi_test/file1.ext";
+  auto mtime = utils::file::last_write_time(file).value();
   testController.runSession(plan, true);
 
   REQUIRE(LogTestController::getInstance().contains("from ListSFTP to relationship success"));
@@ -523,9 +518,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps one file ti
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
-
+  auto file = src_dir + "/vfs/nifi_test/file1.ext";
+  auto mtime = utils::file::last_write_time(file).value();
   testController.runSession(plan, true);
 
   REQUIRE(LogTestController::getInstance().contains("from ListSFTP to relationship success"));
@@ -534,7 +528,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps one file ti
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  REQUIRE(true == utils::file::FileUtils::set_last_write_time(file, mtime + 1));
+  REQUIRE(utils::file::set_last_write_time(file, mtime + 1s));
 
   testController.runSession(plan, true);
 
@@ -561,7 +555,7 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Timestamps r
   testController.runSession(plan, true);
 
   std::unordered_map<std::string, std::string> state;
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -579,11 +573,11 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Timestamps r
   plan->setProperty(list_sftp, "Listing Strategy", "Tracking Timestamps");
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -609,7 +603,7 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Timestamps r
   REQUIRE(LogTestController::getInstance().contains("key:filename value:file1.ext"));
 
   std::unordered_map<std::string, std::string> state;
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -622,11 +616,11 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Timestamps r
   plan->setProperty(list_sftp, "Remote Path", "/nifi_test");
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("/nifi_test" == state.at("remote_path"));
@@ -656,7 +650,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Timestamps changed con
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
   plan->setProperty(list_sftp, "Remote Path", "/nifi_test");
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
@@ -702,7 +696,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file one
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
@@ -723,7 +717,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file one
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -360 /* 6 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -6min);
 
   testController.runSession(plan, true);
 
@@ -746,7 +740,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file one
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -6 * 3600 /* 6 hours ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -6h);
 
   testController.runSession(plan, true);
 
@@ -760,9 +754,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file anot
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
-
+  auto file = src_dir + "/vfs/nifi_test/file1.ext";
+  auto mtime = utils::file::last_write_time(file).value();
   testController.runSession(plan, true);
 
   REQUIRE(LogTestController::getInstance().contains("from ListSFTP to relationship success"));
@@ -786,8 +779,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file time
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
+  auto file = src_dir + "/vfs/nifi_test/file1.ext";
+  auto mtime = utils::file::last_write_time(file).value();
 
   testController.runSession(plan, true);
 
@@ -797,7 +790,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file time
   plan->reset();
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  REQUIRE(true == utils::file::FileUtils::set_last_write_time(file, mtime + 1));
+  REQUIRE(utils::file::set_last_write_time(file, mtime + 1s));
 
   testController.runSession(plan, true);
 
@@ -819,8 +812,8 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities one file size
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
 
-  auto file = std::string(src_dir) + "/vfs/nifi_test/file1.ext";
-  auto mtime = utils::file::FileUtils::last_write_time(file);
+  auto file = src_dir + "/vfs/nifi_test/file1.ext";
+  auto mtime = utils::file::last_write_time(file).value();
 
   testController.runSession(plan, true);
 
@@ -864,7 +857,7 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Entities res
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
   std::unordered_map<std::string, std::string> state;
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -873,11 +866,11 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Entities res
   REQUIRE("14" == state.at("entity.0.size"));
   REQUIRE(!state.at("entity.0.timestamp").empty());
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -911,7 +904,7 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Entities res
   REQUIRE(LogTestController::getInstance().contains("key:filename value:file1.ext"));
 
   std::unordered_map<std::string, std::string> state;
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("nifi_test" == state.at("remote_path"));
@@ -924,11 +917,11 @@ TEST_CASE_METHOD(PersistentListSFTPTestsFixture, "ListSFTP Tracking Entities res
   plan->setProperty(list_sftp, "Remote Path", "/nifi_test");
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
-  REQUIRE(plan->getStateManagerProvider()->getCoreComponentStateManager(*list_sftp)->get(state));
+  REQUIRE(plan->getProcessContextForProcessor(list_sftp)->getStateManager()->get(state));
   REQUIRE("localhost" == state.at("hostname"));
   REQUIRE("nifiuser" == state.at("username"));
   REQUIRE("/nifi_test" == state.at("remote_path"));
@@ -958,7 +951,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities changed confi
   LogTestController::getInstance().resetStream(LogTestController::getInstance().log_output);
   plan->setProperty(list_sftp, "Remote Path", "/nifi_test");
 
-  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -240 /* 4 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file2.ext", "Test content 2", -4min);
 
   testController.runSession(plan, true);
 
@@ -974,10 +967,10 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities changed confi
 
 TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities Initial Listing Target Tracking Time Window entity outside window", "[ListSFTP][tracking-entities]") {
   plan->setProperty(list_sftp, "Listing Strategy", "Tracking Entities");
-  plan->setProperty(list_sftp, "Entity Tracking Initial Listing Target", processors::ListSFTP::ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW);
+  plan->setProperty(list_sftp, "Entity Tracking Initial Listing Target", minifi::processors::ListSFTP::ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW);
   plan->setProperty(list_sftp, "Entity Tracking Time Window", "10 minutes");
 
-  createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1", -20*60 /* 20 minutes ago */);
+  createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1", -20min);
 
   testController.runSession(plan, true);
 
@@ -987,7 +980,7 @@ TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities Initial Listi
 
 TEST_CASE_METHOD(ListSFTPTestsFixture, "ListSFTP Tracking Entities Initial Listing Target Tracking Time Window entity inside window", "[ListSFTP][tracking-entities]") {
   plan->setProperty(list_sftp, "Listing Strategy", "Tracking Entities");
-  plan->setProperty(list_sftp, "Entity Tracking Initial Listing Target", processors::ListSFTP::ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW);
+  plan->setProperty(list_sftp, "Entity Tracking Initial Listing Target", minifi::processors::ListSFTP::ENTITY_TRACKING_INITIAL_LISTING_TARGET_TRACKING_TIME_WINDOW);
   plan->setProperty(list_sftp, "Entity Tracking Time Window", "10 minutes");
 
   createFileWithModificationTimeDiff("nifi_test/file1.ext", "Test content 1");
