@@ -29,6 +29,7 @@
 #include "io/StreamPipe.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "Exception.h"
 #include "data/JSONSQLWriter.h"
 #include "data/SQLRowsetProcessor.h"
@@ -56,7 +57,7 @@ const std::string ExecuteSQL::RESULT_ROW_COUNT = "executesql.row.count";
 const std::string ExecuteSQL::INPUT_FLOW_FILE_UUID = "input.flowfile.uuid";
 
 ExecuteSQL::ExecuteSQL(const std::string& name, const utils::Identifier& uuid)
-  : SQLProcessor(name, uuid, logging::LoggerFactory<ExecuteSQL>::getLogger()) {
+  : SQLProcessor(name, uuid, core::logging::LoggerFactory<ExecuteSQL>::getLogger()) {
 }
 
 void ExecuteSQL::initialize() {
@@ -87,10 +88,7 @@ void ExecuteSQL::processOnTrigger(core::ProcessContext& context, core::ProcessSe
                       "No incoming FlowFile and the \"" + SQLSelectQuery.getName() + "\" processor property is not specified");
     }
     logger_->log_debug("Using the contents of the flow file as the SQL statement");
-    auto buffer = std::make_shared<io::BufferStream>();
-    InputStreamPipe content_reader{buffer};
-    session.read(input_flow_file, &content_reader);
-    query = std::string{reinterpret_cast<const char *>(buffer->getBuffer()), buffer->size()};
+    query = to_string(session.readBuffer(input_flow_file));
   }
   if (query.empty()) {
     throw Exception(PROCESSOR_EXCEPTION, "Empty SQL statement");
@@ -100,7 +98,7 @@ void ExecuteSQL::processOnTrigger(core::ProcessContext& context, core::ProcessSe
 
   sql::JSONSQLWriter json_writer{output_format_ == OutputType::JSONPretty};
   FlowFileGenerator flow_file_creator{session, json_writer};
-  sql::SQLRowsetProcessor sql_rowset_processor(row_set, {json_writer, flow_file_creator});
+  sql::SQLRowsetProcessor sql_rowset_processor(std::move(row_set), {json_writer, flow_file_creator});
 
   // Process rowset.
   while (size_t row_count = sql_rowset_processor.process(max_rows_)) {
@@ -121,6 +119,8 @@ void ExecuteSQL::processOnTrigger(core::ProcessContext& context, core::ProcessSe
   }
 }
 
+REGISTER_RESOURCE(ExecuteSQL, "ExecuteSQL to execute SELECT statement via ODBC.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
