@@ -20,25 +20,15 @@
 
 #include "PutSQL.h"
 
-#include <vector>
 #include <string>
-#include <memory>
-
-#include <soci/soci.h>
 
 #include "io/BufferStream.h"
-#include "io/StreamPipe.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "Exception.h"
-#include "data/DatabaseConnectors.h"
-#include "data/JSONSQLWriter.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+namespace org::apache::nifi::minifi::processors {
 
 const std::string PutSQL::ProcessorName("PutSQL");
 
@@ -54,7 +44,7 @@ const core::Property PutSQL::SQLStatement(
 const core::Relationship PutSQL::Success("success", "Database is successfully updated.");
 
 PutSQL::PutSQL(const std::string& name, const utils::Identifier& uuid)
-  : SQLProcessor(name, uuid, logging::LoggerFactory<PutSQL>::getLogger()) {
+  : SQLProcessor(name, uuid, core::logging::LoggerFactory<PutSQL>::getLogger()) {
 }
 
 void PutSQL::initialize() {
@@ -78,10 +68,7 @@ void PutSQL::processOnTrigger(core::ProcessContext& context, core::ProcessSessio
   std::string sql_statement;
   if (!context.getProperty(SQLStatement, sql_statement, flow_file)) {
     logger_->log_debug("Using the contents of the flow file as the SQL statement");
-    auto buffer = std::make_shared<io::BufferStream>();
-    InputStreamPipe read_callback{buffer};
-    session.read(flow_file, &read_callback);
-    sql_statement = std::string{reinterpret_cast<const char*>(buffer->getBuffer()), buffer->size()};
+    sql_statement = to_string(session.readBuffer(flow_file));
   }
   if (sql_statement.empty()) {
     throw Exception(PROCESSOR_EXCEPTION, "Empty SQL statement");
@@ -90,8 +77,6 @@ void PutSQL::processOnTrigger(core::ProcessContext& context, core::ProcessSessio
   connection_->prepareStatement(sql_statement)->execute(collectArguments(flow_file));
 }
 
-}  // namespace processors
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+REGISTER_RESOURCE(PutSQL, "PutSQL to execute SQL command via ODBC.");
+
+}  // namespace org::apache::nifi::minifi::processors
