@@ -25,10 +25,10 @@
 #include <unordered_map>
 #include <memory>
 
-#include "core/Resource.h"
 #include "core/ProcessSession.h"
 #include "SQLProcessor.h"
 #include "FlowFileSource.h"
+#include "data/SQLColumnIdentifier.h"
 
 namespace org {
 namespace apache {
@@ -41,23 +41,23 @@ class QueryDatabaseTable: public SQLProcessor, public FlowFileSource {
  public:
   explicit QueryDatabaseTable(const std::string& name, const utils::Identifier& uuid = {});
 
-  static const std::string RESULT_TABLE_NAME;
-  static const std::string RESULT_ROW_COUNT;
+  EXTENSIONAPI static const std::string RESULT_TABLE_NAME;
+  EXTENSIONAPI static const std::string RESULT_ROW_COUNT;
 
-  static const std::string TABLENAME_KEY;
-  static const std::string MAXVALUE_KEY_PREFIX;
+  EXTENSIONAPI static const std::string TABLENAME_KEY;
+  EXTENSIONAPI static const std::string MAXVALUE_KEY_PREFIX;
 
   //! Processor Name
-  static const std::string ProcessorName;
+  EXTENSIONAPI static const std::string ProcessorName;
 
-  static const core::Property TableName;
-  static const core::Property ColumnNames;
-  static const core::Property MaxValueColumnNames;
-  static const core::Property WhereClause;
+  EXTENSIONAPI static const core::Property TableName;
+  EXTENSIONAPI static const core::Property ColumnNames;
+  EXTENSIONAPI static const core::Property MaxValueColumnNames;
+  EXTENSIONAPI static const core::Property WhereClause;
 
-  static const std::string InitialMaxValueDynamicPropertyPrefix;
+  EXTENSIONAPI static const std::string InitialMaxValueDynamicPropertyPrefix;
 
-  static const core::Relationship Success;
+  EXTENSIONAPI static const core::Relationship Success;
 
   bool supportsDynamicProperties() override {
     return true;
@@ -81,17 +81,15 @@ class QueryDatabaseTable: public SQLProcessor, public FlowFileSource {
 
   bool saveState();
 
-  std::shared_ptr<core::CoreComponentStateManager> state_manager_;
+  core::CoreComponentStateManager* state_manager_;
   std::string table_name_;
-  std::vector<std::string> return_columns_;
+  std::unordered_set<sql::SQLColumnIdentifier> return_columns_;
   std::string queried_columns_;
   std::string extra_where_clause_;
-  std::vector<std::string> max_value_columns_;
-  std::unordered_map<std::string, std::string> max_values_;
+  std::vector<sql::SQLColumnIdentifier> max_value_columns_;
+  std::unordered_map<sql::SQLColumnIdentifier, std::string> max_values_;
 };
 
-REGISTER_RESOURCE(QueryDatabaseTable, "QueryDatabaseTable to execute SELECT statement via ODBC.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
