@@ -15,16 +15,14 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_CONTROLLERS_DATABASESERVICE_H_
-#define LIBMINIFI_INCLUDE_CONTROLLERS_DATABASESERVICE_H_
+#pragma once
+
+#include <memory>
+#include <unordered_map>
 
 #include "core/logging/LoggerConfiguration.h"
 #include "core/controller/ControllerService.h"
 #include "data/DatabaseConnectors.h"
-#include <memory>
-#include <unordered_map>
-
-#include <soci/soci.h>
 
 namespace org {
 namespace apache {
@@ -42,15 +40,13 @@ class DatabaseService : public core::controller::ControllerService {
  public:
   explicit DatabaseService(const std::string &name, const utils::Identifier &uuid = {})
       : ControllerService(name, uuid),
-        initialized_(false),
-        logger_(logging::LoggerFactory<DatabaseService>::getLogger()) {
+        initialized_(false) {
     initialize();
   }
 
   explicit DatabaseService(const std::string &name, const std::shared_ptr<Configure> &configuration)
       : ControllerService(name),
-        initialized_(false),
-        logger_(logging::LoggerFactory<DatabaseService>::getLogger()) {
+        initialized_(false) {
     setConfiguration(configuration);
     initialize();
   }
@@ -58,7 +54,7 @@ class DatabaseService : public core::controller::ControllerService {
   /**
    * Parameters needed.
    */
-  static core::Property ConnectionString;
+  EXTENSIONAPI static core::Property ConnectionString;
 
   void initialize() override;
 
@@ -84,13 +80,13 @@ class DatabaseService : public core::controller::ControllerService {
   // initialization mutex.
   std::recursive_mutex initialization_mutex_;
 
-  bool initialized_{};
+  bool initialized_;
 
   std::string connection_string_;
 
  private:
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<DatabaseService>::getLogger();
 };
 
 } /* namespace controllers */
@@ -99,5 +95,3 @@ class DatabaseService : public core::controller::ControllerService {
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif /* LIBMINIFI_INCLUDE_CONTROLLERS_DATABASESERVICE_H_ */
