@@ -16,6 +16,7 @@
  */
 
 #include "UnorderedMapKeyValueStoreService.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -24,15 +25,12 @@ namespace minifi {
 namespace controllers {
 
 UnorderedMapKeyValueStoreService::UnorderedMapKeyValueStoreService(const std::string& name, const utils::Identifier& uuid /*= utils::Identifier()*/)
-    : PersistableKeyValueStoreService(name, uuid)
-    , logger_(logging::LoggerFactory<UnorderedMapKeyValueStoreService>::getLogger()) {
+    : PersistableKeyValueStoreService(name, uuid) {
 }
 
 UnorderedMapKeyValueStoreService::UnorderedMapKeyValueStoreService(const std::string& name, const std::shared_ptr<Configure> &configuration)
-    : PersistableKeyValueStoreService(name)
-    , logger_(logging::LoggerFactory<UnorderedMapKeyValueStoreService>::getLogger())  {
+    : PersistableKeyValueStoreService(name) {
   setConfiguration(configuration);
-  initialize();
 }
 
 UnorderedMapKeyValueStoreService::~UnorderedMapKeyValueStoreService() = default;
@@ -98,6 +96,8 @@ bool UnorderedMapKeyValueStoreService::update(const std::string& key, const std:
   return true;
 }
 
+REGISTER_RESOURCE(UnorderedMapKeyValueStoreService, "A key-value service implemented by a locked std::unordered_map<std::string, std::string>");
+
 } /* namespace controllers */
 } /* namespace minifi */
 } /* namespace nifi */
