@@ -28,7 +28,6 @@
 #include "properties/Configure.h"
 #include "core/logging/Logger.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "core/Resource.h"
 #include "controllers/keyvalue/PersistableKeyValueStoreService.h"
 
 namespace org {
@@ -37,7 +36,7 @@ namespace nifi {
 namespace minifi {
 namespace controllers {
 
-/// Key-value store serice purely in RAM without disk usage
+/// Key-value store service purely in RAM without disk usage
 class UnorderedMapKeyValueStoreService : virtual public PersistableKeyValueStoreService {
  public:
   explicit UnorderedMapKeyValueStoreService(const std::string& name, const utils::Identifier& uuid = {});
@@ -46,17 +45,11 @@ class UnorderedMapKeyValueStoreService : virtual public PersistableKeyValueStore
   ~UnorderedMapKeyValueStoreService() override;
 
   bool set(const std::string& key, const std::string& value) override;
-
   bool get(const std::string& key, std::string& value) override;
-
   bool get(std::unordered_map<std::string, std::string>& kvs) override;
-
   bool remove(const std::string& key) override;
-
   bool clear() override;
-
   bool update(const std::string& key, const std::function<bool(bool /*exists*/, std::string& /*value*/)>& update_func) override;
-
   bool persist() override {
     return true;
   }
@@ -66,11 +59,9 @@ class UnorderedMapKeyValueStoreService : virtual public PersistableKeyValueStore
   std::recursive_mutex mutex_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<UnorderedMapKeyValueStoreService>::getLogger();
 };
 
-REGISTER_RESOURCE(UnorderedMapKeyValueStoreService, "A key-value service implemented by a locked std::unordered_map<std::string, std::string>");
-
 }  // namespace controllers
 }  // namespace minifi
 }  // namespace nifi
