@@ -22,6 +22,7 @@
 
 #include "utils/file/FileUtils.h"
 #include "utils/StringUtils.h"
+#include "core/Resource.h"
 
 namespace {
   std::string escape(const std::string& str) {
@@ -61,17 +62,14 @@ core::Property UnorderedMapPersistableKeyValueStoreService::File(
 UnorderedMapPersistableKeyValueStoreService::UnorderedMapPersistableKeyValueStoreService(const std::string& name, const utils::Identifier& uuid /*= utils::Identifier()*/)
     : PersistableKeyValueStoreService(name, uuid)
     , AbstractAutoPersistingKeyValueStoreService(name, uuid)
-    , UnorderedMapKeyValueStoreService(name, uuid)
-    , logger_(logging::LoggerFactory<UnorderedMapPersistableKeyValueStoreService>::getLogger()) {
+    , UnorderedMapKeyValueStoreService(name, uuid) {
 }
 
 UnorderedMapPersistableKeyValueStoreService::UnorderedMapPersistableKeyValueStoreService(const std::string& name, const std::shared_ptr<Configure> &configuration)
     : PersistableKeyValueStoreService(name)
     , AbstractAutoPersistingKeyValueStoreService(name)
-    , UnorderedMapKeyValueStoreService(name)
-    , logger_(logging::LoggerFactory<UnorderedMapPersistableKeyValueStoreService>::getLogger())  {
+    , UnorderedMapKeyValueStoreService(name) {
   setConfiguration(configuration);
-  initializeNonVirtual();
 }
 
 UnorderedMapPersistableKeyValueStoreService::~UnorderedMapPersistableKeyValueStoreService() {
@@ -133,11 +131,9 @@ bool UnorderedMapPersistableKeyValueStoreService::parseLine(const std::string& l
   return true;
 }
 
-void UnorderedMapPersistableKeyValueStoreService::initializeNonVirtual() {
+void UnorderedMapPersistableKeyValueStoreService::initialize() {
   AbstractAutoPersistingKeyValueStoreService::initialize();
-  std::set<core::Property> supportedProperties;
-  supportedProperties.insert(File);
-  updateSupportedProperties(supportedProperties);
+  updateSupportedProperties({File});
 }
 
 void UnorderedMapPersistableKeyValueStoreService::onEnable() {
@@ -249,6 +245,8 @@ bool UnorderedMapPersistableKeyValueStoreService::load() {
   return true;
 }
 
+REGISTER_RESOURCE(UnorderedMapPersistableKeyValueStoreService, "A persistable key-value service implemented by a locked std::unordered_map<std::string, std::string> and persisted into a file");
+
 }  // namespace controllers
 }  // namespace minifi
 }  // namespace nifi
