@@ -29,7 +29,6 @@
 #include "properties/Configure.h"
 #include "core/logging/Logger.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -48,9 +47,7 @@ class UnorderedMapPersistableKeyValueStoreService : public AbstractAutoPersistin
   static core::Property File;
 
   void onEnable() override;
-  void initialize() override {
-    initializeNonVirtual();
-  }
+  void initialize() override;
   void notifyStop() override;
 
   bool set(const std::string& key, const std::string& value) override;
@@ -82,13 +79,11 @@ class UnorderedMapPersistableKeyValueStoreService : public AbstractAutoPersistin
   bool parseLine(const std::string& line, std::string& key, std::string& value);
 
  private:
-  void initializeNonVirtual();
   bool persistNonVirtual();
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<UnorderedMapPersistableKeyValueStoreService>::getLogger();
 };
 
-REGISTER_RESOURCE(UnorderedMapPersistableKeyValueStoreService, "A persistable key-value service implemented by a locked std::unordered_map<std::string, std::string> and persisted into a file");
 static_assert(std::is_convertible<UnorderedMapKeyValueStoreService*, PersistableKeyValueStoreService*>::value, "UnorderedMapKeyValueStoreService is a PersistableKeyValueStoreService");
 
 }  // namespace controllers
