@@ -22,14 +22,17 @@
 
 #include <memory>
 #include <string>
+#include <optional>
+#include <regex>
+#include <shared_mutex>
 
 #include "core/Property.h"
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -37,41 +40,44 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-// AppendHostInfo Class
 class AppendHostInfo : public core::Processor {
  public:
-  // Constructor
-  /*!
-   * Create a new processor
-   */
-  AppendHostInfo(const std::string& name, const utils::Identifier& uuid = {}) // NOLINT
-      : core::Processor(name, uuid),
-        logger_(logging::LoggerFactory<AppendHostInfo>::getLogger()) {
-  }
-  // Destructor
-  virtual ~AppendHostInfo() = default;
-  // Processor Name
-  static constexpr char const* ProcessorName = "AppendHostInfo";
-  // Supported Properties
-  static core::Property InterfaceName;
-  static core::Property HostAttribute;
-  static core::Property IPAttribute;
+  static constexpr const char* REFRESH_POLICY_ON_TRIGGER = "On every trigger";
+  static constexpr const char* REFRESH_POLICY_ON_SCHEDULE = "On schedule";
 
-  // Supported Relationships
-  static core::Relationship Success;
+  explicit AppendHostInfo(const std::string& name, const utils::Identifier& uuid = {})
+      : core::Processor(name, uuid),
+        refresh_on_trigger_(false) {
+  }
+  ~AppendHostInfo() override = default;
+  static constexpr char const* ProcessorName = "AppendHostInfo";
+
+  EXTENSIONAPI static core::Property InterfaceNameFilter;
+  EXTENSIONAPI static core::Property HostAttribute;
+  EXTENSIONAPI static core::Property IPAttribute;
+  EXTENSIONAPI static core::Property RefreshPolicy;
+
+  EXTENSIONAPI static core::Relationship Success;
 
  public:
-  // OnTrigger method, implemented by NiFi AppendHostInfo
-  virtual void onTrigger(core::ProcessContext *context, core::ProcessSession *session);
-  // Initialize, over write by NiFi AppendHostInfo
-  virtual void initialize(void);
+  void onSchedule(const std::shared_ptr<core::ProcessContext>& context, const std::shared_ptr<core::ProcessSessionFactory>& sessionFactory) override;
+  void onTrigger(core::ProcessContext* context, core::ProcessSession* session) override;
+  void initialize() override;
+
+ protected:
+  virtual void refreshHostInfo();
 
  private:
-  // Logger
-  std::shared_ptr<logging::Logger> logger_;
-};
+  std::shared_mutex shared_mutex_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AppendHostInfo>::getLogger();
+  std::string hostname_attribute_name_;
+  std::string ipaddress_attribute_name_;
+  std::optional<std::regex> interface_name_filter_;
+  bool refresh_on_trigger_;
 
-REGISTER_RESOURCE(AppendHostInfo, "Appends host information such as IP address and hostname as an attribute to incoming flowfiles.");
+  std::string hostname_;
+  std::optional<std::string> ipaddresses_;
+};
 
 }  // namespace processors
 }  // namespace minifi
