@@ -24,9 +24,11 @@
 #include <set>
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "utils/StringUtils.h"
 #include "utils/TimeUtil.h"
 #include "core/TypedValues.h"
+#include "utils/gsl.h"
 
 #if defined(__clang__)
 #pragma clang diagnostic push
@@ -38,6 +40,8 @@
 #pragma GCC diagnostic ignored "-Wunused-result"
 #endif
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -92,11 +96,9 @@ void ExecuteProcess::onTrigger(core::ProcessContext *context, core::ProcessSessi
   if (context->getProperty(WorkingDir, value, flow_file)) {
     this->_workingDir = value;
   }
-  if (context->getProperty(BatchDuration.getName(), value)) {
-    core::TimeUnit unit;
-    if (core::Property::StringToTime(value, _batchDuration, unit) && core::Property::ConvertTimeUnitToMS(_batchDuration, unit, _batchDuration)) {
-      logger_->log_debug("Setting _batchDuration");
-    }
+  if (auto batch_duration = context->getProperty<core::TimePeriodValue>(BatchDuration)) {
+    _batchDuration = batch_duration->getMilliseconds();
+    logger_->log_debug("Setting _batchDuration");
   }
   if (context->getProperty(RedirectErrorStream.getName(), value)) {
     _redirectErrorStream =  org::apache::nifi::minifi::utils::StringUtils::toBool(value).value_or(false);
@@ -156,63 +158,61 @@ void ExecuteProcess::onTrigger(core::ProcessContext *context, core::ProcessSessi
       default:  // this is the code the parent runs
         // the parent isn't going to write to the pipe
         close(_pipefd[1]);
-        if (_batchDuration > 0) {
-          while (1) {
-            std::this_thread::sleep_for(std::chrono::milliseconds(_batchDuration));
+        if (_batchDuration > 0ms) {
+          while (true) {
+            std::this_thread::sleep_for(_batchDuration);
             char buffer[4096];
-            int numRead = read(_pipefd[0], buffer, sizeof(buffer));
+            const auto  numRead = read(_pipefd[0], buffer, sizeof(buffer));
             if (numRead <= 0)
               break;
-            logger_->log_debug("Execute Command Respond %d", numRead);
-            ExecuteProcess::WriteCallback callback(buffer, numRead);
+            logger_->log_debug("Execute Command Respond %zd", numRead);
             auto flowFile = session->create();
             if (!flowFile)
               continue;
             flowFile->addAttribute("command", _command);
             flowFile->addAttribute("command.arguments", _commandArgument);
-            session->write(flowFile, &callback);
+            session->writeBuffer(flowFile, gsl::make_span(buffer, gsl::narrow<size_t>(numRead)));
             session->transfer(flowFile, Success);
             session->commit();
           }
         } else {
           char buffer[4096];
           char *bufPtr = buffer;
-          int totalRead = 0;
+          size_t totalRead = 0;
           std::shared_ptr<core::FlowFile> flowFile = nullptr;
           while (true) {
-            int numRead = read(_pipefd[0], bufPtr, (sizeof(buffer) - totalRead));
+            const auto numRead = read(_pipefd[0], bufPtr, (sizeof(buffer) - totalRead));
             if (numRead <= 0) {
               if (totalRead > 0) {
-                logger_->log_debug("Execute Command Respond %d", totalRead);
+                logger_->log_debug("Execute Command Respond %zu", totalRead);
                 // child exits and close the pipe
-                ExecuteProcess::WriteCallback callback(buffer, totalRead);
+                const auto buffer_span = gsl::make_span(buffer, totalRead);
                 if (!flowFile) {
                   flowFile = session->create();
                   if (!flowFile)
                     break;
                   flowFile->addAttribute("command", _command);
                   flowFile->addAttribute("command.arguments", _commandArgument);
-                  session->write(flowFile, &callback);
+                  session->writeBuffer(flowFile, buffer_span);
                 } else {
-                  session->append(flowFile, &callback);
+                  session->appendBuffer(flowFile, buffer_span);
                 }
                 session->transfer(flowFile, Success);
               }
               break;
             } else {
-              if (numRead == static_cast<int>((sizeof(buffer) - totalRead))) {
+              if (numRead == static_cast<ssize_t>((sizeof(buffer) - totalRead))) {
                 // we reach the max buffer size
-                logger_->log_debug("Execute Command Max Respond %d", sizeof(buffer));
-                ExecuteProcess::WriteCallback callback(buffer, sizeof(buffer));
+                logger_->log_debug("Execute Command Max Respond %zu", sizeof(buffer));
                 if (!flowFile) {
                   flowFile = session->create();
                   if (!flowFile)
                     continue;
                   flowFile->addAttribute("command", _command);
                   flowFile->addAttribute("command.arguments", _commandArgument);
-                  session->write(flowFile, &callback);
+                  session->writeBuffer(flowFile, buffer);
                 } else {
-                  session->append(flowFile, &callback);
+                  session->appendBuffer(flowFile, buffer);
                 }
                 // Rewind
                 totalRead = 0;
@@ -238,6 +238,11 @@ void ExecuteProcess::onTrigger(core::ProcessContext *context, core::ProcessSessi
     }
   }
 }
+
+REGISTER_RESOURCE(ExecuteProcess, "Runs an operating system command specified by the user and writes the output of that command to a FlowFile. If the command is expected to be long-running,"
+                  "the Processor can output the partial data on a specified interval. When this option is used, the output is expected to be in textual format,"
+                  "as it typically does not make sense to split binary data on arbitrary time-based intervals.");
+
 #endif
 } /* namespace processors */
 } /* namespace minifi */
