@@ -39,9 +39,9 @@
 #include "core/logging/LoggerConfiguration.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
-#include "core/Resource.h"
 #include "FlowFileRecord.h"
 #include "io/BaseStream.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -58,16 +58,14 @@ class ExecuteProcess : public core::Processor {
    * Create a new processor
    */
   ExecuteProcess(const std::string& name, const utils::Identifier& uuid = {}) // NOLINT
-      : Processor(name, uuid),
-        logger_(logging::LoggerFactory<ExecuteProcess>::getLogger()) {
+      : Processor(name, uuid) {
     _redirectErrorStream = false;
-    _batchDuration = 0;
     _workingDir = ".";
     _processRunning = false;
     _pid = 0;
   }
   // Destructor
-  virtual ~ExecuteProcess() {
+  ~ExecuteProcess() override {
     if (_processRunning && _pid > 0)
       kill(_pid, SIGTERM);
   }
@@ -82,38 +80,20 @@ class ExecuteProcess : public core::Processor {
   // Supported Relationships
   static core::Relationship Success;
 
-  // Nest Callback Class for write stream
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(char *data, uint64_t size)
-        : _data(data),
-          _dataSize(size) {
-    }
-    char *_data;
-    uint64_t _dataSize;
-    // void process(std::ofstream *stream) {
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      int64_t ret = 0;
-      if (_data && _dataSize > 0)
-        ret = stream->write(reinterpret_cast<uint8_t*>(_data), _dataSize);
-      return ret;
-    }
-  };
-
  public:
   // OnTrigger method, implemented by NiFi ExecuteProcess
-  virtual void onTrigger(core::ProcessContext *context, core::ProcessSession *session);
+  void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
   // Initialize, over write by NiFi ExecuteProcess
-  virtual void initialize(void);
+  void initialize() override;
 
  private:
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ExecuteProcess>::getLogger();
   // Property
   std::string _command;
   std::string _commandArgument;
   std::string _workingDir;
-  int64_t _batchDuration;
+  std::chrono::milliseconds _batchDuration  = std::chrono::milliseconds(0);
   bool _redirectErrorStream;
   // Full command
   std::string _fullCommand;
@@ -123,9 +103,6 @@ class ExecuteProcess : public core::Processor {
   pid_t _pid;
 };
 
-REGISTER_RESOURCE(ExecuteProcess, "Runs an operating system command specified by the user and writes the output of that command to a FlowFile. If the command is expected to be long-running,"
-                  "the Processor can output the partial data on a specified interval. When this option is used, the output is expected to be in textual format,"
-                  "as it typically does not make sense to split binary data on arbitrary time-based intervals.");
 #endif
 }  // namespace processors
 }  // namespace minifi
