@@ -23,7 +23,6 @@
 #include <memory>
 #include <map>
 #include <set>
-#include <regex>
 #include <iostream>
 #include <sstream>
 #include <utility>
@@ -31,8 +30,9 @@
 #include "ExtractText.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "core/FlowFile.h"
-
+#include "utils/gsl.h"
 #include "utils/RegexUtils.h"
 
 namespace org {
@@ -107,16 +107,16 @@ void ExtractText::onTrigger(core::ProcessContext *context, core::ProcessSession
     return;
   }
 
-  ReadCallback cb(flowFile, context, logger_);
-  session->read(flowFile, &cb);
+  session->read(flowFile, ReadCallback{flowFile, context, logger_});
   session->transfer(flowFile, Success);
 }
 
-int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-  int64_t ret = 0;
-  uint64_t read_size = 0;
+int64_t ExtractText::ReadCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) const {
+  size_t read_size = 0;
   bool regex_mode;
-  uint64_t size_limit = flowFile_->getSize();
+  size_t size_limit = flowFile_->getSize();
+  std::vector<std::byte> buffer;
+  buffer.resize(std::min(gsl::narrow<size_t>(flowFile_->getSize()), MAX_BUFFER_SIZE));
 
   std::string attrKey, sizeLimitStr;
   ctx_->getProperty(Attribute.getName(), attrKey);
@@ -126,22 +126,22 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
   if (sizeLimitStr.empty())
     size_limit = DEFAULT_SIZE_LIMIT;
   else if (sizeLimitStr != "0")
-    size_limit = std::stoi(sizeLimitStr);
+    size_limit = static_cast<size_t>(std::stoi(sizeLimitStr));
 
   std::ostringstream contentStream;
 
   while (read_size < size_limit) {
     // Don't read more than config limit or the size of the buffer
-    int length = gsl::narrow<int>(std::min<uint64_t>(size_limit - read_size, buffer_.size()));
-    ret = stream->read(buffer_, length);
+    const auto length = std::min(size_limit - read_size, buffer.size());
+    const auto ret = stream->read(gsl::make_span(buffer).subspan(0, length));
 
-    if (ret < 0) {
+    if (io::isError(ret)) {
       return -1;  // Stream error
     } else if (ret == 0) {
       break;  // End of stream, no more data
     }
 
-    contentStream.write(reinterpret_cast<const char*>(buffer_.data()), ret);
+    contentStream.write(reinterpret_cast<const char*>(buffer.data()), gsl::narrow<std::streamsize>(ret));
     read_size += ret;
     if (contentStream.fail()) {
       return -1;
@@ -149,11 +149,10 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
   }
 
   if (regex_mode) {
-    std::vector<utils::Regex::Mode> rgx_mode;
-
     bool insensitive;
+    std::vector<utils::Regex::Mode> regex_flags;
     if (ctx_->getProperty(InsensitiveMatch.getName(), insensitive) && insensitive) {
-      rgx_mode.push_back(utils::Regex::Mode::ICASE);
+      regex_flags.push_back(utils::Regex::Mode::ICASE);
     }
 
     bool ignoregroupzero;
@@ -162,9 +161,11 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
     bool repeatingcapture;
     ctx_->getProperty(EnableRepeatingCaptureGroup.getName(), repeatingcapture);
 
-    int maxCaptureSizeProperty;
-    ctx_->getProperty(MaxCaptureGroupLen.getName(), maxCaptureSizeProperty);
-    size_t maxCaptureSize = gsl::narrow<size_t>(maxCaptureSizeProperty);
+    const size_t maxCaptureSize = [this] {
+      uint64_t val;
+      ctx_->getProperty(MaxCaptureGroupLen.getName(), val);
+      return gsl::narrow<size_t>(val);
+    }();
 
     std::string contentStr = contentStream.str();
 
@@ -179,9 +180,9 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
       int matchcount = 0;
 
       try {
-        utils::Regex rgx(value, rgx_mode);
-        while (rgx.match(workStr)) {
-          const std::vector<std::string> &matches = rgx.getResult();
+        utils::Regex rgx(value, regex_flags);
+        utils::SMatch matches;
+        while (utils::regexSearch(workStr, matches, rgx)) {
           size_t i = ignoregroupzero ? 1 : 0;
 
           for (; i < matches.size(); ++i, ++matchcount) {
@@ -197,7 +198,7 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
           if (!repeatingcapture) {
             break;
           }
-          workStr = rgx.getSuffix();
+          workStr = matches.suffix();
         }
       } catch (const Exception &e) {
         logger_->log_error("%s error encountered when trying to construct regular expression from property (key: %s) value: %s",
@@ -212,16 +213,17 @@ int64_t ExtractText::ReadCallback::process(const std::shared_ptr<io::BaseStream>
   } else {
     flowFile_->setAttribute(attrKey, contentStream.str());
   }
-  return read_size;
+  return gsl::narrow<int64_t>(read_size);
 }
 
-ExtractText::ReadCallback::ReadCallback(std::shared_ptr<core::FlowFile> flowFile, core::ProcessContext *ctx,  std::shared_ptr<logging::Logger> lgr)
+ExtractText::ReadCallback::ReadCallback(std::shared_ptr<core::FlowFile> flowFile, core::ProcessContext *ctx,  std::shared_ptr<core::logging::Logger> lgr)
     : flowFile_(std::move(flowFile)),
       ctx_(ctx),
       logger_(std::move(lgr)) {
-  buffer_.resize(std::min(gsl::narrow<size_t>(flowFile_->getSize()), MAX_BUFFER_SIZE));
 }
 
+REGISTER_RESOURCE(ExtractText, "Extracts the content of a FlowFile and places it into an attribute.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
