@@ -40,6 +40,7 @@
 #include "core/ProcessSession.h"
 #include "core/PropertyValidation.h"
 #include "core/TypedValues.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -148,16 +149,16 @@ void GenerateFlowFile::onTrigger(core::ProcessContext* /*context*/, core::Proces
       if (fileSize_ > 0) {
         generateData(data, textData_);
       }
-      GenerateFlowFile::WriteCallback callback(std::move(data));
-      session->write(flowFile, &callback);
+      session->writeBuffer(flowFile, data);
     } else {
-      GenerateFlowFile::WriteCallback callback(data_);
-      session->write(flowFile, &callback);
+      session->writeBuffer(flowFile, data_);
     }
     session->transfer(flowFile, Success);
   }
 }
 
+REGISTER_RESOURCE(GenerateFlowFile, "This processor creates FlowFiles with random data or custom content. GenerateFlowFile is useful for load testing, configuration, and simulation.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
