@@ -29,8 +29,8 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "utils/gsl.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -45,47 +45,32 @@ class GenerateFlowFile : public core::Processor {
    * Create a new processor
    */
   GenerateFlowFile(const std::string& name, const utils::Identifier& uuid = {}) // NOLINT
-      : Processor(name, uuid), logger_(logging::LoggerFactory<GenerateFlowFile>::getLogger()) {
+      : Processor(name, uuid) {
     batchSize_ = 1;
     uniqueFlowFile_ = true;
     fileSize_ = 1024;
     textData_ = false;
   }
   // Destructor
-  virtual ~GenerateFlowFile() = default;
+  ~GenerateFlowFile() override = default;
   // Processor Name
-  static constexpr char const* ProcessorName = "GenerateFlowFile";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "GenerateFlowFile";
   // Supported Properties
-  static core::Property FileSize;
-  static core::Property BatchSize;
-  static core::Property DataFormat;
-  static core::Property UniqueFlowFiles;
-  static core::Property CustomText;
-  static const char *DATA_FORMAT_TEXT;
+  EXTENSIONAPI static core::Property FileSize;
+  EXTENSIONAPI static core::Property BatchSize;
+  EXTENSIONAPI static core::Property DataFormat;
+  EXTENSIONAPI static core::Property UniqueFlowFiles;
+  EXTENSIONAPI static core::Property CustomText;
+  EXTENSIONAPI static const char *DATA_FORMAT_TEXT;
   // Supported Relationships
-  static core::Relationship Success;
-  // Nest Callback Class for write stream
-  class WriteCallback : public OutputStreamCallback {
-   public:
-    WriteCallback(std::vector<char> && data) : data_(std::move(data)) { // NOLINT
-    }
-    WriteCallback(const std::vector<char>& data) : data_(data) { // NOLINT
-    }
-    std::vector<char> data_;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      int64_t ret = 0;
-      if (data_.size() > 0)
-        ret = stream->write(reinterpret_cast<uint8_t*>(&data_[0]), gsl::narrow<int>(data_.size()));
-      return ret;
-    }
-  };
+  EXTENSIONAPI static core::Relationship Success;
 
  public:
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) override;
   // OnTrigger method, implemented by NiFi GenerateFlowFile
   void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
   // Initialize, over write by NiFi GenerateFlowFile
-  void initialize(void) override;
+  void initialize() override;
 
  protected:
   std::vector<char> data_;
@@ -101,11 +86,9 @@ class GenerateFlowFile : public core::Processor {
   }
 
   // logger instance
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GenerateFlowFile>::getLogger();
 };
 
-REGISTER_RESOURCE(GenerateFlowFile, "This processor creates FlowFiles with random data or custom content. GenerateFlowFile is useful for load testing, configuration, and simulation.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
