@@ -25,23 +25,23 @@
 #include <queue>
 #include <map>
 #include <memory>
+#include <regex>
 #include <set>
 #include <string>
 
 #include "utils/StringUtils.h"
 #include "utils/file/FileUtils.h"
 #include "utils/TimeUtil.h"
-#include "utils/RegexUtils.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "core/TypedValues.h"
 #include "utils/FileReaderCallback.h"
+#include "utils/RegexUtils.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace processors {
+using namespace std::literals::chrono_literals;
+
+namespace org::apache::nifi::minifi::processors {
 
 core::Property GetFile::BatchSize(
     core::PropertyBuilder::createProperty("Batch Size")->withDescription("The maximum number of files to pull in each iteration")->withDefaultValue<uint32_t>(10)->build());
@@ -118,8 +118,10 @@ void GetFile::onSchedule(core::ProcessContext *context, core::ProcessSessionFact
     request_.keepSourceFile = org::apache::nifi::minifi::utils::StringUtils::toBool(value).value_or(false);
   }
 
-  context->getProperty(MaxAge.getName(), request_.maxAge);
-  context->getProperty(MinAge.getName(), request_.minAge);
+  if (auto max_age = context->getProperty<core::TimePeriodValue>(MaxAge))
+    request_.maxAge = max_age->getMilliseconds();
+  if (auto min_age = context->getProperty<core::TimePeriodValue>(MinAge))
+    request_.minAge = min_age->getMilliseconds();
 
   if (context->getProperty(MaxSize.getName(), value)) {
     core::Property::StringToInt(value, request_.maxSize);
@@ -128,7 +130,9 @@ void GetFile::onSchedule(core::ProcessContext *context, core::ProcessSessionFact
     core::Property::StringToInt(value, request_.minSize);
   }
 
-  context->getProperty(PollInterval.getName(), request_.pollInterval);
+  if (const auto poll_interval = context->getProperty<core::TimePeriodValue>(PollInterval)) {
+    request_.pollInterval = poll_interval->getMilliseconds();
+  }
 
   if (context->getProperty(Recurse.getName(), value)) {
     request_.recursive = org::apache::nifi::minifi::utils::StringUtils::toBool(value).value_or(true);
@@ -141,7 +145,7 @@ void GetFile::onSchedule(core::ProcessContext *context, core::ProcessSessionFact
   if (!context->getProperty(Directory.getName(), value)) {
     throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Input Directory property is missing");
   }
-  if (!utils::file::FileUtils::is_directory(value.c_str())) {
+  if (!utils::file::is_directory(value)) {
     throw Exception(PROCESS_SCHEDULE_EXCEPTION, "Input Directory \"" + value + "\" is not a directory");
   }
   request_.inputDirectory = value;
@@ -153,9 +157,9 @@ void GetFile::onTrigger(core::ProcessContext* /*context*/, core::ProcessSession*
   const bool is_dir_empty_before_poll = isListingEmpty();
   logger_->log_debug("Listing is %s before polling directory", is_dir_empty_before_poll ? "empty" : "not empty");
   if (is_dir_empty_before_poll) {
-    if (request_.pollInterval == 0 || (utils::timeutils::getTimeMillis() - last_listing_time_) > request_.pollInterval) {
+    if (request_.pollInterval == 0ms || (std::chrono::system_clock::now() - last_listing_time_.load()) > request_.pollInterval) {
       performListing(request_);
-      last_listing_time_.store(utils::timeutils::getTimeMillis());
+      last_listing_time_.store(std::chrono::system_clock::now());
     }
   }
 
@@ -186,8 +190,7 @@ void GetFile::getSingleFile(core::ProcessSession& session, const std::string& fi
   flow_file->addAttribute(core::SpecialFlowAttribute::ABSOLUTE_PATH, file_name);
 
   try {
-    utils::FileReaderCallback file_reader_callback{file_name};
-    session.write(flow_file, &file_reader_callback);
+    session.write(flow_file, utils::FileReaderCallback{file_name});
     session.transfer(flow_file, Success);
     if (!request_.keepSourceFile) {
       auto remove_status = remove(file_name.c_str());
@@ -229,19 +232,17 @@ std::queue<std::string> GetFile::pollListing(uint64_t batch_size) {
 bool GetFile::fileMatchesRequestCriteria(std::string fullName, std::string name, const GetFileRequest &request) {
   logger_->log_trace("Checking file: %s", fullName);
 
-#ifdef WIN32
-  struct _stat64 statbuf;
-  if (_stat64(fullName.c_str(), &statbuf) != 0) {
+  std::error_code ec;
+  uint64_t file_size = std::filesystem::file_size(fullName, ec);
+  if (ec) {
+    logger_->log_error("file_size of %s: %s", fullName, ec.message());
     return false;
   }
-#else
-  struct stat statbuf;
-  if (stat(fullName.c_str(), &statbuf) != 0) {
+  const auto modifiedTime = std::filesystem::last_write_time(fullName, ec);
+  if (ec) {
+    logger_->log_error("last_write_time of %s: %s", fullName, ec.message());
     return false;
   }
-#endif
-  uint64_t file_size = gsl::narrow<uint64_t>(statbuf.st_size);
-  uint64_t modifiedTime = gsl::narrow<uint64_t>(statbuf.st_mtime) * 1000;
 
   if (request.minSize > 0 && file_size < request.minSize)
     return false;
@@ -249,17 +250,17 @@ bool GetFile::fileMatchesRequestCriteria(std::string fullName, std::string name,
   if (request.maxSize > 0 && file_size > request.maxSize)
     return false;
 
-  uint64_t fileAge = utils::timeutils::getTimeMillis() - modifiedTime;
-  if (request.minAge > 0 && fileAge < request.minAge)
+  auto fileAge = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::file_clock::now() - modifiedTime);
+  if (request.minAge > 0ms && fileAge < request.minAge)
     return false;
-  if (request.maxAge > 0 && fileAge > request.maxAge)
+  if (request.maxAge > 0ms && fileAge > request.maxAge)
     return false;
 
-  if (request.ignoreHiddenFile && utils::file::FileUtils::is_hidden(fullName))
+  if (request.ignoreHiddenFile && utils::file::is_hidden(fullName))
     return false;
 
   utils::Regex rgx(request.fileFilter);
-  if (!rgx.match(name)) {
+  if (!utils::regexSearch(name, rgx)) {
     return false;
   }
 
@@ -270,13 +271,13 @@ bool GetFile::fileMatchesRequestCriteria(std::string fullName, std::string name,
 
 void GetFile::performListing(const GetFileRequest &request) {
   auto callback = [this, request](const std::string& dir, const std::string& filename) -> bool {
-    std::string fullpath = dir + utils::file::FileUtils::get_separator() + filename;
+    std::string fullpath = dir + utils::file::get_separator() + filename;
     if (fileMatchesRequestCriteria(fullpath, filename, request)) {
       putListing(fullpath);
     }
     return isRunning();
   };
-  utils::file::FileUtils::list_dir(request.inputDirectory, callback, logger_, request.recursive);
+  utils::file::list_dir(request.inputDirectory, callback, logger_, request.recursive);
 }
 
 int16_t GetFile::getMetricNodes(std::vector<std::shared_ptr<state::response::ResponseNode>> &metric_vector) {
@@ -284,8 +285,6 @@ int16_t GetFile::getMetricNodes(std::vector<std::shared_ptr<state::response::Res
   return 0;
 }
 
-}  // namespace processors
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+REGISTER_RESOURCE(GetFile, "Creates FlowFiles from files in a directory. MiNiFi will ignore files for which it doesn't have read permissions.");
+
+}  // namespace org::apache::nifi::minifi::processors
