@@ -29,8 +29,8 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -41,12 +41,12 @@ namespace processors {
 struct GetFileRequest {
   bool recursive = true;
   bool keepSourceFile = false;
-  uint64_t minAge = 0;
-  uint64_t maxAge = 0;
+  std::chrono::milliseconds minAge{0};
+  std::chrono::milliseconds maxAge{0};
   uint64_t minSize = 0;
   uint64_t maxSize = 0;
   bool ignoreHiddenFile = true;
-  uint64_t pollInterval = 0;
+  std::chrono::milliseconds pollInterval{0};
   uint64_t batchSize = 10;
   std::string fileFilter = "[^\\.].*";
   std::string inputDirectory;
@@ -113,29 +113,27 @@ class GetFile : public core::Processor, public state::response::MetricsNodeSourc
    */
   explicit GetFile(const std::string& name, const utils::Identifier& uuid = {})
       : Processor(name, uuid),
-        metrics_(std::make_shared<GetFileMetrics>()),
-        last_listing_time_(0),
-        logger_(logging::LoggerFactory<GetFile>::getLogger()) {
+        metrics_(std::make_shared<GetFileMetrics>()) {
   }
   // Destructor
   ~GetFile() override = default;
 
   // Processor Name
-  static constexpr char const* ProcessorName = "GetFile";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "GetFile";
   // Supported Properties
-  static core::Property Directory;
-  static core::Property Recurse;
-  static core::Property KeepSourceFile;
-  static core::Property MinAge;
-  static core::Property MaxAge;
-  static core::Property MinSize;
-  static core::Property MaxSize;
-  static core::Property IgnoreHiddenFile;
-  static core::Property PollInterval;
-  static core::Property BatchSize;
-  static core::Property FileFilter;
+  EXTENSIONAPI static core::Property Directory;
+  EXTENSIONAPI static core::Property Recurse;
+  EXTENSIONAPI static core::Property KeepSourceFile;
+  EXTENSIONAPI static core::Property MinAge;
+  EXTENSIONAPI static core::Property MaxAge;
+  EXTENSIONAPI static core::Property MinSize;
+  EXTENSIONAPI static core::Property MaxSize;
+  EXTENSIONAPI static core::Property IgnoreHiddenFile;
+  EXTENSIONAPI static core::Property PollInterval;
+  EXTENSIONAPI static core::Property BatchSize;
+  EXTENSIONAPI static core::Property FileFilter;
   // Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
 
  public:
   /**
@@ -153,7 +151,7 @@ class GetFile : public core::Processor, public state::response::MetricsNodeSourc
   void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
 
   // Initialize, over write by NiFi GetFile
-  void initialize(void) override;
+  void initialize() override;
   /**
    * performs a listing on the directory.
    * @param request get file request.
@@ -177,12 +175,10 @@ class GetFile : public core::Processor, public state::response::MetricsNodeSourc
   GetFileRequest request_;
   std::queue<std::string> directory_listing_;
   mutable std::mutex directory_listing_mutex_;
-  std::atomic<uint64_t> last_listing_time_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::atomic<std::chrono::time_point<std::chrono::system_clock>> last_listing_time_{};
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GetFile>::getLogger();
 };
 
-REGISTER_RESOURCE(GetFile, "Creates FlowFiles from files in a directory. MiNiFi will ignore files for which it doesn't have read permissions.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
