@@ -31,11 +31,14 @@
 #include <string>
 
 #include "io/ClientSocket.h"
+#include "io/StreamFactory.h"
 #include "utils/gsl.h"
 #include "utils/StringUtils.h"
 #include "utils/TimeUtil.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/ProcessSessionFactory.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -82,9 +85,7 @@ int16_t DataHandler::handle(std::string source, uint8_t *message, size_t size, b
   std::shared_ptr<core::ProcessSession> my_session = sessionFactory_->createSession();
   std::shared_ptr<core::FlowFile> flowFile = my_session->create();
 
-  DataHandlerCallback callback(message, size);
-
-  my_session->write(flowFile, &callback);
+  my_session->writeBuffer(flowFile, gsl::make_span(reinterpret_cast<const std::byte*>(message), size));
 
   my_session->putAttribute(flowFile, SOURCE_ENDPOINT_ATTRIBUTE, source);
 
@@ -143,17 +144,16 @@ void GetTCP::onSchedule(const std::shared_ptr<core::ProcessContext> &context, co
     logger_->log_trace("EOM is passed in as %s", value);
     int64_t byteValue = 0;
     core::Property::StringToInt(value, byteValue);
-    endOfMessageByte = byteValue & 0xFF;
+    endOfMessageByte = static_cast<std::byte>(byteValue & 0xFF);
   }
 
-  logger_->log_trace("EOM is defined as %i", endOfMessageByte);
+  logger_->log_trace("EOM is defined as %i", static_cast<int>(endOfMessageByte));
 
-  std::string reconnect_interval_str;
-  if (context->getProperty(ReconnectInterval.getName(), reconnect_interval_str) &&
-      core::Property::getTimeMSFromString(reconnect_interval_str, reconnect_interval_)) {
-    logger_->log_debug("Reconnect interval is %llu ms", reconnect_interval_);
+  if (auto reconnect_interval = context->getProperty<core::TimePeriodValue>(ReconnectInterval)) {
+    reconnect_interval_ = reconnect_interval->getMilliseconds();
+    logger_->log_debug("Reconnect interval is %" PRId64 " ms", reconnect_interval_.count());
   } else {
-    logger_->log_debug("Reconnect interval using default value of %llu ms", reconnect_interval_);
+    logger_->log_debug("Reconnect interval using default value of %" PRId64 " ms", reconnect_interval_.count());
   }
 
   handler_ = std::unique_ptr<DataHandler>(new DataHandler(sessionFactory));
@@ -161,20 +161,20 @@ void GetTCP::onSchedule(const std::shared_ptr<core::ProcessContext> &context, co
   f_ex = [&] {
     std::unique_ptr<io::Socket> socket_ptr;
     // reuse the byte buffer.
-      std::vector<uint8_t> buffer;
+      std::vector<std::byte> buffer;
       int reconnects = 0;
       do {
         if ( socket_ring_buffer_.try_dequeue(socket_ptr) ) {
           buffer.resize(receive_buffer_size_);
-          int size_read = socket_ptr->read(buffer.data(), gsl::narrow<int>(receive_buffer_size_), false);
-          if (size_read >= 0) {
-            if (size_read > 0) {
+          const auto size_read = socket_ptr->read(buffer, false);
+          if (!io::isError(size_read)) {
+            if (size_read != 0) {
               // determine cut location
-              int startLoc = 0, i = 0;
-              for (; i < size_read; i++) {
+              size_t startLoc = 0;
+              for (size_t i = 0; i < size_read; i++) {
                 if (buffer.at(i) == endOfMessageByte && i > 0) {
                   if (i-startLoc > 0) {
-                    handler_->handle(socket_ptr->getHostname(), buffer.data()+startLoc, (i-startLoc), true);
+                    handler_->handle(socket_ptr->getHostname(), reinterpret_cast<uint8_t*>(buffer.data())+startLoc, (i-startLoc), true);
                   }
                   startLoc = i;
                 }
@@ -182,34 +182,34 @@ void GetTCP::onSchedule(const std::shared_ptr<core::ProcessContext> &context, co
               if (startLoc > 0) {
                 logger_->log_trace("Starting at %i, ending at %i", startLoc, size_read);
                 if (size_read-startLoc > 0) {
-                  handler_->handle(socket_ptr->getHostname(), buffer.data()+startLoc, (size_read-startLoc), true);
+                  handler_->handle(socket_ptr->getHostname(), reinterpret_cast<uint8_t*>(buffer.data())+startLoc, (size_read-startLoc), true);
                 }
               } else {
                 logger_->log_trace("Handling at %i, ending at %i", startLoc, size_read);
                 if (size_read > 0) {
-                  handler_->handle(socket_ptr->getHostname(), buffer.data(), size_read, false);
+                  handler_->handle(socket_ptr->getHostname(), reinterpret_cast<uint8_t*>(buffer.data()), size_read, false);
                 }
               }
               reconnects = 0;
             }
             socket_ring_buffer_.enqueue(std::move(socket_ptr));
-          } else if (size_read == -2 && stay_connected_) {
+          } else if (size_read == static_cast<size_t>(-2) && stay_connected_) {
             if (++reconnects > connection_attempt_limit_) {
               logger_->log_info("Too many reconnects, exiting thread");
               socket_ptr->close();
               return -1;
             }
-            logger_->log_info("Sleeping for %" PRIu64 " msec before attempting to reconnect", reconnect_interval_);
-            std::this_thread::sleep_for(std::chrono::milliseconds(reconnect_interval_));
+            logger_->log_info("Sleeping for %" PRId64 " msec before attempting to reconnect", int64_t{reconnect_interval_.count()});
+            std::this_thread::sleep_for(reconnect_interval_);
             socket_ring_buffer_.enqueue(std::move(socket_ptr));
           } else {
             socket_ptr->close();
-            std::this_thread::sleep_for(std::chrono::milliseconds(reconnect_interval_));
+            std::this_thread::sleep_for(reconnect_interval_);
             logger_->log_info("Read response returned a -1 from socket, exiting thread");
             return -1;
           }
         } else {
-          std::this_thread::sleep_for(std::chrono::milliseconds(reconnect_interval_));
+          std::this_thread::sleep_for(reconnect_interval_);
           logger_->log_info("Could not use socket, exiting thread");
           return -1;
         }
@@ -287,18 +287,16 @@ void GetTCP::onTrigger(const std::shared_ptr<core::ProcessContext> &context, con
       auto* future = new std::future<int>();
       std::unique_ptr<utils::AfterExecute<int>> after_execute = std::unique_ptr<utils::AfterExecute<int>>(new SocketAfterExecute(running_, endpoint, &live_clients_, &mutex_));
       utils::Worker<int> functor(f_ex, "workers", std::move(after_execute));
-      if (client_thread_pool_.execute(std::move(functor), *future)) {
-        live_clients_[endpoint] = future;
-      }
+      client_thread_pool_.execute(std::move(functor), *future);
+      live_clients_[endpoint] = future;
     } else {
       if (!endPointFuture->second->valid()) {
         delete endPointFuture->second;
         auto* future = new std::future<int>();
         std::unique_ptr<utils::AfterExecute<int>> after_execute = std::unique_ptr<utils::AfterExecute<int>>(new SocketAfterExecute(running_, endpoint, &live_clients_, &mutex_));
         utils::Worker<int> functor(f_ex, "workers", std::move(after_execute));
-        if (client_thread_pool_.execute(std::move(functor), *future)) {
-          live_clients_[endpoint] = future;
-        }
+        client_thread_pool_.execute(std::move(functor), *future);
+        live_clients_[endpoint] = future;
       } else {
         logger_->log_debug("Thread still running for %s", endPointFuture->first);
         // we have a thread corresponding to this.
@@ -314,6 +312,8 @@ int16_t GetTCP::getMetricNodes(std::vector<std::shared_ptr<state::response::Resp
   return 0;
 }
 
+REGISTER_RESOURCE(GetTCP, "Establishes a TCP Server that defines and retrieves one or more byte messages from clients");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
