@@ -30,12 +30,13 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "concurrentqueue.h"
+#include "io/ClientSocket.h"
 #include "utils/ThreadPool.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "controllers/SSLContextService.h"
 #include "utils/gsl.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -88,24 +89,6 @@ class SocketAfterExecute : public utils::AfterExecute<int> {
   std::map<std::string, std::future<int>*> *list_;
 };
 
-class DataHandlerCallback : public OutputStreamCallback {
- public:
-  DataHandlerCallback(uint8_t *message, size_t size)
-      : message_(message),
-        size_(size) {
-  }
-
-  ~DataHandlerCallback() override = default;
-
-  int64_t process(const std::shared_ptr<io::BaseStream>& stream) override {
-    return stream->write(message_, gsl::narrow<int>(size_));
-  }
-
- private:
-  uint8_t *message_;
-  size_t size_;
-};
-
 class DataHandler {
  public:
   DataHandler(std::shared_ptr<core::ProcessSessionFactory> sessionFactory) // NOLINT
@@ -168,41 +151,39 @@ class GetTCPMetrics : public state::response::ResponseNode {
 // GetTCP Class
 class GetTCP : public core::Processor, public state::response::MetricsNodeSource {
  public:
-// Constructor
-  /*!
-   * Create a new processor
-   */
   explicit GetTCP(const std::string& name, const utils::Identifier& uuid = {})
       : Processor(name, uuid),
         running_(false),
         stay_connected_(true),
         concurrent_handlers_(2),
-        endOfMessageByte(13),
-        reconnect_interval_(5000),
+        endOfMessageByte(static_cast<std::byte>(13)),
         receive_buffer_size_(16 * 1024 * 1024),
         connection_attempt_limit_(3),
-        ssl_service_(nullptr),
-        logger_(logging::LoggerFactory<GetTCP>::getLogger()) {
+        ssl_service_(nullptr) {
     metrics_ = std::make_shared<GetTCPMetrics>();
   }
-// Destructor
-  ~GetTCP() override = default;
+
+  ~GetTCP() override {
+    // thread pool must be shut down first before members it is using are destructed, otherwise segfault is possible
+    client_thread_pool_.shutdown();
+  }
+
 // Processor Name
-  static constexpr char const* ProcessorName = "GetTCP";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "GetTCP";
 
   // Supported Properties
-  static core::Property EndpointList;
-  static core::Property ConcurrentHandlers;
-  static core::Property ReconnectInterval;
-  static core::Property StayConnected;
-  static core::Property ReceiveBufferSize;
-  static core::Property SSLContextService;
-  static core::Property ConnectionAttemptLimit;
-  static core::Property EndOfMessageByte;
+  EXTENSIONAPI static core::Property EndpointList;
+  EXTENSIONAPI static core::Property ConcurrentHandlers;
+  EXTENSIONAPI static core::Property ReconnectInterval;
+  EXTENSIONAPI static core::Property StayConnected;
+  EXTENSIONAPI static core::Property ReceiveBufferSize;
+  EXTENSIONAPI static core::Property SSLContextService;
+  EXTENSIONAPI static core::Property ConnectionAttemptLimit;
+  EXTENSIONAPI static core::Property EndOfMessageByte;
 
   // Supported Relationships
-  static core::Relationship Success;
-  static core::Relationship Partial;
+  EXTENSIONAPI static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Partial;
 
  public:
   /**
@@ -246,17 +227,15 @@ class GetTCP : public core::Processor, public state::response::MetricsNodeSource
 
   std::map<std::string, std::future<int>*> live_clients_;
 
-  utils::ThreadPool<int> client_thread_pool_;
-
   moodycamel::ConcurrentQueue<std::unique_ptr<io::Socket>> socket_ring_buffer_;
 
   bool stay_connected_;
 
   uint16_t concurrent_handlers_;
 
-  int8_t endOfMessageByte;
+  std::byte endOfMessageByte;
 
-  uint64_t reconnect_interval_;
+  std::chrono::milliseconds reconnect_interval_{5000};
 
   uint64_t receive_buffer_size_;
 
@@ -270,15 +249,11 @@ class GetTCP : public core::Processor, public state::response::MetricsNodeSource
 
   std::shared_ptr<minifi::controllers::SSLContextService> ssl_service_;
 
-  // last listing time for root directory ( if recursive, we will consider the root
-  // as the top level time.
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GetTCP>::getLogger();
 
-  std::shared_ptr<logging::Logger> logger_;
+  utils::ThreadPool<int> client_thread_pool_;
 };
 
-REGISTER_RESOURCE(GetTCP, "Establishes a TCP Server that defines and retrieves one or more byte messages from clients");
-
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
