@@ -29,9 +29,9 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/gsl.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -47,27 +47,26 @@ class LogAttribute : public core::Processor {
    * Create a new processor
    */
   explicit LogAttribute(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(std::move(name), uuid),
+      : Processor(name, uuid),
         flowfiles_to_log_(1),
         hexencode_(false),
-        max_line_length_(80U),
-        logger_(logging::LoggerFactory<LogAttribute>::getLogger()) {
+        max_line_length_(80U) {
   }
   // Destructor
-  virtual ~LogAttribute() = default;
+  ~LogAttribute() override = default;
   // Processor Name
-  static constexpr char const* ProcessorName = "LogAttribute";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "LogAttribute";
   // Supported Properties
-  static core::Property LogLevel;
-  static core::Property AttributesToLog;
-  static core::Property AttributesToIgnore;
-  static core::Property LogPayload;
-  static core::Property HexencodePayload;
-  static core::Property MaxPayloadLineLength;
-  static core::Property LogPrefix;
-  static core::Property FlowFilesToLog;
+  EXTENSIONAPI static core::Property LogLevel;
+  EXTENSIONAPI static core::Property AttributesToLog;
+  EXTENSIONAPI static core::Property AttributesToIgnore;
+  EXTENSIONAPI static core::Property LogPayload;
+  EXTENSIONAPI static core::Property HexencodePayload;
+  EXTENSIONAPI static core::Property MaxPayloadLineLength;
+  EXTENSIONAPI static core::Property LogPrefix;
+  EXTENSIONAPI static core::Property FlowFilesToLog;
   // Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
   enum LogAttrLevel {
     LogAttrLevelTrace,
     LogAttrLevelDebug,
@@ -96,34 +95,13 @@ class LogAttribute : public core::Processor {
       return false;
     }
   }
-  // Nest Callback Class for read stream
-  class ReadCallback : public InputStreamCallback {
-   public:
-    ReadCallback(std::shared_ptr<logging::Logger> logger, size_t size)
-        : logger_(std::move(logger))
-        , buffer_(size)  {
-    }
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      if (buffer_.size() == 0U) {
-        return 0U;
-      }
-      int ret = stream->read(buffer_.data(), gsl::narrow<int>(buffer_.size()));
-      if (ret < 0 || static_cast<uint64_t>(ret) != buffer_.size()) {
-        logger_->log_error("%zu bytes were requested from the stream but %d bytes were read. Rolling back.", buffer_.size(), ret);
-        throw Exception(PROCESSOR_EXCEPTION, "Failed to read the entire FlowFile.");
-      }
-      return buffer_.size();
-    }
-    std::shared_ptr<logging::Logger> logger_;
-    std::vector<uint8_t> buffer_;
-  };
 
  public:
   void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &factory) override;
   // OnTrigger method, implemented by NiFi LogAttribute
   void onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) override;
   // Initialize, over write by NiFi LogAttribute
-  void initialize(void) override;
+  void initialize() override;
 
  private:
   core::annotation::Input getInputRequirement() const override {
@@ -134,11 +112,9 @@ class LogAttribute : public core::Processor {
   bool hexencode_;
   uint32_t max_line_length_;
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<LogAttribute>::getLogger();
 };
 
-REGISTER_RESOURCE(LogAttribute, "Logs attributes of flow files in the MiNiFi application log.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
