@@ -19,16 +19,20 @@
  */
 
 #include "PutFile.h"
+#include <array>
 #include <cstdint>
 #include <cstdio>
 #include <iostream>
 #include <memory>
 #include <string>
 #include <set>
+#include <utility>
 #ifdef WIN32
 #include <Windows.h>
 #endif
 #include "utils/file/FileUtils.h"
+#include "utils/gsl.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -143,12 +147,12 @@ void PutFile::onTrigger(core::ProcessContext *context, core::ProcessSession *ses
 
   // Determine dest full file paths
   std::stringstream destFileSs;
-  destFileSs << directory << utils::file::FileUtils::get_separator() << filename;
+  destFileSs << directory << utils::file::get_separator() << filename;
   std::string destFile = destFileSs.str();
 
   logger_->log_debug("PutFile writing file %s into directory %s", filename, directory);
 
-  if ((max_dest_files_ != -1) && utils::file::FileUtils::is_directory(directory.c_str())) {
+  if ((max_dest_files_ != -1) && utils::file::is_directory(directory)) {
     int64_t count = 0;
 
     // Callback, called for each file entry in the listed directory
@@ -157,7 +161,7 @@ void PutFile::onTrigger(core::ProcessContext *context, core::ProcessSession *ses
       return ++count < max_dest_files_;
     };
 
-    utils::file::FileUtils::list_dir(directory, lambda, logger_, false);
+    utils::file::list_dir(directory, lambda, logger_, false);
 
     if (count >= max_dest_files_) {
       logger_->log_warn("Routing to failure because the output directory %s has at least %u files, which exceeds the "
@@ -186,12 +190,12 @@ std::string PutFile::tmpWritePath(const std::string &filename, const std::string
   utils::Identifier tmpFileUuid = id_generator_->generate();
   std::stringstream tmpFileSs;
   tmpFileSs << directory;
-  auto lastSeparatorPos = filename.find_last_of(utils::file::FileUtils::get_separator());
+  auto lastSeparatorPos = filename.find_last_of(utils::file::get_separator());
 
   if (lastSeparatorPos == std::string::npos) {
-    tmpFileSs << utils::file::FileUtils::get_separator() << "." << filename;
+    tmpFileSs << utils::file::get_separator() << "." << filename;
   } else {
-    tmpFileSs << utils::file::FileUtils::get_separator() << filename.substr(0, lastSeparatorPos) << utils::file::FileUtils::get_separator() << "." << filename.substr(lastSeparatorPos + 1);
+    tmpFileSs << utils::file::get_separator() << filename.substr(0, lastSeparatorPos) << utils::file::get_separator() << "." << filename.substr(lastSeparatorPos + 1);
   }
 
   tmpFileSs << "." << tmpFileUuid.to_string();
@@ -206,7 +210,7 @@ bool PutFile::putFile(core::ProcessSession *session, std::shared_ptr<core::FlowF
 
     logger_->log_debug("Destination directory does not exist; will attempt to create: ", destDir);
     size_t i = 0;
-    auto pos = destFile.find(utils::file::FileUtils::get_separator());
+    auto pos = destFile.find(utils::file::get_separator());
 
     while (pos != std::string::npos) {
       auto dir_path_component = destFile.substr(i, pos - i);
@@ -215,23 +219,23 @@ bool PutFile::putFile(core::ProcessSession *session, std::shared_ptr<core::FlowF
 
       if (!dir_path_component.empty()) {
         logger_->log_debug("Attempting to create directory if it does not already exist: %s", dir_path);
-        if (!utils::file::FileUtils::exists(dir_path)) {
-          utils::file::FileUtils::create_dir(dir_path, false);
+        if (!utils::file::exists(dir_path)) {
+          utils::file::create_dir(dir_path, false);
 #ifndef WIN32
           if (directory_permissions_.valid()) {
-            utils::file::FileUtils::set_permissions(dir_path, directory_permissions_.getValue());
+            utils::file::set_permissions(dir_path, directory_permissions_.getValue());
           }
 #endif
         }
 
-        dir_path_stream << utils::file::FileUtils::get_separator();
+        dir_path_stream << utils::file::get_separator();
       } else if (pos == 0) {
         // Support absolute paths
-        dir_path_stream << utils::file::FileUtils::get_separator();
+        dir_path_stream << utils::file::get_separator();
       }
 
       i = pos + 1;
-      pos = destFile.find(utils::file::FileUtils::get_separator(), pos + 1);
+      pos = destFile.find(utils::file::get_separator(), pos + 1);
     }
   }
 
@@ -239,7 +243,7 @@ bool PutFile::putFile(core::ProcessSession *session, std::shared_ptr<core::FlowF
 
   if (flowFile->getSize() > 0) {
     ReadCallback cb(tmpFile, destFile);
-    session->read(flowFile, &cb);
+    session->read(flowFile, std::ref(cb));
     logger_->log_debug("Committing %s", destFile);
     success = cb.commit();
   } else {
@@ -253,7 +257,7 @@ bool PutFile::putFile(core::ProcessSession *session, std::shared_ptr<core::FlowF
 
 #ifndef WIN32
   if (permissions_.valid()) {
-    utils::file::FileUtils::set_permissions(destFile, permissions_.getValue());
+    utils::file::set_permissions(destFile, permissions_.getValue());
   }
 #endif
 
@@ -304,32 +308,25 @@ void PutFile::getDirectoryPermissions(core::ProcessContext *context) {
 }
 #endif
 
-PutFile::ReadCallback::ReadCallback(const std::string &tmp_file, const std::string &dest_file)
-    : tmp_file_(tmp_file),
-      dest_file_(dest_file) {
+PutFile::ReadCallback::ReadCallback(std::string tmp_file, std::string dest_file)
+    : tmp_file_(std::move(tmp_file)),
+      dest_file_(std::move(dest_file)) {
 }
 
 // Copy the entire file contents to the temporary file
-int64_t PutFile::ReadCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
+int64_t PutFile::ReadCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) {
   // Copy file contents into tmp file
   write_succeeded_ = false;
   size_t size = 0;
-  uint8_t buffer[1024];
+  std::array<std::byte, 1024> buffer{};
 
   std::ofstream tmp_file_os(tmp_file_, std::ios::out | std::ios::binary);
 
   do {
-    int read = stream->read(buffer, 1024);
-
-    if (read < 0) {
-      return -1;
-    }
-
-    if (read == 0) {
-      break;
-    }
-
-    tmp_file_os.write(reinterpret_cast<char *>(buffer), read);
+    const auto read = stream->read(buffer);
+    if (io::isError(read)) return -1;
+    if (read == 0) break;
+    tmp_file_os.write(reinterpret_cast<char *>(buffer.data()), gsl::narrow<std::streamsize>(read));
     size += read;
   } while (size < stream->size());
 
@@ -339,7 +336,7 @@ int64_t PutFile::ReadCallback::process(const std::shared_ptr<io::BaseStream>& st
     write_succeeded_ = true;
   }
 
-  return size;
+  return gsl::narrow<int64_t>(size);
 }
 
 // Renames tmp file to final destination
@@ -369,6 +366,8 @@ PutFile::ReadCallback::~ReadCallback() {
   std::remove(tmp_file_.c_str());
 }
 
+REGISTER_RESOURCE(PutFile, "Writes the contents of a FlowFile to the local file system");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
