@@ -28,9 +28,9 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Id.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -50,24 +50,23 @@ class PutFile : public core::Processor {
    * Create a new processor
    */
   PutFile(const std::string& name,  const utils::Identifier& uuid = {}) // NOLINT
-      : core::Processor(std::move(name), uuid),
-        logger_(logging::LoggerFactory<PutFile>::getLogger()) {
+      : core::Processor(name, uuid) {
   }
 
   ~PutFile() override = default;
 
   // Supported Properties
-  static core::Property Directory;
-  static core::Property ConflictResolution;
-  static core::Property CreateDirs;
-  static core::Property MaxDestFiles;
+  EXTENSIONAPI static core::Property Directory;
+  EXTENSIONAPI static core::Property ConflictResolution;
+  EXTENSIONAPI static core::Property CreateDirs;
+  EXTENSIONAPI static core::Property MaxDestFiles;
 #ifndef WIN32
-  static core::Property Permissions;
-  static core::Property DirectoryPermissions;
+  EXTENSIONAPI static core::Property Permissions;
+  EXTENSIONAPI static core::Property DirectoryPermissions;
 #endif
   // Supported Relationships
-  static core::Relationship Success;
-  static core::Relationship Failure;
+  EXTENSIONAPI static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Failure;
 
   /**
    * Function that's executed when the processor is scheduled.
@@ -80,15 +79,15 @@ class PutFile : public core::Processor {
   void onTrigger(core::ProcessContext *context, core::ProcessSession *session) override;
   void initialize() override;
 
-  class ReadCallback : public InputStreamCallback {
+  class ReadCallback {
    public:
-    ReadCallback(const std::string &tmp_file, const std::string &dest_file);
-    ~ReadCallback() override;
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
+    ReadCallback(std::string tmp_file, std::string dest_file);
+    ~ReadCallback();
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream);
     bool commit();
 
    private:
-    std::shared_ptr<logging::Logger> logger_{ logging::LoggerFactory<PutFile::ReadCallback>::getLogger() };
+    std::shared_ptr<core::logging::Logger> logger_{ core::logging::LoggerFactory<PutFile::ReadCallback>::getLogger() };
     bool write_succeeded_ = false;
     std::string tmp_file_;
     std::string dest_file_;
@@ -116,7 +115,7 @@ class PutFile : public core::Processor {
                const std::string &tmpFile,
                const std::string &destFile,
                const std::string &destDir);
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<PutFile>::getLogger();
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 
 #ifndef WIN32
@@ -136,8 +135,6 @@ class PutFile : public core::Processor {
 #endif
 };
 
-REGISTER_RESOURCE(PutFile, "Writes the contents of a FlowFile to the local file system");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
