@@ -20,6 +20,7 @@
 
 #include <atomic>
 #include <memory>
+#include <optional>
 #include <queue>
 #include <string>
 #include <utility>
@@ -30,9 +31,9 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/OptionalUtils.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -43,26 +44,25 @@ namespace processors {
 class RetryFlowFile : public core::Processor {
  public:
   explicit RetryFlowFile(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(name, uuid),
-        logger_(logging::LoggerFactory<RetryFlowFile>::getLogger()) {}
+      : Processor(name, uuid) {}
   // Destructor
-  virtual ~RetryFlowFile() = default;
+  ~RetryFlowFile() override = default;
   // Processor Name
-  static constexpr char const* ProcessorName = "RetryFlowFile";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "RetryFlowFile";
   // Supported Properties
-  static core::Property RetryAttribute;
-  static core::Property MaximumRetries;
-  static core::Property PenalizeRetries;
-  static core::Property FailOnNonNumericalOverwrite;
-  static core::Property ReuseMode;
+  EXTENSIONAPI static core::Property RetryAttribute;
+  EXTENSIONAPI static core::Property MaximumRetries;
+  EXTENSIONAPI static core::Property PenalizeRetries;
+  EXTENSIONAPI static core::Property FailOnNonNumericalOverwrite;
+  EXTENSIONAPI static core::Property ReuseMode;
   // Supported Relationships
-  static core::Relationship Retry;
-  static core::Relationship RetriesExceeded;
-  static core::Relationship Failure;
+  EXTENSIONAPI static core::Relationship Retry;
+  EXTENSIONAPI static core::Relationship RetriesExceeded;
+  EXTENSIONAPI static core::Relationship Failure;
   // ReuseMode allowable values
-  static constexpr char const* FAIL_ON_REUSE = "Fail on Reuse";
-  static constexpr char const* WARN_ON_REUSE = "Warn on Reuse";
-  static constexpr char const*   RESET_REUSE = "Reset Reuse";
+  EXTENSIONAPI static constexpr char const* FAIL_ON_REUSE = "Fail on Reuse";
+  EXTENSIONAPI static constexpr char const* WARN_ON_REUSE = "Warn on Reuse";
+  EXTENSIONAPI static constexpr char const*   RESET_REUSE = "Reset Reuse";
 
  public:
   bool supportsDynamicProperties() override {
@@ -88,7 +88,7 @@ class RetryFlowFile : public core::Processor {
 
  private:
   void readDynamicPropertyKeys(core::ProcessContext* context);
-  utils::optional<uint64_t> getRetryPropertyValue(const std::shared_ptr<core::FlowFile>& flow_file) const;
+  std::optional<uint64_t> getRetryPropertyValue(const std::shared_ptr<core::FlowFile>& flow_file) const;
   void setRetriesExceededAttributesOnFlowFile(core::ProcessContext* context, const std::shared_ptr<core::FlowFile>& flow_file) const;
 
   core::annotation::Input getInputRequirement() const override {
@@ -102,18 +102,9 @@ class RetryFlowFile : public core::Processor {
   std::string reuse_mode_;
   std::vector<core::Property> exceeded_flowfile_attribute_keys_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<RetryFlowFile>::getLogger();
 };
 
-REGISTER_RESOURCE(RetryFlowFile,
-    "FlowFiles passed to this Processor have a 'Retry Attribute' value checked against a configured 'Maximum Retries' value. "
-    "If the current attribute value is below the configured maximum, the FlowFile is passed to a retry relationship. "
-    "The FlowFile may or may not be penalized in that condition. If the FlowFile's attribute value exceeds the configured maximum, "
-    "the FlowFile will be passed to a 'retries_exceeded' relationship. "
-    "WARNING: If the incoming FlowFile has a non-numeric value in the configured 'Retry Attribute' attribute, it will be reset to '1'. "
-    "You may choose to fail the FlowFile instead of performing the reset. Additional dynamic properties can be defined for any attributes "
-    "you wish to add to the FlowFiles transferred to 'retries_exceeded'. These attributes support attribute expression language.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
