@@ -31,16 +31,24 @@
 #include <utility>
 #include <vector>
 
+#include "range/v3/action/sort.hpp"
+#include "range/v3/range/conversion.hpp"
+#include "range/v3/view/transform.hpp"
+
+#include "FlowFileRecord.h"
 #include "io/CRCStream.h"
 #include "utils/file/FileUtils.h"
 #include "utils/file/PathUtils.h"
 #include "utils/TimeUtil.h"
 #include "utils/StringUtils.h"
-#include "utils/RegexUtils.h"
 #include "utils/ProcessorConfigUtils.h"
+#include "TextFragmentUtils.h"
 #include "TailFile.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
+#include "utils/RegexUtils.h"
+
 
 namespace org {
 namespace apache {
@@ -48,20 +56,20 @@ namespace nifi {
 namespace minifi {
 namespace processors {
 
-core::Property TailFile::FileName(
+const core::Property TailFile::FileName(
     core::PropertyBuilder::createProperty("File to Tail")
         ->withDescription("Fully-qualified filename of the file that should be tailed when using single file mode, or a file regex when using multifile mode")
         ->isRequired(true)
         ->build());
 
-core::Property TailFile::StateFile(
+const core::Property TailFile::StateFile(
     core::PropertyBuilder::createProperty("State File")
         ->withDescription("DEPRECATED. Only use it for state migration from the legacy state file.")
         ->isRequired(false)
         ->withDefaultValue<std::string>("TailFileState")
         ->build());
 
-core::Property TailFile::Delimiter(
+const core::Property TailFile::Delimiter(
     core::PropertyBuilder::createProperty("Input Delimiter")
         ->withDescription("Specifies the character that should be used for delimiting the data being tailed"
          "from the incoming file. If none is specified, data will be ingested as it becomes available.")
@@ -69,7 +77,7 @@ core::Property TailFile::Delimiter(
         ->withDefaultValue<std::string>("\\n")
         ->build());
 
-core::Property TailFile::TailMode(
+const core::Property TailFile::TailMode(
     core::PropertyBuilder::createProperty("tail-mode", "Tailing Mode")
         ->withDescription("Specifies the tail file mode. In 'Single file' mode only a single file will be watched. "
         "In 'Multiple file' mode a regex may be used. Note that in multiple file mode we will still continue to watch for rollover on the initial set of watched files. "
@@ -77,13 +85,15 @@ core::Property TailFile::TailMode(
         ->withAllowableValue<std::string>("Single file")->withAllowableValue("Multiple file")->withDefaultValue("Single file")
         ->build());
 
-core::Property TailFile::BaseDirectory(
+const core::Property TailFile::BaseDirectory(
     core::PropertyBuilder::createProperty("tail-base-directory", "Base Directory")
-        ->withDescription("Base directory used to look for files to tail. This property is required when using Multiple file mode.")
+        ->withDescription("Base directory used to look for files to tail. This property is required when using Multiple file mode. "
+                          "Can contain expression language placeholders if Attribute Provider Service is set.")
         ->isRequired(false)
+        ->supportsExpressionLanguage(true)
         ->build());
 
-core::Property TailFile::RecursiveLookup(
+const core::Property TailFile::RecursiveLookup(
     core::PropertyBuilder::createProperty("Recursive lookup")
         ->withDescription("When using Multiple file mode, this property determines whether files are tailed in "
         "child directories of the Base Directory or not.")
@@ -91,7 +101,7 @@ core::Property TailFile::RecursiveLookup(
         ->withDefaultValue<bool>(false)
         ->build());
 
-core::Property TailFile::LookupFrequency(
+const core::Property TailFile::LookupFrequency(
     core::PropertyBuilder::createProperty("Lookup frequency")
         ->withDescription("When using Multiple file mode, this property specifies the minimum duration "
         "the processor will wait between looking for new files to tail in the Base Directory.")
@@ -99,7 +109,7 @@ core::Property TailFile::LookupFrequency(
         ->withDefaultValue<core::TimePeriodValue>("10 min")
         ->build());
 
-core::Property TailFile::RollingFilenamePattern(
+const core::Property TailFile::RollingFilenamePattern(
     core::PropertyBuilder::createProperty("Rolling Filename Pattern")
         ->withDescription("If the file to tail \"rolls over\" as would be the case with log files, this filename pattern will be used to "
         "identify files that have rolled over so MiNiFi can read the remaining of the rolled-over file and then continue with the new log file. "
@@ -109,7 +119,7 @@ core::Property TailFile::RollingFilenamePattern(
         ->withDefaultValue<std::string>("${filename}.*")
         ->build());
 
-core::Property TailFile::InitialStartPosition(
+const core::Property TailFile::InitialStartPosition(
     core::PropertyBuilder::createProperty("Initial Start Position")
         ->withDescription("When the Processor first begins to tail data, this property specifies where the Processor should begin reading data. "
                           "Once data has been ingested from a file, the Processor will continue from the last point from which it has received data.\n"
@@ -122,7 +132,14 @@ core::Property TailFile::InitialStartPosition(
         ->withAllowableValues(InitialStartPositions::values())
         ->build());
 
-core::Relationship TailFile::Success("success", "All files are routed to success");
+const core::Property TailFile::AttributeProviderService(
+    core::PropertyBuilder::createProperty("Attribute Provider Service")
+        ->withDescription("Provides a list of key-value pair records which can be used in the Base Directory property using Expression Language. "
+                          "Requires Multiple file mode.")
+        ->asType<minifi::controllers::AttributeProviderService>()
+        ->build());
+
+const core::Relationship TailFile::Success("success", "All files are routed to success");
 
 const char *TailFile::CURRENT_STR = "CURRENT.";
 const char *TailFile::POSITION_STR = "POSITION.";
@@ -170,13 +187,13 @@ std::map<std::string, TailState> update_keys_in_legacy_states(const std::map<std
   std::map<std::string, TailState> new_tail_states;
   for (const auto &key_value_pair : legacy_tail_states) {
     const TailState &state = key_value_pair.second;
-    std::string full_file_name = utils::file::FileUtils::concat_path(state.path_, state.file_name_);
+    std::string full_file_name = utils::file::concat_path(state.path_, state.file_name_);
     new_tail_states.emplace(full_file_name, state);
   }
   return new_tail_states;
 }
 
-void openFile(const std::string &file_name, uint64_t offset, std::ifstream &input_stream, const std::shared_ptr<logging::Logger> &logger) {
+void openFile(const std::string &file_name, uint64_t offset, std::ifstream &input_stream, const std::shared_ptr<core::logging::Logger> &logger) {
   logger->log_debug("Opening %s", file_name);
   input_stream.open(file_name.c_str(), std::fstream::in | std::fstream::binary);
   if (!input_stream.is_open() || !input_stream.good()) {
@@ -194,19 +211,18 @@ void openFile(const std::string &file_name, uint64_t offset, std::ifstream &inpu
 
 constexpr std::size_t BUFFER_SIZE = 4096;
 
-class FileReaderCallback : public OutputStreamCallback {
+class FileReaderCallback {
  public:
   FileReaderCallback(const std::string &file_name,
                      uint64_t offset,
                      char input_delimiter,
                      uint64_t checksum)
     : input_delimiter_(input_delimiter),
-      checksum_(checksum),
-      logger_(logging::LoggerFactory<TailFile>::getLogger()) {
+      checksum_(checksum) {
     openFile(file_name, offset, input_stream_, logger_);
   }
 
-  int64_t process(const std::shared_ptr<io::BaseStream>& output_stream) override {
+  int64_t operator()(const std::shared_ptr<io::BaseStream>& output_stream) {
     io::CRCStream<io::BaseStream> crc_stream{gsl::make_not_null(output_stream.get()), checksum_};
 
     uint64_t num_bytes_written = 0;
@@ -214,7 +230,7 @@ class FileReaderCallback : public OutputStreamCallback {
 
     while (hasMoreToRead() && !found_delimiter) {
       if (begin_ == end_) {
-        input_stream_.read(reinterpret_cast<char *>(buffer_.data()), buffer_.size());
+        input_stream_.read(reinterpret_cast<char *>(buffer_.data()), gsl::narrow<std::streamsize>(buffer_.size()));
 
         const auto num_bytes_read = input_stream_.gcount();
         logger_->log_trace("Read %jd bytes of input", std::intmax_t{num_bytes_read});
@@ -226,15 +242,10 @@ class FileReaderCallback : public OutputStreamCallback {
       char *delimiter_pos = std::find(begin_, end_, input_delimiter_);
       found_delimiter = (delimiter_pos != end_);
 
-      ptrdiff_t zlen{std::distance(begin_, delimiter_pos)};
-      if (found_delimiter) {
-        zlen += 1;
-      }
-      const int len = gsl::narrow<int>(zlen);
-
-      crc_stream.write(reinterpret_cast<uint8_t*>(begin_), len);
-      num_bytes_written += len;
-      begin_ += len;
+      const auto zlen = gsl::narrow<size_t>(std::distance(begin_, delimiter_pos)) + (found_delimiter ? 1 : 0);
+      crc_stream.write(reinterpret_cast<uint8_t*>(begin_), zlen);
+      num_bytes_written += zlen;
+      begin_ += zlen;
     }
 
     if (found_delimiter) {
@@ -262,7 +273,7 @@ class FileReaderCallback : public OutputStreamCallback {
   char input_delimiter_;
   uint64_t checksum_;
   std::ifstream input_stream_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<TailFile>::getLogger();
 
   std::array<char, BUFFER_SIZE> buffer_;
   char *begin_ = buffer_.data();
@@ -271,13 +282,12 @@ class FileReaderCallback : public OutputStreamCallback {
   bool latest_flow_file_ends_with_delimiter_ = true;
 };
 
-class WholeFileReaderCallback : public OutputStreamCallback {
+class WholeFileReaderCallback {
  public:
   WholeFileReaderCallback(const std::string &file_name,
                           uint64_t offset,
                           uint64_t checksum)
-    : checksum_(checksum),
-      logger_(logging::LoggerFactory<TailFile>::getLogger()) {
+    : checksum_(checksum) {
     openFile(file_name, offset, input_stream_, logger_);
   }
 
@@ -285,7 +295,7 @@ class WholeFileReaderCallback : public OutputStreamCallback {
     return checksum_;
   }
 
-  int64_t process(const std::shared_ptr<io::BaseStream>& output_stream) override {
+  int64_t operator()(const std::shared_ptr<io::BaseStream>& output_stream) {
     std::array<char, BUFFER_SIZE> buffer;
 
     io::CRCStream<io::BaseStream> crc_stream{gsl::make_not_null(output_stream.get()), checksum_};
@@ -312,31 +322,27 @@ class WholeFileReaderCallback : public OutputStreamCallback {
  private:
   uint64_t checksum_;
   std::ifstream input_stream_;
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<TailFile>::getLogger();
 };
 }  // namespace
 
 void TailFile::initialize() {
-  // Set the supported properties
-  std::set<core::Property> properties;
-  properties.insert(FileName);
-  properties.insert(StateFile);
-  properties.insert(Delimiter);
-  properties.insert(TailMode);
-  properties.insert(BaseDirectory);
-  properties.insert(RecursiveLookup);
-  properties.insert(LookupFrequency);
-  properties.insert(RollingFilenamePattern);
-  properties.insert(InitialStartPosition);
-  setSupportedProperties(properties);
-  // Set the supported relationships
-  std::set<core::Relationship> relationships;
-  relationships.insert(Success);
-  setSupportedRelationships(relationships);
+  setSupportedProperties({
+    FileName,
+    StateFile,
+    Delimiter,
+    TailMode,
+    BaseDirectory,
+    RecursiveLookup,
+    LookupFrequency,
+    RollingFilenamePattern,
+    InitialStartPosition,
+    AttributeProviderService});
+  setSupportedRelationships({Success});
 }
 
 void TailFile::onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory>& /*sessionFactory*/) {
-  std::lock_guard<std::mutex> tail_lock(tail_file_mutex_);
+  gsl_Expects(context);
 
   tail_states_.clear();
 
@@ -359,11 +365,13 @@ void TailFile::onSchedule(const std::shared_ptr<core::ProcessContext> &context,
   if (mode == "Multiple file") {
     tail_mode_ = Mode::MULTIPLE;
 
+    parseAttributeProviderServiceProperty(*context);
+
     if (!context->getProperty(BaseDirectory.getName(), base_dir_)) {
       throw minifi::Exception(ExceptionType::PROCESSOR_EXCEPTION, "Base directory is required for multiple tail mode.");
     }
 
-    if (!utils::file::FileUtils::is_directory(base_dir_.c_str())) {
+    if (!attribute_provider_service_ && !utils::file::is_directory(base_dir_)) {
       throw minifi::Exception(ExceptionType::PROCESSOR_EXCEPTION, "Base directory does not exist or is not a directory");
     }
 
@@ -380,7 +388,7 @@ void TailFile::onSchedule(const std::shared_ptr<core::ProcessContext> &context,
 
     recoverState(context);
 
-    doMultifileLookup();
+    doMultifileLookup(*context);
 
   } else {
     tail_mode_ = Mode::SINGLE;
@@ -399,7 +407,25 @@ void TailFile::onSchedule(const std::shared_ptr<core::ProcessContext> &context,
   std::string rolling_filename_pattern_glob;
   context->getProperty(RollingFilenamePattern.getName(), rolling_filename_pattern_glob);
   rolling_filename_pattern_ = utils::file::globToRegex(rolling_filename_pattern_glob);
-  initial_start_position_ = utils::parsePropertyWithAllowableValuesOrThrow(*context, InitialStartPosition.getName(), InitialStartPositions::values());
+  initial_start_position_ = InitialStartPositions{utils::parsePropertyWithAllowableValuesOrThrow(*context, InitialStartPosition.getName(), InitialStartPositions::values())};
+}
+
+void TailFile::parseAttributeProviderServiceProperty(core::ProcessContext& context) {
+  const auto attribute_provider_service_name = context.getProperty(AttributeProviderService);
+  if (!attribute_provider_service_name || attribute_provider_service_name->empty()) {
+    return;
+  }
+
+  std::shared_ptr<core::controller::ControllerService> controller_service = context.getControllerService(*attribute_provider_service_name);
+  if (!controller_service) {
+    throw minifi::Exception{ExceptionType::PROCESS_SCHEDULE_EXCEPTION, utils::StringUtils::join_pack("Controller service '", *attribute_provider_service_name, "' not found")};
+  }
+
+  // we drop ownership of the service here -- in the long term, getControllerService() should return a non-owning pointer or optional reference
+  attribute_provider_service_ = dynamic_cast<minifi::controllers::AttributeProviderService*>(controller_service.get());
+  if (!attribute_provider_service_) {
+    throw minifi::Exception{ExceptionType::PROCESS_SCHEDULE_EXCEPTION, utils::StringUtils::join_pack("Controller service '", *attribute_provider_service_name, "' is not an AttributeProviderService")};
+  }
 }
 
 void TailFile::parseStateFileLine(char *buf, std::map<std::string, TailState> &state) const {
@@ -511,7 +537,7 @@ bool TailFile::getStateFromStateManager(std::map<std::string, TailState> &new_ta
         const std::string& current = state_map.at("file." + std::to_string(i) + ".current");
         uint64_t position = std::stoull(state_map.at("file." + std::to_string(i) + ".position"));
         uint64_t checksum = readOptionalUint64(state_map, "file." + std::to_string(i) + ".checksum");
-        std::chrono::system_clock::time_point last_read_time{std::chrono::milliseconds{
+        std::chrono::file_clock::time_point last_read_time{std::chrono::milliseconds{
             readOptionalInt64(state_map, "file." + std::to_string(i) + ".last_read_time")
         }};
 
@@ -561,8 +587,8 @@ bool TailFile::getStateFromLegacyStateFile(const std::shared_ptr<core::ProcessCo
 
 void TailFile::logState() {
   logger_->log_info("State of the TailFile processor %s:", name_);
-  for (const auto& key_value_pair : tail_states_) {
-    logging::LOG_INFO(logger_) << key_value_pair.first << " => { " << key_value_pair.second << " }";
+  for (const auto& [key, value] : tail_states_) {
+    core::logging::LOG_INFO(logger_) << key << " => { " << value << " }";
   }
 }
 
@@ -605,16 +631,17 @@ std::vector<TailState> TailFile::findAllRotatedFiles(const TailState &state) con
 
   std::vector<TailStateWithMtime> matched_files_with_mtime;
   auto collect_matching_files = [&](const std::string &path, const std::string &file_name) -> bool {
-    if (file_name != state.file_name_ && utils::Regex::matchesFullInput(pattern, file_name)) {
-      std::string full_file_name = path + utils::file::FileUtils::get_separator() + file_name;
-      TailStateWithMtime::TimePoint mtime{utils::file::FileUtils::last_write_time_point(full_file_name)};
+    utils::Regex pattern_regex(pattern);
+    if (file_name != state.file_name_ && utils::regexMatch(file_name, pattern_regex)) {
+      std::string full_file_name = path + utils::file::get_separator() + file_name;
+      TailStateWithMtime::TimePoint mtime{utils::file::last_write_time_point(full_file_name)};
       logger_->log_debug("File %s with mtime %" PRId64 " matches rolling filename pattern %s, so we are reading it", file_name, int64_t{mtime.time_since_epoch().count()}, pattern);
       matched_files_with_mtime.emplace_back(TailState{path, file_name}, mtime);
     }
     return true;
   };
 
-  utils::file::FileUtils::list_dir(state.path_, collect_matching_files, logger_, false);
+  utils::file::list_dir(state.path_, collect_matching_files, logger_, false);
 
   return sortAndSkipMainFilePrefix(state, matched_files_with_mtime);
 }
@@ -626,9 +653,10 @@ std::vector<TailState> TailFile::findRotatedFilesAfterLastReadTime(const TailSta
 
   std::vector<TailStateWithMtime> matched_files_with_mtime;
   auto collect_matching_files = [&](const std::string &path, const std::string &file_name) -> bool {
-    if (file_name != state.file_name_ && utils::Regex::matchesFullInput(pattern, file_name)) {
-      std::string full_file_name = path + utils::file::FileUtils::get_separator() + file_name;
-      TailStateWithMtime::TimePoint mtime{utils::file::FileUtils::last_write_time_point(full_file_name)};
+    utils::Regex pattern_regex(pattern);
+    if (file_name != state.file_name_ && utils::regexMatch(file_name, pattern_regex)) {
+      std::string full_file_name = path + utils::file::get_separator() + file_name;
+      TailStateWithMtime::TimePoint mtime{utils::file::last_write_time_point(full_file_name)};
       logger_->log_debug("File %s with mtime %" PRId64 " matches rolling filename pattern %s", file_name, int64_t{mtime.time_since_epoch().count()}, pattern);
       if (mtime >= std::chrono::time_point_cast<std::chrono::seconds>(state.last_read_time_)) {
         logger_->log_debug("File %s has mtime >= last read time, so we are going to read it", file_name);
@@ -638,22 +666,23 @@ std::vector<TailState> TailFile::findRotatedFilesAfterLastReadTime(const TailSta
     return true;
   };
 
-  utils::file::FileUtils::list_dir(state.path_, collect_matching_files, logger_, false);
+  utils::file::list_dir(state.path_, collect_matching_files, logger_, false);
 
   return sortAndSkipMainFilePrefix(state, matched_files_with_mtime);
 }
 
 std::vector<TailState> TailFile::sortAndSkipMainFilePrefix(const TailState &state, std::vector<TailStateWithMtime>& matched_files_with_mtime) const {
-  std::sort(matched_files_with_mtime.begin(), matched_files_with_mtime.end(), [](const TailStateWithMtime &left, const TailStateWithMtime &right) {
+  const auto first_by_mtime_then_by_name = [](const auto& left, const auto& right) {
     return std::tie(left.mtime_, left.tail_state_.file_name_) <
            std::tie(right.mtime_, right.tail_state_.file_name_);
-  });
+  };
+  matched_files_with_mtime |= ranges::actions::sort(first_by_mtime_then_by_name);
 
   if (!matched_files_with_mtime.empty() && state.position_ > 0) {
     TailState &first_rotated_file = matched_files_with_mtime[0].tail_state_;
     std::string full_file_name = first_rotated_file.fileNameWithPath();
-    if (utils::file::FileUtils::file_size(full_file_name) >= state.position_) {
-      uint64_t checksum = utils::file::FileUtils::computeChecksum(full_file_name, state.position_);
+    if (utils::file::file_size(full_file_name) >= state.position_) {
+      uint64_t checksum = utils::file::computeChecksum(full_file_name, state.position_);
       if (checksum == state.checksum_) {
         first_rotated_file.position_ = state.position_;
         first_rotated_file.checksum_ = state.checksum_;
@@ -668,13 +697,13 @@ std::vector<TailState> TailFile::sortAndSkipMainFilePrefix(const TailState &stat
   return matched_files;
 }
 
-void TailFile::onTrigger(const std::shared_ptr<core::ProcessContext> &, const std::shared_ptr<core::ProcessSession> &session) {
-  std::lock_guard<std::mutex> tail_lock(tail_file_mutex_);
+void TailFile::onTrigger(const std::shared_ptr<core::ProcessContext>& context, const std::shared_ptr<core::ProcessSession>& session) {
+  gsl_Expects(context && session);
 
   if (tail_mode_ == Mode::MULTIPLE) {
     if (last_multifile_lookup_ + lookup_frequency_ < std::chrono::steady_clock::now()) {
       logger_->log_debug("Lookup frequency %" PRId64 " ms have elapsed, doing new multifile lookup", int64_t{lookup_frequency_.count()});
-      doMultifileLookup();
+      doMultifileLookup(*context);
     } else {
       logger_->log_trace("Skipping multifile lookup");
     }
@@ -694,7 +723,7 @@ void TailFile::onTrigger(const std::shared_ptr<core::ProcessContext> &, const st
 
 bool TailFile::isOldFileInitiallyRead(TailState &state) const {
   // This is our initial processing and no stored state was found
-  return first_trigger_ && state.last_read_time_ == std::chrono::system_clock::time_point{};
+  return first_trigger_ && state.last_read_time_ == std::chrono::file_clock::time_point{};
 }
 
 void TailFile::processFile(const std::shared_ptr<core::ProcessSession> &session,
@@ -704,14 +733,14 @@ void TailFile::processFile(const std::shared_ptr<core::ProcessSession> &session,
     if (initial_start_position_ == InitialStartPositions::BEGINNING_OF_TIME) {
       processAllRotatedFiles(session, state);
     } else if (initial_start_position_ == InitialStartPositions::CURRENT_TIME) {
-      state.position_ = utils::file::FileUtils::file_size(full_file_name);
-      state.last_read_time_ = std::chrono::system_clock::now();
-      state.checksum_ = utils::file::FileUtils::computeChecksum(full_file_name, state.position_);
+      state.position_ = utils::file::file_size(full_file_name);
+      state.last_read_time_ = std::chrono::file_clock::now();
+      state.checksum_ = utils::file::computeChecksum(full_file_name, state.position_);
       storeState();
       return;
     }
   } else {
-    uint64_t fsize = utils::file::FileUtils::file_size(full_file_name);
+    uint64_t fsize = utils::file::file_size(full_file_name);
     if (fsize < state.position_) {
       processRotatedFilesAfterLastReadTime(session, state);
     } else if (fsize == state.position_) {
@@ -747,7 +776,7 @@ void TailFile::processSingleFile(const std::shared_ptr<core::ProcessSession> &se
                                  TailState &state) {
   std::string fileName = state.file_name_;
 
-  if (utils::file::FileUtils::file_size(full_file_name) == 0u) {
+  if (utils::file::file_size(full_file_name) == 0u) {
     logger_->log_warn("Unable to read file %s as it does not exist or has size zero", full_file_name);
     return;
   }
@@ -767,7 +796,7 @@ void TailFile::processSingleFile(const std::shared_ptr<core::ProcessSession> &se
 
     while (file_reader.hasMoreToRead()) {
       auto flow_file = session->create();
-      session->write(flow_file, &file_reader);
+      session->write(flow_file, std::ref(file_reader));
 
       if (file_reader.useLatestFlowFile()) {
         updateFlowFileAttributes(full_file_name, state_copy, fileName, baseName, extension, flow_file);
@@ -787,7 +816,7 @@ void TailFile::processSingleFile(const std::shared_ptr<core::ProcessSession> &se
   } else {
     WholeFileReaderCallback file_reader{full_file_name, state.position_, state.checksum_};
     auto flow_file = session->create();
-    session->write(flow_file, &file_reader);
+    session->write(flow_file, std::ref(file_reader));
 
     updateFlowFileAttributes(full_file_name, state, fileName, baseName, extension, flow_file);
     session->transfer(flow_file, Success);
@@ -800,22 +829,35 @@ void TailFile::updateFlowFileAttributes(const std::string &full_file_name, const
                                         const std::string &extension,
                                         std::shared_ptr<core::FlowFile> &flow_file) const {
   logger_->log_info("TailFile %s for %" PRIu64 " bytes", fileName, flow_file->getSize());
-  std::string logName = baseName + "." + std::to_string(state.position_) + "-" +
-                        std::to_string(state.position_ + flow_file->getSize() - 1) + "." + extension;
+  std::string logName = textfragmentutils::createFileName(baseName, extension, state.position_, flow_file->getSize());
   flow_file->setAttribute(core::SpecialFlowAttribute::PATH, state.path_);
   flow_file->addAttribute(core::SpecialFlowAttribute::ABSOLUTE_PATH, full_file_name);
   flow_file->setAttribute(core::SpecialFlowAttribute::FILENAME, logName);
+
+  flow_file->setAttribute(textfragmentutils::BASE_NAME_ATTRIBUTE, baseName);
+  flow_file->setAttribute(textfragmentutils::POST_NAME_ATTRIBUTE, extension);
+  flow_file->setAttribute(textfragmentutils::OFFSET_ATTRIBUTE, std::to_string(state.position_));
+
+  if (extra_attributes_.contains(state.path_)) {
+    std::string prefix;
+    if (attribute_provider_service_) {
+      prefix = std::string(attribute_provider_service_->name()) + ".";
+    }
+    for (const auto& [key, value] : extra_attributes_.at(state.path_)) {
+      flow_file->setAttribute(prefix + key, value);
+    }
+  }
 }
 
 void TailFile::updateStateAttributes(TailState &state, uint64_t size, uint64_t checksum) const {
   state.position_ += size;
-  state.last_read_time_ = std::chrono::system_clock::now();
+  state.last_read_time_ = std::chrono::file_clock::now();
   state.checksum_ = checksum;
 }
 
-void TailFile::doMultifileLookup() {
+void TailFile::doMultifileLookup(core::ProcessContext& context) {
   checkForRemovedFiles();
-  checkForNewFiles();
+  checkForNewFiles(context);
   last_multifile_lookup_ = std::chrono::steady_clock::now();
 }
 
@@ -825,8 +867,9 @@ void TailFile::checkForRemovedFiles() {
   for (const auto &kv : tail_states_) {
     const std::string &full_file_name = kv.first;
     const TailState &state = kv.second;
-    if (utils::file::FileUtils::file_size(state.fileNameWithPath()) == 0u ||
-        !utils::Regex::matchesFullInput(file_to_tail_, state.file_name_)) {
+    utils::Regex pattern_regex(file_to_tail_);
+    if (utils::file::file_size(state.fileNameWithPath()) == 0u ||
+        !utils::regexMatch(state.file_name_, pattern_regex)) {
       file_names_to_remove.push_back(full_file_name);
     }
   }
@@ -836,22 +879,53 @@ void TailFile::checkForRemovedFiles() {
   }
 }
 
-void TailFile::checkForNewFiles() {
+void TailFile::checkForNewFiles(core::ProcessContext& context) {
   auto add_new_files_callback = [&](const std::string &path, const std::string &file_name) -> bool {
-    std::string full_file_name = path + utils::file::FileUtils::get_separator() + file_name;
-    if (!containsKey(tail_states_, full_file_name) && utils::Regex::matchesFullInput(file_to_tail_, file_name)) {
+    std::string full_file_name = path + utils::file::get_separator() + file_name;
+    utils::Regex file_to_tail_regex(file_to_tail_);
+    if (!containsKey(tail_states_, full_file_name) && utils::regexMatch(file_name, file_to_tail_regex)) {
       tail_states_.emplace(full_file_name, TailState{path, file_name});
     }
     return true;
   };
 
-  utils::file::FileUtils::list_dir(base_dir_, add_new_files_callback, logger_, recursive_lookup_);
+  if (!attribute_provider_service_) {
+    utils::file::list_dir(base_dir_, add_new_files_callback, logger_, recursive_lookup_);
+    return;
+  }
+
+  const auto attribute_maps = attribute_provider_service_->getAttributes();
+  if (!attribute_maps) {
+    logger_->log_error("Could not get attributes from the Attribute Provider Service");
+    return;
+  }
+
+  for (const auto& attribute_map : *attribute_maps) {
+    std::string base_dir = baseDirectoryFromAttributes(attribute_map, context);
+    extra_attributes_[base_dir] = attribute_map;
+    utils::file::list_dir(base_dir, add_new_files_callback, logger_, recursive_lookup_);
+  }
+}
+
+std::string TailFile::baseDirectoryFromAttributes(const controllers::AttributeProviderService::AttributeMap& attribute_map, core::ProcessContext& context) const {
+  auto flow_file = std::make_shared<FlowFileRecord>();
+  for (const auto& [key, value] : attribute_map) {
+    flow_file->setAttribute(key, value);
+  }
+  return context.getProperty(BaseDirectory, flow_file).value();
 }
 
 std::chrono::milliseconds TailFile::getLookupFrequency() const {
   return lookup_frequency_;
 }
 
+REGISTER_RESOURCE(TailFile, "\"Tails\" a file, or a list of files, ingesting data from the file as it is written to the file. The file is expected to be textual."
+                  " Data is ingested only when a new line is encountered (carriage return or new-line character or combination). If the file to tail is periodically \"rolled over\","
+                  " as is generally the case with log files, an optional Rolling Filename Pattern can be used to retrieve data from files that have rolled over, even if the rollover"
+                  " occurred while NiFi was not running (provided that the data still exists upon restart of NiFi). It is generally advisable to set the Run Schedule to a few seconds,"
+                  " rather than running with the default value of 0 secs, as this Processor will consume a lot of resources if scheduled very aggressively. At this time, this Processor"
+                  " does not support ingesting files that have been compressed when 'rolled over'.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
