@@ -24,16 +24,18 @@
 #include <memory>
 #include <utility>
 #include <string>
+#include <unordered_map>
 #include <vector>
 #include <set>
 
+#include "controllers/AttributeProviderService.h"
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/Enum.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -43,17 +45,17 @@ namespace processors {
 
 struct TailState {
   TailState(std::string path, std::string file_name, uint64_t position,
-            std::chrono::system_clock::time_point last_read_time,
+            std::chrono::file_clock::time_point last_read_time,
             uint64_t checksum)
       : path_(std::move(path)), file_name_(std::move(file_name)), position_(position), last_read_time_(last_read_time), checksum_(checksum) {}
 
   TailState(std::string path, std::string file_name)
-      : TailState{std::move(path), std::move(file_name), 0, std::chrono::system_clock::time_point{}, 0} {}
+      : TailState{std::move(path), std::move(file_name), 0, std::chrono::file_clock::time_point{}, 0} {}
 
   TailState() = default;
 
   std::string fileNameWithPath() const {
-    return path_ + utils::file::FileUtils::get_separator() + file_name_;
+    return path_ + utils::file::get_separator() + file_name_;
   }
 
   int64_t lastReadTimeInMilliseconds() const {
@@ -63,7 +65,7 @@ struct TailState {
   std::string path_;
   std::string file_name_;
   uint64_t position_ = 0;
-  std::chrono::system_clock::time_point last_read_time_;
+  std::chrono::file_clock::time_point last_read_time_;
   uint64_t checksum_ = 0;
 };
 
@@ -82,28 +84,28 @@ SMART_ENUM(InitialStartPositions,
 class TailFile : public core::Processor {
  public:
   explicit TailFile(const std::string& name, const utils::Identifier& uuid = {})
-      : core::Processor(std::move(name), uuid),
-        logger_(logging::LoggerFactory<TailFile>::getLogger()) {
+      : core::Processor(name, uuid) {
   }
 
   ~TailFile() override = default;
 
   // Processor Name
-  static constexpr char const* ProcessorName = "TailFile";
+  EXTENSIONAPI static constexpr char const* ProcessorName = "TailFile";
 
   // Supported Properties
-  static core::Property FileName;
-  static core::Property StateFile;
-  static core::Property Delimiter;
-  static core::Property TailMode;
-  static core::Property BaseDirectory;
-  static core::Property RecursiveLookup;
-  static core::Property LookupFrequency;
-  static core::Property RollingFilenamePattern;
-  static core::Property InitialStartPosition;
+  EXTENSIONAPI static const core::Property FileName;
+  EXTENSIONAPI static const core::Property StateFile;
+  EXTENSIONAPI static const core::Property Delimiter;
+  EXTENSIONAPI static const core::Property TailMode;
+  EXTENSIONAPI static const core::Property BaseDirectory;
+  EXTENSIONAPI static const core::Property RecursiveLookup;
+  EXTENSIONAPI static const core::Property LookupFrequency;
+  EXTENSIONAPI static const core::Property RollingFilenamePattern;
+  EXTENSIONAPI static const core::Property InitialStartPosition;
+  EXTENSIONAPI static const core::Property AttributeProviderService;
 
   // Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static const core::Relationship Success;
 
   /**
    * Function that's executed when the processor is scheduled.
@@ -127,7 +129,7 @@ class TailFile : public core::Processor {
 
  private:
   struct TailStateWithMtime {
-    using TimePoint = std::chrono::time_point<std::chrono::system_clock, std::chrono::seconds>;
+    using TimePoint = std::chrono::time_point<std::chrono::file_clock, std::chrono::seconds>;
 
     TailStateWithMtime(TailState tail_state, TimePoint mtime)
       : tail_state_(std::move(tail_state)), mtime_(mtime) {}
@@ -140,6 +142,11 @@ class TailFile : public core::Processor {
     return core::annotation::Input::INPUT_FORBIDDEN;
   }
 
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
+  void parseAttributeProviderServiceProperty(core::ProcessContext& context);
   void parseStateFileLine(char *buf, std::map<std::string, TailState> &state) const;
   void processAllRotatedFiles(const std::shared_ptr<core::ProcessSession> &session, TailState &state);
   void processRotatedFiles(const std::shared_ptr<core::ProcessSession> &session, TailState &state, std::vector<TailState> &rotated_file_states);
@@ -157,9 +164,10 @@ class TailFile : public core::Processor {
   bool getStateFromStateManager(std::map<std::string, TailState> &state) const;
   bool getStateFromLegacyStateFile(const std::shared_ptr<core::ProcessContext>& context,
                                    std::map<std::string, TailState> &new_tail_states) const;
-  void doMultifileLookup();
+  void doMultifileLookup(core::ProcessContext& context);
   void checkForRemovedFiles();
-  void checkForNewFiles();
+  void checkForNewFiles(core::ProcessContext& context);
+  std::string baseDirectoryFromAttributes(const controllers::AttributeProviderService::AttributeMap& attribute_map, core::ProcessContext& context) const;
   void updateFlowFileAttributes(const std::string &full_file_name, const TailState &state, const std::string &fileName,
                                 const std::string &baseName, const std::string &extension,
                                 std::shared_ptr<core::FlowFile> &flow_file) const;
@@ -170,29 +178,23 @@ class TailFile : public core::Processor {
   static const char *POSITION_STR;
   static const int BUFFER_SIZE = 512;
 
-  std::mutex tail_file_mutex_;
   std::string delimiter_;  // Delimiter for the data incoming from the tailed file.
-  std::shared_ptr<core::CoreComponentStateManager> state_manager_;
+  core::CoreComponentStateManager* state_manager_ = nullptr;
   std::map<std::string, TailState> tail_states_;
   Mode tail_mode_ = Mode::UNDEFINED;
   std::string file_to_tail_;
   std::string base_dir_;
   bool recursive_lookup_ = false;
-  std::chrono::milliseconds lookup_frequency_;
+  std::chrono::milliseconds lookup_frequency_{};
   std::chrono::steady_clock::time_point last_multifile_lookup_;
   std::string rolling_filename_pattern_;
   InitialStartPositions initial_start_position_;
   bool first_trigger_{true};
-  std::shared_ptr<logging::Logger> logger_;
+  controllers::AttributeProviderService* attribute_provider_service_ = nullptr;
+  std::unordered_map<std::string, controllers::AttributeProviderService::AttributeMap> extra_attributes_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<TailFile>::getLogger();
 };
 
-REGISTER_RESOURCE(TailFile, "\"Tails\" a file, or a list of files, ingesting data from the file as it is written to the file. The file is expected to be textual."
-                  " Data is ingested only when a new line is encountered (carriage return or new-line character or combination). If the file to tail is periodically \"rolled over\","
-                  " as is generally the case with log files, an optional Rolling Filename Pattern can be used to retrieve data from files that have rolled over, even if the rollover"
-                  " occurred while NiFi was not running (provided that the data still exists upon restart of NiFi). It is generally advisable to set the Run Schedule to a few seconds,"
-                  " rather than running with the default value of 0 secs, as this Processor will consume a lot of resources if scheduled very aggressively. At this time, this Processor"
-                  " does not support ingesting files that have been compressed when 'rolled over'.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
