@@ -32,6 +32,7 @@
 #include "HTTPClient.h"
 #include "processors/GetTCP.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/StringUtils.h"
 #include "core/Core.h"
 #include "core/logging/Logger.h"
@@ -55,8 +56,7 @@ class SecureSocketTest : public IntegrationBase {
  public:
   explicit SecureSocketTest(bool isSecure)
       : isSecure{ isSecure }, isRunning_{ false } {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
@@ -64,9 +64,9 @@ class SecureSocketTest : public IntegrationBase {
     LogTestController::getInstance().setDebug<minifi::io::TLSContext>();
     LogTestController::getInstance().setDebug<core::ProcessSession>();
     LogTestController::getInstance().setTrace<minifi::io::TLSSocket>();
-    LogTestController::getInstance().setTrace<processors::GetTCP>();
+    LogTestController::getInstance().setTrace<minifi::processors::GetTCP>();
     std::fstream file;
-    ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+    ss << dir << utils::file::get_separator() << "tstFile.ext";
     file.open(ss.str(), std::ios::out);
     file << "tempFile";
     file.close();
@@ -86,23 +86,23 @@ class SecureSocketTest : public IntegrationBase {
   }
 
   void queryRootProcessGroup(std::shared_ptr<core::ProcessGroup> pg) override {
-    std::shared_ptr<core::Processor> proc = pg->findProcessorByName("invoke");
+    auto proc = pg->findProcessorByName("invoke");
     assert(proc != nullptr);
 
-    std::shared_ptr<minifi::processors::GetTCP> inv = std::dynamic_pointer_cast<minifi::processors::GetTCP>(proc);
+    auto inv = dynamic_cast<minifi::processors::GetTCP*>(proc);
 
     assert(inv != nullptr);
-    std::string url = "";
-    configuration->set("nifi.remote.input.secure", "true");
+    std::string url;
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_remote_input_secure, "true");
     std::string path = key_dir + "cn.crt.pem";
-    configuration->set("nifi.security.client.certificate", path);
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_security_client_certificate, path);
     path = key_dir + "cn.ckey.pem";
-    configuration->set("nifi.security.client.private.key", path);
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_security_client_private_key, path);
     path = key_dir + "cn.pass";
-    configuration->set("nifi.security.client.pass.phrase", path);
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_security_client_pass_phrase, path);
     path = key_dir + "nifi-cert.pem";
-    configuration->set("nifi.security.client.ca.certificate", path);
-    configuration->set("nifi.c2.enable", "false");
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_security_client_ca_certificate, path);
+    configuration->set(org::apache::nifi::minifi::Configuration::nifi_c2_enable, "false");
     std::string endpoint;
     inv->getProperty(minifi::processors::GetTCP::EndpointList.getName(), endpoint);
     auto endpoints = utils::StringUtils::split(endpoint, ",");
@@ -117,23 +117,15 @@ class SecureSocketTest : public IntegrationBase {
     assert(0 == server_socket_->initialize());
 
     isRunning_ = true;
-    check = [this]() -> bool {
-      return isRunning_;
+    auto handler = [](std::vector<uint8_t> *b) {
+      *b = {'h', 'e', 'l', 'l', 'o', ' ', 'w', 'o', 'r', 'l', 'd', 0, 0, 0, 0, 0, 0, 0, 0, 0};
+      assert(b->size() == 20);
+      return b->size();
     };
-    handler = [](std::vector<uint8_t> *b, int *size) {
-      std::cout << "oh write!" << std::endl;
-      b->reserve(20);
-      memset(b->data(), 0x00, 20);
-      memcpy(b->data(), "hello world", 11);
-      *size = 20;
-      return *size;
-    };
-    server_socket_->registerCallback(check, handler, std::chrono::milliseconds(50));
+    server_socket_->registerCallback([this] { return isRunning_.load(); }, std::move(handler), std::chrono::milliseconds(50));
   }
 
  protected:
-  std::function<bool()> check;
-  std::function<int(std::vector<uint8_t>*b, int *size)> handler;
   bool isSecure;
   std::atomic<bool> isRunning_;
   std::string dir;
