@@ -32,10 +32,14 @@
 #ifdef WIN32
 #include <ws2tcpip.h>
 #include <winsock2.h>
+#pragma comment(lib, "Ws2_32.lib")
 #endif
 #include "properties/Configure.h"
 #include "io/tls/TLSSocket.h"
 #include "io/tls/TLSServerSocket.h"
+#include "utils/net/Socket.h"
+
+namespace minifi = org::apache::nifi::minifi;
 
 #ifdef WIN32
 using SocketDescriptor = SOCKET;
@@ -83,7 +87,7 @@ std::string str_addr(const sockaddr* const sa) {
       const auto addr_str = inet_ntop(AF_INET, &sin.sin_addr, buf, sizeof(buf));
 #endif
       if (!addr_str) {
-        throw std::runtime_error{minifi::io::get_last_socket_error_message()};
+        throw std::runtime_error{minifi::utils::net::get_last_socket_error().message()};
       }
       return std::string{addr_str};
     }
@@ -96,7 +100,7 @@ std::string str_addr(const sockaddr* const sa) {
       const auto addr_str = inet_ntop(AF_INET, &sin6.sin6_addr, buf, sizeof(buf));
 #endif
       if (!addr_str) {
-        throw std::runtime_error{minifi::io::get_last_socket_error_message()};
+        throw std::runtime_error{minifi::utils::net::get_last_socket_error().message()};
       }
       return std::string{addr_str};
     }
@@ -104,7 +108,7 @@ std::string str_addr(const sockaddr* const sa) {
   }
 }
 
-void log_addrinfo(addrinfo* const ai, logging::Logger& logger) {
+void log_addrinfo(addrinfo* const ai, minifi::core::logging::Logger& logger) {
   logger.log_debug(".ai_family: %d %s\n", ai->ai_family, str_family(ai->ai_family));
   logger.log_debug(".ai_socktype: %d %s\n", ai->ai_socktype, str_socktype(ai->ai_socktype));
   logger.log_debug(".ai_protocol: %d %s\n", ai->ai_protocol, str_proto(ai->ai_protocol));
@@ -148,9 +152,9 @@ class SimpleSSLTestClient  {
   SocketDescriptor sfd_;
   std::string host_;
   std::string port_;
-  gsl::not_null<std::shared_ptr<logging::Logger>> logger_{gsl::make_not_null(logging::LoggerFactory<SimpleSSLTestClient>::getLogger())};
+  gsl::not_null<std::shared_ptr<minifi::core::logging::Logger>> logger_{gsl::make_not_null(minifi::core::logging::LoggerFactory<SimpleSSLTestClient>::getLogger())};
 
-  static SocketDescriptor openConnection(const char *host_name, const char *port, logging::Logger& logger) {
+  static SocketDescriptor openConnection(const char *host_name, const char *port, minifi::core::logging::Logger& logger) {
     struct addrinfo hints;
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = AF_UNSPEC;
@@ -164,14 +168,14 @@ class SimpleSSLTestClient  {
       log_addrinfo(addr, logger);
       sfd = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
       if (sfd == INVALID_SOCKET) {
-        logger.log_error("socket: %s\n", minifi::io::get_last_socket_error_message());
+        logger.log_error("socket: %s\n", minifi::utils::net::get_last_socket_error().message());
         continue;
       }
       const auto connect_result = connect(sfd, addr->ai_addr, addr->ai_addrlen);
       if (connect_result == 0) {
         break;
       } else {
-        logger.log_error("connect to %s: %s\n", str_addr(addr->ai_addr), minifi::io::get_last_socket_error_message());
+        logger.log_error("connect to %s: %s\n", str_addr(addr->ai_addr), minifi::utils::net::get_last_socket_error().message());
       }
       sfd = INVALID_SOCKET;
 #ifdef WIN32
@@ -230,7 +234,7 @@ class TLSServerSocketSupportedProtocolsTest {
 
  protected:
     void configureSecurity() {
-      host_ = org::apache::nifi::minifi::io::Socket::getMyHostName();
+      host_ = minifi::io::Socket::getMyHostName();
       port_ = "38778";
       if (!key_dir_.empty()) {
         configuration_->set(minifi::Configure::nifi_remote_input_secure, "true");
@@ -243,22 +247,18 @@ class TLSServerSocketSupportedProtocolsTest {
     }
 
     void createServerSocket() {
-      const auto socket_context = std::make_shared<org::apache::nifi::minifi::io::TLSContext>(configuration_);
-      server_socket_ = utils::make_unique<org::apache::nifi::minifi::io::TLSServerSocket>(socket_context, host_, std::stoi(port_), 3);
+      const auto socket_context = std::make_shared<minifi::io::TLSContext>(configuration_);
+      server_socket_ = std::make_unique<minifi::io::TLSServerSocket>(socket_context, host_, std::stoi(port_), 3);
       assert(0 == server_socket_->initialize());
 
       is_running_ = true;
-      auto check = [this]() -> bool {
-        return is_running_;
+      auto handler = [](std::vector<uint8_t> *bytes_written) {
+        const char contents[] = "hello world";
+        *bytes_written = {std::begin(contents), std::end(contents)};
+        assert(12 == bytes_written->size());
+        return bytes_written->size();
       };
-      auto handler = [](std::vector<uint8_t> *bytes_written, int *size) {
-        std::string contents = "hello world";
-        *bytes_written = {contents.begin(), contents.end()};
-        bytes_written->push_back(0);
-        *size = bytes_written->size();
-        return *size;
-      };
-      server_socket_->registerCallback(check, handler, std::chrono::milliseconds(50));
+      server_socket_->registerCallback([this]{ return is_running_.load(); }, std::move(handler), std::chrono::milliseconds(50));
     }
 
     void verifyTLSServerSocketExclusiveCompatibilityWithTLSv1_2() {
@@ -278,7 +278,7 @@ class TLSServerSocketSupportedProtocolsTest {
     }
 
     std::atomic<bool> is_running_;
-    std::unique_ptr<org::apache::nifi::minifi::io::TLSServerSocket> server_socket_;
+    std::unique_ptr<minifi::io::TLSServerSocket> server_socket_;
     std::string host_;
     std::string port_;
     std::string key_dir_;
