@@ -23,6 +23,7 @@
 #include <fstream>
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/file/FileUtils.h"
 #include "GenerateFlowFile.h"
 #include "PutFile.h"
@@ -32,8 +33,7 @@ TEST_CASE("GenerateFlowFileTest", "[generateflowfiletest]") {
   LogTestController::getInstance().setTrace<TestPlan>();
   LogTestController::getInstance().setWarn<minifi::processors::GenerateFlowFile>();
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   std::shared_ptr<TestPlan> plan = testController.createPlan();
 
@@ -58,13 +58,12 @@ TEST_CASE("GenerateFlowFileTest", "[generateflowfiletest]") {
   std::vector<std::string> file_contents;
 
   auto lambda = [&file_contents](const std::string& path, const std::string& filename) -> bool {
-    std::ifstream is(path + utils::file::FileUtils::get_separator() + filename, std::ifstream::binary);
-    std::string file_content((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>());
-    file_contents.push_back(file_content);
+    std::ifstream is(path + utils::file::get_separator() + filename, std::ifstream::binary);
+    file_contents.push_back(std::string((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>()));
     return true;
   };
 
-  utils::file::FileUtils::list_dir(dir, lambda, plan->getLogger(), false);
+  utils::file::list_dir(dir, lambda, plan->getLogger(), false);
 
   REQUIRE(file_contents.size() == 2);
   REQUIRE(file_contents[0].size() == 10);
@@ -78,8 +77,7 @@ TEST_CASE("GenerateFlowFileWithNonUniqueBinaryData", "[generateflowfiletest]") {
   LogTestController::getInstance().setTrace<TestPlan>();
   LogTestController::getInstance().setWarn<minifi::processors::GenerateFlowFile>();
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   std::shared_ptr<TestPlan> plan = testController.createPlan();
 
@@ -103,7 +101,7 @@ TEST_CASE("GenerateFlowFileWithNonUniqueBinaryData", "[generateflowfiletest]") {
   std::vector<std::vector<char>> fileContents;
 
   auto lambda = [&fileContents](const std::string& path, const std::string& filename) -> bool {
-    std::ifstream is(path + utils::file::FileUtils::get_separator() + filename, std::ifstream::binary);
+    std::ifstream is(path + utils::file::get_separator() + filename, std::ifstream::binary);
 
     is.seekg(0, is.end);
     size_t length = gsl::narrow<size_t>(is.tellg());
@@ -118,7 +116,7 @@ TEST_CASE("GenerateFlowFileWithNonUniqueBinaryData", "[generateflowfiletest]") {
     return true;
   };
 
-  utils::file::FileUtils::list_dir(dir, lambda, plan->getLogger(), false);
+  utils::file::list_dir(dir, lambda, plan->getLogger(), false);
 
   REQUIRE(fileContents.size() == 2);
   REQUIRE(fileContents[0].size() == 10);
@@ -131,8 +129,7 @@ TEST_CASE("GenerateFlowFileTestEmpty", "[generateemptyfiletest]") {
   TestController testController;
   LogTestController::getInstance().setTrace<TestPlan>();
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   std::shared_ptr<TestPlan> plan = testController.createPlan();
 
@@ -150,7 +147,7 @@ TEST_CASE("GenerateFlowFileTestEmpty", "[generateemptyfiletest]") {
   size_t counter = 0;
 
   auto lambda = [&counter](const std::string& path, const std::string& filename) -> bool {
-    std::ifstream is(path + utils::file::FileUtils::get_separator() + filename, std::ifstream::binary);
+    std::ifstream is(path + utils::file::get_separator() + filename, std::ifstream::binary);
 
     is.seekg(0, is.end);
     REQUIRE(is.tellg() == 0);
@@ -160,7 +157,7 @@ TEST_CASE("GenerateFlowFileTestEmpty", "[generateemptyfiletest]") {
     return true;
   };
 
-  utils::file::FileUtils::list_dir(dir, lambda, plan->getLogger(), false);
+  utils::file::list_dir(dir, lambda, plan->getLogger(), false);
 
   REQUIRE(counter == 1);
 }
@@ -169,8 +166,7 @@ TEST_CASE("GenerateFlowFileCustomTextTest", "[generateflowfiletest]") {
   TestController test_controller;
   LogTestController::getInstance().setTrace<TestPlan>();
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = test_controller.createTempDirectory(format);
+  auto dir = test_controller.createTempDirectory();
 
   std::shared_ptr<TestPlan> plan = test_controller.createPlan();
 
@@ -191,13 +187,12 @@ TEST_CASE("GenerateFlowFileCustomTextTest", "[generateflowfiletest]") {
   std::vector<std::string> file_contents;
 
   auto lambda = [&file_contents](const std::string& path, const std::string& filename) -> bool {
-    std::ifstream is(path + utils::file::FileUtils::get_separator() + filename, std::ifstream::binary);
-    std::string file_content((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>());
-    file_contents.push_back(file_content);
+    std::ifstream is(path + utils::file::get_separator() + filename, std::ifstream::binary);
+    file_contents.push_back(std::string((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>()));
     return true;
   };
 
-  utils::file::FileUtils::list_dir(dir, lambda, plan->getLogger(), false);
+  utils::file::list_dir(dir, lambda, plan->getLogger(), false);
 
   REQUIRE(file_contents.size() == 1);
   REQUIRE(file_contents[0].size() == 36);
@@ -207,8 +202,7 @@ TEST_CASE("GenerateFlowFileCustomTextEmptyTest", "[generateflowfiletest]") {
   TestController test_controller;
   LogTestController::getInstance().setTrace<TestPlan>();
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = test_controller.createTempDirectory(format);
+  auto dir = test_controller.createTempDirectory();
 
   std::shared_ptr<TestPlan> plan = test_controller.createPlan();
 
@@ -230,13 +224,12 @@ TEST_CASE("GenerateFlowFileCustomTextEmptyTest", "[generateflowfiletest]") {
   std::vector<std::string> file_contents;
 
   auto lambda = [&file_contents](const std::string& path, const std::string& filename) -> bool {
-    std::ifstream is(path + utils::file::FileUtils::get_separator() + filename, std::ifstream::binary);
-    std::string file_content((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>());
-    file_contents.push_back(file_content);
+    std::ifstream is(path + utils::file::get_separator() + filename, std::ifstream::binary);
+    file_contents.push_back(std::string((std::istreambuf_iterator<char>(is)), std::istreambuf_iterator<char>()));
     return true;
   };
 
-  utils::file::FileUtils::list_dir(dir, lambda, plan->getLogger(), false);
+  utils::file::list_dir(dir, lambda, plan->getLogger(), false);
 
   REQUIRE(file_contents.size() == 1);
   REQUIRE(file_contents[0].size() == 10);
