@@ -27,8 +27,10 @@
 #include <iostream>
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "TestUtils.h"
 #include "core/Core.h"
+#include "unit/ProvenanceTestHelper.h"
 
 #include "core/FlowFile.h"
 #include "core/Processor.h"
@@ -68,9 +70,7 @@ TEST_CASE("Test usage of ExtractText", "[extracttextTest]") {
   std::shared_ptr<TestPlan> plan = testController.createPlan();
   std::shared_ptr<TestRepository> repo = std::make_shared<TestRepository>();
 
-  char dir[] = "/tmp/gt.XXXXXX";
-
-  auto tempdir = testController.createTempDirectory(dir);
+  auto tempdir = testController.createTempDirectory();
   REQUIRE(!tempdir.empty());
 
   std::shared_ptr<core::Processor> getfile = plan->addProcessor("GetFile", "getfileCreate2");
@@ -96,7 +96,7 @@ TEST_CASE("Test usage of ExtractText", "[extracttextTest]") {
       core::Relationship("success", "description"), true);
 
   std::stringstream ss1;
-  ss1 << tempdir << utils::file::FileUtils::get_separator() << TEST_FILE;
+  ss1 << tempdir << utils::file::get_separator() << TEST_FILE;
   std::string test_file_path = ss1.str();
 
   std::ofstream test_file(test_file_path, std::ios::binary);
@@ -133,6 +133,7 @@ TEST_CASE("Test usage of ExtractText", "[extracttextTest]") {
 }
 
 TEST_CASE("TestingFailOnEmptyProperty", "[HashContentPropertiesCheck]") {
+  using minifi::processors::HashContent;
   TestController testController;
   LogTestController::getInstance().setTrace<org::apache::nifi::minifi::processors::LogAttribute>();
   LogTestController::getInstance().setTrace<org::apache::nifi::minifi::processors::GetFile>();
@@ -140,26 +141,38 @@ TEST_CASE("TestingFailOnEmptyProperty", "[HashContentPropertiesCheck]") {
   LogTestController::getInstance().setTrace<org::apache::nifi::minifi::processors::HashContent>();
   std::shared_ptr<TestPlan> plan = testController.createPlan();
 
-  auto tempdir = minifi::utils::createTempDir(&testController);
+  auto tempdir = testController.createTempDirectory();
   std::shared_ptr<core::Processor> getfile = plan->addProcessor("GetFile", "getfileCreate2");
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), tempdir);
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::KeepSourceFile.getName(), "true");
 
   std::shared_ptr<core::Processor> md5processor = plan->addProcessor("HashContent", "HashContentMD5",
                                                                      core::Relationship("success", "description"), true);
-  plan->setProperty(md5processor, org::apache::nifi::minifi::processors::HashContent::HashAttribute.getName(), MD5_ATTR);
-  plan->setProperty(md5processor, org::apache::nifi::minifi::processors::HashContent::HashAlgorithm.getName(), "MD5");
-  plan->setProperty(md5processor, org::apache::nifi::minifi::processors::HashContent::FailOnEmpty.getName(), "true");
+  plan->setProperty(md5processor, HashContent::HashAttribute.getName(), MD5_ATTR);
+  plan->setProperty(md5processor, HashContent::HashAlgorithm.getName(), "MD5");
+
+  md5processor->setAutoTerminatedRelationships({HashContent::Success, HashContent::Failure});
 
   std::stringstream stream_dir;
-  stream_dir << tempdir << utils::file::FileUtils::get_separator() << TEST_FILE;
+  stream_dir << tempdir << utils::file::get_separator() << TEST_FILE;
   std::string test_file_path = stream_dir.str();
   std::ofstream test_file(test_file_path, std::ios::binary);
 
-  plan->runNextProcessor();
-  plan->runNextProcessor();
+  SECTION("with an empty file and fail on empty property set to false") {
+    plan->setProperty(md5processor, HashContent::FailOnEmpty.getName(), "false");
 
-  REQUIRE(LogTestController::getInstance().contains("Failure as flow file is empty"));
+    plan->runNextProcessor();
+    plan->runNextProcessor();
+
+    REQUIRE(LogTestController::getInstance().contains("attempting read"));
+  }
+  SECTION("with an empty file and fail on empty property set to true") {
+    plan->setProperty(md5processor, HashContent::FailOnEmpty.getName(), "true");
+
+    plan->runNextProcessor();
+    plan->runNextProcessor();
+
+    REQUIRE(LogTestController::getInstance().contains("Failure as flow file is empty"));
+  }
 }
-
 #endif  // OPENSSL_SUPPORT
