@@ -15,6 +15,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#define EXTENSION_LIST "*minifi-*,!*http-curl*,!*coap*,!*splunk*"
+
 #include <cstdio>
 #include <utility>
 #include <memory>
@@ -29,6 +31,7 @@
 #endif /* WIN32 */
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "LogAttribute.h"
 #include "GetFile.h"
 #include "unit/ProvenanceTestHelper.h"
@@ -40,8 +43,11 @@
 #include "core/ProcessSession.h"
 #include "core/ProcessorNode.h"
 #include "core/reporting/SiteToSiteProvenanceReportingTask.h"
+#include "core/Resource.h"
+#include "utils/gsl.h"
 #include "utils/PropertyErrors.h"
 #include "utils/IntegrationTestUtils.h"
+#include "Utils.h"
 
 TEST_CASE("Test Creation of GetFile", "[getfileCreate]") {
   TestController testController;
@@ -63,27 +69,26 @@ TEST_CASE("Test GetFileMultiple", "[getfileCreate3]") {
   std::shared_ptr<core::Repository> test_repo = std::make_shared<TestRepository>();
   std::shared_ptr<TestRepository> repo = std::static_pointer_cast<TestRepository>(test_repo);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   utils::Identifier processoruuid = processor->getUUID();
   REQUIRE(processoruuid);
 
-  std::shared_ptr<minifi::Connection> connection = std::make_shared<minifi::Connection>(test_repo, content_repo, "getfileCreate2Connection");
+  auto connection = std::make_unique<minifi::Connection>(test_repo, content_repo, "getfileCreate2Connection");
 
   connection->addRelationship(core::Relationship("success", "description"));
 
   // link the connections so that we can test results at the end for this
-  connection->setSource(processor);
-  connection->setDestination(processor);
+  connection->setSource(processor.get());
+  connection->setDestination(processor.get());
 
   connection->setSourceUUID(processoruuid);
   connection->setDestinationUUID(processoruuid);
 
-  processor->addConnection(connection);
+  processor->addConnection(connection.get());
   REQUIRE(!dir.empty());
 
-  std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(processor);
+  auto node = std::make_shared<core::ProcessorNode>(processor.get());
   auto context = std::make_shared<core::ProcessContext>(node, nullptr, repo, repo, content_repo);
 
   context->setProperty(org::apache::nifi::minifi::processors::GetFile::Directory, dir);
@@ -106,7 +111,7 @@ TEST_CASE("Test GetFileMultiple", "[getfileCreate3]") {
     auto records = reporter->getEvents();
     record = session->get();
     REQUIRE(record == nullptr);
-    REQUIRE(records.size() == 0);
+    REQUIRE(records.empty());
 
     std::fstream file;
     std::stringstream ss;
@@ -125,7 +130,7 @@ TEST_CASE("Test GetFileMultiple", "[getfileCreate3]") {
 
     records = reporter->getEvents();
 
-    for (auto provEventRecord : records) {
+    for (const auto& provEventRecord : records) {
       REQUIRE(provEventRecord->getComponentType() == processor->getName());
     }
     session->commit();
@@ -134,7 +139,7 @@ TEST_CASE("Test GetFileMultiple", "[getfileCreate3]") {
 
     // one CREATE, one MODIFY, and one FF contents, as we use the same
     // underlying repo for both provenance and flowFileRepo
-    REQUIRE(repo->getRepoMap().size() == 3 * i);
+    REQUIRE(repo->getRepoMap().size() == gsl::narrow<size_t>(3 * i));
   }
 }
 
@@ -148,8 +153,7 @@ TEST_CASE("Test GetFile Ignore", "[getfileCreate3]") {
   std::shared_ptr<core::Repository> test_repo = std::make_shared<TestRepository>();
   std::shared_ptr<TestRepository> repo = std::static_pointer_cast<TestRepository>(test_repo);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  const auto dir = testController.createTempDirectory(format);
+  const auto dir = testController.createTempDirectory();
 
   utils::Identifier processoruuid = processor->getUUID();
   REQUIRE(processoruuid);
@@ -159,16 +163,16 @@ TEST_CASE("Test GetFile Ignore", "[getfileCreate3]") {
   connection->addRelationship(core::Relationship("success", "description"));
 
   // link the connections so that we can test results at the end for this
-  connection->setSource(processor);
-  connection->setDestination(processor);
+  connection->setSource(processor.get());
+  connection->setDestination(processor.get());
 
   connection->setSourceUUID(processoruuid);
   connection->setDestinationUUID(processoruuid);
 
-  processor->addConnection(connection);
+  processor->addConnection(connection.get());
   REQUIRE(!dir.empty());
 
-  std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(processor);
+  auto node = std::make_shared<core::ProcessorNode>(processor.get());
   auto context = std::make_shared<core::ProcessContext>(node, nullptr, repo, repo, content_repo);
 
   context->setProperty(org::apache::nifi::minifi::processors::GetFile::Directory, dir);
@@ -239,8 +243,8 @@ TEST_CASE("TestConnectionFull", "[ConnectionFull]") {
   content_repo->initialize(std::make_shared<minifi::Configure>());
   std::shared_ptr<core::Processor> processor = std::make_shared<org::apache::nifi::minifi::processors::GenerateFlowFile>("GFF");
   processor->initialize();
-  processor->setProperty(processors::GenerateFlowFile::BatchSize, "10");
-  processor->setProperty(processors::GenerateFlowFile::FileSize, "0");
+  processor->setProperty(minifi::processors::GenerateFlowFile::BatchSize, "10");
+  processor->setProperty(minifi::processors::GenerateFlowFile::FileSize, "0");
 
 
   std::shared_ptr<core::Repository> test_repo = std::make_shared<TestRepository>();
@@ -254,16 +258,16 @@ TEST_CASE("TestConnectionFull", "[ConnectionFull]") {
   utils::Identifier processoruuid = processor->getUUID();
 
   // link the connections so that we can test results at the end for this
-  connection->setSource(processor);
-  connection->setDestination(processor);
+  connection->setSource(processor.get());
+  connection->setDestination(processor.get());
 
   connection->setSourceUUID(processoruuid);
   connection->setDestinationUUID(processoruuid);
 
-  processor->addConnection(connection);
+  processor->addConnection(connection.get());
   processor->setScheduledState(core::ScheduledState::RUNNING);
 
-  std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(processor);
+  auto node = std::make_shared<core::ProcessorNode>(processor.get());
   auto context = std::make_shared<core::ProcessContext>(node, nullptr, repo, repo, content_repo);
 
   auto factory = std::make_shared<core::ProcessSessionFactory>(context);
@@ -294,8 +298,7 @@ TEST_CASE("LogAttributeTest", "[getfileCreate3]") {
 
   plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   testController.runSession(plan, false);
@@ -338,12 +341,11 @@ TEST_CASE("LogAttributeTestInvalid", "[TestLogAttribute]") {
 
   auto loggattr = plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::BatchSize.getName(), "1");
-  REQUIRE_THROWS_AS(plan->setProperty(loggattr, org::apache::nifi::minifi::processors::LogAttribute::FlowFilesToLog.getName(), "-1"), utils::internal::ParseException&);
+  REQUIRE_THROWS_AS(plan->setProperty(loggattr, org::apache::nifi::minifi::processors::LogAttribute::FlowFilesToLog.getName(), "-1"), utils::internal::ParseException);
   LogTestController::getInstance().reset();
 }
 
@@ -357,8 +359,7 @@ void testMultiplesLogAttribute(int fileCount, int flowsToLog, std::string verify
 
   auto loggattr = plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   auto flowsToLogStr = std::to_string(flowsToLog);
   if (verifyStringFlowsLogged.empty())
@@ -425,9 +426,8 @@ TEST_CASE("Test Find file", "[getfileCreate3]") {
   std::shared_ptr<core::Processor> processorReport = std::make_shared<org::apache::nifi::minifi::core::reporting::SiteToSiteProvenanceReportingTask>(
       minifi::io::StreamFactory::getInstance(std::make_shared<org::apache::nifi::minifi::Configure>()), std::make_shared<org::apache::nifi::minifi::Configure>());
   plan->addProcessor(processorReport, "reporter", core::Relationship("success", "description"), false);
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
 
+  auto dir = testController.createTempDirectory();
   plan->setProperty(processor, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   testController.runSession(plan, false);
   auto records = plan->getProvenanceRecords();
@@ -456,8 +456,8 @@ TEST_CASE("Test Find file", "[getfileCreate3]") {
   REQUIRE(2 == repo->getRepoMap().size());
 
   for (auto entry : repo->getRepoMap()) {
-    provenance::ProvenanceEventRecord newRecord;
-    newRecord.DeSerialize(reinterpret_cast<uint8_t*>(const_cast<char*>(entry.second.data())), entry.second.length());
+    minifi::provenance::ProvenanceEventRecord newRecord;
+    newRecord.DeSerialize(gsl::make_span(entry.second).as_span<const std::byte>());
 
     bool found = false;
     for (auto provRec : records) {
@@ -479,7 +479,7 @@ TEST_CASE("Test Find file", "[getfileCreate3]") {
       org::apache::nifi::minifi::core::reporting::SiteToSiteProvenanceReportingTask>(processorReport);
   taskReport->setBatchSize(1);
   std::vector<std::shared_ptr<core::SerializableComponent>> recordsReport;
-  recordsReport.push_back(std::make_shared<provenance::ProvenanceEventRecord>());
+  recordsReport.push_back(std::make_shared<minifi::provenance::ProvenanceEventRecord>());
   processorReport->incrementActiveTasks();
   processorReport->setScheduledState(core::ScheduledState::RUNNING);
   std::string jsonStr;
@@ -571,7 +571,7 @@ void testRPGBypass(const std::string &host, const std::string &port, const std::
   rpg->initialize();
   REQUIRE(rpg->setProperty(minifi::RemoteProcessorGroupPort::hostName, host));
   rpg->setProperty(minifi::RemoteProcessorGroupPort::port, port);
-  std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(rpg);
+  auto node = std::make_shared<core::ProcessorNode>(rpg.get());
   auto context = std::make_shared<core::ProcessContext>(node, nullptr, repo, repo, content_repo);
   auto psf = std::make_shared<core::ProcessSessionFactory>(context);
   if (hasException) {
@@ -635,14 +635,14 @@ ProcessorWithIncomingConnectionTest::ProcessorWithIncomingConnectionTest() {
   const auto content_repo = std::make_shared<core::repository::VolatileContentRepository>();
   content_repo->initialize(std::make_shared<minifi::Configure>());
 
-  processor_ = std::make_shared<processors::LogAttribute>("test_processor");
+  processor_ = std::make_shared<minifi::processors::LogAttribute>("test_processor");
   incoming_connection_ = std::make_shared<minifi::Connection>(repo, content_repo, "incoming_connection");
   incoming_connection_->addRelationship(core::Relationship{"success", ""});
   incoming_connection_->setDestinationUUID(processor_->getUUID());
-  processor_->addConnection(incoming_connection_);
+  processor_->addConnection(incoming_connection_.get());
   processor_->initialize();
 
-  const auto processor_node = std::make_shared<core::ProcessorNode>(processor_);
+  const auto processor_node = std::make_shared<core::ProcessorNode>(processor_.get());
   const auto context = std::make_shared<core::ProcessContext>(processor_node, nullptr, repo, repo, content_repo);
   const auto session_factory = std::make_shared<core::ProcessSessionFactory>(context);
   session_ = session_factory->createSession();
@@ -780,3 +780,50 @@ TEST_CASE("InputRequirementTestRequired", "[InputRequirement]") {
   REQUIRE_THROWS_WITH(plan->validateAnnotations(), Catch::EndsWith("INPUT_REQUIRED was specified for the processor, but no incoming connections were found"));
   testController.runSession(plan);
 }
+
+TEST_CASE("isSingleThreaded - one thread for a multithreaded processor", "[isSingleThreaded]") {
+  TestController testController;
+
+  std::shared_ptr<TestPlan> plan = testController.createPlan();
+  auto processor = plan->addProcessor("GenerateFlowFile", "myProc");
+  // default max concurrent tasks value is 1 for every processor
+
+  REQUIRE_NOTHROW(plan->validateAnnotations());
+  REQUIRE(processor->getMaxConcurrentTasks() == 1);
+}
+
+TEST_CASE("isSingleThreaded - two threads for a multithreaded processor", "[isSingleThreaded]") {
+  TestController testController;
+
+  std::shared_ptr<TestPlan> plan = testController.createPlan();
+  auto processor = plan->addProcessor("GenerateFlowFile", "myProc");
+  processor->setMaxConcurrentTasks(2);
+
+  REQUIRE_NOTHROW(plan->validateAnnotations());
+  REQUIRE(processor->getMaxConcurrentTasks() == 2);
+}
+
+TEST_CASE("isSingleThreaded - one thread for a single threaded processor", "[isSingleThreaded]") {
+  TestController testController;
+
+  std::shared_ptr<TestPlan> plan = testController.createPlan();
+  auto processor = plan->addProcessor("TailFile", "myProc");
+  // default max concurrent tasks value is 1 for every processor
+
+  REQUIRE_NOTHROW(plan->validateAnnotations());
+  REQUIRE(processor->getMaxConcurrentTasks() == 1);
+}
+
+TEST_CASE("isSingleThreaded - two threads for a single threaded processor", "[isSingleThreaded]") {
+  TestController testController;
+  LogTestController::getInstance().setDebug<minifi::core::Processor>();
+
+  std::shared_ptr<TestPlan> plan = testController.createPlan();
+  auto processor = plan->addProcessor("TailFile", "myProc");
+  processor->setMaxConcurrentTasks(2);
+
+  REQUIRE_NOTHROW(plan->validateAnnotations());
+  REQUIRE(processor->getMaxConcurrentTasks() == 1);
+  REQUIRE(LogTestController::getInstance().contains("[warning] Processor myProc can not be run in parallel, its "
+                                                    "\"max concurrent tasks\" value is too high. It was set to 1 from 2."));
+}
