@@ -28,6 +28,7 @@
 
 #include "utils/file/FileUtils.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "TestUtils.h"
 #include "processors/LogAttribute.h"
 #include "processors/GetFile.h"
@@ -65,10 +66,8 @@ TEST_CASE("PutFileTest", "[getfileputpfile]") {
 
   plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
 
@@ -81,7 +80,7 @@ TEST_CASE("PutFileTest", "[getfileputpfile]") {
 
   std::fstream file;
   std::stringstream ss;
-  ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  ss << dir << utils::file::get_separator() << "tstFile.ext";
   file.open(ss.str(), std::ios::out);
   file << "tempFile";
   file.close();
@@ -103,7 +102,7 @@ TEST_CASE("PutFileTest", "[getfileputpfile]") {
   // verify that the fle was moved
   REQUIRE(false == std::ifstream(ss.str()).good());
   std::stringstream movedFile;
-  movedFile << putfiledir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  movedFile << putfiledir << utils::file::get_separator() << "tstFile.ext";
   REQUIRE(true == std::ifstream(movedFile.str()).good());
 
   file.open(movedFile.str(), std::ios::in);
@@ -130,10 +129,8 @@ TEST_CASE("PutFileTestFileExists", "[getfileputpfile]") {
 
   plan->addProcessor("LogAttribute", "logattribute", core::Relationship("failure", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
 
@@ -146,13 +143,13 @@ TEST_CASE("PutFileTestFileExists", "[getfileputpfile]") {
 
   std::fstream file;
   std::stringstream ss;
-  ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  ss << dir << utils::file::get_separator() << "tstFile.ext";
   file.open(ss.str(), std::ios::out);
   file << "tempFile";
   file.close();
 //
   std::stringstream movedFile;
-  movedFile << putfiledir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  movedFile << putfiledir << utils::file::get_separator() << "tstFile.ext";
   file.open(movedFile.str(), std::ios::out);
   file << "tempFile";
   file.close();
@@ -195,10 +192,8 @@ TEST_CASE("PutFileTestFileExistsIgnore", "[getfileputpfile]") {
 
   plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::ConflictResolution.getName(), "ignore");
@@ -212,17 +207,17 @@ TEST_CASE("PutFileTestFileExistsIgnore", "[getfileputpfile]") {
 
   std::fstream file;
   std::stringstream ss;
-  ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  ss << dir << utils::file::get_separator() << "tstFile.ext";
   file.open(ss.str(), std::ios::out);
   file << "tempFile";
   file.close();
 //
   std::stringstream movedFile;
-  movedFile << putfiledir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  movedFile << putfiledir << utils::file::get_separator() << "tstFile.ext";
   file.open(movedFile.str(), std::ios::out);
   file << "tempFile";
   file.close();
-  auto filemodtime = fileutils::FileUtils::last_write_time(movedFile.str());
+  auto filemodtime = utils::file::last_write_time(movedFile.str());
 
   std::this_thread::sleep_for(std::chrono::milliseconds(1000));
   plan->reset();
@@ -243,7 +238,7 @@ TEST_CASE("PutFileTestFileExistsIgnore", "[getfileputpfile]") {
   // verify that the fle was moved
   REQUIRE(false == std::ifstream(ss.str()).good());
   REQUIRE(true == std::ifstream(movedFile.str()).good());
-  REQUIRE(filemodtime == fileutils::FileUtils::last_write_time(movedFile.str()));
+  REQUIRE(filemodtime == utils::file::last_write_time(movedFile.str()));
   LogTestController::getInstance().reset();
 }
 
@@ -263,10 +258,8 @@ TEST_CASE("PutFileTestFileExistsReplace", "[getfileputpfile]") {
 
   plan->addProcessor("LogAttribute", "logattribute", { core::Relationship("success", "d"), core::Relationship("failure", "d") }, true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::ConflictResolution.getName(), "replace");
@@ -280,17 +273,17 @@ TEST_CASE("PutFileTestFileExistsReplace", "[getfileputpfile]") {
 
   std::fstream file;
   std::stringstream ss;
-  ss << dir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  ss << dir << utils::file::get_separator() << "tstFile.ext";
   file.open(ss.str(), std::ios::out);
   file << "tempFile";
   file.close();
 //
   std::stringstream movedFile;
-  movedFile << putfiledir << utils::file::FileUtils::get_separator() << "tstFile.ext";
+  movedFile << putfiledir << utils::file::get_separator() << "tstFile.ext";
   file.open(movedFile.str(), std::ios::out);
   file << "tempFile";
   file.close();
-  auto filemodtime = fileutils::FileUtils::last_write_time(movedFile.str());
+  auto filemodtime = utils::file::last_write_time(movedFile.str());
 
   std::this_thread::sleep_for(std::chrono::milliseconds(1000));
   plan->reset();
@@ -312,7 +305,7 @@ TEST_CASE("PutFileTestFileExistsReplace", "[getfileputpfile]") {
   REQUIRE(false == std::ifstream(ss.str()).good());
   REQUIRE(true == std::ifstream(movedFile.str()).good());
 #ifndef WIN32
-  REQUIRE(filemodtime != fileutils::FileUtils::last_write_time(movedFile.str()));
+  REQUIRE(filemodtime != utils::file::last_write_time(movedFile.str()));
 #endif
   LogTestController::getInstance().reset();
 }
@@ -320,7 +313,7 @@ TEST_CASE("PutFileTestFileExistsReplace", "[getfileputpfile]") {
 TEST_CASE("Test generation of temporary write path", "[putfileTmpWritePath]") {
   auto processor = std::make_shared<org::apache::nifi::minifi::processors::PutFile>("processorname");
   std::stringstream prefix;
-  prefix << "a" << utils::file::FileUtils::get_separator() << "b" << utils::file::FileUtils::get_separator();
+  prefix << "a" << utils::file::get_separator() << "b" << utils::file::get_separator();
   std::string path = prefix.str() + "c";
   std::string expected_path = prefix.str() + ".c";
   REQUIRE(processor->tmpWritePath(path, "").substr(1, expected_path.length()) == expected_path);
@@ -342,10 +335,8 @@ TEST_CASE("PutFileMaxFileCountTest", "[getfileputpfilemaxcount]") {
 
   plan->addProcessor("LogAttribute", "logattribute", { core::Relationship("success", "d"), core::Relationship("failure", "d") }, true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  const auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::BatchSize.getName(), "1");
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
@@ -355,7 +346,7 @@ TEST_CASE("PutFileMaxFileCountTest", "[getfileputpfilemaxcount]") {
 
   for (int i = 0; i < 2; ++i) {
     std::stringstream ss;
-    ss << dir << utils::file::FileUtils::get_separator() << "tstFile" << i << ".ext";
+    ss << dir << utils::file::get_separator() << "tstFile" << i << ".ext";
     std::fstream file;
     file.open(ss.str(), std::ios::out);
     file << "tempFile";
@@ -371,7 +362,7 @@ TEST_CASE("PutFileMaxFileCountTest", "[getfileputpfilemaxcount]") {
   testController.runSession(plan);
 
 
-  REQUIRE(LogTestController::getInstance().contains("key:absolute.path value:" + std::string(dir) + utils::file::FileUtils::get_separator() + "tstFile0.ext"));
+  REQUIRE(LogTestController::getInstance().contains("key:absolute.path value:" + std::string(dir) + utils::file::get_separator() + "tstFile0.ext"));
   REQUIRE(LogTestController::getInstance().contains("Size:8 Offset:0"));
   REQUIRE(LogTestController::getInstance().contains("key:path value:" + std::string(dir)));
 
@@ -381,7 +372,7 @@ TEST_CASE("PutFileMaxFileCountTest", "[getfileputpfilemaxcount]") {
 
   for (int i = 0; i < 2; ++i) {
     std::stringstream ss;
-    ss << putfiledir << utils::file::FileUtils::get_separator() << "tstFile" << i << ".ext";
+    ss << putfiledir << utils::file::get_separator() << "tstFile" << i << ".ext";
     std::ifstream file(ss.str());
     if (file.is_open() && file.good()) {
       files_in_dir++;
@@ -409,21 +400,19 @@ TEST_CASE("PutFileEmptyTest", "[EmptyFilePutTest]") {
 
   std::shared_ptr<core::Processor> putfile = plan->addProcessor("PutFile", "putfile", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2);
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory();
 
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
 
-  std::ofstream of(std::string(dir) + utils::file::FileUtils::get_separator() + "tstFile.ext");
+  std::ofstream of(std::string(dir) + utils::file::get_separator() + "tstFile.ext");
   of.close();
 
   plan->runNextProcessor();  // Get
   plan->runNextProcessor();  // Put
 
-  std::ifstream is(std::string(putfiledir) + utils::file::FileUtils::get_separator() + "tstFile.ext", std::ifstream::binary);
+  std::ifstream is(std::string(putfiledir) + utils::file::get_separator() + "tstFile.ext", std::ifstream::binary);
 
   REQUIRE(is.is_open());
   is.seekg(0, is.end);
@@ -444,10 +433,8 @@ TEST_CASE("TestPutFilePermissions", "[PutFilePermissions]") {
 
   std::shared_ptr<core::Processor> putfile = plan->addProcessor("PutFile", "putfile", core::Relationship("success", "description"), true);
 
-  char format[] = "/tmp/gt.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
-  char format2[] = "/tmp/ft.XXXXXX";
-  auto putfiledir = testController.createTempDirectory(format2) + utils::file::FileUtils::get_separator() + "test_dir";
+  const auto dir = testController.createTempDirectory();
+  const auto putfiledir = testController.createTempDirectory() + utils::file::get_separator() + "test_dir";
 
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
@@ -484,9 +471,9 @@ TEST_CASE("PutFileCreateDirectoryTest", "[PutFileProperties]") {
   plan->addProcessor("LogAttribute", "logattribute", core::Relationship("success", "description"), true);
 
   // Define Directory
-  auto dir = minifi::utils::createTempDir(&testController);
+  auto dir = testController.createTempDirectory();
   // Defining a sub directory
-  auto putfiledir = minifi::utils::createTempDir(&testController) + utils::file::FileUtils::get_separator() + "test_dir";
+  auto putfiledir = testController.createTempDirectory() + utils::file::FileUtils::get_separator() + "test_dir";
 
   plan->setProperty(putfile, org::apache::nifi::minifi::processors::PutFile::Directory.getName(), putfiledir);
   plan->setProperty(getfile, org::apache::nifi::minifi::processors::GetFile::Directory.getName(), dir);
@@ -501,8 +488,8 @@ TEST_CASE("PutFileCreateDirectoryTest", "[PutFileProperties]") {
     plan->runNextProcessor();
     plan->runNextProcessor();
 
-    REQUIRE(org::apache::nifi::minifi::utils::file::exists(putfiledir));
-    REQUIRE(org::apache::nifi::minifi::utils::file::exists(path));
+    REQUIRE(utils::file::exists(putfiledir));
+    REQUIRE(utils::file::exists(path));
   }
 
   SECTION("with an empty file and create directory property set to false") {
@@ -516,8 +503,8 @@ TEST_CASE("PutFileCreateDirectoryTest", "[PutFileProperties]") {
     plan->runNextProcessor();
     plan->runNextProcessor();
 
-    REQUIRE_FALSE(org::apache::nifi::minifi::utils::file::exists(putfiledir));
-    REQUIRE_FALSE(org::apache::nifi::minifi::utils::file::exists(path));
+    REQUIRE_FALSE(utils::file::exists(putfiledir));
+    REQUIRE_FALSE(utils::file::exists(path));
     std::string check = "Failed to create empty file: " + path;
     REQUIRE(LogTestController::getInstance().contains(check));
   }
@@ -533,8 +520,8 @@ TEST_CASE("PutFileCreateDirectoryTest", "[PutFileProperties]") {
     plan->runNextProcessor();
     plan->runNextProcessor();
 
-    REQUIRE(org::apache::nifi::minifi::utils::file::exists(putfiledir));
-    REQUIRE(org::apache::nifi::minifi::utils::file::exists(path));
+    REQUIRE(utils::file::exists(putfiledir));
+    REQUIRE(utils::file::exists(path));
   }
 
   SECTION("with a non-empty file and create directory property set to false") {
@@ -549,8 +536,8 @@ TEST_CASE("PutFileCreateDirectoryTest", "[PutFileProperties]") {
     plan->runNextProcessor();
     plan->runNextProcessor();
 
-    REQUIRE_FALSE(org::apache::nifi::minifi::utils::file::exists(putfiledir));
-    REQUIRE_FALSE(org::apache::nifi::minifi::utils::file::exists(path));
+    REQUIRE_FALSE(utils::file::exists(putfiledir));
+    REQUIRE_FALSE(utils::file::exists(path));
     std::string check = "PutFile commit put file operation to " + path + " failed because write failed";
     REQUIRE(LogTestController::getInstance().contains(check));
   }
