@@ -16,13 +16,14 @@
  * limitations under the License.
  */
 
-#define CATCH_CONFIG_MAIN
-
 #include <memory>
+#include <optional>
 #include <string>
 #include <set>
+#include <regex>
 
 #include "TestBase.h"
+#include "Catch.h"
 
 #include "processors/GenerateFlowFile.h"
 #include "processors/UpdateAttribute.h"
@@ -30,13 +31,10 @@
 #include "processors/PutFile.h"
 #include "processors/LogAttribute.h"
 #include "utils/file/FileUtils.h"
-#include "utils/OptionalUtils.h"
-#include "utils/RegexUtils.h"
 #include "utils/TestUtils.h"
 
 namespace {
-using org::apache::nifi::minifi::utils::createTempDir;
-using org::apache::nifi::minifi::utils::optional;
+using std::optional;
 namespace FileUtils = org::apache::nifi::minifi::utils::file;
 
 class RetryFlowFileTest {
@@ -86,8 +84,6 @@ class RetryFlowFileTest {
       optional<bool> processor_uuid_matches_flowfile) {
     reInitialize();
 
-    const std::string output_dir = createTempDir(testController_.get());
-
     // Relationships
     const core::Relationship success         {"success", "description"};
     const core::Relationship retry           {RetryFlowFile::Retry};
@@ -140,9 +136,9 @@ class RetryFlowFileTest {
     plan_->setProperty(retryflowfile, "retries_exceeded_property_key_1", "retries_exceeded_property_value_1", true);
     plan_->setProperty(retryflowfile, "retries_exceeded_property_key_2", "retries_exceeded_property_value_2", true);
 
-    const std::string retry_dir            = createTempDir(testController_.get());
-    const std::string retries_exceeded_dir = createTempDir(testController_.get());
-    const std::string failure_dir          = createTempDir(testController_.get());
+    const std::string retry_dir            = testController_->createTempDirectory();
+    const std::string retries_exceeded_dir = testController_->createTempDirectory();
+    const std::string failure_dir          = testController_->createTempDirectory();
 
     plan_->setProperty(putfile_on_retry, PutFile::Directory.getName(), retry_dir);
     plan_->setProperty(putfile_on_retries_exceeded, PutFile::Directory.getName(), retries_exceeded_dir);
@@ -172,8 +168,8 @@ class RetryFlowFileTest {
   }
 
   bool flowfileWasPenalizedARetryflowfile() {
-    utils::Regex re(R"(\[org::apache::nifi::minifi::core::ProcessSession\] \[info\] Penalizing [0-9a-z\-]+ for [0-9]*ms at retryflowfile)");
-    return re.match(LogTestController::getInstance().log_output.str());
+    std::regex re(R"(\[org::apache::nifi::minifi::core::ProcessSession\] \[info\] Penalizing [0-9a-z\-]+ for [0-9]*ms at retryflowfile)");
+    return std::regex_search(LogTestController::getInstance().log_output.str(), re);
   }
 
   bool retryFlowfileWarnedForReuse() {
