@@ -17,12 +17,19 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+#include "GetUSBCamera.h"
+
 #include <png.h>
 
 #include <utility>
+#include <string>
 #include <vector>
+#include <set>
+#include <algorithm>
 
-#include "GetUSBCamera.h"
+#include "utils/gsl.h"
+#include "core/ProcessSessionFactory.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -76,7 +83,7 @@ void GetUSBCamera::onFrame(uvc_frame_t *frame, void *ptr) {
     return;
   }
 
-  auto now = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::system_clock::now().time_since_epoch());
+  auto now = std::chrono::steady_clock::now();
 
   if (now - cb_data->last_frame_time < std::chrono::milliseconds(static_cast<int>(1000.0 / cb_data->target_fps))) {
     return;
@@ -102,25 +109,18 @@ void GetUSBCamera::onFrame(uvc_frame_t *frame, void *ptr) {
     flow_file->getAttribute("filename", flow_file_name);
     cb_data->logger->log_info("Created flow file: %s", flow_file_name);
 
-    // Initialize callback according to output format
-    std::shared_ptr<OutputStreamCallback> write_cb;
-
-    if (cb_data->format == "PNG") {
-      write_cb = std::make_shared<GetUSBCamera::PNGWriteCallback>(cb_data->png_write_mtx,
-                                                                  cb_data->frame_buffer,
-                                                                  cb_data->device_width,
-                                                                  cb_data->device_height);
-    } else if (cb_data->format == "RAW") {
-      write_cb = std::make_shared<GetUSBCamera::RawWriteCallback>(cb_data->frame_buffer);
+    if (cb_data->format == "RAW") {
+      session->writeBuffer(flow_file, gsl::make_span(static_cast<const std::byte*>(cb_data->frame_buffer->data), cb_data->frame_buffer->data_bytes));
     } else {
-      cb_data->logger->log_warn("Invalid format specified (%s); defaulting to PNG", cb_data->format);
-      write_cb = std::make_shared<GetUSBCamera::PNGWriteCallback>(cb_data->png_write_mtx,
-                                                                  cb_data->frame_buffer,
-                                                                  cb_data->device_width,
-                                                                  cb_data->device_height);
+      if (cb_data->format != "PNG") {
+        cb_data->logger->log_warn("Invalid format specified (%s); defaulting to PNG", cb_data->format);
+      }
+      session->write(flow_file, GetUSBCamera::PNGWriteCallback{
+          cb_data->png_write_mtx,
+          cb_data->frame_buffer,
+          cb_data->device_width,
+          cb_data->device_height});
     }
-
-    session->write(flow_file, write_cb.get());
     session->transfer(flow_file, GetUSBCamera::Success);
     session->commit();
   } catch (std::exception &exception) {
@@ -333,7 +333,7 @@ void GetUSBCamera::onSchedule(core::ProcessContext *context, core::ProcessSessio
         cb_data_.device_height = height;
         cb_data_.device_fps = fps;
         cb_data_.target_fps = target_fps;
-        cb_data_.last_frame_time = std::chrono::milliseconds(0);
+        cb_data_.last_frame_time = std::chrono::steady_clock::time_point();
 
         res = uvc_start_streaming(devh_, &ctrl, onFrame, &cb_data_, 0);
 
@@ -397,11 +397,10 @@ GetUSBCamera::PNGWriteCallback::PNGWriteCallback(std::shared_ptr<std::mutex> wri
     : png_write_mtx_(std::move(write_mtx)),
       frame_(frame),
       width_(width),
-      height_(height),
-      logger_(logging::LoggerFactory<PNGWriteCallback>::getLogger()) {
+      height_(height) {
 }
 
-int64_t GetUSBCamera::PNGWriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
+int64_t GetUSBCamera::PNGWriteCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) {
   std::lock_guard<std::mutex> lock(*png_write_mtx_);
   logger_->log_info("Writing %d bytes of raw capture data to PNG output", frame_->data_bytes);
   png_structp png = png_create_write_struct(PNG_LIBPNG_VER_STRING, nullptr, nullptr, nullptr);
@@ -457,18 +456,11 @@ int64_t GetUSBCamera::PNGWriteCallback::process(const std::shared_ptr<io::BaseSt
     throw;
   }
 
-  return stream->write(png_output_buf_.data(), png_output_buf_.size());
+  const auto write_ret = stream->write(png_output_buf_.data(), png_output_buf_.size());
+  return io::isError(write_ret) ? -1 : gsl::narrow<int64_t>(write_ret);
 }
 
-GetUSBCamera::RawWriteCallback::RawWriteCallback(uvc_frame_t *frame)
-    : frame_(frame),
-      logger_(logging::LoggerFactory<RawWriteCallback>::getLogger()) {
-}
-
-int64_t GetUSBCamera::RawWriteCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
-  logger_->log_info("Writing %d bytes of raw capture data", frame_->data_bytes);
-  return stream->write(reinterpret_cast<uint8_t *>(frame_->data), frame_->data_bytes);
-}
+REGISTER_RESOURCE(GetUSBCamera, "Gets images from USB Video Class (UVC)-compatible devices. Outputs one flow file per frame at the rate specified by the FPS property in the format specified by the Format property.");  // NOLINT line length
 
 } /* namespace processors */
 } /* namespace minifi */
