@@ -17,21 +17,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef LIBMINIFI_INCLUDE_PROCESSORS_GETUSBCAMERA_H_
-#define LIBMINIFI_INCLUDE_PROCESSORS_GETUSBCAMERA_H_
+#pragma once
 
 #include <list>
 #include <memory>
 #include <string>
+#include <vector>
 
-#include <libuvc/libuvc.h>
+#include "libuvc/libuvc.h"
 
 #include "FlowFileRecord.h"
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -42,13 +41,12 @@ namespace processors {
 class GetUSBCamera : public core::Processor {
  public:
   explicit GetUSBCamera(const std::string &name, const utils::Identifier &uuid = {})
-      : core::Processor(name, uuid),
-        logger_(logging::LoggerFactory<GetUSBCamera>::getLogger()) {
+      : core::Processor(name, uuid) {
     png_write_mtx_ = std::make_shared<std::mutex>();
     dev_access_mtx_ = std::make_shared<std::recursive_mutex>();
   }
 
-  virtual ~GetUSBCamera() {
+  ~GetUSBCamera() override {
     // We cannot interrupt the PNG write process
     std::lock_guard<std::mutex> lock(*png_write_mtx_);
     cleanupUvc();
@@ -80,7 +78,7 @@ class GetUSBCamera : public core::Processor {
   typedef struct {
     core::ProcessContext *context;
     core::ProcessSessionFactory *session_factory;
-    std::shared_ptr<logging::Logger> logger;
+    std::shared_ptr<core::logging::Logger> logger;
     std::shared_ptr<std::mutex> png_write_mtx;
     std::shared_ptr<std::recursive_mutex> dev_access_mtx;
     std::string format;
@@ -89,16 +87,16 @@ class GetUSBCamera : public core::Processor {
     uint16_t device_height;
     uint32_t device_fps;
     double target_fps;
-    std::chrono::milliseconds last_frame_time;
+    std::chrono::steady_clock::time_point last_frame_time;
   } CallbackData;
 
   static void onFrame(uvc_frame_t *frame, void *ptr);
 
   // Write callback for storing camera capture data in PNG format
-  class PNGWriteCallback : public OutputStreamCallback {
+  class PNGWriteCallback {
    public:
     PNGWriteCallback(std::shared_ptr<std::mutex> write_mtx, uvc_frame_t *frame, uint32_t width, uint32_t height);
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
+    int64_t operator()(const std::shared_ptr<io::BaseStream>& stream);
 
    private:
     std::shared_ptr<std::mutex> png_write_mtx_;
@@ -106,22 +104,11 @@ class GetUSBCamera : public core::Processor {
     const uint32_t width_;
     const uint32_t height_;
     std::vector<uint8_t> png_output_buf_;
-    std::shared_ptr<logging::Logger> logger_;
-  };
-
-  // Write callback for storing camera capture data as a raw RGB pixel buffer
-  class RawWriteCallback : public OutputStreamCallback {
-   public:
-    explicit RawWriteCallback(uvc_frame_t *frame);
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) override;
-
-   private:
-    uvc_frame_t *frame_;
-    std::shared_ptr<logging::Logger> logger_;
+    std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<PNGWriteCallback>::getLogger();
   };
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<GetUSBCamera>::getLogger();
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 
   std::shared_ptr<std::thread> camera_thread_;
@@ -138,12 +125,8 @@ class GetUSBCamera : public core::Processor {
   void cleanupUvc();
 };
 
-REGISTER_RESOURCE(GetUSBCamera, "Gets images from USB Video Class (UVC)-compatible devices. Outputs one flow file per frame at the rate specified by the FPS property in the format specified by the Format property.");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif  // LIBMINIFI_INCLUDE_PROCESSORS_GETUSBCAMERA_H_
