@@ -24,21 +24,24 @@
 #include <map>
 #include <set>
 #include <sstream>
-#include <stdio.h>
 #include <string>
-#include <iostream>
 #include <memory>
 #include <codecvt>
+#include <utility>
 
 #include "io/BufferStream.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/ProcessSessionFactory.h"
+#include "core/Resource.h"
 
 #include "utils/gsl.h"
 
 #pragma comment(lib, "wevtapi.lib")
 #pragma comment(lib, "Wecapi.lib")
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -59,13 +62,12 @@ static core::Relationship s_success("success", "Relationship for successfully co
 
 
 CollectorInitiatedSubscription::CollectorInitiatedSubscription(const std::string& name, const utils::Identifier& uuid)
-  : core::Processor(name, uuid), logger_(logging::LoggerFactory<CollectorInitiatedSubscription>::getLogger()) {
+  : core::Processor(name, uuid), logger_(core::logging::LoggerFactory<CollectorInitiatedSubscription>::getLogger()) {
   char buff[MAX_COMPUTERNAME_LENGTH + 1];
   DWORD size = sizeof(buff);
   if (GetComputerName(buff, &size)) {
     computerName_ = buff;
-  }
-  else {
+  } else {
     LOG_SUBSCRIPTION_WINDOWS_ERROR("GetComputerName");
   }
 
@@ -226,8 +228,7 @@ void CollectorInitiatedSubscription::onTrigger(const std::shared_ptr<core::Proce
 
   if (flowFileCount > 0) {
     lastActivityTimestamp_ = now;
-  }
-  else if (inactiveDurationToReconnect_.value() > 0) {
+  } else if (inactiveDurationToReconnect_.value() > 0) {
     if ((now - lastActivityTimestamp_) > inactiveDurationToReconnect_.value()) {
       logger_->log_info("Exceeds configured 'inactive duration to reconnect' %lld ms. Unsubscribe to reconnect..", inactiveDurationToReconnect_.value());
       unsubscribe();
@@ -285,8 +286,7 @@ bool CollectorInitiatedSubscription::checkSubscriptionRuntimeStatus() {
           LOG_SUBSCRIPTION_WINDOWS_ERROR("EcGetObjectArrayProperty");
           return false;
         }
-      }
-      else {
+      } else {
         LOG_SUBSCRIPTION_WINDOWS_ERROR("EcGetObjectArrayProperty");
         return false;
       }
@@ -302,8 +302,8 @@ bool CollectorInitiatedSubscription::checkSubscriptionRuntimeStatus() {
     buffer.resize(sizeof(EC_VARIANT));
     DWORD dwBufferSize{};
     if (!EcGetSubscriptionRunTimeStatus(
-          subscriptionName_.value().c_str(), 
-          statusInfoID, 
+          subscriptionName_.value().c_str(),
+          statusInfoID,
           eventSource.c_str(),
           flags,
           static_cast<DWORD>(buffer.size()),
@@ -516,7 +516,7 @@ bool CollectorInitiatedSubscription::createSubscription(const std::shared_ptr<co
   if (!getSubscriptionProperty(hSubscription, EcSubscriptionEventSources, 0, buffer, vProperty))
     return false;
 
-  // Event Sources is a collection. Ensure that we have obtained handle to the Array Property. 
+  // Event Sources is a collection. Ensure that we have obtained handle to the Array Property.
   if (vProperty->Type != EcVarTypeNull && vProperty->Type != EcVarObjectArrayPropertyHandle) {
     logInvalidSubscriptionPropertyType(__LINE__, vProperty->Type);
     return false;
@@ -627,34 +627,16 @@ void CollectorInitiatedSubscription::unsubscribe() {
 }
 
 int CollectorInitiatedSubscription::processQueue(const std::shared_ptr<core::ProcessSession> &session) {
-  struct WriteCallback: public OutputStreamCallback {
-    WriteCallback(const std::string& str)
-      : str_(str) {
-      status_ = 0;
-    }
-
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      return stream->write((uint8_t*)&str_[0], gsl::narrow<int>(str_.size()));
-    }
-
-    std::string str_;
-    int status_;
-  };
-
   int flowFileCount = 0;
 
   std::string xml;
   while (renderedXMLs_.try_dequeue(xml)) {
     auto flowFile = session->create();
 
-    {
-      WriteCallback wc{ xml };
-      session->write(flowFile, &wc);
-    }
+    session->writeBuffer(flowFile, xml);
     session->putAttribute(flowFile, core::SpecialFlowAttribute::MIME_TYPE, "application/xml");
-    session->getProvenanceReporter()->receive(flowFile, provenanceUri_, getUUIDStr(), "Consume windows event logs", 0);
+    session->getProvenanceReporter()->receive(flowFile, provenanceUri_, getUUIDStr(), "Consume windows event logs", 0ms);
     session->transfer(flowFile, s_success);
-    session->commit();
 
     flowFileCount++;
   }
@@ -699,14 +681,16 @@ void CollectorInitiatedSubscription::logWindowsError(int line, const std::string
     error,
     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
     (LPTSTR)&lpMsg,
-    0, 
+    0,
     NULL);
 
-  logger_->log_error("Line %d: '%s': error %d: %s\n", line, info.c_str(), (int)error, (char *)lpMsg);
+  logger_->log_error("Line %d: '%s': error %d: %s\n", line, info.c_str(), static_cast<int>(error), reinterpret_cast<char *>(lpMsg));
 
   LocalFree(lpMsg);
 }
 
+REGISTER_RESOURCE(CollectorInitiatedSubscription, "Windows Event Log Subscribe Callback to receive FlowFiles from Events on Windows.");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
