@@ -19,12 +19,14 @@
  */
 
 #include "ConsumeWindowsEventLog.h"
+#include <stdio.h>
 #include <vector>
+#include <tuple>
+#include <utility>
 #include <queue>
 #include <map>
 #include <set>
 #include <sstream>
-#include <stdio.h>
 #include <string>
 #include <iostream>
 #include <memory>
@@ -39,14 +41,18 @@
 #include "io/BufferStream.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
+#include "core/Resource.h"
 #include "Bookmark.h"
 #include "utils/Deleters.h"
+#include "logging/LoggerConfiguration.h"
 
 #include "utils/gsl.h"
 
 #pragma comment(lib, "wevtapi.lib")
 #pragma comment(lib, "ole32.lib")
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -127,8 +133,10 @@ core::Property ConsumeWindowsEventLog::EventHeaderDelimiter(
 core::Property ConsumeWindowsEventLog::EventHeader(
   core::PropertyBuilder::createProperty("Event Header")->
   isRequired(false)->
-  withDefaultValue("LOG_NAME=Log Name, SOURCE = Source, TIME_CREATED = Date,EVENT_RECORDID=Record ID,EVENTID = Event ID,TASK_CATEGORY = Task Category,LEVEL = Level,KEYWORDS = Keywords,USER = User,COMPUTER = Computer, EVENT_TYPE = EventType")->
-  withDescription("Comma seperated list of key/value pairs with the following keys LOG_NAME, SOURCE, TIME_CREATED,EVENT_RECORDID,EVENTID,TASK_CATEGORY,LEVEL,KEYWORDS,USER,COMPUTER, and EVENT_TYPE. Eliminating fields will remove them from the header.")->
+  withDefaultValue("LOG_NAME=Log Name, SOURCE = Source, TIME_CREATED = Date,EVENT_RECORDID=Record ID,EVENTID = Event ID,"
+      "TASK_CATEGORY = Task Category,LEVEL = Level,KEYWORDS = Keywords,USER = User,COMPUTER = Computer, EVENT_TYPE = EventType")->
+  withDescription("Comma seperated list of key/value pairs with the following keys LOG_NAME, SOURCE, TIME_CREATED,EVENT_RECORDID,"
+      "EVENTID,TASK_CATEGORY,LEVEL,KEYWORDS,USER,COMPUTER, and EVENT_TYPE. Eliminating fields will remove them from the header.")->
   build());
 
 core::Property ConsumeWindowsEventLog::OutputFormat(
@@ -172,7 +180,7 @@ core::Relationship ConsumeWindowsEventLog::Success("success", "Relationship for
 
 ConsumeWindowsEventLog::ConsumeWindowsEventLog(const std::string& name, const utils::Identifier& uuid)
   : core::Processor(name, uuid),
-    logger_(logging::LoggerFactory<ConsumeWindowsEventLog>::getLogger()) {
+    logger_(core::logging::LoggerFactory<ConsumeWindowsEventLog>::getLogger()) {
   char buff[MAX_COMPUTERNAME_LENGTH + 1];
   DWORD size = sizeof(buff);
   if (GetComputerName(buff, &size)) {
@@ -325,10 +333,11 @@ void ConsumeWindowsEventLog::onSchedule(const std::shared_ptr<core::ProcessConte
   logger_->log_trace("Successfully configured CWEL");
 }
 
-bool ConsumeWindowsEventLog::commitAndSaveBookmark(const std::wstring &bookmark_xml, const std::shared_ptr<core::ProcessSession> &session) {
+bool ConsumeWindowsEventLog::commitAndSaveBookmark(const std::wstring &bookmark_xml, const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
   {
     const TimeDiff time_diff;
     session->commit();
+    context->getStateManager()->beginTransaction();
     logger_->log_debug("processQueue commit took %" PRId64 " ms", time_diff());
   }
 
@@ -378,18 +387,18 @@ std::tuple<size_t, std::wstring> ConsumeWindowsEventLog::processEventLogs(const
 }
 
 void ConsumeWindowsEventLog::onTrigger(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session) {
-  if (!bookmark_) {
-    logger_->log_debug("bookmark_ is null");
-    context->yield();
-    return;
-  }
-
   std::unique_lock<std::mutex> lock(on_trigger_mutex_, std::try_to_lock);
   if (!lock.owns_lock()) {
     logger_->log_warn("processor was triggered before previous listing finished, configuration should be revised!");
     return;
   }
 
+  if (!bookmark_) {
+    logger_->log_debug("bookmark_ is null");
+    context->yield();
+    return;
+  }
+
   logger_->log_trace("CWEL onTrigger");
 
   size_t processed_event_count = 0;
@@ -427,14 +436,13 @@ void ConsumeWindowsEventLog::onTrigger(const std::shared_ptr<core::ProcessContex
   std::wstring bookmark_xml;
   std::tie(processed_event_count, bookmark_xml) = processEventLogs(context, session, event_query_results);
 
-  if (processed_event_count == 0 || !commitAndSaveBookmark(bookmark_xml, session)) {
+  if (processed_event_count == 0 || !commitAndSaveBookmark(bookmark_xml, context, session)) {
     context->yield();
     return;
   }
 }
 
 wel::WindowsEventLogHandler ConsumeWindowsEventLog::getEventLogHandler(const std::string & name) {
-  std::lock_guard<std::mutex> lock(cache_mutex_);
   logger_->log_trace("Getting Event Log Handler corresponding to %s", name.c_str());
   auto provider = providers_.find(name);
   if (provider != std::end(providers_)) {
@@ -463,8 +471,8 @@ void ConsumeWindowsEventLog::substituteXMLPercentageItems(pugi::xml_document& do
   }
 
   struct TreeWalker: public pugi::xml_tree_walker {
-    TreeWalker(HMODULE hMsobjsDll, std::unordered_map<std::string, std::string>& xmlPercentageItemsResolutions, std::shared_ptr<logging::Logger> logger)
-      : hMsobjsDll_(hMsobjsDll), xmlPercentageItemsResolutions_(xmlPercentageItemsResolutions), logger_(logger) {
+    TreeWalker(HMODULE hMsobjsDll, std::unordered_map<std::string, std::string>& xmlPercentageItemsResolutions, std::shared_ptr<core::logging::Logger> logger)
+      : hMsobjsDll_(hMsobjsDll), xmlPercentageItemsResolutions_(xmlPercentageItemsResolutions), logger_(std::move(logger)) {
     }
 
     bool for_each(pugi::xml_node& node) override {
@@ -491,15 +499,12 @@ void ConsumeWindowsEventLog::substituteXMLPercentageItems(pugi::xml_document& do
         const auto it = xmlPercentageItemsResolutions_.find(key);
         if (it == xmlPercentageItemsResolutions_.end()) {
           LPTSTR pBuffer{};
-          if (FormatMessage(
-            FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_HMODULE | FORMAT_MESSAGE_IGNORE_INSERTS,
-            hMsobjsDll_,
-            number,
-            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-            (LPTSTR)&pBuffer,
-            1024,
-            0
-          )) {
+          if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_HMODULE | FORMAT_MESSAGE_IGNORE_INSERTS,
+                            hMsobjsDll_,
+                            number,
+                            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+                            (LPTSTR)&pBuffer,
+                            1024, 0)) {
             value = pBuffer;
             LocalFree(pBuffer);
 
@@ -533,7 +538,7 @@ void ConsumeWindowsEventLog::substituteXMLPercentageItems(pugi::xml_document& do
    private:
     HMODULE hMsobjsDll_;
     std::unordered_map<std::string, std::string>& xmlPercentageItemsResolutions_;
-    std::shared_ptr<logging::Logger> logger_;
+    std::shared_ptr<core::logging::Logger> logger_;
   } treeWalker(hMsobjsDll_, xmlPercentageItemsResolutions_, logger_);
 
   doc.traverse(treeWalker);
@@ -656,7 +661,7 @@ void ConsumeWindowsEventLog::refreshTimeZoneData() {
   DYNAMIC_TIME_ZONE_INFORMATION tzinfo;
   auto ret = GetDynamicTimeZoneInformation(&tzinfo);
   std::wstring tzstr;
-  long tzbias = 0;
+  long tzbias = 0;  // NOLINT long comes from WINDOWS API
   bool dst = false;
   switch (ret) {
     case TIME_ZONE_ID_INVALID:
@@ -669,7 +674,7 @@ void ConsumeWindowsEventLog::refreshTimeZoneData() {
     case TIME_ZONE_ID_DAYLIGHT:
       tzstr = tzinfo.DaylightName;
       dst = true;
-      // [[fallthrough]];
+      [[fallthrough]];
     case TIME_ZONE_ID_STANDARD:
       tzstr = tzstr.empty() ? tzinfo.StandardName : tzstr;  // Use standard timezome name in case there is no daylight name or in case it's not DST
       tzbias = tzinfo.Bias + (dst ? tzinfo.DaylightBias : tzinfo.StandardBias);
@@ -688,27 +693,12 @@ void ConsumeWindowsEventLog::refreshTimeZoneData() {
 }
 
 void ConsumeWindowsEventLog::putEventRenderFlowFileToSession(const EventRender& eventRender, core::ProcessSession& session) const {
-  struct WriteCallback : public OutputStreamCallback {
-    WriteCallback(const std::string& str)
-      : str_(str) {
-    }
-
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) {
-      return stream->write(reinterpret_cast<uint8_t*>(const_cast<char*>(str_.c_str())), gsl::narrow<int>(str_.size()));
-    }
-
-    const std::string& str_;
-  };
-
   auto commitFlowFile = [&] (const std::shared_ptr<core::FlowFile>& flowFile, const std::string& content, const std::string& mimeType) {
-    {
-      WriteCallback wc{ content };
-      session.write(flowFile, &wc);
-    }
+    session.writeBuffer(flowFile, content);
     session.putAttribute(flowFile, core::SpecialFlowAttribute::MIME_TYPE, mimeType);
-    session.putAttribute(flowFile, "Timezone name", timezone_name_);
-    session.putAttribute(flowFile, "Timezone offset", timezone_offset_);
-    session.getProvenanceReporter()->receive(flowFile, provenanceUri_, getUUIDStr(), "Consume windows event logs", 0);
+    session.putAttribute(flowFile, "timezone.name", timezone_name_);
+    session.putAttribute(flowFile, "timezone.offset", timezone_offset_);
+    session.getProvenanceReporter()->receive(flowFile, provenanceUri_, getUUIDStr(), "Consume windows event logs", 0ms);
     session.transfer(flowFile, Success);
   };
 
@@ -755,11 +745,13 @@ void ConsumeWindowsEventLog::LogWindowsError(std::string error) const {
     (LPTSTR)&lpMsg,
     0, NULL);
 
-  logger_->log_error((error + " %x: %s\n").c_str(), (int)error_id, (char *)lpMsg);
+  logger_->log_error((error + " %x: %s\n").c_str(), static_cast<int>(error_id), reinterpret_cast<char *>(lpMsg));
 
   LocalFree(lpMsg);
 }
 
+REGISTER_RESOURCE(ConsumeWindowsEventLog, "Windows Event Log Subscribe Callback to receive FlowFiles from Events on Windows.");
+
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
