@@ -20,21 +20,29 @@
 
 #pragma once
 
-#include "core/Core.h"
-#include "wel/WindowsEventLog.h"
-#include "FlowFileRecord.h"
-#include "concurrentqueue.h"
-#include "core/Processor.h"
-#include "core/ProcessSession.h"
-#include "pugixml.hpp"
+#include <Windows.h>
 #include <winevt.h>
+#include <Objbase.h>
+
 #include <sstream>
 #include <regex>
 #include <codecvt>
-#include "utils/OsUtils.h"
-#include <Objbase.h>
 #include <mutex>
 #include <unordered_map>
+#include <tuple>
+#include <map>
+#include <memory>
+#include <string>
+
+#include "core/Core.h"
+#include "core/Processor.h"
+#include "core/ProcessSession.h"
+#include "utils/OsUtils.h"
+#include "wel/WindowsEventLog.h"
+#include "FlowFileRecord.h"
+#include "concurrentqueue.h"
+#include "pugixml.hpp"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -53,40 +61,40 @@ class Bookmark;
 
 //! ConsumeWindowsEventLog Class
 class ConsumeWindowsEventLog : public core::Processor {
-public:
+ public:
   //! Constructor
   /*!
   * Create a new processor
   */
-  ConsumeWindowsEventLog(const std::string& name, const utils::Identifier& uuid = {});
+  explicit ConsumeWindowsEventLog(const std::string& name, const utils::Identifier& uuid = {});
 
   //! Destructor
   virtual ~ConsumeWindowsEventLog();
 
   //! Processor Name
-  static const std::string ProcessorName;
+  EXTENSIONAPI static const std::string ProcessorName;
 
   //! Supported Properties
-  static core::Property Channel;
-  static core::Property Query;
-  static core::Property RenderFormatXML;
-  static core::Property MaxBufferSize;
-  static core::Property InactiveDurationToReconnect;
-  static core::Property IdentifierMatcher;
-  static core::Property IdentifierFunction;
-  static core::Property ResolveAsAttributes;
-  static core::Property EventHeaderDelimiter;
-  static core::Property EventHeader;
-  static core::Property OutputFormat;
-  static core::Property JSONFormat;
-  static core::Property BatchCommitSize;
-  static core::Property BookmarkRootDirectory;
-  static core::Property ProcessOldEvents;
+  EXTENSIONAPI static core::Property Channel;
+  EXTENSIONAPI static core::Property Query;
+  EXTENSIONAPI static core::Property RenderFormatXML;
+  EXTENSIONAPI static core::Property MaxBufferSize;
+  EXTENSIONAPI static core::Property InactiveDurationToReconnect;
+  EXTENSIONAPI static core::Property IdentifierMatcher;
+  EXTENSIONAPI static core::Property IdentifierFunction;
+  EXTENSIONAPI static core::Property ResolveAsAttributes;
+  EXTENSIONAPI static core::Property EventHeaderDelimiter;
+  EXTENSIONAPI static core::Property EventHeader;
+  EXTENSIONAPI static core::Property OutputFormat;
+  EXTENSIONAPI static core::Property JSONFormat;
+  EXTENSIONAPI static core::Property BatchCommitSize;
+  EXTENSIONAPI static core::Property BookmarkRootDirectory;
+  EXTENSIONAPI static core::Property ProcessOldEvents;
 
   //! Supported Relationships
-  static core::Relationship Success;
+  EXTENSIONAPI static core::Relationship Success;
 
-public:
+ public:
   /**
   * Function that's executed when the processor is scheduled.
   * @param context process context.
@@ -100,7 +108,7 @@ public:
   void initialize(void) override;
   void notifyStop() override;
 
-protected:
+ protected:
   void refreshTimeZoneData();
   void putEventRenderFlowFileToSession(const EventRender& eventRender, core::ProcessSession& session) const;
   wel::WindowsEventLogHandler getEventLogHandler(const std::string & name);
@@ -117,7 +125,7 @@ protected:
   static constexpr const char* JSONSimple = "Simple";
   static constexpr const char* JSONFlattened = "Flattened";
 
-private:
+ private:
   struct TimeDiff {
     auto operator()() const {
       return int64_t{ std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - time_).count() };
@@ -129,13 +137,18 @@ private:
     return core::annotation::Input::INPUT_FORBIDDEN;
   }
 
-  bool commitAndSaveBookmark(const std::wstring &bookmarkXml, const std::shared_ptr<core::ProcessSession> &session);
+  bool isSingleThreaded() const override {
+    return true;
+  }
+
+  bool commitAndSaveBookmark(const std::wstring &bookmarkXml, const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session);
+
   std::tuple<size_t, std::wstring> processEventLogs(const std::shared_ptr<core::ProcessContext> &context,
     const std::shared_ptr<core::ProcessSession> &session, const EVT_HANDLE& event_query_results);
 
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
-  std::shared_ptr<core::CoreComponentStateManager> state_manager_;
+  std::shared_ptr<core::logging::Logger> logger_;
+  core::CoreComponentStateManager* state_manager_;
   wel::METADATA_NAMES header_names_;
   std::string header_delimiter_;
   std::string channel_;
@@ -148,7 +161,6 @@ private:
   std::string computerName_;
   uint64_t maxBufferSize_{};
   DWORD lastActivityTimestamp_{};
-  std::mutex cache_mutex_;
   std::map<std::string, wel::WindowsEventLogHandler > providers_;
   uint64_t batch_commit_size_{};
 
@@ -175,8 +187,6 @@ private:
   std::string timezone_offset_;  // Represented as UTC offset in (+|-)HH:MM format, like +02:00
 };
 
-REGISTER_RESOURCE(ConsumeWindowsEventLog, "Windows Event Log Subscribe Callback to receive FlowFiles from Events on Windows.");
-
 }  // namespace processors
 }  // namespace minifi
 }  // namespace nifi
