@@ -17,8 +17,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#ifndef TAIL_EVENT_LOG_H_
-#define TAIL_EVENT_LOG_H_
+#pragma once
+
+#include <memory>
+#include <string>
 
 #include "core/Core.h"
 #include "FlowFileRecord.h"
@@ -36,9 +38,9 @@ const LPCWSTR pEventTypeNames[] = { L"Error", L"Warning", L"Informational", L"Au
 const char log_name[255] = "Application";
 
 class TailEventLog : public core::Processor {
-public:
-  TailEventLog(const std::string& name, const utils::Identifier& uuid = {})
-  : core::Processor(name, uuid), logger_(logging::LoggerFactory<TailEventLog>::getLogger()), max_events_(1) {
+ public:
+  explicit TailEventLog(const std::string& name, const utils::Identifier& uuid = {})
+      : core::Processor(name, uuid) {
   }
   virtual ~TailEventLog() = default;
   static const std::string ProcessorName;
@@ -50,7 +52,7 @@ public:
   // Supported Relationships
   static core::Relationship Success;
 
-public:
+ public:
   /**
    * Function that's executed when the processor is scheduled.
    * @param context process context.
@@ -63,8 +65,7 @@ public:
   // Initialize, over write by NiFi TailEventLog
   void initialize(void) override;
 
-protected:
-
+ protected:
   void notifyStop() override {
     CloseEventLog(log_handle_);
   }
@@ -120,26 +121,25 @@ protected:
       (LPTSTR)&lpMsg,
       0, NULL);
 
-    logger_->log_debug("Error %d: %s\n", (int)error_id, (char *)lpMsg);
+    logger_->log_debug("Error %d: %s\n", static_cast<int>(error_id), reinterpret_cast<char *>(lpMsg));
+
+    LocalFree(lpMsg);
   }
 
-private:
+ private:
   std::mutex log_mutex_;
   std::string log_source_;
-  uint32_t max_events_;
+  uint32_t max_events_ = 1;
   DWORD current_record_;
   DWORD num_records_;
 
   HANDLE log_handle_;
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<TailEventLog>::getLogger();
 };
-REGISTER_RESOURCE(TailEventLog, "Windows event log reader that functions as a stateful tail of the provided windows event log name");
 
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
 } /* namespace org */
-
-#endif
