@@ -17,12 +17,19 @@
 
 #pragma once
 
+#include <fstream>
+#include <iterator>
+#include <memory>
+#include <optional>
+#include <string>
+#include <utility>
+
 #include "ConsumeWindowsEventLog.h"
 #include "processors/PutFile.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/TestUtils.h"
 #include "utils/file/FileUtils.h"
-#include "utils/OptionalUtils.h"
 
 core::Relationship Success{"success", "Everything is fine"};
 
@@ -31,7 +38,7 @@ using PutFile = org::apache::nifi::minifi::processors::PutFile;
 
 class OutputFormatTestController : public TestController {
  public:
-  OutputFormatTestController(std::string channel, std::string query, std::string output_format, utils::optional<std::string> json_format = {})
+  OutputFormatTestController(std::string channel, std::string query, std::string output_format, std::optional<std::string> json_format = {})
     : channel_(std::move(channel)),
       query_(std::move(query)),
       output_format_(std::move(output_format)),
@@ -50,7 +57,7 @@ class OutputFormatTestController : public TestController {
       test_plan->setProperty(cwel_processor, ConsumeWindowsEventLog::JSONFormat.getName(), json_format_.value());
     }
 
-    auto dir = utils::createTempDir(this);
+    auto dir = createTempDirectory();
 
     auto put_file = test_plan->addProcessor("PutFile", "putFile", Success, true);
     test_plan->setProperty(put_file, PutFile::Directory.getName(), dir);
@@ -85,5 +92,5 @@ class OutputFormatTestController : public TestController {
   std::string channel_;
   std::string query_;
   std::string output_format_;
-  utils::optional<std::string> json_format_;
+  std::optional<std::string> json_format_;
 };
