@@ -21,8 +21,10 @@
 #include "processors/LogAttribute.h"
 #include "processors/PutFile.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "utils/TestUtils.h"
 #include "utils/file/FileUtils.h"
+
 #include "rapidjson/document.h"
 
 #include "CWELTestUtils.h"
@@ -66,16 +68,9 @@ TEST_CASE("ConsumeWindowsEventLog constructor works", "[create]") {
   TestController test_controller;
   std::shared_ptr<TestPlan> test_plan = test_controller.createPlan();
 
-  REQUIRE_NOTHROW(ConsumeWindowsEventLog processor_one("one"));
-
-  REQUIRE_NOTHROW(
-    utils::Identifier uuid = utils::IdGenerator::getIdGenerator()->generate();
-    ConsumeWindowsEventLog processor_two("two", uuid);
-  );  // NOLINT
-
-  REQUIRE_NOTHROW(
-    auto processor = test_plan->addProcessor("ConsumeWindowsEventLog", "cwel");
-  );  // NOLINT
+  REQUIRE_NOTHROW(ConsumeWindowsEventLog("one"));
+  REQUIRE_NOTHROW(ConsumeWindowsEventLog("two", utils::IdGenerator::getIdGenerator()->generate()));
+  REQUIRE_NOTHROW(test_plan->addProcessor("ConsumeWindowsEventLog", "cwel"));
 }
 
 TEST_CASE("ConsumeWindowsEventLog properties work with default values", "[create][properties]") {
@@ -164,8 +159,8 @@ TEST_CASE("ConsumeWindowsEventLog can consume new events", "[onTrigger]") {
     REQUIRE(LogTestController::getInstance().contains("<EventData><Data>Event one</Data></EventData>"));
 
     // make sure timezone attributes are present
-    REQUIRE(LogTestController::getInstance().contains("key:Timezone offset value:"));
-    REQUIRE(LogTestController::getInstance().contains("key:Timezone name value:"));
+    REQUIRE(LogTestController::getInstance().contains("key:timezone.offset value:"));
+    REQUIRE(LogTestController::getInstance().contains("key:timezone.name value:"));
   }
 
   SECTION("Read two events") {
@@ -389,8 +384,8 @@ TEST_CASE("ConsumeWindowsEventLog prints events in JSON::Raw correctly", "[onTri
             "children": [{
               "name": "Data",
               "text": "Event one"
-            }] 
-          }      
+            }]
+          }
         ]
       }
     ]
