@@ -24,6 +24,7 @@
 #include <set>
 
 #include "TestBase.h"
+#include "Catch.h"
 #include "core/Core.h"
 #include "wel/MetadataWalker.h"
 #include "wel/XMLString.h"
@@ -55,13 +56,13 @@ std::string readFile(const std::string &file_name) {
 }
 
 const std::string METADATA_WALKER_TESTS_LOG_NAME = "MetadataWalkerTests";
-const short event_type_index = 178;
+const short event_type_index = 178;  // NOLINT short comes from WINDOWS API
 
 class FakeWindowsEventLogMetadata : public WindowsEventLogMetadata {
  public:
   std::string getEventData(EVT_FORMAT_MESSAGE_FLAGS flags) const override { return "event_data_for_flag_" + std::to_string(flags); }
   std::string getEventTimestamp() const override { return "event_timestamp"; }
-  short getEventTypeIndex() const override { return event_type_index; }
+  short getEventTypeIndex() const override { return event_type_index; }  // NOLINT short comes from WINDOWS API
 };
 
 }  // namespace
@@ -152,12 +153,12 @@ TEST_CASE("MetadataWalker extracts mappings correctly when there is a single Sid
 
   const std::map<std::string, std::string> expected_identifiers{{"S-1-0-0", "S-1-0-0"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {EVENT_RECORDID, "2575952"}};
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::EVENT_RECORDID, "2575952"}};
 
   const std::map<std::string, std::string> expected_field_values{};
 
@@ -175,18 +176,18 @@ TEST_CASE("MetadataWalker extracts mappings correctly when there is a single Sid
 
   const std::map<std::string, std::string> expected_identifiers{{"S-1-0-0", "Nobody"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {LOG_NAME, "MetadataWalkerTests"},
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {OPCODE, "event_data_for_flag_4"},
-      {EVENT_RECORDID, "2575952"},
-      {EVENT_TYPE, "178"},
-      {TASK_CATEGORY, "event_data_for_flag_3"},
-      {LEVEL, "event_data_for_flag_2"},
-      {KEYWORDS, "event_data_for_flag_5"}};
+      {METADATA::LOG_NAME, "MetadataWalkerTests"},
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::OPCODE, "event_data_for_flag_4"},
+      {METADATA::EVENT_RECORDID, "2575952"},
+      {METADATA::EVENT_TYPE, "178"},
+      {METADATA::TASK_CATEGORY, "event_data_for_flag_3"},
+      {METADATA::LEVEL, "event_data_for_flag_2"},
+      {METADATA::KEYWORDS, "event_data_for_flag_5"}};
 
   SECTION("update_xml is false => fields are collected into walker.getFieldValues()") {
     const std::map<std::string, std::string> expected_field_values{
@@ -212,12 +213,12 @@ TEST_CASE("MetadataWalker extracts mappings correctly when there are multiple Si
 
   const std::map<std::string, std::string> expected_identifiers{{"S-1-0-0", "S-1-0-0"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {EVENT_RECORDID, "2575952"}};
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::EVENT_RECORDID, "2575952"}};
 
   const std::map<std::string, std::string> expected_field_values{};
 
@@ -241,18 +242,18 @@ TEST_CASE("MetadataWalker extracts mappings correctly when there are multiple Si
       {"S-1-0-0", "Nobody"},
       {"S-1-1-0", "Everyone"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {LOG_NAME, "MetadataWalkerTests"},
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {OPCODE, "event_data_for_flag_4"},
-      {EVENT_RECORDID, "2575952"},
-      {EVENT_TYPE, "178"},
-      {TASK_CATEGORY, "event_data_for_flag_3"},
-      {LEVEL, "event_data_for_flag_2"},
-      {KEYWORDS, "event_data_for_flag_5"}};
+      {METADATA::LOG_NAME, "MetadataWalkerTests"},
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::OPCODE, "event_data_for_flag_4"},
+      {METADATA::EVENT_RECORDID, "2575952"},
+      {METADATA::EVENT_TYPE, "178"},
+      {METADATA::TASK_CATEGORY, "event_data_for_flag_3"},
+      {METADATA::LEVEL, "event_data_for_flag_2"},
+      {METADATA::KEYWORDS, "event_data_for_flag_5"}};
 
   SECTION("update_xml is false => fields are collected into walker.getFieldValues()") {
     const std::map<std::string, std::string> expected_field_values{
@@ -278,12 +279,12 @@ TEST_CASE("MetadataWalker extracts mappings correctly when the Sid is unknown an
 
   const std::map<std::string, std::string> expected_identifiers{{"S-1-8-6-5-3-0-9", "S-1-8-6-5-3-0-9"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {EVENT_RECORDID, "2575952"}};
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::EVENT_RECORDID, "2575952"}};
 
   const std::map<std::string, std::string> expected_field_values{};
 
@@ -301,18 +302,18 @@ TEST_CASE("MetadataWalker extracts mappings correctly when the Sid is unknown an
 
   const std::map<std::string, std::string> expected_identifiers{{"S-1-8-6-5-3-0-9", "S-1-8-6-5-3-0-9"}};
 
-  using namespace org::apache::nifi::minifi::wel;
+  using org::apache::nifi::minifi::wel::METADATA;
   const std::map<METADATA, std::string> expected_metadata{
-      {LOG_NAME, "MetadataWalkerTests"},
-      {SOURCE, "Microsoft-Windows-Security-Auditing"},
-      {TIME_CREATED, "event_timestamp"},
-      {EVENTID, "4672"},
-      {OPCODE, "event_data_for_flag_4"},
-      {EVENT_RECORDID, "2575952"},
-      {EVENT_TYPE, "178"},
-      {TASK_CATEGORY, "event_data_for_flag_3"},
-      {LEVEL, "event_data_for_flag_2"},
-      {KEYWORDS, "event_data_for_flag_5"}};
+      {METADATA::LOG_NAME, "MetadataWalkerTests"},
+      {METADATA::SOURCE, "Microsoft-Windows-Security-Auditing"},
+      {METADATA::TIME_CREATED, "event_timestamp"},
+      {METADATA::EVENTID, "4672"},
+      {METADATA::OPCODE, "event_data_for_flag_4"},
+      {METADATA::EVENT_RECORDID, "2575952"},
+      {METADATA::EVENT_TYPE, "178"},
+      {METADATA::TASK_CATEGORY, "event_data_for_flag_3"},
+      {METADATA::LEVEL, "event_data_for_flag_2"},
+      {METADATA::KEYWORDS, "event_data_for_flag_5"}};
 
   SECTION("update_xml is false => fields are collected into walker.getFieldValues()") {
     const std::map<std::string, std::string> expected_field_values{
