@@ -40,14 +40,9 @@ namespace org {
 namespace apache {
 namespace nifi {
 namespace minifi {
-// Connection Class
 
-class Connection : public core::Connectable, public std::enable_shared_from_this<Connection> {
+class Connection : public core::Connectable {
  public:
-  // Constructor
-  /*
-   * Create a new processor
-   */
   explicit Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name);
   explicit Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name, const utils::Identifier &uuid);
   explicit Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name, const utils::Identifier &uuid,
@@ -55,7 +50,7 @@ class Connection : public core::Connectable, public std::enable_shared_from_this
   explicit Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name, const utils::Identifier &uuid,
                       const utils::Identifier &srcUUID, const utils::Identifier &destUUID);
   // Destructor
-  virtual ~Connection() = default;
+  ~Connection() override = default;
 
   // Set Source Processor UUID
   void setSourceUUID(const utils::Identifier &uuid) {
@@ -75,19 +70,19 @@ class Connection : public core::Connectable, public std::enable_shared_from_this
   }
 
   // Set Connection Source Processor
-  void setSource(std::shared_ptr<core::Connectable> source) {
+  void setSource(core::Connectable* source) {
     source_connectable_ = source;
   }
   // ! Get Connection Source Processor
-  std::shared_ptr<core::Connectable> getSource() {
+  core::Connectable* getSource() const {
     return source_connectable_;
   }
   // Set Connection Destination Processor
-  void setDestination(std::shared_ptr<core::Connectable> dest) {
+  void setDestination(core::Connectable* dest) {
     dest_connectable_ = dest;
   }
   // ! Get Connection Destination Processor
-  std::shared_ptr<core::Connectable> getDestination() {
+  core::Connectable* getDestination() {
     return dest_connectable_;
   }
 
@@ -124,11 +119,11 @@ class Connection : public core::Connectable, public std::enable_shared_from_this
     return max_data_queue_size_;
   }
   // Set Flow expiration duration in millisecond
-  void setFlowExpirationDuration(uint64_t duration) {
+  void setFlowExpirationDuration(std::chrono::milliseconds duration) {
     expired_duration_ = duration;
   }
   // Get Flow expiration duration in millisecond
-  uint64_t getFlowExpirationDuration() {
+  std::chrono::milliseconds getFlowExpirationDuration() {
     return expired_duration_;
   }
 
@@ -143,7 +138,7 @@ class Connection : public core::Connectable, public std::enable_shared_from_this
   // Check whether the queue is empty
   bool isEmpty() const;
   // Check whether the queue is full to apply back pressure
-  bool isFull();
+  bool isFull() const;
   // Get queue size
   uint64_t getQueueSize() {
     std::lock_guard<std::mutex> lock(mutex_);
@@ -183,31 +178,31 @@ class Connection : public core::Connectable, public std::enable_shared_from_this
   // Relationship for this connection
   std::set<core::Relationship> relationships_;
   // Source Processor (ProcessNode/Port)
-  std::shared_ptr<core::Connectable> source_connectable_;
+  core::Connectable* source_connectable_ = nullptr;
   // Destination Processor (ProcessNode/Port)
-  std::shared_ptr<core::Connectable> dest_connectable_;
+  core::Connectable* dest_connectable_ = nullptr;
   // Max queue size to apply back pressure
-  std::atomic<uint64_t> max_queue_size_;
+  std::atomic<uint64_t> max_queue_size_ = 0;
   // Max queue data size to apply back pressure
-  std::atomic<uint64_t> max_data_queue_size_;
+  std::atomic<uint64_t> max_data_queue_size_ = 0;
   // Flow File Expiration Duration in= MilliSeconds
-  std::atomic<uint64_t> expired_duration_;
+  std::atomic<std::chrono::milliseconds> expired_duration_ = std::chrono::milliseconds(0);
   // flow file repository
   std::shared_ptr<core::Repository> flow_repository_;
   // content repository reference.
   std::shared_ptr<core::ContentRepository> content_repo_;
 
  private:
-  bool drop_empty_;
+  bool drop_empty_ = false;
   // Mutex for protection
   mutable std::mutex mutex_;
   // Queued data size
-  std::atomic<uint64_t> queued_data_size_;
+  std::atomic<uint64_t> queued_data_size_ = 0;
   // Queue for the Flow File
   utils::FlowFileQueue queue_;
   // flow repository
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<Connection>::getLogger();
   // Prevent default copy constructor and assignment operation
   // Only support pass by reference or pointer
   Connection(const Connection &parent);
