@@ -22,6 +22,7 @@
 
 #include <algorithm>
 #include <atomic>
+#include <functional>
 #include <map>
 #include <memory>
 #include <mutex>
@@ -57,10 +58,11 @@
 #include "utils/Id.h"
 #include "utils/file/FileSystem.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
+namespace org::apache::nifi::minifi {
+
+namespace state {
+class ProcessorController;
+}  // namespace state
 
 // Default NiFi Root Group Name
 #define DEFAULT_ROOT_GROUP_NAME ""
@@ -69,18 +71,18 @@ namespace minifi {
  * Flow Controller class. Generally used by FlowController factory
  * as a singleton.
  */
-class FlowController : public core::controller::ForwardingControllerServiceProvider,  public state::StateMonitor, public c2::C2Client, public std::enable_shared_from_this<FlowController> {
+class FlowController : public core::controller::ForwardingControllerServiceProvider,  public state::StateMonitor, public c2::C2Client {
  public:
   FlowController(std::shared_ptr<core::Repository> provenance_repo, std::shared_ptr<core::Repository> flow_file_repo,
                  std::shared_ptr<Configure> configure, std::unique_ptr<core::FlowConfiguration> flow_configuration,
-                 std::shared_ptr<core::ContentRepository> content_repo, std::string name = DEFAULT_ROOT_GROUP_NAME,
-                 bool headless_mode = false, std::shared_ptr<utils::file::FileSystem> filesystem = std::make_shared<utils::file::FileSystem>());
+                 std::shared_ptr<core::ContentRepository> content_repo, const std::string& name = DEFAULT_ROOT_GROUP_NAME,
+                 std::shared_ptr<utils::file::FileSystem> filesystem = std::make_shared<utils::file::FileSystem>(),
+                 std::function<void()> request_restart = []{});
 
   FlowController(std::shared_ptr<core::Repository> provenance_repo, std::shared_ptr<core::Repository> flow_file_repo,
                  std::shared_ptr<Configure> configure, std::unique_ptr<core::FlowConfiguration> flow_configuration,
-                 std::shared_ptr<core::ContentRepository> content_repo, std::shared_ptr<utils::file::FileSystem> filesystem)
-      : FlowController(std::move(provenance_repo), std::move(flow_file_repo), std::move(configure), std::move(flow_configuration),
-                       std::move(content_repo), DEFAULT_ROOT_GROUP_NAME, false, std::move(filesystem)) {}
+                 std::shared_ptr<core::ContentRepository> content_repo, std::shared_ptr<utils::file::FileSystem> filesystem,
+                 std::function<void()> request_restart = []{});
 
   ~FlowController() override;
 
@@ -89,13 +91,8 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
     return this->provenance_repo_;
   }
 
-  // Get the flowfile repository
-  virtual std::shared_ptr<core::Repository> getFlowFileRepository() {
-    return this->flow_file_repo_;
-  }
-
   // Load flow xml from disk, after that, create the root process group and its children, initialize the flows
-  virtual void load(const std::shared_ptr<core::ProcessGroup> &root = nullptr, bool reload = false);
+  virtual void load(std::unique_ptr<core::ProcessGroup> root = nullptr, bool reload = false);
 
   // Whether the Flow Controller is start running
   bool isRunning() override {
@@ -117,9 +114,8 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
     return -1;
   }
 
-  std::vector<std::shared_ptr<state::StateController>> getComponents(const std::string &name) override;
-
-  std::vector<std::shared_ptr<state::StateController>> getAllComponents() override;
+  void executeOnComponent(const std::string &name, std::function<void(state::StateController&)> func) override;
+  void executeOnAllComponents(std::function<void(state::StateController&)> func) override;
 
   int16_t clearConnection(const std::string &connection) override;
 
@@ -185,12 +181,14 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
    * Retrieves the agent manifest to be sent as a response to C2 DESCRIBE manifest
    * @return the agent manifest response node
    */
-  std::shared_ptr<state::response::ResponseNode> getAgentManifest() const override;
+  std::shared_ptr<state::response::ResponseNode> getAgentManifest() override;
 
   uint64_t getUptime() override;
 
   std::vector<BackTrace> getTraces() override;
 
+  std::map<std::string, std::unique_ptr<io::InputStream>> getDebugInfo() override;
+
  private:
   /**
    * Loads the flow as specified in the flow config file or if not present
@@ -202,9 +200,8 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
  protected:
   // function to load the flow file repo.
   void loadFlowRepo();
-  void initializeExternalComponents();
 
-  utils::optional<std::chrono::milliseconds> loadShutdownTimeoutFromConfiguration();
+  std::optional<std::chrono::milliseconds> loadShutdownTimeoutFromConfiguration();
 
  private:
   template <typename T, typename = typename std::enable_if<std::is_base_of<SchedulingAgent, T>::value>::type>
@@ -224,8 +221,6 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
 
   // Whether it has already been initialized (load the flow XML already)
   std::atomic<bool> initialized_;
-  // Thread pool for schedulers
-  utils::ThreadPool<utils::TaskRescheduleInfo> thread_pool_;
   // Flow Timer Scheduler
   std::shared_ptr<TimerDrivenSchedulingAgent> timer_scheduler_;
   // Flow Event Scheduler
@@ -238,14 +233,27 @@ class FlowController : public core::controller::ForwardingControllerServiceProvi
   std::chrono::steady_clock::time_point start_time_;
 
  private:
+  std::vector<state::StateController*> getAllComponents();
+
+  state::StateController* getComponent(const std::string &name);
+
+  state::StateController* getProcessorController(const std::string& name,
+                                                 const std::function<std::unique_ptr<state::ProcessorController>(core::Processor&)>& controllerFactory);
+
+  std::vector<state::StateController*> getAllProcessorControllers(
+          const std::function<std::unique_ptr<state::ProcessorController>(core::Processor&)>& controllerFactory);
+
+  std::unique_ptr<state::ProcessorController> createController(core::Processor& processor);
+
   std::chrono::milliseconds shutdown_check_interval_{1000};
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<FlowController>::getLogger();
   std::string serial_number_;
+
+  // Thread pool for schedulers
+  utils::ThreadPool<utils::TaskRescheduleInfo> thread_pool_;
+  std::map<utils::Identifier, std::unique_ptr<state::ProcessorController>> processor_to_controller_;
 };
 
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+}  // namespace org::apache::nifi::minifi
 
 #endif  // LIBMINIFI_INCLUDE_FLOWCONTROLLER_H_
