@@ -35,6 +35,7 @@
 #include "io/StreamFactory.h"
 #include "controllers/SSLContextService.h"
 #include "core/logging/LoggerConfiguration.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -87,10 +88,9 @@ class RemoteProcessorGroupPort : public core::Processor {
         direction_(sitetosite::SEND),
         transmitting_(false),
         timeout_(0),
-        http_enabled_(false),
         bypass_rest_api_(false),
         ssl_service(nullptr),
-        logger_(logging::LoggerFactory<RemoteProcessorGroupPort>::getLogger()) {
+        logger_(core::logging::LoggerFactory<RemoteProcessorGroupPort>::getLogger()) {
     client_type_ = sitetosite::CLIENT_TYPE::RAW;
     stream_factory_ = stream_factory;
     protocol_uuid_ = uuid;
@@ -103,15 +103,15 @@ class RemoteProcessorGroupPort : public core::Processor {
   virtual ~RemoteProcessorGroupPort() = default;
 
   // Processor Name
-  static const char *ProcessorName;
+  MINIFIAPI static const char *ProcessorName;
   // Supported Properties
-  static core::Property hostName;
-  static core::Property SSLContext;
-  static core::Property port;
-  static core::Property portUUID;
-  static core::Property idleTimeout;
+  MINIFIAPI static core::Property hostName;
+  MINIFIAPI static core::Property SSLContext;
+  MINIFIAPI static core::Property port;
+  MINIFIAPI static core::Property portUUID;
+  MINIFIAPI static core::Property idleTimeout;
   // Supported Relationships
-  static core::Relationship relation;
+  MINIFIAPI static core::Relationship relation;
 
  public:
   virtual void onSchedule(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory);
@@ -127,7 +127,7 @@ class RemoteProcessorGroupPort : public core::Processor {
       this->setTriggerWhenEmpty(true);
   }
   // Set Timeout
-  void setTimeOut(uint64_t timeout) {
+  void setTimeout(uint64_t timeout) {
     timeout_ = timeout;
   }
   // SetTransmitting
@@ -207,12 +207,11 @@ class RemoteProcessorGroupPort : public core::Processor {
 
   utils::Identifier protocol_uuid_;
 
-  std::chrono::milliseconds idle_timeout_{15000};
+  std::chrono::milliseconds idle_timeout_ = std::chrono::seconds(15);
 
   // rest API end point info
   std::vector<struct RPG> nifi_instances_;
 
-  bool http_enabled_;
   // http proxy
   utils::HTTPProxy proxy_;
 
@@ -232,7 +231,7 @@ class RemoteProcessorGroupPort : public core::Processor {
 
  private:
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_;
   static const char* RPG_SSL_CONTEXT_SERVICE_NAME;
 };
 
