@@ -23,6 +23,8 @@
 #include <string>
 #include <utility>
 #include <vector>
+#include "core/controller/ControllerService.h"
+#include "core/ClassLoader.h"
 #include "core/expect.h"
 #include "core/Property.h"
 #include "core/Relationship.h"
@@ -30,10 +32,7 @@
 #include "core/Annotation.h"
 #include "io/validation.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
+namespace org::apache::nifi::minifi {
 
 class ClassDescription {
  public:
@@ -60,6 +59,7 @@ class ClassDescription {
   std::vector<core::Relationship> class_relationships_;
   bool dynamic_properties_ = false;
   std::string inputRequirement_;
+  bool isSingleThreaded_ = false;
   bool dynamic_relationships_ = false;
   bool is_controller_service_ = false;
 };
@@ -68,6 +68,10 @@ struct Components {
   std::vector<ClassDescription> processors_;
   std::vector<ClassDescription> controller_services_;
   std::vector<ClassDescription> other_components_;
+
+  [[nodiscard]] bool empty() const noexcept {
+    return processors_.empty() && controller_services_.empty() && other_components_.empty();
+  }
 };
 
 struct BundleDetails {
@@ -118,14 +122,9 @@ class ExternalBuildDescription {
 
 class BuildDescription {
  public:
-  static struct Components getClassDescriptions(const std::string group = "minifi-system") {
-    static std::map<std::string, struct Components> class_mappings;
-#ifndef WIN32
-    if (UNLIKELY(IsNullOrEmpty(class_mappings[group].processors_) && IsNullOrEmpty(class_mappings[group].processors_))) {
-#else
-      if (class_mappings[group].processors_.empty()) {
-#endif
-      for (auto clazz : core::ClassLoader::getDefaultClassLoader().getClasses(group)) {
+  struct Components getClassDescriptions(const std::string& group = "minifi-system") {
+    if (class_mappings_[group].empty()) {
+      for (const auto& clazz : core::ClassLoader::getDefaultClassLoader().getClasses(group)) {
         std::string class_name = clazz;
         auto lastOfIdx = clazz.find_last_of("::");
         if (lastOfIdx != std::string::npos) {
@@ -135,15 +134,17 @@ class BuildDescription {
         }
         auto obj = core::ClassLoader::getDefaultClassLoader().instantiate(class_name, class_name);
 
-        std::shared_ptr<core::ConfigurableComponent> component = std::dynamic_pointer_cast<core::ConfigurableComponent>(obj);
+        std::unique_ptr<core::ConfigurableComponent> component{
+          utils::dynamic_unique_cast<core::ConfigurableComponent>(std::move(obj))
+        };
 
         std::string classDescriptionName = clazz;
         utils::StringUtils::replaceAll(classDescriptionName, "::", ".");
         ClassDescription description(classDescriptionName);
-        if (nullptr != component) {
-          auto processor = std::dynamic_pointer_cast<core::Processor>(obj);
-          bool is_processor = processor != nullptr;
-          bool is_controller_service = LIKELY(is_processor == true) ? false : std::dynamic_pointer_cast<core::controller::ControllerService>(obj) != nullptr;
+        if (component) {
+          auto processor = dynamic_cast<core::Processor*>(component.get());
+          const bool is_processor = processor != nullptr;
+          const bool is_controller_service = LIKELY(is_processor == true) ? false : dynamic_cast<core::controller::ControllerService*>(component.get()) != nullptr;
 
           component->initialize();
           description.class_properties_ = component->getProperties();
@@ -151,23 +152,24 @@ class BuildDescription {
           description.dynamic_relationships_ = component->supportsDynamicRelationships();
           if (is_processor) {
             description.inputRequirement_ = processor->getInputRequirementAsString();
+            description.isSingleThreaded_ = processor->isSingleThreaded();
             description.class_relationships_ = processor->getSupportedRelationships();
-            class_mappings[group].processors_.emplace_back(description);
+            class_mappings_[group].processors_.emplace_back(description);
           } else if (is_controller_service) {
-            class_mappings[group].controller_services_.emplace_back(description);
+            class_mappings_[group].controller_services_.emplace_back(description);
           } else {
-            class_mappings[group].other_components_.emplace_back(description);
+            class_mappings_[group].other_components_.emplace_back(description);
           }
         }
       }
     }
-    return class_mappings[group];
+    return class_mappings_[group];
   }
-}; // NOLINT
 
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+ private:
+  std::map<std::string, struct Components> class_mappings_;
+};
+
+}  // namespace org::apache::nifi::minifi
 
 #endif  // LIBMINIFI_INCLUDE_AGENT_BUILD_DESCRIPTION_H_
