@@ -18,15 +18,16 @@
 
 #pragma once
 
-#include <map>
-#include <string>
-#include <vector>
-#include <utility>
 #include <functional>
 #include <future>
+#include <map>
 #include <memory>
 #include <mutex>
+#include <optional>
+#include <string>
 #include <thread>
+#include <utility>
+#include <vector>
 
 #include "../core/state/nodes/MetricsBase.h"
 #include "../core/state/Value.h"
@@ -66,10 +67,12 @@ class C2Agent : public state::UpdateController {
 
   C2Agent(core::controller::ControllerServiceProvider *controller,
           state::Pausable *pause_handler,
-          const std::shared_ptr<state::StateMonitor> &updateSink,
-          const std::shared_ptr<Configure> &configure,
-          const std::shared_ptr<utils::file::FileSystem> &filesystem = std::make_shared<utils::file::FileSystem>());
-  virtual ~C2Agent() noexcept {
+          state::StateMonitor* updateSink,
+          std::shared_ptr<Configure> configure,
+          std::shared_ptr<utils::file::FileSystem> filesystem,
+          std::function<void()> request_restart);
+
+  ~C2Agent() noexcept override {
     delete protocol_.load();
   }
 
@@ -83,16 +86,14 @@ class C2Agent : public state::UpdateController {
    */
   void performHeartBeat();
 
-  int64_t getHeartBeatDelay() {
+  std::chrono::milliseconds getHeartBeatDelay() {
     std::lock_guard<std::mutex> lock(heartbeat_mutex);
     return heart_beat_period_;
   }
 
-  utils::optional<std::string> fetchFlow(const std::string& uri) const;
+  std::optional<std::string> fetchFlow(const std::string& uri) const;
 
  protected:
-  void restart_agent();
-
   /**
    * Check the collection of triggers for any updates that need to be handled.
    * This is an optional step
@@ -118,9 +119,7 @@ class C2Agent : public state::UpdateController {
   /**
    * Enqueues a C2 server response for us to evaluate and parse.
    */
-  void enqueue_c2_server_response(C2Payload &&resp) {
-    responses.enqueue(std::move(resp));
-  }
+  void enqueue_c2_server_response(C2Payload &&resp);
 
   /**
    * Enqueues a c2 payload for a response to the C2 server.
@@ -135,6 +134,8 @@ class C2Agent : public state::UpdateController {
    */
   virtual void handle_c2_server_response(const C2ContentResponse &resp);
 
+  void handle_clear(const C2ContentResponse &resp);
+
   /**
    * Handles an update request
    * @param C2ContentResponse response
@@ -146,10 +147,20 @@ class C2Agent : public state::UpdateController {
    */
   void handle_describe(const C2ContentResponse &resp);
 
+
+  enum class UpdateResult {
+    NO_UPDATE,
+    UPDATE_SUCCESSFUL,
+    UPDATE_FAILED
+  };
   /**
    * Updates a property
    */
-  bool update_property(const std::string &property_name, const std::string &property_value,  bool persist);
+  UpdateResult update_property(const std::string &property_name, const std::string &property_value,  PropertyChangeLifetime lifetime);
+
+  void handle_transfer(const C2ContentResponse &resp);
+
+  C2Payload bundleDebugInfo(std::map<std::string, std::unique_ptr<io::InputStream>>& files);
 
   /**
    * Creates configuration options C2 payload for response
@@ -161,6 +172,11 @@ class C2Agent : public state::UpdateController {
   utils::TaskRescheduleInfo consume();
 
   bool handleConfigurationUpdate(const C2ContentResponse &resp);
+  void handlePropertyUpdate(const C2ContentResponse &resp);
+
+  std::optional<std::string> resolveFlowUrl(const std::string& url) const;
+
+  std::optional<std::string> resolveUrl(const std::string& url) const;
 
  protected:
   std::timed_mutex metrics_mutex_;
@@ -183,7 +199,7 @@ class C2Agent : public state::UpdateController {
   utils::ConcurrentQueue<C2Payload> requests;
 
   // heart beat period.
-  int64_t heart_beat_period_;
+  std::chrono::milliseconds heart_beat_period_;
 
   // maximum number of queued messages to send to the c2 server
   size_t max_c2_responses;
@@ -195,7 +211,7 @@ class C2Agent : public state::UpdateController {
   std::function<utils::TaskRescheduleInfo()> c2_producer_;
 
   // reference to the update sink, against which we will execute updates.
-  std::shared_ptr<state::StateMonitor> update_sink_;
+  state::StateMonitor* update_sink_;
 
   // functions that will be used for the udpate controller.
   std::vector<std::function<utils::TaskRescheduleInfo()>> functions_;
@@ -212,18 +228,15 @@ class C2Agent : public state::UpdateController {
 
   std::shared_ptr<utils::file::FileSystem> filesystem_;
 
-  // shared pointer to the running c2 configuration.
-  std::shared_ptr<Configure> running_c2_configuration;
-
   std::mutex heartbeat_mutex;
 
-  std::vector<std::shared_ptr<HeartbeatReporter>> heartbeat_protocols_;
+  std::vector<std::unique_ptr<HeartbeatReporter>> heartbeat_protocols_;
 
-  std::vector<std::shared_ptr<C2Trigger>> triggers_;
+  std::vector<std::unique_ptr<C2Trigger>> triggers_;
 
   std::atomic<C2Protocol*> protocol_;
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<C2Agent>::getLogger();
 
   utils::ThreadPool<utils::TaskRescheduleInfo> thread_pool_;
 
@@ -232,6 +245,9 @@ class C2Agent : public state::UpdateController {
   bool manifest_sent_;
 
   const uint64_t C2RESPONSE_POLL_MS = 100;
+
+  std::atomic<bool> restart_needed_ = false;
+  std::function<void()> request_restart_;
 };
 
 }  // namespace c2
