@@ -28,6 +28,7 @@
 #include "../core/state/Value.h"
 #include "core/state/UpdateController.h"
 #include "utils/Enum.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -43,13 +44,35 @@ SMART_ENUM(Operation,
   (DESCRIBE, "describe"),
   (HEARTBEAT, "heartbeat"),
   (UPDATE, "update"),
-  (VALIDATE, "validate"),
   (CLEAR, "clear"),
   (TRANSFER, "transfer"),
   (PAUSE, "pause"),
   (RESUME, "resume")
 )
 
+SMART_ENUM(DescribeOperand,
+  (METRICS, "metrics"),
+  (CONFIGURATION, "configuration"),
+  (MANIFEST, "manifest"),
+  (JSTACK, "jstack"),
+  (CORECOMPONENTSTATE, "corecomponentstate")
+)
+
+SMART_ENUM(UpdateOperand,
+  (CONFIGURATION, "configuration"),
+  (PROPERTIES, "properties")
+)
+
+SMART_ENUM(TransferOperand,
+  (DEBUG, "debug")
+)
+
+SMART_ENUM(ClearOperand,
+  (CONNECTION, "connection"),
+  (REPOSITORIES, "repositories"),
+  (CORECOMPONENTSTATE, "corecomponentstate")
+)
+
 #define PAYLOAD_NO_STATUS 0
 #define PAYLOAD_SUCCESS 1
 #define PAYLOAD_FAILURE 2
@@ -63,7 +86,7 @@ struct AnnotatedValue : state::response::ValueNode {
   using state::response::ValueNode::ValueNode;
   using state::response::ValueNode::operator=;
 
-  utils::optional<std::reference_wrapper<const AnnotatedValue>> getAnnotation(const std::string& name) const {
+  [[nodiscard]] std::optional<std::reference_wrapper<const AnnotatedValue>> getAnnotation(const std::string& name) const {
     auto it = annotations.find(name);
     if (it == annotations.end()) {
       return {};
@@ -71,6 +94,8 @@ struct AnnotatedValue : state::response::ValueNode {
     return std::cref(it->second);
   }
 
+  friend std::ostream& operator<<(std::ostream& out, const AnnotatedValue& val);
+
   std::map<std::string, AnnotatedValue> annotations;
 };
 
@@ -91,6 +116,8 @@ struct C2ContentResponse {
 
   bool operator!=(const C2ContentResponse &rhs) const { return !(*this == rhs); }
 
+  friend std::ostream& operator<<(std::ostream& out, const C2ContentResponse& response);
+
   Operation op;
   // determines if the operation is required
   bool required{ false };
@@ -128,14 +155,17 @@ class C2Payload : public state::Update {
   ~C2Payload() override = default;
 
   void setIdentifier(std::string ident) { ident_ = std::move(ident); }
+  [[nodiscard]]
   std::string getIdentifier() const { return ident_; }
 
   void setLabel(std::string label) { label_ = std::move(label); }
+  [[nodiscard]]
   std::string getLabel() const { return label_; }
 
   /**
    * Gets the operation for this payload. May be nested or a single operation.
    */
+  [[nodiscard]]
   Operation getOperation() const noexcept { return op_; }
 
   /**
@@ -143,6 +173,7 @@ class C2Payload : public state::Update {
    */
   bool validate() override { return true; }
 
+  [[nodiscard]]
   const std::vector<C2ContentResponse> &getContent() const noexcept { return content_; }
 
   /**
@@ -153,6 +184,7 @@ class C2Payload : public state::Update {
   /**
    * Determines if this object contains raw data.
    */
+  [[nodiscard]]
   bool isRaw() const noexcept { return raw_; }
 
   /**
@@ -160,12 +192,13 @@ class C2Payload : public state::Update {
    */
   void setRawData(const std::string&);
   void setRawData(const std::vector<char>&);
-  void setRawData(const std::vector<uint8_t>&);
+  void setRawData(gsl::span<const std::byte>);
 
   /**
    * Returns raw data.
    */
-  std::vector<char> getRawData() const { return raw_data_; }
+  [[nodiscard]] std::vector<std::byte> getRawData() const noexcept { return raw_data_; }
+  [[nodiscard]] std::string getRawDataAsString() const { return utils::span_to<std::string>(gsl::make_span(getRawData()).as_span<const char>()); }
 
   /**
    * Add a nested payload.
@@ -173,12 +206,15 @@ class C2Payload : public state::Update {
    */
   void addPayload(C2Payload &&payload);
 
+  [[nodiscard]]
   bool isCollapsible() const noexcept { return is_collapsible_; }
   void setCollapsible(bool is_collapsible) noexcept { is_collapsible_ = is_collapsible; }
 
+  [[nodiscard]]
   bool isContainer() const noexcept { return is_container_; }
   void setContainer(bool is_container) noexcept { is_container_ = is_container; }
 
+  [[nodiscard]]
   const std::vector<C2Payload> &getNestedPayloads() const & noexcept { return payloads_; }
 
   std::vector<C2Payload>&& getNestedPayloads() && noexcept {return std::move(payloads_);}
@@ -192,6 +228,14 @@ class C2Payload : public state::Update {
 
   bool operator!=(const C2Payload &rhs) const { return !(*this == rhs); }
 
+  friend std::ostream& operator<<(std::ostream& out, const C2Payload& payload);
+
+  [[nodiscard]] std::string str() const {
+    std::stringstream ss;
+    ss << *this;
+    return std::move(ss).str();
+  }
+
  protected:
   std::string ident_;  // identifier for this payload.
   std::string label_;
@@ -199,7 +243,7 @@ class C2Payload : public state::Update {
   std::vector<C2ContentResponse> content_;
   Operation op_;
   bool raw_{ false };
-  std::vector<char> raw_data_;
+  std::vector<std::byte> raw_data_;
   bool is_container_{ false };
   bool is_collapsible_{ true };
 };
