@@ -23,7 +23,6 @@
 #include <string>
 #include <vector>
 
-#include "core/Resource.h"
 #include "HeartbeatReporter.h"
 #include "io/StreamFactory.h"
 #include "core/logging/LoggerConfiguration.h"
@@ -41,8 +40,7 @@ namespace c2 {
 class ControllerSocketProtocol : public HeartbeatReporter {
  public:
   ControllerSocketProtocol(const std::string& name, const utils::Identifier& uuid = {}) // NOLINT
-      : HeartbeatReporter(name, uuid),
-        logger_(logging::LoggerFactory<ControllerSocketProtocol>::getLogger()) {
+      : HeartbeatReporter(name, uuid) {
   }
 
   /**
@@ -51,14 +49,14 @@ class ControllerSocketProtocol : public HeartbeatReporter {
    * @param updateSink update mechanism that will be used to stop/clear elements
    * @param configuration configuration class.
    */
-  virtual void initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<state::StateMonitor> &updateSink,
-                          const std::shared_ptr<Configure> &configuration);
+  void initialize(core::controller::ControllerServiceProvider* controller, state::StateMonitor* updateSink,
+                          const std::shared_ptr<Configure> &configuration) override;
 
   /**
    * Handles the heartbeat
    * @param payload incoming payload. From this function we only care about queue metrics.
    */
-  virtual int16_t heartbeat(const C2Payload &payload);
+  int16_t heartbeat(const C2Payload &payload) override;
 
  protected:
   /**
@@ -81,11 +79,9 @@ class ControllerSocketProtocol : public HeartbeatReporter {
   std::shared_ptr<minifi::io::StreamFactory> stream_factory_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<ControllerSocketProtocol>::getLogger();
 };
 
-REGISTER_RESOURCE(ControllerSocketProtocol, "Creates a reporter that can handle basic c2 operations for a localized environment through a simple TCP socket.");
-
 }  // namespace c2
 }  // namespace minifi
 }  // namespace nifi
