@@ -24,7 +24,6 @@
 #include "c2/C2Trigger.h"
 #include "utils/StringUtils.h"
 #include "utils/file/FileUtils.h"
-#include "core/Resource.h"
 #include "c2/C2Payload.h"
 #include "properties/Configure.h"
 
@@ -41,17 +40,15 @@ namespace c2 {
  */
 class FileUpdateTrigger : public C2Trigger {
  public:
-  FileUpdateTrigger(const std::string& name, const utils::Identifier& uuid = {}) // NOLINT
+  FileUpdateTrigger(const std::string &name, const utils::Identifier &uuid = {}) // NOLINT
       : C2Trigger(name, uuid),
-        last_update_(0),
-        update_(false),
-        logger_(logging::LoggerFactory<FileUpdateTrigger>::getLogger()) {
+        update_(false) {
   }
 
   void initialize(const std::shared_ptr<minifi::Configure> &configuration) {
     if (nullptr != configuration) {
       if (configuration->get(minifi::Configure::nifi_c2_file_watch, "c2.file.watch", file_)) {
-        last_update_ = utils::file::FileUtils::last_write_time(file_);
+        setLastUpdate(utils::file::last_write_time(file_));
       } else {
         logger_->log_trace("Could not configure file");
       }
@@ -59,23 +56,24 @@ class FileUpdateTrigger : public C2Trigger {
   }
 
   virtual bool triggered() {
-    if (last_update_ == 0) {
+    if (!getLastUpdate().has_value()) {
       logger_->log_trace("Last Update is zero");
       return false;
     }
-    auto update_time = utils::file::FileUtils::last_write_time(file_);
-    logger_->log_trace("Last Update is %d and update time is %d", last_update_.load(), update_time);
-    if (update_time > last_update_) {
-      last_update_ = update_time;
+    auto update_time = utils::file::last_write_time(file_);
+    auto last_update_l = getLastUpdate().value().time_since_epoch().count();
+    logger_->log_trace("Last Update is %d and update time is %d", last_update_l , update_time.value().time_since_epoch().count());
+    if (update_time > getLastUpdate()) {
+      setLastUpdate(update_time);
       update_ = true;
       return true;
     }
     return false;
   }
 
+
   virtual void reset() {
-    // reset the last write time
-    last_update_ = utils::file::FileUtils::last_write_time(file_);
+    setLastUpdate(utils::file::last_write_time(file_));
     update_ = false;
   }
 
@@ -107,16 +105,19 @@ class FileUpdateTrigger : public C2Trigger {
     return true;
   }
 
+  std::optional<std::filesystem::file_time_type> getLastUpdate() const;
+
+  void setLastUpdate(const std::optional<std::filesystem::file_time_type> &last_update);
+
  protected:
   std::string file_;
-  std::atomic<uint64_t> last_update_;
   std::atomic<bool> update_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<FileUpdateTrigger>::getLogger();
+  mutable std::mutex last_update_lock;
+  std::optional<std::filesystem::file_time_type> last_update_;
 };
-// add the trigger to the known resources.
-REGISTER_RESOURCE(FileUpdateTrigger, "Defines a file update trigger when the last write time of a file has been changed.");
 
 }  // namespace c2
 }  // namespace minifi
