@@ -23,7 +23,6 @@
 #include <vector>
 #include <iostream>
 #include <memory>
-#include "core/Resource.h"
 #include "utils/StringUtils.h"
 #include "io/validation.h"
 #include "core/controller/ControllerService.h"
@@ -49,8 +48,7 @@ class LinuxPowerManagerService : public ThreadManagementService {
         wait_period_(0),
         last_time_(0),
         trigger_(0),
-        low_battery_trigger_(0),
-        logger_(logging::LoggerFactory<LinuxPowerManagerService>::getLogger()) {
+        low_battery_trigger_(0) {
   }
 
   explicit LinuxPowerManagerService(const std::string &name, const std::shared_ptr<Configure> &configuration)
@@ -124,11 +122,9 @@ class LinuxPowerManagerService : public ThreadManagementService {
   std::string status_keyword_;
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<LinuxPowerManagerService>::getLogger();
 };
 
-REGISTER_RESOURCE(LinuxPowerManagerService, "Linux power management service that enables control of power usage in the agent through Linux power management information. Use name \"ThreadPoolManager\" to throttle battery consumption"); // NOLINT
-
 }  // namespace controllers
 }  // namespace minifi
 }  // namespace nifi
