@@ -36,12 +36,12 @@
 #include <iostream>
 #include <memory>
 #include <string>
-#include "core/Resource.h"
 #include "utils/StringUtils.h"
 #include "utils/tls/ExtendedKeyUsage.h"
 #include "io/validation.h"
 #include "../core/controller/ControllerService.h"
 #include "core/logging/LoggerConfiguration.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -85,14 +85,14 @@ class SSLContextService : public core::controller::ControllerService {
       : ControllerService(name, uuid),
         initialized_(false),
         valid_(false),
-        logger_(logging::LoggerFactory<SSLContextService>::getLogger()) {
+        logger_(core::logging::LoggerFactory<SSLContextService>::getLogger()) {
   }
 
   explicit SSLContextService(const std::string &name, const std::shared_ptr<Configure> &configuration)
       : ControllerService(name),
         initialized_(false),
         valid_(false),
-        logger_(logging::LoggerFactory<SSLContextService>::getLogger()) {
+        logger_(core::logging::LoggerFactory<SSLContextService>::getLogger()) {
     setConfiguration(configuration);
     initialize();
 
@@ -172,24 +172,24 @@ class SSLContextService : public core::controller::ControllerService {
 
   virtual void onEnable();
 
-  static const core::Property ClientCertificate;
-  static const core::Property PrivateKey;
-  static const core::Property Passphrase;
-  static const core::Property CACertificate;
-  static const core::Property UseSystemCertStore;
+  MINIFIAPI static const core::Property ClientCertificate;
+  MINIFIAPI static const core::Property PrivateKey;
+  MINIFIAPI static const core::Property Passphrase;
+  MINIFIAPI static const core::Property CACertificate;
+  MINIFIAPI static const core::Property UseSystemCertStore;
 #ifdef WIN32
-  static const core::Property CertStoreLocation;
-  static const core::Property ServerCertStore;
-  static const core::Property ClientCertStore;
-  static const core::Property ClientCertCN;
-  static const core::Property ClientCertKeyUsage;
+  MINIFIAPI static const core::Property CertStoreLocation;
+  MINIFIAPI static const core::Property ServerCertStore;
+  MINIFIAPI static const core::Property ClientCertStore;
+  MINIFIAPI static const core::Property ClientCertCN;
+  MINIFIAPI static const core::Property ClientCertKeyUsage;
 #endif  // WIN32
 
  protected:
   virtual void initializeProperties();
 
   std::mutex initialization_mutex_;
-  std::atomic<bool> initialized_;
+  bool initialized_;
   std::atomic<bool> valid_;
   std::string certificate_;
   std::string private_key_;
@@ -218,7 +218,7 @@ class SSLContextService : public core::controller::ControllerService {
 #endif
 
   static bool isFileTypeP12(const std::string& filename) {
-    return utils::StringUtils::endsWithIgnoreCase(filename, "p12");
+    return utils::StringUtils::endsWith(filename, "p12", false);
   }
 
  private:
@@ -233,10 +233,9 @@ class SSLContextService : public core::controller::ControllerService {
 #endif  // WIN32
 #endif  // OPENSSL_SUPPORT
 
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_;
 };
 typedef int (SSLContextService::*ptr)(char *, int, int, void *);
-REGISTER_RESOURCE(SSLContextService, "Controller service that provides SSL/TLS capabilities to consuming interfaces");
 
 }  // namespace controllers
 }  // namespace minifi
