@@ -22,12 +22,12 @@
 #include <iostream>
 #include <memory>
 #include <limits>
-#include "core/Resource.h"
 #include "utils/StringUtils.h"
 #include "io/validation.h"
 #include "core/controller/ControllerService.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "core/state/UpdatePolicy.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -42,10 +42,7 @@ namespace controllers {
 class UpdatePolicyControllerService : public core::controller::ControllerService, public std::enable_shared_from_this<UpdatePolicyControllerService> {
  public:
   explicit UpdatePolicyControllerService(const std::string &name, const utils::Identifier &uuid = {})
-      : ControllerService(name, uuid),
-        persist_updates_(false),
-        policy_(new state::UpdatePolicy(false)),
-        logger_(logging::LoggerFactory<UpdatePolicyControllerService>::getLogger()) {
+      : ControllerService(name, uuid) {
   }
 
   explicit UpdatePolicyControllerService(const std::string &name, const std::shared_ptr<Configure> &configuration)
@@ -54,20 +51,20 @@ class UpdatePolicyControllerService : public core::controller::ControllerService
     initialize();
   }
 
-  static core::Property AllowAllProperties;
-  static core::Property PersistUpdates;
-  static core::Property AllowedProperties;
-  static core::Property DisallowedProperties;
+  MINIFIAPI static core::Property AllowAllProperties;
+  MINIFIAPI static core::Property PersistUpdates;
+  MINIFIAPI static core::Property AllowedProperties;
+  MINIFIAPI static core::Property DisallowedProperties;
 
-  void initialize();
+  void initialize() override;
 
-  void yield();
+  void yield() override;
 
-  bool isRunning();
+  bool isRunning() override;
 
-  bool isWorkAvailable();
+  bool isWorkAvailable() override;
 
-  void onEnable();
+  void onEnable() override;
 
   bool canUpdate(const std::string &property) const {
     return policy_->canUpdate(property);
@@ -78,14 +75,11 @@ class UpdatePolicyControllerService : public core::controller::ControllerService
   }
 
  private:
-  bool persist_updates_;
-  std::unique_ptr<state::UpdatePolicy> policy_;
-  std::shared_ptr<logging::Logger> logger_;
+  bool persist_updates_ = false;
+  std::unique_ptr<state::UpdatePolicy> policy_ = std::make_unique<state::UpdatePolicy>(false);
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<UpdatePolicyControllerService>::getLogger();
 };
 
-REGISTER_RESOURCE(UpdatePolicyControllerService, "UpdatePolicyControllerService allows a flow specific policy on allowing or disallowing updates. "
-    "Since the flow dictates the purpose of a device it will also be used to dictate updates to specific components.");
-
 }  // namespace controllers
 }  // namespace minifi
 }  // namespace nifi
