@@ -28,7 +28,7 @@
 #include "properties/Configure.h"
 #include "core/logging/Logger.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "core/Resource.h"
+#include "utils/Export.h"
 
 
 namespace org {
@@ -43,8 +43,8 @@ class AbstractAutoPersistingKeyValueStoreService : virtual public PersistableKey
 
   ~AbstractAutoPersistingKeyValueStoreService() override;
 
-  static core::Property AlwaysPersist;
-  static core::Property AutoPersistenceInterval;
+  MINIFIAPI static core::Property AlwaysPersist;
+  MINIFIAPI static core::Property AutoPersistenceInterval;
 
   void initialize() override;
   void onEnable() override;
@@ -52,7 +52,7 @@ class AbstractAutoPersistingKeyValueStoreService : virtual public PersistableKey
 
  protected:
   bool always_persist_;
-  uint64_t auto_persistence_interval_;
+  std::chrono::milliseconds auto_persistence_interval_;
 
   std::thread persisting_thread_;
   bool running_;
@@ -61,7 +61,7 @@ class AbstractAutoPersistingKeyValueStoreService : virtual public PersistableKey
   void persistingThreadFunc();
 
  private:
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AbstractAutoPersistingKeyValueStoreService>::getLogger();
 
   void stopPersistingThread();
 };
