@@ -217,21 +217,25 @@ template<typename T>
 bool ConfigurableComponent::getProperty(const std::string name, T &value) const {
   std::lock_guard<std::mutex> lock(configuration_mutex_);
 
-  auto &&it = properties_.find(name);
-  if (it != properties_.end()) {
-    const Property& item = it->second;
-    if (item.getValue().getValue() == nullptr) {
+  const auto property_name_and_object = properties_.find(name);
+  if (property_name_and_object != properties_.end()) {
+    const Property& property = property_name_and_object->second;
+    if (property.getValue().getValue() == nullptr) {
       // empty value
-      if (item.getRequired()) {
-        logger_->log_error("Component %s required property %s is empty", name, item.getName());
-        throw utils::internal::RequiredPropertyMissingException("Required property is empty: " + item.getName());
+      if (property.getRequired()) {
+        logger_->log_error("Component %s required property %s is empty", name, property.getName());
+        throw utils::internal::RequiredPropertyMissingException("Required property is empty: " + property.getName());
       }
-      logger_->log_debug("Component %s property name %s, empty value", name, item.getName());
+      logger_->log_debug("Component %s property name %s, empty value", name, property.getName());
       return false;
     }
-    logger_->log_debug("Component %s property name %s value %s", name, item.getName(), item.getValue().to_string());
-    // cast throws if the value is invalid
-    value = static_cast<T>(item.getValue());
+    logger_->log_debug("Component %s property name %s value %s", name, property.getName(), property.getValue().to_string());
+
+    if constexpr (std::is_base_of<TransformableValue, T>::value) {
+      value = T(property.getValue().operator std::string());
+    } else {
+      value = static_cast<T>(property.getValue());  // cast throws if the value is invalid
+    }
     return true;
   } else {
     logger_->log_warn("Could not find property %s", name);
