@@ -20,12 +20,10 @@
 #ifndef WIN32_LEAN_AND_MEAN
 #define WIN32_LEAN_AND_MEAN 1
 #endif
-#include <properties/Configure.h>
 
-#include <cstdlib>
-#include <iostream>
 #include <memory>
 #include <string>
+#include <string_view>
 
 #ifdef WIN32
 #pragma comment(lib, "shlwapi.lib")
@@ -66,74 +64,43 @@
 #endif
 
 #include "utils/Id.h"
+#include "properties/Configure.h"
+#include "utils/StringUtils.h"
 
 /**
  * namespace aliasing
  */
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace utils {
-namespace file {
-}
-}
-namespace processors {
-}
-namespace provenance {
-
-}
-namespace core {
+namespace org::apache::nifi::minifi::core {
 
 template<typename T>
 static inline std::string getClassName() {
 #ifndef WIN32
   char *b = abi::__cxa_demangle(typeid(T).name(), 0, 0, 0);
   if (b == nullptr)
-    return std::string();
+    return {};
   std::string name = b;
   std::free(b);
   return name;
 #else
-  std::string adjusted_name = typeid(T).name();
-  // can probably skip class  manually for slightly higher performance
-  const std::string clazz = "class ";
-  auto haz_clazz = adjusted_name.find(clazz);
-  if (haz_clazz == 0)
-    adjusted_name = adjusted_name.substr(clazz.length(), adjusted_name.length() - clazz.length());
-  return adjusted_name;
+  std::string_view name = typeid(T).name();
+  const std::string_view class_prefix = "class ";
+  const std::string_view struct_prefix = "struct ";
+
+  if (utils::StringUtils::startsWith(name, class_prefix)) {
+    name.remove_prefix(class_prefix.length());
+  } else if (utils::StringUtils::startsWith(name, struct_prefix)) {
+    name.remove_prefix(struct_prefix.length());
+  }
+  return std::string{name};
 #endif
 }
 
 template<typename T>
-struct class_operations {
-  template<typename Q = T>
-  static std::true_type canDestruct(decltype(std::declval<Q>().~Q()) *) {
-    return std::true_type();
-  }
-
-  template<typename Q = T>
-  static std::false_type canDestruct(...) {
-    return std::false_type();
-  }
-
-  typedef decltype(canDestruct<T>(0)) type;
-
-  static const bool value = type::value;
-};
-
-template<typename T>
-typename std::enable_if<!class_operations<T>::value, std::shared_ptr<T>>::type instantiate(const std::string name = "") {
-  (void)name;  // against unused variable warnings
-  throw std::runtime_error("Cannot instantiate class");
-}
-
-template<typename T>
-typename std::enable_if<class_operations<T>::value, std::shared_ptr<T>>::type instantiate(const std::string name = "") {
-  if (name.length() == 0) {
-    return std::make_shared<T>();
+std::unique_ptr<T> instantiate(const std::string name = {}) {
+  if (name.empty()) {
+    return std::make_unique<T>();
   } else {
-    return std::make_shared<T>(name);
+    return std::make_unique<T>(name);
   }
 }
 
@@ -144,31 +111,16 @@ typename std::enable_if<class_operations<T>::value, std::shared_ptr<T>>::type in
  */
 class CoreComponent {
  public:
-  /**
-   * Constructor that sets the name and uuid.
-   */
-
-  explicit CoreComponent(const std::string &name, const utils::Identifier &uuid = {}, const std::shared_ptr<utils::IdGenerator> &idGenerator = utils::IdGenerator::getIdGenerator())
-      : name_(name) {
-    if (uuid.isNil()) {
-      // Generate the global UUID for the flow record
-      uuid_ = idGenerator->generate();
-    } else {
-      uuid_ = uuid;
-    }
-  }
-
-  explicit CoreComponent(const CoreComponent &other) = default;
-  /**
-   * Move Constructor.
-   */
-
-  explicit CoreComponent(CoreComponent &&other) = default;
+  explicit CoreComponent(const std::string &name, const utils::Identifier &uuid = {}, const std::shared_ptr<utils::IdGenerator> &idGenerator = utils::IdGenerator::getIdGenerator());
+  CoreComponent(const CoreComponent &other) = default;
+  CoreComponent(CoreComponent &&other) = default;
+  CoreComponent& operator=(const CoreComponent&) = default;
+  CoreComponent& operator=(CoreComponent&&) = default;
 
   virtual ~CoreComponent() = default;
 
   // Get component name Name
-  virtual std::string getName() const;
+  [[nodiscard]] virtual std::string getName() const;
 
   /**
    * Set name.
@@ -186,13 +138,12 @@ class CoreComponent {
    * Returns the UUID.
    * @return the uuid of the component
    */
-  utils::Identifier getUUID() const;
+  [[nodiscard]] utils::Identifier getUUID() const;
 
-  // unsigned const char *getUUID();
   /**
    * Return the UUID string
    */
-  utils::SmallString<36> getUUIDStr() const {
+  [[nodiscard]] utils::SmallString<36> getUUIDStr() const {
     return uuid_.to_string();
   }
 
@@ -206,30 +157,10 @@ class CoreComponent {
   // A global unique identifier
   utils::Identifier uuid_;
 
-  // Connectable's name
+  // CoreComponent's name
   std::string name_;
 };
 
-namespace logging {
-}
-}  // namespace core
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
-
-namespace fileutils = org::apache::nifi::minifi::utils::file;
-
-namespace minifi = org::apache::nifi::minifi;
-
-namespace core = org::apache::nifi::minifi::core;
-
-namespace processors = org::apache::nifi::minifi::processors;
-
-namespace logging = org::apache::nifi::minifi::core::logging;
-
-namespace utils = org::apache::nifi::minifi::utils;
-
-namespace provenance = org::apache::nifi::minifi::provenance;
+}  // namespace org::apache::nifi::minifi::core
 
 #endif  // LIBMINIFI_INCLUDE_CORE_CORE_H_
