@@ -21,12 +21,11 @@
 #include "Core.h"
 
 #include <cstdint>
-#include <memory>
-#include <unordered_map>
 #include <map>
+#include <memory>
+#include <optional>
 #include <string>
-
-#include "utils/OptionalUtils.h"
+#include <unordered_map>
 
 namespace org {
 namespace apache {
@@ -41,30 +40,33 @@ class CoreComponentStateManager {
   virtual ~CoreComponentStateManager() = default;
 
   virtual bool set(const CoreComponentState& kvs) = 0;
-
   virtual bool get(CoreComponentState& kvs) = 0;
 
-  utils::optional<std::unordered_map<std::string, std::string>> get() {
+  std::optional<std::unordered_map<std::string, std::string>> get() {
     std::unordered_map<std::string, std::string> out;
     if (get(out)) {
       return out;
     } else {
-      return utils::nullopt;
+      return std::nullopt;
     }
   }
 
   virtual bool clear() = 0;
-
   virtual bool persist() = 0;
+
+  virtual bool isTransactionInProgress() const = 0;
+  virtual bool beginTransaction() = 0;
+  virtual bool commit() = 0;
+  virtual bool rollback() = 0;
 };
 
 class CoreComponentStateManagerProvider {
  public:
   virtual ~CoreComponentStateManagerProvider() = default;
 
-  virtual std::shared_ptr<CoreComponentStateManager> getCoreComponentStateManager(const utils::Identifier& uuid) = 0;
+  virtual std::unique_ptr<CoreComponentStateManager> getCoreComponentStateManager(const utils::Identifier& uuid) = 0;
 
-  virtual std::shared_ptr<CoreComponentStateManager> getCoreComponentStateManager(const CoreComponent& component) {
+  virtual std::unique_ptr<CoreComponentStateManager> getCoreComponentStateManager(const CoreComponent& component) {
     return getCoreComponentStateManager(component.getUUID());
   }
 
