@@ -19,9 +19,10 @@
 #define LIBMINIFI_INCLUDE_CORE_FLOWCONFIGURATION_H_
 
 #include <memory>
+#include <optional>
 #include <string>
-#include <vector>
 #include <utility>
+#include <vector>
 
 #include "core/Core.h"
 #include "Connection.h"
@@ -32,7 +33,6 @@
 #include "core/reporting/SiteToSiteProvenanceReportingTask.h"
 
 #include "core/Processor.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "core/ProcessContext.h"
 #include "core/ProcessSession.h"
@@ -40,7 +40,7 @@
 #include "io/StreamFactory.h"
 #include "core/state/nodes/FlowInformation.h"
 #include "utils/file/FileSystem.h"
-#include "utils/OptionalUtils.h"
+#include "utils/ChecksumCalculator.h"
 
 namespace org {
 namespace apache {
@@ -66,43 +66,16 @@ class FlowConfiguration : public CoreComponent {
    * Constructor that will be used for configuring
    * the flow controller.
    */
-  explicit FlowConfiguration(std::shared_ptr<core::Repository> /*repo*/, std::shared_ptr<core::Repository> flow_file_repo,
+  explicit FlowConfiguration(std::shared_ptr<core::Repository> repo, std::shared_ptr<core::Repository> flow_file_repo,
                              std::shared_ptr<core::ContentRepository> content_repo, std::shared_ptr<io::StreamFactory> stream_factory,
-                             std::shared_ptr<Configure> configuration, const utils::optional<std::string>& path,
-                             std::shared_ptr<utils::file::FileSystem> filesystem = std::make_shared<utils::file::FileSystem>())
-      : CoreComponent(core::getClassName<FlowConfiguration>()),
-        flow_file_repo_(std::move(flow_file_repo)),
-        content_repo_(std::move(content_repo)),
-        stream_factory_(std::move(stream_factory)),
-        configuration_(configuration),
-        filesystem_(std::move(filesystem)),
-        logger_(logging::LoggerFactory<FlowConfiguration>::getLogger()) {
-    controller_services_ = std::make_shared<core::controller::ControllerServiceMap>();
-    service_provider_ = std::make_shared<core::controller::StandardControllerServiceProvider>(controller_services_, nullptr, configuration);
-    std::string flowUrl = "", bucket_id = "default", flowId = "";
-    configuration->get(Configure::nifi_c2_flow_id, flowId);
-    configuration->get(Configure::nifi_c2_flow_url, flowUrl);
-    flow_version_ = std::make_shared<state::response::FlowVersion>(flowUrl, bucket_id, flowId);
-
-    if (!path) {
-      logger_->log_error("Configuration path is not specified.");
-    } else {
-      config_path_ = utils::file::PathUtils::canonicalize(*path);
-      if (!config_path_) {
-        logger_->log_error("Couldn't find config file \"%s\".", *path);
-        config_path_ = path;
-      }
-    }
-
-    // it is okay if this has already been called
-    initialize_static_functions();
-  }
+                             std::shared_ptr<Configure> configuration, const std::optional<std::string>& path,
+                             std::shared_ptr<utils::file::FileSystem> filesystem = std::make_shared<utils::file::FileSystem>());
 
   ~FlowConfiguration() override;
 
   // Create Processor (Node/Input/Output Port) based on the name
-  std::shared_ptr<core::Processor> createProcessor(const std::string &name, const utils::Identifier &uuid);
-  std::shared_ptr<core::Processor> createProcessor(const std::string &name, const std::string &fullname, const utils::Identifier &uuid);
+  std::unique_ptr<core::Processor> createProcessor(const std::string &name, const utils::Identifier &uuid);
+  std::unique_ptr<core::Processor> createProcessor(const std::string &name, const std::string &fullname, const utils::Identifier &uuid);
   // Create Root Processor Group
 
   std::unique_ptr<core::ProcessGroup> createRootProcessGroup(const std::string &name, const utils::Identifier &uuid, int version);
@@ -113,9 +86,9 @@ class FlowConfiguration : public CoreComponent {
       const utils::Identifier &uuid);
 
   // Create Connection
-  std::shared_ptr<minifi::Connection> createConnection(const std::string &name, const utils::Identifier &uuid) const;
+  std::unique_ptr<minifi::Connection> createConnection(const std::string &name, const utils::Identifier &uuid) const;
   // Create Provenance Report Task
-  std::shared_ptr<core::Processor> createProvenanceReportTask(void);
+  std::unique_ptr<core::reporting::SiteToSiteProvenanceReportingTask> createProvenanceReportTask();
 
   std::shared_ptr<state::response::FlowVersion> getFlowVersion() const {
     return flow_version_;
@@ -131,7 +104,7 @@ class FlowConfiguration : public CoreComponent {
    * Returns the configuration path string
    * @return config_path_
    */
-  const utils::optional<std::string> &getConfigurationPath() {
+  const std::optional<std::string> &getConfigurationPath() {
     return config_path_;
   }
 
@@ -149,34 +122,15 @@ class FlowConfiguration : public CoreComponent {
     return service_provider_;
   }
 
-  static bool add_static_func(std::string functor) {
-    std::lock_guard<std::mutex> lock(get_static_functions().atomic_initialization_);
-    get_static_functions().statics_sl_funcs_.push_back(functor);
-    return true;
-  }
-
-  static void initialize_static_functions() {
-    std::lock_guard<std::mutex> lock(get_static_functions().atomic_initialization_);
-    for (auto sl_func : get_static_functions().statics_sl_funcs_) {
-      core::ClassLoader::getDefaultClassLoader().registerResource("", sl_func);
-    }
-  }
+  utils::ChecksumCalculator& getChecksumCalculator() { return checksum_calculator_; }
 
  protected:
-  void registerResource(const std::string &resource_function) {
-    core::ClassLoader::getDefaultClassLoader().registerResource("", resource_function);
-  }
-
-  void registerResource(const std::string &resource_location, const std::string &resource_function) {
-    core::ClassLoader::getDefaultClassLoader().registerResource(resource_location, resource_function);
-  }
-
   // service provider reference.
   std::shared_ptr<core::controller::StandardControllerServiceProvider> service_provider_;
   // based, shared controller service map.
   std::shared_ptr<core::controller::ControllerServiceMap> controller_services_;
   // configuration path
-  utils::optional<std::string> config_path_;
+  std::optional<std::string> config_path_;
   // flow file repo
   std::shared_ptr<core::Repository> flow_file_repo_;
   // content repository.
@@ -185,8 +139,8 @@ class FlowConfiguration : public CoreComponent {
   std::shared_ptr<io::StreamFactory> stream_factory_;
   std::shared_ptr<Configure> configuration_;
   std::shared_ptr<state::response::FlowVersion> flow_version_;
-
   std::shared_ptr<utils::file::FileSystem> filesystem_;
+  utils::ChecksumCalculator checksum_calculator_;
 
  private:
   std::shared_ptr<logging::Logger> logger_;
