@@ -20,17 +20,19 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
+#include <unordered_set>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "utils/OptionalUtils.h"
 #include "utils/TimeUtil.h"
 #include "ResourceClaim.h"
 #include "Connectable.h"
 #include "WeakReference.h"
 #include "utils/FlatMap.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -51,7 +53,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Returns a pointer to this flow file record's
    * claim
    */
-  std::shared_ptr<ResourceClaim> getResourceClaim();
+  [[nodiscard]] std::shared_ptr<ResourceClaim> getResourceClaim() const;
   /**
    * Sets _claim to the inbound claim argument
    */
@@ -93,7 +95,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * is marked as deleted.
    * @return marked deleted
    */
-  bool isDeleted() const;
+  [[nodiscard]] bool isDeleted() const;
 
   /**
    * Sets whether to mark this flow file record
@@ -106,24 +108,24 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Get entry date for this record
    * @return entry date uint64_t
    */
-  uint64_t getEntryDate() const;
+  [[nodiscard]] std::chrono::system_clock::time_point getEntryDate() const;
 
   /**
    * Gets the event time.
    * @return event time.
    */
-  uint64_t getEventTime() const;
+  [[nodiscard]] std::chrono::system_clock::time_point getEventTime() const;
   /**
    * Get lineage start date
    * @return lineage start date uint64_t
    */
-  uint64_t getlineageStartDate() const;
+  [[nodiscard]] std::chrono::system_clock::time_point getlineageStartDate() const;
 
   /**
    * Sets the lineage start date
    * @param date new lineage start date
    */
-  void setLineageStartDate(const uint64_t date);
+  void setLineageStartDate(const std::chrono::system_clock::time_point date);
 
   void setLineageIdentifiers(const std::vector<utils::Identifier>& lineage_Identifiers) {
     lineage_Identifiers_ = lineage_Identifiers;
@@ -137,7 +139,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    */
   bool getAttribute(const std::string& key, std::string& value) const;
 
-  utils::optional<std::string> getAttribute(const std::string& key) const;
+  [[nodiscard]] std::optional<std::string> getAttribute(const std::string& key) const;
 
   /**
    * Updates the value in the attribute map that corresponds
@@ -169,7 +171,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Returns the map of attributes
    * @return attributes.
    */
-  std::map<std::string, std::string> getAttributes() const {
+  [[nodiscard]] std::map<std::string, std::string> getAttributes() const {
     return {attributes_.begin(), attributes_.end()};
   }
 
@@ -198,7 +200,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Returns the size of corresponding flow file
    * @return size as a uint64_t
    */
-  uint64_t getSize() const;
+  [[nodiscard]] uint64_t getSize() const;
 
   /**
    * Sets the offset
@@ -213,7 +215,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
     to_be_processed_after_ = std::chrono::steady_clock::now() + duration;
   }
 
-  std::chrono::time_point<std::chrono::steady_clock> getPenaltyExpiration() const {
+  [[nodiscard]] std::chrono::steady_clock::time_point getPenaltyExpiration() const {
     return to_be_processed_after_;
   }
 
@@ -221,13 +223,13 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Gets the offset within the flow file
    * @return size as a uint64_t
    */
-  uint64_t getOffset() const;
+  [[nodiscard]] uint64_t getOffset() const;
 
-  bool isPenalized() const {
+  [[nodiscard]] bool isPenalized() const {
     return to_be_processed_after_ > std::chrono::steady_clock::now();
   }
 
-  uint64_t getId() const {
+  [[nodiscard]] uint64_t getId() const {
     return id_;
   }
 
@@ -235,18 +237,18 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
    * Sets the original connection with a shared pointer.
    * @param connection shared connection.
    */
-  void setConnection(const std::shared_ptr<core::Connectable>& connection);
+  void setConnection(core::Connectable* connection);
   /**
    * Returns the original connection referenced by this record.
    * @return shared original connection pointer.
    */
-  std::shared_ptr<core::Connectable> getConnection() const;
+  [[nodiscard]] Connectable* getConnection() const;
 
   void setStoredToRepository(bool storedInRepository) {
     stored = storedInRepository;
   }
 
-  bool isStored() const {
+  [[nodiscard]] bool isStored() const {
     return stored;
   }
 
@@ -255,11 +257,11 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
   // Mark for deletion
   bool marked_delete_;
   // Date at which the flow file entered the flow
-  uint64_t entry_date_;
+  std::chrono::system_clock::time_point entry_date_{};
   // event time
-  uint64_t event_time_;
+  std::chrono::system_clock::time_point event_time_{};
   // Date at which the origin of this flow file entered the flow
-  uint64_t lineage_start_date_;
+  std::chrono::system_clock::time_point lineage_start_date_{};
   // Date at which the flow file was queued
   uint64_t last_queue_date_;
   // Size in bytes of the data corresponding to this flow file
@@ -270,7 +272,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
   // Offset to the content
   uint64_t offset_;
   // Penalty expiration
-  std::chrono::time_point<std::chrono::steady_clock> to_be_processed_after_;
+  std::chrono::steady_clock::time_point to_be_processed_after_;
   // Attributes key/values pairs for the flow record
   AttributeMap attributes_;
   // Pointer to the associated content resource claim
@@ -283,7 +285,7 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
   std::vector<utils::Identifier> lineage_Identifiers_;
 
   // Orginal connection queue that this flow file was dequeued from
-  std::shared_ptr<core::Connectable> connection_;
+  core::Connectable* connection_ = nullptr;
 
   static std::shared_ptr<logging::Logger> logger_;
   static std::shared_ptr<utils::IdGenerator> id_generator_;
@@ -293,23 +295,30 @@ class FlowFile : public CoreComponent, public ReferenceContainer {
 // FlowFile Attribute
 struct SpecialFlowAttribute {
   // The flowfile's path indicates the relative directory to which a FlowFile belongs and does not contain the filename
-  static const std::string PATH;
+  MINIFIAPI static const std::string PATH;
   // The flowfile's absolute path indicates the absolute directory to which a FlowFile belongs and does not contain the filename
-  static const std::string ABSOLUTE_PATH;
+  MINIFIAPI static const std::string ABSOLUTE_PATH;
   // The filename of the FlowFile. The filename should not contain any directory structure.
-  static const std::string FILENAME;
+  MINIFIAPI static const std::string FILENAME;
   // A unique UUID assigned to this FlowFile.
-  static const std::string UUID;
+  MINIFIAPI static const std::string UUID;
   // A numeric value indicating the FlowFile priority
-  static const std::string priority;
+  MINIFIAPI static const std::string priority;
   // The MIME Type of this FlowFile
-  static const std::string MIME_TYPE;
+  MINIFIAPI static const std::string MIME_TYPE;
   // Specifies the reason that a FlowFile is being discarded
-  static const std::string DISCARD_REASON;
+  MINIFIAPI static const std::string DISCARD_REASON;
   // Indicates an identifier other than the FlowFile's UUID that is known to refer to this FlowFile.
-  static const std::string ALTERNATE_IDENTIFIER;
+  MINIFIAPI static const std::string ALTERNATE_IDENTIFIER;
   // Flow identifier
-  static const std::string FLOW_ID;
+  MINIFIAPI static const std::string FLOW_ID;
+
+  static const auto& getSpecialFlowAttributes() {
+    static const std::array<std::string_view, 9> SPECIAL_FLOW_ATTRIBUTES {
+      PATH, ABSOLUTE_PATH, FILENAME, UUID, priority, MIME_TYPE, DISCARD_REASON, ALTERNATE_IDENTIFIER, FLOW_ID
+    };
+    return SPECIAL_FLOW_ATTRIBUTES;
+  }
 };
 
 }  // namespace core
