@@ -18,15 +18,17 @@
 #ifndef LIBMINIFI_INCLUDE_CORE_PROCESSCONTEXT_H_
 #define LIBMINIFI_INCLUDE_CORE_PROCESSCONTEXT_H_
 
+#include <algorithm>
+#include <atomic>
+#include <map>
+#include <memory>
+#include <mutex>
+#include <optional>
+#include <queue>
 #include <string>
 #include <vector>
-#include <queue>
-#include <map>
 #include <unordered_map>
-#include <mutex>
-#include <atomic>
-#include <algorithm>
-#include <memory>
+
 #include "Property.h"
 #include "core/Core.h"
 #include "core/ContentRepository.h"
@@ -40,7 +42,6 @@
 #include "core/FlowFile.h"
 #include "core/CoreComponentState.h"
 #include "utils/file/FileUtils.h"
-#include "utils/OptionalUtils.h"
 #include "utils/PropertyErrors.h"
 #include "VariableRegistry.h"
 
@@ -100,22 +101,26 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
   }
 
   template<typename T = std::string>
-  typename std::enable_if<std::is_default_constructible<T>::value, utils::optional<T>>::type
-  getProperty(const Property& property) {
+  std::enable_if_t<std::is_default_constructible<T>::value, std::optional<T>>
+  getProperty(const Property& property) const {
     T value;
     try {
-      if (!getProperty(property.getName(), value)) return utils::nullopt;
+      if (!getProperty(property.getName(), value)) return std::nullopt;
     } catch (const utils::internal::ValueException&) {
-      return utils::nullopt;
+      return std::nullopt;
     }
     return value;
   }
 
   template<typename T>
-  bool getProperty(const std::string &name, T &value) const {
+  std::enable_if_t<!std::is_convertible_v<T&, const FlowFile&> && !std::is_convertible_v<T&, const std::shared_ptr<FlowFile>&>,
+      bool> getProperty(const std::string &name, T &value) const {
     return getPropertyImp<typename std::common_type<T>::type>(name, value);
   }
 
+  template<typename T = std::string>
+  std::enable_if_t<std::is_default_constructible_v<T>, std::optional<T>> getProperty(const Property&, const std::shared_ptr<FlowFile>&);
+
   virtual bool getProperty(const Property &property, std::string &value, const std::shared_ptr<FlowFile>& /*flow_file*/) {
     return getProperty(property.getName(), value);
   }
@@ -125,19 +130,36 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
   virtual bool getDynamicProperty(const Property &property, std::string &value, const std::shared_ptr<FlowFile>& /*flow_file*/) {
     return getDynamicProperty(property.getName(), value);
   }
+  bool getDynamicProperty(const Property &property, std::string &value, const std::shared_ptr<FlowFile>& flow_file, const std::map<std::string, std::string>& variables) {
+    std::map<std::string, std::optional<std::string>> original_attributes;
+    for (const auto& [variable, attr_value] : variables) {
+      original_attributes[variable] = flow_file->getAttribute(variable);
+      flow_file->setAttribute(variable, attr_value);
+    }
+    auto onExit = gsl::finally([&]{
+      for (const auto& attr : original_attributes) {
+        if (attr.second) {
+          flow_file->setAttribute(attr.first, attr.second.value());
+        } else {
+          flow_file->removeAttribute(attr.first);
+        }
+      }
+    });
+    return getDynamicProperty(property, value, flow_file);
+  }
   std::vector<std::string> getDynamicPropertyKeys() const {
     return processor_node_->getDynamicPropertyKeys();
   }
   // Sets the property value using the property's string name
-  bool setProperty(const std::string &name, std::string value) {
+  virtual bool setProperty(const std::string &name, std::string value) {
     return processor_node_->setProperty(name, value);
   }  // Sets the dynamic property value using the property's string name
-  bool setDynamicProperty(const std::string &name, std::string value) {
+  virtual bool setDynamicProperty(const std::string &name, std::string value) {
     return processor_node_->setDynamicProperty(name, value);
   }
   // Sets the property value using the Property object
-  bool setProperty(const Property& prop, std::string value) {
-    return processor_node_->setProperty(prop, value);
+  bool setProperty(const Property& property, std::string value) {
+    return setProperty(property.getName(), value);
   }
   // Whether the relationship is supported
   bool isSupportedRelationship(Relationship relationship) const {
@@ -185,7 +207,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
    * @return the ControllerService that is registered with the given
    * identifier
    */
-  std::shared_ptr<core::controller::ControllerService> getControllerService(const std::string &identifier) {
+  std::shared_ptr<core::controller::ControllerService> getControllerService(const std::string &identifier) const override {
     return controller_service_provider_ == nullptr ? nullptr : controller_service_provider_->getControllerServiceForComponent(identifier, processor_node_->getUUID());
   }
 
@@ -196,7 +218,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
    * identifier is not known by this ControllerServiceLookup, returns
    * <code>false</code>
    */
-  bool isControllerServiceEnabled(const std::string &identifier) {
+  bool isControllerServiceEnabled(const std::string &identifier) override {
     return controller_service_provider_->isControllerServiceEnabled(identifier);
   }
 
@@ -207,7 +229,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
    * otherwise returns <code>false</code>. If the given identifier is not
    * known by this ControllerServiceLookup, returns <code>false</code>
    */
-  bool isControllerServiceEnabling(const std::string &identifier) {
+  bool isControllerServiceEnabling(const std::string &identifier) override {
     return controller_service_provider_->isControllerServiceEnabling(identifier);
   }
 
@@ -216,7 +238,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
    * @return the name of the Controller service with the given identifier. If
    * no service can be found with this identifier, returns {@code null}
    */
-  const std::string getControllerServiceName(const std::string &identifier) {
+  const std::string getControllerServiceName(const std::string &identifier) const override {
     return controller_service_provider_->getControllerServiceName(identifier);
   }
 
@@ -232,17 +254,23 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
 
   static constexpr char const* DefaultStateManagerProviderName = "defaultstatemanagerprovider";
 
-  std::shared_ptr<CoreComponentStateManager> getStateManager() {
+  CoreComponentStateManager* getStateManager() {
     if (state_manager_provider_ == nullptr) {
       return nullptr;
     }
-    return state_manager_provider_->getCoreComponentStateManager(*processor_node_);
+    if (!state_manager_) {
+      state_manager_ = state_manager_provider_->getCoreComponentStateManager(*processor_node_);
+    }
+    return state_manager_.get();
+  }
+
+  bool hasStateManager() const {
+    return state_manager_ != nullptr;
   }
 
   static std::shared_ptr<core::CoreComponentStateManagerProvider> getOrCreateDefaultStateManagerProvider(
       controller::ControllerServiceProvider* controller_service_provider,
-      const std::shared_ptr<minifi::Configure>& configuration,
-      const char* base_path = "") {
+      const std::shared_ptr<minifi::Configure>& configuration) {
     static std::mutex mutex;
     std::lock_guard<std::mutex> lock(mutex);
 
@@ -256,6 +284,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
     std::string always_persist, auto_persistence_interval;
     configuration->get(Configure::nifi_state_management_provider_local_always_persist, always_persist);
     configuration->get(Configure::nifi_state_management_provider_local_auto_persistence_interval, auto_persistence_interval);
+    const auto path = configuration->get(Configure::nifi_state_management_provider_local_path);
 
     /* Function to help creating a provider */
     auto create_provider = [&](
@@ -297,8 +326,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
     if (preferredType.empty() || preferredType == "RocksDbPersistableKeyValueStoreService") {
       auto provider = create_provider("RocksDbPersistableKeyValueStoreService",
                                       "org.apache.nifi.minifi.controllers.RocksDbPersistableKeyValueStoreService",
-                                      {{"Directory", utils::file::FileUtils::concat_path(base_path,
-                                                                                         "corecomponentstate")}});
+                                      {{"Directory", path.value_or("corecomponentstate")}});
       if (provider != nullptr) {
         return provider;
       }
@@ -308,7 +336,7 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
     if (preferredType.empty() || preferredType == "UnorderedMapPersistableKeyValueStoreService") {
       auto provider = create_provider("UnorderedMapPersistableKeyValueStoreService",
                                  "org.apache.nifi.minifi.controllers.UnorderedMapPersistableKeyValueStoreService",
-                                 {{"File", utils::file::FileUtils::concat_path(base_path, "corecomponentstate.txt")}});
+                                 {{"File", path.value_or("corecomponentstate.txt")}});
       if (provider != nullptr) {
         return provider;
       }
@@ -358,26 +386,38 @@ class ProcessContext : public controller::ControllerServiceLookup, public core::
     return processor_node_->getProperty<typename std::common_type<T>::type>(name, value);
   }
 
-  // controller service provider.
   controller::ControllerServiceProvider* controller_service_provider_;
-  // state manager provider
   std::shared_ptr<core::CoreComponentStateManagerProvider> state_manager_provider_;
-  // repository shared pointer.
+  std::unique_ptr<CoreComponentStateManager> state_manager_;
   std::shared_ptr<core::Repository> repo_;
   std::shared_ptr<core::Repository> flow_repo_;
-
-  // repository shared pointer.
   std::shared_ptr<core::ContentRepository> content_repo_;
-  // Processor
   std::shared_ptr<ProcessorNode> processor_node_;
-
-  // Logger
   std::shared_ptr<logging::Logger> logger_;
   std::shared_ptr<Configure> configure_;
-
   bool initialized_;
 };
 
+template<typename T>
+inline std::enable_if_t<std::is_default_constructible_v<T>, std::optional<T>> ProcessContext::getProperty(const Property& property, const std::shared_ptr<FlowFile>& flow_file) {
+  T value;
+  std::string string_value;
+  if (!getProperty(property, string_value, flow_file)) return std::nullopt;
+  try {
+    if (!state::response::Value{string_value}.template convertValue<>(value)) return std::nullopt;
+  } catch (const utils::internal::ValueException&) {
+    return std::nullopt;
+  }
+  return value;
+}
+
+template<>
+inline std::optional<std::string> ProcessContext::getProperty<std::string>(const Property& property, const std::shared_ptr<FlowFile>& flow_file) {
+  std::string value;
+  if (!getProperty(property, value, flow_file)) return std::nullopt;
+  return value;
+}
+
 }  // namespace core
 }  // namespace minifi
 }  // namespace nifi
