@@ -44,6 +44,14 @@ namespace apache {
 namespace nifi {
 namespace minifi {
 namespace core {
+namespace detail {
+struct ReadBufferResult {
+  int64_t status;
+  std::vector<std::byte> buffer;
+};
+
+std::string to_string(const ReadBufferResult& read_buffer_result);
+}  // namespace detail
 
 // ProcessSession Class
 class ProcessSession : public ReferenceContainer {
@@ -52,14 +60,7 @@ class ProcessSession : public ReferenceContainer {
   /*!
    * Create a new process session
    */
-  ProcessSession(std::shared_ptr<ProcessContext> processContext = nullptr) // NOLINT
-      : process_context_(std::move(processContext)),
-        logger_(logging::LoggerFactory<ProcessSession>::getLogger()) {
-    logger_->log_trace("ProcessSession created for %s", process_context_->getProcessorNode()->getName());
-    auto repo = process_context_->getProvenanceRepository();
-    provenance_report_ = std::make_shared<provenance::ProvenanceReporter>(repo, process_context_->getProcessorNode()->getName(), process_context_->getProcessorNode()->getName());
-    content_session_ = process_context_->getContentRepository()->createSession();
-  }
+  explicit ProcessSession(std::shared_ptr<ProcessContext> processContext = nullptr);
 
   // Destructor
   virtual ~ProcessSession();
@@ -94,13 +95,21 @@ class ProcessSession : public ReferenceContainer {
   // Remove Flow File
   void remove(const std::shared_ptr<core::FlowFile> &flow);
   // Execute the given read callback against the content
-  int read(const std::shared_ptr<core::FlowFile> &flow, InputStreamCallback *callback);
+  int64_t read(const std::shared_ptr<core::FlowFile> &flow, const io::InputStreamCallback& callback);
+  // Read content into buffer
+  detail::ReadBufferResult readBuffer(const std::shared_ptr<core::FlowFile>& flow);
   // Execute the given write callback against the content
-  void write(const std::shared_ptr<core::FlowFile> &flow, OutputStreamCallback *callback);
+  void write(const std::shared_ptr<core::FlowFile> &flow, const io::OutputStreamCallback& callback);
+  // Read and write the flow file at the same time (eg. for processing it line by line)
+  int64_t readWrite(const std::shared_ptr<core::FlowFile> &flow, const io::InputOutputStreamCallback& callback);
   // Replace content with buffer
   void writeBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const char> buffer);
+  void writeBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const std::byte> buffer);
   // Execute the given write/append callback against the content
-  void append(const std::shared_ptr<core::FlowFile> &flow, OutputStreamCallback *callback);
+  void append(const std::shared_ptr<core::FlowFile> &flow, const io::OutputStreamCallback& callback);
+  // Append buffer to content
+  void appendBuffer(const std::shared_ptr<core::FlowFile>& flow, gsl::span<const char> buffer);
+  void appendBuffer(const std::shared_ptr<core::FlowFile>& flow, gsl::span<const std::byte> buffer);
   // Penalize the flow
   void penalize(const std::shared_ptr<core::FlowFile> &flow);
 
@@ -171,11 +180,11 @@ class ProcessSession : public ReferenceContainer {
   RouteResult routeFlowFile(const std::shared_ptr<FlowFile>& record);
 
   void persistFlowFilesBeforeTransfer(
-      std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<core::FlowFile>>>& transactionMap,
+      std::map<Connectable*, std::vector<std::shared_ptr<core::FlowFile>>>& transactionMap,
       const std::map<utils::Identifier, FlowFileUpdate>& modifiedFlowFiles);
 
   void ensureNonNullResourceClaim(
-      const std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<core::FlowFile>>>& transactionMap);
+      const std::map<Connectable*, std::vector<std::shared_ptr<core::FlowFile>>>& transactionMap);
 
   // Clone the flow file during transfer to multiple connections for a relationship
   std::shared_ptr<core::FlowFile> cloneDuringTransfer(const std::shared_ptr<core::FlowFile> &parent);
@@ -188,6 +197,8 @@ class ProcessSession : public ReferenceContainer {
 
   std::shared_ptr<ContentSession> content_session_;
 
+  CoreComponentStateManager* stateManager_;
+
   static std::shared_ptr<utils::IdGenerator> id_generator_;
 };
 
