@@ -25,28 +25,17 @@
 #include <chrono>
 #include <condition_variable>
 #include <functional>
-#include <map>
 #include <memory>
 #include <mutex>
-#include <queue>
-#include <set>
-#include <stack>
 #include <string>
-#include <vector>
 #include <unordered_set>
 #include <unordered_map>
+#include <utility>
 
 #include "ConfigurableComponent.h"
 #include "Connectable.h"
-#include "Connection.h"
 #include "Core.h"
 #include "core/Annotation.h"
-#include "io/StreamFactory.h"
-#include "ProcessContext.h"
-#include "ProcessSession.h"
-#include "ProcessSessionFactory.h"
-#include "Property.h"
-#include "Relationship.h"
 #include "Scheduling.h"
 #include "utils/TimeUtil.h"
 
@@ -54,22 +43,30 @@ namespace org {
 namespace apache {
 namespace nifi {
 namespace minifi {
+
+class Connection;
+
+namespace io {
+class StreamFactory;
+}
+
 namespace core {
 
+class ProcessContext;
+class ProcessSession;
+class ProcessSessionFactory;
+
 // Minimum scheduling period in Nano Second
-#define MINIMUM_SCHEDULING_NANOS 30000
+constexpr std::chrono::nanoseconds MINIMUM_SCHEDULING_NANOS{30000};
 
 // Default penalization period in second
 
 #define BUILDING_DLL 1
 // Processor Class
-class Processor : public Connectable, public ConfigurableComponent, public std::enable_shared_from_this<Processor> {
+class Processor : public Connectable, public ConfigurableComponent {
  public:
   Processor(const std::string& name, const utils::Identifier& uuid);
   explicit Processor(const std::string& name);
-  virtual ~Processor() {
-    notifyStop();
-  }
 
   bool isRunning() override;
   // Set Processor Scheduled State
@@ -86,23 +83,12 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   SchedulingStrategy getSchedulingStrategy() const {
     return strategy_;
   }
-  // Set Processor Loss Tolerant
-  void setlossTolerant(bool lossTolerant) {
-    loss_tolerant_ = lossTolerant;
-  }
-  // Get Processor Loss Tolerant
-  bool getlossTolerant() const {
-    return loss_tolerant_;
-  }
   // Set Processor Scheduling Period in Nano Second
-  void setSchedulingPeriodNano(uint64_t period) {
-    uint64_t minPeriod = MINIMUM_SCHEDULING_NANOS;
-    // std::max has some variances on c++11-c++14 and then c++14 onward.
-    // to avoid macro conditional checks we can use this simple conditional expr.
-  scheduling_period_nano_ = period > minPeriod ? period : minPeriod;
+  void setSchedulingPeriodNano(std::chrono::nanoseconds period) {
+    scheduling_period_nano_ = std::max(MINIMUM_SCHEDULING_NANOS, period);
   }
   // Get Processor Scheduling Period in Nano Second
-  uint64_t getSchedulingPeriodNano() const {
+  std::chrono::nanoseconds getSchedulingPeriodNano() const {
     return scheduling_period_nano_;
   }
 
@@ -118,25 +104,25 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
    * Returns the cron period
    * @return cron period
    */
-  const std::string getCronPeriod() const {
+  std::string getCronPeriod() const {
     return cron_period_;
   }
 
   // Set Processor Run Duration in Nano Second
-  void setRunDurationNano(uint64_t period) {
+  void setRunDurationNano(std::chrono::nanoseconds period) {
     run_duration_nano_ = period;
   }
   // Get Processor Run Duration in Nano Second
-  uint64_t getRunDurationNano() const {
+  std::chrono::nanoseconds getRunDurationNano() const {
     return (run_duration_nano_);
   }
   // Set Processor yield period in MilliSecond
-  void setYieldPeriodMsec(uint64_t period) {
+  void setYieldPeriodMsec(std::chrono::milliseconds period) {
     yield_period_msec_ = period;
   }
   // Get Processor yield period in MilliSecond
-  uint64_t getYieldPeriodMsec() const {
-    return (yield_period_msec_);
+  std::chrono::milliseconds getYieldPeriodMsec() const {
+    return yield_period_msec_;
   }
 
   void setPenalizationPeriod(std::chrono::milliseconds period) {
@@ -144,13 +130,14 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   }
 
   // Set Processor Maximum Concurrent Tasks
-  void setMaxConcurrentTasks(uint8_t tasks) {
-    max_concurrent_tasks_ = tasks;
-  }
-  // Get Processor Maximum Concurrent Tasks
-  uint8_t getMaxConcurrentTasks() const {
-    return (max_concurrent_tasks_);
+  void setMaxConcurrentTasks(uint8_t tasks) override;
+
+  // Overriding to yield true can be used to indicate that the Processor is not safe for concurrent execution
+  // of its onTrigger() method. By default, Processors are assumed to be safe for concurrent execution.
+  virtual bool isSingleThreaded() const {
+    return false;
   }
+
   // Set Trigger when empty
   void setTriggerWhenEmpty(bool value) {
     _triggerWhenEmpty = value;
@@ -175,38 +162,19 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   void clearActiveTask() {
     active_tasks_ = 0;
   }
-  // Yield based on the yield period
-  void yield() override {
-    yield_expiration_ = (utils::timeutils::getTimeMillis() + yield_period_msec_);
-  }
-  // Yield based on the input time
-  void yield(uint64_t time) {
-    yield_expiration_ = (utils::timeutils::getTimeMillis() + time);
-  }
-  // whether need be to yield
-  virtual bool isYield() {
-    if (yield_expiration_ > 0)
-      return (yield_expiration_ >= utils::timeutils::getTimeMillis());
-    else
-      return false;
-  }
-  // clear yield expiration
-  void clearYield() {
-    yield_expiration_ = 0;
-  }
-  // get yield time
-  uint64_t getYieldTime() const {
-    uint64_t curTime = utils::timeutils::getTimeMillis();
-    if (yield_expiration_ > curTime)
-      return (yield_expiration_ - curTime);
-    else
-      return 0;
-  }
-  // Whether flow file queue full in any of the outgoin connection
+  void yield() override;
+
+  void yield(std::chrono::milliseconds delta_time);
+
+  virtual bool isYield();
+
+  void clearYield();
+
+  std::chrono::milliseconds getYieldTime() const;
+  // Whether flow file queue full in any of the outgoing connection
   bool flowFilesOutGoingFull() const;
 
-  bool addConnection(std::shared_ptr<Connectable> connection);
-  void removeConnection(std::shared_ptr<Connectable> connection);
+  bool addConnection(Connectable* connection);
 
   virtual void onTrigger(const std::shared_ptr<ProcessContext> &context, const std::shared_ptr<ProcessSessionFactory> &sessionFactory);
   void onTrigger(ProcessContext *context, ProcessSessionFactory *sessionFactory);
@@ -241,7 +209,7 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   bool isWorkAvailable() override;
 
   void setStreamFactory(std::shared_ptr<minifi::io::StreamFactory> stream_factory) {
-    stream_factory_ = stream_factory;
+    stream_factory_ = std::move(stream_factory);
   }
 
   bool supportsDynamicProperties() override {
@@ -250,7 +218,7 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
 
   bool isThrottledByBackpressure() const;
 
-  std::shared_ptr<Connectable> pickIncomingConnection() override;
+  Connectable* pickIncomingConnection() override;
 
   void validateAnnotations() const;
 
@@ -265,14 +233,12 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   // Processor state
   std::atomic<ScheduledState> state_;
 
-  // lossTolerant
-  std::atomic<bool> loss_tolerant_;
   // SchedulePeriod in Nano Seconds
-  std::atomic<uint64_t> scheduling_period_nano_;
+  std::atomic<std::chrono::nanoseconds> scheduling_period_nano_;
   // Run Duration in Nano Seconds
-  std::atomic<uint64_t> run_duration_nano_;
+  std::atomic<std::chrono::nanoseconds> run_duration_nano_;
   // Yield Period in Milliseconds
-  std::atomic<uint64_t> yield_period_msec_;
+  std::atomic<std::chrono::milliseconds> yield_period_msec_;
 
   // Active Tasks
   std::atomic<uint8_t> active_tasks_;
@@ -285,7 +251,7 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   // Mutex for protection
   mutable std::mutex mutex_;
   // Yield Expiration
-  std::atomic<uint64_t> yield_expiration_;
+  std::atomic<std::chrono::time_point<std::chrono::system_clock>> yield_expiration_{};
 
   // Prevent default copy constructor and assignment operation
   // Only support pass by reference or pointer
@@ -301,7 +267,7 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   // must hold the graphMutex
   void updateReachability(const std::lock_guard<std::mutex>& graph_lock, bool force = false);
 
-  static bool partOfCycle(const std::shared_ptr<Connection>& conn);
+  static bool partOfCycle(Connection* conn);
 
   virtual annotation::Input getInputRequirement() const {
       // default input requirement
@@ -309,13 +275,12 @@ class Processor : public Connectable, public ConfigurableComponent, public std::
   }
 
   // an outgoing connection allows us to reach these nodes
-  std::unordered_map<std::shared_ptr<Connection>, std::unordered_set<std::shared_ptr<const Processor>>> reachable_processors_;
+  std::unordered_map<Connection*, std::unordered_set<Processor*>> reachable_processors_;
 
   std::shared_ptr<logging::Logger> logger_;
 };
 
 }  // namespace core
-/* namespace core */
 }  // namespace minifi
 }  // namespace nifi
 }  // namespace apache
