@@ -31,6 +31,7 @@
 #include <typeindex>
 #include <utility>
 #include <vector>
+#include <string_view>
 
 #include "CachedValueValidator.h"
 #include "core/Core.h"
@@ -116,7 +117,7 @@ class Property {
   std::shared_ptr<PropertyValidator> getValidator() const;
   const PropertyValue &getValue() const;
   bool getRequired() const;
-  bool supportsExpressionLangauge() const;
+  bool supportsExpressionLanguage() const;
   std::string getValidRegex() const;
   std::vector<std::string> getDependentProperties() const;
   std::vector<std::pair<std::string, std::string>> getExclusiveOfProperties() const;
@@ -180,87 +181,6 @@ class Property {
 // Compare
   bool operator <(const Property & right) const;
 
-  template<typename T>
-  static bool ConvertTimeUnitToMS(int64_t input, TimeUnit unit, T &out) {
-    if (unit == NANOSECOND) {
-      out = input / 1000 / 1000;
-      return true;
-    } else if (unit == MICROSECOND) {
-      out = input / 1000;
-      return true;
-    } else if (unit == MILLISECOND) {
-      out = input;
-      return true;
-    } else if (unit == SECOND) {
-      out = input * 1000;
-      return true;
-    } else if (unit == MINUTE) {
-      out = input * 60 * 1000;
-      return true;
-    } else if (unit == HOUR) {
-      out = input * 60 * 60 * 1000;
-      return true;
-    } else if (unit == DAY) {
-      out = 24 * 60 * 60 * 1000;
-      return true;
-    } else {
-      return false;
-    }
-  }
-
-  static bool ConvertTimeUnitToMS(int64_t input, TimeUnit unit, int64_t &out) {
-    return ConvertTimeUnitToMS<int64_t>(input, unit, out);
-  }
-
-  static bool ConvertTimeUnitToMS(int64_t input, TimeUnit unit, uint64_t &out) {
-    return ConvertTimeUnitToMS<uint64_t>(input, unit, out);
-  }
-
-  template<typename T>
-  static bool ConvertTimeUnitToNS(int64_t input, TimeUnit unit, T &out) {
-    if (unit == NANOSECOND) {
-      out = input;
-      return true;
-    } else if (unit == MICROSECOND) {
-      out = input * 1000;
-      return true;
-    } else if (unit == MILLISECOND) {
-      out = input * 1000 * 1000;
-      return true;
-    } else if (unit == SECOND) {
-      out = input * 1000 * 1000 * 1000;
-      return true;
-    } else if (unit == MINUTE) {
-      out = input * 60 * 1000 * 1000 * 1000;
-      return true;
-    } else if (unit == HOUR) {
-      out = input * 60 * 60 * 1000 * 1000 * 1000;
-      return true;
-    } else if (unit == DAY) {
-      out = input * 24 * 60 * 60 * 1000 * 1000 * 1000;
-      return true;
-    } else {
-      return false;
-    }
-  }
-
-  static bool ConvertTimeUnitToNS(int64_t input, TimeUnit unit, uint64_t &out) {
-    return ConvertTimeUnitToNS<uint64_t>(input, unit, out);
-  }
-
-  static bool ConvertTimeUnitToNS(int64_t input, TimeUnit unit, int64_t &out) {
-    return ConvertTimeUnitToNS<int64_t>(input, unit, out);
-  }
-
-// Convert String
-  static bool StringToTime(std::string input, uint64_t &output, TimeUnit &timeunit) {
-    return utils::internal::StringToTime(input, output, timeunit);
-  }
-
-// Convert String
-  static bool StringToTime(const std::string& input, int64_t &output, TimeUnit &timeunit) {
-    return utils::internal::StringToTime(input, output, timeunit);
-  }
 
   static bool StringToDateTime(const std::string& input, int64_t& output) {
     int64_t temp = utils::timeutils::parseDateTimeStr(input);
@@ -311,18 +231,6 @@ class Property {
     return true;
   }
 
-  static bool getTimeMSFromString(const std::string& str, uint64_t& valInt) {
-    core::TimeUnit unit;
-    return !str.empty() && StringToTime(str, valInt, unit)
-        && ConvertTimeUnitToMS(valInt, unit, valInt);
-  }
-
-  static bool getTimeMSFromString(const std::string& str, int64_t& valInt) {
-    core::TimeUnit unit;
-    return !str.empty() && StringToTime(str, valInt, unit)
-        && ConvertTimeUnitToMS(valInt, unit, valInt);
-  }
-
   // Convert String to Integer
   template<typename T>
   static bool StringToInt(std::string input, T &output) {
@@ -560,6 +468,17 @@ class ConstrainedProperty : public std::enable_shared_from_this<ConstrainedPrope
   friend class PropertyBuilder;
 };
 
+struct ConfigurationProperty {
+  explicit ConfigurationProperty(std::string_view name,
+      gsl::not_null<PropertyValidator*> validator = gsl::make_not_null(StandardValidators::get().VALID_VALIDATOR.get()))
+    : name(name),
+      validator(validator) {
+  }
+
+  std::string_view name;
+  gsl::not_null<PropertyValidator*> validator;
+};
+
 }  // namespace core
 }  // namespace minifi
 }  // namespace nifi
