@@ -29,7 +29,9 @@
 
 #include "spdlog/common.h"
 #include "spdlog/logger.h"
+#include "utils/gsl.h"
 #include "utils/SmallString.h"
+#include "utils/meta/detected.h"
 
 namespace org {
 namespace apache {
@@ -44,7 +46,7 @@ class LoggerControl {
  public:
   LoggerControl();
 
-  bool is_enabled() const;
+  [[nodiscard]] bool is_enabled() const;
 
   void setEnabled(bool status);
 
@@ -52,45 +54,49 @@ class LoggerControl {
   std::atomic<bool> is_enabled_;
 };
 
-inline char const* conditional_conversion(std::string const& str) {
-  return str.c_str();
+template<typename Arg>
+using has_const_c_str_method = decltype(std::declval<const Arg&>().c_str());
+
+template<typename Arg>
+inline decltype(auto) conditional_stringify(Arg&& arg) {
+  if constexpr (utils::meta::is_detected_v<has_const_c_str_method, Arg> || std::is_scalar_v<std::decay_t<Arg>>) {
+    return std::forward<Arg>(arg);
+  } else if constexpr (std::is_invocable_v<Arg>) {
+    return std::forward<Arg>(arg)();
+  }
 }
 
-template<size_t N>
-inline char const* conditional_conversion(const utils::SmallString<N>& arr) {
-  return arr.c_str();
+template<typename Arg>
+inline decltype(auto) conditional_convert(const Arg& val) {
+  if constexpr (std::is_scalar_v<std::decay_t<Arg>>) {
+    return val;
+  } else if constexpr (std::is_same_v<decltype(std::declval<const Arg&>().c_str()), const char*>) {
+    return val.c_str();
+  }
 }
 
-template<typename T, typename = typename std::enable_if<
-    std::is_arithmetic<T>::value ||
-    std::is_enum<T>::value ||
-    std::is_pointer<T>::value>::type>
-inline T conditional_conversion(T t) {
-  return t;
-}
-
-template<typename ... Args>
-inline std::string format_string(int max_size, char const* format_str, Args&&... args) {
+template<typename ...Args>
+std::string format_string(int max_size, char const* format_str, const Args& ...args) {
   // try to use static buffer
   char buf[LOG_BUFFER_SIZE + 1];
-  int result = std::snprintf(buf, LOG_BUFFER_SIZE + 1, format_str, conditional_conversion(std::forward<Args>(args))...);
+  int result = std::snprintf(buf, LOG_BUFFER_SIZE + 1, format_str, conditional_convert(args)...);
   if (result < 0) {
-    return std::string("Error while formatting log message");
+    return "Error while formatting log message";
   }
   if (result <= LOG_BUFFER_SIZE) {
     // static buffer was large enough
-    return std::string(buf, result);
+    return std::string(buf, gsl::narrow<size_t>(result));
   }
   if (max_size >= 0 && max_size <= LOG_BUFFER_SIZE) {
     // static buffer was already larger than allowed, use the filled buffer
     return std::string(buf, LOG_BUFFER_SIZE);
   }
   // try to use dynamic buffer
-  size_t dynamic_buffer_size = max_size < 0 ? result : std::min(result, max_size);
+  size_t dynamic_buffer_size = max_size < 0 ? gsl::narrow<size_t>(result) : gsl::narrow<size_t>(std::min(result, max_size));
   std::vector<char> buffer(dynamic_buffer_size + 1);  // extra '\0' character
-  result = std::snprintf(buffer.data(), buffer.size(), format_str, conditional_conversion(std::forward<Args>(args))...);
+  result = std::snprintf(buffer.data(), buffer.size(), format_str, conditional_convert(args)...);
   if (result < 0) {
-    return std::string("Error while formatting log message");
+    return "Error while formatting log message";
   }
   return std::string(buffer.cbegin(), buffer.cend() - 1);  // -1 to not include the terminating '\0'
 }
@@ -152,9 +158,9 @@ class Logger : public BaseLogger {
    * @param format format string ('man printf' for syntax)
    * @warning does not check @p log or @p format for null. Caller must ensure parameters and format string lengths match
    */
-  template<typename ... Args>
-  void log_error(const char * const format, const Args& ... args) {
-    log(spdlog::level::err, format, args...);
+  template<typename ...Args>
+  void log_error(const char * const format, Args&& ...args) {
+    log(spdlog::level::err, format, std::forward<Args>(args)...);
   }
 
   /**
@@ -162,9 +168,9 @@ class Logger : public BaseLogger {
    * @param format format string ('man printf' for syntax)
    * @warning does not check @p log or @p format for null. Caller must ensure parameters and format string lengths match
    */
-  template<typename ... Args>
-  void log_warn(const char * const format, const Args& ... args) {
-    log(spdlog::level::warn, format, args...);
+  template<typename ...Args>
+  void log_warn(const char * const format, Args&& ...args) {
+    log(spdlog::level::warn, format, std::forward<Args>(args)...);
   }
 
   /**
@@ -172,9 +178,9 @@ class Logger : public BaseLogger {
    * @param format format string ('man printf' for syntax)
    * @warning does not check @p log or @p format for null. Caller must ensure parameters and format string lengths match
    */
-  template<typename ... Args>
-  void log_info(const char * const format, const Args& ... args) {
-    log(spdlog::level::info, format, args...);
+  template<typename ...Args>
+  void log_info(const char * const format, Args&& ...args) {
+    log(spdlog::level::info, format, std::forward<Args>(args)...);
   }
 
   /**
@@ -182,9 +188,9 @@ class Logger : public BaseLogger {
    * @param format format string ('man printf' for syntax)
    * @warning does not check @p log or @p format for null. Caller must ensure parameters and format string lengths match
    */
-  template<typename ... Args>
-  void log_debug(const char * const format, const Args& ... args) {
-    log(spdlog::level::debug, format, args...);
+  template<typename ...Args>
+  void log_debug(const char * const format, Args&& ...args) {
+    log(spdlog::level::debug, format, std::forward<Args>(args)...);
   }
 
   /**
@@ -192,9 +198,9 @@ class Logger : public BaseLogger {
    * @param format format string ('man printf' for syntax)
    * @warning does not check @p log or @p format for null. Caller must ensure parameters and format string lengths match
    */
-  template<typename ... Args>
-  void log_trace(const char * const format, const Args& ... args) {
-    log(spdlog::level::trace, format, args...);
+  template<typename ...Args>
+  void log_trace(const char * const format, Args&& ...args) {
+    log(spdlog::level::trace, format, std::forward<Args>(args)...);
   }
 
   void set_max_log_size(int size) {
@@ -217,15 +223,15 @@ class Logger : public BaseLogger {
   std::mutex mutex_;
 
  private:
-  template<typename ... Args>
-  inline void log(spdlog::level::level_enum level, const char * const format, const Args& ... args) {
+  template<typename ...Args>
+  inline void log(spdlog::level::level_enum level, const char* const format, Args&& ...args) {
     if (controller_ && !controller_->is_enabled())
          return;
     std::lock_guard<std::mutex> lock(mutex_);
     if (!delegate_->should_log(level)) {
       return;
     }
-    const auto str = format_string(max_log_size_.load(), format, conditional_conversion(args)...);
+    const auto str = format_string(max_log_size_.load(), format, conditional_stringify(std::forward<Args>(args))...);
     delegate_->log(level, str);
   }
 
@@ -235,15 +241,15 @@ class Logger : public BaseLogger {
   Logger& operator=(Logger const&);
 };
 
-#define LOG_DEBUG(x) LogBuilder(x.get(), logging::LOG_LEVEL::debug)
+#define LOG_DEBUG(x) LogBuilder((x).get(), org::apache::nifi::minifi::core::logging::LOG_LEVEL::debug)
 
-#define LOG_INFO(x) LogBuilder(x.get(), logging::LOG_LEVEL::info)
+#define LOG_INFO(x) LogBuilder((x).get(), org::apache::nifi::minifi::core::logging::LOG_LEVEL::info)
 
-#define LOG_TRACE(x) LogBuilder(x.get(), logging::LOG_LEVEL::trace)
+#define LOG_TRACE(x) LogBuilder((x).get(), org::apache::nifi::minifi::core::logging::LOG_LEVEL::trace)
 
-#define LOG_ERROR(x) LogBuilder(x.get(), logging::LOG_LEVEL::err)
+#define LOG_ERROR(x) LogBuilder((x).get(), org::apache::nifi::minifi::core::logging::LOG_LEVEL::err)
 
-#define LOG_WARN(x) LogBuilder(x.get(), logging::LOG_LEVEL::warn)
+#define LOG_WARN(x) LogBuilder((x).get(), org::apache::nifi::minifi::core::logging::LOG_LEVEL::warn)
 
 }  // namespace logging
 }  // namespace core
