@@ -23,8 +23,6 @@
 #include <utility>
 #include <vector>
 
-#include "core/Resource.h"
-
 #ifndef WIN32
 #include <arpa/inet.h>
 #include <sys/ioctl.h>
@@ -60,17 +58,13 @@
 #include "core/state/nodes/StateMonitor.h"
 #include "io/ClientSocket.h"
 #include "SchedulingNodes.h"
-#include "utils/OptionalUtils.h"
 #include "utils/OsUtils.h"
 #include "utils/ProcessCpuUsageTracker.h"
 #include "core/AgentIdentificationProvider.h"
+#include "utils/Export.h"
+#include "SupportedOperations.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace state {
-namespace response {
+namespace org::apache::nifi::minifi::state::response {
 
 #define GROUP_STR "org.apache.nifi.minifi"
 
@@ -92,7 +86,7 @@ class ComponentManifest : public DeviceInformation {
     std::vector<SerializedResponseNode> serialized;
     SerializedResponseNode resp;
     resp.name = "componentManifest";
-    struct Components group = BuildDescription::getClassDescriptions(getName());
+    struct Components group = build_description_.getClassDescriptions(getName());
     serializeClassDescription(group.processors_, "processors", resp);
     serializeClassDescription(group.controller_services_, "controllerServices", resp);
     serialized.push_back(resp);
@@ -138,7 +132,7 @@ class ComponentManifest : public DeviceInformation {
 
             SerializedResponseNode supportsExpressionLanguageScope;
             supportsExpressionLanguageScope.name = "expressionLanguageScope";
-            supportsExpressionLanguageScope.value = prop.second.supportsExpressionLangauge() ? "FLOWFILE_ATTRIBUTES" : "NONE";
+            supportsExpressionLanguageScope.value = prop.second.supportsExpressionLanguage() ? "FLOWFILE_ATTRIBUTES" : "NONE";
 
             SerializedResponseNode descriptorRequired;
             descriptorRequired.name = "required";
@@ -176,19 +170,20 @@ class ComponentManifest : public DeviceInformation {
               SerializedResponseNode allowed_type;
               allowed_type.name = "typeProvidedByValue";
               for (const auto &type : allowed_types) {
+                std::string class_name = utils::StringUtils::split(type, "::").back();
                 SerializedResponseNode typeNode;
                 typeNode.name = "type";
                 std::string typeClazz = type;
                 utils::StringUtils::replaceAll(typeClazz, "::", ".");
                 typeNode.value = typeClazz;
-                allowed_type.children.push_back(typeNode);
 
                 SerializedResponseNode bgroup;
                 bgroup.name = "group";
                 bgroup.value = GROUP_STR;
+
                 SerializedResponseNode artifact;
                 artifact.name = "artifact";
-                artifact.value = core::ClassLoader::getDefaultClassLoader().getGroupForClass(type);
+                artifact.value = core::ClassLoader::getDefaultClassLoader().getGroupForClass(class_name).value_or("");
                 allowed_type.children.push_back(typeNode);
                 allowed_type.children.push_back(bgroup);
                 allowed_type.children.push_back(artifact);
@@ -256,6 +251,10 @@ class ComponentManifest : public DeviceInformation {
           inputReq.name = "inputRequirement";
           inputReq.value = group.inputRequirement_;
 
+          SerializedResponseNode isSingleThreaded;
+          isSingleThreaded.name = "isSingleThreaded";
+          isSingleThreaded.value = group.isSingleThreaded_;
+
           SerializedResponseNode relationships;
           relationships.name = "supportedRelationships";
           relationships.array = true;
@@ -277,6 +276,7 @@ class ComponentManifest : public DeviceInformation {
             relationships.children.push_back(child);
           }
           desc.children.push_back(inputReq);
+          desc.children.push_back(isSingleThreaded);
           desc.children.push_back(relationships);
         }
 
@@ -311,6 +311,9 @@ class ComponentManifest : public DeviceInformation {
       response.children.push_back(type);
     }
   }
+
+ private:
+  BuildDescription build_description_;
 };
 
 class ExternalManifest : public ComponentManifest {
@@ -494,27 +497,27 @@ class AgentStatus : public StateMonitorNode {
   }
 
   SerializedResponseNode serializeComponents() const {
-    SerializedResponseNode components_node(false);
+    SerializedResponseNode components_node;
+    components_node.collapsible = false;
     components_node.name = "components";
     if (monitor_ != nullptr) {
-      auto components = monitor_->getAllComponents();
-
-      for (const auto& component : components) {
-        SerializedResponseNode component_node(false);
-        component_node.name = component->getComponentName();
+      monitor_->executeOnAllComponents([&components_node](StateController& component){
+        SerializedResponseNode component_node;
+        component_node.collapsible = false;
+        component_node.name = component.getComponentName();
 
         SerializedResponseNode uuid_node;
         uuid_node.name = "uuid";
-        uuid_node.value = std::string{component->getComponentUUID().to_string()};
+        uuid_node.value = std::string{component.getComponentUUID().to_string()};
 
         SerializedResponseNode component_status_node;
         component_status_node.name = "running";
-        component_status_node.value = component->isRunning();
+        component_status_node.value = component.isRunning();
 
         component_node.children.push_back(component_status_node);
         component_node.children.push_back(uuid_node);
         components_node.children.push_back(component_node);
-      }
+      });
     }
     return components_node;
   }
@@ -548,8 +551,8 @@ class AgentStatus : public StateMonitorNode {
 
   std::map<std::string, std::shared_ptr<core::Repository>> repositories_;
 
-  static utils::ProcessCpuUsageTracker cpu_load_tracker_;
-  static std::mutex cpu_load_tracker_mutex_;
+  MINIFIAPI static utils::ProcessCpuUsageTracker cpu_load_tracker_;
+  MINIFIAPI static std::mutex cpu_load_tracker_mutex_;
 };
 
 class AgentIdentifier {
@@ -582,13 +585,13 @@ class AgentMonitor {
     }
   }
 
-  void setStateMonitor(const std::shared_ptr<state::StateMonitor> &monitor) {
+  void setStateMonitor(state::StateMonitor* monitor) {
     monitor_ = monitor;
   }
 
  protected:
   std::map<std::string, std::shared_ptr<core::Repository>> repositories_;
-  std::shared_ptr<state::StateMonitor> monitor_;
+  state::StateMonitor* monitor_ = nullptr;
 };
 
 /**
@@ -597,83 +600,66 @@ class AgentMonitor {
 class AgentManifest : public DeviceInformation {
  public:
   AgentManifest(const std::string& name, const utils::Identifier& uuid)
-      : DeviceInformation(name, uuid) {
+    : DeviceInformation(name, uuid) {
   }
 
-  AgentManifest(const std::string &name) // NOLINT
-      : DeviceInformation(name) {
+  explicit AgentManifest(const std::string& name)
+    : DeviceInformation(name) {
   }
 
-  std::string getName() const {
+  std::string getName() const override {
     return "agentManifest";
   }
 
-  std::vector<SerializedResponseNode> serialize() {
-    static std::vector<SerializedResponseNode> serialized;
-    if (serialized.empty()) {
-      SerializedResponseNode ident;
+  void setStateMonitor(state::StateMonitor* monitor) {
+    monitor_ = monitor;
+  }
 
-      ident.name = "identifier";
-      ident.value = AgentBuild::BUILD_IDENTIFIER;
+  void setUpdatePolicyController(controllers::UpdatePolicyControllerService* update_policy_controller) {
+    update_policy_controller_ = update_policy_controller;
+  }
 
-      SerializedResponseNode type;
+  void setConfigurationReader(std::function<std::optional<std::string>(const std::string&)> configuration_reader) {
+    configuration_reader_ = std::move(configuration_reader);
+  }
 
-      type.name = "agentType";
-      type.value = "cpp";
-
-      SerializedResponseNode version;
-
-      version.name = "version";
-      version.value = AgentBuild::VERSION;
-
-      SerializedResponseNode buildInfo;
-      buildInfo.name = "buildInfo";
-
-      SerializedResponseNode build_version;
-      build_version.name = "version";
-      build_version.value = AgentBuild::VERSION;
-
-      SerializedResponseNode build_rev;
-      build_rev.name = "revision";
-      build_rev.value = AgentBuild::BUILD_REV;
-
-      SerializedResponseNode build_date;
-      build_date.name = "timestamp";
-      build_date.value = (uint64_t) std::stoull(AgentBuild::BUILD_DATE);
-
-      SerializedResponseNode compiler_command;
-      compiler_command.name = "compiler";
-      compiler_command.value = AgentBuild::COMPILER;
-
-      SerializedResponseNode compiler_flags;
-      compiler_flags.name = "flags";
-      compiler_flags.value = AgentBuild::COMPILER_FLAGS;
-
-      buildInfo.children.push_back(compiler_flags);
-      buildInfo.children.push_back(compiler_command);
-
-      buildInfo.children.push_back(build_version);
-      buildInfo.children.push_back(build_rev);
-      buildInfo.children.push_back(build_date);
-
-      Bundles bundles("bundles");
-
-      serialized.push_back(ident);
-      serialized.push_back(type);
-      serialized.push_back(buildInfo);
-      // serialize the bundle information.
-      for (auto bundle : bundles.serialize()) {
-        serialized.push_back(bundle);
-      }
-
-      SchedulingDefaults defaults("schedulingDefaults");
-
-      for (auto defaultNode : defaults.serialize()) {
-        serialized.push_back(defaultNode);
-      }
+  std::vector<SerializedResponseNode> serialize() override {
+    std::vector<SerializedResponseNode> serialized = {
+        {.name = "identifier", .value = AgentBuild::BUILD_IDENTIFIER},
+        {.name = "agentType", .value = "cpp"},
+        {.name = "buildInfo", .children = {
+            {.name = "flags", .value = AgentBuild::COMPILER_FLAGS},
+            {.name = "compiler", .value = AgentBuild::COMPILER},
+            {.name = "version", .value = AgentBuild::VERSION},
+            {.name = "revision", .value = AgentBuild::BUILD_REV},
+            {.name = "timestamp", .value = static_cast<uint64_t>(std::stoull(AgentBuild::BUILD_DATE))}
+        }}
+    };
+    {
+      auto bundles = Bundles{"bundles"}.serialize();
+      std::move(std::begin(bundles), std::end(bundles), std::back_inserter(serialized));
+    }
+    {
+      auto schedulingDefaults = SchedulingDefaults{"schedulingDefaults"}.serialize();
+      std::move(std::begin(schedulingDefaults), std::end(schedulingDefaults), std::back_inserter(serialized));
+    }
+    {
+      auto supportedOperations = [this]() {
+        SupportedOperations supported_operations("supportedOperations");
+        supported_operations.setStateMonitor(monitor_);
+        supported_operations.setUpdatePolicyController(update_policy_controller_);
+        supported_operations.setConfigurationReader(configuration_reader_);
+        return supported_operations.serialize();
+      }();
+      std::move(std::begin(supportedOperations), std::end(supportedOperations), std::back_inserter(serialized));
     }
     return serialized;
   }
+
+ private:
+  state::StateMonitor* monitor_ = nullptr;
+  controllers::UpdatePolicyControllerService* update_policy_controller_ = nullptr;
+  std::function<std::optional<std::string>(const std::string&)> configuration_reader_;
 };
 
 class AgentNode : public DeviceInformation, public AgentMonitor, public AgentIdentifier {
@@ -688,32 +674,47 @@ class AgentNode : public DeviceInformation, public AgentMonitor, public AgentIde
     setArray(false);
   }
 
+  void setUpdatePolicyController(controllers::UpdatePolicyControllerService* update_policy_controller) {
+    update_policy_controller_ = update_policy_controller;
+  }
+
+  void setConfigurationReader(std::function<std::optional<std::string>(const std::string&)> configuration_reader) {
+    configuration_reader_ = std::move(configuration_reader);
+  }
+
  protected:
-  std::vector<SerializedResponseNode> serialize() {
-    std::vector<SerializedResponseNode> serialized;
+  std::vector<SerializedResponseNode> serialize() override {
+    std::vector<SerializedResponseNode> serialized = {
+        {.name = "identifier", .value = provider_->getAgentIdentifier()},
+    };
 
-    SerializedResponseNode ident;
-
-    ident.name = "identifier";
-    ident.value = provider_->getAgentIdentifier();
-    serialized.push_back(ident);
-
-    utils::optional<std::string> agent_class = provider_->getAgentClass();
+    const auto agent_class = provider_->getAgentClass();
     if (agent_class) {
-      SerializedResponseNode agentClass;
-      agentClass.name = "agentClass";
-      agentClass.value = *agent_class;
-      serialized.push_back(agentClass);
+      serialized.push_back({.name = "agentClass", .value = *agent_class});
     }
 
+    serialized.push_back({.name = "agentManifestHash", .value = getAgentManifestHash()});
     return serialized;
   }
 
   std::vector<SerializedResponseNode> getAgentManifest() const {
-    SerializedResponseNode agentManifest;
-    agentManifest.name = "agentManifest";
-    agentManifest.children = AgentManifest{"manifest"}.serialize();
-    return std::vector<SerializedResponseNode>{ agentManifest };
+    if (agent_manifest_cache_) { return std::vector{*agent_manifest_cache_}; }
+    agent_manifest_cache_ = {.name = "agentManifest", .children = [this] {
+      AgentManifest manifest{"manifest"};
+      manifest.setStateMonitor(monitor_);
+      manifest.setUpdatePolicyController(update_policy_controller_);
+      manifest.setConfigurationReader(configuration_reader_);
+      return manifest.serialize();
+    }()};
+    agent_manifest_hash_cache_.clear();
+    return std::vector{ *agent_manifest_cache_ };
+  }
+
+  std::string getAgentManifestHash() const {
+    if (agent_manifest_hash_cache_.empty()) {
+      agent_manifest_hash_cache_ = hashResponseNodes(getAgentManifest());
+    }
+    return agent_manifest_hash_cache_;
   }
 
   std::vector<SerializedResponseNode> getAgentStatus() const {
@@ -732,6 +733,13 @@ class AgentNode : public DeviceInformation, public AgentMonitor, public AgentIde
     serialized.push_back(agentStatus);
     return serialized;
   }
+
+ private:
+  mutable std::optional<SerializedResponseNode> agent_manifest_cache_;
+  mutable std::string agent_manifest_hash_cache_;
+  controllers::UpdatePolicyControllerService* update_policy_controller_ = nullptr;
+  std::function<std::optional<std::string>(const std::string&)> configuration_reader_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AgentNode>::getLogger();
 };
 
 /**
@@ -753,7 +761,7 @@ class AgentInformation : public AgentNode {
     setArray(false);
   }
 
-  std::string getName() const {
+  std::string getName() const override {
     return "agentInfo";
   }
 
@@ -761,7 +769,7 @@ class AgentInformation : public AgentNode {
     include_agent_status_ = include;
   }
 
-  std::vector<SerializedResponseNode> serialize() {
+  std::vector<SerializedResponseNode> serialize() override {
     std::vector<SerializedResponseNode> serialized(AgentNode::serialize());
     if (include_agent_manifest_) {
       auto manifest = getAgentManifest();
@@ -779,13 +787,6 @@ class AgentInformation : public AgentNode {
   bool include_agent_status_;
 };
 
-REGISTER_RESOURCE(AgentInformation, "Node part of an AST that defines all agent information, to include the manifest, and bundle information as part of a healthy hearbeat.");
-
-}  // namespace response
-}  // namespace state
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+}  // namespace org::apache::nifi::minifi::state::response
 
 #endif  // LIBMINIFI_INCLUDE_CORE_STATE_NODES_AGENTINFORMATION_H_
