@@ -18,8 +18,6 @@
 #ifndef LIBMINIFI_INCLUDE_CORE_STATE_NODES_DEVICEINFORMATION_H_
 #define LIBMINIFI_INCLUDE_CORE_STATE_NODES_DEVICEINFORMATION_H_
 
-#include "core/Resource.h"
-
 #ifndef WIN32
 #if ( defined(__APPLE__) || defined(__MACH__) || defined(BSD))
 #include <net/if_dl.h>
@@ -39,6 +37,7 @@
 
 #else
 #pragma comment(lib, "iphlpapi.lib")
+#include <Windows.h>
 #include <iphlpapi.h>
 
 #endif
@@ -58,7 +57,9 @@
 #include "Connection.h"
 #include "io/ClientSocket.h"
 #include "utils/OsUtils.h"
+#include "utils/NetworkInterfaceInfo.h"
 #include "utils/SystemCpuUsageTracker.h"
+#include "utils/Export.h"
 
 namespace org {
 namespace apache {
@@ -122,52 +123,13 @@ class Device {
   std::vector<std::string> getIpAddresses() {
     static std::vector<std::string> ips;
     if (ips.empty()) {
-#ifndef WIN32
-      struct ifaddrs *ifaddr, *ifa;
-      if (getifaddrs(&ifaddr) == -1) {
-        perror("getifaddrs");
-        exit(EXIT_FAILURE);
-      }
-
-      for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
-        if ((strcmp("lo", ifa->ifa_name) == 0) || !(ifa->ifa_flags & (IFF_RUNNING)))
-          continue;
-        if ((ifa->ifa_addr != NULL) && (ifa->ifa_addr->sa_family == AF_INET)) {
-          ips.push_back(inet_ntoa(((struct sockaddr_in *) ifa->ifa_addr)->sin_addr));
-        }
-      }
-
-      freeifaddrs(ifaddr);
-#else
-      PIP_ADAPTER_INFO adapterPtr;
-      PIP_ADAPTER_INFO adapter = NULL;
-
-      DWORD dwRetVal = 0;
-
-      ULONG adapterLen = sizeof(IP_ADAPTER_INFO);
-      adapterPtr = reinterpret_cast<IP_ADAPTER_INFO*>(malloc(sizeof(IP_ADAPTER_INFO)));
-      if (adapterPtr == NULL) {
-        return ips;
-      }
-      if (GetAdaptersInfo(adapterPtr, &adapterLen) == ERROR_BUFFER_OVERFLOW) {
-        free(adapterPtr);
-        adapterPtr = reinterpret_cast<IP_ADAPTER_INFO*>(malloc(adapterLen));
-        if (adapterPtr == NULL) {
-          return ips;
-        }
-      }
-
-      if ((dwRetVal = GetAdaptersInfo(adapterPtr, &adapterLen)) == NO_ERROR) {
-        adapter = adapterPtr;
-        while (adapter) {
-          ips.emplace_back(adapter->IpAddressList.IpAddress.String);
-          adapter = adapter->Next;
-        }
-      }
-
-      if (adapterPtr)
-      free(adapterPtr);
-#endif
+      const auto filter = [](const utils::NetworkInterfaceInfo& interface_info) {
+        return !interface_info.isLoopback() && interface_info.isRunning();
+      };
+      auto network_interface_infos = utils::NetworkInterfaceInfo::getNetworkInterfaceInfos(filter);
+      for (const auto& network_interface_info : network_interface_infos)
+        for (const auto& ip_v4_address : network_interface_info.getIpV4Addresses())
+          ips.push_back(ip_v4_address);
     }
     return ips;
   }
@@ -477,12 +439,10 @@ class DeviceInfoNode : public DeviceInformation {
   std::string hostname_;
   std::string ip_;
   std::string device_id_;
-  static utils::SystemCpuUsageTracker cpu_load_tracker_;
-  static std::mutex cpu_load_tracker_mutex_;
+  MINIFIAPI static utils::SystemCpuUsageTracker cpu_load_tracker_;
+  MINIFIAPI static std::mutex cpu_load_tracker_mutex_;
 };
 
-REGISTER_RESOURCE(DeviceInfoNode, "Node part of an AST that defines device characteristics to the C2 protocol");
-
 }  // namespace response
 }  // namespace state
 }  // namespace minifi
