@@ -24,8 +24,6 @@
 #include <utility>
 #include <vector>
 
-#include "core/Resource.h"
-
 #if ( defined(__APPLE__) || defined(__MACH__) || defined(BSD))
 #include <net/if_dl.h>
 #include <net/if_types.h>
@@ -143,18 +141,23 @@ class FlowMonitor : public StateMonitorNode {
       : StateMonitorNode(name) {
   }
 
-  void addConnection(const std::shared_ptr<minifi::Connection> &connection) {
+  void updateConnection(minifi::Connection* connection) {
     if (nullptr != connection) {
-      connections_.insert(std::make_pair(connection->getUUIDStr(), connection));
+      connections_[connection->getUUIDStr()] = connection;
     }
   }
 
-  void setFlowVersion(std::shared_ptr<state::response::FlowVersion> flow_version) {
-    flow_version_ = flow_version;
+  void clearConnections() {
+    connections_.clear();
   }
+
+  void setFlowVersion(std::shared_ptr<state::response::FlowVersion> flow_version) {
+    flow_version_ = std::move(flow_version);
+  }
+
  protected:
   std::shared_ptr<state::response::FlowVersion> flow_version_;
-  std::map<std::string, std::shared_ptr<minifi::Connection>> connections_;
+  std::map<std::string, minifi::Connection*> connections_;
 };
 
 /**
@@ -191,11 +194,13 @@ class FlowInformation : public FlowMonitor {
     serialized.push_back(uri);
 
     if (!connections_.empty()) {
-      SerializedResponseNode queues(false);
+      SerializedResponseNode queues;
+      queues.collapsible = false;
       queues.name = "queues";
 
       for (auto &queue : connections_) {
-        SerializedResponseNode repoNode(false);
+        SerializedResponseNode repoNode;
+        repoNode.collapsible = false;
         repoNode.name = queue.second->getName();
 
         SerializedResponseNode queueUUIDNode;
@@ -230,26 +235,27 @@ class FlowInformation : public FlowMonitor {
     }
 
     if (nullptr != monitor_) {
-      auto components = monitor_->getAllComponents();
-      SerializedResponseNode componentsNode(false);
+      SerializedResponseNode componentsNode;
+      componentsNode.collapsible = false;
       componentsNode.name = "components";
 
-      for (auto component : components) {
-        SerializedResponseNode componentNode(false);
-        componentNode.name = component->getComponentName();
+      monitor_->executeOnAllComponents([&componentsNode](StateController& component){
+        SerializedResponseNode componentNode;
+        componentNode.collapsible = false;
+        componentNode.name = component.getComponentName();
 
         SerializedResponseNode uuidNode;
         uuidNode.name = "uuid";
-        uuidNode.value = std::string{component->getComponentUUID().to_string()};
+        uuidNode.value = std::string{component.getComponentUUID().to_string()};
 
         SerializedResponseNode componentStatusNode;
         componentStatusNode.name = "running";
-        componentStatusNode.value = component->isRunning();
+        componentStatusNode.value = component.isRunning();
 
         componentNode.children.push_back(componentStatusNode);
         componentNode.children.push_back(uuidNode);
         componentsNode.children.push_back(componentNode);
-      }
+      });
       serialized.push_back(componentsNode);
     }
 
@@ -259,8 +265,6 @@ class FlowInformation : public FlowMonitor {
  protected:
 };
 
-REGISTER_RESOURCE(FlowInformation, "Node part of an AST that defines the flow ID and flow URL deployed to this agent");
-
 }  // namespace response
 }  // namespace state
 }  // namespace minifi
