@@ -31,7 +31,6 @@
 #include "agent/build_description.h"
 #include "Connection.h"
 #include "core/ClassLoader.h"
-#include "core/Resource.h"
 #include "core/state/UpdateController.h"
 #include "io/ClientSocket.h"
 
@@ -54,11 +53,11 @@ class StateMonitorNode : public DeviceInformation {
         monitor_(nullptr) {
   }
 
-  void setStateMonitor(const std::shared_ptr<state::StateMonitor> &monitor) {
+  void setStateMonitor(state::StateMonitor* monitor) {
     monitor_ = monitor;
   }
  protected:
-  std::shared_ptr<state::StateMonitor> monitor_;
+  state::StateMonitor* monitor_;
 };
 
 }  // namespace response
