@@ -19,7 +19,9 @@
 #define LIBMINIFI_INCLUDE_CORE_YAML_YAMLCONFIGURATION_H_
 
 #include <memory>
+#include <optional>
 #include <string>
+#include <unordered_set>
 
 #include "core/FlowConfiguration.h"
 #include "core/logging/LoggerConfiguration.h"
@@ -31,7 +33,6 @@
 #include "utils/Id.h"
 #include "utils/StringUtils.h"
 #include "utils/file/FileSystem.h"
-#include "utils/OptionalUtils.h"
 #include "yaml-cpp/yaml.h"
 
 class YamlConfigurationTestAccessor;
@@ -42,12 +43,13 @@ namespace nifi {
 namespace minifi {
 namespace core {
 
-#define CONFIG_YAML_FLOW_CONTROLLER_KEY "Flow Controller"
-#define CONFIG_YAML_PROCESSORS_KEY "Processors"
-#define CONFIG_YAML_CONTROLLER_SERVICES_KEY "Controller Services"
-#define CONFIG_YAML_REMOTE_PROCESS_GROUP_KEY "Remote Processing Groups"
-#define CONFIG_YAML_REMOTE_PROCESS_GROUP_KEY_V3 "Remote Process Groups"
-#define CONFIG_YAML_PROVENANCE_REPORT_KEY "Provenance Reporting"
+static constexpr char const* CONFIG_YAML_FLOW_CONTROLLER_KEY = "Flow Controller";
+static constexpr char const* CONFIG_YAML_PROCESSORS_KEY = "Processors";
+static constexpr char const* CONFIG_YAML_CONTROLLER_SERVICES_KEY = "Controller Services";
+static constexpr char const* CONFIG_YAML_REMOTE_PROCESS_GROUP_KEY = "Remote Processing Groups";
+static constexpr char const* CONFIG_YAML_REMOTE_PROCESS_GROUP_KEY_V3 = "Remote Process Groups";
+static constexpr char const* CONFIG_YAML_PROVENANCE_REPORT_KEY = "Provenance Reporting";
+static constexpr char const* CONFIG_YAML_FUNNELS_KEY = "Funnels";
 
 #define YAML_CONFIGURATION_USE_REGEX
 
@@ -60,7 +62,7 @@ class YamlConfiguration : public FlowConfiguration {
  public:
   explicit YamlConfiguration(const std::shared_ptr<core::Repository>& repo, const std::shared_ptr<core::Repository>& flow_file_repo,
                              const std::shared_ptr<core::ContentRepository>& content_repo, const std::shared_ptr<io::StreamFactory>& stream_factory,
-                             const std::shared_ptr<Configure>& configuration, const utils::optional<std::string>& path = {},
+                             const std::shared_ptr<Configure>& configuration, const std::optional<std::string>& path = {},
                              const std::shared_ptr<utils::file::FileSystem>& filesystem = std::make_shared<utils::file::FileSystem>());
 
   ~YamlConfiguration() override = default;
@@ -77,7 +79,7 @@ class YamlConfiguration : public FlowConfiguration {
       logger_->log_error("Cannot instantiate flow, no config file is set.");
       throw Exception(ExceptionType::FLOW_EXCEPTION, "No config file specified");
     }
-    utils::optional<std::string> configuration = filesystem_->read(config_path_.value());
+    const auto configuration = filesystem_->read(config_path_.value());
     if (!configuration) {
       return nullptr;
     }
@@ -142,7 +144,7 @@ class YamlConfiguration : public FlowConfiguration {
    * @param component_name
    * @param yaml_section
    */
-  void validateComponentProperties(const std::shared_ptr<ConfigurableComponent> &component, const std::string &component_name, const std::string &yaml_section) const;
+  void validateComponentProperties(ConfigurableComponent& component, const std::string &component_name, const std::string &yaml_section) const;
 
  protected:
   /**
@@ -246,7 +248,18 @@ class YamlConfiguration : public FlowConfiguration {
    * @param propertiesNode the YAML::Node containing the properties
    * @param processor      the Processor to which to add the resulting properties
    */
-  void parsePropertiesNodeYaml(const YAML::Node& propertiesNode, std::shared_ptr<core::ConfigurableComponent> processor, const std::string& component_name, const std::string& yaml_section);
+  void parsePropertiesNodeYaml(const YAML::Node& propertiesNode, core::ConfigurableComponent& processor, const std::string& component_name, const std::string& yaml_section);
+
+  /**
+   * Parses the Funnels section of a configuration YAML.
+   * The resulting Funnels are added to the parent ProcessGroup.
+   *
+   * @param node   the YAML::Node containing the Funnels section
+   *                 of the configuration YAML
+   * @param parent the root node of flow configuration to which
+   *                 to add the funnels that are parsed
+   */
+  void parseFunnelsYaml(const YAML::Node& node, core::ProcessGroup* parent);
 
   /**
    * A helper function for parsing or generating optional id fields.
@@ -265,6 +278,7 @@ class YamlConfiguration : public FlowConfiguration {
    * @return         the parsed or generated UUID string
    */
   std::string getOrGenerateId(const YAML::Node& yamlNode, const std::string& idField = "id");
+  std::string getRequiredIdField(const YAML::Node& yaml_node, std::string_view yaml_section = "", std::string_view error_message = "");
 
   /**
    * This is a helper function for getting an optional value, if it exists.
@@ -287,12 +301,14 @@ class YamlConfiguration : public FlowConfiguration {
 
  private:
   PropertyValue getValidatedProcessorPropertyForDefaultTypeInfo(const core::Property& propertyFromProcessor, const YAML::Node& propertyValueNode);
-  void parsePropertyValueSequence(const std::string& propertyName, const YAML::Node& propertyValueNode, std::shared_ptr<core::ConfigurableComponent> processor);
-  void parseSingleProperty(const std::string& propertyName, const YAML::Node& propertyValueNode, std::shared_ptr<core::ConfigurableComponent> processor);
-  void parsePropertyNodeElement(const std::string& propertyName, const YAML::Node& propertyValueNode, std::shared_ptr<core::ConfigurableComponent> processor);
+  void parsePropertyValueSequence(const std::string& propertyName, const YAML::Node& propertyValueNode, core::ConfigurableComponent& processor);
+  void parseSingleProperty(const std::string& propertyName, const YAML::Node& propertyValueNode, core::ConfigurableComponent& processor);
+  void parsePropertyNodeElement(const std::string& propertyName, const YAML::Node& propertyValueNode, core::ConfigurableComponent& processor);
+  void addNewId(const std::string& uuid);
 
   std::shared_ptr<logging::Logger> logger_;
   static std::shared_ptr<utils::IdGenerator> id_generator_;
+  std::unordered_set<std::string> uuids_;
 
   /**
    * Raises a human-readable configuration error for the given configuration component/section.
