@@ -43,11 +43,10 @@ class AtomicEntryStream : public BaseStream {
       : length_(0),
         offset_(0),
         key_(key),
-        entry_(entry),
-        logger_(logging::LoggerFactory<AtomicEntryStream()>::getLogger()) {
+        entry_(entry) {
     core::repository::RepoValue<T> *value;
     if (entry_->getValue(key, &value)) {
-      length_ = value->getBufferSize();
+      length_ = value->getBuffer().size();
       entry_->decrementOwnership();
       invalid_stream_ = false;
     } else {
@@ -64,7 +63,13 @@ class AtomicEntryStream : public BaseStream {
    * Skip to the specified offset.
    * @param offset offset to which we will skip
    */
-  void seek(uint64_t offset) override;
+  void seek(size_t offset) override;
+
+
+  size_t tell() const override {
+    return offset_;
+  }
+
 
   size_t size() const override {
     return length_;
@@ -75,14 +80,14 @@ class AtomicEntryStream : public BaseStream {
    * @param buf buffer in which we extract data
    * @param buflen
    */
-  int read(uint8_t *buf, int buflen) override;
+  size_t read(gsl::span<std::byte> buf) override;
 
   /**
    * writes value to stream
    * @param value value to write
    * @param size size of value
    */
-  int write(const uint8_t *value, int size) override;
+  size_t write(const uint8_t *value, size_t size) override;
 
  private:
   size_t length_;
@@ -93,7 +98,7 @@ class AtomicEntryStream : public BaseStream {
   std::recursive_mutex entry_lock_;
 
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<AtomicEntryStream>::getLogger();
 };
 
 template<typename T>
@@ -103,56 +108,54 @@ AtomicEntryStream<T>::~AtomicEntryStream() {
 }
 
 template<typename T>
-void AtomicEntryStream<T>::seek(uint64_t offset) {
+void AtomicEntryStream<T>::seek(size_t offset) {
   std::lock_guard<std::recursive_mutex> lock(entry_lock_);
   offset_ = gsl::narrow<size_t>(offset);
 }
 
+
 // data stream overrides
 template<typename T>
-int AtomicEntryStream<T>::write(const uint8_t *value, int size) {
-  gsl_Expects(size >= 0);
-  if (size == 0) {
-    return 0;
-  }
-  if (nullptr != value && !invalid_stream_) {
-    std::lock_guard<std::recursive_mutex> lock(entry_lock_);
-    if (entry_->insert(key_, const_cast<uint8_t*>(value), size)) {
-      offset_ += size;
-      if (offset_ > length_) {
-        length_ = offset_;
-      }
-      return size;
+size_t AtomicEntryStream<T>::write(const uint8_t *value, size_t size) {
+  if (size == 0) return 0;
+  if (!value || invalid_stream_) return STREAM_ERROR;
+  const auto out_span = gsl::make_span(value, size).as_span<const std::byte>();
+  std::lock_guard<std::recursive_mutex> lock(entry_lock_);
+  if (entry_->insert(key_, out_span)) {
+    offset_ += size;
+    if (offset_ > length_) {
+      length_ = offset_;
     }
+    return size;
   }
-  return -1;
+  return STREAM_ERROR;
 }
 
 template<typename T>
-int AtomicEntryStream<T>::read(uint8_t *buf, int buflen) {
-  gsl_Expects(buflen >= 0);
-  if (buflen == 0) {
+size_t AtomicEntryStream<T>::read(gsl::span<std::byte> buf) {
+  if (buf.empty()) {
     return 0;
   }
-  if (nullptr != buf && !invalid_stream_) {
+  if (!invalid_stream_) {
     std::lock_guard<std::recursive_mutex> lock(entry_lock_);
-    int len = buflen;
+    auto len = buf.size();
     core::repository::RepoValue<T> *value;
     if (entry_->getValue(key_, &value)) {
-      if (offset_ + len > value->getBufferSize()) {
-        len = gsl::narrow<int>(value->getBufferSize()) - gsl::narrow<int>(offset_);
+      if (offset_ + len > value->getBuffer().size()) {
+        len = value->getBuffer().size() - offset_;
         if (len <= 0) {
           entry_->decrementOwnership();
           return 0;
         }
       }
-      std::memcpy(buf, reinterpret_cast<uint8_t*>(const_cast<uint8_t*>(value->getBuffer()) + offset_), len);
+      const auto src_buffer = value->getBuffer().subspan(offset_, len);
+      std::memcpy(buf.data(), src_buffer.data(), src_buffer.size());
       offset_ += len;
       entry_->decrementOwnership();
       return len;
     }
   }
-  return -1;
+  return STREAM_ERROR;
 }
 
 }  // namespace io
