@@ -71,14 +71,13 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
   /*!
    * Create a new control protocol
    */
-  RawSiteToSiteClient(std::unique_ptr<SiteToSitePeer> peer) // NOLINT
-      : logger_(logging::LoggerFactory<RawSiteToSiteClient>::getLogger()) {
+  RawSiteToSiteClient(std::unique_ptr<SiteToSitePeer> peer) { // NOLINT
     peer_ = std::move(peer);
     _batchSize = 0;
     _batchCount = 0;
-    _batchDuration = 0;
-    _batchSendNanos = 5000000000;  // 5 seconds
-    _timeOut = 30000;  // 30 seconds
+    _batchDuration = std::chrono::seconds(0);
+    _batchSendNanos = std::chrono::seconds(5);
+    _timeout = std::chrono::seconds(30);
     _supportedVersion[0] = 5;
     _supportedVersion[1] = 4;
     _supportedVersion[2] = 3;
@@ -91,7 +90,7 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
     _currentCodecVersionIndex = 0;
   }
   // Destructor
-  virtual ~RawSiteToSiteClient() {
+  ~RawSiteToSiteClient() override {
     tearDown();
   }
 
@@ -105,22 +104,22 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
     _batchCount = count;
   }
   // setBatchDuration
-  void setBatchDuration(uint64_t duration) {
+  void setBatchDuration(std::chrono::milliseconds duration) {
     _batchDuration = duration;
   }
-  // setTimeOut
-  void setTimeOut(uint64_t time) {
-    _timeOut = time;
+  // setTimeout
+  void setTimeout(std::chrono::milliseconds time) {
+    _timeout = time;
     if (peer_)
-      peer_->setTimeOut(time);
+      peer_->setTimeout(time);
   }
 
   /**
    * Provides a reference to the time out
    * @returns timeout
    */
-  uint64_t getTimeOut() const {
-    return _timeOut;
+  std::chrono::milliseconds getTimeout() const {
+    return _timeout;
   }
 
   // getResourceName
@@ -133,7 +132,7 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
   }
 
   // get peerList
-  virtual bool getPeerList(std::vector<PeerStatus> &peer);
+  bool getPeerList(std::vector<PeerStatus> &peer) override;
   // negotiateCodec
   virtual bool negotiateCodec();
   // initiateResourceNegotiation
@@ -141,7 +140,7 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
   // initiateCodecResourceNegotiation
   virtual bool initiateCodecResourceNegotiation();
   // tearDown
-  virtual void tearDown();
+  void tearDown() override;
   // write Request Type
   virtual int writeRequestType(RequestType type);
   // read Request Type
@@ -152,38 +151,38 @@ class RawSiteToSiteClient : public sitetosite::SiteToSiteClient {
   // write respond
   virtual int writeRespond(const std::shared_ptr<Transaction> &transaction, RespondCode code, std::string message);
   // getRespondCodeContext
-  virtual RespondCodeContext *getRespondCodeContext(RespondCode code) {
+  RespondCodeContext *getRespondCodeContext(RespondCode code) override {
     return SiteToSiteClient::getRespondCodeContext(code);
   }
 
   // Creation of a new transaction, return the transaction ID if success,
   // Return NULL when any error occurs
-  virtual std::shared_ptr<Transaction> createTransaction(TransferDirection direction);
+  std::shared_ptr<Transaction> createTransaction(TransferDirection direction) override;
 
   //! Transfer string for the process session
-  virtual bool transmitPayload(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session, const std::string &payload,
-                               std::map<std::string, std::string> attributes);
+  bool transmitPayload(const std::shared_ptr<core::ProcessContext> &context, const std::shared_ptr<core::ProcessSession> &session, const std::string &payload,
+      std::map<std::string, std::string> attributes) override;
 
   // bootstrap the protocol to the ready for transaction state by going through the state machine
-  virtual bool bootstrap();
+  bool bootstrap() override;
 
  protected:
   // establish
-  virtual bool establish();
+  bool establish() override;
   // handShake
   virtual bool handShake();
 
  private:
   // Logger
-  std::shared_ptr<logging::Logger> logger_;
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<RawSiteToSiteClient>::getLogger();
   // Batch Count
   std::atomic<uint64_t> _batchCount;
   // Batch Size
   std::atomic<uint64_t> _batchSize;
   // Batch Duration in msec
-  std::atomic<uint64_t> _batchDuration;
+  std::atomic<std::chrono::milliseconds> _batchDuration;
   // Timeout in msec
-  std::atomic<uint64_t> _timeOut;
+  std::atomic<std::chrono::milliseconds> _timeout;
 
   // commsIdentifier
   utils::Identifier _commsIdentifier;
