@@ -17,12 +17,13 @@
 
 #pragma once
 
-#include <string>
 #include <cstring>
+#include <optional>
 #include <set>
+#include <string>
 
-#include "OptionalUtils.h"
 #include "StringUtils.h"
+#include "Macro.h"
 
 namespace org {
 namespace apache {
@@ -30,65 +31,18 @@ namespace nifi {
 namespace minifi {
 namespace utils {
 
-#define COMMA(...) ,
-#define MSVC_HACK(x) x
-
-#define PICK_(_1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, ...) _15
-#define COUNT(...) \
-  MSVC_HACK(PICK_(__VA_ARGS__, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0))
-
-#define CONCAT_(a, b) a ## b
-#define CONCAT(a, b) CONCAT_(a, b)
-
-#define CALL(Fn, ...) MSVC_HACK(Fn(__VA_ARGS__))
-#define SPREAD(...) __VA_ARGS__
-
-#define FOR_EACH(fn, delim, ARGS) \
-  CALL(CONCAT(FOR_EACH_, COUNT ARGS), fn, delim, SPREAD ARGS)
-
-#define FOR_EACH_0(...)
-#define FOR_EACH_1(fn, delim, _1) \
-  fn(_1)
-#define FOR_EACH_2(fn, delim, _1, _2) \
-  fn(_1) delim() fn(_2)
-#define FOR_EACH_3(fn, delim, _1, _2, _3) \
-  fn(_1) delim() FOR_EACH_2(fn, delim, _2, _3)
-#define FOR_EACH_4(fn, delim, _1, _2, _3, _4) \
-  fn(_1) delim() FOR_EACH_3(fn, delim, _2, _3, _4)
-#define FOR_EACH_5(fn, delim, _1, _2, _3, _4, _5) \
-  fn(_1) delim() FOR_EACH_4(fn, delim, _2, _3, _4, _5)
-#define FOR_EACH_6(fn, delim, _1, _2, _3, _4, _5, _6) \
-  fn(_1) delim() FOR_EACH_5(fn, delim, _2, _3, _4, _5, _6)
-#define FOR_EACH_7(fn, delim, _1, _2, _3, _4, _5, _6, _7) \
-  fn(_1) delim() FOR_EACH_6(fn, delim, _2, _3, _4, _5, _6, _7)
-#define FOR_EACH_8(fn, delim, _1, _2, _3, _4, _5, _6, _7, _8) \
-  fn(_1) delim() FOR_EACH_7(fn, delim, _2, _3, _4, _5, _6, _7, _8)
-#define FOR_EACH_9(fn, delim, _1, _2, _3, _4, _5, _6, _7, _8, _9) \
-  fn(_1) delim() FOR_EACH_8(fn, delim, _2, _3, _4, _5, _6, _7, _8, _9)
-#define FOR_EACH_10(fn, delim, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10) \
-  fn(_1) delim() FOR_EACH_9(fn, delim, _2, _3, _4, _5, _6, _7, _8, _9, _10)
-#define FOR_EACH_11(fn, delim, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11) \
-  fn(_1) delim() FOR_EACH_10(fn, delim, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11)
-#define FOR_EACH_12(fn, delim, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12) \
-  fn(_1) delim() FOR_EACH_11(fn, delim, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12)
-
-#define FIRST_(a, b) a
-#define FIRST(x, ...) FIRST_ x
-#define SECOND_(a, b) b
-#define SECOND(x, ...) SECOND_ x
-#define NOTHING()
-
 #define INCLUDE_BASE_FIELD(x) \
   x = Base::x
 
+// [[maybe_unused]] on public members to avoid warnings when used inside an anonymous namespace
 #define SMART_ENUM_BODY(Clazz, ...) \
     constexpr Clazz(Type value = static_cast<Type>(-1)) : value_{value} {} \
-    Clazz(const std::string& str) : value_{parse(str.c_str()).value_} {} \
-    Clazz(const char* str) : value_{parse(str).value_} {} \
+    explicit Clazz(const std::string& str) : value_{parse(str.c_str()).value_} {} \
+    explicit Clazz(const char* str) : value_{parse(str).value_} {} \
    private: \
     Type value_; \
    public: \
-    Type value() const { \
+    [[maybe_unused]] Type value() const { \
       return value_; \
     } \
     struct detail : Base::detail { \
@@ -114,35 +68,35 @@ namespace utils {
       } \
     }; \
     static constexpr int length = Base::length + COUNT(__VA_ARGS__); \
-    friend const char* toString(Type a) { \
+    [[maybe_unused]] friend const char* toString(Type a) { \
       return detail::toStringImpl(a, #Clazz); \
     } \
-    const char* toString() const { \
+    [[maybe_unused]] const char* toString() const { \
       return detail::toStringImpl(value_, #Clazz); \
     } \
-    const char* toStringOr(const char* fallback) const { \
+    [[maybe_unused]] const char* toStringOr(const char* fallback) const { \
       if (*this) { \
         return toString(); \
       } \
       return fallback; \
     } \
-    static std::set<std::string> values() { \
+    [[maybe_unused]] static std::set<std::string> values() { \
       return detail::values(); \
     } \
-    friend bool operator==(Clazz lhs, Clazz rhs) { \
+    [[maybe_unused]] friend bool operator==(Clazz lhs, Clazz rhs) { \
       return lhs.value_ == rhs.value_; \
     } \
-    friend bool operator!=(Clazz lhs, Clazz rhs) { \
+    [[maybe_unused]] friend bool operator!=(Clazz lhs, Clazz rhs) { \
       return lhs.value_ != rhs.value_; \
     } \
-    friend bool operator<(Clazz lhs, Clazz rhs) { \
+    [[maybe_unused]] friend bool operator<(Clazz lhs, Clazz rhs) { \
       return lhs.value_ < rhs.value_;\
     } \
-    explicit operator bool() const { \
+    [[maybe_unused]] explicit operator bool() const { \
       int idx = static_cast<int>(value_); \
       return 0 <= idx && idx < length; \
     } \
-    static Clazz parse(const char* str, const ::org::apache::nifi::minifi::utils::optional<Clazz>& fallback = {}, bool caseSensitive = true) { \
+    [[maybe_unused]] static Clazz parse(const char* str, const ::std::optional<Clazz>& fallback = {}, bool caseSensitive = true) { \
       for (int idx = 0; idx < length; ++idx) { \
         if (::org::apache::nifi::minifi::utils::StringUtils::equals(str, detail::toStringImpl(static_cast<Type>(idx), #Clazz), caseSensitive)) \
           return static_cast<Type>(idx); \
@@ -153,7 +107,7 @@ namespace utils {
       throw std::runtime_error(std::string("Cannot convert \"") + str + "\" to " #Clazz); \
     } \
     template<typename T, typename = typename std::enable_if<std::is_base_of<typename T::detail, detail>::value>::type> \
-    T cast() const { \
+    [[maybe_unused]] T cast() const { \
       if (0 <= value_ && value_ < T::length) { \
         return static_cast<typename T::Type>(value_); \
       } \
