@@ -29,6 +29,7 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 #include <vector>
 
@@ -36,7 +37,6 @@
 #include "controllers/SSLContextService.h"
 #include "core/Deprecated.h"
 #include "utils/gsl.h"
-#include "utils/OptionalUtils.h"
 
 namespace org {
 namespace apache {
@@ -59,7 +59,7 @@ struct HTTPUploadCallback {
   }
   std::mutex mutex;
   std::atomic<bool> stop;
-  ByteInputCallBack *ptr;
+  ByteInputCallback *ptr;
   size_t pos;
 
   size_t getPos() {
@@ -236,7 +236,7 @@ class HTTPRequestResponse {
         if (len <= 0) {
           return 0;
         }
-        char *ptr = callback->ptr->getBuffer(callback->getPos());
+        auto *ptr = callback->ptr->getBuffer(callback->getPos());
 
         if (ptr == nullptr) {
           return 0;
@@ -378,10 +378,10 @@ class URL {
  private:
   std::string protocol_;
   std::string host_;
-  utils::optional<int> port_;
-  utils::optional<std::string> path_;
+  std::optional<int> port_;
+  std::optional<std::string> path_;
   bool is_valid_ = false;
-  std::shared_ptr<logging::Logger> logger_ = logging::LoggerFactory<URL>::getLogger();
+  std::shared_ptr<core::logging::Logger> logger_ = core::logging::LoggerFactory<URL>::getLogger();
 };
 
 }  // namespace utils
