@@ -17,34 +17,27 @@
 #ifndef LIBMINIFI_INCLUDE_UTILS_STRINGUTILS_H_
 #define LIBMINIFI_INCLUDE_UTILS_STRINGUTILS_H_
 
-#include <string>
-#include <utility>
-#include <iostream>
+#include <algorithm>
 #include <cstring>
 #include <functional>
+#include <iostream>
+#include <map>
+#include <optional>
+#include <sstream>
+#include <string>
+#include <string_view>
+#include <type_traits>
+#include <utility>
+#include <vector>
 #ifdef WIN32
   #include <cwctype>
   #include <cctype>
 #endif
-#include <algorithm>
-#include <sstream>
-#include <vector>
-#include <map>
-#include <type_traits>
 #include "utils/FailurePolicy.h"
-#include "utils/GeneralUtils.h"
 #include "utils/gsl.h"
-#include "utils/OptionalUtils.h"
+#include "utils/meta/detected.h"
 
-#if defined(WIN32) || (__cplusplus >= 201103L && (!defined(__GLIBCXX__) || (__cplusplus >= 201402L) ||  (defined(_GLIBCXX_RELEASE) && _GLIBCXX_RELEASE > 4)))
-#define HAVE_REGEX_CPP 1
-#else
-#define HAVE_REGEX_CPP 0
-#endif
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
+namespace org::apache::nifi::minifi {
 namespace utils {
 
 template<class Char>
@@ -65,6 +58,9 @@ struct string_traits<wchar_t>{
   }
 };
 
+struct as_string_tag_t {};
+inline constexpr as_string_tag_t as_string;
+
 /**
  * Stateless String utility class.
  *
@@ -79,11 +75,16 @@ class StringUtils {
    * @param input input string
    * @returns an optional of a boolean: true if the string is "true" (ignoring case), false if it is "false" (ignoring case), nullopt for any other value
    */
-  static utils::optional<bool> toBool(const std::string& input);
+  static std::optional<bool> toBool(const std::string& input);
 
-  static std::string toLower(std::string str);
+  static std::string toLower(std::string_view str);
 
-  // Trim String utils
+  /**
+   * Strips the line ending (\n or \r\n) from the end of the input line.
+   * @param input_line
+   * @return (stripped line, line ending) pair
+   */
+  static std::pair<std::string, std::string> chomp(const std::string& input_line);
 
   /**
    * Trims a string left to right
@@ -113,6 +114,10 @@ class StringUtils {
     return s;
   }
 
+  static std::string_view trim(std::string_view sv);
+
+  static std::string_view trim(const char* str);
+
   /**
    * Compares strings by lower casing them.
    */
@@ -135,8 +140,20 @@ class StringUtils {
     return std::equal(right, right + right_len, left, [](unsigned char lc, unsigned char rc) { return std::tolower(lc) == std::tolower(rc); });
   }
 
+  static inline bool equals(const std::string_view& left, const std::string_view& right, bool case_sensitive = true) {
+    if (case_sensitive) {
+      return left == right;
+    }
+    if (left.length() != right.length()) {
+      return false;
+    }
+    return std::equal(left.begin(), left.end(), right.begin(), [](unsigned char lc, unsigned char rc) { return std::tolower(lc) == std::tolower(rc); });
+  }
+
   static std::vector<std::string> split(const std::string &str, const std::string &delimiter);
+  static std::vector<std::string> splitRemovingEmpty(const std::string& str, const std::string& delimiter);
   static std::vector<std::string> splitAndTrim(const std::string &str, const std::string &delimiter);
+  static std::vector<std::string> splitAndTrimRemovingEmpty(const std::string& str, const std::string& delimiter);
 
   /**
    * Converts a string to a float
@@ -152,22 +169,24 @@ class StringUtils {
 
   static std::string& replaceAll(std::string& source_string, const std::string &from_string, const std::string &to_string);
 
-  inline static bool endsWithIgnoreCase(const std::string &value, const std::string & endString) {
-    if (endString.size() > value.size())
+  inline static bool startsWith(const std::string_view& value, const std::string_view& start, bool case_sensitive = true) {
+    if (start.length() > value.length()) {
       return false;
-    return std::equal(endString.rbegin(), endString.rend(), value.rbegin(), [](unsigned char lc, unsigned char rc) {return tolower(lc) == tolower(rc);});
+    }
+    if (case_sensitive) {
+      return std::equal(start.begin(), start.end(), value.begin());
+    }
+    return std::equal(start.begin(), start.end(), value.begin(), [](unsigned char lc, unsigned char rc) {return tolower(lc) == tolower(rc);});
   }
 
-  inline static bool startsWith(const std::string& value, const std::string& start_string) {
-    if (start_string.size() > value.size())
+  inline static bool endsWith(const std::string_view& value, const std::string_view& end, bool case_sensitive = true) {
+    if (end.length() > value.length()) {
       return false;
-    return std::equal(start_string.begin(), start_string.end(), value.begin());
-  }
-
-  inline static bool endsWith(const std::string& value, const std::string& end_string) {
-    if (end_string.size() > value.size())
-      return false;
-    return std::equal(end_string.rbegin(), end_string.rend(), value.rbegin());
+    }
+    if (case_sensitive) {
+      return std::equal(end.rbegin(), end.rend(), value.rbegin());
+    }
+    return std::equal(end.rbegin(), end.rend(), value.rbegin(), [](unsigned char lc, unsigned char rc) {return tolower(lc) == tolower(rc);});
   }
 
   inline static std::string hex_ascii(const std::string& in) {
@@ -193,47 +212,10 @@ class StringUtils {
   template<typename CharT>
   static size_t size(const CharT* str) noexcept { return std::char_traits<CharT>::length(str); }
 
+  template<typename CharT>
+  static size_t size(const std::basic_string_view<CharT>& str) noexcept { return str.size(); }
+
   struct detail {
-    // add all args
-    template<typename... SizeT>
-    static size_t sum(SizeT... ns) {
-      size_t result = 0;
-      (void)(std::initializer_list<size_t>{( result += ns )...});
-      return result;  // (ns + ...)
-    }
-
-#ifndef _MSC_VER
-
-    // partial detection idiom impl, from cppreference.com
-    struct nonesuch{};
-
-    template<typename Default, typename Void, template<class...> class Op, typename... Args>
-    struct detector {
-      using value_t = std::false_type;
-      using type = Default;
-    };
-
-    template<typename Default, template<class...> class Op, typename... Args>
-    struct detector<Default, void_t<Op<Args...>>, Op, Args...> {
-      using value_t = std::true_type;
-      using type = Op<Args...>;
-    };
-
-    template<template<class...> class Op, typename... Args>
-    using is_detected = typename detector<nonesuch, void, Op, Args...>::value_t;
-
-    // and operation for boolean template argument packs
-    template<bool...>
-    struct and_;
-
-    template<bool Head, bool... Tail>
-    struct and_<Head, Tail...> : std::integral_constant<bool, Head && and_<Tail...>::value>
-    {};
-
-    template<bool B>
-    struct and_<B> : std::integral_constant<bool, B>
-    {};
-
     // implementation detail of join_pack
     template<typename CharT>
     struct str_detector {
@@ -242,18 +224,13 @@ class StringUtils {
     };
 
     template<typename ResultT, typename CharT, typename... Strs>
-    using valid_string_pack_t = typename std::enable_if<and_<is_detected<str_detector<CharT>::template valid_string_t, Strs>::value...>::value, ResultT>::type;
-#else
-    // MSVC is broken without /permissive-
-  template<typename ResultT, typename...>
-  using valid_string_pack_t = ResultT;
-#endif
+    using valid_string_pack_t = std::enable_if_t<(meta::is_detected_v<str_detector<CharT>::template valid_string_t, Strs> && ...), ResultT>;
 
     template<typename CharT, typename... Strs, valid_string_pack_t<void, CharT, Strs...>* = nullptr>
     static std::basic_string<CharT> join_pack(const Strs&... strs) {
       std::basic_string<CharT> result;
-      result.reserve(sum(size(strs)...));
-      (void)(std::initializer_list<int>{( result.append(strs) , 0)...});
+      result.reserve((size(strs) + ...));
+      (result.append(strs), ...);
       return result;
     }
   }; /* struct detail */
@@ -278,6 +255,12 @@ class StringUtils {
     return detail::join_pack<CharT>(head, tail...);
   }
 
+  template<typename CharT, typename... Strs>
+  static detail::valid_string_pack_t<std::basic_string<CharT>, CharT, Strs...>
+  join_pack(const std::basic_string_view<CharT>& head, const Strs&... tail) {
+    return detail::join_pack<CharT>(head, tail...);
+  }
+
   /**
    * Concatenates strings stored in an arbitrary container using the provided separator.
    * @tparam TChar char type of the string (char or wchar_t)
@@ -375,7 +358,7 @@ class StringUtils {
    * @param hex_length the length of hex
    * @return true on success
    */
-  static bool from_hex(uint8_t* data, size_t* data_length, const char* hex, size_t hex_length);
+  static bool from_hex(std::byte* data, size_t* data_length, std::string_view hex);
 
   /**
    * Hexdecodes a string
@@ -383,18 +366,12 @@ class StringUtils {
    * @param hex_length the length of hex
    * @return the vector containing the hexdecoded bytes
    */
-  static std::vector<uint8_t> from_hex(const char* hex, size_t hex_length);
-
-  /**
-   * Hexdecodes a string
-   * @param hex the hexencoded string
-   * @return the hexdecoded string
-   */
-  inline static std::string from_hex(const std::string& hex) {
-    auto data = from_hex(hex.data(), hex.length());
-    return std::string(reinterpret_cast<char*>(data.data()), data.size());
+  static std::vector<std::byte> from_hex(std::string_view hex);
+  static std::string from_hex(std::string_view hex, as_string_tag_t) {
+    return utils::span_to<std::string>(gsl::make_span(from_hex(hex)).as_span<const char>());
   }
 
+
   /**
    * Hexencodes bytes and writes the result to hex
    * @param hex the output buffer where the hexencoded bytes will be written. Must be at least length * 2 bytes long.
@@ -403,7 +380,7 @@ class StringUtils {
    * @param uppercase whether the hexencoded string should be upper case
    * @return the size of hexencoded bytes
    */
-  static size_t to_hex(char* hex, const uint8_t* data, size_t length, bool uppercase);
+  static size_t to_hex(char* hex, gsl::span<const std::byte> data_to_be_transformed, bool uppercase);
 
   /**
    * Creates a hexencoded string from data
@@ -412,7 +389,7 @@ class StringUtils {
    * @param uppercase whether the hexencoded string should be upper case
    * @return the hexencoded string
    */
-  static std::string to_hex(const uint8_t* data, size_t length, bool uppercase = false);
+  static std::string to_hex(gsl::span<const std::byte> data_to_be_transformed, bool uppercase = false);
 
   /**
    * Hexencodes a string
@@ -420,19 +397,10 @@ class StringUtils {
    * @param uppercase whether the hexencoded string should be upper case
    * @return the hexencoded string
    */
-  inline static std::string to_hex(const std::string& str, bool uppercase = false) {
-    return to_hex(reinterpret_cast<const uint8_t*>(str.data()), str.length(), uppercase);
+  inline static std::string to_hex(std::string_view str, bool uppercase = false) {
+    return to_hex(gsl::make_span(str).as_span<const std::byte>(), uppercase);
   }
 
-  /**
-   * Hexencodes a vector of bytes
-   * @param data the vector of bytes to be hexencoded
-   * @param uppercase whether the hexencoded string should be upper case
-   * @return the hexencoded string
-   */
-  inline static std::string to_hex(const std::vector<uint8_t>& data, bool uppercase = false) {
-    return to_hex(data.data(), data.size(), uppercase);
-  }
 
   /**
    * Decodes the Base64 encoded string into data
@@ -442,7 +410,7 @@ class StringUtils {
    * @param base64_length the length of base64
    * @return true on success
    */
-  static bool from_base64(uint8_t* data, size_t* data_length, const char* base64, size_t base64_length);
+  static bool from_base64(std::byte* data, size_t* data_length, std::string_view base64);
 
   /**
    * Base64 decodes a string
@@ -450,38 +418,29 @@ class StringUtils {
    * @param base64_length the length of base64
    * @return the vector containing the decoded bytes
    */
-  static std::vector<uint8_t> from_base64(const char* base64, size_t base64_length);
-
-  /**
-   * Base64 decodes a string
-   * @param base64 the Base64 encoded string
-   * @return the decoded string
-   */
-  inline static std::string from_base64(const std::string& base64) {
-    auto data = from_base64(base64.data(), base64.length());
-    return std::string(reinterpret_cast<char*>(data.data()), data.size());
+  static std::vector<std::byte> from_base64(std::string_view base64);
+  static std::string from_base64(std::string_view base64, as_string_tag_t) {
+    return utils::span_to<std::string>(gsl::make_span(from_base64(base64)).as_span<const char>());
   }
 
   /**
    * Base64 encodes bytes and writes the result to base64
    * @param base64 the output buffer where the Base64 encoded bytes will be written. Must be at least (base64_length / 3 + 1) * 4 bytes long.
-   * @param data the bytes to be Base64 encoded
-   * @param length the length of data. Must not be larger than std::numeric_limits<size_t>::max() * 3 / 4 - 3
+   * @param raw_data the bytes to be Base64 encoded. Must not be larger than std::numeric_limits<size_t>::max() * 3 / 4 - 3
    * @param url if true, the URL-safe Base64 encoding will be used
    * @param padded if true, padding is added to the Base64 encoded string
    * @return the size of Base64 encoded bytes
    */
-  static size_t to_base64(char* base64, const uint8_t* data, size_t length, bool url, bool padded);
+  static size_t to_base64(char* base64, gsl::span<const std::byte> data_to_be_transformed, bool url, bool padded);
 
   /**
    * Creates a Base64 encoded string from data
    * @param data the bytes to be Base64 encoded
-   * @param length the length of the data
    * @param url if true, the URL-safe Base64 encoding will be used
    * @param padded if true, padding is added to the Base64 encoded string
    * @return the Base64 encoded string
    */
-  static std::string to_base64(const uint8_t* data, size_t length, bool url = false, bool padded = true);
+  static std::string to_base64(gsl::span<const std::byte> data_to_be_transformed, bool url = false, bool padded = true);
 
   /**
    * Base64 encodes a string
@@ -490,19 +449,8 @@ class StringUtils {
    * @param padded if true, padding is added to the Base64 encoded string
    * @return the Base64 encoded string
    */
-  inline static std::string to_base64(const std::string& str, bool url = false, bool padded = true) {
-    return to_base64(reinterpret_cast<const uint8_t*>(str.data()), str.length(), url, padded);
-  }
-
-  /**
-   * Base64 encodes a string
-   * @param str the string to be Base64 encoded
-   * @param url if true, the URL-safe Base64 encoding will be used
-   * @param padded if true, padding is added to the Base64 encoded string
-   * @return the Base64 encoded string
-   */
-  inline static std::string to_base64(const std::vector<uint8_t>& str, bool url = false, bool padded = true) {
-    return to_base64(str.data(), str.size(), url, padded);
+  inline static std::string to_base64(std::string_view str, bool url = false, bool padded = true) {
+    return to_base64(gsl::make_span(str).as_span<const std::byte>(), url, padded);
   }
 
   static std::string replaceMap(std::string source_string, const std::map<std::string, std::string> &replace_map);
@@ -537,80 +485,21 @@ class StringUtils {
     return str;
   }
 
+  static std::string escapeUnprintableBytes(gsl::span<const std::byte> data);
+
+  /**
+   * Returns whether sequence of patterns are found in given string in their incoming order
+   * Non-regex search!
+   * @param str string to search in
+   * @param patterns sequence of patterns to search
+   * @return success of string sequence matching
+   */
+  static bool matchesSequence(std::string_view str, const std::vector<std::string>& patterns);
+
  private:
-  inline static char nibble_to_hex(uint8_t nibble, bool uppercase) {
-    if (nibble < 10) {
-      return '0' + nibble;
-    } else {
-      return (uppercase ? 'A' : 'a') + nibble - 10;
-    }
-  }
-
-  inline static void base64_digits_to_bytes(const uint8_t digits[4], uint8_t* bytes) {
-    bytes[0] = digits[0] << 2 | digits[1] >> 4;
-    bytes[1] = (digits[1] & 0x0f) << 4 | digits[2] >> 2;
-    bytes[2] = (digits[2] & 0x03) << 6 | digits[3];
-  }
-
-  static constexpr uint8_t SKIP = 0xff;
-  static constexpr uint8_t ILGL = 0xfe;
-  static constexpr uint8_t PDNG = 0xfd;
-  static constexpr uint8_t hex_lut[128] =
-      {SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
-       0x08, 0x09, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP,
-       SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP, SKIP};
-
-  static constexpr const char base64_enc_lut[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-  static constexpr const char base64_url_enc_lut[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
-  static constexpr uint8_t base64_dec_lut[128] =
-      {ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL,
-       ILGL, ILGL, SKIP, ILGL, ILGL, SKIP, ILGL, ILGL,
-       ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL,
-       ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL,
-       ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL, ILGL,
-       ILGL, ILGL, ILGL, 0x3e, ILGL, 0x3e, ILGL, 0x3f,
-       0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b,
-       0x3c, 0x3d, ILGL, ILGL, ILGL, PDNG, ILGL, ILGL,
-       ILGL, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
-       0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
-       0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16,
-       0x17, 0x18, 0x19, ILGL, ILGL, ILGL, ILGL, 0x3f,
-       ILGL, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20,
-       0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
-       0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30,
-       0x31, 0x32, 0x33, ILGL, ILGL, ILGL, ILGL, ILGL};
 };
 
 }  // namespace utils
-
-namespace core {
-enum TimeUnit {
-  DAY,
-  HOUR,
-  MINUTE,
-  SECOND,
-  MILLISECOND,
-  MICROSECOND,
-  NANOSECOND
-};
-
-}  // namespace core
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+}  // namespace org::apache::nifi::minifi
 
 #endif  // LIBMINIFI_INCLUDE_UTILS_STRINGUTILS_H_
