@@ -18,10 +18,11 @@
 
 #pragma once
 
-#include <string>
+#include <chrono>
+#include <fstream>
 #include <memory>
+#include <string>
 
-#include "../../test/TestBase.h"
 #include "utils/file/FileUtils.h"
 #include "utils/Environment.h"
 #include "utils/Id.h"
@@ -33,19 +34,6 @@ namespace nifi {
 namespace minifi {
 namespace utils {
 
-std::string createTempDir(TestController* testController, char* format = nullptr) {
-  std::string temp_dir;
-  if (format == nullptr) {
-    char dirtemplate[] = "/tmp/gt.XXXXXX";
-    temp_dir = testController->createTempDirectory(dirtemplate);
-  } else {
-    temp_dir = testController->createTempDirectory(format);
-  }
-  REQUIRE(!temp_dir.empty());
-  REQUIRE(file::FileUtils::is_directory(temp_dir.c_str()));
-  return temp_dir;
-}
-
 std::string putFileToDir(const std::string& dir_path, const std::string& file_name, const std::string& content) {
   std::string file_path(file::FileUtils::concat_path(dir_path, file_name));
   std::ofstream out_file(file_path, std::ios::binary | std::ios::out);
@@ -55,16 +43,10 @@ std::string putFileToDir(const std::string& dir_path, const std::string& file_na
   return file_path;
 }
 
-std::string createTempDirWithFile(TestController* testController, const std::string& file_name, const std::string& content) {
-  std::string temp_dir = createTempDir(testController);
-  putFileToDir(temp_dir, file_name, content);
-  return temp_dir;
-}
-
 std::string getFileContent(const std::string& file_name) {
   std::ifstream file_handle(file_name, std::ios::binary | std::ios::in);
-  REQUIRE(file_handle.is_open());
-  const std::string file_content{ (std::istreambuf_iterator<char>(file_handle)), (std::istreambuf_iterator<char>()) };
+  assert(file_handle.is_open());
+  std::string file_content{ (std::istreambuf_iterator<char>(file_handle)), (std::istreambuf_iterator<char>()) };
   return file_content;
 }
 
@@ -76,7 +58,7 @@ Identifier generateUUID() {
 
 class ManualClock : public timeutils::Clock {
  public:
-  std::chrono::milliseconds timeSinceEpoch() const override { return time_; }
+  [[nodiscard]] std::chrono::milliseconds timeSinceEpoch() const override { return time_; }
   void advance(std::chrono::milliseconds elapsed_time) { time_ += elapsed_time; }
 
  private:
