@@ -17,27 +17,35 @@
 #ifndef LIBMINIFI_INCLUDE_UTILS_FILE_FILEUTILS_H_
 #define LIBMINIFI_INCLUDE_UTILS_FILE_FILEUTILS_H_
 
+#include <filesystem>
 #include <fstream>
 #include <memory>
 #include <sstream>
 #include <tuple>
 #include <utility>
 #include <vector>
+#include <cstdio>
+#include <algorithm>
 
-#ifdef USE_BOOST
-#include <dirent.h>
-#include <boost/filesystem.hpp>
-#include <boost/system/error_code.hpp>
 #ifndef WIN32
-#include <sys/stat.h>
-#endif
-#else
-#include <errno.h>
+#include <unistd.h>
+#include <sys/stat.h> //NOLINT
+#include <pwd.h>
+#include <grp.h>
 
-#include <cstdlib>
-#include <cstring>
+#endif
+
+#include <fcntl.h>
 
 #ifdef WIN32
+#include <stdio.h>
+#include <direct.h>
+#include <sys/stat.h>  // stat // NOLINT
+#include <sys/types.h> // NOLINT
+#include <sys/utime.h>  // _utime64
+#include <tchar.h>  // _tcscpy,_tcscat,_tcscmp
+#include <windows.h>  // winapi
+
 #ifndef WIN32_LEAN_AND_MEAN
   #define WIN32_LEAN_AND_MEAN
 #endif
@@ -46,36 +54,16 @@
 #include <WS2tcpip.h>
 
 #pragma comment(lib, "Ws2_32.lib")
-#else
-#include <dirent.h> // NOLINT
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <utime.h>
 
-#endif
-#endif
-#include <cstdio>
-
-#ifndef WIN32
-#include <unistd.h>
-
-#endif
-#include <fcntl.h>
-
-#ifdef WIN32
-#include <direct.h>
-#include <sys/stat.h>  // stat // NOLINT
-#include <sys/types.h> // NOLINT
-#include <sys/utime.h>  // _utime64
-#include <tchar.h>  // _tcscpy,_tcscat,_tcscmp
-#include <windows.h>  // winapi
-
-#include <algorithm>  // replace
-#include <string>  // string
+#include <string>
 
 #include "properties/Properties.h"
 #include "utils/Id.h"
 
+#include "accctrl.h"
+#include "aclapi.h"
+#pragma comment(lib, "advapi32.lib")
+
 #endif
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
@@ -128,6 +116,9 @@ inline char get_separator(bool /*force_posix*/ = false) {
   return '/';
 }
 #endif
+time_t to_time_t(std::filesystem::file_time_type time);
+
+std::chrono::system_clock::time_point to_sys(std::filesystem::file_time_type time);
 
 inline std::string normalize_path_separators(std::string path, bool force_posix = false) {
   const auto normalize_separators = [force_posix](const char c) {
@@ -155,115 +146,57 @@ inline int64_t delete_dir(const std::string &path, bool delete_files_recursively
   if (path.empty()) {
     return -1;
   }
-#ifdef USE_BOOST
   try {
-    if (boost::filesystem::exists(path)) {
+    if (std::filesystem::exists(path)) {
       if (delete_files_recursively) {
-        boost::filesystem::remove_all(path);
+        std::filesystem::remove_all(path);
       } else {
-        boost::filesystem::remove(path);
+        std::filesystem::remove(path);
       }
     }
-  } catch(boost::filesystem::filesystem_error const & e) {
+  } catch (std::filesystem::filesystem_error const &e) {
     return -1;
     // display error message
   }
   return 0;
-#elif defined(WIN32)
-  WIN32_FIND_DATA FindFileData;
-  HANDLE hFind;
-  DWORD Attributes;
-  std::string str;
-
-
-  std::stringstream pathstr;
-  pathstr << path << "\\*";
-  str = pathstr.str();
-  // List files
-  hFind = FindFirstFile(str.c_str(), &FindFileData);
-  if (hFind != INVALID_HANDLE_VALUE) {
-    do {
-      if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0) {
-        std::stringstream strs;
-        strs << path << "\\" << FindFileData.cFileName;
-        str = strs.str();
-
-        Attributes = GetFileAttributes(str.c_str());
-        if (Attributes & FILE_ATTRIBUTE_DIRECTORY) {
-          // is directory
-          delete_dir(str, delete_files_recursively);
-        } else {
-          remove(str.c_str());
-          // not directory
-        }
-      }
-    }while (FindNextFile(hFind, &FindFileData));
-    FindClose(hFind);
-
-    RemoveDirectory(path.c_str());
-  }
-  return 0;
-#else
-  DIR *current_directory = opendir(path.c_str());
-  int r = -1;
-  if (current_directory) {
-    struct dirent *p;
-    r = 0;
-    while (!r && (p = readdir(current_directory))) {
-      int r2 = -1;
-      std::stringstream newpath;
-      if (!strcmp(p->d_name, ".") || !strcmp(p->d_name, "..")) {
-        continue;
-      }
-      struct stat statbuf;
-
-      newpath << path << "/" << p->d_name;
-
-      if (!stat(newpath.str().c_str(), &statbuf)) {
-        if (S_ISDIR(statbuf.st_mode)) {
-          if (delete_files_recursively) {
-            r2 = delete_dir(newpath.str(), delete_files_recursively);
-          }
-        } else {
-          r2 = unlink(newpath.str().c_str());
-        }
-      }
-      r = r2;
-    }
-    closedir(current_directory);
-  }
-
-  if (!r) {
-    return rmdir(path.c_str());
-  }
-
-  return 0;
-#endif
 }
 
-inline uint64_t last_write_time(const std::string &path) {
-#ifdef USE_BOOST
-  boost::system::error_code ec;
-  auto result = boost::filesystem::last_write_time(path, ec);
+inline std::chrono::time_point<std::chrono::file_clock,
+    std::chrono::seconds> last_write_time_point(const std::string &path) {
+  std::error_code ec;
+  auto result = std::filesystem::last_write_time(path, ec);
+  if (ec.value() == 0) {
+    return std::chrono::time_point_cast<std::chrono::seconds>(result);
+  }
+  return std::chrono::time_point<std::chrono::file_clock, std::chrono::seconds>{};
+}
+
+inline std::optional<std::filesystem::file_time_type> last_write_time(const std::string &path) {
+  std::error_code ec;
+  auto result = std::filesystem::last_write_time(path, ec);
   if (ec.value() == 0) {
     return result;
   }
-#elif defined(WIN32)
-  struct _stat64 result;
-  if (_stat64(path.c_str(), &result) == 0) {
-    return result.st_mtime;
-  }
-#else
-  struct stat result;
-  if (stat(path.c_str(), &result) == 0) {
-    return result.st_mtime;
-  }
-#endif
-  return 0;
+  return std::nullopt;
 }
 
-inline std::chrono::time_point<std::chrono::system_clock, std::chrono::seconds> last_write_time_point(const std::string &path) {
-  return std::chrono::time_point<std::chrono::system_clock, std::chrono::seconds>{std::chrono::seconds{last_write_time(path)}};
+inline std::optional<std::string> format_time(const std::filesystem::file_time_type& time, const std::string& format) {
+  auto last_write_time_t = to_time_t(time);
+  std::array<char, 128U> result;
+  if (std::strftime(result.data(), result.size(), format.c_str(), gmtime(&last_write_time_t)) != 0) {
+    return std::string(result.data());
+  }
+  return std::nullopt;
+}
+
+inline std::optional<std::string> get_last_modified_time_formatted_string(const std::string& path, const std::string& format_string) {
+  return last_write_time(path) | utils::flatMap([format_string](auto time) { return format_time(time, format_string); });
+}
+
+inline bool set_last_write_time(const std::string &path, std::filesystem::file_time_type new_time) {
+  std::error_code ec;
+  std::filesystem::last_write_time(path, new_time, ec);
+  return ec.value() == 0;
 }
 
 inline uint64_t file_size(const std::string &path) {
@@ -281,43 +214,37 @@ inline uint64_t file_size(const std::string &path) {
   return 0;
 }
 
-inline bool set_last_write_time(const std::string &path, uint64_t write_time) {
-#ifdef USE_BOOST
-  boost::filesystem::last_write_time(path, write_time);
-  return true;
-#elif defined(WIN32)
-  struct __utimbuf64 utim;
-  utim.actime = write_time;
-  utim.modtime = write_time;
-  return _utime64(path.c_str(), &utim) == 0U;
-#else
-  struct utimbuf utim;
-  utim.actime = write_time;
-  utim.modtime = write_time;
-  return utime(path.c_str(), &utim) == 0U;
-#endif
-}
-
-#ifndef WIN32
 inline bool get_permissions(const std::string &path, uint32_t &permissions) {
-  struct stat result;
-  if (stat(path.c_str(), &result) == 0) {
-    permissions = result.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO);
-    return true;
-  }
-  return false;
+  std::error_code ec;
+  permissions = static_cast<uint32_t>(std::filesystem::status(path, ec).permissions());
+  return ec.value() == 0;
 }
 
 inline int set_permissions(const std::string &path, const uint32_t permissions) {
-#ifdef USE_BOOST
-  boost::system::error_code ec;
-  boost::filesystem::permissions(path, static_cast<boost::filesystem::perms>(permissions), ec);
+  std::error_code ec;
+  std::filesystem::permissions(path, static_cast<std::filesystem::perms>(permissions), ec);
   return ec.value();
-#else
-  return chmod(path.c_str(), permissions);
-#endif
 }
-#endif
+
+inline std::optional<std::string> get_permission_string(const std::string &path) {
+  std::error_code ec;
+  auto permissions = std::filesystem::status(path, ec).permissions();
+  if (ec.value() != 0) {
+    return std::nullopt;
+  }
+
+  std::string permission_string;
+  permission_string += (permissions & std::filesystem::perms::owner_read) != std::filesystem::perms::none ? "r" : "-";
+  permission_string += (permissions & std::filesystem::perms::owner_write) != std::filesystem::perms::none ? "w" : "-";
+  permission_string += (permissions & std::filesystem::perms::owner_exec) != std::filesystem::perms::none ? "x" : "-";
+  permission_string += (permissions & std::filesystem::perms::group_read) != std::filesystem::perms::none ? "r" : "-";
+  permission_string += (permissions & std::filesystem::perms::group_write) != std::filesystem::perms::none ? "w" : "-";
+  permission_string += (permissions & std::filesystem::perms::group_exec) != std::filesystem::perms::none ? "x" : "-";
+  permission_string += (permissions & std::filesystem::perms::others_read) != std::filesystem::perms::none ? "r" : "-";
+  permission_string += (permissions & std::filesystem::perms::others_write) != std::filesystem::perms::none ? "w" : "-";
+  permission_string += (permissions & std::filesystem::perms::others_exec) != std::filesystem::perms::none ? "x" : "-";
+  return permission_string;
+}
 
 #ifndef WIN32
 inline bool get_uid_gid(const std::string &path, uint64_t &uid, uint64_t &gid) {
@@ -331,84 +258,36 @@ inline bool get_uid_gid(const std::string &path, uint64_t &uid, uint64_t &gid) {
 }
 #endif
 
-inline bool is_directory(const char * path) {
-#ifndef WIN32
-  struct stat dir_stat;
-  if (stat(path, &dir_stat) != 0) {
-      return false;
+inline bool is_directory(const std::filesystem::path &path) {
+  std::error_code ec;
+  bool result = std::filesystem::is_directory(path, ec);
+  if (ec.value() == 0) {
+    return result;
   }
-  return S_ISDIR(dir_stat.st_mode) != 0;
-#else
-  struct _stat64 dir_stat;
-  if (_stat64(path, &dir_stat) != 0) {
-      return false;
-  }
-  return S_ISDIR(dir_stat.st_mode) != 0;
-#endif
+  return false;
 }
 
-inline bool exists(const std::string& path) {
-#ifdef USE_BOOST
-  return boost::filesystem::exists(path);
-#elif defined(WIN32)
-  struct _stat64 statbuf;
-  return _stat64(path.c_str(), &statbuf) == 0;
-#else
-  struct stat statbuf;
-  return stat(path.c_str(), &statbuf) == 0;
-#endif
+inline bool exists(const std::string &path) {
+  std::error_code ec;
+  bool result = std::filesystem::exists(path, ec);
+  if (ec.value() == 0) {
+    return result;
+  }
+  return false;
 }
 
-inline int create_dir(const std::string& path, bool recursive = true) {
-#ifdef USE_BOOST
-  boost::filesystem::path dir(path);
-  boost::system::error_code ec;
+inline int create_dir(const std::string &path, bool recursive = true) {
+  std::filesystem::path dir(path);
+  std::error_code ec;
   if (!recursive) {
-    boost::filesystem::create_directory(dir, ec);
+    std::filesystem::create_directory(dir, ec);
   } else {
-    boost::filesystem::create_directories(dir, ec);
+    std::filesystem::create_directories(dir, ec);
   }
   if (ec.value() == 0 || (ec.value() == EEXIST && is_directory(path.c_str()))) {
     return 0;
   }
   return ec.value();
-#else
-  if (!recursive) {
-    if (detail::platform_create_dir(path) != 0 && errno != EEXIST) {
-      return -1;
-    }
-    return 0;
-  }
-  if (detail::platform_create_dir(path) == 0) {
-    return 0;
-  }
-
-  switch (errno) {
-  case ENOENT: {
-    size_t found = path.find_last_of(get_separator());
-
-    if (found == std::string::npos) {
-      return -1;
-    }
-
-    const std::string dir = path.substr(0, found);
-    int res = create_dir(dir, recursive);
-    if (res < 0) {
-      return -1;
-    }
-    return detail::platform_create_dir(path);
-  }
-  case EEXIST: {
-    if (is_directory(path.c_str())) {
-      return 0;
-    }
-    return -1;
-  }
-  default:
-    return -1;
-  }
-  return -1;
-#endif
 }
 
 inline int copy_file(const std::string &path_from, const std::string& dest_path) {
@@ -420,73 +299,25 @@ inline int copy_file(const std::string &path_from, const std::string& dest_path)
   return 0;
 }
 
-inline void addFilesMatchingExtension(const std::shared_ptr<logging::Logger> &logger, const std::string &originalPath, const std::string &extension, std::vector<std::string> &accruedFiles) {
-#ifndef WIN32
-
-  struct stat s;
-  if (stat(originalPath.c_str(), &s) == 0) {
-    if (s.st_mode & S_IFDIR) {
-      DIR *d;
-      d = opendir(originalPath.c_str());
-      if (!d) {
-        return;
-      }
-      // only perform a listing while we are not empty
-      logger->log_debug("Performing file listing on %s", originalPath);
-
-      struct dirent *entry;
-      entry = readdir(d);
-      while (entry != nullptr) {
-        std::string d_name = entry->d_name;
-        std::string path = originalPath + "/" + d_name;
-        struct stat statbuf { };
-        if (stat(path.c_str(), &statbuf) != 0) {
-          logger->log_warn("Failed to stat %s", path);
-          return;
-        }
-        if (S_ISDIR(statbuf.st_mode)) {
-          // if this is a directory
-          if (d_name != ".." && d_name != ".") {
-            addFilesMatchingExtension(logger, path, extension, accruedFiles);
-          }
-        } else {
-          if (utils::StringUtils::endsWith(path, extension)) {
-            logger->log_info("Adding %s to paths", path);
-            accruedFiles.push_back(path);
-          }
-        }
-        entry = readdir(d);
-      }
-      closedir(d);
-    } else if (s.st_mode & S_IFREG) {
-      if (utils::StringUtils::endsWith(originalPath, extension)) {
-        logger->log_info("Adding %s to paths", originalPath);
-        accruedFiles.push_back(originalPath);
-      }
-    } else {
-      logger->log_error("Could not stat", originalPath);
-    }
-
-  } else {
-    logger->log_error("Could not access %s", originalPath);
+inline void addFilesMatchingExtension(const std::shared_ptr<core::logging::Logger> &logger,
+                                      const std::string &originalPath,
+                                      const std::string &extension,
+                                      std::vector<std::string> &accruedFiles) {
+  if (!utils::file::exists(originalPath)) {
+    logger->log_warn("Failed to open directory: %s", originalPath.c_str());
+    return;
   }
-#else
-  HANDLE hFind;
-  WIN32_FIND_DATA FindFileData;
 
-  std::string pathToSearch = originalPath + "\\*" + extension;
-  if ((hFind = FindFirstFileA(pathToSearch.c_str(), &FindFileData)) != INVALID_HANDLE_VALUE) {
-    do {
-      struct _stat64 statbuf {};
+  if (utils::file::is_directory(originalPath)) {
+    // only perform a listing while we are not empty
+    logger->log_debug("Looking for files with %s extension in %s", extension, originalPath);
 
-      std::string path = originalPath + "\\" + FindFileData.cFileName;
-      logger->log_info("Adding %s to paths", path);
-      if (_stat64(path.c_str(), &statbuf) != 0) {
-        logger->log_warn("Failed to stat %s", path);
-        break;
-      }
-      logger->log_info("Adding %s to paths", path);
-      if (S_ISDIR(statbuf.st_mode)) {
+    for (const auto &entry: std::filesystem::directory_iterator(originalPath,
+                                                                std::filesystem::directory_options::skip_permission_denied)) {
+      std::string d_name = entry.path().filename().string();
+      std::string path = entry.path().string();
+
+      if (utils::file::is_directory(path)) {          // if this is a directory
         addFilesMatchingExtension(logger, path, extension, accruedFiles);
       } else {
         if (utils::StringUtils::endsWith(path, extension)) {
@@ -494,96 +325,15 @@ inline void addFilesMatchingExtension(const std::shared_ptr<logging::Logger> &lo
           accruedFiles.push_back(path);
         }
       }
-    }while (FindNextFileA(hFind, &FindFileData));
-    FindClose(hFind);
-  }
-#endif
-}
-
-/*
- * Provides a platform-independent function to list a directory
- * Callback is called for every file found: first argument is the path of the directory, second is the filename
- * Return value of the callback is used to continue (true) or stop (false) listing
- */
-inline void list_dir(const std::string& dir, std::function<bool(const std::string&, const std::string&)> callback,
-                     const std::shared_ptr<logging::Logger> &logger, bool recursive = true) {
-  logger->log_debug("Performing file listing against %s", dir);
-#ifndef WIN32
-  DIR *d = opendir(dir.c_str());
-  if (!d) {
-    logger->log_warn("Failed to open directory: %s", dir.c_str());
-    return;
-  }
-
-  struct dirent *entry;
-  while ((entry = readdir(d)) != NULL) {
-    std::string d_name = entry->d_name;
-    std::string path = dir + get_separator() + d_name;
-
-    struct stat statbuf;
-    if (stat(path.c_str(), &statbuf) != 0) {
-      logger->log_warn("Failed to stat %s", path);
-      continue;
     }
-
-    if (S_ISDIR(statbuf.st_mode)) {
-      // if this is a directory
-      if (recursive && strcmp(d_name.c_str(), "..") != 0 && strcmp(d_name.c_str(), ".") != 0) {
-        list_dir(path, callback, logger, recursive);
-      }
-    } else {
-      if (!callback(dir, d_name)) {
-        break;
-      }
+  } else if (std::filesystem::is_regular_file(originalPath)) {
+    if (utils::StringUtils::endsWith(originalPath, extension)) {
+      logger->log_info("Adding %s to paths", originalPath);
+      accruedFiles.push_back(originalPath);
     }
+  } else {
+    logger->log_error("Could not access %s", originalPath);
   }
-  closedir(d);
-#else
-  HANDLE hFind;
-  WIN32_FIND_DATA FindFileData;
-
-  std::string pathToSearch = dir + "\\*.*";
-  hFind = FindFirstFileA(pathToSearch.c_str(), &FindFileData);
-
-  if (hFind == INVALID_HANDLE_VALUE) {
-    logger->log_warn("Failed to open directory: %s", dir.c_str());
-    return;
-  }
-
-  do {
-    struct _stat64 statbuf {};
-    if (strcmp(FindFileData.cFileName, ".") != 0 && strcmp(FindFileData.cFileName, "..") != 0) {
-      std::string path = dir + get_separator() + FindFileData.cFileName;
-      if (_stat64(path.c_str(), &statbuf) != 0) {
-        logger->log_warn("Failed to stat %s", path);
-        continue;
-      }
-      if (S_ISDIR(statbuf.st_mode)) {
-        if (recursive) {
-          list_dir(path, callback, logger, recursive);
-        }
-      } else {
-        if (!callback(dir, FindFileData.cFileName)) {
-          break;
-        }
-      }
-    }
-  } while (FindNextFileA(hFind, &FindFileData));
-  FindClose(hFind);
-#endif
-}
-
-inline std::vector<std::pair<std::string, std::string>> list_dir_all(const std::string& dir, const std::shared_ptr<logging::Logger> &logger,
-    bool recursive = true)  {
-  std::vector<std::pair<std::string, std::string>> fileList;
-  auto lambda = [&fileList] (const std::string &path, const std::string &filename) {
-    fileList.push_back(make_pair(path, filename));
-    return true;
-  };
-
-  list_dir(dir, lambda, logger, recursive);
-
-  return fileList;
 }
 
 inline std::string concat_path(const std::string& root, const std::string& child, bool force_posix = false) {
@@ -599,6 +349,63 @@ inline std::string concat_path(const std::string& root, const std::string& child
   return new_path.str();
 }
 
+/**
+ * Provides a platform-independent function to list a directory
+ * @param dir The directory to start the enumeration from.
+ * @param callback Callback is called for every file found: first argument is the path of the directory, second is the filename.
+ * Return value of the callback is used to continue (true) or stop (false) listing
+ * @param logger
+ * @param dir_callback Called for every child directory, its return value decides if we should descend and recursively
+ * process that directory or not.
+ */
+inline void list_dir(const std::string& dir,
+                     std::function<bool(const std::string&, const std::string&)> callback,
+                     const std::shared_ptr<core::logging::Logger> &logger,
+                     std::function<bool(const std::string&)> dir_callback) {
+  logger->log_debug("Performing file listing against %s", dir);
+  if (!utils::file::exists(dir)) {
+    logger->log_warn("Failed to open directory: %s", dir.c_str());
+    return;
+  }
+
+  for (const auto &entry: std::filesystem::directory_iterator(dir, std::filesystem::directory_options::skip_permission_denied)) {
+    std::string d_name = entry.path().filename().string();
+    std::string path = entry.path().string();
+
+    if (utils::file::is_directory(path)) {  // if this is a directory
+      if (dir_callback(dir)) {
+        list_dir(path, callback, logger, dir_callback);
+      }
+    } else {
+      if (!callback(dir, d_name)) {
+        break;
+      }
+    }
+  }
+}
+
+inline void list_dir(const std::string& dir,
+                     std::function<bool(const std::string&, const std::string&)> callback,
+                     const std::shared_ptr<core::logging::Logger> &logger,
+                     bool recursive = true) {
+  list_dir(dir, callback, logger, [&] (const std::string&) {
+    return recursive;
+  });
+}
+
+inline std::vector<std::pair<std::string, std::string>> list_dir_all(const std::string& dir, const std::shared_ptr<core::logging::Logger> &logger,
+    bool recursive = true)  {
+  std::vector<std::pair<std::string, std::string>> fileList;
+  auto lambda = [&fileList] (const std::string &path, const std::string &filename) {
+    fileList.push_back(make_pair(path, filename));
+    return true;
+  };
+
+  list_dir(dir, lambda, logger, recursive);
+
+  return fileList;
+}
+
 inline std::string create_temp_directory(char* format) {
 #ifdef WIN32
   const std::string tempDirectory = concat_path(get_temp_directory(),
@@ -722,35 +529,35 @@ inline std::string get_executable_path() {
   }
 #elif defined(__APPLE__)
   std::vector<char> buf(PATH_MAX);
-    uint32_t buf_size = buf.size();
-    while (_NSGetExecutablePath(buf.data(), &buf_size) != 0) {
-      buf.resize(buf_size);
-    }
-    std::vector<char> resolved_name(PATH_MAX);
-    if (realpath(buf.data(), resolved_name.data()) == nullptr) {
-      return "";
-    }
-    return std::string(resolved_name.data());
-#elif defined(WIN32)
-    HMODULE hModule = GetModuleHandleA(nullptr);
-    if (hModule == nullptr) {
-      return "";
-    }
-    std::vector<char> buf(1024U);
-    while (true) {
-      size_t ret = GetModuleFileNameA(hModule, buf.data(), gsl::narrow<DWORD>(buf.size()));
-      if (ret == 0U) {
-        return "";
-      }
-      if (ret == buf.size() && GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
-        /* It has been truncated */
-        buf.resize(buf.size() * 2);
-        continue;
-      }
-      return std::string(buf.data());
-    }
-#else
+  uint32_t buf_size = buf.size();
+  while (_NSGetExecutablePath(buf.data(), &buf_size) != 0) {
+    buf.resize(buf_size);
+  }
+  std::vector<char> resolved_name(PATH_MAX);
+  if (realpath(buf.data(), resolved_name.data()) == nullptr) {
     return "";
+  }
+  return std::string(resolved_name.data());
+#elif defined(WIN32)
+  HMODULE hModule = GetModuleHandleA(nullptr);
+  if (hModule == nullptr) {
+    return "";
+  }
+  std::vector<char> buf(1024U);
+  while (true) {
+    size_t ret = GetModuleFileNameA(hModule, buf.data(), gsl::narrow<DWORD>(buf.size()));
+    if (ret == 0U) {
+      return "";
+    }
+    if (ret == buf.size() && GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
+      /* It has been truncated */
+      buf.resize(buf.size() * 2);
+      continue;
+    }
+    return std::string(buf.data());
+  }
+#else
+  return "";
 #endif
 }
 
@@ -787,7 +594,6 @@ inline int access(const char *path_name, int mode) {
   return ::access(path_name, mode);
 #endif
 }
-
 #ifdef WIN32
 inline std::error_code hide_file(const char* const file_name) {
     const bool success = SetFileAttributesA(file_name, FILE_ATTRIBUTE_HIDDEN);
@@ -802,17 +608,158 @@ inline std::error_code hide_file(const char* const file_name) {
 
 uint64_t computeChecksum(const std::string &file_name, uint64_t up_to_position);
 
-inline std::string get_file_content(const std::string &file_name) {
-  std::ifstream file(file_name);
+inline std::string get_content(const std::string &file_name) {
+  std::ifstream file(file_name, std::ifstream::binary);
   std::string content((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
   return content;
 }
 
+void put_content(const std::filesystem::path& filename, std::string_view new_contents);
+
+bool contains(const std::filesystem::path& file_path, std::string_view text_to_search);
+
+
+inline std::optional<std::string> get_file_owner(const std::string& file_path) {
+#ifndef WIN32
+  struct stat info;
+  if (stat(file_path.c_str(), &info) != 0) {
+    return std::nullopt;
+  }
+
+  struct passwd pw;
+  pw.pw_name = 0;
+  struct passwd *result = nullptr;
+  char localbuf[1024] = {};
+  if (getpwuid_r(info.st_uid, &pw, localbuf, sizeof(localbuf), &result) != 0 || pw.pw_name == 0) {
+    return std::nullopt;
+  }
+
+  return std::string(pw.pw_name);
+#else
+  DWORD return_code = 0;
+  PSID sid_owner = NULL;
+  BOOL bool_return = TRUE;
+  LPTSTR account_name = NULL;
+  LPTSTR domain_name = NULL;
+  DWORD account_name_dword = 1;
+  DWORD domain_name_dword = 1;
+  SID_NAME_USE sid_type = SidTypeUnknown;
+  HANDLE file_handle;
+  PSECURITY_DESCRIPTOR sec_descriptor = NULL;
+
+  // Get the handle of the file object.
+  file_handle = CreateFile(
+    TEXT(file_path.c_str()),
+    GENERIC_READ,
+    FILE_SHARE_READ,
+    NULL,
+    OPEN_EXISTING,
+    FILE_ATTRIBUTE_NORMAL,
+    NULL);
+
+  // Check GetLastError for CreateFile error code.
+  if (file_handle == INVALID_HANDLE_VALUE) {
+    return std::nullopt;
+  }
+
+  // Get the owner SID of the file.
+  return_code = GetSecurityInfo(
+    file_handle,
+    SE_FILE_OBJECT,
+    OWNER_SECURITY_INFORMATION,
+    &sid_owner,
+    NULL,
+    NULL,
+    NULL,
+    &sec_descriptor);
+
+  // Check GetLastError for GetSecurityInfo error condition.
+  if (return_code != ERROR_SUCCESS) {
+    return std::nullopt;
+  }
+
+  // First call to LookupAccountSid to get the buffer sizes.
+  bool_return = LookupAccountSid(
+    NULL,
+    sid_owner,
+    account_name,
+    (LPDWORD)&account_name_dword,
+    domain_name,
+    (LPDWORD)&domain_name_dword,
+    &sid_type);
+
+  // Reallocate memory for the buffers.
+  account_name = (LPTSTR)GlobalAlloc(
+    GMEM_FIXED,
+    account_name_dword);
+
+  // Check GetLastError for GlobalAlloc error condition.
+  if (account_name == NULL) {
+    return std::nullopt;
+  }
+  auto cleanup_account_name = gsl::finally([&account_name] { GlobalFree(account_name); });
+
+  domain_name = (LPTSTR)GlobalAlloc(
+    GMEM_FIXED,
+    domain_name_dword);
+
+  // Check GetLastError for GlobalAlloc error condition.
+  if (domain_name == NULL) {
+    return std::nullopt;
+  }
+  auto cleanup_domain_name = gsl::finally([&domain_name] { GlobalFree(domain_name); });
+
+  // Second call to LookupAccountSid to get the account name.
+  bool_return = LookupAccountSid(
+    NULL,                   // name of local or remote computer
+    sid_owner,              // security identifier
+    account_name,               // account name buffer
+    (LPDWORD)&account_name_dword,   // size of account name buffer
+    domain_name,             // domain name
+    (LPDWORD)&domain_name_dword,  // size of domain name buffer
+    &sid_type);                 // SID type
+
+  // Check GetLastError for LookupAccountSid error condition.
+  if (bool_return == FALSE) {
+    return std::nullopt;
+  }
+
+  auto result = std::string(account_name);
+  return result;
+#endif
+}
+
+#ifndef WIN32
+inline std::optional<std::string> get_file_group(const std::string& file_path) {
+  struct stat info;
+  if (stat(file_path.c_str(), &info) != 0) {
+    return std::nullopt;
+  }
+
+  struct group gr;
+  gr.gr_name = 0;
+  struct group *result = nullptr;
+  char localbuf[1024] = {};
+  if ((getgrgid_r(info.st_uid, &gr, localbuf, sizeof(localbuf), &result) != 0) || gr.gr_name == 0) {
+    return std::nullopt;
+  }
+
+  return std::string(gr.gr_name);
+}
+#endif
+
+inline std::optional<std::string> get_relative_path(const std::string& path, const std::string& base_path) {
+  if (!utils::StringUtils::startsWith(path, base_path)) {
+    return std::nullopt;
+  }
+
+  return std::filesystem::relative(path, base_path).string();
+}
+
 }  // namespace file
 }  // namespace utils
 }  // namespace minifi
 }  // namespace nifi
 }  // namespace apache
 }  // namespace org
-
 #endif  // LIBMINIFI_INCLUDE_UTILS_FILE_FILEUTILS_H_
