@@ -21,6 +21,8 @@
 #include "utils/gsl.h"
 
 #include "core/logging/LoggerConfiguration.h"
+#include "utils/StringUtils.h"
+#include "properties/Configuration.h"
 
 namespace org {
 namespace apache {
@@ -28,22 +30,25 @@ namespace nifi {
 namespace minifi {
 
 bool Configure::get(const std::string& key, std::string& value) const {
-  bool found = getString(key, value);
-  if (decryptor_ && found && isEncrypted(key)) {
-    value = decryptor_->decrypt(value);
+  if (auto opt_value = getRawValue(key)) {
+    value = *opt_value;
+    if (decryptor_ && isEncrypted(key)) {
+      value = decryptor_->decrypt(value);
+    }
+    return true;
   }
-  return found;
+  return false;
 }
 
 bool Configure::get(const std::string& key, const std::string& alternate_key, std::string& value) const {
   if (get(key, value)) {
     if (get(alternate_key)) {
-      const auto logger = logging::LoggerFactory<Configure>::getLogger();
+      const auto logger = core::logging::LoggerFactory<Configure>::getLogger();
       logger->log_warn("Both the property '%s' and an alternate property '%s' are set. Using '%s'.", key, alternate_key, key);
     }
     return true;
   } else if (get(alternate_key, value)) {
-    const auto logger = logging::LoggerFactory<Configure>::getLogger();
+    const auto logger = core::logging::LoggerFactory<Configure>::getLogger();
     logger->log_warn("%s is an alternate property that may not be supported in future releases. Please use %s instead.", alternate_key, key);
     return true;
   } else {
@@ -51,24 +56,35 @@ bool Configure::get(const std::string& key, const std::string& alternate_key, st
   }
 }
 
-utils::optional<std::string> Configure::get(const std::string& key) const {
-  utils::optional<std::string> value = getString(key);
-  if (decryptor_ && value && isEncrypted(key)) {
-    return decryptor_->decrypt(*value);
-  } else {
+std::optional<std::string> Configure::get(const std::string& key) const {
+  std::string value;
+  if (get(key, value)) {
     return value;
   }
+  return std::nullopt;
+}
+
+std::optional<std::string> Configure::getRawValue(const std::string& key) const {
+  static constexpr std::string_view log_prefix = "nifi.log.";
+  if (utils::StringUtils::startsWith(key, log_prefix)) {
+    if (logger_properties_) {
+      return logger_properties_->getString(key.substr(log_prefix.length()));
+    }
+    return std::nullopt;
+  }
+
+  return getString(key);
 }
 
 bool Configure::isEncrypted(const std::string& key) const {
   gsl_Expects(decryptor_);
-  utils::optional<std::string> encryption_marker = getString(key + ".protected");
+  const auto encryption_marker = getString(key + ".protected");
   return decryptor_->isValidEncryptionMarker(encryption_marker);
 }
 
-utils::optional<std::string> Configure::getAgentClass() const {
+std::optional<std::string> Configure::getAgentClass() const {
   std::string agent_class;
-  if (get("nifi.c2.agent.class", "c2.agent.class", agent_class) && !agent_class.empty()) {
+  if (get(Configuration::nifi_c2_agent_class, "c2.agent.class", agent_class) && !agent_class.empty()) {
     return agent_class;
   }
   return {};
@@ -76,7 +92,7 @@ utils::optional<std::string> Configure::getAgentClass() const {
 
 std::string Configure::getAgentIdentifier() const {
   std::string agent_id;
-  if (!get("nifi.c2.agent.identifier", "c2.agent.identifier", agent_id) || agent_id.empty()) {
+  if (!get(Configuration::nifi_c2_agent_identifier, "c2.agent.identifier", agent_id) || agent_id.empty()) {
     std::lock_guard<std::mutex> guard(fallback_identifier_mutex_);
     return fallback_identifier_;
   }
@@ -88,6 +104,31 @@ void Configure::setFallbackAgentIdentifier(const std::string& id) {
   fallback_identifier_ = id;
 }
 
+void Configure::set(const std::string& key, const std::string& value, PropertyChangeLifetime lifetime) {
+  const std::string_view log_prefix = "nifi.log.";
+  if (utils::StringUtils::startsWith(key, log_prefix)) {
+    if (logger_properties_) {
+      logger_properties_changed_ = true;
+      logger_properties_->set(key.substr(log_prefix.length()), value, lifetime);
+    }
+  } else {
+    Configuration::set(key, value, lifetime);
+  }
+}
+
+bool Configure::commitChanges() {
+  bool success = true;
+  if (logger_properties_) {
+    success &= logger_properties_->commitChanges();
+    if (logger_properties_changed_) {
+      core::logging::LoggerConfiguration::getConfiguration().initialize(logger_properties_);
+      logger_properties_changed_ = false;
+    }
+  }
+  success &= Configuration::commitChanges();
+  return success;
+}
+
 } /* namespace minifi */
 } /* namespace nifi */
 } /* namespace apache */
