@@ -33,6 +33,8 @@
 #include "core/Processor.h"
 #include "core/logging/LoggerConfiguration.h"
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -41,32 +43,14 @@ namespace minifi {
 Connection::Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name)
     : core::Connectable(name),
       flow_repository_(flow_repository),
-      content_repo_(content_repo),
-      logger_(logging::LoggerFactory<Connection>::getLogger()) {
-  source_connectable_ = nullptr;
-  dest_connectable_ = nullptr;
-  max_queue_size_ = 0;
-  max_data_queue_size_ = 0;
-  expired_duration_ = 0;
-  queued_data_size_ = 0;
-  drop_empty_ = false;
-
+      content_repo_(content_repo) {
   logger_->log_debug("Connection %s created", name_);
 }
 
 Connection::Connection(const std::shared_ptr<core::Repository> &flow_repository, const std::shared_ptr<core::ContentRepository> &content_repo, const std::string &name, const utils::Identifier &uuid)
     : core::Connectable(name, uuid),
       flow_repository_(flow_repository),
-      content_repo_(content_repo),
-      logger_(logging::LoggerFactory<Connection>::getLogger()) {
-  source_connectable_ = nullptr;
-  dest_connectable_ = nullptr;
-  max_queue_size_ = 0;
-  max_data_queue_size_ = 0;
-  expired_duration_ = 0;
-  queued_data_size_ = 0;
-  drop_empty_ = false;
-
+      content_repo_(content_repo) {
   logger_->log_debug("Connection %s created", name_);
 }
 
@@ -74,19 +58,8 @@ Connection::Connection(const std::shared_ptr<core::Repository> &flow_repository,
                        const utils::Identifier& srcUUID)
     : core::Connectable(name, uuid),
       flow_repository_(flow_repository),
-      content_repo_(content_repo),
-      logger_(logging::LoggerFactory<Connection>::getLogger()) {
-
+      content_repo_(content_repo) {
   src_uuid_ = srcUUID;
-
-  source_connectable_ = nullptr;
-  dest_connectable_ = nullptr;
-  max_queue_size_ = 0;
-  max_data_queue_size_ = 0;
-  expired_duration_ = 0;
-  queued_data_size_ = 0;
-  drop_empty_ = false;
-
   logger_->log_debug("Connection %s created", name_);
 }
 
@@ -94,20 +67,10 @@ Connection::Connection(const std::shared_ptr<core::Repository> &flow_repository,
                        const utils::Identifier& srcUUID, const utils::Identifier& destUUID)
     : core::Connectable(name, uuid),
       flow_repository_(flow_repository),
-      content_repo_(content_repo),
-      logger_(logging::LoggerFactory<Connection>::getLogger()) {
+      content_repo_(content_repo) {
 
   src_uuid_ = srcUUID;
   dest_uuid_ = destUUID;
-
-  source_connectable_ = nullptr;
-  dest_connectable_ = nullptr;
-  max_queue_size_ = 0;
-  max_data_queue_size_ = 0;
-  expired_duration_ = 0;
-  queued_data_size_ = 0;
-  drop_empty_ = false;
-
   logger_->log_debug("Connection %s created", name_);
 }
 
@@ -117,7 +80,7 @@ bool Connection::isEmpty() const {
   return queue_.empty();
 }
 
-bool Connection::isFull() {
+bool Connection::isFull() const {
   std::lock_guard<std::mutex> lock(mutex_);
 
   if (max_queue_size_ <= 0 && max_data_queue_size_ <= 0)
@@ -185,27 +148,25 @@ std::shared_ptr<core::FlowFile> Connection::poll(std::set<std::shared_ptr<core::
     std::shared_ptr<core::FlowFile> item = queue_.pop();
     queued_data_size_ -= item->getSize();
 
-    if (expired_duration_ > 0) {
+    if (expired_duration_.load() > 0ms) {
       // We need to check for flow expiration
-      if (utils::timeutils::getTimeMillis() > (item->getEntryDate() + expired_duration_)) {
+      if (std::chrono::system_clock::now() > (item->getEntryDate() + expired_duration_.load())) {
         // Flow record expired
         expiredFlowRecords.insert(item);
         logger_->log_debug("Delete flow file UUID %s from connection %s, because it expired", item->getUUIDStr(), name_);
       } else {
-        std::shared_ptr<Connectable> connectable = std::static_pointer_cast<Connectable>(shared_from_this());
-        item->setConnection(connectable);
+        item->setConnection(this);
         logger_->log_debug("Dequeue flow file UUID %s from connection %s", item->getUUIDStr(), name_);
         return item;
       }
     } else {
-      std::shared_ptr<Connectable> connectable = std::static_pointer_cast<Connectable>(shared_from_this());
-      item->setConnection(connectable);
+      item->setConnection(this);
       logger_->log_debug("Dequeue flow file UUID %s from connection %s", item->getUUIDStr(), name_);
       return item;
     }
   }
 
-  return NULL;
+  return nullptr;
 }
 
 void Connection::drain(bool delete_permanently) {
