@@ -17,7 +17,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#include <time.h>
 #include <vector>
 #include <map>
 #include <chrono>
@@ -29,18 +28,12 @@
 
 #include "FlowController.h"
 #include "core/state/nodes/AgentInformation.h"
-#include "core/state/nodes/BuildInformation.h"
-#include "core/state/nodes/DeviceInformation.h"
 #include "core/state/nodes/FlowInformation.h"
-#include "core/state/nodes/ProcessMetrics.h"
-#include "core/state/nodes/QueueMetrics.h"
 #include "core/state/nodes/RepositoryMetrics.h"
-#include "core/state/nodes/SystemMetrics.h"
 #include "core/state/ProcessorController.h"
 #include "c2/C2Agent.h"
 #include "core/ProcessGroup.h"
 #include "core/Core.h"
-#include "core/ClassLoader.h"
 #include "SchedulingAgent.h"
 #include "core/controller/ControllerServiceProvider.h"
 #include "core/controller/ForwardingControllerServiceProvider.h"
@@ -49,27 +42,23 @@
 #include "utils/file/PathUtils.h"
 #include "utils/file/FileSystem.h"
 #include "utils/HTTPClient.h"
-#include "utils/GeneralUtils.h"
 #include "io/NetworkPrioritizer.h"
-#include "io/validation.h"
+#include "io/FileStream.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
+namespace org::apache::nifi::minifi {
 
 FlowController::FlowController(std::shared_ptr<core::Repository> provenance_repo, std::shared_ptr<core::Repository> flow_file_repo,
                                std::shared_ptr<Configure> configure, std::unique_ptr<core::FlowConfiguration> flow_configuration,
-                               std::shared_ptr<core::ContentRepository> content_repo, const std::string /*name*/, bool headless_mode,
-                               std::shared_ptr<utils::file::FileSystem> filesystem)
+                               std::shared_ptr<core::ContentRepository> content_repo, const std::string& /*name*/,
+                               std::shared_ptr<utils::file::FileSystem> filesystem, std::function<void()> request_restart)
     : core::controller::ForwardingControllerServiceProvider(core::getClassName<FlowController>()),
       c2::C2Client(std::move(configure), std::move(provenance_repo), std::move(flow_file_repo),
-                   std::move(content_repo), std::move(flow_configuration), std::move(filesystem)),
+                   std::move(content_repo), std::move(flow_configuration), std::move(filesystem),
+                   std::move(request_restart), core::logging::LoggerFactory<c2::C2Client>::getLogger()),
       running_(false),
       updating_(false),
       initialized_(false),
-      thread_pool_(2, false, nullptr, "Flowcontroller threadpool"),
-      logger_(logging::LoggerFactory<FlowController>::getLogger()) {
+      thread_pool_(2, false, nullptr, "Flowcontroller threadpool") {
   if (provenance_repo_ == nullptr)
     throw std::runtime_error("Provenance Repo should not be null");
   if (flow_file_repo_ == nullptr)
@@ -80,36 +69,25 @@ FlowController::FlowController(std::shared_ptr<core::Repository> provenance_repo
   running_ = false;
   initialized_ = false;
 
-  protocol_ = utils::make_unique<FlowControlProtocol>(this, configuration_);
-
-  if (!headless_mode) {
-    initializeExternalComponents();
-  }
+  protocol_ = std::make_unique<FlowControlProtocol>(this, configuration_);
 }
 
-void FlowController::initializeExternalComponents() {
-  auto jvmCreator = core::ClassLoader::getDefaultClassLoader().instantiate("JVMCreator", "JVMCreator");
-  if (nullptr != jvmCreator) {
-    logger_->log_debug("JVMCreator loaded...");
-    jvmCreator->configure(configuration_);
-  }
+FlowController::FlowController(std::shared_ptr<core::Repository> provenance_repo, std::shared_ptr<core::Repository> flow_file_repo,
+                 std::shared_ptr<Configure> configure, std::unique_ptr<core::FlowConfiguration> flow_configuration,
+                 std::shared_ptr<core::ContentRepository> content_repo, std::shared_ptr<utils::file::FileSystem> filesystem,
+                 std::function<void()> request_restart)
+      : FlowController(std::move(provenance_repo), std::move(flow_file_repo), std::move(configure), std::move(flow_configuration),
+                       std::move(content_repo), DEFAULT_ROOT_GROUP_NAME, std::move(filesystem), std::move(request_restart)) {}
 
-  auto pythoncreator = core::ClassLoader::getDefaultClassLoader().instantiate("PythonCreator", "PythonCreator");
-  if (nullptr != pythoncreator) {
-    logger_->log_debug("PythonCreator loaded...");
-    pythoncreator->configure(configuration_);
-  }
-}
-
-utils::optional<std::chrono::milliseconds> FlowController::loadShutdownTimeoutFromConfiguration() {
+std::optional<std::chrono::milliseconds> FlowController::loadShutdownTimeoutFromConfiguration() {
   std::string shutdown_timeout_str;
   if (configuration_->get(minifi::Configure::nifi_flowcontroller_drain_timeout, shutdown_timeout_str)) {
-    const utils::optional<core::TimePeriodValue> time_from_config = core::TimePeriodValue::fromString(shutdown_timeout_str);
+    const auto time_from_config = core::TimePeriodValue::fromString(shutdown_timeout_str);
     if (time_from_config) {
       return { std::chrono::milliseconds{ time_from_config.value().getMilliseconds() }};
     }
   }
-  return utils::nullopt;
+  return std::nullopt;
 }
 
 FlowController::~FlowController() {
@@ -147,10 +125,12 @@ bool FlowController::applyConfiguration(const std::string &source, const std::st
   controller_map_->clear();
   auto prevRoot = std::move(this->root_);
   this->root_ = std::move(newRoot);
+  processor_to_controller_.clear();
+  updateResponseNodeConnections();
   initialized_ = false;
   bool started = false;
   try {
-    load(this->root_, true);
+    load(std::move(root_), true);
     flow_update_ = true;
     started = start() == 0;
 
@@ -168,7 +148,7 @@ bool FlowController::applyConfiguration(const std::string &source, const std::st
     }
   } catch (...) {
     this->root_ = std::move(prevRoot);
-    load(this->root_, true);
+    load(std::move(this->root_), true);
     flow_update_ = true;
     updating_ = false;
   }
@@ -183,7 +163,7 @@ int16_t FlowController::stop() {
     logger_->log_info("Stop Flow Controller");
     if (this->root_) {
       // stop source processors first
-      this->root_->stopProcessing(timer_scheduler_, event_scheduler_, cron_scheduler_, [] (const std::shared_ptr<core::Processor>& proc) -> bool {
+      this->root_->stopProcessing(timer_scheduler_, event_scheduler_, cron_scheduler_, [] (const core::Processor* proc) -> bool {
         return !proc->hasIncomingConnections();
       });
       // we enable C2 to progressively increase the timeout
@@ -204,7 +184,7 @@ int16_t FlowController::stop() {
     thread_pool_.shutdown();
     /* STOP! Before you change it, consider the following:
      * -Stopping the schedulers doesn't actually quit the onTrigger functions of processors
-     * -They only guarantee that the processors are not scheduled any more
+     * -They only guarantee that the processors are not scheduled anymore
      * -After the threadpool is stopped we can make sure that processors don't need repos and controllers anymore */
     if (this->root_) {
       this->root_->drainConnections();
@@ -268,7 +248,7 @@ std::unique_ptr<core::ProcessGroup> FlowController::loadInitialFlow() {
   // since we don't have access to the flow definition, the C2 communication
   // won't be able to use the services defined there, e.g. SSLContextService
   controller_service_provider_impl_ = flow_configuration_->getControllerServiceProvider();
-  C2Client::initialize(this, this, shared_from_this());
+  C2Client::initialize(this, this, this);
   auto opt_source = fetchFlow(*opt_flow_url);
   if (!opt_source) {
     logger_->log_error("Couldn't fetch flow configuration from C2 server");
@@ -284,7 +264,7 @@ std::unique_ptr<core::ProcessGroup> FlowController::loadInitialFlow() {
   return root;
 }
 
-void FlowController::load(const std::shared_ptr<core::ProcessGroup> &root, bool reload) {
+void FlowController::load(std::unique_ptr<core::ProcessGroup> root, bool reload) {
   std::lock_guard<std::recursive_mutex> flow_lock(mutex_);
   if (running_) {
     stop();
@@ -296,10 +276,12 @@ void FlowController::load(const std::shared_ptr<core::ProcessGroup> &root, bool
 
     if (root) {
       logger_->log_info("Load Flow Controller from provided root");
-      this->root_ = root;
+      this->root_ = std::move(root);
+      processor_to_controller_.clear();
+      updateResponseNodeConnections();
     } else {
       logger_->log_info("Instantiating new flow");
-      this->root_ = std::shared_ptr<core::ProcessGroup>(loadInitialFlow());
+      this->root_ = loadInitialFlow();
     }
 
     if (root_) {
@@ -309,12 +291,11 @@ void FlowController::load(const std::shared_ptr<core::ProcessGroup> &root, bool
     logger_->log_info("Loaded root processor Group");
     logger_->log_info("Initializing timers");
     controller_service_provider_impl_ = flow_configuration_->getControllerServiceProvider();
-    auto base_shared_ptr = std::dynamic_pointer_cast<core::controller::ControllerServiceProvider>(shared_from_this());
 
     if (!thread_pool_.isRunning() || reload) {
       thread_pool_.shutdown();
       thread_pool_.setMaxConcurrentTasks(configuration_->getInt(Configure::nifi_flow_engine_threads, 2));
-      thread_pool_.setControllerServiceProvider(base_shared_ptr);
+      thread_pool_.setControllerServiceProvider(this);
       thread_pool_.start();
     }
 
@@ -322,7 +303,7 @@ void FlowController::load(const std::shared_ptr<core::ProcessGroup> &root, bool
     conditionalReloadScheduler<EventDrivenSchedulingAgent>(event_scheduler_, !event_scheduler_ || reload);
     conditionalReloadScheduler<CronDrivenSchedulingAgent>(cron_scheduler_, !cron_scheduler_ || reload);
 
-    std::static_pointer_cast<core::controller::StandardControllerServiceProvider>(controller_service_provider_impl_)->setRootGroup(root_);
+    std::static_pointer_cast<core::controller::StandardControllerServiceProvider>(controller_service_provider_impl_)->setRootGroup(root_.get());
     std::static_pointer_cast<core::controller::StandardControllerServiceProvider>(controller_service_provider_impl_)->setSchedulingAgent(
         std::static_pointer_cast<minifi::SchedulingAgent>(event_scheduler_));
 
@@ -350,8 +331,8 @@ void FlowController::load(const std::shared_ptr<core::ProcessGroup> &root, bool
 void FlowController::loadFlowRepo() {
   if (this->flow_file_repo_ != nullptr) {
     logger_->log_debug("Getting connection map");
-    std::map<std::string, std::shared_ptr<core::Connectable>> connectionMap;
-    std::map<std::string, std::shared_ptr<core::Connectable>> containers;
+    std::map<std::string, core::Connectable*> connectionMap;
+    std::map<std::string, core::Connectable*> containers;
     if (this->root_ != nullptr) {
       this->root_->getConnections(connectionMap);
       this->root_->getFlowFileContainers(containers);
@@ -383,7 +364,7 @@ int16_t FlowController::start() {
         // as the thread_pool_ is started in load()
         this->root_->startProcessing(timer_scheduler_, event_scheduler_, cron_scheduler_);
       }
-      C2Client::initialize(this, this, shared_from_this());
+      C2Client::initialize(this, this, this);
       running_ = true;
       this->protocol_->start();
       this->provenance_repo_->start();
@@ -433,7 +414,7 @@ int16_t FlowController::applyUpdate(const std::string &source, const std::string
 int16_t FlowController::clearConnection(const std::string &connection) {
   if (root_ != nullptr) {
     logger_->log_info("Attempting to clear connection %s", connection);
-    std::map<std::string, std::shared_ptr<Connection>> connections;
+    std::map<std::string, Connection*> connections;
     root_->getConnections(connections);
     auto conn = connections.find(connection);
     if (conn != connections.end()) {
@@ -444,63 +425,68 @@ int16_t FlowController::clearConnection(const std::string &connection) {
   return -1;
 }
 
-std::shared_ptr<state::response::ResponseNode> FlowController::getAgentManifest() const {
+std::shared_ptr<state::response::ResponseNode> FlowController::getAgentManifest() {
   auto agentInfo = std::make_shared<state::response::AgentInformation>("agentInfo");
+  agentInfo->setUpdatePolicyController(std::static_pointer_cast<controllers::UpdatePolicyControllerService>(getControllerService(c2::C2Agent::UPDATE_NAME)).get());
   agentInfo->setAgentIdentificationProvider(configuration_);
+  agentInfo->setConfigurationReader([this](const std::string& key){
+    return configuration_->getRawValue(key);
+  });
+  agentInfo->setStateMonitor(this);
   agentInfo->includeAgentStatus(false);
   return agentInfo;
 }
 
-std::vector<std::shared_ptr<state::StateController>> FlowController::getAllComponents() {
-  std::vector<std::shared_ptr<state::StateController>> vec;
-  vec.push_back(shared_from_this());
-  std::vector<std::shared_ptr<core::Processor>> processors;
-  if (root_ != nullptr) {
-    root_->getAllProcessors(processors);
-    for (auto &processor : processors) {
-      switch (processor->getSchedulingStrategy()) {
-        case core::SchedulingStrategy::TIMER_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, timer_scheduler_));
-          break;
-        case core::SchedulingStrategy::EVENT_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, event_scheduler_));
-          break;
-        case core::SchedulingStrategy::CRON_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, cron_scheduler_));
-          break;
-        default:
-          break;
-      }
-    }
+void FlowController::executeOnAllComponents(std::function<void(state::StateController&)> func) {
+  std::lock_guard<std::recursive_mutex> lock(mutex_);
+  for (auto* component: getAllComponents()) {
+    func(*component);
   }
-  return vec;
 }
-std::vector<std::shared_ptr<state::StateController>> FlowController::getComponents(const std::string &name) {
-  std::vector<std::shared_ptr<state::StateController>> vec;
 
-  if (name == "FlowController") {
-    vec.push_back(shared_from_this());
-  } else if (root_) {
-    // check processors
-    std::shared_ptr<core::Processor> processor = root_->findProcessorByName(name);
-    if (processor != nullptr) {
-      switch (processor->getSchedulingStrategy()) {
-        case core::SchedulingStrategy::TIMER_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, timer_scheduler_));
-          break;
-        case core::SchedulingStrategy::EVENT_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, event_scheduler_));
-          break;
-        case core::SchedulingStrategy::CRON_DRIVEN:
-          vec.push_back(std::make_shared<state::ProcessorController>(processor, cron_scheduler_));
-          break;
-        default:
-          break;
-      }
-    }
+void FlowController::executeOnComponent(const std::string &name, std::function<void(state::StateController&)> func) {
+  std::lock_guard<std::recursive_mutex> lock(mutex_);
+  if (auto* component = getComponent(name); component != nullptr) {
+    func(*component);
+  } else {
+    logger_->log_error("Could not get execute requested callback for component \"%s\", because component was not found", name);
+  }
+}
+
+std::vector<state::StateController*> FlowController::getAllComponents() {
+  if (root_) {
+    auto controllerFactory = [this] (core::Processor& p) {
+      return createController(p);
+    };
+    return getAllProcessorControllers(controllerFactory);
   }
 
-  return vec;
+  return {this};
+}
+
+state::StateController* FlowController::getComponent(const std::string& name) {
+  if (name == "FlowController") {
+    return this;
+  } else if (root_) {
+    auto controllerFactory = [this] (core::Processor& p) {
+      return createController(p);
+    };
+    return getProcessorController(name, controllerFactory);
+  }
+
+  return nullptr;
+}
+
+std::unique_ptr<state::ProcessorController> FlowController::createController(core::Processor& processor) {
+  switch (processor.getSchedulingStrategy()) {
+    case core::SchedulingStrategy::TIMER_DRIVEN:
+      return std::make_unique<state::ProcessorController>(&processor, timer_scheduler_);
+    case core::SchedulingStrategy::EVENT_DRIVEN:
+      return std::make_unique<state::ProcessorController>(&processor, event_scheduler_);
+    case core::SchedulingStrategy::CRON_DRIVEN:
+      return std::make_unique<state::ProcessorController>(&processor, cron_scheduler_);
+  }
+  return {};
 }
 
 uint64_t FlowController::getUptime() {
@@ -520,7 +506,50 @@ std::vector<BackTrace> FlowController::getTraces() {
   return traces;
 }
 
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+std::map<std::string, std::unique_ptr<io::InputStream>> FlowController::getDebugInfo() {
+  std::map<std::string, std::unique_ptr<io::InputStream>> debug_info;
+  if (auto logs = core::logging::LoggerConfiguration::getCompressedLog(true)) {
+    debug_info["minifi.log.gz"] = std::move(logs);
+  }
+  if (auto opt_flow_path = flow_configuration_->getConfigurationPath()) {
+    debug_info["config.yml"] = std::make_unique<io::FileStream>(opt_flow_path.value(), 0, false);
+  }
+  debug_info["minifi.properties"] = std::make_unique<io::FileStream>(configuration_->getFilePath(), 0, false);
+
+  return debug_info;
+}
+
+std::vector<state::StateController*> FlowController::getAllProcessorControllers(
+        const std::function<std::unique_ptr<state::ProcessorController>(core::Processor&)>& controllerFactory) {
+  std::vector<state::StateController*> controllerVec{this};
+  std::vector<core::Processor*> processorVec;
+  root_->getAllProcessors(processorVec);
+
+  for (const auto& processor : processorVec) {
+    // reference to the existing or newly created controller
+    auto& controller = processor_to_controller_[processor->getUUID()];
+    if (!controller) {
+      controller = controllerFactory(*processor);
+    }
+    controllerVec.push_back(controller.get());
+  }
+
+  return controllerVec;
+}
+
+state::StateController* FlowController::getProcessorController(const std::string& name, const std::function<std::unique_ptr<state::ProcessorController>(core::Processor&)>& controllerFactory) {
+  auto* processor = root_->findProcessorByName(name);
+  if (processor == nullptr) {
+    logger_->log_error("Could not get processor controller for requested name \"%s\", because processor was not found either", name);
+    return nullptr;
+  }
+
+  // reference to the existing or newly created controller
+  auto& foundController = processor_to_controller_[processor->getUUID()];
+  if (!foundController) {
+    foundController = controllerFactory(*processor);
+  }
+  return foundController.get();
+}
+
+}  // namespace org::apache::nifi::minifi
