@@ -46,6 +46,7 @@
 #include "core/Relationship.h"
 #include "utils/HTTPClient.h"
 
+#undef GetObject  // windows.h #defines GetObject = GetObjectA or GetObjectW, which conflicts with rapidjson
 namespace org {
 namespace apache {
 namespace nifi {
@@ -141,16 +142,6 @@ void RemoteProcessorGroupPort::onSchedule(const std::shared_ptr<core::ProcessCon
     protocol_uuid_ = value;
   }
 
-  std::string http_enabled_str;
-  if (configure_->get(Configure::nifi_remote_input_http, http_enabled_str)) {
-    http_enabled_ = utils::StringUtils::toBool(http_enabled_str).value_or(false);
-    if (http_enabled_) {
-      if (client_type_ == sitetosite::CLIENT_TYPE::RAW) {
-        logger_->log_trace("Remote Input HTTP Enabled, but raw has been suggested for %s", protocol_uuid_.to_string());
-      }
-    }
-  }
-
   std::string context_name;
   if (!context->getProperty(SSLContext.getName(), context_name) || IsNullOrEmpty(context_name)) {
     context_name = RPG_SSL_CONTEXT_SERVICE_NAME;
@@ -166,17 +157,12 @@ void RemoteProcessorGroupPort::onSchedule(const std::shared_ptr<core::ProcessCon
     }
   }
   {
-    uint64_t idleTimeoutVal = 15000;
-    std::string idleTimeoutStr;
-    if (!context->getProperty(idleTimeout.getName(), idleTimeoutStr)
-        || !core::Property::getTimeMSFromString(idleTimeoutStr, idleTimeoutVal)) {
-      logger_->log_debug("%s attribute is invalid, so default value of %s will be used", idleTimeout.getName(),
-                         idleTimeout.getValue());
-      if (!core::Property::getTimeMSFromString(idleTimeout.getValue(), idleTimeoutVal)) {
-        assert(false);  // Couldn't parse our default value
-      }
+    if (auto idle_timeout = context->getProperty<core::TimePeriodValue>(idleTimeout)) {
+      idle_timeout_ = idle_timeout->getMilliseconds();
+    } else {
+      logger_->log_debug("%s attribute is invalid, so default value of %s will be used", idleTimeout.getName(), idleTimeout.getDefaultValue());
+      idle_timeout_ = core::TimePeriodValue(idleTimeout.getDefaultValue().to_string()).getMilliseconds();
     }
-    idle_timeout_ = std::chrono::milliseconds(idleTimeoutVal);
   }
 
   std::lock_guard<std::mutex> lock(peer_mutex_);
@@ -412,7 +398,7 @@ void RemoteProcessorGroupPort::refreshPeerList() {
   if (protocol)
     protocol->getPeerList(peers_);
 
-  logging::LOG_INFO(logger_) << "Have " << peers_.size() << " peers";
+  core::logging::LOG_INFO(logger_) << "Have " << peers_.size() << " peers";
 
   if (peers_.size() > 0)
     peer_index_ = 0;
