@@ -23,22 +23,20 @@
 #include <utility>
 #include <memory>
 #include "core/Processor.h"
-#include "utils/GeneralUtils.h"
 #include "utils/gsl.h"
 
+namespace {
+bool hasWorkToDo(org::apache::nifi::minifi::core::Processor* processor) {
+  // Whether it has work to do
+  return processor->getTriggerWhenEmpty() || !processor->hasIncomingConnections() || processor->isWorkAvailable();
+}
+}  // namespace
+
 namespace org {
 namespace apache {
 namespace nifi {
 namespace minifi {
 
-bool SchedulingAgent::hasWorkToDo(const std::shared_ptr<core::Processor>& processor) {
-  // Whether it has work to do
-  if (processor->getTriggerWhenEmpty() || !processor->hasIncomingConnections() || processor->isWorkAvailable())
-    return true;
-  else
-    return false;
-}
-
 std::future<utils::TaskRescheduleInfo> SchedulingAgent::enableControllerService(std::shared_ptr<core::controller::ControllerServiceNode> &serviceNode) {
   logger_->log_info("Enabling CSN in SchedulingAgent %s", serviceNode->getName());
   // reference the enable function from serviceNode
@@ -48,7 +46,7 @@ std::future<utils::TaskRescheduleInfo> SchedulingAgent::enableControllerService(
     };
 
   // only need to run this once.
-  auto monitor = utils::make_unique<utils::ComplexMonitor>();
+  auto monitor = std::make_unique<utils::ComplexMonitor>();
   utils::Worker<utils::TaskRescheduleInfo> functor(f_ex, serviceNode->getUUIDStr(), std::move(monitor));
   // move the functor into the thread pool. While a future is returned
   // we aren't terribly concerned with the result.
@@ -68,7 +66,7 @@ std::future<utils::TaskRescheduleInfo> SchedulingAgent::disableControllerService
   };
 
   // only need to run this once.
-  auto monitor = utils::make_unique<utils::ComplexMonitor>();
+  auto monitor = std::make_unique<utils::ComplexMonitor>();
   utils::Worker<utils::TaskRescheduleInfo> functor(f_ex, serviceNode->getUUIDStr(), std::move(monitor));
 
   // move the functor into the thread pool. While a future is returned
@@ -80,11 +78,7 @@ std::future<utils::TaskRescheduleInfo> SchedulingAgent::disableControllerService
   return future;
 }
 
-bool SchedulingAgent::hasTooMuchOutGoing(const std::shared_ptr<core::Processor>& processor) {
-  return processor->flowFilesOutGoingFull();
-}
-
-bool SchedulingAgent::onTrigger(const std::shared_ptr<core::Processor> &processor, const std::shared_ptr<core::ProcessContext> &processContext,
+bool SchedulingAgent::onTrigger(core::Processor* processor, const std::shared_ptr<core::ProcessContext> &processContext,
                                 const std::shared_ptr<core::ProcessSessionFactory> &sessionFactory) {
   if (processor->isYield()) {
     logger_->log_debug("Not running %s since it must yield", processor->getName());
@@ -99,7 +93,7 @@ bool SchedulingAgent::onTrigger(const std::shared_ptr<core::Processor> &processo
     return true;
   }
   if (processor->isThrottledByBackpressure()) {
-    logger_->log_debug("backpressure applied because too much outgoing for %s", processor->getUUIDStr());
+    logger_->log_debug("backpressure applied because too much outgoing for %s %s", processor->getUUIDStr(), processor->getName());
     // need to apply backpressure
     return true;
   }
