@@ -18,15 +18,16 @@
 
 #include "c2/C2Agent.h"
 
+#include <cinttypes>
 #include <cstdio>
 #include <csignal>
-#include <utility>
 #include <limits>
-#include <vector>
 #include <map>
-#include <string>
 #include <memory>
-#include <cinttypes>
+#include <optional>
+#include <string>
+#include <utility>
+#include <vector>
 
 #include "c2/ControllerSocketProtocol.h"
 #include "core/ProcessContext.h"
@@ -34,43 +35,39 @@
 #include "core/state/UpdateController.h"
 #include "core/logging/Logger.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "utils/file/DiffUtils.h"
 #include "utils/file/FileUtils.h"
 #include "utils/file/FileManager.h"
 #include "utils/file/FileSystem.h"
-#include "utils/GeneralUtils.h"
 #include "utils/HTTPClient.h"
 #include "utils/Environment.h"
 #include "utils/Monitors.h"
-#include "utils/OptionalUtils.h"
 #include "utils/StringUtils.h"
+#include "io/ArchiveStream.h"
+#include "io/StreamPipe.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace c2 {
+using namespace std::literals::chrono_literals;
+
+namespace org::apache::nifi::minifi::c2 {
 
 C2Agent::C2Agent(core::controller::ControllerServiceProvider *controller,
                  state::Pausable *pause_handler,
-                 const std::shared_ptr<state::StateMonitor> &updateSink,
-                 const std::shared_ptr<Configure> &configuration,
-                 const std::shared_ptr<utils::file::FileSystem> &filesystem)
-    : heart_beat_period_(3000),
+                 state::StateMonitor* updateSink,
+                 std::shared_ptr<Configure> configuration,
+                 std::shared_ptr<utils::file::FileSystem> filesystem,
+                 std::function<void()> request_restart)
+    : heart_beat_period_(3s),
       max_c2_responses(5),
       update_sink_(updateSink),
       update_service_(nullptr),
       controller_(controller),
       pause_handler_(pause_handler),
-      configuration_(configuration),
-      filesystem_(filesystem),
+      configuration_(std::move(configuration)),
+      filesystem_(std::move(filesystem)),
       protocol_(nullptr),
-      logger_(logging::LoggerFactory<C2Agent>::getLogger()),
-      thread_pool_(2, false, nullptr, "C2 threadpool") {
+      thread_pool_(2, false, nullptr, "C2 threadpool"),
+      request_restart_(std::move(request_restart)) {
   manifest_sent_ = false;
 
-  running_c2_configuration = std::make_shared<Configure>();
-
   last_run_ = std::chrono::steady_clock::now();
 
   if (nullptr != controller_) {
@@ -81,7 +78,7 @@ C2Agent::C2Agent(core::controller::ControllerServiceProvider *controller,
     // create a stubbed service for updating the flow identifier
   }
 
-  configure(configuration, false);
+  configure(configuration_, false);
 
   functions_.emplace_back([this] {return produce();});
   functions_.emplace_back([this] {return consume();});
@@ -95,8 +92,7 @@ void C2Agent::start() {
   for (const auto& function : functions_) {
     utils::Identifier uuid = utils::IdGenerator::getIdGenerator()->generate();
     task_ids_.push_back(uuid);
-    auto monitor = utils::make_unique<utils::ComplexMonitor>();
-    utils::Worker<utils::TaskRescheduleInfo> functor(function, uuid.to_string(), std::move(monitor));
+    utils::Worker<utils::TaskRescheduleInfo> functor(function, uuid.to_string(), std::make_unique<utils::ComplexMonitor>());
     std::future<utils::TaskRescheduleInfo> future;
     thread_pool_.execute(std::move(functor), future);
   }
@@ -138,7 +134,7 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
   std::string clazz, heartbeat_period, device;
 
   if (!reconfigure) {
-    if (!configure->get("nifi.c2.agent.protocol.class", "c2.agent.protocol.class", clazz)) {
+    if (!configure->get(Configuration::nifi_c2_agent_protocol_class, "c2.agent.protocol.class", clazz)) {
       clazz = "RESTSender";
     }
     logger_->log_info("Class is %s", clazz);
@@ -150,7 +146,7 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
       if (!protocol) {
         const char* errmsg = "Attempted to load RESTSender. To enable C2, please specify an active protocol for this agent.";
         logger_->log_error(errmsg);
-        throw minifi::Exception{ minifi::GENERAL_EXCEPTION, errmsg };
+        throw Exception{ GENERAL_EXCEPTION, errmsg };
       }
 
       logger_->log_info("Class is RESTSender");
@@ -164,27 +160,24 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
     protocol_.load()->update(configure);
   }
 
-  if (configure->get("nifi.c2.agent.heartbeat.period", "c2.agent.heartbeat.period", heartbeat_period)) {
-    core::TimeUnit unit;
-
+  if (configure->get(Configuration::nifi_c2_agent_heartbeat_period, "c2.agent.heartbeat.period", heartbeat_period)) {
     try {
-      int64_t schedulingPeriod = 0;
-      if (core::Property::StringToTime(heartbeat_period, schedulingPeriod, unit) && core::Property::ConvertTimeUnitToMS(schedulingPeriod, unit, schedulingPeriod)) {
-        heart_beat_period_ = schedulingPeriod;
-        logger_->log_debug("Using %u ms as the heartbeat period", heart_beat_period_);
+      if (auto heartbeat_period_ms = utils::timeutils::StringToDuration<std::chrono::milliseconds>(heartbeat_period)) {
+        heart_beat_period_ = *heartbeat_period_ms;
+        logger_->log_debug("Using %u ms as the heartbeat period", heart_beat_period_.count());
       } else {
-        heart_beat_period_ = std::stoi(heartbeat_period);
+        heart_beat_period_ = std::chrono::milliseconds(std::stoi(heartbeat_period));
       }
     } catch (const std::invalid_argument &) {
-      heart_beat_period_ = 3000;
+      heart_beat_period_ = 3s;
     }
   } else {
     if (!reconfigure)
-      heart_beat_period_ = 3000;
+      heart_beat_period_ = 3s;
   }
 
   std::string heartbeat_reporters;
-  if (configure->get("nifi.c2.agent.heartbeat.reporter.classes", "c2.agent.heartbeat.reporter.classes", heartbeat_reporters)) {
+  if (configure->get(Configuration::nifi_c2_agent_heartbeat_reporter_classes, "c2.agent.heartbeat.reporter.classes", heartbeat_reporters)) {
     std::vector<std::string> reporters = utils::StringUtils::splitAndTrim(heartbeat_reporters, ",");
     std::lock_guard<std::mutex> lock(heartbeat_mutex);
     for (const auto& reporter : reporters) {
@@ -193,13 +186,13 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
         logger_->log_error("Could not instantiate %s", reporter);
       } else {
         heartbeat_reporter_obj->initialize(controller_, update_sink_, configuration_);
-        heartbeat_protocols_.push_back(heartbeat_reporter_obj);
+        heartbeat_protocols_.push_back(std::move(heartbeat_reporter_obj));
       }
     }
   }
 
   std::string trigger_classes;
-  if (configure->get("nifi.c2.agent.trigger.classes", "c2.agent.trigger.classes", trigger_classes)) {
+  if (configure->get(Configuration::nifi_c2_agent_trigger_classes, "c2.agent.trigger.classes", trigger_classes)) {
     std::vector<std::string> triggers = utils::StringUtils::splitAndTrim(trigger_classes, ",");
     std::lock_guard<std::mutex> lock(heartbeat_mutex);
     for (const auto& trigger : triggers) {
@@ -208,7 +201,7 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
         logger_->log_error("Could not instantiate %s", trigger);
       } else {
         trigger_obj->initialize(configuration_);
-        triggers_.push_back(trigger_obj);
+        triggers_.push_back(std::move(trigger_obj));
       }
     }
   }
@@ -219,14 +212,14 @@ void C2Agent::configure(const std::shared_ptr<Configure> &configure, bool reconf
     logger_->log_error("Could not instantiate %s", base_reporter);
   } else {
     heartbeat_reporter_obj->initialize(controller_, update_sink_, configuration_);
-    heartbeat_protocols_.push_back(heartbeat_reporter_obj);
+    heartbeat_protocols_.push_back(std::move(heartbeat_reporter_obj));
   }
 }
 
 void C2Agent::performHeartBeat() {
   C2Payload payload(Operation::HEARTBEAT);
   logger_->log_trace("Performing heartbeat");
-  std::shared_ptr<state::response::NodeReporter> reporter = std::dynamic_pointer_cast<state::response::NodeReporter>(update_sink_);
+  auto reporter = dynamic_cast<state::response::NodeReporter*>(update_sink_);
   std::vector<std::shared_ptr<state::response::ResponseNode>> metrics;
   if (reporter) {
     if (!manifest_sent_) {
@@ -246,22 +239,22 @@ void C2Agent::performHeartBeat() {
       payload.addPayload(std::move(child_metric_payload));
     }
   }
-  C2Payload && response = protocol_.load()->consumePayload(payload);
+  C2Payload response = protocol_.load()->consumePayload(payload);
 
   enqueue_c2_server_response(std::move(response));
 
   std::lock_guard<std::mutex> lock(heartbeat_mutex);
 
-  for (auto reporter : heartbeat_protocols_) {
+  for (auto& reporter : heartbeat_protocols_) {
     reporter->heartbeat(payload);
   }
 }
 
 void C2Agent::serializeMetrics(C2Payload &metric_payload, const std::string &name, const std::vector<state::response::SerializedResponseNode> &metrics, bool is_container, bool is_collapsible) {
-  const auto payloads = std::count_if(begin(metrics), end(metrics), [](const state::response::SerializedResponseNode& metric) { return !metric.children.empty(); });
+  const auto payloads = std::count_if(begin(metrics), end(metrics), [](const state::response::SerializedResponseNode& metric) { return !metric.children.empty() || metric.keep_empty; });
   metric_payload.reservePayloads(metric_payload.getNestedPayloads().size() + payloads);
   for (const auto &metric : metrics) {
-    if (metric.children.size() > 0) {
+    if (metric.children.size() > 0 || (metric.children.size() == 0 && metric.keep_empty)) {
       C2Payload child_metric_payload(metric_payload.getOperation());
       if (metric.array) {
         child_metric_payload.setContainer(true);
@@ -316,53 +309,31 @@ void C2Agent::extractPayload(const C2Payload &resp) {
     case state::UpdateState::READ_ERROR:
       logger_->log_debug("Received read error event from protocol");
       break;
+    case state::UpdateState::NO_OPERATION:
+      logger_->log_debug("Received no operation event from protocol");
+      break;
     default:
       logger_->log_error("Received unknown event (%d) from protocol", static_cast<int>(resp.getStatus().getState()));
       break;
   }
 }
 
+namespace {
+
+struct C2TransferError : public std::runtime_error {
+  using runtime_error::runtime_error;
+};
+
+struct C2DebugBundleError : public C2TransferError {
+  using C2TransferError::C2TransferError;
+};
+
+}  // namespace
+
 void C2Agent::handle_c2_server_response(const C2ContentResponse &resp) {
   switch (resp.op.value()) {
     case Operation::CLEAR:
-      // we've been told to clear something
-      if (resp.name == "connection") {
-        for (const auto& connection : resp.operation_arguments) {
-          logger_->log_debug("Clearing connection %s", connection.second.to_string());
-          update_sink_->clearConnection(connection.second.to_string());
-        }
-        C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-        enqueue_c2_response(std::move(response));
-      } else if (resp.name == "repositories") {
-        update_sink_->drainRepositories();
-        C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-        enqueue_c2_response(std::move(response));
-      } else if (resp.name == "corecomponentstate") {
-        // TODO(bakaid): untested
-        std::vector<std::shared_ptr<state::StateController>> components = update_sink_->getComponents(resp.name);
-        auto state_manager_provider = core::ProcessContext::getStateManagerProvider(logger_, controller_, configuration_);
-        if (state_manager_provider != nullptr) {
-          for (auto &component : components) {
-            logger_->log_debug("Clearing state for component %s", component->getComponentName());
-            auto state_manager = state_manager_provider->getCoreComponentStateManager(component->getComponentUUID());
-            if (state_manager != nullptr) {
-              component->stop();
-              state_manager->clear();
-              state_manager->persist();
-              component->start();
-            } else {
-              logger_->log_warn("Failed to get StateManager for component %s", component->getComponentUUID().to_string());
-            }
-          }
-        } else {
-          logger_->log_error("Failed to get StateManagerProvider");
-        }
-        C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-        enqueue_c2_response(std::move(response));
-      } else {
-        logger_->log_debug("Clearing unknown %s", resp.name);
-      }
-
+      handle_clear(resp);
       break;
     case Operation::UPDATE:
       handle_update(resp);
@@ -374,7 +345,7 @@ void C2Agent::handle_c2_server_response(const C2ContentResponse &resp) {
       update_sink_->stop();
       C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
       protocol_.load()->consumePayload(std::move(response));
-      restart_agent();
+      restart_needed_ = true;
     }
       break;
     case Operation::START:
@@ -383,15 +354,15 @@ void C2Agent::handle_c2_server_response(const C2ContentResponse &resp) {
         raise(SIGTERM);
       }
 
-      std::vector<std::shared_ptr<state::StateController>> components = update_sink_->getComponents(resp.name);
       // stop all referenced components.
-      for (auto &component : components) {
-        logger_->log_debug("Stopping component %s", component->getComponentName());
-        if (resp.op == Operation::STOP)
-          component->stop();
-        else
-          component->start();
-      }
+      update_sink_->executeOnComponent(resp.name, [this, &resp] (state::StateController& component) {
+        logger_->log_debug("Stopping component %s", component.getComponentName());
+        if (resp.op == Operation::STOP) {
+          component.stop();
+        } else {
+          component.start();
+        }
+      });
 
       if (resp.ident.length() > 0) {
         C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
@@ -414,6 +385,18 @@ void C2Agent::handle_c2_server_response(const C2ContentResponse &resp) {
         logger_->log_warn("Resume functionality is not supported!");
       }
       break;
+    case Operation::TRANSFER: {
+      try {
+        handle_transfer(resp);
+        C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
+        enqueue_c2_response(std::move(response));
+      } catch (const std::runtime_error& error) {
+        C2Payload response(Operation::ACKNOWLEDGE, state::UpdateState::SET_ERROR, resp.ident, true);
+        response.setRawData(error.what());
+        enqueue_c2_response(std::move(response));
+      }
+      break;
+    }
     default:
       break;
       // do nothing
@@ -442,165 +425,295 @@ C2Payload C2Agent::prepareConfigurationOptions(const C2ContentResponse &resp) co
     return response;
 }
 
+void C2Agent::handle_clear(const C2ContentResponse &resp) {
+  ClearOperand operand;
+  try {
+    operand = ClearOperand::parse(resp.name.c_str());
+  } catch(const std::runtime_error&) {
+    logger_->log_debug("Clearing unknown %s", resp.name);
+    return;
+  }
+
+  switch (operand.value()) {
+    case ClearOperand::CONNECTION: {
+      for (const auto& connection : resp.operation_arguments) {
+        logger_->log_debug("Clearing connection %s", connection.second.to_string());
+        update_sink_->clearConnection(connection.second.to_string());
+      }
+      break;
+    }
+    case ClearOperand::REPOSITORIES: {
+      update_sink_->drainRepositories();
+      break;
+    }
+    case ClearOperand::CORECOMPONENTSTATE: {
+      for (const auto& corecomponent : resp.operation_arguments) {
+        auto state_manager_provider = core::ProcessContext::getStateManagerProvider(logger_, controller_, configuration_);
+        if (state_manager_provider != nullptr) {
+          update_sink_->executeOnComponent(corecomponent.second.to_string(), [this, &state_manager_provider] (state::StateController& component) {
+            logger_->log_debug("Clearing state for component %s", component.getComponentName());
+            auto state_manager = state_manager_provider->getCoreComponentStateManager(component.getComponentUUID());
+            if (state_manager != nullptr) {
+              component.stop();
+              state_manager->clear();
+              state_manager->persist();
+              component.start();
+            } else {
+              logger_->log_warn("Failed to get StateManager for component %s", component.getComponentUUID().to_string());
+            }
+          });
+        } else {
+          logger_->log_error("Failed to get StateManagerProvider");
+        }
+      }
+      break;
+    }
+    default:
+      logger_->log_error("Unknown clear operand %s", resp.name);
+  }
+
+  C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
+  enqueue_c2_response(std::move(response));
+}
+
 /**
  * Descriptions are special types of requests that require information
  * to be put into the acknowledgement
  */
 void C2Agent::handle_describe(const C2ContentResponse &resp) {
-  auto reporter = std::dynamic_pointer_cast<state::response::NodeReporter>(update_sink_);
-  if (resp.name == "metrics") {
+  DescribeOperand operand;
+  try {
+    operand = DescribeOperand::parse(resp.name.c_str());
+  } catch(const std::runtime_error&) {
     C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-    if (reporter != nullptr) {
-      auto iter = resp.operation_arguments.find("metricsClass");
-      std::string metricsClass;
-      if (iter != resp.operation_arguments.end()) {
-        metricsClass = iter->second.to_string();
-      }
-      auto metricsNode = reporter->getMetricsNode(metricsClass);
-      C2Payload metrics(Operation::ACKNOWLEDGE);
-      metricsClass.empty() ? metrics.setLabel("metrics") : metrics.setLabel(metricsClass);
-      if (metricsNode) {
-        serializeMetrics(metrics, metricsNode->getName(), metricsNode->serialize(), metricsNode->isArray());
-      }
-      response.addPayload(std::move(metrics));
-    }
-    enqueue_c2_response(std::move(response));
-    return;
-  } else if (resp.name == "configuration") {
-    auto configOptions = prepareConfigurationOptions(resp);
-    enqueue_c2_response(std::move(configOptions));
-    return;
-  } else if (resp.name == "manifest") {
-    C2Payload response(prepareConfigurationOptions(resp));
-    if (reporter != nullptr) {
-      C2Payload agentInfo(Operation::ACKNOWLEDGE, resp.ident, true);
-      agentInfo.setLabel("agentInfo");
-
-      const auto manifest = reporter->getAgentManifest();
-      serializeMetrics(agentInfo, manifest->getName(), manifest->serialize());
-      response.addPayload(std::move(agentInfo));
-    }
-    enqueue_c2_response(std::move(response));
-    return;
-  } else if (resp.name == "jstack") {
-    if (update_sink_->isRunning()) {
-      const std::vector<BackTrace> traces = update_sink_->getTraces();
-      for (const auto &trace : traces) {
-        for (const auto & line : trace.getTraces()) {
-          logger_->log_trace("%s -- %s", trace.getName(), line);
-        }
-      }
-      auto keys = configuration_->getConfiguredKeys();
-      C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-      for (const auto &trace : traces) {
-        C2Payload options(Operation::ACKNOWLEDGE, resp.ident, true);
-        options.setLabel(trace.getName());
-        std::string value;
-        for (const auto &line : trace.getTraces()) {
-          C2ContentResponse option(Operation::ACKNOWLEDGE);
-          option.name = line;
-          option.operation_arguments[line] = line;
-          options.addContent(std::move(option));
-        }
-        response.addPayload(std::move(options));
-      }
-      enqueue_c2_response(std::move(response));
-      return;
-    }
-  } else if (resp.name == "corecomponentstate") {
-    C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-    response.setLabel("corecomponentstate");
-    C2Payload states(Operation::ACKNOWLEDGE, resp.ident, true);
-    states.setLabel("corecomponentstate");
-    auto state_manager_provider = core::ProcessContext::getStateManagerProvider(logger_, controller_, configuration_);
-    if (state_manager_provider != nullptr) {
-      auto core_component_states = state_manager_provider->getAllCoreComponentStates();
-      for (const auto& core_component_state : core_component_states) {
-        C2Payload state(Operation::ACKNOWLEDGE, resp.ident, true);
-        state.setLabel(core_component_state.first.to_string());
-        for (const auto& kv : core_component_state.second) {
-          C2ContentResponse entry(Operation::ACKNOWLEDGE);
-          entry.name = kv.first;
-          entry.operation_arguments[kv.first] = kv.second;
-          state.addContent(std::move(entry));
-        }
-        states.addPayload(std::move(state));
-      }
-    }
-    response.addPayload(std::move(states));
     enqueue_c2_response(std::move(response));
     return;
   }
-  C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
-  enqueue_c2_response(std::move(response));
+
+  auto reporter = dynamic_cast<state::response::NodeReporter*>(update_sink_);
+  switch (operand.value()) {
+    case DescribeOperand::METRICS: {
+      C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
+      if (reporter != nullptr) {
+        auto iter = resp.operation_arguments.find("metricsClass");
+        std::string metricsClass;
+        if (iter != resp.operation_arguments.end()) {
+          metricsClass = iter->second.to_string();
+        }
+        auto metricsNode = reporter->getMetricsNode(metricsClass);
+        C2Payload metrics(Operation::ACKNOWLEDGE);
+        metricsClass.empty() ? metrics.setLabel("metrics") : metrics.setLabel(metricsClass);
+        if (metricsNode) {
+          serializeMetrics(metrics, metricsNode->getName(), metricsNode->serialize(), metricsNode->isArray());
+        }
+        response.addPayload(std::move(metrics));
+      }
+      enqueue_c2_response(std::move(response));
+      break;
+    }
+    case DescribeOperand::CONFIGURATION: {
+      auto configOptions = prepareConfigurationOptions(resp);
+      enqueue_c2_response(std::move(configOptions));
+      break;
+    }
+    case DescribeOperand::MANIFEST: {
+      C2Payload response(prepareConfigurationOptions(resp));
+      if (reporter != nullptr) {
+        C2Payload agentInfo(Operation::ACKNOWLEDGE, resp.ident, true);
+        agentInfo.setLabel("agentInfo");
+
+        const auto manifest = reporter->getAgentManifest();
+        serializeMetrics(agentInfo, manifest->getName(), manifest->serialize());
+        response.addPayload(std::move(agentInfo));
+      }
+      enqueue_c2_response(std::move(response));
+      break;
+    }
+    case DescribeOperand::JSTACK: {
+      if (update_sink_->isRunning()) {
+        const std::vector<BackTrace> traces = update_sink_->getTraces();
+        for (const auto &trace : traces) {
+          for (const auto & line : trace.getTraces()) {
+            logger_->log_trace("%s -- %s", trace.getName(), line);
+          }
+        }
+        auto keys = configuration_->getConfiguredKeys();
+        C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
+        for (const auto &trace : traces) {
+          C2Payload options(Operation::ACKNOWLEDGE, resp.ident, true);
+          options.setLabel(trace.getName());
+          std::string value;
+          for (const auto &line : trace.getTraces()) {
+            C2ContentResponse option(Operation::ACKNOWLEDGE);
+            option.name = line;
+            option.operation_arguments[line] = line;
+            options.addContent(std::move(option));
+          }
+          response.addPayload(std::move(options));
+        }
+        enqueue_c2_response(std::move(response));
+      }
+      break;
+    }
+    case DescribeOperand::CORECOMPONENTSTATE: {
+      C2Payload response(Operation::ACKNOWLEDGE, resp.ident, true);
+      response.setLabel("corecomponentstate");
+      C2Payload states(Operation::ACKNOWLEDGE, resp.ident, true);
+      states.setLabel("corecomponentstate");
+      auto state_manager_provider = core::ProcessContext::getStateManagerProvider(logger_, controller_, configuration_);
+      if (state_manager_provider != nullptr) {
+        auto core_component_states = state_manager_provider->getAllCoreComponentStates();
+        for (const auto& core_component_state : core_component_states) {
+          C2Payload state(Operation::ACKNOWLEDGE, resp.ident, true);
+          state.setLabel(core_component_state.first.to_string());
+          for (const auto& kv : core_component_state.second) {
+            C2ContentResponse entry(Operation::ACKNOWLEDGE);
+            entry.name = kv.first;
+            entry.operation_arguments[kv.first] = kv.second;
+            state.addContent(std::move(entry));
+          }
+          states.addPayload(std::move(state));
+        }
+      }
+      response.addPayload(std::move(states));
+      enqueue_c2_response(std::move(response));
+      break;
+    }
+  }
 }
 
 void C2Agent::handle_update(const C2ContentResponse &resp) {
-  // we've been told to update something
-  if (resp.name == "configuration") {
-    handleConfigurationUpdate(resp);
-  } else if (resp.name == "properties") {
-    state::UpdateState result = state::UpdateState::FULLY_APPLIED;
-    for (auto entry : resp.operation_arguments) {
-      bool persist = (
-          entry.second.getAnnotation("persist")
-          | utils::map(&AnnotatedValue::to_string)
-          | utils::flatMap(utils::StringUtils::toBool)).value_or(false);
-      if (!update_property(entry.first, entry.second.to_string(), persist)) {
-        result = state::UpdateState::PARTIALLY_APPLIED;
-      }
-    }
-    C2Payload response(Operation::ACKNOWLEDGE, result, resp.ident, true);
-    enqueue_c2_response(std::move(response));
-  } else if (resp.name == "c2") {
-    // prior configuration options were already in place. thus
-    // we clear the map so that we don't go through replacing
-    // unnecessary objects.
-    running_c2_configuration->clear();
-
-    for (auto entry : resp.operation_arguments) {
-      bool can_update = true;
-      if (nullptr != update_service_) {
-        can_update = update_service_->canUpdate(entry.first);
-      }
-      if (can_update)
-        running_c2_configuration->set(entry.first, entry.second.to_string());
-    }
-
-    if (resp.operation_arguments.size() > 0)
-      configure(running_c2_configuration);
-    C2Payload response(Operation::ACKNOWLEDGE, state::UpdateState::FULLY_APPLIED, resp.ident, true);
-    enqueue_c2_response(std::move(response));
-  } else {
+  UpdateOperand operand;
+  try {
+    operand = UpdateOperand::parse(resp.name.c_str());
+  } catch(const std::runtime_error&) {
     C2Payload response(Operation::ACKNOWLEDGE, state::UpdateState::NOT_APPLIED, resp.ident, true);
     enqueue_c2_response(std::move(response));
+    return;
   }
+
+  switch (operand.value()) {
+    case UpdateOperand::CONFIGURATION: {
+      handleConfigurationUpdate(resp);
+      break;
+    }
+    case UpdateOperand::PROPERTIES: {
+      handlePropertyUpdate(resp);
+      break;
+    }
+  }
+}
+
+void C2Agent::handlePropertyUpdate(const C2ContentResponse &resp) {
+  state::UpdateState result = state::UpdateState::NO_OPERATION;
+  auto changeUpdateState = [&result](UpdateResult update_result) {
+    if (result == state::UpdateState::NO_OPERATION) {
+      if (update_result == UpdateResult::UPDATE_SUCCESSFUL) {
+        result = state::UpdateState::FULLY_APPLIED;
+      } else if (update_result == UpdateResult::UPDATE_FAILED) {
+        result = state::UpdateState::NOT_APPLIED;
+      }
+    } else if ((result == state::UpdateState::FULLY_APPLIED && update_result == UpdateResult::UPDATE_FAILED) ||
+               (result == state::UpdateState::NOT_APPLIED && update_result == UpdateResult::UPDATE_SUCCESSFUL)) {
+      result = state::UpdateState::PARTIALLY_APPLIED;
+    }
+  };
+
+  for (auto entry : resp.operation_arguments) {
+    bool persist = (
+        entry.second.getAnnotation("persist")
+        | utils::map(&AnnotatedValue::to_string)
+        | utils::flatMap(utils::StringUtils::toBool)).value_or(true);
+    PropertyChangeLifetime lifetime = persist ? PropertyChangeLifetime::PERSISTENT : PropertyChangeLifetime::TRANSIENT;
+    changeUpdateState(update_property(entry.first, entry.second.to_string(), lifetime));
+  }
+  // apply changes and persist properties requested to be persisted
+  const bool propertyWasUpdated = result == state::UpdateState::FULLY_APPLIED || result == state::UpdateState::PARTIALLY_APPLIED;
+  if (propertyWasUpdated && !configuration_->commitChanges()) {
+    result = state::UpdateState::PARTIALLY_APPLIED;
+  }
+  C2Payload response(Operation::ACKNOWLEDGE, result, resp.ident, true);
+  enqueue_c2_response(std::move(response));
+  if (propertyWasUpdated) { restart_needed_ = true; }
 }
 
 /**
  * Updates a property
  */
-bool C2Agent::update_property(const std::string &property_name, const std::string &property_value, bool persist) {
-  if (update_service_ && !update_service_->canUpdate(property_name)) {
-    return false;
+C2Agent::UpdateResult C2Agent::update_property(const std::string &property_name, const std::string &property_value, PropertyChangeLifetime lifetime) {
+  if (!Configuration::validatePropertyValue(property_name, property_value) ||
+      (update_service_ && !update_service_->canUpdate(property_name))) {
+    return UpdateResult::UPDATE_FAILED;
   }
-  configuration_->set(property_name, property_value);
-  if (!persist) {
-    return true;
+
+  std::string value;
+  if (configuration_->get(property_name, value) && value == property_value) {
+    return UpdateResult::NO_UPDATE;
   }
-  return configuration_->persistProperties();
+
+  configuration_->set(property_name, property_value, lifetime);
+  return UpdateResult::UPDATE_SUCCESSFUL;
 }
 
-void C2Agent::restart_agent() {
-  std::string cwd = utils::Environment::getCurrentWorkingDirectory();
-  if (cwd.empty()) {
-    logger_->log_error("Could not restart the agent because the working directory could not be determined");
-    return;
+C2Payload C2Agent::bundleDebugInfo(std::map<std::string, std::unique_ptr<io::InputStream>>& files) {
+  C2Payload payload(Operation::TRANSFER, false);
+  auto stream_provider = core::ClassLoader::getDefaultClassLoader().instantiate<io::ArchiveStreamProvider>(
+      "ArchiveStreamProvider", "ArchiveStreamProvider");
+  if (!stream_provider) {
+    throw C2DebugBundleError("Couldn't instantiate archiver provider");
+  }
+  auto bundle = std::make_shared<io::BufferStream>();
+  auto archiver = stream_provider->createWriteStream(9, "gzip", bundle, logger_);
+  if (!archiver) {
+    throw C2DebugBundleError("Couldn't instantiate archiver");
+  }
+  for (auto&[filename, stream] : files) {
+    size_t file_size = stream->size();
+    if (!archiver->newEntry({filename, file_size})) {
+      throw C2DebugBundleError("Couldn't initialize archive entry for '" + filename + "'");
+    }
+    if (gsl::narrow<int64_t>(file_size) != internal::pipe(*stream, *archiver)) {
+      // we have touched the input streams, they cannot be reused
+      throw C2DebugBundleError("Error while writing file '" + filename + "' into the debug bundle");
+    }
+  }
+  if (!archiver->finish()) {
+    throw C2DebugBundleError("Failed to complete debug bundle archive");
+  }
+  C2Payload file(Operation::TRANSFER, true);
+  file.setLabel("debug.tar.gz");
+  file.setRawData(bundle->moveBuffer());
+  payload.addPayload(std::move(file));
+  return payload;
+}
+
+void C2Agent::handle_transfer(const C2ContentResponse &resp) {
+  TransferOperand operand;
+  try {
+    operand = TransferOperand::parse(resp.name.c_str());
+  } catch(const std::runtime_error&) {
+    throw C2TransferError("Unknown operand '" + resp.name + "'");
   }
 
-  std::string command = cwd + "/bin/minifi.sh restart";
-  if (system(command.c_str()) != 0) {
-    logger_->log_error("System command '%s' failed", command);
+  switch (operand.value()) {
+    case TransferOperand::DEBUG: {
+      auto target_it = resp.operation_arguments.find("target");
+      if (target_it == resp.operation_arguments.end()) {
+        throw C2DebugBundleError("Missing argument for debug operation: 'target'");
+      }
+      std::optional<std::string> url = resolveUrl(target_it->second.to_string());
+      if (!url) {
+        throw C2DebugBundleError("Invalid url");
+      }
+      std::map<std::string, std::unique_ptr<io::InputStream>> files = update_sink_->getDebugInfo();
+
+      auto bundle = bundleDebugInfo(files);
+      C2Payload &&response = protocol_.load()->consumePayload(url.value(), bundle, TRANSMIT, false);
+      if (response.getStatus().getState() == state::UpdateState::READ_ERROR) {
+        throw C2DebugBundleError("Error while uploading");
+      }
+      break;
+    }
   }
 }
 
@@ -631,6 +744,12 @@ utils::TaskRescheduleInfo C2Agent::produce() {
           }
         });
 
+    if (restart_needed_ && requests.empty()) {
+      configuration_->commitChanges();
+      request_restart_();
+      return utils::TaskRescheduleInfo::Done();
+    }
+
     try {
       performHeartBeat();
     }
@@ -644,7 +763,7 @@ utils::TaskRescheduleInfo C2Agent::produce() {
 
   checkTriggers();
 
-  return utils::TaskRescheduleInfo::RetryIn(std::chrono::milliseconds(heart_beat_period_));
+  return utils::TaskRescheduleInfo::RetryIn(heart_beat_period_);
 }
 
 utils::TaskRescheduleInfo C2Agent::consume() {
@@ -659,10 +778,50 @@ utils::TaskRescheduleInfo C2Agent::consume() {
   return utils::TaskRescheduleInfo::RetryIn(std::chrono::milliseconds(C2RESPONSE_POLL_MS));
 }
 
-utils::optional<std::string> C2Agent::fetchFlow(const std::string& uri) const {
+std::optional<std::string> C2Agent::resolveFlowUrl(const std::string& url) const {
+  if (utils::StringUtils::startsWith(url, "http")) {
+    return url;
+  }
+  std::string base;
+  if (configuration_->get(Configure::nifi_c2_flow_base_url, base)) {
+    base = utils::StringUtils::trim(base);
+    if (!utils::StringUtils::endsWith(base, "/")) {
+      base += "/";
+    }
+    base += url;
+    return base;
+  } else if (configuration_->get(Configuration::nifi_c2_rest_url, "c2.rest.url", base)) {
+    utils::URL base_url{utils::StringUtils::trim(base)};
+    if (base_url.isValid()) {
+      return base_url.hostPort() + "/c2/api/" + url;
+    }
+    logger_->log_error("Could not parse C2 REST URL '%s'", base);
+    return std::nullopt;
+  }
+  return url;
+}
+
+std::optional<std::string> C2Agent::resolveUrl(const std::string& url) const {
+  if (!utils::StringUtils::startsWith(url, "/")) {
+    return url;
+  }
+  std::string base;
+  if (!configuration_->get(Configuration::nifi_c2_rest_url, "c2.rest.url", base)) {
+    logger_->log_error("Missing C2 REST URL");
+    return std::nullopt;
+  }
+  utils::URL base_url{utils::StringUtils::trim(base)};
+  if (base_url.isValid()) {
+    return base_url.hostPort() + url;
+  }
+  logger_->log_error("Could not parse C2 REST URL '%s'", base);
+  return std::nullopt;
+}
+
+std::optional<std::string> C2Agent::fetchFlow(const std::string& uri) const {
   if (!utils::StringUtils::startsWith(uri, "http") || protocol_.load() == nullptr) {
     // try to open the file
-    utils::optional<std::string> content = filesystem_->read(uri);
+    auto content = filesystem_->read(uri);
     if (content) {
       return content;
     }
@@ -672,33 +831,15 @@ utils::optional<std::string> C2Agent::fetchFlow(const std::string& uri) const {
     return {};
   }
 
-  std::string resolved_url = uri;
-  if (!utils::StringUtils::startsWith(uri, "http")) {
-    std::stringstream adjusted_url;
-    std::string base;
-    if (configuration_->get(minifi::Configure::nifi_c2_flow_base_url, base)) {
-      base = utils::StringUtils::trim(base);
-      adjusted_url << base;
-      if (!utils::StringUtils::endsWith(base, "/")) {
-        adjusted_url << "/";
-      }
-      adjusted_url << uri;
-      resolved_url = adjusted_url.str();
-    } else if (configuration_->get("nifi.c2.rest.url", "c2.rest.url", base)) {
-      utils::URL base_url{utils::StringUtils::trim(base)};
-      if (!base_url.isValid()) {
-        logger_->log_error("Could not parse C2 REST URL '%s'", base);
-        return {};
-      }
-      resolved_url = base_url.hostPort() + "/c2/api/" + uri;
-    }
+  std::optional<std::string> resolved_url = resolveFlowUrl(uri);
+  if (!resolved_url) {
+    return std::nullopt;
   }
 
   C2Payload payload(Operation::TRANSFER, true);
-  C2Payload &&response = protocol_.load()->consumePayload(resolved_url, payload, RECEIVE, false);
+  C2Payload &&response = protocol_.load()->consumePayload(resolved_url.value(), payload, RECEIVE, false);
 
-  auto raw_data = response.getRawData();
-  return std::string(raw_data.data(), raw_data.size());
+  return response.getRawDataAsString();
 }
 
 bool C2Agent::handleConfigurationUpdate(const C2ContentResponse &resp) {
@@ -709,7 +850,7 @@ bool C2Agent::handleConfigurationUpdate(const C2ContentResponse &resp) {
 
   if (url != resp.operation_arguments.end()) {
     file_uri = url->second.to_string();
-    utils::optional<std::string> optional_configuration_str = fetchFlow(file_uri);
+    std::optional<std::string> optional_configuration_str = fetchFlow(file_uri);
     if (!optional_configuration_str) {
       logger_->log_error("Couldn't load new flow configuration from: \"%s\"", file_uri);
       C2Payload response(Operation::ACKNOWLEDGE, state::UpdateState::SET_ERROR, resp.ident, true);
@@ -753,14 +894,16 @@ bool C2Agent::handleConfigurationUpdate(const C2ContentResponse &resp) {
 
   if (should_persist) {
     // update the flow id
-    configuration_->persistProperties();
+    configuration_->commitChanges();
   }
 
   return true;
 }
 
-}  // namespace c2
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+void C2Agent::enqueue_c2_server_response(C2Payload &&resp) {
+  logger_->log_trace("Server response: %s", [&] {return resp.str();});
+
+  responses.enqueue(std::move(resp));
+}
+
+}  // namespace org::apache::nifi::minifi::c2
