@@ -26,6 +26,7 @@
 
 #include "utils/gsl.h"
 #include "utils/StringUtils.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -33,7 +34,7 @@ namespace nifi {
 namespace minifi {
 namespace c2 {
 
-void ControllerSocketProtocol::initialize(core::controller::ControllerServiceProvider* controller, const std::shared_ptr<state::StateMonitor> &updateSink,
+void ControllerSocketProtocol::initialize(core::controller::ControllerServiceProvider* controller, state::StateMonitor* updateSink,
                                           const std::shared_ptr<Configure> &configuration) {
   HeartbeatReporter::initialize(controller, updateSink, configuration);
   stream_factory_ = minifi::io::StreamFactory::getInstance(configuration);
@@ -94,12 +95,11 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
         case Operation::START:
         {
           std::string componentStr;
-          int size = stream->read(componentStr);
-          if ( size != -1 ) {
-            auto components = update_sink_->getComponents(componentStr);
-            for (const auto& component : components) {
-              component->start();
-            }
+          const auto size = stream->read(componentStr);
+          if (!io::isError(size)) {
+            update_sink_->executeOnComponent(componentStr, [](state::StateController& component) {
+              component.start();
+            });
           } else {
             logger_->log_debug("Connection broke");
           }
@@ -108,12 +108,11 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
         case Operation::STOP:
         {
           std::string componentStr;
-          int size = stream->read(componentStr);
-          if ( size != -1 ) {
-            auto components = update_sink_->getComponents(componentStr);
-            for (const auto& component : components) {
-              component->stop();
-            }
+          const auto size = stream->read(componentStr);
+          if (!io::isError(size)) {
+            update_sink_->executeOnComponent(componentStr, [](state::StateController& component) {
+              component.stop();
+            });
           } else {
             logger_->log_debug("Connection broke");
           }
@@ -122,8 +121,8 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
         case Operation::CLEAR:
         {
           std::string connection;
-          int size = stream->read(connection);
-          if ( size != -1 ) {
+          const auto size = stream->read(connection);
+          if (!io::isError(size)) {
             update_sink_->clearConnection(connection);
           }
         }
@@ -131,21 +130,25 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
         case Operation::UPDATE:
         {
           std::string what;
-          int size = stream->read(what);
-          if (size == -1) {
-            logger_->log_debug("Connection broke");
-            break;
-          }
-          if (what == "flow") {
-            std::string ff_loc;
-            int size = stream->read(ff_loc);
-            std::ifstream tf(ff_loc);
-            std::string configuration((std::istreambuf_iterator<char>(tf)),
-                std::istreambuf_iterator<char>());
-            if (size == -1) {
+          {
+            const auto size = stream->read(what);
+            if (io::isError(size)) {
               logger_->log_debug("Connection broke");
               break;
             }
+          }
+          if (what == "flow") {
+            std::string ff_loc;
+            {
+              const auto size = stream->read(ff_loc);
+              if (io::isError(size)) {
+                logger_->log_debug("Connection broke");
+                break;
+              }
+            }
+            std::ifstream tf(ff_loc);
+            std::string configuration((std::istreambuf_iterator<char>(tf)),
+                std::istreambuf_iterator<char>());
             update_sink_->applyUpdate("ControllerSocketProtocol", configuration);
           }
         }
@@ -153,15 +156,15 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
         case Operation::DESCRIBE:
         {
           std::string what;
-          int size = stream->read(what);
-          if (size == -1) {
+          const auto size = stream->read(what);
+          if (io::isError(size)) {
             logger_->log_debug("Connection broke");
             break;
           }
           if (what == "queue") {
             std::string connection;
-            int size = stream->read(connection);
-            if (size == -1) {
+            const auto size_ = stream->read(connection);
+            if (io::isError(size_)) {
               logger_->log_debug("Connection broke");
               break;
             }
@@ -173,17 +176,21 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
             io::BufferStream resp;
             resp.write(&head, 1);
             resp.write(response.str());
-            stream->write(const_cast<uint8_t*>(resp.getBuffer()), gsl::narrow<int>(resp.size()));
+            stream->write(resp.getBuffer());
           } else if (what == "components") {
+            std::vector<std::pair<std::string, bool>> components;
+            update_sink_->executeOnAllComponents([&components](state::StateController& component){
+              components.emplace_back(component.getComponentName(), component.isRunning());
+            });
             io::BufferStream resp;
             resp.write(&head, 1);
-            uint16_t size = gsl::narrow<uint16_t>(update_sink_->getAllComponents().size());
-            resp.write(size);
-            for (const auto &component : update_sink_->getAllComponents()) {
-              resp.write(component->getComponentName());
-              resp.write(component->isRunning() ? "true" : "false");
+            resp.write(gsl::narrow<uint16_t>(components.size()));
+            for (const auto& [name, isRunning] : components) {
+              resp.write(name);
+              resp.write(isRunning ? "true" : "false");
             }
-            stream->write(const_cast<uint8_t*>(resp.getBuffer()), gsl::narrow<int>(resp.size()));
+
+            stream->write(resp.getBuffer());
           } else if (what == "jstack") {
             io::BufferStream resp;
             resp.write(&head, 1);
@@ -199,34 +206,34 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
                 resp.write(line);
               }
             }
-            stream->write(const_cast<uint8_t*>(resp.getBuffer()), gsl::narrow<int>(resp.size()));
+            stream->write(resp.getBuffer());
           } else if (what == "connections") {
             io::BufferStream resp;
             resp.write(&head, 1);
-            uint16_t size = gsl::narrow<uint16_t>(queue_full_.size());
-            resp.write(size);
+            const auto size_ = gsl::narrow<uint16_t>(queue_full_.size());
+            resp.write(size_);
             for (const auto &connection : queue_full_) {
               resp.write(connection.first, false);
             }
-            stream->write(const_cast<uint8_t*>(resp.getBuffer()), gsl::narrow<int>(resp.size()));
+            stream->write(resp.getBuffer());
           } else if (what == "getfull") {
             std::vector<std::string> full_connections;
             {
               std::lock_guard<std::mutex> lock(controller_mutex_);
-              for (auto conn : queue_full_) {
-                if (conn.second == true) {
+              for (const auto& conn : queue_full_) {
+                if (conn.second) {
                   full_connections.push_back(conn.first);
                 }
               }
             }
             io::BufferStream resp;
             resp.write(&head, 1);
-            uint16_t full_connection_count = gsl::narrow<uint16_t>(full_connections.size());
+            const auto full_connection_count = gsl::narrow<uint16_t>(full_connections.size());
             resp.write(full_connection_count);
-            for (auto conn : full_connections) {
+            for (const auto& conn : full_connections) {
               resp.write(conn);
             }
-            stream->write(const_cast<uint8_t*>(resp.getBuffer()), gsl::narrow<int>(resp.size()));
+            stream->write(resp.getBuffer());
           }
         }
         break;
@@ -241,10 +248,10 @@ void ControllerSocketProtocol::initialize(core::controller::ControllerServicePro
 void ControllerSocketProtocol::parse_content(const std::vector<C2ContentResponse> &content) {
   for (const auto &payload_content : content) {
     if (payload_content.name == "Components") {
-      for (auto content : payload_content.operation_arguments) {
-        bool is_enabled = minifi::utils::StringUtils::toBool(content.second.to_string()).value_or(false);
+      for (const auto& operation_argument : payload_content.operation_arguments) {
+        bool is_enabled = minifi::utils::StringUtils::toBool(operation_argument.second.to_string()).value_or(false);
         std::lock_guard<std::mutex> lock(controller_mutex_);
-        component_map_[content.first] = is_enabled;
+        component_map_[operation_argument.first] = is_enabled;
       }
     }
   }
@@ -253,21 +260,20 @@ void ControllerSocketProtocol::parse_content(const std::vector<C2ContentResponse
 int16_t ControllerSocketProtocol::heartbeat(const C2Payload &payload) {
   if (server_socket_ == nullptr)
     return 0;
-  const std::vector<C2ContentResponse> &content = payload.getContent();
   for (const auto &pc : payload.getNestedPayloads()) {
     if (pc.getLabel() == "flowInfo" || pc.getLabel() == "metrics") {
       for (const auto &metrics_payload : pc.getNestedPayloads()) {
         if (metrics_payload.getLabel() == "QueueMetrics" || metrics_payload.getLabel() == "queues") {
           for (const auto &queue_metrics : metrics_payload.getNestedPayloads()) {
-            auto metric_content = queue_metrics.getContent();
-            for (const auto &payload_content : queue_metrics.getContent()) {
+            const auto& metric_content = queue_metrics.getContent();
+            for (const auto &payload_content : metric_content) {
               uint64_t size = 0;
               uint64_t max = 0;
-              for (auto content : payload_content.operation_arguments) {
+              for (const auto& content : payload_content.operation_arguments) {
                 if (content.first == "datasize") {
-                  size = std::stol(content.second.to_string());
+                  size = std::stoull(content.second.to_string());
                 } else if (content.first == "datasizemax") {
-                  max = std::stol(content.second.to_string());
+                  max = std::stoull(content.second.to_string());
                 }
               }
               std::lock_guard<std::mutex> lock(controller_mutex_);
@@ -285,7 +291,7 @@ int16_t ControllerSocketProtocol::heartbeat(const C2Payload &payload) {
     }
   }
 
-  parse_content(content);
+  parse_content(payload.getContent());
 
   std::vector<uint8_t> buffer;
   buffer.resize(1024);
@@ -293,6 +299,8 @@ int16_t ControllerSocketProtocol::heartbeat(const C2Payload &payload) {
   return 0;
 }
 
+REGISTER_RESOURCE(ControllerSocketProtocol, "Creates a reporter that can handle basic c2 operations for a localized environment through a simple TCP socket.");
+
 }  // namespace c2
 }  // namespace minifi
 }  // namespace nifi
