@@ -16,6 +16,7 @@
  */
 
 #include "c2/HeartbeatJsonSerializer.h"
+
 #include "rapidjson/document.h"
 #include "rapidjson/writer.h"
 #include "rapidjson/stringbuffer.h"
@@ -46,6 +47,8 @@ static void serializeOperationInfo(rapidjson::Value& target, const C2Payload& pa
         return "PARTIALLY_APPLIED";
       case state::UpdateState::READ_ERROR:
         return "OPERATION_NOT_UNDERSTOOD";
+      case state::UpdateState::NO_OPERATION:
+        return "NO_OPERATION";
       case state::UpdateState::SET_ERROR:
       default:
         return "NOT_APPLIED";
@@ -55,9 +58,7 @@ static void serializeOperationInfo(rapidjson::Value& target, const C2Payload& pa
   rapidjson::Value state(rapidjson::kObjectType);
 
   state.AddMember("state", rapidjson::Value(state_str.c_str(), alloc), alloc);
-  const auto details = payload.getRawData();
-
-  state.AddMember("details", rapidjson::Value(std::string{details.data(), details.size()}.c_str(), alloc), alloc);
+  state.AddMember("details", rapidjson::Value(payload.getRawDataAsString().c_str(), alloc), alloc);
 
   target.AddMember("operationState", state, alloc);
   target.AddMember("identifier", rapidjson::Value(id.c_str(), alloc), alloc);
@@ -222,13 +223,10 @@ struct NamedValue {
       }
       return;
     }
-    rapidjson::Value& val = [&] () -> rapidjson::Value& {
-      return values[0].IsObject() && values[0].HasMember(member_key) ? values[0][member_key] : values[0];
-    }();
     if (target.IsArray()) {
-      target.PushBack(val, alloc);
+      target.PushBack(values[0], alloc);
     } else {
-      target.AddMember(member_key, val, alloc);
+      target.AddMember(member_key, values[0], alloc);
     }
   }
 };
