@@ -18,27 +18,23 @@
 
 #ifdef WIN32
 #pragma push_macro("GetObject")
-#undef GetObject
+#undef GetObject  // windows.h #defines GetObject = GetObjectA or GetObjectW, which conflicts with rapidjson
 #endif
 
 #include "c2/protocols/RESTProtocol.h"
 
-#include <algorithm>
 #include <list>
-#include <map>
 #include <memory>
 #include <string>
 #include <utility>
-#include <vector>
 
 #include "core/TypedValues.h"
 #include "utils/gsl.h"
+#include "properties/Configuration.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace c2 {
+#undef GetObject  // windows.h #defines GetObject = GetObjectA or GetObjectW, which conflicts with rapidjson
+
+namespace org::apache::nifi::minifi::c2 {
 
 
 AnnotatedValue parseAnnotatedValue(const rapidjson::Value& jsonValue) {
@@ -59,11 +55,12 @@ AnnotatedValue parseAnnotatedValue(const rapidjson::Value& jsonValue) {
   return result;
 }
 
-const C2Payload RESTProtocol::parseJsonResponse(const C2Payload &payload, const std::vector<char> &response) {
+C2Payload RESTProtocol::parseJsonResponse(const C2Payload &payload, gsl::span<const std::byte> response) {
   rapidjson::Document root;
+  const auto char_span = response.as_span<const char>();
 
   try {
-    rapidjson::ParseResult ok = root.Parse(response.data(), response.size());
+    rapidjson::ParseResult ok = root.Parse(char_span.data(), char_span.size());
     if (ok) {
       std::string identifier;
       for (auto key : {"operationid", "operationId", "identifier"}) {
@@ -142,15 +139,15 @@ const C2Payload RESTProtocol::parseJsonResponse(const C2Payload &payload, const
   return C2Payload(payload.getOperation(), state::UpdateState::READ_COMPLETE);
 }
 
-RESTProtocol::RESTProtocol() : logger_(logging::LoggerFactory<RESTProtocol>::getLogger()) {}
+RESTProtocol::RESTProtocol() = default;
 
 void RESTProtocol::initialize(core::controller::ControllerServiceProvider* /*controller*/, const std::shared_ptr<Configure> &configure) {
   if (configure) {
     std::string value_str;
-    if (configure->get("nifi.c2.rest.heartbeat.minimize.updates", "c2.rest.heartbeat.minimize.updates", value_str)) {
+    if (configure->get(minifi::Configuration::nifi_c2_rest_heartbeat_minimize_updates, "c2.rest.heartbeat.minimize.updates", value_str)) {
       auto opt_value = utils::StringUtils::toBool(value_str);
       if (!opt_value) {
-        logger_->log_error("Cannot convert '%s' to bool for property '%s'", value_str, "nifi.c2.rest.heartbeat.minimize.updates");
+        logger_->log_error("Cannot convert '%s' to bool for property '%s'", value_str, minifi::Configuration::nifi_c2_rest_heartbeat_minimize_updates);
         minimize_updates_ = false;
       } else {
         minimize_updates_ = opt_value.value();
@@ -180,8 +177,4 @@ bool RESTProtocol::containsPayload(const C2Payload &o) {
 #ifdef WIN32
 #pragma pop_macro("GetObject")
 #endif
-}  // namespace c2
-}  // namespace minifi
-}  // namespace nifi
-}  // namespace apache
-}  // namespace org
+}  // namespace org::apache::nifi::minifi::c2
