@@ -17,9 +17,11 @@
  */
 
 #include "core/FlowConfiguration.h"
+
 #include <memory>
 #include <vector>
 #include <string>
+
 #include "core/ClassLoader.h"
 #include "utils/StringUtils.h"
 #include "processors/ProcessorUtils.h"
@@ -30,6 +32,37 @@ namespace nifi {
 namespace minifi {
 namespace core {
 
+FlowConfiguration::FlowConfiguration(
+    std::shared_ptr<core::Repository> /*repo*/, std::shared_ptr<core::Repository> flow_file_repo,
+    std::shared_ptr<core::ContentRepository> content_repo, std::shared_ptr<io::StreamFactory> stream_factory,
+    std::shared_ptr<Configure> configuration, const std::optional<std::string>& path,
+    std::shared_ptr<utils::file::FileSystem> filesystem)
+    : CoreComponent(core::getClassName<FlowConfiguration>()),
+      flow_file_repo_(std::move(flow_file_repo)),
+      content_repo_(std::move(content_repo)),
+      stream_factory_(std::move(stream_factory)),
+      configuration_(configuration),
+      filesystem_(std::move(filesystem)),
+      logger_(logging::LoggerFactory<FlowConfiguration>::getLogger()) {
+  controller_services_ = std::make_shared<core::controller::ControllerServiceMap>();
+  service_provider_ = std::make_shared<core::controller::StandardControllerServiceProvider>(controller_services_, nullptr, configuration);
+  std::string flowUrl = "", bucket_id = "default", flowId = "";
+  configuration->get(Configure::nifi_c2_flow_id, flowId);
+  configuration->get(Configure::nifi_c2_flow_url, flowUrl);
+  flow_version_ = std::make_shared<state::response::FlowVersion>(flowUrl, bucket_id, flowId);
+
+  if (!path) {
+    logger_->log_error("Configuration path is not specified.");
+  } else {
+    config_path_ = utils::file::PathUtils::canonicalize(*path);
+    if (!config_path_) {
+      logger_->log_error("Couldn't find config file \"%s\".", *path);
+      config_path_ = path;
+    }
+    checksum_calculator_.setFileLocation(*config_path_);
+  }
+}
+
 static_initializers &get_static_functions() {
   static static_initializers static_sl_funcs;
   return static_sl_funcs;
@@ -42,7 +75,7 @@ FlowConfiguration::~FlowConfiguration() {
   }
 }
 
-std::shared_ptr<core::Processor> FlowConfiguration::createProcessor(const std::string &name, const utils::Identifier &uuid) {
+std::unique_ptr<core::Processor> FlowConfiguration::createProcessor(const std::string &name, const utils::Identifier &uuid) {
   auto processor = minifi::processors::ProcessorUtils::createProcessor(name, name, uuid, stream_factory_);
   if (nullptr == processor) {
     logger_->log_error("No Processor defined for %s", name);
@@ -51,7 +84,7 @@ std::shared_ptr<core::Processor> FlowConfiguration::createProcessor(const std::s
   return processor;
 }
 
-std::shared_ptr<core::Processor> FlowConfiguration::createProcessor(const std::string &name, const std::string &fullname, const utils::Identifier &uuid) {
+std::unique_ptr<core::Processor> FlowConfiguration::createProcessor(const std::string &name, const std::string &fullname, const utils::Identifier &uuid) {
   auto processor = minifi::processors::ProcessorUtils::createProcessor(name, fullname, uuid, stream_factory_);
   if (nullptr == processor) {
     logger_->log_error("No Processor defined for %s", fullname);
@@ -60,9 +93,8 @@ std::shared_ptr<core::Processor> FlowConfiguration::createProcessor(const std::s
   return processor;
 }
 
-std::shared_ptr<core::Processor> FlowConfiguration::createProvenanceReportTask() {
-  std::shared_ptr<core::Processor> processor = std::make_shared<org::apache::nifi::minifi::core::reporting::SiteToSiteProvenanceReportingTask>(stream_factory_, this->configuration_);
-  // initialize the processor
+std::unique_ptr<core::reporting::SiteToSiteProvenanceReportingTask> FlowConfiguration::createProvenanceReportTask() {
+  auto processor = std::make_unique<org::apache::nifi::minifi::core::reporting::SiteToSiteProvenanceReportingTask>(stream_factory_, this->configuration_);
   processor->initialize();
   return processor;
 }
@@ -109,23 +141,25 @@ bool FlowConfiguration::persist(const std::string &configuration) {
     logger_->log_debug("Copy %s to %s", *config_path_, config_file_backup);
   }
 
-  return filesystem_->write(*config_path_, configuration);
+  const bool status = filesystem_->write(*config_path_, configuration);
+  checksum_calculator_.invalidateChecksum();
+  return status;
 }
 
 std::unique_ptr<core::ProcessGroup> FlowConfiguration::createRootProcessGroup(const std::string &name, const utils::Identifier &uuid, int version) {
-  return utils::make_unique<core::ProcessGroup>(core::ROOT_PROCESS_GROUP, name, uuid, version);
+  return std::make_unique<core::ProcessGroup>(core::ROOT_PROCESS_GROUP, name, uuid, version);
 }
 
 std::unique_ptr<core::ProcessGroup> FlowConfiguration::createSimpleProcessGroup(const std::string &name, const utils::Identifier &uuid, int version) {
-  return utils::make_unique<core::ProcessGroup>(core::SIMPLE_PROCESS_GROUP, name, uuid, version);
+  return std::make_unique<core::ProcessGroup>(core::SIMPLE_PROCESS_GROUP, name, uuid, version);
 }
 
 std::unique_ptr<core::ProcessGroup> FlowConfiguration::createRemoteProcessGroup(const std::string &name, const utils::Identifier &uuid) {
-  return utils::make_unique<core::ProcessGroup>(core::REMOTE_PROCESS_GROUP, name, uuid);
+  return std::make_unique<core::ProcessGroup>(core::REMOTE_PROCESS_GROUP, name, uuid);
 }
 
-std::shared_ptr<minifi::Connection> FlowConfiguration::createConnection(const std::string& name, const utils::Identifier& uuid) const {
-  return std::make_shared<minifi::Connection>(flow_file_repo_, content_repo_, name, uuid);
+std::unique_ptr<minifi::Connection> FlowConfiguration::createConnection(const std::string& name, const utils::Identifier& uuid) const {
+  return std::make_unique<minifi::Connection>(flow_file_repo_, content_repo_, name, uuid);
 }
 
 std::shared_ptr<core::controller::ControllerServiceNode> FlowConfiguration::createControllerService(const std::string &class_name, const std::string &full_class_name, const std::string &name,
