@@ -31,6 +31,7 @@
 #include <vector>
 
 #include "core/ProcessSessionReadCallback.h"
+#include "io/StreamSlice.h"
 #include "utils/gsl.h"
 
 /* This implementation is only for native Windows systems.  */
@@ -59,9 +60,30 @@ namespace nifi {
 namespace minifi {
 namespace core {
 
+std::string detail::to_string(const detail::ReadBufferResult& read_buffer_result) {
+  return std::string(reinterpret_cast<const char*>(read_buffer_result.buffer.data()), read_buffer_result.buffer.size());
+}
+
 std::shared_ptr<utils::IdGenerator> ProcessSession::id_generator_ = utils::IdGenerator::getIdGenerator();
 
+ProcessSession::ProcessSession(std::shared_ptr<ProcessContext> processContext)
+        : process_context_(std::move(processContext)),
+          logger_(logging::LoggerFactory<ProcessSession>::getLogger()),
+          stateManager_(process_context_->hasStateManager() ? process_context_->getStateManager() : nullptr) {
+  logger_->log_trace("ProcessSession created for %s", process_context_->getProcessorNode()->getName());
+  auto repo = process_context_->getProvenanceRepository();
+  provenance_report_ = std::make_shared<provenance::ProvenanceReporter>(repo, process_context_->getProcessorNode()->getName(), process_context_->getProcessorNode()->getName());
+  content_session_ = process_context_->getContentRepository()->createSession();
+
+  if (stateManager_ && !stateManager_->beginTransaction()) {
+    throw Exception(PROCESS_SESSION_EXCEPTION, "State manager transaction could not be initiated.");
+  }
+}
+
 ProcessSession::~ProcessSession() {
+  if (stateManager_ && stateManager_->isTransactionInProgress()) {
+    logger_->log_error("Session has ended without decision on state (commit or rollback).");
+  }
   removeReferences();
 }
 
@@ -210,17 +232,22 @@ void ProcessSession::transfer(const std::shared_ptr<core::FlowFile> &flow, Relat
   flow->setDeleted(false);
 }
 
-void ProcessSession::write(const std::shared_ptr<core::FlowFile> &flow, OutputStreamCallback *callback) {
+void ProcessSession::write(const std::shared_ptr<core::FlowFile> &flow, const io::OutputStreamCallback& callback) {
+  auto flow_file_equality_checker = [&flow](const auto& flow_file) { return flow == flow_file; };
+  gsl_ExpectsAudit(_updatedFlowFiles.contains(flow->getUUID())
+      || _addedFlowFiles.contains(flow->getUUID())
+      || std::any_of(_clonedFlowFiles.begin(), _clonedFlowFiles.end(), flow_file_equality_checker));
+
   std::shared_ptr<ResourceClaim> claim = content_session_->create();
 
   try {
-    uint64_t startTime = utils::timeutils::getTimeMillis();
+    auto start_time = std::chrono::steady_clock::now();
     std::shared_ptr<io::BaseStream> stream = content_session_->write(claim);
     // Call the callback to write the content
     if (nullptr == stream) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to open flowfile content for write");
     }
-    if (callback->process(stream) < 0) {
+    if (callback(stream) < 0) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to process flowfile content");
     }
 
@@ -230,8 +257,8 @@ void ProcessSession::write(const std::shared_ptr<core::FlowFile> &flow, OutputSt
 
     stream->close();
     std::string details = process_context_->getProcessorNode()->getName() + " modify flow record content " + flow->getUUIDStr();
-    uint64_t endTime = utils::timeutils::getTimeMillis();
-    provenance_report_->modifyContent(flow, details, endTime - startTime);
+    auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start_time);
+    provenance_report_->modifyContent(flow, details, duration);
   } catch (std::exception &exception) {
     logger_->log_debug("Caught Exception %s", exception.what());
     throw;
@@ -242,18 +269,21 @@ void ProcessSession::write(const std::shared_ptr<core::FlowFile> &flow, OutputSt
 }
 
 void ProcessSession::writeBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const char> buffer) {
-  struct BufferOutputStreamCallback : OutputStreamCallback {
-    explicit BufferOutputStreamCallback(gsl::span<const char> buffer) :buffer{buffer} {}
-    int64_t process(const std::shared_ptr<io::BaseStream>& stream) final {
-      return stream->write(reinterpret_cast<const uint8_t*>(buffer.data()), buffer.size());
-    }
-    gsl::span<const char> buffer;
-  };
-  BufferOutputStreamCallback cb{ buffer };
-  write(flow_file, &cb);
+  writeBuffer(flow_file, buffer.as_span<const std::byte>());
+}
+void ProcessSession::writeBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const std::byte> buffer) {
+  write(flow_file, [buffer](const std::shared_ptr<io::BaseStream>& output_stream) {
+    const auto write_status = output_stream->write(buffer);
+    return io::isError(write_status) ? -1 : gsl::narrow<int64_t>(write_status);
+  });
 }
 
-void ProcessSession::append(const std::shared_ptr<core::FlowFile> &flow, OutputStreamCallback *callback) {
+void ProcessSession::append(const std::shared_ptr<core::FlowFile> &flow, const io::OutputStreamCallback& callback) {
+  auto flow_file_equality_checker = [&flow](const auto& flow_file) { return flow == flow_file; };
+  gsl_ExpectsAudit(_updatedFlowFiles.contains(flow->getUUID())
+      || _addedFlowFiles.contains(flow->getUUID())
+      || std::any_of(_clonedFlowFiles.begin(), _clonedFlowFiles.end(), flow_file_equality_checker));
+
   std::shared_ptr<ResourceClaim> claim = flow->getResourceClaim();
   if (!claim) {
     // No existed claim for append, we need to create new claim
@@ -261,26 +291,27 @@ void ProcessSession::append(const std::shared_ptr<core::FlowFile> &flow, OutputS
   }
 
   try {
-    uint64_t startTime = utils::timeutils::getTimeMillis();
+    auto start_time = std::chrono::steady_clock::now();
     std::shared_ptr<io::BaseStream> stream = content_session_->write(claim, ContentSession::WriteMode::APPEND);
     if (nullptr == stream) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to open flowfile content for append");
     }
     // Call the callback to write the content
 
-    size_t oldPos = stream->size();
+    size_t flow_file_size = flow->getSize();
+    size_t stream_size_before_callback = stream->size();
     // this prevents an issue if we write, above, with zero length.
-    if (oldPos > 0)
-      stream->seek(oldPos + 1);
-    if (callback->process(stream) < 0) {
+    if (stream_size_before_callback > 0)
+      stream->seek(stream_size_before_callback + 1);
+    if (callback(stream) < 0) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to process flowfile content");
     }
-    flow->setSize(stream->size());
+    flow->setSize(flow_file_size + (stream->size() - stream_size_before_callback));
 
     std::stringstream details;
     details << process_context_->getProcessorNode()->getName() << " modify flow record content " << flow->getUUIDStr();
-    uint64_t endTime = utils::timeutils::getTimeMillis();
-    provenance_report_->modifyContent(flow, details.str(), endTime - startTime);
+    auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start_time);
+    provenance_report_->modifyContent(flow, details.str(), duration);
   } catch (std::exception &exception) {
     logger_->log_debug("Caught Exception %s", exception.what());
     throw;
@@ -289,8 +320,17 @@ void ProcessSession::append(const std::shared_ptr<core::FlowFile> &flow, OutputS
     throw;
   }
 }
+void ProcessSession::appendBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const char> buffer) {
+  appendBuffer(flow_file, buffer.as_span<const std::byte>());
+}
+void ProcessSession::appendBuffer(const std::shared_ptr<core::FlowFile>& flow_file, gsl::span<const std::byte> buffer) {
+  append(flow_file, [buffer](const std::shared_ptr<io::BaseStream>& output_stream) {
+    const auto write_status = output_stream->write(buffer);
+    return io::isError(write_status) ? -1 : gsl::narrow<int64_t>(write_status);
+  });
+}
 
-int ProcessSession::read(const std::shared_ptr<core::FlowFile> &flow, InputStreamCallback *callback) {
+int64_t ProcessSession::read(const std::shared_ptr<core::FlowFile> &flow, const io::InputStreamCallback& callback) {
   try {
     std::shared_ptr<ResourceClaim> claim = nullptr;
 
@@ -311,13 +351,12 @@ int ProcessSession::read(const std::shared_ptr<core::FlowFile> &flow, InputStrea
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to open flowfile content for read");
     }
 
-    stream->seek(flow->getOffset());
-
-    auto ret = callback->process(stream);
+    auto flow_file_stream = std::make_shared<io::StreamSlice>(stream, flow->getOffset(), flow->getSize());
+    auto ret = callback(flow_file_stream);
     if (ret < 0) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Failed to process flowfile content");
     }
-    return gsl::narrow<int>(ret);
+    return ret;
   } catch (std::exception &exception) {
     logger_->log_debug("Caught Exception %s", exception.what());
     throw;
@@ -327,6 +366,68 @@ int ProcessSession::read(const std::shared_ptr<core::FlowFile> &flow, InputStrea
   }
 }
 
+
+int64_t ProcessSession::readWrite(const std::shared_ptr<core::FlowFile> &flow, const io::InputOutputStreamCallback& callback) {
+  gsl_Expects(callback);
+
+  try {
+    if (flow->getResourceClaim() == nullptr) {
+      logger_->log_debug("For %s, no resource claim but size is %d", flow->getUUIDStr(), flow->getSize());
+      if (flow->getSize() == 0) {
+        return 0;
+      }
+      throw Exception(FILE_OPERATION_EXCEPTION, "No Content Claim existed for read");
+    }
+
+    std::shared_ptr<ResourceClaim> input_claim = flow->getResourceClaim();
+    std::shared_ptr<io::BaseStream> input_stream = content_session_->read(input_claim);
+    if (!input_stream) {
+      throw Exception(FILE_OPERATION_EXCEPTION, "Failed to open flowfile content for read");
+    }
+    input_stream->seek(flow->getOffset());
+
+    std::shared_ptr<ResourceClaim> output_claim = content_session_->create();
+    std::shared_ptr<io::BaseStream> output_stream = content_session_->write(output_claim);
+    if (!output_stream) {
+      throw Exception(FILE_OPERATION_EXCEPTION, "Failed to open flowfile content for write");
+    }
+
+    int64_t bytes_written = callback(input_stream, output_stream);
+    if (bytes_written < 0) {
+      throw Exception(FILE_OPERATION_EXCEPTION, "Failed to process flowfile content");
+    }
+
+    input_stream->close();
+    output_stream->close();
+
+    flow->setSize(gsl::narrow<uint64_t>(bytes_written));
+    flow->setOffset(0);
+    flow->setResourceClaim(output_claim);
+
+    return bytes_written;
+  } catch (const std::exception& exception) {
+    logger_->log_debug("Caught exception %s during process session readWrite", exception.what());
+    throw;
+  } catch (...) {
+    logger_->log_debug("Caught unknown exception during process session readWrite");
+    throw;
+  }
+}
+
+detail::ReadBufferResult ProcessSession::readBuffer(const std::shared_ptr<core::FlowFile>& flow) {
+  detail::ReadBufferResult result;
+  result.status = read(flow, [&result, this](const std::shared_ptr<io::BaseStream>& input_stream) {
+    result.buffer.resize(input_stream->size());
+    const auto read_status = input_stream->read(result.buffer);
+    if (read_status != result.buffer.size()) {
+      logger_->log_error("readBuffer: %zu bytes were requested from the stream but %zu bytes were read. Rolling back.", result.buffer.size(), read_status);
+      throw Exception(PROCESSOR_EXCEPTION, "Failed to read the entire FlowFile.");
+    }
+    return gsl::narrow<int64_t>(read_status);
+  });
+  return result;
+}
+
 void ProcessSession::importFrom(io::InputStream&& stream, const std::shared_ptr<core::FlowFile> &flow) {
   importFrom(stream, flow);
 }
@@ -337,24 +438,25 @@ void ProcessSession::importFrom(io::InputStream&& stream, const std::shared_ptr<
  *
  */
 void ProcessSession::importFrom(io::InputStream &stream, const std::shared_ptr<core::FlowFile> &flow) {
-  std::shared_ptr<ResourceClaim> claim = content_session_->create();
-  int max_read = getpagesize();
-  std::vector<uint8_t> charBuffer(max_read);
+  const std::shared_ptr<ResourceClaim> claim = content_session_->create();
+  const auto max_read = gsl::narrow_cast<size_t>(getpagesize());
+  std::vector<std::byte> buffer(max_read);
 
   try {
-    auto startTime = utils::timeutils::getTimeMillis();
+    auto start_time = std::chrono::steady_clock::now();
     std::shared_ptr<io::BaseStream> content_stream = content_session_->write(claim);
 
     if (nullptr == content_stream) {
       throw Exception(FILE_OPERATION_EXCEPTION, "Could not obtain claim for " + claim->getContentFullPath());
     }
-    int position = 0;
-    const int max_size = gsl::narrow<int>(stream.size());
+    size_t position = 0;
+    const auto max_size = stream.size();
     while (position < max_size) {
-      const int read_size = std::min(max_read, max_size - position);
-      stream.read(charBuffer, read_size);
+      const auto read_size = std::min(max_read, max_size - position);
+      const auto subbuffer = gsl::make_span(buffer).subspan(0, read_size);
+      stream.read(subbuffer);
 
-      content_stream->write(charBuffer.data(), read_size);
+      content_stream->write(subbuffer);
       position += read_size;
     }
     // Open the source file and stream to the flow file
@@ -369,8 +471,8 @@ void ProcessSession::importFrom(io::InputStream &stream, const std::shared_ptr<c
     content_stream->close();
     std::stringstream details;
     details << process_context_->getProcessorNode()->getName() << " modify flow record content " << flow->getUUIDStr();
-    auto endTime = utils::timeutils::getTimeMillis();
-    provenance_report_->modifyContent(flow, details.str(), endTime - startTime);
+    auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start_time);
+    provenance_report_->modifyContent(flow, details.str(), duration);
   } catch (std::exception &exception) {
     logger_->log_debug("Caught Exception %s", exception.what());
     throw;
@@ -386,7 +488,7 @@ void ProcessSession::import(std::string source, const std::shared_ptr<FlowFile>
   std::vector<uint8_t> charBuffer(size);
 
   try {
-    auto startTime = utils::timeutils::getTimeMillis();
+    auto start_time = std::chrono::steady_clock::now();
     std::ifstream input;
     input.open(source.c_str(), std::fstream::in | std::fstream::binary);
     std::shared_ptr<io::BaseStream> stream = content_session_->write(claim);
@@ -407,12 +509,12 @@ void ProcessSession::import(std::string source, const std::shared_ptr<FlowFile>
       while (input.good()) {
         input.read(reinterpret_cast<char*>(charBuffer.data()), size);
         if (input) {
-          if (stream->write(charBuffer.data(), gsl::narrow<int>(size)) < 0) {
+          if (io::isError(stream->write(charBuffer.data(), size))) {
             invalidWrite = true;
             break;
           }
         } else {
-          if (stream->write(reinterpret_cast<uint8_t*>(charBuffer.data()), gsl::narrow<int>(input.gcount())) < 0) {
+          if (io::isError(stream->write(reinterpret_cast<uint8_t*>(charBuffer.data()), gsl::narrow<size_t>(input.gcount())))) {
             invalidWrite = true;
             break;
           }
@@ -433,8 +535,8 @@ void ProcessSession::import(std::string source, const std::shared_ptr<FlowFile>
           std::remove(source.c_str());
         std::stringstream details;
         details << process_context_->getProcessorNode()->getName() << " modify flow record content " << flow->getUUIDStr();
-        auto endTime = utils::timeutils::getTimeMillis();
-        provenance_report_->modifyContent(flow, details.str(), endTime - startTime);
+        auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start_time);
+        provenance_report_->modifyContent(flow, details.str(), duration);
       } else {
         stream->close();
         input.close();
@@ -471,7 +573,6 @@ void ProcessSession::import(const std::string& source, std::vector<std::shared_p
         throw Exception(FILE_OPERATION_EXCEPTION, utils::StringUtils::join_pack("File Import Error: Couldn't seek to offset ", std::to_string(offset)));
       }
     }
-    uint64_t startTime = 0U;
     while (input.good()) {
       input.read(reinterpret_cast<char*>(buffer.data()), buffer.size());
       std::streamsize read = input.gcount();
@@ -487,9 +588,9 @@ void ProcessSession::import(const std::string& source, std::vector<std::shared_p
       uint8_t* begin = buffer.data();
       uint8_t* end = begin + read;
       while (true) {
-        startTime = utils::timeutils::getTimeMillis();
+        auto start_time = std::chrono::steady_clock::now();
         uint8_t* delimiterPos = std::find(begin, end, static_cast<uint8_t>(inputDelimiter));
-        const auto len = gsl::narrow<int>(delimiterPos - begin);
+        const auto len = gsl::narrow<size_t>(delimiterPos - begin);
 
         logging::LOG_TRACE(logger_) << "Read input of " << read << " length is " << len << " is at end?" << (delimiterPos == end);
         /*
@@ -503,7 +604,7 @@ void ProcessSession::import(const std::string& source, std::vector<std::shared_p
 
         /* Create claim and stream if needed and append data */
         if (claim == nullptr) {
-          startTime = utils::timeutils::getTimeMillis();
+          start_time = std::chrono::steady_clock::now();
           claim = content_session_->create();
         }
         if (stream == nullptr) {
@@ -531,8 +632,8 @@ void ProcessSession::import(const std::string& source, std::vector<std::shared_p
                                     << ", FlowFile UUID " << flowFile->getUUIDStr();
         stream->close();
         std::string details = process_context_->getProcessorNode()->getName() + " modify flow record content " + flowFile->getUUIDStr();
-        uint64_t endTime = utils::timeutils::getTimeMillis();
-        provenance_report_->modifyContent(flowFile, details, endTime - startTime);
+        auto duration = std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - start_time);
+        provenance_report_->modifyContent(flowFile, details, duration);
         flows.push_back(flowFile);
 
         /* Reset these to start processing the next FlowFile with a clean slate */
@@ -583,7 +684,7 @@ bool ProcessSession::exportContent(const std::string &destination, const std::st
   logger_->log_debug("Exporting content of %s to %s", flow->getUUIDStr(), destination);
 
   ProcessSessionReadCallback cb(tmpFile, destination, logger_);
-  read(flow, &cb);
+  read(flow, std::ref(cb));
 
   logger_->log_info("Committing %s", destination);
   bool commit_ok = cb.commit();
@@ -656,7 +757,7 @@ ProcessSession::RouteResult ProcessSession::routeFlowFile(const std::shared_ptr<
   }
   Relationship relationship = itRelationship->second;
   // Find the relationship, we need to find the connections for that relationship
-  std::set<std::shared_ptr<Connectable>> connections = process_context_->getProcessorNode()->getOutGoingConnections(relationship.getName());
+  const auto connections = process_context_->getProcessorNode()->getOutGoingConnections(relationship.getName());
   if (connections.empty()) {
     // No connection
     if (!process_context_->getProcessorNode()->isAutoTerminated(relationship)) {
@@ -670,7 +771,7 @@ ProcessSession::RouteResult ProcessSession::routeFlowFile(const std::shared_ptr<
   } else {
     // We connections, clone the flow and assign the connection accordingly
     for (auto itConnection = connections.begin(); itConnection != connections.end(); ++itConnection) {
-      std::shared_ptr<Connectable> connection = *itConnection;
+      auto connection = *itConnection;
       if (itConnection == connections.begin()) {
         // First connection which the flow need be routed to
         record->setConnection(connection);
@@ -707,9 +808,9 @@ void ProcessSession::commit() {
       }
     }
 
-    std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<FlowFile>>> connectionQueues;
+    std::map<Connectable*, std::vector<std::shared_ptr<FlowFile>>> connectionQueues;
 
-    std::shared_ptr<Connectable> connection = nullptr;
+    Connectable* connection = nullptr;
     // Complete process the added and update flow files for the session, send the flow file to its queue
     for (const auto &it : _updatedFlowFiles) {
       auto record = it.second.modified;
@@ -747,23 +848,27 @@ void ProcessSession::commit() {
     }
 
     for (const auto& record : _deletedFlowFiles) {
-        if (!record->isDeleted()) {
-          continue;
-        }
-        if (record->isStored() && process_context_->getFlowFileRepository()->Delete(record->getUUIDStr())) {
-          // mark for deletion in the flowFileRepository
-          record->setStoredToRepository(false);
-        }
+      if (!record->isDeleted()) {
+        continue;
+      }
+      if (record->isStored() && process_context_->getFlowFileRepository()->Delete(record->getUUIDStr())) {
+        // mark for deletion in the flowFileRepository
+        record->setStoredToRepository(false);
+      }
     }
 
     ensureNonNullResourceClaim(connectionQueues);
 
     content_session_->commit();
 
+    if (stateManager_ && !stateManager_->commit()) {
+      throw Exception(PROCESS_SESSION_EXCEPTION, "State manager commit failed.");
+    }
+
     persistFlowFilesBeforeTransfer(connectionQueues, _updatedFlowFiles);
 
     for (auto& cq : connectionQueues) {
-      auto connection = std::dynamic_pointer_cast<Connection>(cq.first);
+      auto connection = dynamic_cast<Connection*>(cq.first);
       if (connection) {
         connection->multiPut(cq.second);
       } else {
@@ -795,7 +900,7 @@ void ProcessSession::commit() {
 void ProcessSession::rollback() {
   // new FlowFiles are only persisted during commit
   // no need to delete them here
-  std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<FlowFile>>> connectionQueues;
+  std::map<Connectable*, std::vector<std::shared_ptr<FlowFile>>> connectionQueues;
 
   try {
     // Requeue the snapshot of the flowfile back
@@ -817,7 +922,7 @@ void ProcessSession::rollback() {
 
     // put everything back where it came from
     for (auto& cq : connectionQueues) {
-      auto connection = std::dynamic_pointer_cast<Connection>(cq.first);
+      auto connection = dynamic_cast<Connection*>(cq.first);
       if (connection) {
         connection->multiPut(cq.second);
       } else {
@@ -829,6 +934,10 @@ void ProcessSession::rollback() {
 
     content_session_->rollback();
 
+    if (stateManager_ && !stateManager_->rollback()) {
+      throw Exception(PROCESS_SESSION_EXCEPTION, "State manager rollback failed.");
+    }
+
     _clonedFlowFiles.clear();
     _addedFlowFiles.clear();
     _updatedFlowFiles.clear();
@@ -844,7 +953,7 @@ void ProcessSession::rollback() {
 }
 
 void ProcessSession::persistFlowFilesBeforeTransfer(
-    std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<core::FlowFile> > >& transactionMap,
+    std::map<Connectable*, std::vector<std::shared_ptr<core::FlowFile> > >& transactionMap,
     const std::map<utils::Identifier, FlowFileUpdate>& modifiedFlowFiles) {
 
   std::vector<std::pair<std::string, std::unique_ptr<io::BufferStream>>> flowData;
@@ -852,11 +961,9 @@ void ProcessSession::persistFlowFilesBeforeTransfer(
   auto flowFileRepo = process_context_->getFlowFileRepository();
   auto contentRepo = process_context_->getContentRepository();
 
-  for (auto& transaction : transactionMap) {
-    const std::shared_ptr<Connectable>& target = transaction.first;
-    std::shared_ptr<Connection> connection = std::dynamic_pointer_cast<Connection>(target);
-    const bool shouldDropEmptyFiles = connection ? connection->getDropEmptyFlowFiles() : false;
-    auto& flows = transaction.second;
+  for (auto& [target, flows] : transactionMap) {
+    const auto connection = dynamic_cast<Connection*>(target);
+    const bool shouldDropEmptyFiles = connection && connection->getDropEmptyFlowFiles();
     for (auto &ff : flows) {
       if (shouldDropEmptyFiles && ff->getSize() == 0) {
         // the receiver will drop this FF
@@ -875,11 +982,9 @@ void ProcessSession::persistFlowFilesBeforeTransfer(
     throw Exception(PROCESS_SESSION_EXCEPTION, "Failed to put flowfiles to repository");
   }
 
-  for (auto& transaction : transactionMap) {
-    const std::shared_ptr<Connectable>& target = transaction.first;
-    std::shared_ptr<Connection> connection = std::dynamic_pointer_cast<Connection>(target);
-    const bool shouldDropEmptyFiles = connection ? connection->getDropEmptyFlowFiles() : false;
-    auto& flows = transaction.second;
+  for (auto& [target, flows] : transactionMap) {
+    const auto connection = dynamic_cast<Connection*>(target);
+    const bool shouldDropEmptyFiles = connection && connection->getDropEmptyFlowFiles();
     for (auto &ff : flows) {
       utils::Identifier uuid = ff->getUUID();
       auto snapshotIt = modifiedFlowFiles.find(uuid);
@@ -907,7 +1012,7 @@ void ProcessSession::persistFlowFilesBeforeTransfer(
 }
 
 void ProcessSession::ensureNonNullResourceClaim(
-    const std::map<std::shared_ptr<Connectable>, std::vector<std::shared_ptr<core::FlowFile>>> &transactionMap) {
+    const std::map<Connectable*, std::vector<std::shared_ptr<core::FlowFile>>> &transactionMap) {
   for (auto& transaction : transactionMap) {
     for (auto& flowFile : transaction.second) {
       auto claim = flowFile->getResourceClaim();
@@ -915,22 +1020,27 @@ void ProcessSession::ensureNonNullResourceClaim(
         logger_->log_debug("Processor %s (%s) did not create a ResourceClaim, creating an empty one",
                            process_context_->getProcessorNode()->getUUIDStr(),
                            process_context_->getProcessorNode()->getName());
-        OutputStreamPipe emptyStreamCallback(std::make_shared<io::BufferStream>());
-        write(flowFile, &emptyStreamCallback);
+        io::BufferStream emptyBufferStream;
+        write(flowFile, OutputStreamPipe{emptyBufferStream});
       }
     }
   }
 }
 
 std::shared_ptr<core::FlowFile> ProcessSession::get() {
-  std::shared_ptr<Connectable> first = process_context_->getProcessorNode()->pickIncomingConnection();
+  const auto first = process_context_->getProcessorNode()->pickIncomingConnection();
 
   if (first == nullptr) {
     logger_->log_trace("Get is null for %s", process_context_->getProcessorNode()->getName());
     return nullptr;
   }
 
-  std::shared_ptr<Connection> current = std::static_pointer_cast<Connection>(first);
+  auto current = dynamic_cast<Connection*>(first);
+  if (!current) {
+    logger_->log_error("The incoming connection [%s] of the processor [%s] \"%s\" is not actually a Connection.",
+                       first->getUUIDStr(), process_context_->getProcessorNode()->getUUIDStr(), process_context_->getProcessorNode()->getName());
+    return {};
+  }
 
   do {
     std::set<std::shared_ptr<core::FlowFile> > expired;
@@ -961,7 +1071,7 @@ std::shared_ptr<core::FlowFile> ProcessSession::get() {
       }
       return ret;
     }
-    current = std::static_pointer_cast<Connection>(process_context_->getProcessorNode()->pickIncomingConnection());
+    current = dynamic_cast<Connection*>(process_context_->getProcessorNode()->pickIncomingConnection());
   } while (current != nullptr && current != first);
 
   return nullptr;
@@ -972,10 +1082,10 @@ void ProcessSession::flushContent() {
 }
 
 bool ProcessSession::outgoingConnectionsFull(const std::string& relationship) {
-  std::set<std::shared_ptr<Connectable>> connections = process_context_->getProcessorNode()->getOutGoingConnections(relationship);
+  std::set<Connectable*> connections = process_context_->getProcessorNode()->getOutGoingConnections(relationship);
   Connection * connection = nullptr;
-  for (const auto& conn : connections) {
-    connection = dynamic_cast<Connection*>(conn.get());
+  for (const auto conn : connections) {
+    connection = dynamic_cast<Connection*>(conn);
     if (connection && connection->isFull()) {
       return true;
     }
