@@ -18,12 +18,14 @@
  * limitations under the License.
  */
 #include "core/ProcessSessionReadCallback.h"
+#include <array>
 #include <cstdio>
 #include <memory>
 #include <string>
 
 #include "core/logging/LoggerConfiguration.h"
 #include "io/BaseStream.h"
+#include "utils/gsl.h"
 
 namespace org {
 namespace apache {
@@ -41,26 +43,22 @@ ProcessSessionReadCallback::ProcessSessionReadCallback(const std::string &tmpFil
 }
 
 // Copy the entire file contents to the temporary file
-int64_t ProcessSessionReadCallback::process(const std::shared_ptr<io::BaseStream>& stream) {
+int64_t ProcessSessionReadCallback::operator()(const std::shared_ptr<io::BaseStream>& stream) {
   // Copy file contents into tmp file
   _writeSucceeded = false;
   size_t size = 0;
-  uint8_t buffer[8192];
+  std::array<std::byte, 8192> buffer{};
   do {
-    int read = stream->read(buffer, 8192);
-    if (read < 0) {
-      return -1;
-    }
-    if (read == 0) {
-      break;
-    }
-    if (!_tmpFileOs.write(reinterpret_cast<char*>(buffer), read)) {
+    const auto read = stream->read(buffer);
+    if (io::isError(read)) return -1;
+    if (read == 0) break;
+    if (!_tmpFileOs.write(reinterpret_cast<char*>(buffer.data()), read)) {
       return -1;
     }
     size += read;
   } while (size < stream->size());
   _writeSucceeded = true;
-  return size;
+  return gsl::narrow<int64_t>(size);
 }
 
 // Renames tmp file to final destination
