@@ -22,17 +22,22 @@
 
 #include <sys/stat.h>
 #include <algorithm>
-#include <vector>
-#include <queue>
-#include <memory>
+#include <atomic>
 #include <map>
+#include <memory>
+#include <optional>
+#include <queue>
 #include <string>
+#include <vector>
 
 #include "core/Core.h"
 #include "utils/StringUtils.h"
 #include "utils/ClassUtils.h"
 #include "utils/file/FileUtils.h"
 #include "utils/Environment.h"
+#include "core/logging/internal/LogCompressorSink.h"
+#include "utils/Literals.h"
+#include "core/TypedValues.h"
 
 #include "spdlog/spdlog.h"
 #include "spdlog/sinks/stdout_sinks.h"
@@ -60,6 +65,27 @@ namespace logging {
 
 const char* LoggerConfiguration::spdlog_default_pattern = "[%Y-%m-%d %H:%M:%S.%e] [%n] [%l] %v";
 
+namespace {
+std::optional<spdlog::level::level_enum> parse_log_level(const std::string& level_name) {
+  if (utils::StringUtils::equalsIgnoreCase(level_name, "trace")) {
+    return spdlog::level::trace;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "debug")) {
+    return spdlog::level::debug;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "info")) {
+    return spdlog::level::info;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "warn")) {
+    return spdlog::level::warn;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "error")) {
+    return spdlog::level::err;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "critical")) {
+    return spdlog::level::critical;
+  } else if (utils::StringUtils::equalsIgnoreCase(level_name, "off")) {
+    return spdlog::level::off;
+  }
+  return std::nullopt;
+}
+}  // namespace
+
 std::vector<std::string> LoggerProperties::get_keys_of_type(const std::string &type) {
   std::vector<std::string> appenders;
   std::string prefix = type + ".";
@@ -82,9 +108,15 @@ LoggerConfiguration::LoggerConfiguration()
   loggers.push_back(logger_);
 }
 
+LoggerConfiguration& LoggerConfiguration::getConfiguration() {
+  static LoggerConfiguration instance;
+  return instance;
+}
+
 void LoggerConfiguration::initialize(const std::shared_ptr<LoggerProperties> &logger_properties) {
   std::lock_guard<std::mutex> lock(mutex);
   root_namespace_ = initialize_namespaces(logger_properties);
+  initializeCompression(lock, logger_properties);
   std::string spdlog_pattern;
   if (!logger_properties->getString("spdlog.pattern", spdlog_pattern)) {
     spdlog_pattern = spdlog_default_pattern;
@@ -116,6 +148,10 @@ void LoggerConfiguration::initialize(const std::shared_ptr<LoggerProperties> &lo
 
 std::shared_ptr<Logger> LoggerConfiguration::getLogger(const std::string &name) {
   std::lock_guard<std::mutex> lock(mutex);
+  return getLogger(name, lock);
+}
+
+std::shared_ptr<Logger> LoggerConfiguration::getLogger(const std::string &name, const std::lock_guard<std::mutex>& /*lock*/) {
   std::string adjusted_name = name;
   const std::string clazz = "class ";
   auto haz_clazz = name.find(clazz);
@@ -130,6 +166,10 @@ std::shared_ptr<Logger> LoggerConfiguration::getLogger(const std::string &name)
   return result;
 }
 
+std::shared_ptr<spdlog::logger> LoggerConfiguration::getSpdlogLogger(const std::string& name) {
+  return spdlog::get(name);
+}
+
 std::shared_ptr<internal::LoggerNamespace> LoggerConfiguration::initialize_namespaces(const std::shared_ptr<LoggerProperties> &logger_properties) {
   std::map<std::string, std::shared_ptr<spdlog::sinks::sink>> sink_map = logger_properties->initial_sinks();
 
@@ -145,43 +185,7 @@ std::shared_ptr<internal::LoggerNamespace> LoggerConfiguration::initialize_names
     if ("nullappender" == appender_type || "null appender" == appender_type || "null" == appender_type) {
       sink_map[appender_name] = std::make_shared<spdlog::sinks::null_sink_st>();
     } else if ("rollingappender" == appender_type || "rolling appender" == appender_type || "rolling" == appender_type) {
-      std::string file_name;
-      if (!logger_properties->getString(appender_key + ".file_name", file_name)) {
-        file_name = "minifi-app.log";
-      }
-      std::string directory;
-      if (!logger_properties->getString(appender_key + ".directory", directory)) {
-        // The below part assumes logger_properties->getHome() is existing
-        // Cause minifiHome must be set at MiNiFiMain.cpp?
-        directory = logger_properties->getHome() + utils::file::FileUtils::get_separator() + "logs";
-      }
-
-      if (utils::file::FileUtils::create_dir(directory) == -1) {
-        std::cerr << directory << " cannot be created\n";
-        exit(1);
-      }
-      file_name = directory + utils::file::FileUtils::get_separator() + file_name;
-
-      int max_files = 3;
-      std::string max_files_str = "";
-      if (logger_properties->getString(appender_key + ".max_files", max_files_str)) {
-        try {
-          max_files = std::stoi(max_files_str);
-        } catch (const std::invalid_argument &) {
-        } catch (const std::out_of_range &) {
-        }
-      }
-
-      int max_file_size = 5 * 1024 * 1024;
-      std::string max_file_size_str = "";
-      if (logger_properties->getString(appender_key + ".max_file_size", max_file_size_str)) {
-        try {
-          max_file_size = std::stoi(max_file_size_str);
-        } catch (const std::invalid_argument &) {
-        } catch (const std::out_of_range &) {
-        }
-      }
-      sink_map[appender_name] = std::make_shared<spdlog::sinks::rotating_file_sink_mt>(file_name, max_file_size, max_files);
+      sink_map[appender_name] = getRotatingFileSink(appender_key, logger_properties);
     } else if ("stdout" == appender_type) {
       sink_map[appender_name] = std::make_shared<spdlog::sinks::stdout_sink_mt>();
     } else if ("stderr" == appender_type) {
@@ -207,19 +211,9 @@ std::shared_ptr<internal::LoggerNamespace> LoggerConfiguration::initialize_names
       std::string level_name = utils::StringUtils::trim(segment);
       if (first) {
         first = false;
-        std::transform(level_name.begin(), level_name.end(), level_name.begin(), ::tolower);
-        if ("trace" == level_name) {
-          level = spdlog::level::trace;
-        } else if ("debug" == level_name) {
-          level = spdlog::level::debug;
-        } else if ("warn" == level_name) {
-          level = spdlog::level::warn;
-        } else if ("critical" == level_name) {
-          level = spdlog::level::critical;
-        } else if ("error" == level_name) {
-          level = spdlog::level::err;
-        } else if ("off" == level_name) {
-          level = spdlog::level::off;
+        auto opt_level = parse_log_level(level_name);
+        if (opt_level) {
+          level = *opt_level;
         }
       } else {
         sinks.push_back(sink_map[level_name]);
@@ -258,6 +252,7 @@ std::shared_ptr<spdlog::logger> LoggerConfiguration::get_logger(std::shared_ptr<
   }
   std::shared_ptr<internal::LoggerNamespace> current_namespace = root_namespace;
   std::vector<std::shared_ptr<spdlog::sinks::sink>> sinks = root_namespace->sinks;
+  std::vector<std::shared_ptr<spdlog::sinks::sink>> inherited_sinks;
   spdlog::level::level_enum level = root_namespace->level;
   std::string current_namespace_str = "";
   std::string sink_namespace_str = "root";
@@ -268,6 +263,7 @@ std::shared_ptr<spdlog::logger> LoggerConfiguration::get_logger(std::shared_ptr<
     if (child_pair == current_namespace->children.end()) {
       break;
     }
+    std::copy(current_namespace->exported_sinks.begin(), current_namespace->exported_sinks.end(), std::back_inserter(inherited_sinks));
     current_namespace = child_pair->second;
     if (current_namespace->sinks.size() > 0) {
       sinks = current_namespace->sinks;
@@ -283,9 +279,10 @@ std::shared_ptr<spdlog::logger> LoggerConfiguration::get_logger(std::shared_ptr<
     const auto levelView(spdlog::level::to_string_view(level));
     logger->log_debug("%s logger got sinks from namespace %s and level %s from namespace %s", name, sink_namespace_str, std::string(levelView.begin(), levelView.end()), level_namespace_str);
   }
+  std::copy(inherited_sinks.begin(), inherited_sinks.end(), std::back_inserter(sinks));
   spdlogger = std::make_shared<spdlog::logger>(name, begin(sinks), end(sinks));
   spdlogger->set_level(level);
-  spdlogger->set_formatter(formatter -> clone());
+  spdlogger->set_formatter(formatter->clone());
   spdlogger->flush_on(std::max(spdlog::level::info, current_namespace->level));
   try {
     spdlog::register_logger(spdlogger);
@@ -318,6 +315,67 @@ std::shared_ptr<internal::LoggerNamespace> LoggerConfiguration::create_default_r
   return result;
 }
 
+void LoggerConfiguration::initializeCompression(const std::lock_guard<std::mutex>& lock, const std::shared_ptr<LoggerProperties>& properties) {
+  auto compression_sink = compression_manager_.initialize(properties, logger_, [&] (const std::string& name) {return getLogger(name, lock);});
+  if (compression_sink) {
+    root_namespace_->sinks.push_back(compression_sink);
+    root_namespace_->exported_sinks.push_back(compression_sink);
+  }
+}
+
+std::shared_ptr<spdlog::sinks::rotating_file_sink_mt> LoggerConfiguration::getRotatingFileSink(const std::string& appender_key, const std::shared_ptr<LoggerProperties>& properties) {
+  // According to spdlog docs, if two loggers write to the same file, they must use the same sink object.
+  // Note that some logging configuration changes will not take effect until MiNiFi is restarted.
+  static std::map<std::filesystem::path, std::shared_ptr<spdlog::sinks::rotating_file_sink_mt>> rotating_file_sinks;
+  static std::mutex sink_map_mtx;
+
+  std::string file_name;
+  if (!properties->getString(appender_key + ".file_name", file_name)) {
+    file_name = "minifi-app.log";
+  }
+  std::string directory;
+  if (!properties->getString(appender_key + ".directory", directory)) {
+    // The below part assumes logger_properties->getHome() is existing
+    // Cause minifiHome must be set at MiNiFiMain.cpp?
+    directory = properties->getHome() + utils::file::FileUtils::get_separator() + "logs";
+  }
+
+  file_name = directory + utils::file::FileUtils::get_separator() + file_name;
+  if (utils::file::FileUtils::create_dir(directory) == -1) {
+    std::cerr << directory << " cannot be created\n";
+    exit(1);
+  }
+
+  int max_files = 3;
+  std::string max_files_str = "";
+  if (properties->getString(appender_key + ".max_files", max_files_str)) {
+    try {
+      max_files = std::stoi(max_files_str);
+    } catch (const std::invalid_argument &) {
+    } catch (const std::out_of_range &) {
+    }
+  }
+
+  int max_file_size = 5_MiB;
+  std::string max_file_size_str = "";
+  if (properties->getString(appender_key + ".max_file_size", max_file_size_str)) {
+    try {
+      max_file_size = std::stoi(max_file_size_str);
+    } catch (const std::invalid_argument &) {
+    } catch (const std::out_of_range &) {
+    }
+  }
+
+  std::lock_guard<std::mutex> guard(sink_map_mtx);
+  auto it = rotating_file_sinks.find(file_name);
+  if (it != rotating_file_sinks.end()) {
+    return it->second;
+  }
+  auto sink = std::make_shared<spdlog::sinks::rotating_file_sink_mt>(file_name, max_file_size, max_files);
+  rotating_file_sinks.emplace(file_name, sink);
+  return sink;
+}
+
 } /* namespace logging */
 } /* namespace core */
 } /* namespace minifi */
