@@ -16,6 +16,7 @@
  * limitations under the License.
  */
 #include "core/state/nodes/AgentInformation.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
@@ -27,6 +28,8 @@ namespace response {
 utils::ProcessCpuUsageTracker AgentStatus::cpu_load_tracker_;
 std::mutex AgentStatus::cpu_load_tracker_mutex_;
 
+REGISTER_RESOURCE(AgentInformation, "Node part of an AST that defines all agent information, to include the manifest, and bundle information as part of a healthy hearbeat.");
+
 } /* namespace response */
 } /* namespace state */
 } /* namespace minifi */
