@@ -46,43 +46,15 @@
 #include "io/validation.h"
 #include "core/logging/LoggerConfiguration.h"
 #include "utils/file/FileUtils.h"
-#include "utils/GeneralUtils.h"
+#include "utils/gsl.h"
+#include "utils/OsUtils.h"
+#include "utils/net/DNS.h"
+#include "utils/net/Socket.h"
+
 namespace util = org::apache::nifi::minifi::utils;
 namespace mio = org::apache::nifi::minifi::io;
 
 namespace {
-std::string get_last_getaddrinfo_err_str(int getaddrinfo_result) {
-#ifdef WIN32
-  (void)getaddrinfo_result;  // against unused warnings on windows
-  return mio::get_last_socket_error_message();
-#else
-  return gai_strerror(getaddrinfo_result);
-#endif /* WIN32 */
-}
-
-std::string sockaddr_ntop(const sockaddr* const sa) {
-  std::string result;
-  if (sa->sa_family == AF_INET) {
-    sockaddr_in sa_in{};
-    std::memcpy(reinterpret_cast<void*>(&sa_in), sa, sizeof(sockaddr_in));
-    result.resize(INET_ADDRSTRLEN);
-    if (inet_ntop(AF_INET, &sa_in.sin_addr, &result[0], INET_ADDRSTRLEN) == nullptr) {
-      throw minifi::Exception{ minifi::ExceptionType::GENERAL_EXCEPTION, mio::get_last_socket_error_message() };
-    }
-  } else if (sa->sa_family == AF_INET6) {
-    sockaddr_in6 sa_in6{};
-    std::memcpy(reinterpret_cast<void*>(&sa_in6), sa, sizeof(sockaddr_in6));
-    result.resize(INET6_ADDRSTRLEN);
-    if (inet_ntop(AF_INET6, &sa_in6.sin6_addr, &result[0], INET6_ADDRSTRLEN) == nullptr) {
-      throw minifi::Exception{ minifi::ExceptionType::GENERAL_EXCEPTION, mio::get_last_socket_error_message() };
-    }
-  } else {
-    throw minifi::Exception{ minifi::ExceptionType::GENERAL_EXCEPTION, "sockaddr_ntop: unknown address family" };
-  }
-  result.resize(strlen(result.c_str()));  // discard remaining null bytes at the end
-  return result;
-}
-
 template<typename T, typename Pred, typename Adv>
 auto find_if_custom_linked_list(T* const list, const Adv advance_func, const Pred predicate) ->
     typename std::enable_if<std::is_convertible<decltype(advance_func(std::declval<T*>())), T*>::value && std::is_convertible<decltype(predicate(std::declval<T*>())), bool>::value, T*>::type
@@ -94,7 +66,7 @@ auto find_if_custom_linked_list(T* const list, const Adv advance_func, const Pre
 }
 
 #ifndef WIN32
-std::error_code bind_to_local_network_interface(const minifi::io::SocketDescriptor fd, const minifi::io::NetworkInterface& interface) {
+std::error_code bind_to_local_network_interface(const mio::SocketDescriptor fd, const mio::NetworkInterface& interface) {
   using ifaddrs_uniq_ptr = std::unique_ptr<ifaddrs, util::ifaddrs_deleter>;
   const auto if_list_ptr = []() -> ifaddrs_uniq_ptr {
     ifaddrs *list = nullptr;
@@ -119,14 +91,14 @@ std::error_code bind_to_local_network_interface(const minifi::io::SocketDescript
 }
 #endif /* !WIN32 */
 
-std::error_code set_non_blocking(const minifi::io::SocketDescriptor fd) noexcept {
+std::error_code set_non_blocking(const mio::SocketDescriptor fd) noexcept {
 #ifndef WIN32
   if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
     return { errno, std::generic_category() };
   }
 #else
   u_long iMode = 1;
-  if (ioctlsocket(fd, FIONBIO, &iMode) == SOCKET_ERROR) {
+  if (ioctlsocket(fd, FIONBIO, &iMode) == mio::SocketError) {
     return { WSAGetLastError(), std::system_category() };
   }
 #endif /* !WIN32 */
@@ -140,18 +112,10 @@ namespace nifi {
 namespace minifi {
 namespace io {
 
-std::string get_last_socket_error_message() {
-#ifdef WIN32
-  const auto error_code = WSAGetLastError();
-#else
-  const auto error_code = errno;
-#endif /* WIN32 */
-  return std::system_category().message(error_code);
-}
 
 bool valid_socket(const SocketDescriptor fd) noexcept {
 #ifdef WIN32
-  return fd != INVALID_SOCKET && fd >= 0;
+  return fd != InvalidSocket && fd >= 0;
 #else
   return fd >= 0;
 #endif /* WIN32 */
@@ -161,7 +125,7 @@ Socket::Socket(const std::shared_ptr<SocketContext>& /*context*/, std::string ho
     : requested_hostname_(std::move(hostname)),
       port_(port),
       listeners_(listeners),
-      logger_(logging::LoggerFactory<Socket>::getLogger()) {
+      logger_(core::logging::LoggerFactory<Socket>::getLogger()) {
   FD_ZERO(&total_list_);
   FD_ZERO(&read_fds_);
   initialize_socket();
@@ -195,12 +159,12 @@ Socket::Socket(Socket &&other) noexcept
 
 Socket& Socket::operator=(Socket &&other) noexcept {
   if (&other == this) return *this;
-  requested_hostname_ = util::exchange(other.requested_hostname_, "");
-  canonical_hostname_ = util::exchange(other.canonical_hostname_, "");
-  port_ = util::exchange(other.port_, 0);
-  is_loopback_only_ = util::exchange(other.is_loopback_only_, false);
-  local_network_interface_ = util::exchange(other.local_network_interface_, {});
-  socket_file_descriptor_ = util::exchange(other.socket_file_descriptor_, INVALID_SOCKET);
+  requested_hostname_ = std::exchange(other.requested_hostname_, "");
+  canonical_hostname_ = std::exchange(other.canonical_hostname_, "");
+  port_ = std::exchange(other.port_, 0);
+  is_loopback_only_ = std::exchange(other.is_loopback_only_, false);
+  local_network_interface_ = std::exchange(other.local_network_interface_, {});
+  socket_file_descriptor_ = std::exchange(other.socket_file_descriptor_, InvalidSocket);
   total_list_ = other.total_list_;
   FD_ZERO(&other.total_list_);
   read_fds_ = other.read_fds_;
@@ -211,8 +175,8 @@ Socket& Socket::operator=(Socket &&other) noexcept {
   other.total_written_.exchange(0);
   total_read_.exchange(other.total_read_);
   other.total_read_.exchange(0);
-  listeners_ = util::exchange(other.listeners_, 0);
-  nonBlocking_ = util::exchange(other.nonBlocking_, false);
+  listeners_ = std::exchange(other.listeners_, 0);
+  nonBlocking_ = std::exchange(other.nonBlocking_, false);
   logger_ = other.logger_;
   return *this;
 }
@@ -223,13 +187,13 @@ Socket::~Socket() {
 
 void Socket::close() {
   if (valid_socket(socket_file_descriptor_)) {
-    logging::LOG_DEBUG(logger_) << "Closing " << socket_file_descriptor_;
+    core::logging::LOG_DEBUG(logger_) << "Closing " << socket_file_descriptor_;
 #ifdef WIN32
     closesocket(socket_file_descriptor_);
 #else
     ::close(socket_file_descriptor_);
 #endif
-    socket_file_descriptor_ = INVALID_SOCKET;
+    socket_file_descriptor_ = InvalidSocket;
   }
   if (total_written_ > 0) {
     local_network_interface_.log_write(gsl::narrow<uint32_t>(total_written_.load()));
@@ -250,7 +214,7 @@ void Socket::setNonBlocking() {
 int8_t Socket::createConnection(const addrinfo* const destination_addresses) {
   for (const auto *current_addr = destination_addresses; current_addr; current_addr = current_addr->ai_next) {
     if (!valid_socket(socket_file_descriptor_ = socket(current_addr->ai_family, current_addr->ai_socktype, current_addr->ai_protocol))) {
-      logger_->log_warn("socket: %s", get_last_socket_error_message());
+      logger_->log_warn("socket: %s", utils::net::get_last_socket_error().message());
       continue;
     }
     setSocketOptions(socket_file_descriptor_);
@@ -258,20 +222,20 @@ int8_t Socket::createConnection(const addrinfo* const destination_addresses) {
     if (listeners_ > 0) {
       // server socket
       const auto bind_result = bind(socket_file_descriptor_, current_addr->ai_addr, current_addr->ai_addrlen);
-      if (bind_result == SOCKET_ERROR) {
-        logger_->log_warn("bind: %s", get_last_socket_error_message());
+      if (bind_result == SocketError) {
+        logger_->log_warn("bind: %s", utils::net::get_last_socket_error().message());
         close();
         continue;
       }
 
       const auto listen_result = listen(socket_file_descriptor_, listeners_);
-      if (listen_result == SOCKET_ERROR) {
-        logger_->log_warn("listen: %s", get_last_socket_error_message());
+      if (listen_result == SocketError) {
+        logger_->log_warn("listen: %s", utils::net::get_last_socket_error().message());
         close();
         continue;
       }
 
-      logger_->log_info("Listening on %s:%" PRIu16 " with backlog %" PRIu16, sockaddr_ntop(current_addr->ai_addr), port_, listeners_);
+      logger_->log_info("Listening on %s:%" PRIu16 " with backlog %" PRIu16, utils::net::sockaddr_ntop(current_addr->ai_addr), port_, listeners_);
     } else {
       // client socket
 #ifndef WIN32
@@ -283,13 +247,13 @@ int8_t Socket::createConnection(const addrinfo* const destination_addresses) {
 #endif /* !WIN32 */
 
       const auto connect_result = connect(socket_file_descriptor_, current_addr->ai_addr, current_addr->ai_addrlen);
-      if (connect_result == SOCKET_ERROR) {
-        logger_->log_warn("Couldn't connect to %s:%" PRIu16 ": %s", sockaddr_ntop(current_addr->ai_addr), port_, get_last_socket_error_message());
+      if (connect_result == SocketError) {
+        logger_->log_warn("Couldn't connect to %s:%" PRIu16 ": %s", utils::net::sockaddr_ntop(current_addr->ai_addr), port_, utils::net::get_last_socket_error().message());
         close();
         continue;
       }
 
-      logger_->log_info("Connected to %s:%" PRIu16, sockaddr_ntop(current_addr->ai_addr), port_);
+      logger_->log_info("Connected to %s:%" PRIu16, utils::net::sockaddr_ntop(current_addr->ai_addr), port_);
     }
 
     FD_SET(socket_file_descriptor_, &total_list_);
@@ -314,8 +278,8 @@ int8_t Socket::createConnection(const addrinfo *, ip4addr &addr) {
     sa.sin_family = AF_INET;
     sa.sin_port = htons(port_);
     sa.sin_addr.s_addr = htonl(is_loopback_only_ ? INADDR_LOOPBACK : INADDR_ANY);
-    if (bind(socket_file_descriptor_, reinterpret_cast<const sockaddr*>(&sa), sizeof(struct sockaddr_in)) == SOCKET_ERROR) {
-      logger_->log_error("Could not bind to socket, reason %s", get_last_socket_error_message());
+    if (bind(socket_file_descriptor_, reinterpret_cast<const sockaddr*>(&sa), sizeof(struct sockaddr_in)) == SocketError) {
+      logger_->log_error("Could not bind to socket, reason %s", utils::net::get_last_socket_error().message());
       return -1;
     }
 
@@ -344,7 +308,7 @@ int8_t Socket::createConnection(const addrinfo *, ip4addr &addr) {
 #ifdef WIN32
       sa_loc.sin_addr.s_addr = addr.s_addr;
     }
-    if (connect(socket_file_descriptor_, reinterpret_cast<const sockaddr*>(&sa_loc), sizeof(sockaddr_in)) == SOCKET_ERROR) {
+    if (connect(socket_file_descriptor_, reinterpret_cast<const sockaddr*>(&sa_loc), sizeof(sockaddr_in)) == SocketError) {
       int err = WSAGetLastError();
       if (err == WSAEADDRNOTAVAIL) {
         logger_->log_error("invalid or unknown IP");
@@ -371,31 +335,20 @@ int8_t Socket::createConnection(const addrinfo *, ip4addr &addr) {
 }
 
 int Socket::initialize() {
-  addrinfo hints{};
-  memset(&hints, 0, sizeof hints);  // make sure the struct is empty
-  hints.ai_family = AF_UNSPEC;
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_flags = AI_CANONNAME;
-  if (listeners_ > 0 && !is_loopback_only_)
-    hints.ai_flags = AI_PASSIVE;
-  hints.ai_protocol = 0; /* any protocol */
-
-  const char* const gai_node = [this]() -> const char* {
+  const char* const hostname = [this]() -> const char* {
     if (is_loopback_only_) return "localhost";
     if (!is_loopback_only_ && listeners_ > 0) return nullptr;  // all non-localhost server sockets listen on wildcard address
     if (!requested_hostname_.empty()) return requested_hostname_.c_str();
     return nullptr;
   }();
-  const auto gai_service = std::to_string(port_);
-  addrinfo* getaddrinfo_result = nullptr;
-  const int errcode = getaddrinfo(gai_node, gai_service.c_str(), &hints, &getaddrinfo_result);
-  const std::unique_ptr<addrinfo, util::addrinfo_deleter> addr_info{ getaddrinfo_result };
-  getaddrinfo_result = nullptr;
-  if (errcode != 0) {
-    logger_->log_error("getaddrinfo: %s", get_last_getaddrinfo_err_str(errcode));
+  const bool is_server = hostname == nullptr;
+  const auto addr_info_or_error = utils::net::resolveHost(hostname, port_, utils::net::IpProtocol::Tcp, !is_server);
+  if (!addr_info_or_error) {
+    logger_->log_error("getaddrinfo: %s", addr_info_or_error.error().message());
     return -1;
   }
-  socket_file_descriptor_ = INVALID_SOCKET;
+  const auto& addr_info = *addr_info_or_error;
+  socket_file_descriptor_ = InvalidSocket;
 
   // AI_CANONNAME always sets ai_canonname of the first addrinfo structure
   canonical_hostname_ = !IsNullOrEmpty(addr_info->ai_canonname) ? addr_info->ai_canonname : requested_hostname_;
@@ -499,43 +452,39 @@ std::string Socket::getHostname() const {
 
 // data stream overrides
 
-int Socket::write(const uint8_t *value, int size) {
-  gsl_Expects(size >= 0);
-
-  int ret = 0, bytes = 0;
-
+size_t Socket::write(const uint8_t *value, size_t size) {
+  size_t bytes = 0;
   int fd = select_descriptor(1000);
-  if (fd < 0) { return -1; }
+  if (fd < 0) { return STREAM_ERROR; }
   while (bytes < size) {
-    ret = send(fd, reinterpret_cast<const char*>(value) + bytes, size - bytes, 0);
+    const auto send_ret = send(fd, reinterpret_cast<const char*>(value) + bytes, size - bytes, 0);
     // check for errors
-    if (ret <= 0) {
+    if (send_ret <= 0) {
       utils::file::FileUtils::close(fd);
-      logger_->log_error("Could not send to %d, error: %s", fd, get_last_socket_error_message());
-      return ret;
+      logger_->log_error("Could not send to %d, error: %s", fd, utils::net::get_last_socket_error().message());
+      return STREAM_ERROR;
     }
-    bytes += ret;
+    bytes += gsl::narrow<size_t>(send_ret);
   }
 
-  if (ret)
+  if (bytes > 0)
     logger_->log_trace("Send data size %d over socket %d", size, fd);
   total_written_ += bytes;
   return bytes;
 }
 
-int Socket::read(uint8_t *buf, int buflen, bool retrieve_all_bytes) {
-  gsl_Expects(buflen >= 0);
-  int32_t total_read = 0;
-  while (buflen) {
+size_t Socket::read(gsl::span<std::byte> buf, bool retrieve_all_bytes) {
+  size_t total_read = 0;
+  while (!buf.empty()) {
     int16_t fd = select_descriptor(1000);
     if (fd < 0) {
       if (listeners_ <= 0) {
-        logger_->log_debug("fd %d close %i", fd, buflen);
+        logger_->log_debug("fd %d close %i", fd, buf.size());
         utils::file::FileUtils::close(socket_file_descriptor_);
       }
-      return -1;
+      return STREAM_ERROR;
     }
-    int bytes_read = recv(fd, reinterpret_cast<char*>(buf), buflen, 0);
+    const auto bytes_read = recv(fd, reinterpret_cast<char*>(buf.data()), buf.size(), 0);
     logger_->log_trace("Recv call %d", bytes_read);
     if (bytes_read <= 0) {
       if (bytes_read == 0) {
@@ -545,23 +494,22 @@ int Socket::read(uint8_t *buf, int buflen, bool retrieve_all_bytes) {
         int err = WSAGetLastError();
         if (err == WSAEWOULDBLOCK) {
           // continue
-          return -2;
+          return static_cast<size_t>(-2);
         }
         logger_->log_error("Could not recv on %d (port %d), error code: %d", fd, port_, err);
 #else
         if (errno == EAGAIN || errno == EWOULDBLOCK) {
           // continue
-          return -2;
+          return static_cast<size_t>(-2);
         }
         logger_->log_error("Could not recv on %d (port %d), error: %s", fd, port_, strerror(errno));
 
 #endif  // WIN32
       }
-      return -1;
+      return STREAM_ERROR;
     }
-    buflen -= bytes_read;
-    buf += bytes_read;
-    total_read += bytes_read;
+    buf = buf.subspan(gsl::narrow<size_t>(bytes_read));
+    total_read += gsl::narrow<size_t>(bytes_read);
     if (!retrieve_all_bytes) {
       break;
     }
