@@ -17,19 +17,16 @@
  */
 
 #include <fstream>
-#include <vector>
 #include <memory>
+#include <vector>
 #include <string>
 #include <Exception.h>
 #include "io/validation.h"
 #include "io/FileStream.h"
 #include "io/InputStream.h"
-#include "io/OutputStream.h"
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace io {
+#include "utils/gsl.h"
+
+namespace org::apache::nifi::minifi::io {
 
 constexpr const char *FILE_OPENING_ERROR_MSG = "Error opening file: ";
 constexpr const char *READ_ERROR_MSG = "Error reading from file: ";
@@ -37,7 +34,6 @@ constexpr const char *WRITE_ERROR_MSG = "Error writing to file: ";
 constexpr const char *SEEK_ERROR_MSG = "Error seeking in file: ";
 constexpr const char *INVALID_FILE_STREAM_ERROR_MSG = "invalid file stream";
 constexpr const char *TELLG_CALL_ERROR_MSG = "tellg call on file stream failed";
-constexpr const char *INVALID_BUFFER_ERROR_MSG = "invalid buffer";
 constexpr const char *FLUSH_CALL_ERROR_MSG = "flush call on file stream failed";
 constexpr const char *WRITE_CALL_ERROR_MSG = "write call on file stream failed";
 constexpr const char *EMPTY_MESSAGE_ERROR_MSG = "empty message";
@@ -46,35 +42,33 @@ constexpr const char *SEEKP_CALL_ERROR_MSG = "seekp call on file stream failed";
 
 FileStream::FileStream(const std::string &path, bool append)
     : offset_(0),
-      path_(path),
-      logger_(logging::LoggerFactory<FileStream>::getLogger()) {
-  file_stream_ = std::unique_ptr<std::fstream>(new std::fstream());
+      path_(path) {
+  file_stream_ = std::make_unique<std::fstream>();
   if (append) {
     file_stream_->open(path.c_str(), std::fstream::in | std::fstream::out | std::fstream::app | std::fstream::binary);
     if (file_stream_->is_open()) {
       seekToEndOfFile(FILE_OPENING_ERROR_MSG);
       auto len = file_stream_->tellg();
       if (len == std::streampos(-1))
-        logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << TELLG_CALL_ERROR_MSG;
+        core::logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << TELLG_CALL_ERROR_MSG;
       length_ = len > 0 ? gsl::narrow<size_t>(len) : 0;
       seek(offset_);
     } else {
-      logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
+      core::logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
     }
   } else {
     file_stream_->open(path.c_str(), std::fstream::out | std::fstream::binary);
     length_ = 0;
     if (!file_stream_->is_open()) {
-      logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
+      core::logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
     }
   }
 }
 
 FileStream::FileStream(const std::string &path, uint32_t offset, bool write_enable)
     : offset_(offset),
-      path_(path),
-      logger_(logging::LoggerFactory<FileStream>::getLogger()) {
-  file_stream_ = std::unique_ptr<std::fstream>(new std::fstream());
+      path_(path) {
+  file_stream_ = std::make_unique<std::fstream>();
   if (write_enable) {
     file_stream_->open(path.c_str(), std::fstream::in | std::fstream::out | std::fstream::binary);
   } else {
@@ -84,11 +78,11 @@ FileStream::FileStream(const std::string &path, uint32_t offset, bool write_enab
     seekToEndOfFile(FILE_OPENING_ERROR_MSG);
     auto len = file_stream_->tellg();
     if (len == std::streampos(-1))
-      logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << TELLG_CALL_ERROR_MSG;
+      core::logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << TELLG_CALL_ERROR_MSG;
     length_ = len > 0 ? gsl::narrow<size_t>(len) : 0;
     seek(offset_);
   } else {
-    logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
+    core::logging::LOG_ERROR(logger_) << FILE_OPENING_ERROR_MSG << path << " " << strerror(errno);
   }
 }
 
@@ -97,98 +91,85 @@ void FileStream::close() {
   file_stream_.reset();
 }
 
-void FileStream::seek(uint64_t offset) {
+void FileStream::seek(size_t offset) {
   std::lock_guard<std::mutex> lock(file_lock_);
   if (file_stream_ == nullptr || !file_stream_->is_open()) {
-    logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
+    core::logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
     return;
   }
-  offset_ = gsl::narrow<size_t>(offset);
+  offset_ = offset;
   file_stream_->clear();
   if (!file_stream_->seekg(offset_))
-    logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << SEEKG_CALL_ERROR_MSG;
+    core::logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << SEEKG_CALL_ERROR_MSG;
   if (!file_stream_->seekp(offset_))
-    logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << SEEKP_CALL_ERROR_MSG;
+    core::logging::LOG_ERROR(logger_) << SEEK_ERROR_MSG << SEEKP_CALL_ERROR_MSG;
 }
 
-int FileStream::write(const uint8_t *value, int size) {
-  gsl_Expects(size >= 0);
-  if (size == 0) {
-    return 0;
-  }
-  if (!IsNullOrEmpty(value)) {
-    std::lock_guard<std::mutex> lock(file_lock_);
-    if (file_stream_ == nullptr || !file_stream_->is_open()) {
-      logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
-      return -1;
-    }
-    if (file_stream_->write(reinterpret_cast<const char*>(value), size)) {
-      offset_ += size;
-      if (offset_ > length_) {
-        length_ = offset_;
-      }
-      if (!file_stream_->flush()) {
-        logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << FLUSH_CALL_ERROR_MSG;
-        return -1;
-      }
-      return size;
-    } else {
-      logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << WRITE_CALL_ERROR_MSG;
-      return -1;
-    }
-  } else {
-    logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << EMPTY_MESSAGE_ERROR_MSG;
-    return -1;
-  }
+size_t FileStream::tell() const {
+  return offset_;
 }
 
-int FileStream::read(uint8_t *buf, int buflen) {
-  gsl_Expects(buflen >= 0);
-  if (buflen == 0) {
-    return 0;
+size_t FileStream::write(const uint8_t *value, size_t size) {
+  if (size == 0) return 0;
+  if (IsNullOrEmpty(value)) {
+    core::logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << EMPTY_MESSAGE_ERROR_MSG;
+    return STREAM_ERROR;
   }
-  if (!IsNullOrEmpty(buf)) {
-    std::lock_guard<std::mutex> lock(file_lock_);
-    if (file_stream_ == nullptr || !file_stream_->is_open()) {
-      logging::LOG_ERROR(logger_) << READ_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
-      return -1;
-    }
-    file_stream_->read(reinterpret_cast<char*>(buf), buflen);
-    if (file_stream_->eof() || file_stream_->fail()) {
-      file_stream_->clear();
-      seekToEndOfFile(READ_ERROR_MSG);
-      auto tellg_result = file_stream_->tellg();
-      if (tellg_result == std::streampos(-1)) {
-        logging::LOG_ERROR(logger_) << READ_ERROR_MSG << TELLG_CALL_ERROR_MSG;
-        return -1;
-      }
-      size_t len = gsl::narrow<size_t>(tellg_result);
-      size_t ret = len - offset_;
-      offset_ = len;
-      length_ = len;
-      logging::LOG_DEBUG(logger_) << path_ << " eof bit, ended at " << offset_;
-      return gsl::narrow<int>(ret);
-    } else {
-      offset_ += buflen;
-      file_stream_->seekp(offset_);
-      return buflen;
-    }
+  std::lock_guard<std::mutex> lock(file_lock_);
+  if (file_stream_ == nullptr || !file_stream_->is_open()) {
+    core::logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
+    return STREAM_ERROR;
+  }
+  if (!file_stream_->write(reinterpret_cast<const char*>(value), gsl::narrow<std::streamsize>(size))) {
+    core::logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << WRITE_CALL_ERROR_MSG;
+    return STREAM_ERROR;
+  }
+  offset_ += size;
+  if (offset_ > length_) {
+    length_ = offset_;
+  }
+  if (!file_stream_->flush()) {
+    core::logging::LOG_ERROR(logger_) << WRITE_ERROR_MSG << FLUSH_CALL_ERROR_MSG;
+    return STREAM_ERROR;
+  }
+  return size;
+}
 
+size_t FileStream::read(gsl::span<std::byte> buf) {
+  if (buf.empty()) { return 0; }
+  std::lock_guard<std::mutex> lock(file_lock_);
+  if (file_stream_ == nullptr || !file_stream_->is_open()) {
+    core::logging::LOG_ERROR(logger_) << READ_ERROR_MSG << INVALID_FILE_STREAM_ERROR_MSG;
+    return STREAM_ERROR;
+  }
+  file_stream_->read(reinterpret_cast<char*>(buf.data()), gsl::narrow<std::streamsize>(buf.size()));
+  if (file_stream_->eof() || file_stream_->fail()) {
+    file_stream_->clear();
+    seekToEndOfFile(READ_ERROR_MSG);
+    auto tellg_result = file_stream_->tellg();
+    if (tellg_result == std::streampos(-1)) {
+      core::logging::LOG_ERROR(logger_) << READ_ERROR_MSG << TELLG_CALL_ERROR_MSG;
+      return STREAM_ERROR;
+    }
+    const auto len = gsl::narrow<size_t>(tellg_result);
+    size_t ret = len - offset_;
+    offset_ = len;
+    length_ = len;
+    core::logging::LOG_DEBUG(logger_) << path_ << " eof bit, ended at " << offset_;
+    return ret;
   } else {
-    logging::LOG_ERROR(logger_) << READ_ERROR_MSG << INVALID_BUFFER_ERROR_MSG;
-    return -1;
+    offset_ += buf.size();
+    file_stream_->seekp(gsl::narrow<std::streamoff>(offset_));
+    return buf.size();
   }
 }
 
 void FileStream::seekToEndOfFile(const char *caller_error_msg) {
   if (!file_stream_->seekg(0, file_stream_->end))
-    logging::LOG_ERROR(logger_) << caller_error_msg << SEEKG_CALL_ERROR_MSG;
+    core::logging::LOG_ERROR(logger_) << caller_error_msg << SEEKG_CALL_ERROR_MSG;
   if (!file_stream_->seekp(0, file_stream_->end))
-    logging::LOG_ERROR(logger_) << caller_error_msg << SEEKP_CALL_ERROR_MSG;
+    core::logging::LOG_ERROR(logger_) << caller_error_msg << SEEKP_CALL_ERROR_MSG;
 }
-} /* namespace io */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
+
+}  // namespace org::apache::nifi::minifi::io
 
