@@ -32,7 +32,6 @@
 #include "properties/Configure.h"
 #include "utils/StringUtils.h"
 #include "core/logging/LoggerConfiguration.h"
-#include "utils/GeneralUtils.h"
 #include "utils/gsl.h"
 #include "utils/tls/TLSUtils.h"
 
@@ -44,7 +43,6 @@ namespace io {
 
 TLSContext::TLSContext(const std::shared_ptr<Configure> &configure, std::shared_ptr<minifi::controllers::SSLContextService> ssl_service)
     : SocketContext(configure),
-      logger_(logging::LoggerFactory<TLSContext>::getLogger()),
       configure_(configure),
       ssl_service_(std::move(ssl_service)),
       ctx(nullptr, deleteContext),
@@ -168,25 +166,25 @@ void TLSSocket::close() {
  */
 TLSSocket::TLSSocket(const std::shared_ptr<TLSContext> &context, const std::string &hostname, const uint16_t port, const uint16_t listeners)
     : Socket(context, hostname, port, listeners) {
-  logger_ = logging::LoggerFactory<TLSSocket>::getLogger();
+  logger_ = core::logging::LoggerFactory<TLSSocket>::getLogger();
   context_ = context;
 }
 
 TLSSocket::TLSSocket(const std::shared_ptr<TLSContext> &context, const std::string &hostname, const uint16_t port)
     : Socket(context, hostname, port, 0) {
-  logger_ = logging::LoggerFactory<TLSSocket>::getLogger();
+  logger_ = core::logging::LoggerFactory<TLSSocket>::getLogger();
   context_ = context;
 }
 
 TLSSocket::TLSSocket(TLSSocket &&other)
     : Socket(std::move(other)),
-      context_{ utils::exchange(other.context_, nullptr) } {
+      context_{ std::exchange(other.context_, nullptr) } {
   std::lock_guard<std::mutex> lg{ other.ssl_mutex_ };
 
   connected_.exchange(other.connected_.load());
   other.connected_.exchange(false);
-  ssl_ = utils::exchange(other.ssl_, nullptr);
-  ssl_map_ = utils::exchange(other.ssl_map_, {});
+  ssl_ = std::exchange(other.ssl_, nullptr);
+  ssl_map_ = std::exchange(other.ssl_map_, {});
 }
 
 TLSSocket& TLSSocket::operator=(TLSSocket&& other) {
@@ -195,9 +193,9 @@ TLSSocket& TLSSocket::operator=(TLSSocket&& other) {
   std::lock_guard<std::mutex> lg{ other.ssl_mutex_ };
   connected_.exchange(other.connected_.load());
   other.connected_.exchange(false);
-  context_ = utils::exchange(other.context_, nullptr);
-  ssl_ = utils::exchange(other.ssl_, nullptr);
-  ssl_map_ = utils::exchange(other.ssl_map_, {});
+  context_ = std::exchange(other.context_, nullptr);
+  ssl_ = std::exchange(other.ssl_, nullptr);
+  ssl_map_ = std::exchange(other.ssl_map_, {});
   return *this;
 }
 
@@ -294,7 +292,7 @@ int16_t TLSSocket::select_descriptor(const uint16_t msec) {
     if (listeners_ > 0) {
       const auto newfd = accept(socket_file_descriptor_, nullptr, nullptr);
       if (!valid_socket(newfd)) {
-        logger_->log_error("accept: %s", get_last_socket_error_message());
+        logger_->log_error("accept: %s", utils::net::get_last_socket_error().message());
         return -1;
       }
       FD_SET(newfd, &total_list_);  // add to master set
@@ -350,100 +348,104 @@ int16_t TLSSocket::select_descriptor(const uint16_t msec) {
   return -1;
 }
 
-int TLSSocket::read(uint8_t *buf, int buflen, bool /*retrieve_all_bytes*/) {
-  gsl_Expects(buflen >= 0);
-  int total_read = 0;
+size_t TLSSocket::read(gsl::span<std::byte> buffer, bool) {
+  size_t total_read = 0;
   int status = 0;
   int loc = 0;
+  auto* buf = buffer.data();
+  auto buflen = buffer.size();
   int16_t fd = select_descriptor(1000);
   if (fd < 0) {
     close();
-    return -1;
+    return STREAM_ERROR;
   }
   auto fd_ssl = get_ssl(fd);
   if (IsNullOrEmpty(fd_ssl)) {
-    return -1;
+    return STREAM_ERROR;
   }
   if (!SSL_pending(fd_ssl)) {
     return 0;
   }
   while (buflen) {
     if (fd <= 0) {
-      return -1;
+      return STREAM_ERROR;
     }
     int sslStatus;
     do {
-      status = SSL_read(fd_ssl, buf + loc, buflen);
+      const auto ssl_read_size = gsl::narrow<int>(std::min(buflen, gsl::narrow<size_t>(std::numeric_limits<int>::max())));
+      status = SSL_read(fd_ssl, buf + loc, ssl_read_size);
       sslStatus = SSL_get_error(fd_ssl, status);
     } while (status < 0 && sslStatus == SSL_ERROR_WANT_READ && SSL_pending(fd_ssl));
 
-    buflen -= status;
+    if (status < 0) break;
+
+    buflen -= gsl::narrow<size_t>(status);
     loc += status;
-    total_read += status;
+    total_read += gsl::narrow<size_t>(status);
   }
 
   return total_read;
 }
 
-int TLSSocket::writeData(const uint8_t *value, unsigned int size, int fd) {
-  unsigned int bytes = 0;
-  int sent = 0;
+size_t TLSSocket::writeData(const uint8_t *value, size_t size, int fd) {
+  size_t bytes = 0;
   auto fd_ssl = get_ssl(fd);
   if (IsNullOrEmpty(fd_ssl)) {
-    return -1;
+    return STREAM_ERROR;
   }
   while (bytes < size) {
-    sent = SSL_write(fd_ssl, value + bytes, gsl::narrow<int>(size - bytes));
+    const auto sent = SSL_write(fd_ssl, value + bytes, gsl::narrow<int>(size - bytes));
     // check for errors
     if (sent < 0) {
       int ret = 0;
       ret = SSL_get_error(fd_ssl, sent);
       logger_->log_trace("WriteData socket %d send failed %s %d", fd, strerror(errno), ret);
-
-      return sent;
+      return STREAM_ERROR;
     }
     logger_->log_trace("WriteData socket %d send succeed %d", fd, sent);
-    bytes += sent;
+    bytes += gsl::narrow<size_t>(sent);
   }
-  return gsl::narrow<int>(size);
+  return size;
 }
 
-int TLSSocket::write(const uint8_t *value, int size) {
-  int fd = select_descriptor(1000);
+size_t TLSSocket::write(const uint8_t *value, size_t size) {
+  const int fd = select_descriptor(1000);
   if (fd < 0) {
     close();
-    return -1;
+    return STREAM_ERROR;
   }
   return writeData(value, size, fd);
 }
 
-int TLSSocket::read(uint8_t *buf, int buflen) {
-  gsl_Expects(buflen >= 0);
-  int total_read = 0;
+size_t TLSSocket::read(gsl::span<std::byte> buffer) {
+  size_t total_read = 0;
   int status = 0;
+  auto* buf = buffer.data();
+  auto buflen = buffer.size();
   while (buflen) {
-    int16_t fd = select_descriptor(1000);
+    const int16_t fd = select_descriptor(1000);
     if (fd < 0) {
       close();
-      return -1;
+      return STREAM_ERROR;
     }
 
     int sslStatus;
     do {
-      auto fd_ssl = get_ssl(fd);
+      const auto fd_ssl = get_ssl(fd);
       if (IsNullOrEmpty(fd_ssl)) {
-        return -1;
+        return STREAM_ERROR;
       }
-      status = SSL_read(fd_ssl, buf, buflen);
+      const auto ssl_read_size = gsl::narrow<int>(std::min(buflen, gsl::narrow<size_t>(std::numeric_limits<int>::max())));
+      status = SSL_read(fd_ssl, buf, ssl_read_size);
       sslStatus = SSL_get_error(fd_ssl, status);
-    } while (status < 0 && sslStatus == SSL_ERROR_WANT_READ);
+    } while (status <= 0 && sslStatus == SSL_ERROR_WANT_READ);
 
-    if (status < 0)
+    if (status <= 0)
       break;
 
-    buflen -= status;
+    buflen -= gsl::narrow<size_t>(status);
     buf += status;
-    total_read += status;
+    total_read += gsl::narrow<size_t>(status);
   }
 
   return total_read;
