@@ -38,20 +38,17 @@ namespace minifi {
 namespace provenance {
 
 std::shared_ptr<utils::IdGenerator> ProvenanceEventRecord::id_generator_ = utils::IdGenerator::getIdGenerator();
-std::shared_ptr<logging::Logger> ProvenanceEventRecord::logger_ = logging::LoggerFactory<ProvenanceEventRecord>::getLogger();
+std::shared_ptr<core::logging::Logger> ProvenanceEventRecord::logger_ = core::logging::LoggerFactory<ProvenanceEventRecord>::getLogger();
 
 const char *ProvenanceEventRecord::ProvenanceEventTypeStr[REPLAY + 1] = { "CREATE", "RECEIVE", "FETCH", "SEND", "DOWNLOAD", "DROP", "EXPIRE", "FORK", "JOIN", "CLONE", "CONTENT_MODIFIED",
     "ATTRIBUTES_MODIFIED", "ROUTE", "ADDINFO", "REPLAY" };
 
 ProvenanceEventRecord::ProvenanceEventRecord(ProvenanceEventRecord::ProvenanceEventType event, std::string componentId, std::string componentType)
-    : core::SerializableComponent(core::getClassName<ProvenanceEventRecord>()),
-      _entryDate(0),
-      _lineageStartDate(0),
-      _eventDuration(0) {
+    : core::SerializableComponent(core::getClassName<ProvenanceEventRecord>()) {
   _eventType = event;
   _componentId = componentId;
   _componentType = componentType;
-  _eventTime = utils::timeutils::getTimeMillis();
+  _eventTime = std::chrono::system_clock::now();
 }
 
 // DeSerialize
@@ -74,7 +71,7 @@ bool ProvenanceEventRecord::DeSerialize(const std::shared_ptr<core::Serializable
     logger_->log_debug("NiFi Provenance Read event %s", getUUIDStr());
   }
 
-  org::apache::nifi::minifi::io::BufferStream stream((const uint8_t*) value.data(), gsl::narrow<int>(value.length()));
+  org::apache::nifi::minifi::io::BufferStream stream(value);
 
   ret = DeSerialize(stream);
 
@@ -88,134 +85,162 @@ bool ProvenanceEventRecord::DeSerialize(const std::shared_ptr<core::Serializable
 }
 
 bool ProvenanceEventRecord::Serialize(org::apache::nifi::minifi::io::BufferStream& outStream) {
-  int ret;
-
-  ret = outStream.write(this->uuid_);
-  if (ret <= 0) {
-    return false;
-  }
-
-  uint32_t eventType = this->_eventType;
-  ret = outStream.write(eventType);
-  if (ret != 4) {
-    return false;
-  }
-
-  ret = outStream.write(this->_eventTime);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_entryDate);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_eventDuration);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_lineageStartDate);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_componentId);
-  if (ret <= 0) {
-    return false;
-  }
-
-  ret = outStream.write(this->_componentType);
-  if (ret <= 0) {
-    return false;
-  }
-
-  ret = outStream.write(this->flow_uuid_);
-  if (ret <= 0) {
-    return false;
-  }
-
-  ret = outStream.write(this->_details);
-  if (ret <= 0) {
-    return false;
-  }
-
-  // write flow attributes
-  uint32_t numAttributes = gsl::narrow<uint32_t>(this->_attributes.size());
-  ret = outStream.write(numAttributes);
-  if (ret != 4) {
-    return false;
-  }
-
-  for (const auto& itAttribute : _attributes) {
-    ret = outStream.write(itAttribute.first);
-    if (ret <= 0) {
-      return false;
-    }
-    ret = outStream.write(itAttribute.second);
-    if (ret <= 0) {
+  {
+    const auto ret = outStream.write(this->uuid_);
+    if (ret == 0 || io::isError(ret)) {
       return false;
     }
   }
-
-  ret = outStream.write(this->_contentFullPath);
-  if (ret <= 0) {
-    return false;
-  }
-
-  ret = outStream.write(this->_size);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_offset);
-  if (ret != 8) {
-    return false;
-  }
-
-  ret = outStream.write(this->_sourceQueueIdentifier);
-  if (ret <= 0) {
-    return false;
-  }
-
-  if (this->_eventType == ProvenanceEventRecord::FORK || this->_eventType == ProvenanceEventRecord::CLONE || this->_eventType == ProvenanceEventRecord::JOIN) {
-    // write UUIDs
-    uint32_t parent_uuids_count = gsl::narrow<uint32_t>(this->_parentUuids.size());
-    ret = outStream.write(parent_uuids_count);
+  {
+    uint32_t eventType = this->_eventType;
+    const auto ret = outStream.write(eventType);
     if (ret != 4) {
       return false;
     }
-    for (const auto& parentUUID : _parentUuids) {
-      ret = outStream.write(parentUUID);
-      if (ret <= 0) {
+  }
+  {
+    uint64_t event_time_ms = std::chrono::duration_cast<std::chrono::milliseconds>(_eventTime.time_since_epoch()).count();
+    const auto ret = outStream.write(event_time_ms);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    uint64_t entry_date_ms = std::chrono::duration_cast<std::chrono::milliseconds>(_entryDate.time_since_epoch()).count();
+    const auto ret = outStream.write(entry_date_ms);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    uint64_t event_duration_ms = this->_eventDuration.count();
+    const auto ret = outStream.write(event_duration_ms);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    uint64_t lineage_start_date_ms = std::chrono::duration_cast<std::chrono::milliseconds>(_lineageStartDate.time_since_epoch()).count();
+    const auto ret = outStream.write(lineage_start_date_ms);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_componentId);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_componentType);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->flow_uuid_);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_details);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
+  }
+  // write flow attributes
+  {
+    const auto numAttributes = gsl::narrow<uint32_t>(this->_attributes.size());
+    const auto ret = outStream.write(numAttributes);
+    if (ret != 4) {
+      return false;
+    }
+  }
+  for (const auto& itAttribute : _attributes) {
+    {
+      const auto ret = outStream.write(itAttribute.first);
+      if (ret == 0 || io::isError(ret)) {
         return false;
       }
     }
-    uint32_t children_uuids_count = gsl::narrow<uint32_t>(this->_childrenUuids.size());
-    ret = outStream.write(children_uuids_count);
-    if (ret != 4) {
+    {
+      const auto ret = outStream.write(itAttribute.second);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_contentFullPath);
+    if (ret == 0 || io::isError(ret)) {
       return false;
     }
+  }
+  {
+    const auto ret = outStream.write(this->_size);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_offset);
+    if (ret != 8) {
+      return false;
+    }
+  }
+  {
+    const auto ret = outStream.write(this->_sourceQueueIdentifier);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
+  }
+  if (this->_eventType == ProvenanceEventRecord::FORK || this->_eventType == ProvenanceEventRecord::CLONE || this->_eventType == ProvenanceEventRecord::JOIN) {
+    // write UUIDs
+    {
+      const auto parent_uuids_count = gsl::narrow<uint32_t>(this->_parentUuids.size());
+      const auto ret = outStream.write(parent_uuids_count);
+      if (ret != 4) {
+        return false;
+      }
+    }
+    for (const auto& parentUUID : _parentUuids) {
+      const auto ret = outStream.write(parentUUID);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
+    }
+    {
+      const auto children_uuids_count = gsl::narrow<uint32_t>(this->_childrenUuids.size());
+      const auto ret = outStream.write(children_uuids_count);
+      if (ret != 4) {
+        return false;
+      }
+    }
     for (const auto& childUUID : _childrenUuids) {
-      ret = outStream.write(childUUID);
-      if (ret <= 0) {
+      const auto ret = outStream.write(childUUID);
+      if (ret == 0 || io::isError(ret)) {
         return false;
       }
     }
   } else if (this->_eventType == ProvenanceEventRecord::SEND || this->_eventType == ProvenanceEventRecord::FETCH) {
-    ret = outStream.write(this->_transitUri);
-    if (ret <= 0) {
+    const auto ret = outStream.write(this->_transitUri);
+    if (ret == 0 || io::isError(ret)) {
       return false;
     }
   } else if (this->_eventType == ProvenanceEventRecord::RECEIVE) {
-    ret = outStream.write(this->_transitUri);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.write(this->_transitUri);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
-    ret = outStream.write(this->_sourceSystemFlowFileIdentifier);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.write(this->_sourceSystemFlowFileIdentifier);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
   }
 
@@ -228,152 +253,206 @@ bool ProvenanceEventRecord::Serialize(const std::shared_ptr<core::SerializableCo
   Serialize(outStream);
 
   // Persist to the DB
-  if (!repo->Serialize(getUUIDStr(), const_cast<uint8_t*>(outStream.getBuffer()), outStream.size())) {
+  if (!repo->Serialize(getUUIDStr(), const_cast<uint8_t*>(outStream.getBuffer().as_span<const uint8_t>().data()), outStream.size())) {
     logger_->log_error("NiFi Provenance Store event %s size %llu fail", getUUIDStr(), outStream.size());
   }
   return true;
 }
 
-bool ProvenanceEventRecord::DeSerialize(const uint8_t *buffer, const size_t bufferSize) {
-  int ret;
+bool ProvenanceEventRecord::DeSerialize(const gsl::span<const std::byte> buffer) {
+  org::apache::nifi::minifi::io::BufferStream outStream(buffer);
 
-  org::apache::nifi::minifi::io::BufferStream outStream(buffer, gsl::narrow<unsigned int>(bufferSize));
-
-  ret = outStream.read(uuid_);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(uuid_);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
   uint32_t eventType;
-  ret = outStream.read(eventType);
-  if (ret != 4) {
-    return false;
+  {
+    const auto ret = outStream.read(eventType);
+    if (ret != 4) {
+      return false;
+    }
   }
+
   this->_eventType = (ProvenanceEventRecord::ProvenanceEventType) eventType;
-
-  ret = outStream.read(this->_eventTime);
-  if (ret != 8) {
-    return false;
+  {
+    uint64_t event_time_in_ms;
+    const auto ret = outStream.read(event_time_in_ms);
+    if (ret != 8) {
+      return false;
+    }
+    _eventTime = std::chrono::system_clock::time_point() + std::chrono::milliseconds(event_time_in_ms);
   }
 
-  ret = outStream.read(this->_entryDate);
-  if (ret != 8) {
-    return false;
+  {
+    uint64_t entry_date_in_ms;
+    const auto ret = outStream.read(entry_date_in_ms);
+    if (ret != 8) {
+      return false;
+    }
+    _entryDate = std::chrono::system_clock::time_point() + std::chrono::milliseconds(entry_date_in_ms);
   }
 
-  ret = outStream.read(this->_eventDuration);
-  if (ret != 8) {
-    return false;
+  {
+    uint64_t event_duration_ms;
+    const auto ret = outStream.read(event_duration_ms);
+    if (ret != 8) {
+      return false;
+    }
+    _eventDuration = std::chrono::milliseconds(event_duration_ms);
   }
 
-  ret = outStream.read(this->_lineageStartDate);
-  if (ret != 8) {
-    return false;
+  {
+    uint64_t lineage_start_date_in_ms;
+    const auto ret = outStream.read(lineage_start_date_in_ms);
+    if (ret != 8) {
+      return false;
+    }
+    _lineageStartDate = std::chrono::system_clock::time_point() + std::chrono::milliseconds(lineage_start_date_in_ms);
   }
 
-  ret = outStream.read(this->_componentId);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_componentId);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->_componentType);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_componentType);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->flow_uuid_);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->flow_uuid_);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->_details);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_details);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
   // read flow attributes
   uint32_t numAttributes = 0;
-  ret = outStream.read(numAttributes);
-  if (ret != 4) {
-    return false;
+  {
+    const auto ret = outStream.read(numAttributes);
+    if (ret != 4) {
+      return false;
+    }
   }
 
   for (uint32_t i = 0; i < numAttributes; i++) {
     std::string key;
-    ret = outStream.read(key);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.read(key);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
     std::string value;
-    ret = outStream.read(value);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.read(value);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
     this->_attributes[key] = value;
   }
 
-  ret = outStream.read(this->_contentFullPath);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_contentFullPath);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->_size);
-  if (ret != 8) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_size);
+    if (ret != 8) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->_offset);
-  if (ret != 8) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_offset);
+    if (ret != 8) {
+      return false;
+    }
   }
 
-  ret = outStream.read(this->_sourceQueueIdentifier);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = outStream.read(this->_sourceQueueIdentifier);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
   if (this->_eventType == ProvenanceEventRecord::FORK || this->_eventType == ProvenanceEventRecord::CLONE || this->_eventType == ProvenanceEventRecord::JOIN) {
     // read UUIDs
     uint32_t number = 0;
-    ret = outStream.read(number);
-    if (ret != 4) {
-      return false;
+    {
+      const auto ret = outStream.read(number);
+      if (ret != 4) {
+        return false;
+      }
     }
 
     for (uint32_t i = 0; i < number; i++) {
       utils::Identifier parentUUID;
-      ret = outStream.read(parentUUID);
-      if (ret <= 0) {
-        return false;
+      {
+        const auto ret = outStream.read(parentUUID);
+        if (ret == 0 || io::isError(ret)) {
+          return false;
+        }
       }
       this->addParentUuid(parentUUID);
     }
     number = 0;
-    ret = outStream.read(number);
-    if (ret != 4) {
-      return false;
+    {
+      const auto ret = outStream.read(number);
+      if (ret != 4) {
+        return false;
+      }
     }
     for (uint32_t i = 0; i < number; i++) {
       utils::Identifier childUUID;
-      ret = outStream.read(childUUID);
-      if (ret <= 0) {
-        return false;
+      {
+        const auto ret = outStream.read(childUUID);
+        if (ret == 0 || io::isError(ret)) {
+          return false;
+        }
       }
       this->addChildUuid(childUUID);
     }
   } else if (this->_eventType == ProvenanceEventRecord::SEND || this->_eventType == ProvenanceEventRecord::FETCH) {
-    ret = outStream.read(this->_transitUri);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.read(this->_transitUri);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
   } else if (this->_eventType == ProvenanceEventRecord::RECEIVE) {
-    ret = outStream.read(this->_transitUri);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.read(this->_transitUri);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
-    ret = outStream.read(this->_sourceSystemFlowFileIdentifier);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = outStream.read(this->_sourceSystemFlowFileIdentifier);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
   }
 
@@ -410,7 +489,7 @@ void ProvenanceReporter::create(std::shared_ptr<core::FlowFile> flow, std::strin
   }
 }
 
-void ProvenanceReporter::route(std::shared_ptr<core::FlowFile> flow, core::Relationship relation, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::route(std::shared_ptr<core::FlowFile> flow, core::Relationship relation, std::string detail, std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::ROUTE, flow);
 
   if (event) {
@@ -430,7 +509,7 @@ void ProvenanceReporter::modifyAttributes(std::shared_ptr<core::FlowFile> flow,
   }
 }
 
-void ProvenanceReporter::modifyContent(std::shared_ptr<core::FlowFile> flow, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::modifyContent(std::shared_ptr<core::FlowFile> flow, std::string detail, std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::CONTENT_MODIFIED, flow);
 
   if (event) {
@@ -450,7 +529,7 @@ void ProvenanceReporter::clone(std::shared_ptr<core::FlowFile> parent, std::shar
   }
 }
 
-void ProvenanceReporter::join(std::vector<std::shared_ptr<core::FlowFile> > parents, std::shared_ptr<core::FlowFile> child, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::join(std::vector<std::shared_ptr<core::FlowFile> > parents, std::shared_ptr<core::FlowFile> child, std::string detail, std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::JOIN, child);
 
   if (event) {
@@ -466,7 +545,7 @@ void ProvenanceReporter::join(std::vector<std::shared_ptr<core::FlowFile> > pare
   }
 }
 
-void ProvenanceReporter::fork(std::vector<std::shared_ptr<core::FlowFile> > child, std::shared_ptr<core::FlowFile> parent, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::fork(std::vector<std::shared_ptr<core::FlowFile> > child, std::shared_ptr<core::FlowFile> parent, std::string detail, std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::FORK, parent);
 
   if (event) {
@@ -501,7 +580,7 @@ void ProvenanceReporter::drop(std::shared_ptr<core::FlowFile> flow, std::string
   }
 }
 
-void ProvenanceReporter::send(std::shared_ptr<core::FlowFile> flow, std::string transitUri, std::string detail, uint64_t processingDuration, bool force) {
+void ProvenanceReporter::send(std::shared_ptr<core::FlowFile> flow, std::string transitUri, std::string detail, std::chrono::milliseconds processingDuration, bool force) {
   auto event = allocate(ProvenanceEventRecord::SEND, flow);
 
   if (event) {
@@ -517,7 +596,11 @@ void ProvenanceReporter::send(std::shared_ptr<core::FlowFile> flow, std::string
   }
 }
 
-void ProvenanceReporter::receive(std::shared_ptr<core::FlowFile> flow, std::string transitUri, std::string sourceSystemFlowFileIdentifier, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::receive(std::shared_ptr<core::FlowFile> flow,
+                                 std::string transitUri,
+                                 std::string sourceSystemFlowFileIdentifier,
+                                 std::string detail,
+                                 std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::RECEIVE, flow);
 
   if (event) {
@@ -529,7 +612,7 @@ void ProvenanceReporter::receive(std::shared_ptr<core::FlowFile> flow, std::stri
   }
 }
 
-void ProvenanceReporter::fetch(std::shared_ptr<core::FlowFile> flow, std::string transitUri, std::string detail, uint64_t processingDuration) {
+void ProvenanceReporter::fetch(std::shared_ptr<core::FlowFile> flow, std::string transitUri, std::string detail, std::chrono::milliseconds processingDuration) {
   auto event = allocate(ProvenanceEventRecord::FETCH, flow);
 
   if (event) {
