@@ -33,6 +33,8 @@
 #include "sitetosite/Peer.h"
 #include "utils/gsl.h"
 
+using namespace std::literals::chrono_literals;
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -114,28 +116,31 @@ bool RawSiteToSiteClient::initiateResourceNegotiation() {
 
   logger_->log_debug("Negotiate protocol version with destination port %s current version %d", port_id_.to_string(), _currentVersion);
 
-  int ret = peer_->write(getResourceName());
-
-  logger_->log_trace("result of writing resource name is %i", ret);
-  if (ret <= 0) {
-    logger_->log_debug("result of writing resource name is %i", ret);
-    // tearDown();
-    return false;
+  {
+    const auto ret = peer_->write(getResourceName());
+    logger_->log_trace("result of writing resource name is %i", ret);
+    if (ret == 0 || io::isError(ret)) {
+      logger_->log_debug("result of writing resource name is %i", ret);
+      // tearDown();
+      return false;
+    }
   }
 
-  ret = peer_->write(_currentVersion);
-
-  if (ret <= 0) {
-    logger_->log_debug("result of writing version is %i", ret);
-    return false;
+  {
+    const auto ret = peer_->write(_currentVersion);
+    if (ret == 0 || io::isError(ret)) {
+      logger_->log_debug("result of writing version is %i", ret);
+      return false;
+    }
   }
 
   uint8_t statusCode;
-  ret = peer_->read(statusCode);
-
-  if (ret <= 0) {
-    logger_->log_debug("result of writing version status code  %i", ret);
-    return false;
+  {
+    const auto ret = peer_->read(statusCode);
+    if (ret == 0 || io::isError(ret)) {
+      logger_->log_debug("result of writing version status code  %i", ret);
+      return false;
+    }
   }
   logger_->log_debug("status code is %i", statusCode);
   switch (statusCode) {
@@ -144,12 +149,14 @@ bool RawSiteToSiteClient::initiateResourceNegotiation() {
       return true;
     case DIFFERENT_RESOURCE_VERSION:
       uint32_t serverVersion;
-      ret = peer_->read(serverVersion);
-      if (ret <= 0) {
-        return false;
+      {
+        const auto ret = peer_->read(serverVersion);
+        if (ret == 0 || io::isError(ret)) {
+          return false;
+        }
       }
 
-      logging::LOG_INFO(logger_) << "Site2Site Server Response asked for a different protocol version " << serverVersion;
+      core::logging::LOG_INFO(logger_) << "Site2Site Server Response asked for a different protocol version " << serverVersion;
 
       for (unsigned int i = (_currentVersionIndex + 1); i < sizeof(_supportedVersion) / sizeof(uint32_t); i++) {
         if (serverVersion >= _supportedVersion[i]) {
@@ -178,38 +185,42 @@ bool RawSiteToSiteClient::initiateCodecResourceNegotiation() {
 
   logger_->log_trace("Negotiate Codec version with destination port %s current version %d", port_id_.to_string(), _currentCodecVersion);
 
-  int ret = peer_->write(getCodecResourceName());
-
-  if (ret <= 0) {
-    logger_->log_debug("result of getCodecResourceName is %i", ret);
-    return false;
+  {
+    const auto ret = peer_->write(getCodecResourceName());
+    if (ret == 0 || io::isError(ret)) {
+      logger_->log_debug("result of getCodecResourceName is %i", ret);
+      return false;
+    }
   }
 
-  ret = peer_->write(_currentCodecVersion);
-
-  if (ret <= 0) {
-    logger_->log_debug("result of _currentCodecVersion is %i", ret);
-    return false;
+  {
+    const auto ret = peer_->write(_currentCodecVersion);
+    if (ret == 0 || io::isError(ret)) {
+      logger_->log_debug("result of _currentCodecVersion is %i", ret);
+      return false;
+    }
   }
 
   uint8_t statusCode;
-  ret = peer_->read(statusCode);
-
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = peer_->read(statusCode);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
-
   switch (statusCode) {
     case RESOURCE_OK:
       logger_->log_trace("Site2Site Codec Negotiate version OK");
       return true;
     case DIFFERENT_RESOURCE_VERSION:
       uint32_t serverVersion;
-      ret = peer_->read(serverVersion);
-      if (ret <= 0) {
-        return false;
+      {
+        const auto ret = peer_->read(serverVersion);
+        if (ret == 0 || io::isError(ret)) {
+          return false;
+        }
       }
-      logging::LOG_INFO(logger_) << "Site2Site Server Response asked for a different protocol version " << serverVersion;
+      core::logging::LOG_INFO(logger_) << "Site2Site Server Response asked for a different protocol version " << serverVersion;
 
       for (unsigned int i = (_currentCodecVersionIndex + 1); i < sizeof(_supportedCodecVersion) / sizeof(uint32_t); i++) {
         if (serverVersion >= _supportedCodecVersion[i]) {
@@ -237,47 +248,54 @@ bool RawSiteToSiteClient::handShake() {
   logger_->log_debug("Site2Site Protocol Perform hand shake with destination port %s", port_id_.to_string());
   _commsIdentifier = id_generator_->generate();
 
-  int ret = peer_->write(_commsIdentifier);
-
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = peer_->write(_commsIdentifier);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
   std::map<std::string, std::string> properties;
   properties[HandShakePropertyStr[GZIP]] = "false";
   properties[HandShakePropertyStr[PORT_IDENTIFIER]] = port_id_.to_string();
-  properties[HandShakePropertyStr[REQUEST_EXPIRATION_MILLIS]] = std::to_string(_timeOut);
+  properties[HandShakePropertyStr[REQUEST_EXPIRATION_MILLIS]] = std::to_string(_timeout.load().count());
   if (_currentVersion >= 5) {
     if (_batchCount > 0)
       properties[HandShakePropertyStr[BATCH_COUNT]] = std::to_string(_batchCount);
     if (_batchSize > 0)
       properties[HandShakePropertyStr[BATCH_SIZE]] = std::to_string(_batchSize);
-    if (_batchDuration > 0)
-      properties[HandShakePropertyStr[BATCH_DURATION]] = std::to_string(_batchDuration);
+    if (_batchDuration.load() > 0ms)
+      properties[HandShakePropertyStr[BATCH_DURATION]] = std::to_string(_batchDuration.load().count());
   }
 
   if (_currentVersion >= 3) {
-    ret = peer_->write(peer_->getURL());
-    if (ret <= 0) {
+    const auto ret = peer_->write(peer_->getURL());
+    if (ret == 0 || io::isError(ret)) {
       return false;
     }
   }
 
-  uint32_t size = gsl::narrow<uint32_t>(properties.size());
-  ret = peer_->write(size);
-  if (ret <= 0) {
-    return false;
+  {
+    const auto size = gsl::narrow<uint32_t>(properties.size());
+    const auto ret = peer_->write(size);
+    if (ret == 0 || io::isError(ret)) {
+      return false;
+    }
   }
 
   std::map<std::string, std::string>::iterator it;
   for (it = properties.begin(); it != properties.end(); it++) {
-    ret = peer_->write(it->first);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = peer_->write(it->first);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
-    ret = peer_->write(it->second);
-    if (ret <= 0) {
-      return false;
+    {
+      const auto ret = peer_->write(it->second);
+      if (ret == 0 || io::isError(ret)) {
+        return false;
+      }
     }
     logger_->log_debug("Site2Site Protocol Send handshake properties %s %s", it->first, it->second);
   }
@@ -285,13 +303,16 @@ bool RawSiteToSiteClient::handShake() {
   RespondCode code;
   std::string message;
 
-  ret = readRespond(nullptr, code, message);
-
-  if (ret <= 0) {
-    return false;
+  {
+    const auto ret = readRespond(nullptr, code, message);
+    if (ret <= 0) {
+      return false;
+    }
   }
 
-  std::string error;
+  auto logPortStateError = [this](const std::string& error) {
+    logger_->log_error("Site2Site HandShake Failed because destination port, %s, is %s", port_id_.to_string(), error);
+  };
 
   switch (code) {
     case PROPERTIES_OK:
@@ -299,25 +320,21 @@ bool RawSiteToSiteClient::handShake() {
       peer_state_ = HANDSHAKED;
       return true;
     case PORT_NOT_IN_VALID_STATE:
-      error = "in invalid state";
-      break;
+      logPortStateError("in invalid state");
+      return false;
     case UNKNOWN_PORT:
-      error = "an unknown port";
-      break;
+      logPortStateError("an unknown port");
+      return false;
     case PORTS_DESTINATION_FULL:
-      error = "full";
-      break;
-    // Unknown error
+      logPortStateError("full");
+      return false;
+    case UNAUTHORIZED:
+      logger_->log_error("Site2Site HandShake on port %s failed: UNAUTHORIZED", port_id_.to_string());
+      return false;
     default:
-      logger_->log_error("HandShake Failed because of unknown respond code %d", code);
-      ret = -1;
+      logger_->log_error("Site2Site HandShake on port %s failed: unknown respond code %d", port_id_.to_string(), code);
       return false;
   }
-
-  // All known error cases handled here
-  logger_->log_error("Site2Site HandShake Failed because destination port, %s, is %s", port_id_.to_string(), error);
-  ret = -1;
-  return false;
 }
 
 void RawSiteToSiteClient::tearDown() {
@@ -334,49 +351,55 @@ void RawSiteToSiteClient::tearDown() {
 
 bool RawSiteToSiteClient::getPeerList(std::vector<PeerStatus> &peers) {
   if (establish() && handShake()) {
-    int status = writeRequestType(REQUEST_PEER_LIST);
-
-    if (status <= 0) {
+    if (writeRequestType(REQUEST_PEER_LIST) <= 0) {
       tearDown();
       return false;
     }
 
-    uint32_t number;
-    status = peer_->read(number);
-
-    if (status <= 0) {
-      tearDown();
-      return false;
-    }
-
-    for (uint32_t i = 0; i < number; i++) {
-      std::string host;
-      status = peer_->read(host);
-      if (status <= 0) {
+    uint32_t number_of_peers;
+    {
+      const auto ret = peer_->read(number_of_peers);
+      if (ret == 0 || io::isError(ret)) {
         tearDown();
         return false;
       }
+    }
+
+    for (uint32_t i = 0; i < number_of_peers; i++) {
+      std::string host;
+      {
+        const auto ret = peer_->read(host);
+        if (ret == 0 || io::isError(ret)) {
+          tearDown();
+          return false;
+        }
+      }
       uint32_t port;
-      status = peer_->read(port);
-      if (status <= 0) {
-        tearDown();
-        return false;
+      {
+        const auto ret = peer_->read(port);
+        if (ret == 0 || io::isError(ret)) {
+          tearDown();
+          return false;
+        }
       }
       uint8_t secure;
-      status = peer_->read(secure);
-      if (status <= 0) {
-        tearDown();
-        return false;
+      {
+        const auto ret = peer_->read(secure);
+        if (ret == 0 || io::isError(ret)) {
+          tearDown();
+          return false;
+        }
       }
       uint32_t count;
-      status = peer_->read(count);
-      if (status <= 0) {
-        tearDown();
-        return false;
+      {
+        const auto ret = peer_->read(count);
+        if (ret == 0 || io::isError(ret)) {
+          tearDown();
+          return false;
+        }
       }
-      PeerStatus status(std::make_shared<Peer>(port_id_, host, gsl::narrow<uint16_t>(port), secure != 0), count, true);
-      peers.push_back(std::move(status));
-      logging::LOG_TRACE(logger_) << "Site2Site Peer host " << host << " port " << port << " Secure " << std::to_string(secure);
+      peers.push_back(PeerStatus{std::make_shared<Peer>(port_id_, host, gsl::narrow<uint16_t>(port), secure != 0), count, true});
+      core::logging::LOG_TRACE(logger_) << "Site2Site Peer host " << host << " port " << port << " Secure " << std::to_string(secure);
     }
 
     tearDown();
@@ -391,21 +414,21 @@ bool RawSiteToSiteClient::getPeerList(std::vector<PeerStatus> &peers) {
     if (type >= MAX_REQUEST_TYPE)
       return -1;
 
-    return peer_->write(SiteToSiteRequest::RequestTypeStr[type]);
+    const auto write_result = peer_->write(SiteToSiteRequest::RequestTypeStr[type]);
+    return io::isError(write_result) ? -1 : gsl::narrow<int>(write_result);
   }
 
   int RawSiteToSiteClient::readRequestType(RequestType &type) {
     std::string requestTypeStr;
 
-    int ret = peer_->read(requestTypeStr);
-
-    if (ret <= 0)
-      return ret;
+    const auto ret = peer_->read(requestTypeStr);
+    if (ret == 0 || io::isError(ret))
+      return static_cast<int>(ret);
 
     for (int i = NEGOTIATE_FLOWFILE_CODEC; i <= SHUTDOWN; i++) {
       if (SiteToSiteRequest::RequestTypeStr[i] == requestTypeStr) {
         type = (RequestType) i;
-        return ret;
+        return static_cast<int>(ret);
       }
     }
 
@@ -417,7 +440,7 @@ int RawSiteToSiteClient::readRespond(const std::shared_ptr<Transaction> &transac
 }
 
 int RawSiteToSiteClient::writeRespond(const std::shared_ptr<Transaction> &transaction, RespondCode code, std::string message) {
-  return writeResponse(transaction, code, message);
+  return writeResponse(transaction, code, std::move(message));
 }
 
 bool RawSiteToSiteClient::negotiateCodec() {
@@ -567,7 +590,7 @@ bool RawSiteToSiteClient::transmitPayload(const std::shared_ptr<core::ProcessCon
     if (resp == -1) {
       throw Exception(SITE2SITE_EXCEPTION, "Send Failed in transaction " + transactionID.to_string());
     }
-    logging::LOG_INFO(logger_) << "Site2Site transaction " << transactionID.to_string() << " sent bytes length" << payload.length();
+    core::logging::LOG_INFO(logger_) << "Site2Site transaction " << transactionID.to_string() << " sent bytes length" << payload.length();
 
     if (!confirm(transactionID)) {
       throw Exception(SITE2SITE_EXCEPTION, "Confirm Failed in transaction " + transactionID.to_string());
@@ -575,7 +598,7 @@ bool RawSiteToSiteClient::transmitPayload(const std::shared_ptr<core::ProcessCon
     if (!complete(transactionID)) {
       throw Exception(SITE2SITE_EXCEPTION, "Complete Failed in transaction " + transactionID.to_string());
     }
-    logging::LOG_INFO(logger_) << "Site2Site transaction " << transactionID.to_string()
+    core::logging::LOG_INFO(logger_) << "Site2Site transaction " << transactionID.to_string()
         << " successfully send flow record " << transaction->current_transfers_ << " content bytes " << transaction->_bytes;
   } catch (std::exception &exception) {
     if (transaction)
