@@ -22,11 +22,12 @@
 #include <map>
 
 #include "utils/gsl.h"
+#include "Exception.h"
 
 #ifdef __linux__
 #include <sys/sysinfo.h>
+#include <optional>
 #include <sstream>
-#include "utils/OptionalUtils.h"
 #endif
 
 #ifdef _WIN32
@@ -36,16 +37,19 @@
 #include <Windows.h>
 #include <sddl.h>
 #include <psapi.h>
+#include <winsock2.h>
 #include <vector>
 #include <algorithm>
+#include <WS2tcpip.h>
 #pragma comment(lib, "Ws2_32.lib")
 #else
 #include <sys/utsname.h>
 #include <pwd.h>
 #include <sys/types.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <fstream>
-
+#include <cstring>
 #endif
 
 #ifdef __APPLE__
@@ -208,8 +212,8 @@ int64_t OsUtils::getSystemPhysicalMemoryUsage() {
   std::ifstream meminfo_file("/proc/meminfo");
   std::string line;
 
-  minifi::utils::optional<uint64_t> total_memory_kByte;
-  minifi::utils::optional<uint64_t> available_memory_kByte;
+  std::optional<uint64_t> total_memory_kByte;
+  std::optional<uint64_t> available_memory_kByte;
   while ((!total_memory_kByte.has_value() || !available_memory_kByte.has_value()) && std::getline(meminfo_file, line)) {
     if (line.rfind(total_memory_prefix, 0) == 0) {
       std::istringstream total_memory_line(line.substr(total_memory_prefix.length()));
@@ -222,7 +226,7 @@ int64_t OsUtils::getSystemPhysicalMemoryUsage() {
     }
   }
   if (total_memory_kByte.has_value() && available_memory_kByte.has_value())
-    return (total_memory_kByte.value() - available_memory_kByte.value()) * 1024;
+    return (gsl::narrow<int64_t>(total_memory_kByte.value()) - gsl::narrow<int64_t>(available_memory_kByte.value())) * 1024;
 
   return -1;
 #elif defined(__APPLE__)
@@ -252,11 +256,11 @@ int64_t OsUtils::getSystemPhysicalMemoryUsage() {
 
 int64_t OsUtils::getSystemTotalPhysicalMemory() {
 #if defined(__linux__)
-  struct sysinfo memory_info;
+  struct sysinfo memory_info{};
   sysinfo(&memory_info);
   uint64_t total_physical_memory = memory_info.totalram;
   total_physical_memory *= memory_info.mem_unit;
-  return total_physical_memory;
+  return gsl::narrow<int64_t>(total_physical_memory);
 #elif defined(__APPLE__)
   int mib[2];
   int64_t total_physical_memory = 0;
@@ -312,8 +316,10 @@ std::string OsUtils::getMachineArchitecture() {
   else
     return buf.machine;
 #endif
+
   return "unknown";
 }
+
 }  // namespace utils
 }  // namespace minifi
 }  // namespace nifi
