@@ -27,6 +27,7 @@
 #include <iostream>
 #include "FlowController.h"
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "core/Core.h"
 #include "../../include/core/FlowFile.h"
 #include "../unit/ProvenanceTestHelper.h"
@@ -41,55 +42,50 @@
 #include "processors/PutFile.h"
 #include "utils/file/FileUtils.h"
 #include "../Utils.h"
+#include "utils/gsl.h"
 
-class ReadCallback: public minifi::InputStreamCallback {
+class ReadCallback {
  public:
-  explicit ReadCallback(size_t size) :
-      read_size_(0) {
-    buffer_size_ = size;
-    buffer_ = new uint8_t[buffer_size_];
-    archive_buffer_ = nullptr;
-    archive_buffer_size_ = 0;
-  }
-  ~ReadCallback() {
-    if (buffer_)
-      delete[] buffer_;
-    if (archive_buffer_)
-      delete[] archive_buffer_;
-  }
-  int64_t process(const std::shared_ptr<minifi::io::BaseStream>& stream) {
+  explicit ReadCallback(size_t size)
+      :buffer_{size}
+  {}
+  ReadCallback(const ReadCallback&) = delete;
+  ReadCallback(ReadCallback&&) = delete;
+  ReadCallback& operator=(const ReadCallback&) = delete;
+  ReadCallback& operator=(ReadCallback&&) = delete;
+
+  int64_t operator()(const std::shared_ptr<minifi::io::BaseStream>& stream) {
     int64_t total_read = 0;
-    int64_t ret = 0;
     do {
-      ret = stream->read(buffer_ + read_size_, gsl::narrow<int>(buffer_size_ - read_size_));
+      const auto ret = stream->read(gsl::make_span(buffer_).subspan(read_size_));
       if (ret == 0) break;
-      if (ret < 0) return ret;
+      if (minifi::io::isError(ret)) return -1;
       read_size_ += gsl::narrow<size_t>(ret);
       total_read += ret;
-    } while (buffer_size_ != read_size_);
+    } while (buffer_.size() != read_size_);
     return total_read;
   }
   void archive_read() {
-    struct archive *a;
-    a = archive_read_new();
-    archive_read_support_format_all(a);
-    archive_read_support_filter_all(a);
-    archive_read_open_memory(a, buffer_, read_size_);
+    struct archive_read_deleter { void operator()(struct archive* p) const noexcept { archive_read_free(p); } };
+    std::unique_ptr<struct archive, archive_read_deleter> a{archive_read_new()};
+    archive_read_support_format_all(a.get());
+    archive_read_support_filter_all(a.get());
+    archive_read_open_memory(a.get(), buffer_.data(), read_size_);
     struct archive_entry *ae;
 
-    REQUIRE(archive_read_next_header(a, &ae) == ARCHIVE_OK);
-    int size = gsl::narrow<int>(archive_entry_size(ae));
-    archive_buffer_ = new char[size];
-    archive_buffer_size_ = size;
-    archive_read_data(a, archive_buffer_, size);
-    archive_read_free(a);
+    REQUIRE(archive_read_next_header(a.get(), &ae) == ARCHIVE_OK);
+    const auto size = [&] {
+      const auto size = archive_entry_size(ae);
+      REQUIRE(size >= 0);
+      return gsl::narrow<size_t>(size);
+    }();
+    archive_buffer_.resize(size);
+    archive_read_data(a.get(), archive_buffer_.data(), size);
   }
 
-  uint8_t *buffer_;
-  size_t buffer_size_;
-  size_t read_size_;
-  char *archive_buffer_;
-  int archive_buffer_size_;
+  std::vector<std::byte> buffer_;
+  size_t read_size_ = 0;
+  std::vector<std::byte> archive_buffer_;
 };
 
 /**
@@ -109,8 +105,8 @@ class CompressDecompressionTestController : public TestController{
   }
 
   void setupFlow() {
-    LogTestController::getInstance().setTrace<processors::CompressContent>();
-    LogTestController::getInstance().setTrace<processors::LogAttribute>();
+    LogTestController::getInstance().setTrace<minifi::processors::CompressContent>();
+    LogTestController::getInstance().setTrace<minifi::processors::LogAttribute>();
     LogTestController::getInstance().setTrace<core::ProcessSession>();
     LogTestController::getInstance().setTrace<core::ProcessContext>();
     LogTestController::getInstance().setTrace<core::repository::VolatileContentRepository>();
@@ -120,31 +116,66 @@ class CompressDecompressionTestController : public TestController{
 
     std::shared_ptr<TestRepository> repo = std::make_shared<TestRepository>();
 
-    processor = std::make_shared<processors::CompressContent>("compresscontent");
+    processor = std::make_shared<minifi::processors::CompressContent>("compresscontent");
     processor->initialize();
     utils::Identifier processoruuid = processor->getUUID();
     REQUIRE(processoruuid);
 
     std::shared_ptr<core::ContentRepository> content_repo = std::make_shared<core::repository::VolatileContentRepository>();
     content_repo->initialize(std::make_shared<minifi::Configure>());
-    // connection from compress processor to log attribute
+    // connection from compress processor to success
     output = std::make_shared<minifi::Connection>(repo, content_repo, "Output");
     output->addRelationship(core::Relationship("success", "compress successful output"));
-    output->setSource(processor);
+    output->setSource(processor.get());
     output->setSourceUUID(processoruuid);
-    processor->addConnection(output);
+    processor->addConnection(output.get());
     // connection to compress processor
     input = std::make_shared<minifi::Connection>(repo, content_repo, "Input");
-    input->setDestination(processor);
+    input->setDestination(processor.get());
     input->setDestinationUUID(processoruuid);
-    processor->addConnection(input);
+    processor->addConnection(input.get());
 
-    processor->setAutoTerminatedRelationships({{"failure", ""}});
+    // connection from compress processor to failure
+    failure_output = std::make_shared<minifi::Connection>(repo, content_repo, "FailureOutput");
+    failure_output->addRelationship(core::Relationship("failure", "compress failure output"));
+    failure_output->setSource(processor.get());
+    failure_output->setSourceUUID(processoruuid);
+    processor->addConnection(failure_output.get());
 
     processor->incrementActiveTasks();
     processor->setScheduledState(core::ScheduledState::RUNNING);
 
-    context = std::make_shared<core::ProcessContext>(std::make_shared<core::ProcessorNode>(processor), nullptr, repo, repo, content_repo);
+    context = std::make_shared<core::ProcessContext>(std::make_shared<core::ProcessorNode>(processor.get()), nullptr, repo, repo, content_repo);
+    helper_session = std::make_shared<core::ProcessSession>(context);
+  }
+
+  std::shared_ptr<core::FlowFile> importFlowFile(const std::string& content_path) {
+    std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast<core::FlowFile>(helper_session->create());
+    helper_session->import(content_path, flow, true, 0);
+    helper_session->flushContent();
+    input->put(flow);
+    return flow;
+  }
+
+  template<typename T>
+  std::shared_ptr<core::FlowFile> importFlowFileFrom(T&& source) {
+    std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast<core::FlowFile>(helper_session->create());
+    helper_session->importFrom(std::forward<T>(source), flow);
+    helper_session->flushContent();
+    input->put(flow);
+    return flow;
+  }
+
+  void trigger() {
+    auto factory = std::make_shared<core::ProcessSessionFactory>(context);
+    processor->onSchedule(context, factory);
+    auto session = std::make_shared<core::ProcessSession>(context);
+    processor->onTrigger(context, session);
+    session->commit();
+  }
+
+  void read(const std::shared_ptr<core::FlowFile>& file, ReadCallback& reader) {
+    helper_session->read(file, std::ref(reader));
   }
 
  public:
@@ -176,11 +207,15 @@ class CompressDecompressionTestController : public TestController{
     return RawContent{std::move(contents)};
   }
 
+
+
   virtual ~CompressDecompressionTestController() = 0;
 
   std::shared_ptr<core::Processor> processor;
+  std::shared_ptr<core::ProcessSession> helper_session;
   std::shared_ptr<core::ProcessContext> context;
   std::shared_ptr<minifi::Connection> output;
+  std::shared_ptr<minifi::Connection> failure_output;
   std::shared_ptr<minifi::Connection> input;
 };
 
@@ -207,8 +242,7 @@ class CompressTestController : public CompressDecompressionTestController {
 
  public:
   CompressTestController() {
-    char CompressionFormat[] = "/tmp/test.XXXXXX";
-    tempDir_ = get_global_controller().createTempDirectory(CompressionFormat);
+    tempDir_ = get_global_controller().createTempDirectory();
     REQUIRE(!tempDir_.empty());
     raw_content_path_ = utils::file::FileUtils::concat_path(tempDir_, "minifi-expect-compresscontent.txt");
     compressed_content_path_ = utils::file::FileUtils::concat_path(tempDir_, "minifi-compresscontent");
@@ -235,27 +269,19 @@ class DecompressTestController : public CompressDecompressionTestController{
   }
 };
 
-using CompressionFormat = processors::CompressContent::ExtendedCompressionFormat;
-using CompressionMode = processors::CompressContent::CompressionMode;
+using CompressionFormat = minifi::processors::CompressContent::ExtendedCompressionFormat;
+using CompressionMode = minifi::processors::CompressContent::CompressionMode;
 
 TEST_CASE_METHOD(CompressTestController, "CompressFileGZip", "[compressfiletest1]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(rawContentPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
+  auto flow = importFlowFile(rawContentPath());
+  flow->setAttribute(core::SpecialFlowAttribute::FILENAME, "inputfile");
 
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  trigger();
 
   // validate the compress content
   std::set<std::shared_ptr<core::FlowFile>> expiredFlowRecords;
@@ -263,37 +289,31 @@ TEST_CASE_METHOD(CompressTestController, "CompressFileGZip", "[compressfiletest1
   REQUIRE(flow1->getSize() > 0);
   {
     REQUIRE(flow1->getSize() != flow->getSize());
-    std::string mime;
-    flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime);
-    REQUIRE(mime == "application/gzip");
+    std::string attribute_value;
+    flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, attribute_value);
+    REQUIRE(attribute_value == "application/gzip");
+    flow1->getAttribute(core::SpecialFlowAttribute::FILENAME, attribute_value);
+    REQUIRE(attribute_value == "inputfile.tar.gz");
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
+    read(flow1, callback);
     callback.archive_read();
-    std::string content(reinterpret_cast<char *> (callback.archive_buffer_), callback.archive_buffer_size_);
+    std::string content(reinterpret_cast<char *> (callback.archive_buffer_.data()), callback.archive_buffer_.size());
     REQUIRE(getRawContent() == content);
     // write the compress content for next test
-    writeCompressed(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    writeCompressed(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
   }
 }
 
 TEST_CASE_METHOD(DecompressTestController, "DecompressFileGZip", "[compressfiletest2]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(compressedPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
+  auto flow = importFlowFile(compressedPath());
+  flow->setAttribute(core::SpecialFlowAttribute::FILENAME, "inputfile.tar.gz");
 
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  trigger();
 
   // validate the compress content
   std::set<std::shared_ptr<core::FlowFile>> expiredFlowRecords;
@@ -301,33 +321,25 @@ TEST_CASE_METHOD(DecompressTestController, "DecompressFileGZip", "[compressfilet
   REQUIRE(flow1->getSize() > 0);
   {
     REQUIRE(flow1->getSize() != flow->getSize());
-    std::string mime;
-    REQUIRE(flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime) == false);
+    std::string attribute_value;
+    REQUIRE_FALSE(flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, attribute_value));
+    flow1->getAttribute(core::SpecialFlowAttribute::FILENAME, attribute_value);
+    REQUIRE(attribute_value == "inputfile");
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
-    std::string content(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    read(flow1, callback);
+    std::string content(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
     REQUIRE(getRawContent() == content);
   }
 }
 
 TEST_CASE_METHOD(CompressTestController, "CompressFileBZip", "[compressfiletest3]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::BZIP2));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::BZIP2));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(rawContentPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
-
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  auto flow = importFlowFile(rawContentPath());
+  trigger();
 
   // validate the compress content
   std::set<std::shared_ptr<core::FlowFile>> expiredFlowRecords;
@@ -339,34 +351,25 @@ TEST_CASE_METHOD(CompressTestController, "CompressFileBZip", "[compressfiletest3
     flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime);
     REQUIRE(mime == "application/bzip2");
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
+    read(flow1, callback);
     callback.archive_read();
-    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_), callback.archive_buffer_size_);
+    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_.data()), callback.archive_buffer_.size());
     REQUIRE(getRawContent() == contents);
     // write the compress content for next test
-    writeCompressed(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    writeCompressed(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
   }
 }
 
 
 TEST_CASE_METHOD(DecompressTestController, "DecompressFileBZip", "[compressfiletest4]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::BZIP2));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::BZIP2));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(compressedPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
+  auto flow = importFlowFile(compressedPath());
 
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  trigger();
 
   // validate the compress content
   std::set<std::shared_ptr<core::FlowFile>> expiredFlowRecords;
@@ -377,30 +380,20 @@ TEST_CASE_METHOD(DecompressTestController, "DecompressFileBZip", "[compressfilet
     std::string mime;
     REQUIRE(flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime) == false);
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
-    std::string contents(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    read(flow1, callback);
+    std::string contents(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
     REQUIRE(getRawContent() == contents);
   }
 }
 
 TEST_CASE_METHOD(CompressTestController, "CompressFileLZMA", "[compressfiletest5]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::LZMA));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::LZMA));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(rawContentPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
-
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  auto flow = importFlowFile(rawContentPath());
+  trigger();
 
   if (LogTestController::getInstance().contains("compression not supported on this platform")) {
     // platform not support LZMA
@@ -418,35 +411,25 @@ TEST_CASE_METHOD(CompressTestController, "CompressFileLZMA", "[compressfiletest5
     flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime);
     REQUIRE(mime == "application/x-lzma");
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
+    read(flow1, callback);
     callback.archive_read();
-    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_), callback.archive_buffer_size_);
+    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_.data()), callback.archive_buffer_.size());
     REQUIRE(getRawContent() == contents);
     // write the compress content for next test
-    writeCompressed(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    writeCompressed(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
   }
 }
 
 
 TEST_CASE_METHOD(DecompressTestController, "DecompressFileLZMA", "[compressfiletest6]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::USE_MIME_TYPE));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::USE_MIME_TYPE));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(compressedPath(), flow, true, 0);
+  auto flow = importFlowFile(compressedPath());
   flow->setAttribute(core::SpecialFlowAttribute::MIME_TYPE, "application/x-lzma");
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
-
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  trigger();
 
   if (LogTestController::getInstance().contains("compression not supported on this platform")) {
     // platform not support LZMA
@@ -463,30 +446,20 @@ TEST_CASE_METHOD(DecompressTestController, "DecompressFileLZMA", "[compressfilet
     std::string mime;
     REQUIRE(flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime) == false);
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
-    std::string contents(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    read(flow1, callback);
+    std::string contents(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
     REQUIRE(getRawContent() == contents);
   }
 }
 
 TEST_CASE_METHOD(CompressTestController, "CompressFileXYLZMA", "[compressfiletest7]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::XZ_LZMA2));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::XZ_LZMA2));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(rawContentPath(), flow, true, 0);
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
-
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  auto flow = importFlowFile(rawContentPath());
+  trigger();
 
   if (LogTestController::getInstance().contains("compression not supported on this platform")) {
     // platform not support LZMA
@@ -504,35 +477,25 @@ TEST_CASE_METHOD(CompressTestController, "CompressFileXYLZMA", "[compressfiletes
     flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime);
     REQUIRE(mime == "application/x-xz");
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
+    read(flow1, callback);
     callback.archive_read();
-    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_), callback.archive_buffer_size_);
+    std::string contents(reinterpret_cast<char *> (callback.archive_buffer_.data()), callback.archive_buffer_.size());
     REQUIRE(getRawContent() == contents);
     // write the compress content for next test
-    writeCompressed(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    writeCompressed(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
   }
 }
 
 
 TEST_CASE_METHOD(DecompressTestController, "DecompressFileXYLZMA", "[compressfiletest8]") {
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::USE_MIME_TYPE));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::USE_MIME_TYPE));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
 
-  core::ProcessSession sessionGenFlowFile(context);
-  std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast < core::FlowFile > (sessionGenFlowFile.create());
-  sessionGenFlowFile.import(compressedPath(), flow, true, 0);
+  auto flow = importFlowFile(compressedPath());
   flow->setAttribute(core::SpecialFlowAttribute::MIME_TYPE, "application/x-xz");
-  sessionGenFlowFile.flushContent();
-  input->put(flow);
-
-  REQUIRE(processor->getName() == "compresscontent");
-  auto factory = std::make_shared<core::ProcessSessionFactory>(context);
-  processor->onSchedule(context, factory);
-  auto session = std::make_shared<core::ProcessSession>(context);
-  processor->onTrigger(context, session);
-  session->commit();
+  trigger();
 
   if (LogTestController::getInstance().contains("compression not supported on this platform")) {
     // platform not support LZMA
@@ -549,23 +512,21 @@ TEST_CASE_METHOD(DecompressTestController, "DecompressFileXYLZMA", "[compressfil
     std::string mime;
     REQUIRE(flow1->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime) == false);
     ReadCallback callback(gsl::narrow<size_t>(flow1->getSize()));
-    sessionGenFlowFile.read(flow1, &callback);
-    std::string contents(reinterpret_cast<char *> (callback.buffer_), callback.read_size_);
+    read(flow1, callback);
+    std::string contents(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
     REQUIRE(getRawContent() == contents);
   }
 }
 
 TEST_CASE_METHOD(TestController, "RawGzipCompressionDecompression", "[compressfiletest8]") {
-  LogTestController::getInstance().setTrace<processors::CompressContent>();
-  LogTestController::getInstance().setTrace<processors::PutFile>();
+  LogTestController::getInstance().setTrace<minifi::processors::CompressContent>();
+  LogTestController::getInstance().setTrace<minifi::processors::PutFile>();
 
   // Create temporary directories
-  char format_src[] = "/tmp/archives.XXXXXX";
-  std::string src_dir = createTempDirectory(format_src);
+  std::string src_dir = createTempDirectory();
   REQUIRE(!src_dir.empty());
 
-  char format_dst[] = "/tmp/archived.XXXXXX";
-  std::string dst_dir = createTempDirectory(format_dst);
+  std::string dst_dir = createTempDirectory();
   REQUIRE(!dst_dir.empty());
 
   // Define files
@@ -665,19 +626,14 @@ TEST_CASE_METHOD(CompressTestController, "Batch CompressFileGZip", "[compressFil
   };
   const std::size_t batchSize = 3;
 
-  context->setProperty(processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
-  context->setProperty(processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
-  context->setProperty(processors::CompressContent::CompressLevel, "9");
-  context->setProperty(processors::CompressContent::UpdateFileName, "true");
-  context->setProperty(processors::CompressContent::BatchSize, std::to_string(batchSize));
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Compress));
+  context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
+  context->setProperty(minifi::processors::CompressContent::BatchSize, std::to_string(batchSize));
 
-
-  core::ProcessSession sessionGenFlowFile(context);
   for (const auto& content : flowFileContents) {
-    auto flow = sessionGenFlowFile.create();
-    sessionGenFlowFile.importFrom(minifi::io::BufferStream(content), flow);
-    sessionGenFlowFile.flushContent();
-    input->put(flow);
+    importFlowFileFrom(minifi::io::BufferStream(content));
   }
 
   REQUIRE(processor->getName() == "compresscontent");
@@ -717,9 +673,49 @@ TEST_CASE_METHOD(CompressTestController, "Batch CompressFileGZip", "[compressFil
     file->getAttribute(core::SpecialFlowAttribute::MIME_TYPE, mime);
     REQUIRE(mime == "application/gzip");
     ReadCallback callback(gsl::narrow<size_t>(file->getSize()));
-    sessionGenFlowFile.read(file, &callback);
+    read(file, callback);
     callback.archive_read();
-    std::string content(reinterpret_cast<char *> (callback.archive_buffer_), callback.archive_buffer_size_);
+    std::string content(reinterpret_cast<char *> (callback.archive_buffer_.data()), callback.archive_buffer_.size());
     REQUIRE(flowFileContents[idx] == content);
   }
 }
+
+TEST_CASE_METHOD(DecompressTestController, "Invalid archive decompression", "[compressfiletest9]") {
+  context->setProperty(minifi::processors::CompressContent::CompressMode, toString(CompressionMode::Decompress));
+  SECTION("GZIP") {
+    context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::GZIP));
+  }
+  SECTION("LZMA") {
+    context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::LZMA));
+  }
+  SECTION("XZ_LZMA2") {
+    context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::XZ_LZMA2));
+  }
+  SECTION("BZIP2") {
+    context->setProperty(minifi::processors::CompressContent::CompressFormat, toString(CompressionFormat::BZIP2));
+  }
+  context->setProperty(minifi::processors::CompressContent::CompressLevel, "9");
+  context->setProperty(minifi::processors::CompressContent::UpdateFileName, "true");
+
+  importFlowFileFrom(minifi::io::BufferStream(std::string{"banana bread"}));
+  trigger();
+
+  if (LogTestController::getInstance().contains("compression not supported on this platform")) {
+    return;
+  }
+
+  // validate the compress content
+  std::set<std::shared_ptr<core::FlowFile>> expiredFlowRecords;
+  REQUIRE_FALSE(output->poll(expiredFlowRecords));
+  REQUIRE(expiredFlowRecords.empty());
+
+  auto invalid_flow = failure_output->poll(expiredFlowRecords);
+  REQUIRE(invalid_flow);
+  REQUIRE(expiredFlowRecords.empty());
+  {
+    ReadCallback callback(gsl::narrow<size_t>(invalid_flow->getSize()));
+    read(invalid_flow, callback);
+    std::string contents(reinterpret_cast<char *> (callback.buffer_.data()), callback.read_size_);
+    REQUIRE(contents == "banana bread");
+  }
+}
