@@ -26,6 +26,7 @@
 
 #include "core/Processor.h"
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "processors/GetFile.h"
 #include "processors/LogAttribute.h"
 #include "processors/UpdateAttribute.h"
@@ -34,8 +35,6 @@
 #include "utils/TestUtils.h"
 #include "AWSCredentialsProvider.h"
 
-using org::apache::nifi::minifi::utils::createTempDir;
-
 template<typename T>
 class S3TestsFixture {
  public:
@@ -44,17 +43,10 @@ class S3TestsFixture {
   const std::string S3_BUCKET = "testBucket";
 
   S3TestsFixture() {
-    // Disable retrieving AWS metadata for tests
-    #ifdef WIN32
-    _putenv_s("AWS_EC2_METADATA_DISABLED", "true");
-    #else
-    setenv("AWS_EC2_METADATA_DISABLED", "true", 1);
-    #endif
-
     LogTestController::getInstance().setDebug<TestPlan>();
     LogTestController::getInstance().setDebug<minifi::core::Processor>();
     LogTestController::getInstance().setTrace<minifi::core::ProcessSession>();
-    LogTestController::getInstance().setDebug<processors::LogAttribute>();
+    LogTestController::getInstance().setDebug<minifi::processors::LogAttribute>();
     LogTestController::getInstance().setTrace<T>();
     LogTestController::getInstance().setDebug<minifi::aws::AWSCredentialsProvider>();
 
@@ -73,7 +65,7 @@ class S3TestsFixture {
 
   template<typename Component>
   void setCredentialFile(const Component &component) {
-    auto temp_path = createTempDir(&test_controller);
+    auto temp_path = test_controller.createTempDirectory();
     REQUIRE(!temp_path.empty());
     std::string aws_credentials_file(temp_path + utils::file::FileUtils::get_separator() + "aws_creds.conf");
     std::ofstream aws_credentials_file_stream(aws_credentials_file);
@@ -135,16 +127,16 @@ class FlowProcessorS3TestsFixture : public S3TestsFixture<T> {
   const std::string INPUT_DATA = "input_data";
 
   FlowProcessorS3TestsFixture() {
-    LogTestController::getInstance().setTrace<processors::GetFile>();
-    LogTestController::getInstance().setDebug<processors::UpdateAttribute>();
+    LogTestController::getInstance().setTrace<minifi::processors::GetFile>();
+    LogTestController::getInstance().setDebug<minifi::processors::UpdateAttribute>();
 
-    auto input_dir = createTempDir(&this->test_controller);
+    auto input_dir = this->test_controller.createTempDirectory();
     std::ofstream input_file_stream(input_dir + utils::file::FileUtils::get_separator() + INPUT_FILENAME);
     input_file_stream << INPUT_DATA;
     input_file_stream.close();
     auto get_file = this->plan->addProcessor("GetFile", "GetFile");
-    this->plan->setProperty(get_file, processors::GetFile::Directory.getName(), input_dir);
-    this->plan->setProperty(get_file, processors::GetFile::KeepSourceFile.getName(), "false");
+    this->plan->setProperty(get_file, minifi::processors::GetFile::Directory.getName(), input_dir);
+    this->plan->setProperty(get_file, minifi::processors::GetFile::KeepSourceFile.getName(), "false");
     update_attribute = this->plan->addProcessor(
       "UpdateAttribute",
       "UpdateAttribute",
@@ -160,7 +152,7 @@ class FlowProcessorS3TestsFixture : public S3TestsFixture<T> {
       "LogAttribute",
       core::Relationship("success", "d"),
       true);
-    this->plan->setProperty(log_attribute, processors::LogAttribute::FlowFilesToLog.getName(), "0");
+    this->plan->setProperty(log_attribute, minifi::processors::LogAttribute::FlowFilesToLog.getName(), "0");
   }
 
   void setAccesKeyCredentialsInProcessor() override {
@@ -202,7 +194,7 @@ class FlowProducerS3TestsFixture : public S3TestsFixture<T> {
       "LogAttribute",
       core::Relationship("success", "d"),
       true);
-    this->plan->setProperty(log_attribute, processors::LogAttribute::FlowFilesToLog.getName(), "0");
+    this->plan->setProperty(log_attribute, minifi::processors::LogAttribute::FlowFilesToLog.getName(), "0");
   }
 
   void setAccesKeyCredentialsInProcessor() override {
