@@ -25,7 +25,9 @@
 #include <YamlConfiguration.h>
 #include "core/Processor.h"
 #include "TestBase.h"
+#include "Catch.h"
 #include "processors/GenerateFlowFile.h"
+#include "core/Resource.h"
 
 namespace org {
 namespace apache {
