@@ -22,11 +22,16 @@
 #include <memory>
 #include <utility>
 
+#include "FlowController.h"
+#include "unit/ProvenanceTestHelper.h"
+#include "repository/VolatileContentRepository.h"
+#include "CustomProcessors.h"
+
 class TestControllerWithFlow: public TestController{
  public:
   explicit TestControllerWithFlow(const char* yamlConfigContent) {
-    LogTestController::getInstance().setTrace<processors::TestProcessor>();
-    LogTestController::getInstance().setTrace<processors::TestFlowFileGenerator>();
+    LogTestController::getInstance().setTrace<minifi::processors::TestProcessor>();
+    LogTestController::getInstance().setTrace<minifi::processors::TestFlowFileGenerator>();
     LogTestController::getInstance().setTrace<minifi::Connection>();
     LogTestController::getInstance().setTrace<core::Connectable>();
     LogTestController::getInstance().setTrace<minifi::SchedulingAgent>();
@@ -35,8 +40,7 @@ class TestControllerWithFlow: public TestController{
     LogTestController::getInstance().setTrace<minifi::TimerDrivenSchedulingAgent>();
     LogTestController::getInstance().setTrace<minifi::EventDrivenSchedulingAgent>();
 
-    char format[] = "/tmp/flowTest.XXXXXX";
-    std::string dir = createTempDirectory(format);
+    std::string dir = createTempDirectory();
 
     std::string yamlPath = utils::file::FileUtils::concat_path(dir, "config.yml");
     std::ofstream{yamlPath} << yamlConfigContent;
@@ -51,16 +55,18 @@ class TestControllerWithFlow: public TestController{
     REQUIRE(content_repo->initialize(configuration_));
     std::shared_ptr<minifi::io::StreamFactory> stream_factory = minifi::io::StreamFactory::getInstance(configuration_);
 
-    std::unique_ptr<core::FlowConfiguration> flow = utils::make_unique<core::YamlConfiguration>(prov_repo, ff_repo, content_repo, stream_factory, configuration_, yamlPath);
-    root_ = flow->getRoot();
+    auto flow = std::make_unique<core::YamlConfiguration>(prov_repo, ff_repo, content_repo, stream_factory, configuration_, yamlPath);
+    auto root = flow->getRoot();
+    root_ = root.get();
     controller_ = std::make_shared<minifi::FlowController>(
         prov_repo, ff_repo, configuration_,
         std::move(flow),
-        content_repo, DEFAULT_ROOT_GROUP_NAME, true);
+        content_repo, DEFAULT_ROOT_GROUP_NAME,
+        std::make_shared<utils::file::FileSystem>(), []{});
+    controller_->load(std::move(root));
   }
 
   void startFlow() {
-    controller_->load(root_);
     controller_->start();
   }
 
@@ -72,5 +78,5 @@ class TestControllerWithFlow: public TestController{
 
   std::shared_ptr<minifi::Configure> configuration_;
   std::shared_ptr<minifi::FlowController> controller_;
-  std::shared_ptr<core::ProcessGroup> root_;
+  core::ProcessGroup* root_;
 };
