@@ -21,10 +21,10 @@
 #include "IntegrationBase.h"
 #include "core/logging/Logger.h"
 #include "core/Scheduling.h"
-#include "core/state/UpdateController.h"
 #include "core/state/ProcessorController.h"
 #include "../TestBase.h"
-#include "../KamikazeProcessor.h"
+#include "../Catch.h"
+#include "../../../extensions/test-processors/KamikazeProcessor.h"
 #include "utils/StringUtils.h"
 #include "utils/IntegrationTestUtils.h"
 
@@ -34,7 +34,7 @@ class KamikazeErrorHandlingTests : public IntegrationBase {
  public:
   void runAssertions() override {
     using org::apache::nifi::minifi::utils::verifyEventHappenedInPollTime;
-    assert(verifyEventHappenedInPollTime(std::chrono::milliseconds(wait_time_), [&] {
+    assert(verifyEventHappenedInPollTime(wait_time_, [&] {
       const std::string logs = LogTestController::getInstance().log_output.str();
       const auto result = utils::StringUtils::countOccurrences(logs, minifi::processors::KamikazeProcessor::OnScheduleExceptionStr);
       const int occurrences = result.second;
@@ -75,22 +75,32 @@ class KamikazeErrorHandlingTests : public IntegrationBase {
 /*Verify that event driven processors without incoming connections are not scheduled*/
 class EventDriverScheduleErrorHandlingTests: public IntegrationBase {
  public:
-  void updateProperties(std::shared_ptr<minifi::FlowController> fc) override {
-    auto controller_vec = fc->getAllComponents();
+  void updateProperties(minifi::FlowController& fc) override {
     /* This tests depends on a configuration that contains only one KamikazeProcessor named kamikaze
      * (See testOnScheduleRetry.yml)
      * In this case there are two components in the flowcontroller: first is the controller itself,
      * second is the processor that the test uses.
      * Added here some assertions to make it clear. In case any of these fail without changing the corresponding yml file,
      * that most probably means a breaking change. */
-    assert(controller_vec.size() == 2);
-    assert(controller_vec[0]->getComponentName() == "FlowController");
-    assert(controller_vec[1]->getComponentName() == "kamikaze");
+    size_t controllerVecIdx = 0;
 
-    auto process_controller = dynamic_cast<org::apache::nifi::minifi::state::ProcessorController*>(controller_vec[1].get());
-    assert(process_controller != nullptr);
+    fc.executeOnAllComponents([&controllerVecIdx](org::apache::nifi::minifi::state::StateController& component){
+      if (controllerVecIdx == 0) {
+        assert(component.getComponentName() == "FlowController");
+      } else if (controllerVecIdx == 1) {
+        assert(component.getComponentName() == "kamikaze");
 
-    process_controller->getProcessor()->setSchedulingStrategy(org::apache::nifi::minifi::core::SchedulingStrategy::EVENT_DRIVEN);
+        auto process_controller = dynamic_cast<org::apache::nifi::minifi::state::ProcessorController*>(&component);
+        assert(process_controller != nullptr);
+
+        process_controller->getProcessor()->setSchedulingStrategy(org::apache::nifi::minifi::core::SchedulingStrategy::EVENT_DRIVEN);
+      }
+
+      ++controllerVecIdx;
+    });
+
+    // check controller vector size
+    assert(controllerVecIdx == 2);
   }
 
   void runAssertions() override {
