@@ -48,8 +48,7 @@
 class PcapTestHarness : public IntegrationBase {
  public:
   PcapTestHarness() {
-    char format[] = "/tmp/ssth.XXXXXX";
-    dir = testController.createTempDirectory(format);
+    dir = testController.createTempDirectory();
   }
 
   void testSetup() override {
@@ -78,16 +77,15 @@ class PcapTestHarness : public IntegrationBase {
         "because it matches .*"));
   }
 
-  void updateProperties(std::shared_ptr<minifi::FlowController> fc) override {
-    auto components = fc->getComponents("pcap");
-    for (const auto& component : components) {
-      auto proccontroller = std::dynamic_pointer_cast<minifi::state::ProcessorController>(component);
+  void updateProperties(minifi::FlowController& fc) override {
+    fc.executeOnComponent("pcap", [this] (minifi::state::StateController& component) {
+      auto proccontroller = dynamic_cast<minifi::state::ProcessorController*>(&component);
       if (proccontroller) {
         auto processor = proccontroller->getProcessor();
         processor->setProperty(minifi::processors::CapturePacket::BaseDir.getName(), dir);
         processor->setProperty(minifi::processors::CapturePacket::NetworkControllers.getName(), ".*");
       }
-    }
+    });
   }
 
  protected:
