@@ -30,8 +30,8 @@
 #include "properties/Configure.h"
 #include "../unit/ProvenanceTestHelper.h"
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "../../extensions/libarchive/MergeContent.h"
-#include "../test/BufferReader.h"
 #include "core/repository/VolatileFlowFileRepository.h"
 #include "../../extensions/rocksdb-repos/DatabaseContentRepository.h"
 #include "utils/gsl.h"
@@ -50,53 +50,56 @@ const std::string PERSISTENCETEST_FLOWFILE_CHECKPOINT_DIR = "./persistencetest_f
 
 struct TestFlow{
   TestFlow(const std::shared_ptr<core::Repository>& ff_repository, const std::shared_ptr<core::ContentRepository>& content_repo, const std::shared_ptr<core::Repository>& prov_repo,
-        const std::function<std::shared_ptr<core::Processor>(utils::Identifier&)>& processorGenerator, const core::Relationship& relationshipToOutput)
+        const std::function<std::unique_ptr<core::Processor>(utils::Identifier&)>& processorGenerator, const core::Relationship& relationshipToOutput)
       : ff_repository(ff_repository), content_repo(content_repo), prov_repo(prov_repo) {
     // setup processor
+    auto processor = processorGenerator(mainProcUUID());
     {
-      processor = processorGenerator(mainProcUUID());
-      std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(processor);
+      processor_ = processor.get();
+      auto node = std::make_shared<core::ProcessorNode>(processor_);
       processorContext = std::make_shared<core::ProcessContext>(node, nullptr, prov_repo, ff_repository, content_repo);
     }
 
     // setup INPUT processor
     {
       inputProcessor = std::make_shared<core::Processor>("source", inputProcUUID());
-      std::shared_ptr<core::ProcessorNode> node = std::make_shared<core::ProcessorNode>(inputProcessor);
+      auto node = std::make_shared<core::ProcessorNode>(inputProcessor.get());
       inputContext = std::make_shared<core::ProcessContext>(node, nullptr, prov_repo,
                                                             ff_repository, content_repo);
     }
 
     // setup Input Connection
+    auto input = std::make_unique<Connection>(ff_repository, content_repo, "Input", inputConnUUID());
     {
-      input = std::make_shared<Connection>(ff_repository, content_repo, "Input", inputConnUUID());
+      input_ = input.get();
       input->setRelationship({"input", "d"});
       input->setDestinationUUID(mainProcUUID());
       input->setSourceUUID(inputProcUUID());
-      inputProcessor->addConnection(input);
+      inputProcessor->addConnection(input.get());
     }
 
     // setup Output Connection
+    auto output = std::make_unique<Connection>(ff_repository, content_repo, "Output", outputConnUUID());
     {
-      output = std::make_shared<Connection>(ff_repository, content_repo, "Output", outputConnUUID());
+      output_ = output.get();
       output->setRelationship(relationshipToOutput);
       output->setSourceUUID(mainProcUUID());
     }
 
     // setup ProcessGroup
     {
-      root = std::make_shared<core::ProcessGroup>(core::ProcessGroupType::ROOT_PROCESS_GROUP, "root");
-      root->addProcessor(processor);
-      root->addConnection(input);
-      root->addConnection(output);
+      root_ = std::make_unique<core::ProcessGroup>(core::ProcessGroupType::ROOT_PROCESS_GROUP, "root");
+      root_->addProcessor(std::move(processor));
+      root_->addConnection(std::move(input));
+      root_->addConnection(std::move(output));
     }
 
     // prepare Merge Processor for execution
-    processor->setScheduledState(core::ScheduledState::RUNNING);
-    processor->onSchedule(processorContext.get(), new core::ProcessSessionFactory(processorContext));
+    processor_->setScheduledState(core::ScheduledState::RUNNING);
+    processor_->onSchedule(processorContext.get(), new core::ProcessSessionFactory(processorContext));
   }
   std::shared_ptr<core::FlowFile> write(const std::string& data) {
-    minifi::io::BufferStream stream(reinterpret_cast<const uint8_t*>(data.c_str()), gsl::narrow<int>(data.length()));
+    minifi::io::BufferStream stream(data);
     core::ProcessSession sessionGenFlowFile(inputContext);
     std::shared_ptr<core::FlowFile> flow = std::static_pointer_cast<core::FlowFile>(sessionGenFlowFile.create());
     sessionGenFlowFile.importFrom(stream, flow);
@@ -106,21 +109,17 @@ struct TestFlow{
     return flow;
   }
   std::string read(const std::shared_ptr<core::FlowFile>& file) {
-    core::ProcessSession session(processorContext);
-    std::vector<uint8_t> buffer;
-    BufferReader reader(buffer);
-    session.read(file, &reader);
-    return {buffer.data(), buffer.data() + buffer.size()};
+    return to_string(core::ProcessSession{processorContext}.readBuffer(file));
   }
   void trigger() {
     auto session = std::make_shared<core::ProcessSession>(processorContext);
-    processor->onTrigger(processorContext, session);
+    processor_->onTrigger(processorContext, session);
     session->commit();
   }
 
-  std::shared_ptr<Connection> input;
-  std::shared_ptr<Connection> output;
-  std::shared_ptr<core::ProcessGroup> root;
+  Connection* input_;
+  Connection* output_;
+  std::unique_ptr<core::ProcessGroup> root_;
 
  private:
   static utils::Identifier& mainProcUUID() {static auto id = utils::IdGenerator::getIdGenerator()->generate(); return id;}
@@ -129,7 +128,7 @@ struct TestFlow{
   static utils::Identifier& outputConnUUID() {static auto id = utils::IdGenerator::getIdGenerator()->generate(); return id;}
 
   std::shared_ptr<core::Processor> inputProcessor;
-  std::shared_ptr<core::Processor> processor;
+  core::Processor* processor_;
   std::shared_ptr<core::Repository> ff_repository;
   std::shared_ptr<core::ContentRepository> content_repo;
   std::shared_ptr<core::Repository> prov_repo;
@@ -137,8 +136,8 @@ struct TestFlow{
   std::shared_ptr<core::ProcessContext> processorContext;
 };
 
-std::shared_ptr<MergeContent> setupMergeProcessor(const utils::Identifier& id) {
-  auto processor = std::make_shared<MergeContent>("MergeContent", id);
+std::unique_ptr<MergeContent> setupMergeProcessor(const utils::Identifier& id) {
+  auto processor = std::make_unique<MergeContent>("MergeContent", id);
   processor->initialize();
   processor->setAutoTerminatedRelationships({{"original", "d"}});
 
@@ -162,8 +161,7 @@ TEST_CASE("Processors Can Store FlowFiles", "[TestP1]") {
   LogTestController::getInstance().setTrace<minifi::FlowFileRecord>();
   LogTestController::getInstance().setTrace<core::repository::FlowFileRepository>();
 
-  char format[] = "/var/tmp/test.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   auto config = std::make_shared<minifi::Configure>();
   config->set(minifi::Configure::nifi_dbcontent_repository_directory_default, utils::file::FileUtils::concat_path(dir, "content_repository"));
@@ -175,14 +173,14 @@ TEST_CASE("Processors Can Store FlowFiles", "[TestP1]") {
   ff_repository->initialize(config);
   content_repo->initialize(config);
 
-  auto flowConfig = std::unique_ptr<core::FlowConfiguration>{new core::FlowConfiguration(prov_repo, ff_repository, content_repo, nullptr, config, "")};
+  auto flowConfig = std::make_unique<core::FlowConfiguration>(prov_repo, ff_repository, content_repo, nullptr, config, "");
   auto flowController = std::make_shared<minifi::FlowController>(
-      prov_repo, ff_repository, config, std::move(flowConfig), content_repo, "", true);
+      prov_repo, ff_repository, config, std::move(flowConfig), content_repo, "", std::make_shared<utils::file::FileSystem>(), []{});
 
   {
     TestFlow flow(ff_repository, content_repo, prov_repo, setupMergeProcessor, MergeContent::Merge);
 
-    flowController->load(flow.root);
+    flowController->load(std::move(flow.root_));
     ff_repository->start();
 
     // write two files into the input
@@ -197,11 +195,11 @@ TEST_CASE("Processors Can Store FlowFiles", "[TestP1]") {
 
     // check if the processor has taken ownership
     std::set<std::shared_ptr<core::FlowFile>> expired;
-    auto file = flow.input->poll(expired);
+    auto file = flow.input_->poll(expired);
     REQUIRE(!file);
     REQUIRE(expired.empty());
 
-    file = flow.output->poll(expired);
+    file = flow.output_->poll(expired);
     REQUIRE(!file);
     REQUIRE(expired.empty());
   }
@@ -210,7 +208,7 @@ TEST_CASE("Processors Can Store FlowFiles", "[TestP1]") {
   {
     TestFlow flow(ff_repository, content_repo, prov_repo, setupMergeProcessor, MergeContent::Merge);
 
-    flowController->load(flow.root);
+    flowController->load(std::move(flow.root_));
     ff_repository->start();
     // wait for FlowFileRepository to start and notify the owners of
     // the resurrected FlowFiles
@@ -226,18 +224,15 @@ TEST_CASE("Processors Can Store FlowFiles", "[TestP1]") {
     std::shared_ptr<org::apache::nifi::minifi::core::FlowFile> file = nullptr;
     std::set<std::shared_ptr<core::FlowFile>> expired;
     const auto flowFileArrivedInOutput = [&file, &expired, &flow] {
-      file = flow.output->poll(expired);
+      file = flow.output_->poll(expired);
       return file != nullptr;
     };
     assert(verifyEventHappenedInPollTime(std::chrono::seconds(1), flowFileArrivedInOutput, std::chrono::milliseconds(50)));
     REQUIRE(expired.empty());
 
     auto content = flow.read(file);
-    auto isOneOfPossibleResults =
-        Catch::Equals("_Header_one_Demarcator_two_Demarcator_three_Footer_")
-        || Catch::Equals("_Header_two_Demarcator_one_Demarcator_three_Footer_");
-
-    REQUIRE_THAT(content, isOneOfPossibleResults);
+    // See important note about matchers at: https://github.com/catchorg/Catch2/blob/e8cdfdca87ebacd993befdd08ea6aa7e8068ef3d/docs/matchers.md#using-matchers
+    REQUIRE_THAT(content, Catch::Equals("_Header_one_Demarcator_two_Demarcator_three_Footer_") || Catch::Equals("_Header_two_Demarcator_one_Demarcator_three_Footer_"));
   }
 }
 
@@ -247,14 +242,14 @@ class ContentUpdaterProcessor : public core::Processor{
   void onTrigger(core::ProcessContext* /*context*/, core::ProcessSession *session) override {
     auto ff = session->get();
     std::string data = "<override>";
-    minifi::io::BufferStream stream(reinterpret_cast<const uint8_t*>(data.c_str()), gsl::narrow<int>(data.length()));
+    minifi::io::BufferStream stream(data);
     session->importFrom(stream, ff);
     session->transfer(ff, {"success", "d"});
   }
 };
 
-std::shared_ptr<core::Processor> setupContentUpdaterProcessor(const utils::Identifier& id) {
-  return std::make_shared<ContentUpdaterProcessor>("Updater", id);
+std::unique_ptr<core::Processor> setupContentUpdaterProcessor(const utils::Identifier& id) {
+  return std::make_unique<ContentUpdaterProcessor>("Updater", id);
 }
 
 TEST_CASE("Persisted flowFiles are updated on modification", "[TestP1]") {
@@ -268,8 +263,7 @@ TEST_CASE("Persisted flowFiles are updated on modification", "[TestP1]") {
   LogTestController::getInstance().setTrace<core::repository::VolatileRepository<minifi::ResourceClaim::Path>>();
   LogTestController::getInstance().setTrace<core::repository::DatabaseContentRepository>();
 
-  char format[] = "/var/tmp/test.XXXXXX";
-  auto dir = testController.createTempDirectory(format);
+  auto dir = testController.createTempDirectory();
 
   auto config = std::make_shared<minifi::Configure>();
   config->set(minifi::Configure::nifi_dbcontent_repository_directory_default, utils::file::FileUtils::concat_path(dir, "content_repository"));
@@ -293,14 +287,14 @@ TEST_CASE("Persisted flowFiles are updated on modification", "[TestP1]") {
   ff_repository->initialize(config);
   content_repo->initialize(config);
 
-  auto flowConfig = std::unique_ptr<core::FlowConfiguration>{new core::FlowConfiguration(prov_repo, ff_repository, content_repo, nullptr, config, "")};
+  auto flowConfig = std::make_unique<core::FlowConfiguration>(prov_repo, ff_repository, content_repo, nullptr, config, "");
   auto flowController = std::make_shared<minifi::FlowController>(
-      prov_repo, ff_repository, config, std::move(flowConfig), content_repo, "", true);
+      prov_repo, ff_repository, config, std::move(flowConfig), content_repo, "", std::make_shared<utils::file::FileSystem>(), []{});
 
   {
     TestFlow flow(ff_repository, content_repo, prov_repo, setupContentUpdaterProcessor, {"success", "d"});
 
-    flowController->load(flow.root);
+    flowController->load(std::move(flow.root_));
     ff_repository->start();
 
     std::string removedResource;
@@ -336,7 +330,7 @@ TEST_CASE("Persisted flowFiles are updated on modification", "[TestP1]") {
   {
     TestFlow flow(ff_repository, content_repo, prov_repo, setupContentUpdaterProcessor, {"success", "d"});
 
-    flowController->load(flow.root);
+    flowController->load(std::move(flow.root_));
     ff_repository->start();
     // wait for FlowFileRepository to start and notify the owners of
     // the resurrected FlowFiles
@@ -345,7 +339,7 @@ TEST_CASE("Persisted flowFiles are updated on modification", "[TestP1]") {
     std::shared_ptr<org::apache::nifi::minifi::core::FlowFile> file = nullptr;
     using org::apache::nifi::minifi::utils::verifyEventHappenedInPollTime;
     const auto flowFileArrivedInOutput = [&file, &expired, &flow] {
-      file = flow.output->poll(expired);
+      file = flow.output_->poll(expired);
       return file != nullptr;
     };
     assert(verifyEventHappenedInPollTime(std::chrono::seconds(1), flowFileArrivedInOutput, std::chrono::milliseconds(50)));
