@@ -16,6 +16,7 @@
  * limitations under the License.
  */
 
+#include <array>
 #include <memory>
 #include <string>
 
@@ -25,14 +26,14 @@
 #include "DatabaseContentRepository.h"
 #include "FlowFileRecord.h"
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "utils/gsl.h"
 
 template<typename ContentRepositoryClass>
 class ContentSessionController : public TestController {
  public:
   ContentSessionController() {
-    char format[] = "/var/tmp/content_repo.XXXXXX";
-    std::string contentRepoPath = createTempDirectory(format);
+    std::string contentRepoPath = createTempDirectory();
     auto config = std::make_shared<minifi::Configure>();
     config->set(minifi::Configure::nifi_dbcontent_repository_directory_default, contentRepoPath);
     contentRepository = std::make_shared<ContentRepositoryClass>();
@@ -47,21 +48,18 @@ class ContentSessionController : public TestController {
 };
 
 const std::shared_ptr<minifi::io::BaseStream>& operator<<(const std::shared_ptr<minifi::io::BaseStream>& stream, const std::string& str) {
-  int length = gsl::narrow<int>(str.length());
-  REQUIRE(stream->write(const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(str.data())), length) == length);
+  REQUIRE(stream->write(reinterpret_cast<const uint8_t*>(str.data()), str.length()) == str.length());
   return stream;
 }
 
 const std::shared_ptr<minifi::io::BaseStream>& operator>>(const std::shared_ptr<minifi::io::BaseStream>& stream, std::string& str) {
   str = "";
-  uint8_t buffer[4096]{};
+  std::array<std::byte, 4096> buffer{};
   while (true) {
-    auto ret = stream->read(buffer, sizeof(buffer));
-    REQUIRE(ret >= 0);
-    if (ret == 0) {
-      break;
-    }
-    str += std::string{reinterpret_cast<char*>(buffer), static_cast<std::size_t>(ret)};
+    const auto ret = stream->read(buffer);
+    REQUIRE_FALSE(minifi::io::isError(ret));
+    if (ret == 0) { break; }
+    str += std::string{reinterpret_cast<char*>(buffer.data()), ret};
   }
   return stream;
 }
