@@ -21,17 +21,11 @@
 #include <type_traits>
 
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "utils/GeneralUtils.h"
 
 namespace utils = org::apache::nifi::minifi::utils;
 
-static_assert(std::is_same<decltype(utils::make_unique<char16_t>()), std::unique_ptr<char16_t>>::value, "utils::make_unique type must be correct");
-
-TEST_CASE("GeneralUtils::make_unique", "[make_unique]") {
-  const auto pstr = utils::make_unique<std::string>("test string");
-  REQUIRE("test string" == *pstr);
-}
-
 // intdiv_ceil
 static_assert(0 == utils::intdiv_ceil(0, 1), "");
 static_assert(0 == utils::intdiv_ceil(0, 2), "");
@@ -64,80 +58,22 @@ struct does_compile<N, D,
 static_assert(does_compile<2, 3>::value, "does_compile should work");
 static_assert(!does_compile<1, 0>::value, "constexpr division by zero shouldn't compile");
 
-
-TEST_CASE("GeneralUtils::exchange", "[exchange]") {
-  int a = 1;
-  int b = 2;
-  a = utils::exchange(b, 0);
-  REQUIRE(2 == a);
-  REQUIRE(0 == b);
-}
-
-static_assert(std::is_same<decltype(utils::void_t<char16_t>()), void>::value, "utils::void_t single arg must work");
-static_assert(std::is_same<decltype(utils::void_t<int, double, bool, void, char16_t>()), void>::value, "utils::void_t multi arg must work");
-
-TEST_CASE("GeneralUtils::invoke pointer to member function", "[invoke memfnptr]") {
-  const int result{0xc1ca};
-  struct Tester {
-    bool called{};
-    int memfn(const int arg) {
-      REQUIRE(42 == arg);
-      called = true;
-      return result;
-    }
-  };
-
-  // normal
-  REQUIRE(result == utils::invoke(&Tester::memfn, Tester{}, 42));
-
-  // reference_wrapper
-  Tester t2;
-  const auto ref_wrapper = std::ref(t2);
-  REQUIRE(result == utils::invoke(&Tester::memfn, ref_wrapper, 42));
-  REQUIRE(t2.called);
-
-  // pointer
-  Tester t3;
-  REQUIRE(result == utils::invoke(&Tester::memfn, &t3, 42));
-  REQUIRE(t3.called);
-}
-
-TEST_CASE("GeneralUtils::invoke pointer to data member", "[invoke data member]") {
-  struct Times2 {
-    int value;
-    explicit Times2(const int i) :value{i * 2} {}
-  };
-
-  // normal
-  REQUIRE(24 == utils::invoke(&Times2::value, Times2{12}));
-
-  // reference_wrapper
-  Times2 t2{42};
-  const auto ref_wrapper = std::ref(t2);
-  REQUIRE(84 == utils::invoke(&Times2::value, ref_wrapper));
-
-  // pointer
-  Times2 t3{0};
-  REQUIRE(0 == utils::invoke(&Times2::value, &t3));
-}
-
-namespace {
-bool free_function(const bool b) { return b; }
-}  // namespace
-
-TEST_CASE("GeneralUtils::invoke FunctionObject", "[invoke function object]") {
-  REQUIRE(true == utils::invoke(&free_function, true));
-  REQUIRE(false == utils::invoke(&free_function, false));
-
-  const auto int_timesn = [](const int i) { return 3 * i; };
-
-  // invoking lambda
-  REQUIRE(60 == utils::invoke(int_timesn, 20));
-}
-
 TEST_CASE("GeneralUtils::dereference", "[dereference]") {
   const int a = 42;
-  const auto* const pa = &a;
+  const auto pa = gsl::make_not_null(&a);
   REQUIRE(42 == utils::dereference(pa));
   REQUIRE(&a == &utils::dereference(pa));
+
+  const auto uniq_a = gsl::make_not_null(std::make_unique<int>(99));
+  REQUIRE(99 == utils::dereference(uniq_a));
+}
+
+TEST_CASE("GeneralUtils::unsafe_dereference", "[unsafe_dereference]") {
+  const int a = 42;
+  const int* const pa = &a;
+  REQUIRE(42 == utils::unsafe_dereference(pa));
+  REQUIRE(&a == &utils::unsafe_dereference(pa));
+
+  const auto uniq_a = std::make_unique<int>(99);
+  REQUIRE(99 == utils::unsafe_dereference(uniq_a));
 }
