@@ -17,6 +17,7 @@
  */
 
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "core/ConfigurableComponent.h"
 #include "utils/PropertyErrors.h"
 #include "core/PropertyValidation.h"
@@ -34,7 +35,7 @@ using namespace utils::internal;
  */
 TEST_CASE("Some default values get coerced to typed variants") {
   auto prop = Property("prop", "d", "true");
-  REQUIRE_THROWS_AS(prop.setValue("banana"), ConversionException&);
+  REQUIRE_THROWS_AS(prop.setValue("banana"), ConversionException);
 
   const std::string SPACE = " ";
   auto prop2 = Property("prop", "d", SPACE + "true");
@@ -45,16 +46,16 @@ TEST_CASE("Converting invalid PropertyValue") {
   auto prop = PropertyBuilder::createProperty("prop")
       ->withDefaultValue<int>(0)
       ->build();
-  REQUIRE_THROWS_AS(prop.setValue("not int"), ParseException&);
+  REQUIRE_THROWS_AS(prop.setValue("not int"), ParseException);
   auto cast_check = [&]{ return static_cast<int>(prop.getValue()) == 0; };  // To avoid unused-value warning
-  REQUIRE_THROWS_AS(cast_check(), InvalidValueException&);
+  REQUIRE_THROWS_AS(cast_check(), InvalidValueException);
 }
 
 TEST_CASE("Parsing int has baggage after") {
   auto prop = PropertyBuilder::createProperty("prop")
       ->withDefaultValue<int>(0)
       ->build();
-  REQUIRE_THROWS_AS(prop.setValue("55almost int"), ParseException&);
+  REQUIRE_THROWS_AS(prop.setValue("55almost int"), ParseException);
 }
 
 TEST_CASE("Parsing int has spaces") {
@@ -69,14 +70,14 @@ TEST_CASE("Parsing int out of range") {
   auto prop = PropertyBuilder::createProperty("prop")
       ->withDefaultValue<int>(0)
       ->build();
-  REQUIRE_THROWS_AS(prop.setValue("  5000000000  "), ParseException&);
+  REQUIRE_THROWS_AS(prop.setValue("  5000000000  "), ParseException);
 }
 
 TEST_CASE("Parsing bool has baggage after") {
   auto prop = PropertyBuilder::createProperty("prop")
       ->withDefaultValue<bool>(true)
       ->build();
-  REQUIRE_THROWS_AS(prop.setValue("false almost bool"), ParseException&);
+  REQUIRE_THROWS_AS(prop.setValue("false almost bool"), ParseException);
 }
 
 class TestConfigurableComponent : public ConfigurableComponent {
@@ -131,7 +132,7 @@ TEST_CASE("Missing Required Without Default") {
   TestConfigurableComponent component;
   component.setSupportedProperties({prop});
   std::string value;
-  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), RequiredPropertyMissingException&);
+  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), RequiredPropertyMissingException);
 }
 
 TEST_CASE("Missing Optional Without Default") {
@@ -163,9 +164,9 @@ TEST_CASE("Invalid With Default") {
       ->build();
   TestConfigurableComponent component;
   component.setSupportedProperties({prop});
-  REQUIRE_THROWS_AS(component.setProperty("prop", "banana"), ParseException&);
+  REQUIRE_THROWS_AS(component.setProperty("prop", "banana"), ParseException);
   std::string value;
-  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), InvalidValueException&);
+  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), InvalidValueException);
 }
 
 TEST_CASE("Valid With Default") {
@@ -187,7 +188,7 @@ TEST_CASE("Invalid conversion") {
   TestConfigurableComponent component;
   component.setSupportedProperties({prop});
   bool value;
-  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), ConversionException&);
+  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), value), ConversionException);
 }
 
 TEST_CASE("Write Invalid Then Override With Valid") {
@@ -197,7 +198,7 @@ TEST_CASE("Write Invalid Then Override With Valid") {
       ->build();
   TestConfigurableComponent component;
   component.setSupportedProperties({prop});
-  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "banana"), ConversionException&);
+  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "banana"), ConversionException);
   component.setProperty(prop.getName(), "98");
   int value;
   REQUIRE(component.getProperty(prop.getName(), value));
@@ -214,7 +215,7 @@ TEST_CASE("Property Change notification gets called even on erroneous assignment
   component.setPropertyModifiedCallback([&](const Property &, const Property &) {
     ++callbackCount;
   });
-  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "banana"), ConversionException&);
+  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "banana"), ConversionException);
   REQUIRE(callbackCount == 1);
 }
 
@@ -228,10 +229,37 @@ TEST_CASE("Correctly Typed Property With Invalid Validation") {
   component.setPropertyModifiedCallback([&](const Property &, const Property &) {
     ++callbackCount;
   });
-  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "20"), InvalidValueException&);
+  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "20"), InvalidValueException);
   REQUIRE(callbackCount == 1);
 }
 
+TEST_CASE("TimePeriodValue Property") {
+  using namespace std::literals::chrono_literals;
+  auto prop = PropertyBuilder::createProperty("prop")
+      ->withDefaultValue<TimePeriodValue>("10 minutes")
+      ->build();
+  TestConfigurableComponent component;
+  component.setSupportedProperties({prop});
+  TimePeriodValue time_period_value;
+  REQUIRE(component.getProperty(prop.getName(), time_period_value));
+  CHECK(time_period_value.getMilliseconds() == 10min);
+  REQUIRE_THROWS_AS(component.setProperty(prop.getName(), "20"), ParseException);
+}
+
+TEST_CASE("TimePeriodValue Property without validator") {
+  using namespace std::literals::chrono_literals;
+  auto prop = PropertyBuilder::createProperty("prop")
+      ->withDefaultValue("60 minutes")
+      ->build();
+  TestConfigurableComponent component;
+  component.setSupportedProperties({prop});
+  TimePeriodValue time_period_value;
+  REQUIRE(component.getProperty(prop.getName(), time_period_value));
+  CHECK(time_period_value.getMilliseconds() == 1h);
+  REQUIRE_NOTHROW(component.setProperty(prop.getName(), "20"));
+  REQUIRE_THROWS_AS(component.getProperty(prop.getName(), time_period_value), ValueException);
+}
+
 } /* namespace core */
 } /* namespace minifi */
 } /* namespace nifi */
