@@ -42,17 +42,23 @@
 #pragma GCC diagnostic ignored "-Woverloaded-virtual"
 #endif
 
+using namespace std::literals::chrono_literals;
+
 /**
  * Test repository
  */
-class TestRepository : public core::Repository {
+class TestRepository : public org::apache::nifi::minifi::core::Repository {
  public:
   TestRepository()
-      : core::SerializableComponent("repo_name"),
-        Repository("repo_name", "./dir", 1000, 100, 0) {
+      : org::apache::nifi::minifi::core::SerializableComponent("repo_name"),
+        Repository("repo_name", "./dir", 1s, 100, 0ms) {
   }
 
-  bool initialize(const std::shared_ptr<minifi::Configure> &) override {
+  ~TestRepository() override {
+    stop();
+  }
+
+  bool initialize(const std::shared_ptr<org::apache::nifi::minifi::Configure> &) override {
     return true;
   }
 
@@ -60,16 +66,10 @@ class TestRepository : public core::Repository {
     running_ = true;
   }
 
-  void stop() override {
-    running_ = false;
-  }
-
   void setFull() {
     repo_full_ = true;
   }
 
-  ~TestRepository() override = default;
-
   bool isNoop() override {
     return false;
   }
@@ -80,9 +80,10 @@ class TestRepository : public core::Repository {
     return true;
   }
 
-  bool MultiPut(const std::vector<std::pair<std::string, std::unique_ptr<minifi::io::BufferStream>>>& data) override {
+  bool MultiPut(const std::vector<std::pair<std::string, std::unique_ptr<org::apache::nifi::minifi::io::BufferStream>>>& data) override {
     for (const auto& item : data) {
-      if (!Put(item.first, item.second->getBuffer(), item.second->size())) {
+      const auto buf = item.second->getBuffer().as_span<const uint8_t>();
+      if (!Put(item.first, buf.data(), buf.size())) {
         return false;
       }
     }
@@ -110,36 +111,36 @@ class TestRepository : public core::Repository {
     }
   }
 
-  bool Serialize(std::vector<std::shared_ptr<core::SerializableComponent>>& /*store*/, size_t /*max_size*/) override {
+  bool Serialize(std::vector<std::shared_ptr<org::apache::nifi::minifi::core::SerializableComponent>>& /*store*/, size_t /*max_size*/) override {
     return false;
   }
 
-  bool DeSerialize(std::vector<std::shared_ptr<core::SerializableComponent>> &store, size_t &max_size) override {
+  bool DeSerialize(std::vector<std::shared_ptr<org::apache::nifi::minifi::core::SerializableComponent>> &store, size_t &max_size) override {
     std::lock_guard<std::mutex> lock{repository_results_mutex_};
     max_size = 0;
     for (const auto &entry : repository_results_) {
       if (max_size >= store.size()) {
         break;
       }
-      std::shared_ptr<core::SerializableComponent> eventRead = store.at(max_size);
-      eventRead->DeSerialize(reinterpret_cast<const uint8_t*>(entry.second.data()), entry.second.length());
+      const auto eventRead = store.at(max_size);
+      eventRead->DeSerialize(gsl::make_span(entry.second).as_span<const std::byte>());
       ++max_size;
     }
     return true;
   }
 
-  bool Serialize(const std::shared_ptr<core::SerializableComponent>& /*store*/) override {
+  bool Serialize(const std::shared_ptr<org::apache::nifi::minifi::core::SerializableComponent>& /*store*/) override {
     return false;
   }
 
-  bool DeSerialize(const std::shared_ptr<core::SerializableComponent> &store) override {
+  bool DeSerialize(const std::shared_ptr<org::apache::nifi::minifi::core::SerializableComponent> &store) override {
     std::string value;
     Get(store->getUUIDStr(), value);
-    store->DeSerialize(reinterpret_cast<const uint8_t*>(value.c_str()), value.size());
+    store->DeSerialize(gsl::make_span(value).as_span<const std::byte>());
     return true;
   }
 
-  bool DeSerialize(const uint8_t* /*buffer*/, const size_t /*bufferSize*/) override {
+  bool DeSerialize(gsl::span<const std::byte>) override {
     return false;
   }
 
@@ -148,14 +149,14 @@ class TestRepository : public core::Repository {
     return repository_results_;
   }
 
-  void getProvenanceRecord(std::vector<std::shared_ptr<provenance::ProvenanceEventRecord>> &records, int maxSize) {
+  void getProvenanceRecord(std::vector<std::shared_ptr<org::apache::nifi::minifi::provenance::ProvenanceEventRecord>> &records, int maxSize) {
     std::lock_guard<std::mutex> lock{repository_results_mutex_};
     for (const auto &entry : repository_results_) {
       if (records.size() >= static_cast<uint64_t>(maxSize))
         break;
-      std::shared_ptr<provenance::ProvenanceEventRecord> eventRead = std::make_shared<provenance::ProvenanceEventRecord>();
+      const auto eventRead = std::make_shared<org::apache::nifi::minifi::provenance::ProvenanceEventRecord>();
 
-      if (eventRead->DeSerialize(reinterpret_cast<const uint8_t*>(entry.second.data()), entry.second.length())) {
+      if (eventRead->DeSerialize(gsl::make_span(entry.second).as_span<const std::byte>())) {
         records.push_back(eventRead);
       }
     }
@@ -170,19 +171,21 @@ class TestRepository : public core::Repository {
   std::map<std::string, std::string> repository_results_;
 };
 
-class TestFlowRepository : public core::Repository {
+class TestFlowRepository : public org::apache::nifi::minifi::core::Repository {
  public:
   TestFlowRepository()
-      : core::SerializableComponent("ff"),
-        core::Repository("ff", "./dir", 1000, 100, 0) {
+      : org::apache::nifi::minifi::core::SerializableComponent("ff"),
+        org::apache::nifi::minifi::core::Repository("ff", "./dir", 1s, 100, 0ms) {
+  }
+
+  ~TestFlowRepository() override {
+    stop();
   }
 
   bool initialize(const std::shared_ptr<org::apache::nifi::minifi::Configure> &) override {
     return true;
   }
 
-  ~TestFlowRepository() override = default;
-
   bool Put(std::string key, const uint8_t *buf, size_t bufLen) override {
     std::lock_guard<std::mutex> lock{repository_results_mutex_};
     repository_results_.emplace(key, std::string{reinterpret_cast<const char*>(buf), bufLen});
@@ -211,20 +214,20 @@ class TestFlowRepository : public core::Repository {
     return repository_results_;
   }
 
-  void getProvenanceRecord(std::vector<std::shared_ptr<provenance::ProvenanceEventRecord>> &records, int maxSize) {
+  void getProvenanceRecord(std::vector<std::shared_ptr<org::apache::nifi::minifi::provenance::ProvenanceEventRecord>> &records, int maxSize) {
     std::lock_guard<std::mutex> lock{repository_results_mutex_};
     for (const auto &entry : repository_results_) {
       if (records.size() >= static_cast<uint64_t>(maxSize))
         break;
-      std::shared_ptr<provenance::ProvenanceEventRecord> eventRead = std::make_shared<provenance::ProvenanceEventRecord>();
+      const auto eventRead = std::make_shared<org::apache::nifi::minifi::provenance::ProvenanceEventRecord>();
 
-      if (eventRead->DeSerialize(reinterpret_cast<const uint8_t*>(entry.second.data()), entry.second.length())) {
+      if (eventRead->DeSerialize(gsl::make_span(entry.second).as_span<const std::byte>())) {
         records.push_back(eventRead);
       }
     }
   }
 
-  void loadComponent(const std::shared_ptr<core::ContentRepository>& /*content_repo*/) override {
+  void loadComponent(const std::shared_ptr<org::apache::nifi::minifi::core::ContentRepository>& /*content_repo*/) override {
   }
 
   void run() override {
@@ -236,15 +239,18 @@ class TestFlowRepository : public core::Repository {
   std::map<std::string, std::string> repository_results_;
 };
 
-class TestFlowController : public minifi::FlowController {
+class TestFlowController : public org::apache::nifi::minifi::FlowController {
  public:
-  TestFlowController(std::shared_ptr<core::Repository> repo, std::shared_ptr<core::Repository> flow_file_repo, std::shared_ptr<core::ContentRepository> /*content_repo*/)
-      : minifi::FlowController(repo, flow_file_repo, std::make_shared<minifi::Configure>(), nullptr, std::make_shared<core::repository::VolatileContentRepository>(), "", true) {
+  TestFlowController(std::shared_ptr<org::apache::nifi::minifi::core::Repository> repo, std::shared_ptr<org::apache::nifi::minifi::core::Repository> flow_file_repo,
+      const std::shared_ptr<org::apache::nifi::minifi::core::ContentRepository>& /*content_repo*/)
+      :org::apache::nifi::minifi::FlowController(repo, flow_file_repo, std::make_shared<org::apache::nifi::minifi::Configure>(), nullptr,
+          std::make_shared<org::apache::nifi::minifi::core::repository::VolatileContentRepository>(), "",
+          std::make_shared<org::apache::nifi::minifi::utils::file::FileSystem>(), []{}) {
   }
 
   ~TestFlowController() override = default;
 
-  void load(const std::shared_ptr<core::ProcessGroup>& /*root*/ = nullptr, bool /*reload*/ = false) override {
+  void load(std::unique_ptr<org::apache::nifi::minifi::core::ProcessGroup> /*root*/ = nullptr, bool /*reload*/ = false) override {
   }
 
   int16_t start() override {
@@ -276,19 +282,19 @@ class TestFlowController : public minifi::FlowController {
     return true;
   }
 
-  std::shared_ptr<core::Processor> createProcessor(const std::string& /*name*/, const utils::Identifier& /*uuid*/) {
+  std::shared_ptr<org::apache::nifi::minifi::core::Processor> createProcessor(const std::string& /*name*/, const org::apache::nifi::minifi::utils::Identifier& /*uuid*/) {
     return 0;
   }
 
-  core::ProcessGroup *createRootProcessGroup(const std::string& /*name*/, const utils::Identifier& /*uuid*/) {
+  org::apache::nifi::minifi::core::ProcessGroup *createRootProcessGroup(const std::string& /*name*/, const org::apache::nifi::minifi::utils::Identifier& /*uuid*/) {
     return 0;
   }
 
-  core::ProcessGroup *createRemoteProcessGroup(const std::string& /*name*/, const utils::Identifier& /*uuid*/) {
+  org::apache::nifi::minifi::core::ProcessGroup *createRemoteProcessGroup(const std::string& /*name*/, const org::apache::nifi::minifi::utils::Identifier& /*uuid*/) {
     return 0;
   }
 
-  std::shared_ptr<minifi::Connection> createConnection(const std::string& /*name*/, const utils::Identifier& /*uuid*/) {
+  std::shared_ptr<org::apache::nifi::minifi::Connection> createConnection(const std::string& /*name*/, const org::apache::nifi::minifi::utils::Identifier& /*uuid*/) {
     return 0;
   }
 };
