@@ -16,19 +16,30 @@
  * limitations under the License.
  */
 
-#include <string>
-#include <list>
-#include <vector>
-#include <cstdlib>
-#include <random>
 #include <algorithm>
 #include <cstdint>
+#include <cstdlib>
+#include <list>
+#include <optional>
+#include <random>
+#include <string>
+#include <vector>
 #include "../TestBase.h"
+#include "../Catch.h"
 #include "core/Core.h"
 #include "utils/StringUtils.h"
 #include "utils/Environment.h"
 
 using org::apache::nifi::minifi::utils::StringUtils;
+using utils::as_string;
+
+TEST_CASE("StringUtils::chomp works correctly", "[StringUtils][chomp]") {
+  using pair_of = std::pair<std::string, std::string>;
+  CHECK(StringUtils::chomp("foobar") == pair_of{"foobar", ""});
+  CHECK(StringUtils::chomp("foobar\n") == pair_of{"foobar", "\n"});
+  CHECK(StringUtils::chomp("foobar\r\n") == pair_of{"foobar", "\r\n"});
+  CHECK(StringUtils::chomp("foo\rbar\n") == pair_of{"foo\rbar", "\n"});
+}
 
 TEST_CASE("TestStringUtils::split", "[test split no delimiter]") {
   std::vector<std::string> expected = { "hello" };
@@ -55,9 +66,71 @@ TEST_CASE("TestStringUtils::split5", "[test split with delimiter set to empty st
   REQUIRE(expected == StringUtils::split("hello world", ""));
 }
 
-TEST_CASE("TestStringUtils::splitTrasformed", "[test split with trim]") {
+TEST_CASE("TestStringUtils::split6", "[test split with delimiter with empty results]") {
+  std::vector<std::string> expected = {""};
+  REQUIRE(expected == StringUtils::split("", ","));
+  expected = {"", ""};
+  REQUIRE(expected == StringUtils::split(",", ","));
+  expected = {"", " ", "", ""};
+  REQUIRE(expected == StringUtils::split(", ,,", ","));
+}
+
+TEST_CASE("TestStringUtils::splitRemovingEmpty", "[test splitRemovingEmpty multiple delimiter]") {
+  std::vector<std::string> expected = { "hello", "world", "I'm", "a", "unit", "test" };
+  REQUIRE(expected == StringUtils::split("hello world I'm a unit test", " "));
+}
+
+TEST_CASE("TestStringUtils::splitRemovingEmpty2", "[test splitRemovingEmpty no delimiter]") {
+  std::vector<std::string> expected = { "hello" };
+  REQUIRE(expected == StringUtils::splitRemovingEmpty("hello", ","));
+}
+
+TEST_CASE("TestStringUtils::splitRemovingEmpty3", "[test splitRemovingEmpty with delimiter with empty results]") {
+  std::vector<std::string> expected = {};
+  REQUIRE(expected == StringUtils::splitRemovingEmpty("", ","));
+  REQUIRE(expected == StringUtils::splitRemovingEmpty(",", ","));
+  expected = {" "};
+  REQUIRE(expected == StringUtils::splitRemovingEmpty(", ,,", ","));
+}
+
+TEST_CASE("TestStringUtils::splitAndTrim", "[test split with trim with characters]") {
   std::vector<std::string> expected{ "hello", "world peace" };
   REQUIRE(expected == StringUtils::splitAndTrim("hello, world peace", ","));
+  expected = {""};
+  REQUIRE(expected == StringUtils::splitAndTrim("", ","));
+  expected = {"", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim(",", ","));
+  expected = {"", "", "", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim(", ,,", ","));
+}
+
+TEST_CASE("StringUtils::splitAndTrim2", "[test split with trim with words]") {
+  std::vector<std::string> expected{ "tom", "jerry" };
+  REQUIRE(expected == StringUtils::splitAndTrim("tom and jerry", "and"));
+  expected = {"", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim("and", "and"));
+  expected = {"", "", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim("andand", "and"));
+  expected = {"stan", "pan", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim("stan and pan and ", "and"));
+  expected = {"", ""};
+  REQUIRE(expected == StringUtils::splitAndTrim(" and ", "and"));
+  expected = {"a", "b", "c"};
+  REQUIRE(expected == StringUtils::splitAndTrim("a and ... b and ...  c", "and ..."));
+}
+
+TEST_CASE("StringUtils::splitAndTrimRemovingEmpty", "[test split with trim removing empty strings]") {
+  std::vector<std::string> expected{ "tom", "jerry" };
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty("tom and jerry", "and"));
+  expected = {};
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty("and", "and"));
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty("andand", "and"));
+  expected = {"stan", "pan"};
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty("stan and pan and ", "and"));
+  expected = {};
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty(" and ", "and"));
+  expected = {"a", "b", "c"};
+  REQUIRE(expected == StringUtils::splitAndTrimRemovingEmpty("a and ... b and ...  c", "and ..."));
 }
 
 TEST_CASE("StringUtils::replaceEnvironmentVariables works correctly", "[replaceEnvironmentVariables]") {
@@ -99,6 +172,7 @@ TEST_CASE("TestStringUtils::testJoin", "[test string join]") {
 }
 
 TEST_CASE("TestStringUtils::trim", "[test trim]") {
+  REQUIRE("" == StringUtils::trim(""));
   REQUIRE("" == StringUtils::trim(" \n\t"));
   REQUIRE("foobar" == StringUtils::trim("foobar"));
   REQUIRE("foo bar" == StringUtils::trim("foo bar"));
@@ -130,9 +204,11 @@ TEST_CASE("TestStringUtils::trim", "[test trim]") {
   REQUIRE("foobar\n\t " == StringUtils::trimLeft(" \n\tfoobar\n\t "));
 }
 
-TEST_CASE("TestStringUtils::startsWith", "[test startsWith]") {
+TEST_CASE("TestStringUtils::startsWith - case sensitive", "[test startsWith]") {
   REQUIRE(StringUtils::startsWith("abcd", ""));
   REQUIRE(StringUtils::startsWith("abcd", "a"));
+  REQUIRE(!StringUtils::startsWith("Abcd", "a"));
+  REQUIRE(!StringUtils::startsWith("abcd", "A"));
   REQUIRE(StringUtils::startsWith("abcd", "abcd"));
   REQUIRE(StringUtils::startsWith("abcd", "abc"));
   REQUIRE(!StringUtils::startsWith("abcd", "abcde"));
@@ -143,9 +219,11 @@ TEST_CASE("TestStringUtils::startsWith", "[test startsWith]") {
   REQUIRE(!StringUtils::startsWith("abcd", "d"));
 }
 
-TEST_CASE("TestStringUtils::endsWith", "[test endsWith]") {
+TEST_CASE("TestStringUtils::endsWith - case sensitive", "[test endsWith]") {
   REQUIRE(StringUtils::endsWith("abcd", ""));
   REQUIRE(StringUtils::endsWith("abcd", "d"));
+  REQUIRE(!StringUtils::endsWith("abcD", "d"));
+  REQUIRE(!StringUtils::endsWith("abcd", "D"));
   REQUIRE(StringUtils::endsWith("abcd", "abcd"));
   REQUIRE(StringUtils::endsWith("abcd", "bcd"));
   REQUIRE(!StringUtils::endsWith("abcd", "1abcd"));
@@ -156,8 +234,38 @@ TEST_CASE("TestStringUtils::endsWith", "[test endsWith]") {
   REQUIRE(!StringUtils::endsWith("abcd", "a"));
 }
 
+TEST_CASE("TestStringUtils::startsWith - case insensitive", "[test startsWith case insensitive]") {
+  REQUIRE(StringUtils::startsWith("abcd", "", false));
+  REQUIRE(StringUtils::startsWith("abcd", "a", false));
+  REQUIRE(StringUtils::startsWith("Abcd", "a", false));
+  REQUIRE(StringUtils::startsWith("abcd", "A", false));
+  REQUIRE(StringUtils::startsWith("aBcd", "abCd", false));
+  REQUIRE(StringUtils::startsWith("abcd", "abc", false));
+  REQUIRE(!StringUtils::startsWith("abcd", "abcde", false));
+
+  REQUIRE(StringUtils::startsWith("", "", false));
+  REQUIRE(!StringUtils::startsWith("", "abcd", false));
+  REQUIRE(!StringUtils::startsWith("abcd", "b", false));
+  REQUIRE(!StringUtils::startsWith("abcd", "d", false));
+}
+
+TEST_CASE("TestStringUtils::endsWith - case insensitive", "[test endsWith case insensitive]") {
+  REQUIRE(StringUtils::endsWith("abcd", "", false));
+  REQUIRE(StringUtils::endsWith("abcd", "d", false));
+  REQUIRE(StringUtils::endsWith("abcd", "D", false));
+  REQUIRE(StringUtils::endsWith("abcD", "d", false));
+  REQUIRE(StringUtils::endsWith("abcd", "abcd", false));
+  REQUIRE(StringUtils::endsWith("aBcd", "bcD", false));
+  REQUIRE(!StringUtils::endsWith("abCd", "1aBcd", false));
+
+  REQUIRE(StringUtils::endsWith("", "", false));
+  REQUIRE(!StringUtils::endsWith("", "abcd", false));
+  REQUIRE(!StringUtils::endsWith("abcd", "c", false));
+  REQUIRE(!StringUtils::endsWith("abcd", "a", false));
+}
+
 TEST_CASE("TestStringUtils::toBool", "[test toBool]") {
-  std::vector<std::pair<std::string, utils::optional<bool>>> cases{
+  std::vector<std::pair<std::string, std::optional<bool>>> cases{
       {"", {}},
       {"true", true},
       {"false", false},
@@ -174,37 +282,39 @@ TEST_CASE("TestStringUtils::testHexEncode", "[test hex encode]") {
   REQUIRE("" == StringUtils::to_hex(""));
   REQUIRE("6f" == StringUtils::to_hex("o"));
   REQUIRE("666f6f626172" == StringUtils::to_hex("foobar"));
-  REQUIRE("000102030405060708090a0b0c0d0e0f" == StringUtils::to_hex(std::vector<uint8_t>{0x00, 0x01, 0x02, 0x03,
-                                                                                         0x04, 0x05, 0x06, 0x07,
-                                                                                         0x08, 0x09, 0x0a, 0x0b,
-                                                                                         0x0c, 0x0d, 0x0e, 0x0f}));
+  REQUIRE("000102030405060708090a0b0c0d0e0f" == StringUtils::to_hex(gsl::make_span(std::vector<uint8_t>{
+      0x00, 0x01, 0x02, 0x03,
+      0x04, 0x05, 0x06, 0x07,
+      0x08, 0x09, 0x0a, 0x0b,
+      0x0c, 0x0d, 0x0e, 0x0f}).as_span<const std::byte>()));
   REQUIRE("6F" == StringUtils::to_hex("o", true /*uppercase*/));
   REQUIRE("666F6F626172" == StringUtils::to_hex("foobar", true /*uppercase*/));
-  REQUIRE("000102030405060708090A0B0C0D0E0F" == StringUtils::to_hex(std::vector<uint8_t>{0x00, 0x01, 0x02, 0x03,
-                                                                                         0x04, 0x05, 0x06, 0x07,
-                                                                                         0x08, 0x09, 0x0a, 0x0b,
-                                                                                         0x0c, 0x0d, 0x0e, 0x0f}, true /*uppercase*/));
+  REQUIRE("000102030405060708090A0B0C0D0E0F" == StringUtils::to_hex(gsl::make_span(std::vector<uint8_t>{
+      0x00, 0x01, 0x02, 0x03,
+      0x04, 0x05, 0x06, 0x07,
+      0x08, 0x09, 0x0a, 0x0b,
+      0x0c, 0x0d, 0x0e, 0x0f}).as_span<const std::byte>(), true /*uppercase*/));
 }
 
 TEST_CASE("TestStringUtils::testHexDecode", "[test hex decode]") {
-  REQUIRE("" == StringUtils::from_hex(""));
-  REQUIRE("o" == StringUtils::from_hex("6f"));
-  REQUIRE("o" == StringUtils::from_hex("6F"));
-  REQUIRE("foobar" == StringUtils::from_hex("666f6f626172"));
-  REQUIRE("foobar" == StringUtils::from_hex("666F6F626172"));
-  REQUIRE("foobar" == StringUtils::from_hex("66:6F:6F:62:61:72"));
-  REQUIRE("foobar" == StringUtils::from_hex("66 6F 6F 62 61 72"));
+  REQUIRE(StringUtils::from_hex("").empty());
+  REQUIRE("o" == StringUtils::from_hex("6f", utils::as_string));
+  REQUIRE("o" == StringUtils::from_hex("6F", utils::as_string));
+  REQUIRE("foobar" == StringUtils::from_hex("666f6f626172", utils::as_string));
+  REQUIRE("foobar" == StringUtils::from_hex("666F6F626172", utils::as_string));
+  REQUIRE("foobar" == StringUtils::from_hex("66:6F:6F:62:61:72", utils::as_string));
+  REQUIRE("foobar" == StringUtils::from_hex("66 6F 6F 62 61 72", utils::as_string));
   REQUIRE(std::string({0x00, 0x01, 0x02, 0x03,
                        0x04, 0x05, 0x06, 0x07,
                        0x08, 0x09, 0x0a, 0x0b,
-                       0x0c, 0x0d, 0x0e, 0x0f}) == StringUtils::from_hex("000102030405060708090a0b0c0d0e0f"));
+                       0x0c, 0x0d, 0x0e, 0x0f}) == StringUtils::from_hex("000102030405060708090a0b0c0d0e0f", utils::as_string));
   REQUIRE(std::string({0x00, 0x01, 0x02, 0x03,
                        0x04, 0x05, 0x06, 0x07,
                        0x08, 0x09, 0x0a, 0x0b,
-                       0x0c, 0x0d, 0x0e, 0x0f}) == StringUtils::from_hex("000102030405060708090A0B0C0D0E0F"));
+                       0x0c, 0x0d, 0x0e, 0x0f}) == StringUtils::from_hex("000102030405060708090A0B0C0D0E0F", utils::as_string));
 
-  REQUIRE_THROWS_WITH(StringUtils::from_hex("666f6f62617"), "Hexencoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_hex("666f6f6261 7"), "Hexencoded string is malformatted");
+  REQUIRE_THROWS_WITH(StringUtils::from_hex("666f6f62617"), "Hexencoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_hex("666f6f6261 7"), "Hexencoded string is malformed");
 }
 
 TEST_CASE("TestStringUtils::testHexEncodeDecode", "[test hex encode decode]") {
@@ -212,12 +322,12 @@ TEST_CASE("TestStringUtils::testHexEncodeDecode", "[test hex encode decode]") {
   for (size_t i = 0U; i < 1024U; i++) {
     const bool uppercase = gen() % 2;
     const size_t length = gen() % 1024;
-    std::vector<uint8_t> data(length);
-    std::generate_n(data.begin(), data.size(), [&]() -> uint8_t {
-      return gen() % 256;
+    std::vector<std::byte> data(length);
+    std::generate_n(data.begin(), data.size(), [&]() -> std::byte {
+      return static_cast<std::byte>(gen() % 256);
     });
-    auto hex = utils::StringUtils::to_hex(data.data(), data.size(), uppercase);
-    REQUIRE(data == utils::StringUtils::from_hex(hex.data(), hex.size()));
+    auto hex = utils::StringUtils::to_hex(data, uppercase);
+    REQUIRE(data == utils::StringUtils::from_hex(hex));
   }
 }
 
@@ -239,43 +349,45 @@ TEST_CASE("TestStringUtils::testBase64Encode", "[test base64 encode]") {
   REQUIRE("b29vb29v" == StringUtils::to_base64("oooooo", false /*url*/, false /*padded*/));
 
   REQUIRE("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/" ==
-    StringUtils::to_base64(std::vector<uint8_t>{0x00, 0x10, 0x83, 0x10,
-                                                0x51, 0x87, 0x20, 0x92,
-                                                0x8b, 0x30, 0xd3, 0x8f,
-                                                0x41, 0x14, 0x93, 0x51,
-                                                0x55, 0x97, 0x61, 0x96,
-                                                0x9b, 0x71, 0xd7, 0x9f,
-                                                0x82, 0x18, 0xa3, 0x92,
-                                                0x59, 0xa7, 0xa2, 0x9a,
-                                                0xab, 0xb2, 0xdb, 0xaf,
-                                                0xc3, 0x1c, 0xb3, 0xd3,
-                                                0x5d, 0xb7, 0xe3, 0x9e,
-                                                0xbb, 0xf3, 0xdf, 0xbf}));
+      StringUtils::to_base64(gsl::make_span(std::vector<uint8_t>{
+          0x00, 0x10, 0x83, 0x10,
+          0x51, 0x87, 0x20, 0x92,
+          0x8b, 0x30, 0xd3, 0x8f,
+          0x41, 0x14, 0x93, 0x51,
+          0x55, 0x97, 0x61, 0x96,
+          0x9b, 0x71, 0xd7, 0x9f,
+          0x82, 0x18, 0xa3, 0x92,
+          0x59, 0xa7, 0xa2, 0x9a,
+          0xab, 0xb2, 0xdb, 0xaf,
+          0xc3, 0x1c, 0xb3, 0xd3,
+          0x5d, 0xb7, 0xe3, 0x9e,
+          0xbb, 0xf3, 0xdf, 0xbf}).as_span<const std::byte>()));
   REQUIRE("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_" ==
-    StringUtils::to_base64(std::vector<uint8_t>{0x00, 0x10, 0x83, 0x10,
-                                                0x51, 0x87, 0x20, 0x92,
-                                                0x8b, 0x30, 0xd3, 0x8f,
-                                                0x41, 0x14, 0x93, 0x51,
-                                                0x55, 0x97, 0x61, 0x96,
-                                                0x9b, 0x71, 0xd7, 0x9f,
-                                                0x82, 0x18, 0xa3, 0x92,
-                                                0x59, 0xa7, 0xa2, 0x9a,
-                                                0xab, 0xb2, 0xdb, 0xaf,
-                                                0xc3, 0x1c, 0xb3, 0xd3,
-                                                0x5d, 0xb7, 0xe3, 0x9e,
-                                                0xbb, 0xf3, 0xdf, 0xbf}, true /*url*/));
+      StringUtils::to_base64(gsl::make_span(std::vector<uint8_t>{
+          0x00, 0x10, 0x83, 0x10,
+          0x51, 0x87, 0x20, 0x92,
+          0x8b, 0x30, 0xd3, 0x8f,
+          0x41, 0x14, 0x93, 0x51,
+          0x55, 0x97, 0x61, 0x96,
+          0x9b, 0x71, 0xd7, 0x9f,
+          0x82, 0x18, 0xa3, 0x92,
+          0x59, 0xa7, 0xa2, 0x9a,
+          0xab, 0xb2, 0xdb, 0xaf,
+          0xc3, 0x1c, 0xb3, 0xd3,
+          0x5d, 0xb7, 0xe3, 0x9e,
+          0xbb, 0xf3, 0xdf, 0xbf}).as_span<const std::byte>(), true /*url*/));
 }
 
 TEST_CASE("TestStringUtils::testBase64Decode", "[test base64 decode]") {
-  REQUIRE("" == StringUtils::from_base64(""));
-  REQUIRE("o" == StringUtils::from_base64("bw=="));
-  REQUIRE("oo" == StringUtils::from_base64("b28="));
-  REQUIRE("ooo" == StringUtils::from_base64("b29v"));
-  REQUIRE("oooo" == StringUtils::from_base64("b29vbw=="));
-  REQUIRE("ooooo" == StringUtils::from_base64("b29vb28="));
-  REQUIRE("oooooo" == StringUtils::from_base64("b29vb29v"));
-  REQUIRE("\xfb\xff\xbf" == StringUtils::from_base64("-_-_"));
-  REQUIRE("\xfb\xff\xbf" == StringUtils::from_base64("+/+/"));
+  REQUIRE(StringUtils::from_base64("", as_string).empty());
+  REQUIRE("o" == StringUtils::from_base64("bw==", as_string));
+  REQUIRE("oo" == StringUtils::from_base64("b28=", as_string));
+  REQUIRE("ooo" == StringUtils::from_base64("b29v", as_string));
+  REQUIRE("oooo" == StringUtils::from_base64("b29vbw==", as_string));
+  REQUIRE("ooooo" == StringUtils::from_base64("b29vb28=", as_string));
+  REQUIRE("oooooo" == StringUtils::from_base64("b29vb29v", as_string));
+  REQUIRE("\xfb\xff\xbf" == StringUtils::from_base64("-_-_", as_string));
+  REQUIRE("\xfb\xff\xbf" == StringUtils::from_base64("+/+/", as_string));
   REQUIRE(std::string({   0,   16, -125,   16,
                          81, -121,   32, -110,
                        -117,   48,  -45, -113,
@@ -287,7 +399,7 @@ TEST_CASE("TestStringUtils::testBase64Decode", "[test base64 decode]") {
                         -85,  -78,  -37,  -81,
                         -61,   28,  -77,  -45,
                          93,  -73,  -29,  -98,
-                        -69,  -13,  -33,  -65}) == StringUtils::from_base64("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"));
+                        -69,  -13,  -33,  -65}) == StringUtils::from_base64("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/", as_string));
   REQUIRE(std::string({   0,   16, -125,   16,
                          81, -121,   32, -110,
                        -117,   48,  -45, -113,
@@ -299,23 +411,23 @@ TEST_CASE("TestStringUtils::testBase64Decode", "[test base64 decode]") {
                         -85,  -78,  -37,  -81,
                         -61,   28,  -77,  -45,
                          93,  -73,  -29,  -98,
-                        -69,  -13,  -33,  -65}) == StringUtils::from_base64("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_"));
+                        -69,  -13,  -33,  -65}) == StringUtils::from_base64("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_", as_string));
 
-  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm9vYmFyYnV6eg=="));
-  REQUIRE("foobarbuzz"== StringUtils::from_base64("\r\nZm9vYmFyYnV6eg=="));
-  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm9\r\nvYmFyYnV6eg=="));
-  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm\r9vYmFy\n\n\n\n\n\n\n\nYnV6eg=="));
-  REQUIRE("foobarbuzz" == StringUtils::from_base64("\nZ\nm\n9\nv\nY\nm\nF\ny\nY\nn\nV\n6\ne\ng\n=\n=\n"));
+  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm9vYmFyYnV6eg==", as_string));
+  REQUIRE("foobarbuzz"== StringUtils::from_base64("\r\nZm9vYmFyYnV6eg==", as_string));
+  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm9\r\nvYmFyYnV6eg==", as_string));
+  REQUIRE("foobarbuzz" == StringUtils::from_base64("Zm\r9vYmFy\n\n\n\n\n\n\n\nYnV6eg==", as_string));
+  REQUIRE("foobarbuzz" == StringUtils::from_base64("\nZ\nm\n9\nv\nY\nm\nF\ny\nY\nn\nV\n6\ne\ng\n=\n=\n", as_string));
 
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("a"), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaaa"), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa="), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaaaa="), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==?"), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==a"), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==="), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("?"), "Base64 encoded string is malformatted");
-  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaa?"), "Base64 encoded string is malformatted");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("a"), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaaa"), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa="), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaaaa="), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==?"), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==a"), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aa==="), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("?"), "Base64 encoded string is malformed");
+  REQUIRE_THROWS_WITH(StringUtils::from_base64("aaaa?"), "Base64 encoded string is malformed");
 }
 
 TEST_CASE("TestStringUtils::testBase64EncodeDecode", "[test base64 encode decode]") {
@@ -324,12 +436,12 @@ TEST_CASE("TestStringUtils::testBase64EncodeDecode", "[test base64 encode decode
     const bool url = gen() % 2;
     const bool padded = gen() % 2;
     const size_t length = gen() % 1024;
-    std::vector<uint8_t> data(length);
-    std::generate_n(data.begin(), data.size(), [&]() -> uint8_t {
-      return gen() % 256;
+    std::vector<std::byte> data(length);
+    std::generate_n(data.begin(), data.size(), [&]() -> std::byte {
+      return static_cast<std::byte>(gen() % 256);
     });
-    auto base64 = utils::StringUtils::to_base64(data.data(), data.size(), url, padded);
-    REQUIRE(data == utils::StringUtils::from_base64(base64.data(), base64.size()));
+    auto base64 = utils::StringUtils::to_base64(data, url, padded);
+    REQUIRE(data == utils::StringUtils::from_base64(base64));
   }
 }
 
@@ -408,3 +520,25 @@ TEST_CASE("StringUtils::removeFramingCharacters works correctly", "[removeFramin
   REQUIRE(utils::StringUtils::removeFramingCharacters("\"abba\"", '"') == "abba");
   REQUIRE(utils::StringUtils::removeFramingCharacters("\"\"abba\"\"", '"') == "\"abba\"");
 }
+
+// ignore terminating \0 character
+template<size_t N>
+gsl::span<const std::byte> from_cstring(const char (&str)[N]) {
+  return gsl::span<const char>(str, N-1).as_span<const std::byte>();
+}
+
+TEST_CASE("StringUtils::escapeUnprintableBytes", "[escapeUnprintableBytes]") {
+  REQUIRE(StringUtils::escapeUnprintableBytes(from_cstring("abcd")) == "abcd");
+  REQUIRE(StringUtils::escapeUnprintableBytes(from_cstring("ab\n\r\t\v\fde")) == "ab\\n\\r\\t\\v\\fde");
+  REQUIRE(StringUtils::escapeUnprintableBytes(from_cstring("ab\x00""c\x01""d")) == "ab\\x00c\\x01d");
+}
+
+TEST_CASE("StringUtils::matchesSequence works correctly", "[matchesSequence]") {
+  REQUIRE(StringUtils::matchesSequence("abcdef", {"abc", "def"}));
+  REQUIRE(!StringUtils::matchesSequence("abcef", {"abc", "def"}));
+  REQUIRE(StringUtils::matchesSequence("xxxabcxxxdefxxx", {"abc", "def"}));
+  REQUIRE(!StringUtils::matchesSequence("defabc", {"abc", "def"}));
+  REQUIRE(StringUtils::matchesSequence("xxxabcxxxabcxxxdefxxx", {"abc", "def"}));
+  REQUIRE(StringUtils::matchesSequence("xxxabcxxxabcxxxdefxxx", {"abc", "abc", "def"}));
+  REQUIRE(!StringUtils::matchesSequence("xxxabcxxxdefxxx", {"abc", "abc", "def"}));
+}
