@@ -1,5 +1,4 @@
 /**
- *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -19,26 +18,18 @@
 #define MAIN_AGENTDOCS_H_
 
 #include <iostream>
+#include "agent/build_description.h"
 
-namespace org {
-namespace apache {
-namespace nifi {
-namespace minifi {
-namespace docs {
+namespace org::apache::nifi::minifi::docs {
 
 class AgentDocs {
  public:
-  AgentDocs() = default;
-  ~AgentDocs() = default;
   void generate(const std::string &docsdir, std::ostream &genStream);
  private:
-  inline std::string extractClassName(const std::string &processor) const;
+  [[nodiscard]] inline std::string extractClassName(const std::string &processor) const;
+  BuildDescription build_description_;
 };
 
-} /* namespace docs */
-} /* namespace minifi */
-} /* namespace nifi */
-} /* namespace apache */
-} /* namespace org */
+}  // namespace org::apache::nifi::minifi::docs
 
 #endif  // MAIN_AGENTDOCS_H_
