@@ -77,10 +77,13 @@ FILE* __cdecl __imp___iob_func() {
 
 #endif
 
+namespace minifi = org::apache::nifi::minifi;
+namespace utils = minifi::utils;
+namespace logging = minifi::core::logging;
+
 bool validHome(const std::string &home_path) {
-  struct stat stat_result { };
-  const std::string properties_file_path = utils::file::FileUtils::concat_path(home_path, DEFAULT_NIFI_PROPERTIES_FILE);
-  return stat(properties_file_path.c_str(), &stat_result) == 0;
+  const std::string properties_file_path = utils::file::concat_path(home_path, DEFAULT_NIFI_PROPERTIES_FILE);
+  return utils::file::exists(properties_file_path);
 }
 
 void setSyslogLogger() {
@@ -105,12 +108,12 @@ std::string determineMinifiHome(const std::shared_ptr<logging::Logger>& logger)
     }
 
     /* Try to determine MINIFI_HOME relative to the location of the minifi executable */
-    std::string executablePath = utils::file::FileUtils::get_executable_path();
+    std::string executablePath = utils::file::get_executable_path();
     if (executablePath.empty()) {
       logger->log_error("Failed to determine location of the minifi executable");
     } else {
       std::string minifiPath, minifiFileName;
-      std::tie(minifiPath, minifiFileName) = minifi::utils::file::FileUtils::split_path(executablePath);
+      std::tie(minifiPath, minifiFileName) = minifi::utils::file::split_path(executablePath);
       logger->log_info("Inferred " MINIFI_HOME_ENV_KEY "=%s based on the minifi executable location %s", minifiPath, executablePath);
       return minifiPath;
     }
@@ -143,8 +146,8 @@ std::string determineMinifiHome(const std::shared_ptr<logging::Logger>& logger)
     logger->log_info("%s is not a valid " MINIFI_HOME_ENV_KEY ", because there is no " DEFAULT_NIFI_PROPERTIES_FILE " file in it.", minifiHome);
 
     std::string minifiHomeWithoutBin, binDir;
-    std::tie(minifiHomeWithoutBin, binDir) = minifi::utils::file::FileUtils::split_path(minifiHome);
-    if (minifiHomeWithoutBin != "" && (binDir == "bin" || binDir == std::string("bin") + minifi::utils::file::FileUtils::get_separator())) {
+    std::tie(minifiHomeWithoutBin, binDir) = minifi::utils::file::split_path(minifiHome);
+    if (minifiHomeWithoutBin != "" && (binDir == "bin" || binDir == std::string("bin") + minifi::utils::file::get_separator())) {
       if (validHome(minifiHomeWithoutBin)) {
         logger->log_info("%s is a valid " MINIFI_HOME_ENV_KEY ", falling back to it.", minifiHomeWithoutBin);
         minifiHomeValid = true;
