@@ -35,7 +35,6 @@
 #include "core/Processor.h"
 #include "core/ProcessSession.h"
 #include "core/Core.h"
-#include "core/Resource.h"
 #include "core/logging/LoggerConfiguration.h"
 
 namespace org {
@@ -53,8 +52,8 @@ class CallbackProcessor : public core::Processor {
   /*!
    * Create a new processor
    */
-  CallbackProcessor(const std::string& name, const utils::Identifier& uuid = {})
-      : Processor(std::move(name), uuid) {
+  explicit CallbackProcessor(const std::string& name, const utils::Identifier& uuid = {})
+      : Processor(name, uuid) {
   }
   // Destructor
   ~CallbackProcessor() override = default;
@@ -85,11 +84,9 @@ class CallbackProcessor : public core::Processor {
   std::function<void(core::ProcessContext *context)> onschedule_callback_;
  private:
   // Logger
-  std::shared_ptr<logging::Logger> logger_{ logging::LoggerFactory<CallbackProcessor>::getLogger() };
+  std::shared_ptr<core::logging::Logger> logger_{ core::logging::LoggerFactory<CallbackProcessor>::getLogger() };
 };
 
-REGISTER_RESOURCE(CallbackProcessor, "");
-
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
