@@ -42,6 +42,7 @@
 #include "utils/ThreadPool.h"
 #include "core/state/UpdateController.h"
 #include "core/file_utils.h"
+#include "core/extension/ExtensionManager.h"
 
 namespace org {
 namespace apache {
@@ -84,8 +85,8 @@ class Instance {
     utils::Identifier uuid;
     uuid = port;
     rpg_ = std::make_shared<minifi::RemoteProcessorGroupPort>(stream_factory_, url, url, configure_, uuid);
-    proc_node_ = std::make_shared<core::ProcessorNode>(rpg_);
-    core::FlowConfiguration::initialize_static_functions();
+    proc_node_ = std::make_shared<core::ProcessorNode>(rpg_.get());
+    core::extension::ExtensionManager::get().initialize(configure_);
     content_repo_->initialize(configure_);
   }
 
@@ -101,8 +102,8 @@ class Instance {
   void enableAsyncC2(C2_Server *server, c2_stop_callback *c1, c2_start_callback* /*c2*/, c2_update_callback* /*c3*/) {
     running_ = true;
     if (server->type != C2_Server_Type::MQTT) {
-      configure_->set("c2.rest.url", server->url);
-      configure_->set("c2.rest.url.ack", server->ack_url);
+      configure_->set(minifi::Configuration::nifi_c2_rest_url, server->url);
+      configure_->set(minifi::Configuration::nifi_c2_rest_url_ack, server->ack_url);
     }
     agent_ = std::make_shared<c2::C2CallbackAgent>(nullptr, nullptr, nullptr, configure_);
     listener_thread_pool_.start();
@@ -146,19 +147,15 @@ class Instance {
 
  protected:
 
-  bool registerUpdateListener(const std::shared_ptr<state::UpdateController> &updateController, const int64_t& /*delay*/) {
+  void registerUpdateListener(const std::shared_ptr<state::UpdateController> &updateController, const int64_t& /*delay*/) {
     auto functions = updateController->getFunctions();
     // run all functions independently
 
     for (auto function : functions) {
       utils::Worker<utils::TaskRescheduleInfo> functor(function, "listeners");
       std::future<utils::TaskRescheduleInfo> future;
-      if (!listener_thread_pool_.execute(std::move(functor), future)) {
-        // denote failure
-        return false;
-      }
+      listener_thread_pool_.execute(std::move(functor), future);
     }
-    return true;
   }
 
   std::shared_ptr<c2::C2CallbackAgent> agent_;
