@@ -17,6 +17,8 @@
  */
 #include "cxx/CallbackProcessor.h"
 #include "core/cxxstructs.h"
+#include "core/Resource.h"
+
 namespace org {
 namespace apache {
 namespace nifi {
@@ -45,6 +47,8 @@ void CallbackProcessor::onTrigger(core::ProcessContext *context, core::ProcessSe
  }
 }
 
+REGISTER_RESOURCE(CallbackProcessor, "");
+
 } /* namespace processors */
 } /* namespace minifi */
 } /* namespace nifi */
